@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,7 +25,6 @@
 #include "wikipediasettings.h"
 #include "lyricsettings.h"
 #include "othersettings.h"
-#include "localize.h"
 
 ContextSettings::ContextSettings(QWidget *p)
     : QTabWidget(p)
@@ -33,9 +32,9 @@ ContextSettings::ContextSettings(QWidget *p)
     wiki=new WikipediaSettings(this);
     lyrics=new LyricSettings(this);
     other=new OtherSettings(this);
-    addTab(lyrics, i18n("Lyrics Providers"));
-    addTab(wiki, i18n("Wikipedia Languages"));
-    addTab(other, i18n("Other"));
+    addTab(lyrics, tr("Lyrics Providers"));
+    addTab(wiki, tr("Wikipedia Languages"));
+    addTab(other, tr("Other"));
 }
 
 void ContextSettings::load()
@@ -51,3 +50,5 @@ void ContextSettings::save()
     lyrics->save();
     other->save();
 }
+
+#include "moc_contextsettings.cpp"
