@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,38 +25,31 @@
 #include "artistview.h"
 #include "albumview.h"
 #include "songview.h"
-#ifdef ENABLE_ONLINE_SERVICES
 #include "onlineview.h"
-#endif
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 #include "support/utils.h"
+#include "gui/apikeys.h"
 #include "gui/covers.h"
-#include "network/networkaccessmanager.h"
 #include "gui/settings.h"
+#include "network/networkaccessmanager.h"
 #include "wikipediaengine.h"
-#include "support/localize.h"
-#include "backdropcreator.h"
 #include "support/gtkstyle.h"
 #include "widgets/playqueueview.h"
 #include "widgets/treeview.h"
 #include "widgets/thinsplitterhandle.h"
+#ifdef Q_OS_MAC
+#include "support/osxstyle.h"
+#endif
 #include <QHBoxLayout>
 #include <QGridLayout>
 #include <QSpacerItem>
 #include <QStylePainter>
-#if QT_VERSION >= 0x050000
 #include <QUrlQuery>
 #include <QJsonDocument>
-#else
-#include "qjson/parser.h"
-#endif
 #include <QXmlStreamReader>
 #include <QFile>
 #include <QWheelEvent>
 #include <QApplication>
-#ifndef SCALE_CONTEXT_BGND
-#include <QDesktopWidget>
-#endif
 #include <QStackedWidget>
 #include <QAction>
 #include <QPair>
@@ -78,8 +71,6 @@ void ContextWidget::enableDebug()
 }
 
 const QLatin1String ContextWidget::constBackdropFileName("backdrop");
-//const QLatin1String ContextWidget::constHtbApiKey(0); // API key required
-const QLatin1String ContextWidget::constFanArtApiKey("ee86404cb429fa27ac32a1a3c117b006");
 const QLatin1String ContextWidget::constCacheDir("backdrops/");
 
 static QString cacheFileName(const QString &artist, bool createDir)
@@ -91,11 +82,12 @@ class ViewSelectorButton : public QToolButton
 {
 public:
     ViewSelectorButton(QWidget *p) : QToolButton(p) { }
-    void paintEvent(QPaintEvent *ev)
+    void paintEvent(QPaintEvent *ev) override
     {
         Q_UNUSED(ev)
         QPainter painter(this);
         QString txt=text();
+        bool mo=underMouse();
 
         txt.replace("&", "");
         QFont f(font());
@@ -108,7 +100,20 @@ public:
         }
 
         painter.setFont(f);
-        painter.setPen(palette().color(!isChecked() && underMouse() ? QPalette::Highlight : QPalette::Text));
+        if (isChecked() || mo) {
+            int lh=Utils::isHighDpi() ? 5 : 3;
+            #ifdef Q_OS_MAC
+            QColor col=OSXStyle::self()->viewPalette().highlight().color();
+            #else
+            QColor col=palette().color(QPalette::Highlight);
+            #endif
+            if (mo) {
+                col.setAlphaF(isChecked() ? 0.75 : 0.5);
+            }
+            QRect r=rect();
+            painter.fillRect(r.x(), r.y()+r.height()-(lh+1), r.width(), lh, col);
+        }
+        painter.setPen(palette().color(QPalette::Text));
         painter.drawText(rect(), Qt::AlignCenter, txt);
     }
 };
@@ -208,8 +213,8 @@ ThinSplitter::ThinSplitter(QWidget *parent)
 {
     setChildrenCollapsible(true);
     setOrientation(Qt::Horizontal);
-    resetAct=new QAction(i18n("Reset Spacing"), this);
-    connect(resetAct, SIGNAL(triggered(bool)), this, SLOT(reset()));
+    resetAct=new QAction(tr("Reset Spacing"), this);
+    connect(resetAct, SIGNAL(triggered()), this, SLOT(reset()));
     setHandleWidth(0);
 }
 
@@ -225,7 +230,7 @@ QSplitterHandle * ThinSplitter::createHandle()
 void ThinSplitter::reset()
 {
     int totalSize=0;
-    foreach (int s, sizes()) {
+    for (int s: sizes()) {
         totalSize+=s;
     }
     QList<int> newSizes;
@@ -239,22 +244,17 @@ void ThinSplitter::reset()
 
 ContextWidget::ContextWidget(QWidget *parent)
     : QWidget(parent)
-    , job(0)
+    , shown(false)
+    , job(nullptr)
     , alwaysCollapsed(false)
     , backdropType(PlayQueueView::BI_Cover)
     , darkBackground(false)
-    , useFanArt(0!=constFanArtApiKey.latin1())
-    , albumCoverBackdrop(false)
-    , oldIsAlbumCoverBackdrop(false)
     , fadeValue(1.0)
     , isWide(false)
-    , stack(0)
-    #ifdef ENABLE_ONLINE_SERVICES
-    , onlineContext(0)
-    #endif
-    , splitter(0)
-    , viewSelector(0)
-    , creator(0)
+    , stack(nullptr)
+    , onlineContext(nullptr)
+    , splitter(nullptr)
+    , viewSelector(nullptr)
 {
     QHBoxLayout *layout=new QHBoxLayout(this);
     mainStack=new QStackedWidget(this);
@@ -281,20 +281,6 @@ ContextWidget::ContextWidget(QWidget *parent)
     readConfig();
     setZoom();
     setWide(true);
-
-    #ifndef SCALE_CONTEXT_BGND
-    QDesktopWidget *dw=QApplication::desktop();
-    if (dw) {
-        QSize geo=dw->availableGeometry(this).size()-QSize(32, 64);
-        minBackdropSize=geo;
-        minBackdropSize.setWidth(((int)(minBackdropSize.width()/32))*32);
-        minBackdropSize.setHeight(((int)(minBackdropSize.height()/32))*32);
-        maxBackdropSize=QSize(geo.width()*1.25, geo.height()*1.25);
-    } else {
-        minBackdropSize=QSize(Utils::scaleForDpi(1024*3), Utils::scaleForDpi(768*3);
-        maxBackdropSize=QSize(minBackdropSize.width()*2, minBackdropSize.height()*2);
-    }
-    #endif
 }
 
 void ContextWidget::setZoom()
@@ -347,6 +333,9 @@ void ContextWidget::setWide(bool w)
         splitter->addWidget(album);
         splitter->setVisible(true);
         splitter->addWidget(song);
+        splitter->setCollapsible(0, false);
+        splitter->setCollapsible(1, false);
+        splitter->setCollapsible(2, false);
         if (resetSplitter) {
             splitter->reset();
         } else if (!state.isEmpty()) {
@@ -366,9 +355,9 @@ void ContextWidget::setWide(bool w)
         }
         if (!viewSelector) {
             viewSelector=new ViewSelector(standardContext);
-            viewSelector->addItem(i18n("&Artist"), "artist");
-            viewSelector->addItem(i18n("Al&bum"), "album");
-            viewSelector->addItem(i18n("&Track"), "song");
+            viewSelector->addItem(tr("&Artist"), "artist");
+            viewSelector->addItem(tr("Al&bum"), "album");
+            viewSelector->addItem(tr("&Track"), "song");
             viewSelector->setPalette(palette());
             connect(viewSelector, SIGNAL(activated(int)), stack, SLOT(setCurrentIndex(int)));
         }
@@ -404,9 +393,7 @@ void ContextWidget::resizeEvent(QResizeEvent *e)
     if (isVisible()) {
         setWide(width()>minWidth && !alwaysCollapsed);
     }
-    #ifdef SCALE_CONTEXT_BGND
     resizeBackdrop();
-    #endif
     QWidget::resizeEvent(e);
 }
 
@@ -437,8 +424,7 @@ void ContextWidget::readConfig()
         break;
    case PlayQueueView::BI_Custom:
         if (origType!=backdropType || backdropOpacity!=origOpacity || backdropBlur!=origBlur || origCustomBackdropFile!=customBackdropFile) {            
-            updateImage(QImage(customBackdropFile), false);
-            artistsCreatedBackdropsFor.clear();
+            updateImage(customBackdropFile.isEmpty() ? QImage() : QImage(customBackdropFile));
         }
         break;
     }
@@ -467,47 +453,60 @@ void ContextWidget::saveConfig()
 void ContextWidget::useDarkBackground(bool u)
 {
     if (u!=darkBackground) {
-        darkBackground=u;
-        QPalette pal=darkBackground ? palette() : parentWidget()->palette();
-        QColor prevLinkColor;
-        QColor linkCol;
-
-        if (darkBackground) {
-            QColor dark(32, 32, 32);
-            QColor light(240, 240, 240);
-            QColor linkVisited(164, 164, 164);
-            pal.setColor(QPalette::Window, dark);
-            pal.setColor(QPalette::Base, dark);
-            // Dont globally change window/button text - because this can mess up scrollbar buttons
-            // with some styles (e.g. plastique)
-//            pal.setColor(QPalette::WindowText, light);
-//            pal.setColor(QPalette::ButtonText, light);
-            pal.setColor(QPalette::Text, light);
-            pal.setColor(QPalette::Link, light);
-            pal.setColor(QPalette::LinkVisited, linkVisited);
-            prevLinkColor=appLinkColor;
-            linkCol=pal.color(QPalette::Link);
-        } else {
-            linkCol=appLinkColor;
-            prevLinkColor=QColor(240, 240, 240);
-        }
-        setPalette(pal);
-        artist->setPal(pal, linkCol, prevLinkColor);
-        album->setPal(pal, linkCol, prevLinkColor);
-        song->setPal(pal, linkCol, prevLinkColor);
-        if (viewSelector) {
-            viewSelector->setPalette(pal);
-        }
-        QWidget::update();
+        darkBackground = u;
+        updatePalette();
     }
 }
 
+void ContextWidget::updatePalette()
+{
+    QPalette pal=darkBackground ? palette() : parentWidget()->palette();
+    QColor prevLinkColor;
+    QColor linkCol;
+
+    if (darkBackground) {
+        QColor dark(32, 32, 32);
+        QColor light(240, 240, 240);
+        QColor linkVisited(164, 164, 164);
+        pal.setColor(QPalette::Window, dark);
+        pal.setColor(QPalette::Base, dark);
+        // Dont globally change window/button text - because this can mess up scrollbar buttons
+        // with some styles (e.g. plastique)
+        // pal.setColor(QPalette::WindowText, light);
+        // pal.setColor(QPalette::ButtonText, light);
+        pal.setColor(QPalette::Text, light);
+        pal.setColor(QPalette::Link, light);
+        pal.setColor(QPalette::LinkVisited, linkVisited);
+        prevLinkColor=appLinkColor;
+        linkCol=pal.color(QPalette::Link);
+    } else {
+        linkCol=appLinkColor;
+        prevLinkColor=QColor(240, 240, 240);
+        pal.setColor(QPalette::Base, pal.color(QPalette::Window));
+    }
+    setPalette(pal);
+    artist->setPal(pal, linkCol, prevLinkColor);
+    album->setPal(pal, linkCol, prevLinkColor);
+    song->setPal(pal, linkCol, prevLinkColor);
+    if (viewSelector) {
+        viewSelector->setPalette(pal);
+    }
+    QWidget::update();
+}
+
 void ContextWidget::showEvent(QShowEvent *e)
 {
     setWide(width()>minWidth && !alwaysCollapsed);
     if (backdropType) {
         updateBackdrop();
     }
+    if (!shown) {
+        // Some styles (e.g Adwaita-Qt) draw base colour for scrollbar background.
+        // We need to fix this to use the window background. Therefore, the first
+        // time we are shown set the palette.
+        updatePalette();
+        shown = true;
+    }
     QWidget::showEvent(e);
 }
 
@@ -518,27 +517,37 @@ void ContextWidget::paintEvent(QPaintEvent *e)
 
     if (darkBackground) {
         p.fillRect(r, palette().background().color());
+    } else {
+        QColor col = palette().windowText().color();
+        col.setAlphaF(0.15);
+        p.setPen(col);
+        p.drawLine(r.topLeft(), r.topRight());
+        if (parentWidget() && parentWidget()->parentWidget() && 0==qstrcmp(parentWidget()->parentWidget()->metaObject()->className(), "QStackedWidget")) {
+            if (Qt::LeftToRight==layoutDirection()) {
+                p.drawLine(r.topLeft()+QPoint(0, 1), r.bottomLeft());
+            } else {
+                p.drawLine(r.topRight()+QPoint(0, 1), r.bottomRight());
+            }
+        }
     }
     if (backdropType) {
         if (!oldBackdrop.isNull()) {
             if (!qFuzzyCompare(fadeValue, qreal(0.0))) {
                 p.setOpacity(1.0-fadeValue);
             }
-            #ifdef SCALE_CONTEXT_BGND
-            if (!oldIsAlbumCoverBackdrop && oldBackdrop.height()<height()) {
+            if (oldBackdrop.height()<height()) {
                 p.drawPixmap(0, (height()-oldBackdrop.height())/2, oldBackdrop);
-            } else
-            #endif
-            p.fillRect(r, QBrush(oldBackdrop));
+            } else {
+                p.fillRect(r, QBrush(oldBackdrop));
+            }
         }
         if (!currentBackdrop.isNull()) {
             p.setOpacity(fadeValue);
-            #ifdef SCALE_CONTEXT_BGND
-            if (!albumCoverBackdrop && currentBackdrop.height()<height()) {
+            if (currentBackdrop.height()<height()) {
                 p.drawPixmap(0, (height()-currentBackdrop.height())/2, currentBackdrop);
-            } else
-            #endif
-            p.fillRect(r, QBrush(currentBackdrop));
+            } else {
+                p.fillRect(r, QBrush(currentBackdrop));
+            }
         }
     }
     if (!darkBackground) {
@@ -552,26 +561,23 @@ void ContextWidget::setFade(float value)
         fadeValue = value;
         if (qFuzzyCompare(fadeValue, qreal(1.0))) {
             oldBackdrop=QPixmap();
-            oldIsAlbumCoverBackdrop=false;
         }
         QWidget::update();
     }
 }
 
-void ContextWidget::updateImage(QImage img, bool created)
+void ContextWidget::updateImage(QImage img)
 {
     DBUG << img.isNull() << currentBackdrop.isNull();
     oldBackdrop=currentBackdrop;
-    oldIsAlbumCoverBackdrop=albumCoverBackdrop;
     currentBackdrop=QPixmap();
     animator.stop();
-    if (img.isNull()) {
-        backdropAlbums.clear();
-    }
     if (img.isNull() && oldBackdrop.isNull()) {
         return;
     }
-    if (!img.isNull()) {
+    if (img.isNull()) {
+        currentImage=img;
+    } else {
         if (backdropOpacity<100) {
             img=TreeView::setOpacity(img, (backdropOpacity*1.0)/100.0);
         }
@@ -583,13 +589,8 @@ void ContextWidget::updateImage(QImage img, bool created)
             painter.end();
             img = blurred;
         }
-        #ifdef SCALE_CONTEXT_BGND
         currentImage=img;
-        #else
-        currentBackdrop=QPixmap::fromImage(img);
-        #endif
     }
-    albumCoverBackdrop=created;
     resizeBackdrop();
 
     animator.stop();
@@ -601,6 +602,7 @@ void ContextWidget::updateImage(QImage img, bool created)
         animator.setEndValue(1.0);
         animator.start();
     }
+    QWidget::update();
 }
 
 void ContextWidget::search()
@@ -629,7 +631,6 @@ void ContextWidget::update(const Song &s)
         cancel();
     }
 
-    #ifdef ENABLE_ONLINE_SERVICES
     if (Song::OnlineSvrTrack==sng.type) {
         if (!onlineContext) {
             QWidget *onlinePage=new QWidget(mainStack);
@@ -650,7 +651,6 @@ void ContextWidget::update(const Song &s)
         return;
     }
     mainStack->setCurrentIndex(0);
-    #endif
     artist->update(sng);
     album->update(sng);
     song->update(sng);
@@ -682,7 +682,7 @@ void ContextWidget::cancel()
 {
     if (job) {
         job->cancelAndDelete();
-        job=0;
+        job=nullptr;
     }
 }
 
@@ -693,7 +693,6 @@ void ContextWidget::updateBackdrop(bool force)
         return;
     }
     currentArtist=updateArtist;
-    backdropAlbums.clear();
     if (currentArtist.isEmpty()) {
         updateImage(QImage());
         QWidget::update();
@@ -711,7 +710,7 @@ void ContextWidget::updateBackdrop(bool force)
             dirName+=Utils::getDir(currentSong.file);
 
             for (int level=0; level<2; ++level) {
-                foreach (const QString &fileName, names) {
+                for (const QString &fileName: names) {
                     DBUG << "Checking file(1)" << QString(dirName+fileName);
                     if (QFile::exists(dirName+fileName)) {
                         QImage img(dirName+fileName);
@@ -737,7 +736,7 @@ void ContextWidget::updateBackdrop(bool force)
         QString dirName=MPDConnection::self()->getDetails().dirReadable ? MPDConnection::self()->getDetails().dir : QString();
         if (!dirName.isEmpty() && !dirName.startsWith(QLatin1String("http:/"))) {
             dirName+=currentArtist+Utils::constDirSep;
-            foreach (const QString &fileName, names) {
+            for (const QString &fileName: names) {
                 DBUG << "Checking file(2)" << QString(dirName+fileName);
                 if (QFile::exists(dirName+fileName)) {
                     QImage img(dirName+fileName);
@@ -774,13 +773,7 @@ static QString fixArtist(const QString &artist)
 void ContextWidget::getBackdrop()
 {
     cancel();
-    if (artistsCreatedBackdropsFor.contains(currentArtist)) {
-        createBackdrop();
-    } else if (useFanArt) {
-        getFanArtBackdrop();
-    } else {
-        getDiscoGsImage();
-    }
+    getFanArtBackdrop();
 }
 
 void ContextWidget::getFanArtBackdrop()
@@ -793,16 +786,10 @@ static const char * constArtistProp="artist-name";
 void ContextWidget::getMusicbrainzId(const QString &artist)
 {
     QUrl url("http://www.musicbrainz.org/ws/2/artist/");
-    #if QT_VERSION < 0x050000
-    QUrl &query=url;
-    #else
     QUrlQuery query;
-    #endif
 
     query.addQueryItem("query", "artist:"+artist);
-    #if QT_VERSION >= 0x050000
     url.setQuery(query);
-    #endif
 
     job = NetworkAccessManager::self()->get(url);
     DBUG << url.toString();
@@ -810,30 +797,6 @@ void ContextWidget::getMusicbrainzId(const QString &artist)
     connect(job, SIGNAL(finished()), this, SLOT(musicbrainzResponse()));
 }
 
-void ContextWidget::getDiscoGsImage()
-{
-    cancel();
-    QUrl url;
-    #if QT_VERSION < 0x050000
-    QUrl &query=url;
-    #else
-    QUrlQuery query;
-    #endif
-    url.setScheme("http");
-    url.setHost("api.discogs.com");
-    url.setPath("/search");
-    query.addQueryItem("per_page", QString::number(5));
-    query.addQueryItem("type", "artist");
-    query.addQueryItem("q", fixArtist(currentArtist));
-    query.addQueryItem("f", "json");
-    #if QT_VERSION >= 0x050000
-    url.setQuery(query);
-    #endif
-    job=NetworkAccessManager::self()->get(url, 5000);
-    DBUG << url.toString();
-    connect(job, SIGNAL(finished()), this, SLOT(discoGsResponse()));
-}
-
 void ContextWidget::musicbrainzResponse()
 {
     NetworkJob *reply = getReply(sender());
@@ -844,6 +807,8 @@ void ContextWidget::musicbrainzResponse()
     DBUG << "status" << reply->error() << reply->errorString();
 
     QString id;
+    QString currentId;
+    QString artist=reply->property(constArtistProp).toString();
 
     if (reply->ok()) {
         bool inSection=false;
@@ -855,28 +820,42 @@ void ContextWidget::musicbrainzResponse()
             if (doc.isStartElement()) {
                 if (!inSection && QLatin1String("artist-list")==doc.name()) {
                     inSection=true;
-                } if (inSection && QLatin1String("artist")==doc.name()) {
-                    id=doc.attributes().value("id").toString();
-                    break;
+                } else if (inSection && QLatin1String("artist")==doc.name()) {
+                    // Store this artist ID as the current ID
+                    currentId=doc.attributes().value("id").toString();
+                    if (id.isEmpty()) {
+                        // If we have no ID set, then use the first one - for now
+                        id=currentId;
+                    }
+                } else if (inSection && QLatin1String("name")==doc.name()) {
+                    if (doc.readElementText()==artist) {
+                        // Found an arist in the artist-list whose name matches what we are looking for, so use this.
+                        id=currentId;
+                        break;
+                    }
                 }
-            } else if (doc.isEndElement() && inSection && QLatin1String("artist")==doc.name()) {
+            } else if (doc.isEndDocument() && inSection && QLatin1String("artist-list")==doc.name()) {
                 break;
             }
         }
     }
 
     if (id.isEmpty()) {
-        QString artist=reply->property(constArtistProp).toString();
         // MusicBrainz does not seem to like AC/DC, but AC DC works - so if we fail with an artist
         // containing /, then try with space...
         if (!artist.isEmpty() && artist.contains("/")) {
             artist=artist.replace("/", " ");
             getMusicbrainzId(artist);
         } else {
-            getDiscoGsImage();
+            updateImage(QImage());
         }
     } else {
-        QUrl url("http://api.fanart.tv/webservice/artist/"+constFanArtApiKey+"/"+id+"/json/artistbackground/1");
+        QUrl url("http://webservice.fanart.tv/v3/music/"+id);
+        QUrlQuery query;
+
+        ApiKeys::self()->addKey(query, ApiKeys::FanArt);
+        url.setQuery(query);
+
         job=NetworkAccessManager::self()->get(url);
         DBUG << url.toString();
         connect(job, SIGNAL(finished()), this, SLOT(fanArtResponse()));
@@ -894,121 +873,33 @@ void ContextWidget::fanArtResponse()
     QString url;
 
     if (reply->ok()) {
-        #if QT_VERSION >= 0x050000
         QJsonParseError jsonParseError;
         QVariantMap parsed=QJsonDocument::fromJson(reply->readAll(), &jsonParseError).toVariant().toMap();
         bool ok=QJsonParseError::NoError==jsonParseError.error;
-        #else
-        bool ok=false;
-        QVariantMap parsed = QJson::Parser().parse(reply->readAll(), &ok).toMap();
-        #endif
 
         if (ok && !parsed.isEmpty()) {
-            QVariantMap artist=parsed[parsed.keys().first()].toMap();
-            if (artist.contains("artistbackground")) {
-                QVariantList artistbackgrounds=artist["artistbackground"].toList();
-                if (!artistbackgrounds.isEmpty()) {
-                    QVariantMap artistbackground=artistbackgrounds.first().toMap();
-                    if (artistbackground.contains("url")) {
-                        url=artistbackground["url"].toString();
-                    }
+            QVariantList artistbackgrounds;
+
+            if (parsed.contains("artistbackground")) {
+                artistbackgrounds=parsed["artistbackground"].toList();
+            } else {
+                QVariantMap artist=parsed[parsed.keys().first()].toMap();
+
+                if (artist.contains("artistbackground")) {
+                    artistbackgrounds=artist["artistbackground"].toList();
+                }
+            }
+            if (!artistbackgrounds.isEmpty()) {
+                QVariantMap artistbackground=artistbackgrounds.first().toMap();
+                if (artistbackground.contains("url")) {
+                    url=artistbackground["url"].toString();
                 }
             }
         }
     }
 
     if (url.isEmpty()) {
-        getDiscoGsImage();
-    } else {
-        job=NetworkAccessManager::self()->get(QUrl(url));
-        DBUG << url;
-        connect(job, SIGNAL(finished()), this, SLOT(downloadResponse()));
-    }
-}
-
-static bool matchesArtist(const QString &titleOrig, const QString &artistOrig)
-{
-    QString title=titleOrig.toLower();
-    QString artist=artistOrig.toLower();
-
-    if (title==artist) {
-        return true;
-    }
-
-    if (artist.startsWith(QLatin1String("the ")) && title.endsWith(QLatin1String(", the"))) {
-        QString theArtist=artist.mid(4)+QLatin1String(", the");
-        if (title==theArtist) {
-            return true;
-        }
-    }
-
-    typedef QPair<QChar, QChar> ChPair;
-    QList<ChPair> replacements = QList<ChPair>() << ChPair('-', '/') << ChPair('.', 0)
-                                                 << ChPair(QChar(0x00ff /* ÿ */), 'y');
-
-    foreach (const ChPair &r, replacements) {
-        QString a=artist;
-        QString t=title;
-
-        if (r.second.isNull()) {
-            a=a.replace(QString()+r.first, "");
-            t=t.replace(QString()+r.first, "");
-        } else {
-            a=a.replace(r.first, r.second);
-            t=t.replace(r.first, r.second);
-        }
-        if (t==a) {
-            return true;
-        }
-    }
-
-    return false;
-}
-
-void ContextWidget::discoGsResponse()
-{
-    NetworkJob *reply = getReply(sender());
-    if (!reply) {
-        return;
-    }
-
-    DBUG << "status" << reply->error() << reply->errorString();
-    QString url;
-
-    if (reply->ok()) {
-        #if QT_VERSION >= 0x050000
-        QJsonParseError jsonParseError;
-        QVariantMap parsed=QJsonDocument::fromJson(reply->readAll(), &jsonParseError).toVariant().toMap();
-        bool ok=QJsonParseError::NoError==jsonParseError.error;
-        #else
-        bool ok=false;
-        QVariantMap parsed = QJson::Parser().parse(reply->readAll(), &ok).toMap();
-        #endif
-
-        if (ok && parsed.contains("resp")) {
-            QVariantMap response=parsed["resp"].toMap();
-            if (response.contains("search")) {
-                QVariantMap search=response["search"].toMap();
-                if (search.contains("exactresults")) {
-                    QVariantList results=search["exactresults"].toList();
-                    foreach (const QVariant &r, results) {
-                        QVariantMap rm=r.toMap();
-                        if (rm.contains("thumb") && rm.contains("title")) {
-                            QString thumbUrl=rm["thumb"].toString();
-                            QString title=rm["title"].toString();
-                            if (thumbUrl.contains("/image/A-150-") && matchesArtist(title, currentArtist)) {
-                                url=thumbUrl.replace("image/A-150-", "/image/A-");
-                                break;
-                            }
-                        }
-                    }
-                }
-            }
-        }
-    }
-
-    if (url.isEmpty()) {
-        createBackdrop();
+        updateImage(QImage());
     } else {
         job=NetworkAccessManager::self()->get(QUrl(url));
         DBUG << url;
@@ -1033,119 +924,39 @@ void ContextWidget::downloadResponse()
         img=QImage::fromData(data);
     }
 
-    if (img.isNull()) {
-        createBackdrop();
-    } else {
-        updateImage(img);
-        bool saved=false;
-
-        if (Settings::self()->storeBackdropsInMpdDir() && !currentSong.isVariousArtists() &&
-            !currentSong.isNonMPD() && MPDConnection::self()->getDetails().dirReadable) {
-            QString mpdDir=MPDConnection::self()->getDetails().dir;
-            QString songDir=Utils::getDir(currentSong.file);
-            if (!mpdDir.isEmpty() && 2==songDir.split(Utils::constDirSep, QString::SkipEmptyParts).count()) {
-                QDir d(mpdDir+songDir);
-                d.cdUp();
-                QString fileName=Utils::fixPath(d.absolutePath())+constBackdropFileName+".jpg";
-                QFile f(fileName);
-                if (f.open(QIODevice::WriteOnly)) {
-                    f.write(data);
-                    f.close();
-                    DBUG << "Saved backdrop to" << fileName << "for artist" << currentArtist << ", current song" << currentSong.file;
-                    saved=true;
-                }
-            } else {
-                DBUG << "Not saving to mpd folder, mpd dir:" << mpdDir
-                     << "num parts:" << songDir.split(Utils::constDirSep, QString::SkipEmptyParts).count();
-            }
-        } else {
-            DBUG << "Not saving to mpd folder - set to save in mpd?" << Settings::self()->storeBackdropsInMpdDir()
-                 << "isVa:" << currentSong.isVariousArtists() << "isNonMPD:" << currentSong.isNonMPD()
-                 << "mpd readable:" << MPDConnection::self()->getDetails().dirReadable;
+    if (!img.isNull()) {
+        QString cacheName=cacheFileName(currentArtist, true);
+        QFile f(cacheName);
+        if (f.open(QIODevice::WriteOnly)) {
+            DBUG << "Saved backdrop to (cache)" << cacheName << "for artist" << currentArtist << ", current song" << currentSong.file;
+            f.write(data);
+            f.close();
         }
-
-        if (!saved) {
-            QString cacheName=cacheFileName(currentArtist, true);
-            QFile f(cacheName);
-            if (f.open(QIODevice::WriteOnly)) {
-                DBUG << "Saved backdrop to (cache)" << cacheName << "for artist" << currentArtist << ", current song" << currentSong.file;
-                f.write(data);
-                f.close();
-            }
-        }
-        QWidget::update();
-    }
-}
-
-void ContextWidget::createBackdrop()
-{
-    DBUG << currentArtist;
-    if (!creator) {
-        creator = new BackdropCreator();
-        connect(creator, SIGNAL(created(QString,QImage)), SLOT(backdropCreated(QString,QImage)));
-        connect(this, SIGNAL(createBackdrop(QString,QList<Song>)), creator, SLOT(create(QString,QList<Song>)));
-    }
-    QList<Song> artistAlbumsFirstTracks=artist->getArtistAlbumsFirstTracks();
-    QSet<QString> albumNames;
-
-    foreach (const Song &s, artistAlbumsFirstTracks) {
-        albumNames.insert(s.albumArtist()+" - "+s.album);
-    }
-
-    if (backdropAlbums!=albumNames) {
-        backdropAlbums=albumNames;
-        emit createBackdrop(currentArtist, artistAlbumsFirstTracks);
     }
+    updateImage(img);
 }
 
 void ContextWidget::resizeBackdrop()
 {
-    #ifdef SCALE_CONTEXT_BGND
-    if (!albumCoverBackdrop && !currentImage.isNull() &&( currentBackdrop.isNull() || (!currentBackdrop.isNull() && currentBackdrop.width()!=width()))) {
+    if (!currentImage.isNull() &&( currentBackdrop.isNull() || (!currentBackdrop.isNull() && currentBackdrop.width()!=width()))) {
         QSize sz(width(), width()*currentImage.height()/currentImage.width());
         currentBackdrop = QPixmap::fromImage(currentImage.scaled(sz, Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation));
     }
-    #else
-    if (!currentBackdrop.isNull() && !albumCoverBackdrop) {
-        if (currentBackdrop.width()<minBackdropSize.width() && currentBackdrop.height()<minBackdropSize.height()) {
-            QSize size(minBackdropSize);
-            if (currentBackdrop.width()<minBackdropSize.width()/4 && currentBackdrop.height()<minBackdropSize.height()/4) {
-                size=QSize(minBackdropSize.width()/2, minBackdropSize.height()/2);
-            }
-            currentBackdrop=currentBackdrop.scaled(size, Qt::KeepAspectRatio, Qt::SmoothTransformation);
-        } else if (maxBackdropSize.width()>1024 && maxBackdropSize.height()>768 &&
-                   (currentBackdrop.width()>maxBackdropSize.width() || currentBackdrop.height()>maxBackdropSize.height())) {
-            currentBackdrop=currentBackdrop.scaled(maxBackdropSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
-        }
-    }
-    #endif
-}
-
-void ContextWidget::backdropCreated(const QString &artist, const QImage &img)
-{
-    DBUG << artist << img.isNull() << currentArtist;
-    if (artist==currentArtist) {
-        artistsCreatedBackdropsFor.removeAll(artist);
-        artistsCreatedBackdropsFor.append(artist);
-        if (artistsCreatedBackdropsFor.count()>20) {
-            artistsCreatedBackdropsFor.removeFirst();
-        }
-        updateImage(img, true);
-        QWidget::update();
-    }
 }
 
 NetworkJob * ContextWidget::getReply(QObject *obj)
 {
     NetworkJob *reply = qobject_cast<NetworkJob*>(obj);
     if (!reply) {
-        return 0;
+        return nullptr;
     }
 
     reply->deleteLater();
     if (reply!=job) {
-        return 0;
+        return nullptr;
     }
-    job=0;
+    job=nullptr;
     return reply;
 }
+
+#include "moc_contextwidget.cpp"
