@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,9 +24,9 @@
 #ifndef LYRICSDIALOG_H
 #define LYRICSDIALOG_H
 
-#include "dialog.h"
-#include "lineedit.h"
-#include "song.h"
+#include "support/dialog.h"
+#include "support/lineedit.h"
+#include "mpd-interface/song.h"
 
 class LyricsDialog : public Dialog
 {
