@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /* This file is part of Clementine.
@@ -34,14 +34,15 @@ class UltimateLyricsProvider;
 
 class LyricSettings : public ToggleList
 {
+    Q_OBJECT
 public:
     LyricSettings(QWidget *p);
-    virtual ~LyricSettings() { }
+    ~LyricSettings() override { }
 
     void load();
     void save();
 
-    void showEvent(QShowEvent *e);
+    void showEvent(QShowEvent *e) override;
 
 private:
     bool loadedXml;
