@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,7 +25,6 @@
 #include "gui/settings.h"
 #include "support/pathrequester.h"
 #include "widgets/playqueueview.h"
-#include "support/localize.h"
 
 static const char *constValueProperty="value";
 
@@ -39,12 +38,8 @@ OtherSettings::OtherSettings(QWidget *p)
     contextBackdrop_artist->setProperty(constValueProperty, PlayQueueView::BI_Cover);
     contextBackdrop_custom->setProperty(constValueProperty, PlayQueueView::BI_Custom);
     contextBackdropFile->setDirMode(false);
-    #ifdef ENABLE_KDE_SUPPORT
-    contextBackdropFile->setFilter("image/jpeg image/png");
-    #else
-    contextBackdropFile->setFilter(i18n("Images (*.png *.jpg)"));
-    #endif
-    int labelWidth=qMax(fontMetrics().width(QLatin1String("100%")), fontMetrics().width(i18nc("pixels", "10px")));
+    contextBackdropFile->setFilter(tr("Images (*.png *.jpg)"));
+    int labelWidth=qMax(fontMetrics().width(QLatin1String("100%")), fontMetrics().width(tr("10px", "pixels")));
     contextBackdropOpacityLabel->setFixedWidth(labelWidth);
     contextBackdropBlurLabel->setFixedWidth(labelWidth);
     connect(contextBackdropOpacity, SIGNAL(valueChanged(int)), SLOT(setContextBackdropOpacityLabel()));
@@ -66,7 +61,7 @@ void OtherSettings::load()
     contextBackdrop_custom->setChecked(bgnd==contextBackdrop_custom->property(constValueProperty).toInt());
     contextBackdropOpacity->setValue(Settings::self()->contextBackdropOpacity());
     contextBackdropBlur->setValue(Settings::self()->contextBackdropBlur());
-    contextBackdropFile->setText(Settings::self()->contextBackdropFile());
+    contextBackdropFile->setText(Utils::convertPathForDisplay(Settings::self()->contextBackdropFile(), false));
     contextSwitchTime->setValue(Settings::self()->contextSwitchTime());
 
     toggleWikiNote();
@@ -90,7 +85,7 @@ void OtherSettings::save()
     }
     Settings::self()->saveContextBackdropOpacity(contextBackdropOpacity->value());
     Settings::self()->saveContextBackdropBlur(contextBackdropBlur->value());
-    Settings::self()->saveContextBackdropFile(contextBackdropFile->text().trimmed());
+    Settings::self()->saveContextBackdropFile(Utils::convertPathFromDisplay(contextBackdropFile->text(), false));
     Settings::self()->saveContextSwitchTime(contextSwitchTime->value());
 }
 
@@ -101,12 +96,12 @@ void OtherSettings::toggleWikiNote()
 
 void OtherSettings::setContextBackdropOpacityLabel()
 {
-    contextBackdropOpacityLabel->setText(i18nc("value%", "%1%", contextBackdropOpacity->value()));
+    contextBackdropOpacityLabel->setText(tr("%1%", "value%").arg(contextBackdropOpacity->value()));
 }
 
 void OtherSettings::setContextBackdropBlurLabel()
 {
-    contextBackdropBlurLabel->setText(i18nc("pixels", "%1px", contextBackdropBlur->value()));
+    contextBackdropBlurLabel->setText(tr("%1 px", "pixels").arg(contextBackdropBlur->value()));
 }
 
 void OtherSettings::enableContextBackdropOptions()
@@ -116,3 +111,5 @@ void OtherSettings::enableContextBackdropOptions()
     contextBackdropBlur->setEnabled(contextBackdropOpacity->isEnabled());
     contextBackdropBlurLabel->setEnabled(contextBackdropOpacity->isEnabled());
 }
+
+#include "moc_othersettings.cpp"
