@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -31,7 +31,7 @@
 #include "support/squeezedtextlabel.h"
 #include "support/utils.h"
 #include "support/messagebox.h"
-#include "support/localize.h"
+#include "support/monoicon.h"
 #ifdef TAGLIB_FOUND
 #include "tags/tags.h"
 #endif
@@ -42,7 +42,7 @@
 #include "network/networkaccessmanager.h"
 #include "widgets/textbrowser.h"
 #include "gui/stdactions.h"
-#include "mpd/mpdstatus.h"
+#include "mpd-interface/mpdstatus.h"
 #include "qtiocompressor/qtiocompressor.h"
 #include <QFile>
 #include <QDir>
@@ -52,9 +52,8 @@
 #include <QTimer>
 #include <QScrollBar>
 #include <QDesktopServices>
-#if QT_VERSION >= 0x050000
+#include <QUrl>
 #include <QUrlQuery>
-#endif
 
 const QLatin1String SongView::constLyricsDir("lyrics/");
 const QLatin1String SongView::constExtension(".lyrics");
@@ -63,40 +62,32 @@ const QLatin1String SongView::constInfoExt(".html.gz");
 
 static QString infoCacheFileName(const Song &song, const QString &lang, bool createDir)
 {
-    QString artist=song.artist;
-    QString title=song.title;
-    title.replace("/", "_");
-    artist.replace("/", "_");
-    QString dir=Utils::cacheDir(SongView::constCacheDir+Covers::encodeName(artist)+Utils::constDirSep, createDir);
-
+    QString dir=Utils::cacheDir(SongView::constCacheDir+Covers::encodeName(song.basicArtist())+Utils::constDirSep, createDir);
     if (dir.isEmpty()) {
         return QString();
     }
-    return dir+Covers::encodeName(title)+"."+lang+SongView::constInfoExt;
+    return dir+Covers::encodeName(song.basicTitle())+"."+lang+SongView::constInfoExt;
 }
 
 static QString lyricsCacheFileName(const Song &song, bool createDir=false)
 {
-    QString artist=song.artist;
-    QString title=song.title;
-    title.replace("/", "_");
-    artist.replace("/", "_");
-    QString dir=Utils::cacheDir(SongView::constLyricsDir+Covers::encodeName(artist)+Utils::constDirSep, createDir);
-
+    QString dir=Utils::cacheDir(SongView::constLyricsDir+Covers::encodeName(song.basicArtist())+Utils::constDirSep, createDir);
     if (dir.isEmpty()) {
         return QString();
     }
-    return dir+Covers::encodeName(title)+SongView::constExtension;
+    return dir+Covers::encodeName(song.basicTitle())+SongView::constExtension;
 }
 
-static inline QString mpdLyricsFilePath(const QString &songFile)
+#if !defined Q_OS_WIN && !defined Q_OS_MAC
+static QString lyricsOtherFileName(const Song &song)
 {
-    return Utils::changeExtension(MPDConnection::self()->getDetails().dir+songFile, SongView::constExtension);
+    return QDir::homePath()+"/.lyrics/"+Covers::encodeName(song.basicArtist())+" - "+Covers::encodeName(song.basicTitle())+".txt";
 }
+#endif
 
 static inline QString mpdLyricsFilePath(const Song &song)
 {
-    return mpdLyricsFilePath(song.filePath());
+    return Utils::changeExtension(song.filePath(MPDConnection::self()->getDetails().dir), SongView::constExtension);
 }
 
 static inline QString fixNewLines(const QString &o)
@@ -104,39 +95,24 @@ static inline QString fixNewLines(const QString &o)
     return QString(o).replace(QLatin1String("\n\n\n"), QLatin1String("\n\n")).replace("\n", "<br/>");
 }
 
-static QString actualFile(const Song &song)
-{
-    QString songFile=song.filePath();
-
-    if (song.isCantataStream()) {
-        #if QT_VERSION < 0x050000
-        QUrl u(songFile);
-        #else
-        QUrl qu(songFile);
-        QUrlQuery u(qu);
-        #endif
-        songFile=u.hasQueryItem("file") ? u.queryItemValue("file") : QString();
-    }
-    return songFile;
-}
-
 SongView::SongView(QWidget *p)
-    : View(p, QStringList() << i18n("Lyrics") << i18n("Information") << i18n("Tags"))
-    , scrollTimer(0)
+    : View(p, QStringList() << tr("Lyrics") << tr("Information") << tr("Metadata"))
+    , scrollTimer(nullptr)
     , songPos(0)
     , currentProvider(-1)
     , currentRequest(0)
     , mode(Mode_Display)
-    , job(0)
-    , currentProv(0)
+    , job(nullptr)
+    , currentProv(nullptr)
     , lyricsNeedsUpdating(true)
     , infoNeedsUpdating(true)
-    , tagsNeedsUpdating(true)
+    , metadataNeedsUpdating(true)
 {
-    scrollAction = ActionCollection::get()->createAction("scrolllyrics", i18n("Scroll Lyrics"), "go-down");
-    refreshAction = ActionCollection::get()->createAction("refreshlyrics", i18n("Refresh Lyrics"), "view-refresh");
-    editAction = ActionCollection::get()->createAction("editlyrics", i18n("Edit Lyrics"), Icons::self()->editIcon);
-    delAction = ActionCollection::get()->createAction("dellyrics", i18n("Delete Lyrics File"), "edit-delete");
+    QColor iconCol=Utils::monoIconColor();
+    scrollAction = ActionCollection::get()->createAction("scrolllyrics", tr("Scroll Lyrics"), MonoIcon::icon(FontAwesome::chevrondown, iconCol));
+    refreshAction = ActionCollection::get()->createAction("refreshlyrics", tr("Refresh Lyrics"), Icons::self()->refreshIcon);
+    editAction = ActionCollection::get()->createAction("editlyrics", tr("Edit Lyrics"), Icons::self()->editIcon);
+    delAction = ActionCollection::get()->createAction("dellyrics", tr("Delete Lyrics File"), Icons::self()->removeIcon);
 
     scrollAction->setCheckable(true);
     scrollAction->setChecked(Settings::self()->contextAutoScroll());
@@ -147,13 +123,13 @@ SongView::SongView(QWidget *p)
     connect(UltimateLyrics::self(), SIGNAL(lyricsReady(int, QString)), SLOT(lyricsReady(int, QString)));
 
     engine=ContextEngine::create(this);
-    refreshInfoAction = ActionCollection::get()->createAction("refreshtrack", i18n("Refresh Track Information"), "view-refresh");
-    cancelInfoJobAction=new Action(Icons::self()->cancelIcon, i18n("Cancel"), this);
+    refreshInfoAction = ActionCollection::get()->createAction("refreshtrack", tr("Refresh Track Information"), Icons::self()->refreshIcon);
+    cancelInfoJobAction=new Action(Icons::self()->cancelIcon, tr("Cancel"), this);
     cancelInfoJobAction->setEnabled(false);
     connect(refreshInfoAction, SIGNAL(triggered()), SLOT(refreshInfo()));
     connect(cancelInfoJobAction, SIGNAL(triggered()), SLOT(abortInfoSearch()));
     connect(engine, SIGNAL(searchResult(QString,QString)), this, SLOT(infoSearchResponse(QString,QString)));
-    foreach (TextBrowser *t, texts) {
+    for (TextBrowser *t: texts) {
         connect(t, SIGNAL(anchorClicked(QUrl)), SLOT(showMoreInfo(QUrl)));
     }
 
@@ -163,7 +139,7 @@ SongView::SongView(QWidget *p)
     connect(texts.at(Page_Information), SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(showInfoContextMenu(QPoint)));
     connect(this, SIGNAL(viewChanged()), this, SLOT(curentViewChanged()));
     setMode(Mode_Blank);
-    setStandardHeader(i18n("Track"));
+    setStandardHeader(tr("Track"));
     clear();
     toggleScroll();
     setCurrentView(Settings::self()->contextTrackView());
@@ -182,8 +158,8 @@ void SongView::update()
     bool cacheExists=!cacheName.isEmpty() && QFile::exists(cacheName);
 
     if (mpdExists || cacheExists) {
-        switch (MessageBox::warningYesNoCancel(this, i18n("Reload lyrics?\n\nReload from disk, or delete disk copy and download?"), i18n("Reload"),
-                                               GuiItem(i18n("Reload From Disk")), GuiItem(i18n("Download")))) {
+        switch (MessageBox::warningYesNoCancel(this, tr("Reload lyrics?\n\nReload from disk, or delete disk copy and download?"), tr("Reload"),
+                                               GuiItem(tr("Reload From Disk")), GuiItem(tr("Download")))) {
         case MessageBox::Yes:
             break;
         case MessageBox::No:
@@ -215,8 +191,8 @@ void SongView::search()
     LyricsDialog dlg(currentSong, this);
     if (QDialog::Accepted==dlg.exec()) {
         if ((song.artist!=currentSong.artist || song.title!=currentSong.title) &&
-                MessageBox::No==MessageBox::warningYesNo(this, i18n("Current playing song has changed, still perform search?"), i18n("Song Changed"),
-                                                         GuiItem(i18n("Perform Search")), StdGuiItem::cancel())) {
+                MessageBox::No==MessageBox::warningYesNo(this, tr("Current playing song has changed, still perform search?"), tr("Song Changed"),
+                                                         GuiItem(tr("Perform Search")), StdGuiItem::cancel())) {
             return;
         }
         QString mpdName=mpdFileName();
@@ -238,7 +214,7 @@ void SongView::edit()
 
 void SongView::del()
 {
-    if (MessageBox::No==MessageBox::warningYesNo(this, i18n("Delete lyrics file?"), i18n("Delete File"),
+    if (MessageBox::No==MessageBox::warningYesNo(this, tr("Delete lyrics file?"), tr("Delete File"),
                                                  StdGuiItem::del(), StdGuiItem::cancel())) {
         return;
     }
@@ -361,10 +337,10 @@ void SongView::curentViewChanged()
         return;
     }
     switch (currentView()) {
-    case Page_Lyrics:      loadLyrics(); break;
-    case Page_Information: loadInfo();   break;
-    case Page_Tags:        loadTags();   break;
-    default:                             break;
+    case Page_Lyrics:      loadLyrics();   break;
+    case Page_Information: loadInfo();     break;
+    case Page_Metadata:    loadMetadata(); break;
+    default:                               break;
     }
 }
 
@@ -375,44 +351,57 @@ void SongView::loadLyrics()
     }
     lyricsNeedsUpdating=false;
 
-    if (!MPDConnection::self()->getDetails().dir.isEmpty() && !currentSong.file.isEmpty() && !currentSong.isNonMPD()) {
-        QString songFile=actualFile(currentSong);
-        QString mpdLyrics=mpdLyricsFilePath(songFile);
+    if (currentSong.isCantataStream() || (!MPDConnection::self()->getDetails().dir.isEmpty() && !currentSong.file.isEmpty() && !currentSong.isNonMPD())) {
+        QString songFile=currentSong.filePath(MPDConnection::self()->getDetails().dir);
+        QString mpdLyrics=mpdLyricsFilePath(currentSong);
 
-        if (MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http:/"))) {
+        if (!currentSong.isCantataStream() && MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http:/"))) {
             QUrl url(mpdLyrics);
             job=NetworkAccessManager::self()->get(url);
             job->setProperty("file", currentSong.file);
             connect(job, SIGNAL(finished()), this, SLOT(downloadFinished()));
             return;
-        } else {
-            #ifdef TAGLIB_FOUND
-            QString tagLyrics=Tags::readLyrics(MPDConnection::self()->getDetails().dir+songFile);
+        }
+    }
 
-            if (!tagLyrics.isEmpty()) {
-                text->setText(fixNewLines(tagLyrics));
-                setMode(Mode_Display);
-//                    controls->setVisible(false);
-                return;
-            }
-            #endif
-            // Stop here if we found lyrics in the cache dir.
-            if (setLyricsFromFile(mpdLyrics)) {
-                lyricsFile=mpdLyrics;
-                setMode(Mode_Display);
-                return;
-            }
+    loadLyricsFromFile();
+}
+
+void SongView::loadLyricsFromFile()
+{
+    if (currentSong.isCantataStream() || (!MPDConnection::self()->getDetails().dir.isEmpty() && !currentSong.file.isEmpty() && !currentSong.isNonMPD())) {
+        QString songFile=currentSong.filePath(MPDConnection::self()->getDetails().dir);
+        QString mpdLyrics=mpdLyricsFilePath(currentSong);
+
+        #ifdef TAGLIB_FOUND
+        QString tagLyrics=Tags::readLyrics(songFile);
+
+        if (!tagLyrics.isEmpty()) {
+            text->setText(fixNewLines(tagLyrics));
+            setMode(Mode_Display);
+            //                    controls->setVisible(false);
+            return;
+        }
+        #endif
+        // Stop here if we found lyrics in the cache dir.
+        if (setLyricsFromFile(mpdLyrics)) {
+            lyricsFile=mpdLyrics;
+            setMode(Mode_Display);
+            return;
+        }
+        // Try .txt extension
+        mpdLyrics=Utils::changeExtension(mpdLyrics, ".txt");
+        if (setLyricsFromFile(mpdLyrics)) {
+            lyricsFile=mpdLyrics;
+            setMode(Mode_Display);
+            return;
         }
     }
 
     // Check for cached file...
     QString file=lyricsCacheFileName(currentSong);
 
-    /*if (force && QFile::exists(file)) {
-        // Delete the cached lyrics file when the user is force-fully re-fetching the lyrics.
-        // Afterwards we'll simply do getLyrics() to get the new ones.
-        QFile::remove(file);
-    } else */if (setLyricsFromFile(file)) {
+    if (setLyricsFromFile(file)) {
        // We just wanted a normal update without explicit re-fetching. We can return
        // here because we got cached lyrics and we don't want an explicit re-fetch.
        lyricsFile=file;
@@ -420,6 +409,26 @@ void SongView::loadLyrics()
        return;
     }
 
+    file=Utils::changeExtension(file, ".txt");
+    if (setLyricsFromFile(file)) {
+       // We just wanted a normal update without explicit re-fetching. We can return
+       // here because we got cached lyrics and we don't want an explicit re-fetch.
+       lyricsFile=file;
+       setMode(Mode_Display);
+       return;
+    }
+
+    #if !defined Q_OS_WIN && !defined Q_OS_MAC
+    file=lyricsOtherFileName(currentSong);
+    if (setLyricsFromFile(file)) {
+       // We just wanted a normal update without explicit re-fetching. We can return
+       // here because we got cached lyrics and we don't want an explicit re-fetch.
+       lyricsFile=file;
+       setMode(Mode_Display);
+       return;
+    }
+    #endif
+
     getLyrics();
 }
 
@@ -429,7 +438,7 @@ void SongView::loadInfo()
         return;
     }
     infoNeedsUpdating=false;
-    foreach (const QString &lang, engine->getLangs()) {
+    for (const QString &lang: engine->getLangs()) {
         QString prefix=engine->getPrefix(lang);
         QString cachedFile=infoCacheFileName(currentSong, prefix, false);
         if (QFile::exists(cachedFile)) {
@@ -450,9 +459,14 @@ void SongView::loadInfo()
     searchForInfo();
 }
 
+static inline QString fixNewLine(QString s)
+{
+    return s.replace(QLatin1String("\n"), QLatin1String("<br/>"));
+}
+
 static QString createRow(const QString &key, const QString &value)
 {
-    return value.isEmpty() ? QString() : QString("<tr><td>%1:&nbsp;</td><td>%2</td></tr>").arg(key).arg(value);
+    return value.isEmpty() ? QString() : QString("<tr><td>%1:&nbsp;</td><td>%2</td></tr>").arg(key).arg(fixNewLine(value));
 }
 
 struct MapEntry {
@@ -461,6 +475,7 @@ struct MapEntry {
     QString str;
 };
 
+#ifdef TAGLIB_FOUND
 static QString clean(QString v)
 {
     if (v.length()>1) {
@@ -469,19 +484,20 @@ static QString clean(QString v)
     }
     return v;
 }
+#endif
 
-void SongView::loadTags()
+void SongView::loadMetadata()
 {
-    if (!tagsNeedsUpdating) {
+    if (!metadataNeedsUpdating) {
         return;
     }
-    tagsNeedsUpdating=false;
+    metadataNeedsUpdating=false;
 
     QMultiMap<int, QString> tags;
     QMultiMap<int, QString> audioProperties;
     #ifdef TAGLIB_FOUND
-    if (!currentSong.isStandardStream() && !MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http:/"))) {
-        QString songFile=actualFile(currentSong);
+    if (currentSong.isCantataStream() || (!currentSong.isStandardStream() && !MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http:/")))) {
+        QString songFile=currentSong.filePath(MPDConnection::self()->getDetails().dir);
         if (!songFile.isEmpty()) {
             static QMap<QString, MapEntry> tagMap;
             static QMap<QString, MapEntry> tagTimeMap;
@@ -491,40 +507,40 @@ void SongView::loadTags()
 
             if (tagMap.isEmpty()) {
                 int pos=0;
-                tagMap.insert(QLatin1String("ARTIST"), MapEntry(pos++, i18n("Artist")));
-                tagMap.insert(QLatin1String("ALBUMARTIST"), MapEntry(pos++, i18n("Album artist")));
-                tagMap.insert(QLatin1String("COMPOSER"), MapEntry(pos++, i18n("Composer")));
+                tagMap.insert(QLatin1String("ARTIST"), MapEntry(pos++, tr("Artist")));
+                tagMap.insert(QLatin1String("ALBUMARTIST"), MapEntry(pos++, tr("Album artist")));
+                tagMap.insert(QLatin1String("COMPOSER"), MapEntry(pos++, tr("Composer")));
                 pos++;// For performer...
-                tagMap.insert(QLatin1String("LYRICIST"), MapEntry(pos++, i18n("Lyricist")));
-                tagMap.insert(QLatin1String("CONDUCTOR"), MapEntry(pos++, i18n("Conductor")));
-                tagMap.insert(QLatin1String("REMIXER"), MapEntry(pos++, i18n("Remixer")));
-                tagMap.insert(QLatin1String("ALBUM"), MapEntry(pos++, i18n("Album")));
-                tagMap.insert(QLatin1String("SUBTITLE"), MapEntry(pos++, i18n("Subtitle")));
-                tagMap.insert(QLatin1String("TRACKNUMBER"), MapEntry(pos++, i18n("Track number")));
-                tagMap.insert(QLatin1String("DISCNUMBER"), MapEntry(pos++, i18n("Disc number")));
-                tagMap.insert(QLatin1String("GENRE"), MapEntry(pos++, i18n("Genre")));
-                tagMap.insert(QLatin1String("DATE"), MapEntry(pos++, i18n("Date")));
-                tagMap.insert(QLatin1String("ORIGINALDATE"), MapEntry(pos++, i18n("Original date")));
-                tagMap.insert(QLatin1String("COMMENT"), MapEntry(pos++, i18n("Comment")));
-                tagMap.insert(QLatin1String("COPYRIGHT"), MapEntry(pos++, i18n("Copyright")));
-                tagMap.insert(QLatin1String("LABEL"), MapEntry(pos++, i18n("Label")));
-                tagMap.insert(QLatin1String("CATALOGNUMBER"), MapEntry(pos++, i18n("Catalogue number")));
-                tagMap.insert(QLatin1String("TITLESORT"), MapEntry(pos++, i18n("Title sort")));
-                tagMap.insert(QLatin1String("ARTISTSORT"), MapEntry(pos++, i18n("Artist sort")));
-                tagMap.insert(QLatin1String("ALBUMARTISTSORT"), MapEntry(pos++, i18n("Album artist sort")));
-                tagMap.insert(QLatin1String("ALBUMSORT"), MapEntry(pos++, i18n("Album sort")));
-                tagMap.insert(QLatin1String("ENCODEDBY"), MapEntry(pos++, i18n("Encoded by")));
-                tagMap.insert(QLatin1String("ENCODING"), MapEntry(pos++, i18n("Encoder")));
-                tagMap.insert(QLatin1String("MOOD"), MapEntry(pos++, i18n("Mood")));
-                tagMap.insert(QLatin1String("MEDIA"), MapEntry(pos++, i18n("Media")));
-                tagMap.insert(constAudio+QLatin1String("BITRATE"), MapEntry(pos++, i18n("Bitrate")));
-                tagMap.insert(constAudio+QLatin1String("SAMPLERATE"), MapEntry(pos++, i18n("Sample rate")));
-                tagMap.insert(constAudio+QLatin1String("CHANNELS"), MapEntry(pos++, i18n("Channels")));
+                tagMap.insert(QLatin1String("LYRICIST"), MapEntry(pos++, tr("Lyricist")));
+                tagMap.insert(QLatin1String("CONDUCTOR"), MapEntry(pos++, tr("Conductor")));
+                tagMap.insert(QLatin1String("REMIXER"), MapEntry(pos++, tr("Remixer")));
+                tagMap.insert(QLatin1String("ALBUM"), MapEntry(pos++, tr("Album")));
+                tagMap.insert(QLatin1String("SUBTITLE"), MapEntry(pos++, tr("Subtitle")));
+                tagMap.insert(QLatin1String("TRACKNUMBER"), MapEntry(pos++, tr("Track number")));
+                tagMap.insert(QLatin1String("DISCNUMBER"), MapEntry(pos++, tr("Disc number")));
+                tagMap.insert(QLatin1String("GENRE"), MapEntry(pos++, tr("Genre")));
+                tagMap.insert(QLatin1String("DATE"), MapEntry(pos++, tr("Date")));
+                tagMap.insert(QLatin1String("ORIGINALDATE"), MapEntry(pos++, tr("Original date")));
+                tagMap.insert(QLatin1String("COMMENT"), MapEntry(pos++, tr("Comment")));
+                tagMap.insert(QLatin1String("COPYRIGHT"), MapEntry(pos++, tr("Copyright")));
+                tagMap.insert(QLatin1String("LABEL"), MapEntry(pos++, tr("Label")));
+                tagMap.insert(QLatin1String("CATALOGNUMBER"), MapEntry(pos++, tr("Catalogue number")));
+                tagMap.insert(QLatin1String("TITLESORT"), MapEntry(pos++, tr("Title sort")));
+                tagMap.insert(QLatin1String("ARTISTSORT"), MapEntry(pos++, tr("Artist sort")));
+                tagMap.insert(QLatin1String("ALBUMARTISTSORT"), MapEntry(pos++, tr("Album artist sort")));
+                tagMap.insert(QLatin1String("ALBUMSORT"), MapEntry(pos++, tr("Album sort")));
+                tagMap.insert(QLatin1String("ENCODEDBY"), MapEntry(pos++, tr("Encoded by")));
+                tagMap.insert(QLatin1String("ENCODING"), MapEntry(pos++, tr("Encoder")));
+                tagMap.insert(QLatin1String("MOOD"), MapEntry(pos++, tr("Mood")));
+                tagMap.insert(QLatin1String("MEDIA"), MapEntry(pos++, tr("Media")));
+                tagMap.insert(constAudio+QLatin1String("BITRATE"), MapEntry(pos++, tr("Bitrate")));
+                tagMap.insert(constAudio+QLatin1String("SAMPLERATE"), MapEntry(pos++, tr("Sample rate")));
+                tagMap.insert(constAudio+QLatin1String("CHANNELS"), MapEntry(pos++, tr("Channels")));
 
-                tagTimeMap.insert(QLatin1String("TAGGING TIME"), MapEntry(pos++, i18n("Tagging time")));
+                tagTimeMap.insert(QLatin1String("TAGGING TIME"), MapEntry(pos++, tr("Tagging time")));
             }
 
-            QMap<QString, QString> allTags=Tags::readAll(MPDConnection::self()->getDetails().dir+actualFile(currentSong));
+            QMap<QString, QString> allTags=Tags::readAll(songFile);
 
             if (!allTags.isEmpty()) {
                 QMap<QString, QString>::ConstIterator it=allTags.constBegin();
@@ -543,7 +559,7 @@ void SongView::loadTags()
                     } else if (tagTimeMap.contains(it.key())) {
                         tags.insert(tagTimeMap[it.key()].val, createRow(tagTimeMap[it.key()].str, QString(it.value()).replace("T", " ")));
                     } else if (it.key().startsWith(constPerformer)) {
-                        tags.insert(3, createRow(i18n("Performer (%1)", clean(it.key().mid(constPerformer.length()))), it.value()));
+                        tags.insert(3, createRow(tr("Performer (%1)").arg(clean(it.key().mid(constPerformer.length()))), it.value()));
                     } else {
                         tags.insert(tagMap.count()+tagTimeMap.count(), createRow(clean(it.key()), it.value()));
                     }
@@ -553,18 +569,34 @@ void SongView::loadTags()
     }
     #endif
 
+    int audioPos=1024;
+    if (audioProperties.isEmpty()) {
+        if (MPDStatus::self()->bitrate()>0) {
+            audioProperties.insert(audioPos++, createRow(tr("Bitrate"), tr("%1 kb/s").arg(MPDStatus::self()->bitrate())));
+        }
+        if (MPDStatus::self()->samplerate()>0) {
+            audioProperties.insert(audioPos++, createRow(tr("Sample rate"), tr("%1 Hz").arg(MPDStatus::self()->samplerate())));
+        }
+        if (MPDStatus::self()->channels()>0) {
+            audioProperties.insert(audioPos++, createRow(tr("Channels"), QString::number(MPDStatus::self()->channels())));
+        }
+    }
+    if (MPDStatus::self()->bits()>0) {
+        audioProperties.insert(audioPos++, createRow(tr("Bits"), QString::number(MPDStatus::self()->bits())));
+    }
+
     if (tags.isEmpty()) {
         int pos=0;
-        tags.insert(pos++, createRow(i18n("Artist"), currentSong.artist));
-        tags.insert(pos++, createRow(i18n("Album artist"), currentSong.albumartist));
-        tags.insert(pos++, createRow(i18n("Composer"), currentSong.composer()));
-        tags.insert(pos++, createRow(i18n("Performer"), currentSong.performer()));
-        tags.insert(pos++, createRow(i18n("Album"), currentSong.album));
-        tags.insert(pos++, createRow(i18n("Track number"), 0==currentSong.track ? QString() : QString::number(currentSong.track)));
-        tags.insert(pos++, createRow(i18n("Disc number"), 0==currentSong.disc ? QString() : QString::number(currentSong.disc)));
-        tags.insert(pos++, createRow(i18n("Genre"), currentSong.genres().join(", ")));
-        tags.insert(pos++, createRow(i18n("Year"), 0==currentSong.track ? QString() : QString::number(currentSong.year)));
-        tags.insert(pos++, createRow(i18n("Comment"), currentSong.comment()));
+        tags.insert(pos++, createRow(tr("Artist"), currentSong.artist));
+        tags.insert(pos++, createRow(tr("Album artist"), currentSong.albumartist));
+        tags.insert(pos++, createRow(tr("Composer"), currentSong.composer()));
+        tags.insert(pos++, createRow(tr("Performer"), currentSong.performer()));
+        tags.insert(pos++, createRow(tr("Album"), currentSong.album));
+        tags.insert(pos++, createRow(tr("Track number"), 0==currentSong.track ? QString() : QString::number(currentSong.track)));
+        tags.insert(pos++, createRow(tr("Disc number"), 0==currentSong.disc ? QString() : QString::number(currentSong.disc)));
+        tags.insert(pos++, createRow(tr("Genre"), currentSong.displayGenre()));
+        tags.insert(pos++, createRow(tr("Year"), 0==currentSong.track ? QString() : QString::number(currentSong.year)));
+        tags.insert(pos++, createRow(tr("Comment"), fixNewLine(currentSong.comment())));
     }
 
     QString tagInfo;
@@ -593,11 +625,21 @@ void SongView::loadTags()
             }
         }
     }
-    if (!tagInfo.isEmpty()) {
-        tagInfo+=QLatin1String("</table>");
+    if (tagInfo.isEmpty()) {
+        tagInfo=QLatin1String("<table>");
+    } else {
+        tagInfo+=QLatin1String("<tr/>");
     }
+    QString songFile=currentSong.filePath(MPDConnection::self()->getDetails().dir);
+    if (QFile::exists(songFile)) {
+        tagInfo+=createRow(tr("Filename"), QLatin1String("<a href=\"file://")+songFile+QLatin1String("\">")+
+                                           songFile+QLatin1String("</a>"));
+    } else {
+        tagInfo+=createRow(tr("Filename"), songFile);
+    }
+    tagInfo+=QLatin1String("</table>");
 
-    setHtml(tagInfo, Page_Tags);
+    setHtml(tagInfo, Page_Metadata);
 }
 
 void SongView::refreshInfo()
@@ -605,7 +647,7 @@ void SongView::refreshInfo()
     if (currentSong.isEmpty()) {
         return;
     }
-    foreach (const QString &lang, engine->getLangs()) {
+    for (const QString &lang: engine->getLangs()) {
         QFile::remove(infoCacheFileName(currentSong, engine->getPrefix(lang), false));
     }
     searchForInfo();
@@ -614,7 +656,7 @@ void SongView::refreshInfo()
 void SongView::searchForInfo()
 {
     cancelInfoJobAction->setEnabled(true);
-    engine->search(QStringList() << currentSong.artist << currentSong.title, ContextEngine::Track);
+    engine->search(QStringList() << currentSong.basicArtist() << currentSong.basicTitle(), ContextEngine::Track);
     showSpinner(false);
 }
 
@@ -662,18 +704,16 @@ void SongView::abort()
 {
     if (job) {
         job->cancelAndDelete();
-        job=0;
+        job=nullptr;
     }
     currentProvider=-1;
     if (currentProv) {
         currentProv->abort();
-        currentProv=0;
+        currentProv=nullptr;
 
         text->setText(QString());
         // Set lyrics file anyway - so that editing is enabled!
-        lyricsFile=Settings::self()->storeLyricsInMpdDir() && !currentSong.isNonMPD()
-                ? mpdLyricsFilePath(currentSong)
-                : lyricsCacheFileName(currentSong);
+        lyricsFile=lyricsCacheFileName(currentSong);
         setMode(Mode_Display);
     }
     cancelJobAction->setEnabled(false);
@@ -684,7 +724,7 @@ void SongView::update(const Song &s, bool force)
 {
     if (s.isEmpty() || s.title.isEmpty() || s.artist.isEmpty()) {
         currentSong=s;
-        infoNeedsUpdating=tagsNeedsUpdating=lyricsNeedsUpdating=false;
+        infoNeedsUpdating=metadataNeedsUpdating=lyricsNeedsUpdating=false;
         clear();
         abort();
         return;
@@ -722,7 +762,7 @@ void SongView::update(const Song &s, bool force)
             currentProvider=-1;
         }
 
-        infoNeedsUpdating=tagsNeedsUpdating=lyricsNeedsUpdating=true;
+        infoNeedsUpdating=metadataNeedsUpdating=lyricsNeedsUpdating=true;
         setHeader(song.title);
         curentViewChanged();
     }
@@ -734,7 +774,7 @@ void SongView::downloadFinished()
     if (reply) {
         reply->deleteLater();
         if (job==reply) {
-            job=0;
+            job=nullptr;
             if (reply->ok()) {
                 QString file=reply->property("file").toString();
                 if (!file.isEmpty() && file==currentSong.file) {
@@ -750,7 +790,7 @@ void SongView::downloadFinished()
             }
         }
     }
-    getLyrics();
+    loadLyricsFromFile();
 }
 
 void SongView::lyricsReady(int id, QString lyrics)
@@ -776,10 +816,7 @@ void SongView::lyricsReady(int id, QString lyrics)
         } else {
             text->setText(fixNewLines(plain));
             lyricsFile=QString();
-            if (! ( Settings::self()->storeLyricsInMpdDir() && !currentSong.isNonMPD() &&
-                    saveFile(mpdLyricsFilePath(currentSong))) ) {
-                saveFile(lyricsCacheFileName(currentSong, true));
-            }
+            saveFile(lyricsCacheFileName(currentSong, true));
             setMode(Mode_Display);
         }
     }
@@ -790,7 +827,12 @@ bool SongView::saveFile(const QString &fileName)
     QFile f(fileName);
 
     if (f.open(QIODevice::WriteOnly)) {
-        QTextStream(&f) << text->toPlainText();
+        QTextStream stream(&f);
+        #ifdef Q_OS_WIN
+        stream.setCodec("UTF-8");
+        stream.setGenerateByteOrderMark(true);
+        #endif
+        stream << text->toPlainText();
         f.close();
         lyricsFile=fileName;
         return true;
@@ -814,16 +856,14 @@ void SongView::getLyrics()
 {
     currentProv=UltimateLyrics::self()->getNext(currentProvider);
     if (currentProv) {
-        text->setText(i18n("Fetching lyrics via %1", currentProv->displayName()));
+        text->setText(tr("Fetching lyrics via %1").arg(currentProv->displayName()));
         currentProv->fetchInfo(currentRequest, currentSong);
         showSpinner();
     } else {
         text->setText(QString());
         currentProvider=-1;
         // Set lyrics file anyway - so that editing is enabled!
-        lyricsFile=Settings::self()->storeLyricsInMpdDir() && !currentSong.isNonMPD()
-                ? mpdLyricsFilePath(currentSong)
-                : lyricsCacheFileName(currentSong);
+        lyricsFile=lyricsCacheFileName(currentSong);
         setMode(Mode_Display);
     }
 }
@@ -857,7 +897,6 @@ bool SongView::setLyricsFromFile(const QString &filePath)
     if (f.exists() && f.open(QIODevice::ReadOnly)) {
         // Read the file using a QTextStream so we get automatic UTF8 detection.
         QTextStream inputStream(&f);
-
         text->setText(fixNewLines(inputStream.readAll()));
         cancelJobAction->setEnabled(false);
         hideSpinner();
@@ -868,3 +907,5 @@ bool SongView::setLyricsFromFile(const QString &filePath)
 
     return false;
 }
+
+#include "moc_songview.cpp"
