@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -47,7 +47,7 @@ class SongView : public View
     enum Pages {
         Page_Lyrics,
         Page_Information,
-        Page_Tags
+        Page_Metadata
     };
 
 public:
@@ -57,9 +57,9 @@ public:
     static const QLatin1String constInfoExt;
 
     SongView(QWidget *p);
-    ~SongView();
+    ~SongView() override;
 
-    void update(const Song &s, bool force=false);
+    void update(const Song &s, bool force=false) override;
     void saveConfig();
 
 Q_SIGNALS:
@@ -87,11 +87,12 @@ private Q_SLOTS:
 
 private:
     void loadLyrics();
+    void loadLyricsFromFile();
     void loadInfo();
-    void loadTags();
+    void loadMetadata();
     void searchForInfo();
     void hideSpinner();
-    void abort();
+    void abort() override;
     QString mpdFileName() const;
     QString cacheFileName() const;
     void getLyrics();
@@ -128,7 +129,7 @@ private:
 
     bool lyricsNeedsUpdating;
     bool infoNeedsUpdating;
-    bool tagsNeedsUpdating;
+    bool metadataNeedsUpdating;
     Action *refreshInfoAction;
     Action *cancelInfoJobAction;
     ContextEngine *engine;
