@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,10 +22,9 @@
  */
 
 #include "togglelist.h"
-#include "support/localize.h"
-#include "support/icon.h"
 #include "gui/settings.h"
 #include "widgets/basicitemdelegate.h"
+#include "widgets/icons.h"
 #include "support/utils.h"
 
 ToggleList::ToggleList(QWidget *p)
@@ -38,11 +37,11 @@ ToggleList::ToggleList(QWidget *p)
     connect(removeButton, SIGNAL(clicked()), SLOT(remove()));
     connect(available, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), SLOT(availableChanged(QListWidgetItem*)));
     connect(selected, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), SLOT(selectedChanged(QListWidgetItem*)));
-    upButton->setIcon(Icon("go-up"));
-    downButton->setIcon(Icon("go-down"));
+    upButton->setIcon(Icons::self()->upIcon);
+    downButton->setIcon(Icons::self()->downIcon);
     bool rtl=isRightToLeft();
-    addButton->setIcon(Icon(rtl ? "go-previous" : "go-next"));
-    removeButton->setIcon(Icon(rtl ? "go-next" : "go-previous"));
+    addButton->setIcon(rtl ? Icons::self()->leftIcon : Icons::self()->rightIcon);
+    removeButton->setIcon(rtl ? Icons::self()->rightIcon : Icons::self()->leftIcon);
 
     upButton->setEnabled(false);
     downButton->setEnabled(false);
@@ -52,9 +51,6 @@ ToggleList::ToggleList(QWidget *p)
     available->setItemDelegate(new BasicItemDelegate(available));
     selected->setAlternatingRowColors(false);
     selected->setItemDelegate(new BasicItemDelegate(selected));
-    QSize sz=Utils::touchFriendly() ? removeButton->sizeHint() : QSize(0, 0);
-    spacerA->changeSize(sz.width(), sz.height());
-    spacerB->changeSize(sz.width(), sz.height());
 }
 
 void ToggleList::moveUp()
@@ -119,3 +115,5 @@ void ToggleList::selectedChanged(QListWidgetItem *item)
     }
     removeButton->setEnabled(item);
 }
+
+#include "moc_togglelist.cpp"
