@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -30,11 +30,10 @@
 #include "support/actioncollection.h"
 #include "support/action.h"
 #include "widgets/icons.h"
-#include "support/localize.h"
 #include <QLabel>
 #include <QScrollBar>
 #include <QImage>
-#include <QGridLayout>
+#include <QVBoxLayout>
 #include <QNetworkReply>
 #include <QLocale>
 #include <QBuffer>
@@ -44,10 +43,9 @@
 #include <QMenu>
 #include <QMouseEvent>
 #include <QWheelEvent>
+#include <QTimer>
 
-// Uncomment this #define to have header labels an images centered. Thsi is disabled, as the image
-// centering only works if there is some text larger than 1 line to be displayed underneath :-(
-//#define CONTEXT_CENTERED
+#define CONTEXT_CENTERED
 
 static QString headerTag;
 QString View::subTag;
@@ -59,7 +57,7 @@ QString View::encode(const QImage &img)
     buffer.open(QIODevice::WriteOnly);
     img.save(&buffer, "PNG");
     #ifdef CONTEXT_CENTERED
-    return QString("<tr><td align=\"center\"><img src=\"data:image/png;base64,%1\"/></td></tr>").arg(QString(buffer.data().toBase64()));
+    return QString("<table width=\"100%\"><tr><td align=\"center\"><img src=\"data:image/png;base64,%1\"/></td></tr></table>").arg(QString(buffer.data().toBase64()));
     #else
     return QString("<img src=\"data:image/png;base64,%1\"/>").arg(QString(buffer.data().toBase64()));
     #endif
@@ -87,93 +85,14 @@ static TextBrowser * createView(QWidget *parent)
     return text;
 }
 
-ViewTextSelector::ViewTextSelector(QWidget *p)
-    : QLabel(p)
-    , current(0)
-{
-    setAttribute(Qt::WA_Hover, true);
-    menu=new QMenu(this);
-    setStyleSheet(QLatin1String("QLabel:hover {color:palette(highlight);}"));
-    setMargin(Utils::scaleForDpi(2));
-}
-
-static QString viewText(const QString &s)
-{
-    // 0x25BE is unicode for small down arrow...
-    return QLatin1String("<b>")+s+QLatin1String("  ")+QChar(0x25BE)+QLatin1String("</b>");
-}
-
-void ViewTextSelector::addItem(const QString &t)
-{
-    menu->addAction(t, this, SLOT(itemSelected()))->setData(items.count());
-    if (text().isEmpty()) {
-        setText(viewText(t));
-        current=items.count();
-    }
-    items.append(t);
-}
-
-void ViewTextSelector::itemSelected()
-{
-    QAction *act=qobject_cast<QAction *>(sender());
-    if (act) {
-        setCurrentIndex(act->data().toInt());
-    }
-}
-
-bool ViewTextSelector::event(QEvent *e)
-{
-    switch (e->type()) {
-    case QEvent::MouseButtonPress:
-        if (Qt::NoModifier==static_cast<QMouseEvent *>(e)->modifiers() && Qt::LeftButton==static_cast<QMouseEvent *>(e)->button()) {
-            menu->exec(mapToGlobal(static_cast<QMouseEvent *>(e)->pos()));
-        }
-    case QEvent::Wheel: {
-        int numDegrees = static_cast<QWheelEvent *>(e)->delta() / 8;
-        int numSteps = numDegrees / 15;
-        int newIndex = current;
-        if (numSteps > 0) {
-            for (int i = 0; i < numSteps; ++i) {
-                newIndex++;
-                if (newIndex>=items.count()) {
-                    newIndex=0;
-                }
-            }
-        } else {
-            for (int i = 0; i > numSteps; --i) {
-                newIndex--;
-                if (newIndex<0) {
-                    newIndex=items.count()-1;
-                }
-            }
-        }
-        setCurrentIndex(newIndex);
-        break;
-    }
-    default:
-        break;
-    }
-    return QLabel::event(e);
-}
-
-void ViewTextSelector::setCurrentIndex(int v)
-{
-    if (v<0 || v>=items.count() || v==current) {
-        return;
-    }
-    current=v;
-    setText(viewText(items.at(current)));
-    emit activated(current);
-}
-
 View::View(QWidget *parent, const QStringList &views)
     : QWidget(parent)
     , needToUpdate(false)
-    , spinner(0)
-    , selector(0)
-    , stack(0)
+    , spinner(nullptr)
+    , selector(nullptr)
+    , stack(nullptr)
 {
-    QGridLayout *layout=new QGridLayout(this);
+    QVBoxLayout *layout=new QVBoxLayout(this);
     header=new QLabel(this);
 
     if (views.isEmpty()) {
@@ -181,10 +100,11 @@ View::View(QWidget *parent, const QStringList &views)
         texts.append(t);
     } else {
         stack=new QStackedWidget(this);
-        selector=new ViewTextSelector(this);
-        foreach (const QString &v, views) {
+        selector=new SelectorLabel(this);
+        selector->setUseArrow(true);
+        for (const QString &v: views) {
             TextBrowser *t=createView(stack);
-            selector->addItem(v);
+            selector->addItem(v, v);
             stack->addWidget(t);
             texts.append(t);
         }
@@ -192,29 +112,29 @@ View::View(QWidget *parent, const QStringList &views)
         connect(selector, SIGNAL(activated(int)), this, SIGNAL(viewChanged()));
     }
 
-    layout->setMargin(0);
     header->setWordWrap(true);
     header->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+    header->setTextInteractionFlags(Qt::TextSelectableByMouse|Qt::TextSelectableByKeyboard);
 
-    int row=0;
-    layout->addItem(new QSpacerItem(1, layout->spacing(), QSizePolicy::Fixed, QSizePolicy::Fixed), row++, 0);
-    layout->addWidget(header, row++, 0, 1, 2);
+    layout->setMargin(2);
+    layout->addWidget(header);
     if (views.isEmpty()) {
-        layout->addWidget(texts.at(0), row++, 0, 1, 2);
+        layout->addWidget(texts.at(0));
     } else {
-        layout->addWidget(selector, row, 0);
-        layout->addItem(new QSpacerItem(1, 1, QSizePolicy::MinimumExpanding, QSizePolicy::Fixed), row++, 1);
-        layout->addWidget(stack, row++, 0, 1, 2);
+        layout->addWidget(selector);
+        layout->addItem(new QSpacerItem(1, 1, QSizePolicy::MinimumExpanding, QSizePolicy::Fixed));
+        layout->addWidget(stack);
     }
-    layout->addItem(new QSpacerItem(1, fontMetrics().height()/4, QSizePolicy::Fixed, QSizePolicy::Fixed), row++, 0);
+    layout->addItem(new QSpacerItem(1, fontMetrics().height()/4, QSizePolicy::Fixed, QSizePolicy::Fixed));
     if (headerTag.isEmpty()) {
         initHeaderTags();
     }
 
-    cancelJobAction=new Action(Icons::self()->cancelIcon, i18n("Cancel"), this);
+    cancelJobAction=new Action(Icons::self()->cancelIcon, tr("Cancel"), this);
     cancelJobAction->setEnabled(false);
     connect(cancelJobAction, SIGNAL(triggered()), SLOT(abort()));
     text=texts.at(0);
+    setMinimumWidth(Utils::scaleForDpi(300));
 }
 
 View::~View()
@@ -225,7 +145,7 @@ View::~View()
 void View::clear()
 {
     setHeader(stdHeader);
-    foreach (TextBrowser *t, texts) {
+    for (TextBrowser *t: texts) {
         t->clear();
     }
 }
@@ -233,7 +153,7 @@ void View::clear()
 void View::setHeader(const QString &str)
 {
     #ifdef CONTEXT_CENTERED
-    header->setText("<"+headerTag+" align=\"center\">"+str+"</"+headerTag+">");
+    header->setText("<"+headerTag+" width=\"100%\" align=\"center\">"+str+"</"+headerTag+">");
     #else
     header->setText("<"+headerTag+">"+str+"</"+headerTag+">");
     #endif
@@ -241,7 +161,7 @@ void View::setHeader(const QString &str)
 
 void View::setPicSize(const QSize &sz)
 {
-    foreach (TextBrowser *t, texts) {
+    for (TextBrowser *t: texts) {
         t->setPicSize(sz);
     }
 }
@@ -255,7 +175,7 @@ QString View::createPicTag(const QImage &img, const QString &file)
 {
     if (!file.isEmpty() && QFile::exists(file)) {
         #ifdef CONTEXT_CENTERED
-        return QString("<tr><td align=\"center\"><img src=\"%1\"/></td></tr>").arg(file);
+        return QString("<table width=\"100%\"><tr><td align=\"center\"><img src=\"%1\"/></td></tr></table>").arg(file);
         #else
         return QString("<img src=\"%1\"/>").arg(file);
         #endif
@@ -264,7 +184,7 @@ QString View::createPicTag(const QImage &img, const QString &file)
         return QString();
     }
     // No filename given, or file does not exist - therefore encode & scale image.
-    return encode(img.scaled(texts.at(0)->picSize(), Qt::KeepAspectRatio, Qt::SmoothTransformation));
+    return encode(img);
 }
 
 void View::showEvent(QShowEvent *e)
@@ -309,7 +229,7 @@ void View::setEditable(bool e, int index)
 
 void View::setPal(const QPalette &pal, const QColor &linkColor, const QColor &prevLinkColor)
 {
-    foreach (TextBrowser *t, texts) {
+    for (TextBrowser *t: texts) {
         // QTextBrowser seems to save link colour within the HTML, so we need to manually
         // update this when the palette changes!
         QString old=t->toHtml();
@@ -324,13 +244,14 @@ void View::setPal(const QPalette &pal, const QColor &linkColor, const QColor &pr
     header->setPalette(hdrPal);
     if (selector) {
         selector->setPalette(hdrPal);
+        selector->setColor(pal.text().color());
     }
 }
 
 void View::addEventFilter(QObject *obj)
 {
     installEventFilter(obj);
-    foreach (TextBrowser *t, texts) {
+    for (TextBrowser *t: texts) {
         t->installEventFilter(obj);
         t->viewport()->installEventFilter(obj);
     }
@@ -339,7 +260,7 @@ void View::addEventFilter(QObject *obj)
 
 void View::setZoom(int z)
 {
-    foreach (TextBrowser *t, texts) {
+    for (TextBrowser *t: texts) {
         t->setZoom(z);
     }
     QFont f=header->font();
@@ -367,3 +288,12 @@ void View::setHtml(const QString &h, int index)
 void View::abort()
 {
 }
+
+void View::setScaleImage(bool s)
+{
+    for (TextBrowser *t: texts) {
+        t->setFullWidthImage(s);
+    }
+}
+
+#include "moc_view.cpp"
