@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -27,7 +27,8 @@
 #include <QWidget>
 #include <QSize>
 #include <QLabel>
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
+#include "widgets/selectorlabel.h"
 
 class QImage;
 class Spinner;
@@ -38,28 +39,6 @@ class Action;
 class QStackedWidget;
 class QMenu;
 
-class ViewTextSelector : public QLabel
-{
-    Q_OBJECT
-public:
-    ViewTextSelector(QWidget *p);
-    void addItem(const QString &t);
-    bool event(QEvent *e);
-    int currentIndex() const { return current; }
-    void setCurrentIndex(int v);
-
-Q_SIGNALS:
-    void activated(int);
-
-private Q_SLOTS:
-    void itemSelected();
-
-private:
-    int current;
-    QStringList items;
-    QMenu *menu;
-};
-
 class View : public QWidget
 {
     Q_OBJECT
@@ -67,7 +46,7 @@ public:
     static QString subTag;
 
     View(QWidget *p, const QStringList &views=QStringList());
-    virtual ~View();
+    ~View() override;
 
     static QString encode(const QImage &img);
     static QString subHeader(const QString &str) { return "<"+subTag+">"+str+"</"+subTag+">"; }
@@ -79,7 +58,7 @@ public:
     void setPicSize(const QSize &sz);
     QSize picSize() const;
     QString createPicTag(const QImage &img, const QString &file);
-    void showEvent(QShowEvent *e);
+    void showEvent(QShowEvent *e) override;
     void showSpinner(bool enableCancel=true);
     void hideSpinner(bool disableCancel=true);
     void setEditable(bool e, int index=0);
@@ -97,6 +76,7 @@ Q_SIGNALS:
 
 protected Q_SLOTS:
     virtual void abort();
+    void setScaleImage(bool s);
 
 protected:
     Song currentSong;
@@ -106,7 +86,7 @@ protected:
     Spinner *spinner;
     Action *cancelJobAction;
 
-    ViewTextSelector *selector;
+    SelectorLabel *selector;
     QStackedWidget *stack;
     TextBrowser *text; // short-cut to first text item...
     QList<TextBrowser *> texts;
