@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,13 +22,11 @@
  */
 
 #include "wikipediaengine.h"
-#include "networkaccessmanager.h"
-#include "localize.h"
-#include "settings.h"
+#include "network/networkaccessmanager.h"
+#include "gui/settings.h"
+#include "gui/covers.h"
 #include "config.h"
-#if QT_VERSION >= 0x050000
 #include <QUrlQuery>
-#endif
 #include <QXmlStreamReader>
 #include <QRegExp>
 
@@ -43,6 +41,37 @@ void WikipediaEngine::enableDebug()
 static const char * constModeProperty="mode";
 static const char * constQueryProperty="query";
 
+static QString wikipediaSpecialExport(const QString &lang)
+{
+    static QMap<QString, QString> links;
+
+    if (links.isEmpty()) {
+        links.insert(QLatin1String("de"), QString("Spezial:Exportieren"));
+        links.insert(QLatin1String("ru"), QString("Служебная:Экспорт"));
+    }
+
+    QMap<QString, QString>::ConstIterator it=links.find(lang);
+    return "/"+(it==links.constEnd() ? QLatin1String("Special:Export") : it.value())+"/";
+}
+
+static QString fixWikiLink(const QUrl &url)
+{
+    QString lang=url.host().split(".").first();
+    QString path=url.path();
+    QString fixed(path);
+    fixed=fixed.replace("/wiki"+wikipediaSpecialExport(lang), "/wiki/");
+    if (path==fixed) {
+        QStringList parts=fixed.split("/", QString::SkipEmptyParts);
+        if (parts.length()>1) {
+            parts.removeAt(1);
+            fixed=parts.join("/");
+        }
+    }
+    QUrl u(url);
+    u.setPath(fixed);
+    return u.toString();
+}
+
 static QString strip(const QString &string, QString open, QString close, QString inner=QString())
 {
     QString result;
@@ -110,8 +139,8 @@ static QString strip(const QString &string, QString open, QString close, QString
 static QString stripEmptySections(QString answer)
 {
     QStringList headers=QStringList() << "h3" << "h2" << "b";
-    foreach (const QString &h1, headers) {
-        foreach (const QString &h2, headers) {
+    for (const QString &h1: headers) {
+        for (const QString &h2: headers) {
             int end=-1;
             do {
                 end=answer.indexOf("</"+h1+"><"+h2+">");
@@ -138,7 +167,7 @@ static QString stripLastEmptySection(QString answer)
     bool modified=false;
     do {
         modified=false;
-        foreach (const QString &h, headers) {
+        for (const QString &h: headers) {
             if (answer.endsWith("</"+h+">") || answer.endsWith("</"+h+"> ") || answer.endsWith("</"+h+">  ")) {
                 int start=answer.lastIndexOf("<"+h+">", answer.length()-4);
                 if (-1!=start) {
@@ -153,8 +182,7 @@ static QString stripLastEmptySection(QString answer)
 
 static QString wikiToHtml(QString answer, bool introOnly, const QUrl &url)
 {
-    QString u=url.toString();
-    u.replace("/wiki/Special:Export/", "/wiki/");
+    QString u=fixWikiLink(url);
     int start = answer.indexOf('>', answer.indexOf("<text"))+1;
     int end = answer.lastIndexOf(QRegExp("\\n[^\\n]*\\n\\{\\{reflist", Qt::CaseInsensitive));
     if (end < start) {
@@ -220,6 +248,7 @@ static QString wikiToHtml(QString answer, bool introOnly, const QUrl &url)
         answer.replace("==", "¬").replace(QRegExp("¬([^¬]*)¬"), "<h2>\\1</h2>");
     }
     answer.replace("&amp;nbsp;", " ");
+    answer.replace("&ndash;", "-");
     answer.replace("<br><h", "<h");
     if (introOnly) {
         end=answer.indexOf("==", 3);
@@ -245,7 +274,7 @@ static QString wikiToHtml(QString answer, bool introOnly, const QUrl &url)
         answer.replace("<br><br><br>", "<br><br>");
 
         // Remove track listings - we take these from MPD...
-        QString listingText="<h2>"+i18n("Track listing")+"</h2>";
+        QString listingText="<h2>"+QObject::tr("Track listing")+"</h2>";
         start=answer.indexOf(listingText, 0, Qt::CaseInsensitive);
         if (-1!=start) {
             int end=answer.indexOf("<h2>", start+listingText.length(), Qt::CaseInsensitive);
@@ -298,14 +327,18 @@ void WikipediaEngine::setPreferedLangs(const QStringList &l)
 
 QString WikipediaEngine::translateLinks(QString text) const
 {
-    text=text.replace(constReadMorePlaceholder, i18n("Read more on wikipedia"));
-    text=text.replace(constOpenInBrowserPlaceholder, i18n("Open in browser"));
+    text=text.replace(constReadMorePlaceholder, QObject::tr("Read more on wikipedia"));
+    text=text.replace(constOpenInBrowserPlaceholder, QObject::tr("Open in browser"));
     return text;
 }
 
 void WikipediaEngine::search(const QStringList &query, Mode mode)
 {
     titles.clear();
+//    if (Track==mode) {
+//        emit searchResult(QString(), QString());
+//        return;
+//    }
     requestTitles(fixQuery(query), mode, getPrefix(preferredLangs.first()));
 }
 
@@ -313,11 +346,7 @@ void WikipediaEngine::requestTitles(const QStringList &query, Mode mode, const Q
 {
     cancel();
     QUrl url("https://"+lang+".wikipedia.org/w/api.php");
-    #if QT_VERSION < 0x050000
-    QUrl &q=url;
-    #else
     QUrlQuery q;
-    #endif
 
     q.addQueryItem(QLatin1String("action"), QLatin1String("query"));
     q.addQueryItem(QLatin1String("list"), QLatin1String("search"));
@@ -326,9 +355,7 @@ void WikipediaEngine::requestTitles(const QStringList &query, Mode mode, const Q
     q.addQueryItem(QLatin1String("srredirects"), QString::number(1));
     q.addQueryItem(QLatin1String("srlimit"), QString::number(20));
     q.addQueryItem(QLatin1String("format"), QLatin1String("xml"));
-    #if QT_VERSION >= 0x050000
     url.setQuery(q);
-    #endif
 
     job=NetworkAccessManager::self()->get(url);
     job->setProperty(constModeProperty, (int)mode);
@@ -389,6 +416,7 @@ void WikipediaEngine::parseTitles()
         return;
     }
 
+    DBUG << titles;
     getPage(query, mode, hostLang);
 }
 
@@ -409,24 +437,25 @@ void WikipediaEngine::getPage(const QStringList &query, Mode mode, const QString
     QStringList queryCopy(query);
     QStringList queries;
     QStringList simplifiedTitles;
-    foreach (QString t, titles) {
+    for (QString t: titles) {
         simplifiedTitles.append(t.simplified());
     }
 
-    while(!queryCopy.isEmpty()) {
-        QString q=queryCopy.join(" ");
-        QString q2=q;
-        q2.remove("."); // A.S.A.P. -> ASAP
-        queries.append(q);
-        if (q2!=q) {
-            queries.append(q2);
-        }
+    QMap<QString, QString> replacements;
+    replacements.insert(QLatin1String("."), QString()); // A.S.A.P. -> ASAP
+    replacements.insert(QLatin1String("-"), QLatin1String("/")); // AC-DC -> AC/DC
+    QMap<QString, QString>::ConstIterator repEnd=replacements.constEnd();
 
-        q2=q;
-        q2.replace("-", "/"); // AC-DC -> AC/DC
+    while (!queryCopy.isEmpty()) {
+        QString q=queryCopy.join(" ");
         queries.append(q);
-        if (q2!=q) {
-            queries.append(q2);
+
+        for (QMap<QString, QString>::ConstIterator rep=replacements.constBegin(); rep!=repEnd; ++rep) {
+            QString q2=q;
+            q2.replace(rep.key(), rep.value());
+            if (q2!=q) {
+                queries.append(q2);
+            }
         }
 
         queryCopy.takeFirst();
@@ -438,16 +467,20 @@ void WikipediaEngine::getPage(const QStringList &query, Mode mode, const QString
     switch (mode) {
     default:
     case Artist:
-        patterns=i18nc("Search pattern for an artist or band, separated by |", "artist|band|singer|vocalist|musician").split("|", QString::SkipEmptyParts);
+        patterns=tr("artist|band|singer|vocalist|musician", "Search pattern for an artist or band, separated by |").split("|", QString::SkipEmptyParts);
         englishPatterns=QString(QLatin1String("artist|band|singer|vocalist|musician")).split("|");
         break;
     case Album:
-        patterns=i18nc("Search pattern for an album, separated by |", "album|score|soundtrack").split("|", QString::SkipEmptyParts);
+        patterns=tr("album|score|soundtrack", "Search pattern for an album, separated by |").split("|", QString::SkipEmptyParts);
         englishPatterns=QString(QLatin1String("album|score|soundtrack")).split("|");
         break;
+    case Track:
+//        patterns=trc("Search pattern for a song, separated by |", "song|track").split("|", QString::SkipEmptyParts);
+//        englishPatterns=QString(QLatin1String("song|track")).split("|");
+        break;
     }
 
-    foreach (const QString &eng, englishPatterns) {
+    for (const QString &eng: englishPatterns) {
         if (!patterns.contains(eng)) {
             patterns.append(eng);
         }
@@ -456,23 +489,24 @@ void WikipediaEngine::getPage(const QStringList &query, Mode mode, const QString
     DBUG <<  "Titles" << titles;
 
     int index=-1;
-    if (mode==Album && 2==query.count()) {
-        DBUG <<  "Check album";
-        foreach (const QString &pattern, patterns) {
+    if ((mode==Album || mode==Track) && 2==query.count()) {
+        DBUG <<  "Check track/album";
+        for (const QString &pattern: patterns) {
             QString q=query.at(1)+" ("+query.at(0)+" "+pattern+")";
             DBUG <<  "Try" << q;
             index=indexOf(simplifiedTitles, q);
             if (-1!=index) {
-                DBUG <<  "Matched with '$album ($artist pattern)" << index << q;
+                DBUG <<  "Matched with '$album/$track ($artist pattern)" << index << q;
                 break;
             }
         }
     }
+
     if (-1==index) {
-        foreach (const QString &q, queries) {
+        for (const QString &q: queries) {
             DBUG <<  "Query" << q;
             // First check original query with one of the patterns...
-            foreach (const QString &pattern, patterns) {
+            for (const QString &pattern: patterns) {
                 index=indexOf(simplifiedTitles, q+" ("+pattern+")");
                 if (-1!=index) {
                     DBUG <<  "Matched with pattern" << index << QString(q+" ("+pattern+")");
@@ -511,7 +545,7 @@ void WikipediaEngine::getPage(const QStringList &query, Mode mode, const QString
     QUrl url;
     url.setScheme(QLatin1String("https"));
     url.setHost(lang+".wikipedia.org");
-    url.setPath("/wiki/Special:Export/"+title);
+    url.setPath("/wiki"+wikipediaSpecialExport(lang)+title);
     job=NetworkAccessManager::self()->get(url);
     job->setProperty(constModeProperty, (int)mode);
     job->setProperty(constQueryProperty, query);
@@ -539,9 +573,10 @@ void WikipediaEngine::parsePage()
     QUrl url=reply->url();
     QString hostLang=getLang(url);
 
-    if (answer.contains(QLatin1String("{{disambiguation}}")) || answer.contains(QLatin1String("{{disambig}}"))) { // i18n???
-        getPage(reply->property(constQueryProperty).toStringList(), (Mode)reply->property(constModeProperty).toInt(),
-                hostLang);
+    QStringList query=reply->property(constQueryProperty).toStringList();
+    Mode mode=(Mode)reply->property(constModeProperty).toInt();
+    if (answer.contains(QLatin1String("{{disambiguation}}")) || answer.contains(QLatin1String("{{disambig}}"))) { // tr???
+        getPage(query, mode, hostLang);
         return;
     }
 
@@ -553,5 +588,13 @@ void WikipediaEngine::parsePage()
     if (introOnly && resp.isEmpty()) {
         resp=wikiToHtml(answer, false, reply->url());
     }
-    emit searchResult(resp, hostLang);
+
+    // For track results, ensure response contains artist name!
+    if (Track==mode && !resp.contains(query.at(0), Qt::CaseInsensitive) && !resp.contains(Covers::fixArtist(query.at(0)), Qt::CaseInsensitive)) {
+        getPage(query, mode, hostLang);
+    } else {
+        emit searchResult(resp, hostLang);
+    }
 }
+
+#include "moc_wikipediaengine.cpp"
