@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -38,7 +38,8 @@ class GnomeMediaKeys : public MultiMediaKeysInterface
 public:
     GnomeMediaKeys(QObject *p);
 
-    void activate(bool a);
+    bool activate() override;
+    void deactivate() override;
 
 private:
     bool daemonIsRunning();
@@ -56,6 +57,7 @@ private:
     OrgGnomeSettingsDaemonInterface *daemon;
     OrgGnomeSettingsDaemonMediaKeysInterface *mk;
     QDBusServiceWatcher *watcher;
+    QString serviceName;
 };
 
 #endif
