@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -28,8 +28,8 @@
 #include <QStringList>
 #include <QVariantMap>
 #include <QApplication>
-#include "mpd/song.h"
-#include "mpd/mpdstatus.h"
+#include "mpd-interface/song.h"
+#include "mpd-interface/mpdstatus.h"
 #include "gui/stdactions.h"
 
 class QDBusObjectPath;
@@ -66,42 +66,20 @@ class Mpris : public QObject
 
 public:
     Mpris(QObject *p);
-    virtual ~Mpris() { }
+    ~Mpris() override;
 
     // org.mpris.MediaPlayer2.Player
     void Next() { StdActions::self()->nextTrackAction->trigger(); }
     void Previous() { StdActions::self()->prevTrackAction->trigger(); }
-    void Pause() {
-        if (MPDState_Playing==MPDStatus::self()->state()) {
-            StdActions::self()->playPauseTrackAction->trigger();
-        }
-    }
-
+    void Pause();
     void PlayPause() { StdActions::self()->playPauseTrackAction->trigger(); }
     void Stop() { StdActions::self()->stopPlaybackAction->trigger(); }
-    void StopAfterCurrent() { StdActions::self()->stopAfterTrackAction->trigger(); }
-
-    void Play() {
-        MPDStatus * const status = MPDStatus::self();
-
-        if (status->playlistLength() && MPDState_Playing!=status->state()) {
-           StdActions::self()->playPauseTrackAction->trigger();
-        }
-    }
-
-    void Seek(qlonglong pos) { emit setSeekId(-1, pos/1000000); }
-    void SetPosition(const QDBusObjectPath &, qlonglong pos) {emit setSeekId(-1, pos/1000000); }
+    void StopAfterCurrent() { StdActions::self()->stopAfterCurrentTrackAction->trigger(); }
+    void Play();
+    void Seek(qlonglong pos) { emit seek(pos/1000000); }
+    void SetPosition(const QDBusObjectPath &trackId, qlonglong pos);
     void OpenUri(const QString &) { }
-
-    QString PlaybackStatus() {
-        switch(MPDStatus::self()->state()) {
-        case MPDState_Playing: return QLatin1String("Playing");
-        case MPDState_Paused: return QLatin1String("Paused");
-        default:
-        case MPDState_Stopped: return QLatin1String("Stopped");
-        }
-    }
-
+    QString PlaybackStatus() const;
     QString LoopStatus() { return MPDStatus::self()->repeat() ? QLatin1String("Playlist") : QLatin1String("None"); }
     void SetLoopStatus(const QString &s) { emit setRepeat(QLatin1String("None")!=s); }
     QVariantMap Metadata() const;
@@ -115,9 +93,9 @@ public:
     double MinimumRate() const { return 1.0; }
     double MaximumRate() const { return 1.0; }
     bool CanControl() const { return true; }
-    bool CanPlay() const { return MPDState_Playing!=MPDStatus::self()->state() && MPDStatus::self()->playlistLength()>0; }
-    bool CanPause() const { return MPDState_Playing==MPDStatus::self()->state(); }
-    bool CanSeek() const { return -1!=MPDStatus::self()->songId(); }
+    bool CanPlay() const { return true; }
+    bool CanPause() const { return true; }
+    bool CanSeek() const { return -1!=MPDStatus::self()->songId() && !currentSong.isCdda() && !currentSong.isStandardStream() && currentSong.time>5; }
     bool CanGoNext() const { return MPDState_Stopped!=MPDStatus::self()->state() && MPDStatus::self()->playlistLength()>1; }
     bool CanGoPrevious() const { return MPDState_Stopped!=MPDStatus::self()->state() && MPDStatus::self()->playlistLength()>1; }
 
@@ -126,18 +104,7 @@ public:
     bool CanRaise() const { return true; }
     bool HasTrackList() const { return false; }
     QString Identity() const { return QLatin1String("Cantata"); }
-    QString DesktopEntry() const {
-        #ifdef ENABLE_KDE_SUPPORT
-        // Desktop file is installed in $prefix/share/applications/kde4/
-        // rather than in $prefix/share/applications. The standard way to
-        // represent this dir is with a "kde4-" prefix. See:
-        // http://standards.freedesktop.org/menu-spec/1.0/go01.html#term-desktop-file-id
-        return QLatin1String("kde4-cantata");
-        #else
-        return QLatin1String("cantata");
-        #endif
-    }
-
+    QString DesktopEntry() const { return QLatin1String("cantata"); }
     QStringList SupportedUriSchemes() const { return QStringList(); }
     QStringList SupportedMimeTypes() const { return QStringList(); }
 
@@ -153,6 +120,7 @@ Q_SIGNALS:
     void setRandom(bool toggle);
     void setRepeat(bool toggle);
     void setSeekId(qint32 songId, quint32 time);
+    void seek(qint32 offset);
     void setVolume(int vol);
 
     void showMainWindow();
@@ -166,6 +134,7 @@ private Q_SLOTS:
 private:
     void signalUpdate(const QString &property, const QVariant &value);
     void signalUpdate(const QVariantMap &map);
+    QString currentTrackId() const;
 
 private:
     MPDStatusValues status;
