@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,16 +23,11 @@
 
 #include "powermanagement.h"
 #include "support/globalstatic.h"
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/Solid/PowerManagement>
-#include <kdeversion.h>
-#else
 #include "inhibitinterface.h"
 #include "policyagentinterface.h"
 #include "upowerinterface.h"
-#endif
-#include "support/localize.h"
-#include "mpd/mpdstatus.h"
+#include "login1interface.h"
+#include "mpd-interface/mpdstatus.h"
 
 GLOBAL_STATIC(PowerManagement, instance)
 
@@ -40,11 +35,6 @@ PowerManagement::PowerManagement()
     : inhibitSuspendWhilstPlaying(false)
     , cookie(-1)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    #if KDE_IS_VERSION(4, 7, 0)
-    connect(Solid::PowerManagement::notifier(), SIGNAL(resumingFromSuspend()), this, SIGNAL(resuming()));
-    #endif
-    #else
     policy = new OrgKdeSolidPowerManagementPolicyAgentInterface(OrgKdeSolidPowerManagementPolicyAgentInterface::staticInterfaceName(),
                                                                 QLatin1String("/org/kde/Solid/PowerManagement/PolicyAgent"),
                                                                 QDBusConnection::sessionBus(), this);
@@ -53,8 +43,10 @@ PowerManagement::PowerManagement()
                                                                 QDBusConnection::sessionBus(), this);
     upower = new OrgFreedesktopUPowerInterface(OrgFreedesktopUPowerInterface::staticInterfaceName(),
                                                QLatin1String("/org/freedesktop/UPower"), QDBusConnection::systemBus(), this);
+    login1 = new OrgFreedesktopLogin1ManagerInterface("org.freedesktop.login1",
+                                                      QLatin1String("/org/freedesktop/login1"), QDBusConnection::systemBus(), this);
     connect(upower, SIGNAL(Resuming()), this, SIGNAL(resuming()));
-    #endif
+    connect(login1, SIGNAL(PrepareForSleep(bool)), this, SLOT(prepareForSleep(bool)));
 }
 
 void PowerManagement::setInhibitSuspend(bool i)
@@ -80,10 +72,7 @@ void PowerManagement::beginSuppressingSleep()
     if (-1!=cookie) {
         return;
     }
-    QString reason=i18n("Cantata is playing a track");
-    #ifdef ENABLE_KDE_SUPPORT
-    cookie=Solid::PowerManagement::beginSuppressingSleep(reason);
-    #else
+    QString reason=tr("Cantata is playing a track");
     QDBusReply<uint> reply;
     if (policy->isValid()) {
         reply = policy->AddInhibition((uint)1, QCoreApplication::applicationName(), reason);
@@ -92,7 +81,6 @@ void PowerManagement::beginSuppressingSleep()
         reply = inhibit->Inhibit(QCoreApplication::applicationName(), reason);
     }
     cookie=reply.isValid() ? reply : -1;
-    #endif
 }
 
 void PowerManagement::stopSuppressingSleep()
@@ -101,16 +89,12 @@ void PowerManagement::stopSuppressingSleep()
         return;
     }
 
-    #ifdef ENABLE_KDE_SUPPORT
-    Solid::PowerManagement::stopSuppressingSleep(cookie);
-    #else
     if (policy->isValid()) {
         policy->ReleaseInhibition(cookie);
     } else {
         // Fallback to the fd.o Inhibit interface
         inhibit->UnInhibit(cookie);
     }
-    #endif
     cookie=-1;
 }
 
@@ -124,3 +108,12 @@ void PowerManagement::mpdStatusUpdated()
         }
     }
 }
+
+void PowerManagement::prepareForSleep(bool s)
+{
+    if (!s) {
+        emit resuming();
+    }
+}
+
+#include "moc_powermanagement.cpp"
