@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -28,26 +28,19 @@
 #ifdef MUSICBRAINZ5_FOUND
 #include "musicbrainz.h"
 #endif
-#include "support/localize.h"
-#include "gui/plurals.h"
 #include "models/musiclibraryitemsong.h"
-#include "models/musiclibrarymodel.h"
-#include "models/dirviewmodel.h"
+#include "models/mpdlibrarymodel.h"
+#include "models/playqueuemodel.h"
 #include "support/utils.h"
 #include "extractjob.h"
-#include "mpd/mpdconnection.h"
+#include "mpd-interface/mpdconnection.h"
 #include "gui/covers.h"
 #include "gui/settings.h"
+#include "widgets/icons.h"
 #include <QDir>
 #include <QUrl>
-#if QT_VERSION >= 0x050000
 #include <QUrlQuery>
-#endif
-#ifdef ENABLE_KDE_SUPPORT
-#include <solid/block.h>
-#else
 #include "solid-lite/block.h"
-#endif
 
 const QLatin1String AudioCdDevice::constAnyDev("-");
 
@@ -64,11 +57,7 @@ QString AudioCdDevice::coverUrl(QString udi)
 QString AudioCdDevice::getDevice(const QUrl &url)
 {
     if (QLatin1String("cdda")==url.scheme()) {
-        #if QT_VERSION < 0x050000
-        const QUrl &q=url;
-        #else
         QUrlQuery q(url);
-        #endif
         if (q.hasQueryItem("dev")) {
             return q.queryItemValue("dev");
         }
@@ -86,7 +75,7 @@ QString AudioCdDevice::getDevice(const QUrl &url)
     return QString();
 }
 
-AudioCdDevice::AudioCdDevice(MusicModel *m, Solid::Device &dev)
+AudioCdDevice::AudioCdDevice(MusicLibraryModel *m, Solid::Device &dev)
     : Device(m, dev, false, true)
     #ifdef CDDB_FOUND
     , cddb(0)
@@ -100,7 +89,7 @@ AudioCdDevice::AudioCdDevice(MusicModel *m, Solid::Device &dev)
     , lookupInProcess(false)
     , autoPlay(false)
 {
-    icn=Icon("media-optical");
+    icn=Icons::self()->albumMonoIcon;
     drive=dev.parent().as<Solid::OpticalDrive>();
     Solid::Block *block=dev.as<Solid::Block>();
     if (block) {
@@ -127,7 +116,7 @@ AudioCdDevice::AudioCdDevice(MusicModel *m, Solid::Device &dev)
         #else
         connectService(true);
         #endif
-        detailsString=i18n("Reading disc");
+        detailsString=tr("Reading disc");
         setStatusMessage(detailsString);
         lookupInProcess=true;
         connect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
@@ -138,19 +127,6 @@ AudioCdDevice::AudioCdDevice(MusicModel *m, Solid::Device &dev)
 
 AudioCdDevice::~AudioCdDevice()
 {
-    QList<Song> tracks;
-    foreach (const MusicLibraryItem *item, childItems()) {
-        if (MusicLibraryItem::Type_Song==item->itemType()) {
-            Song song=static_cast<const MusicLibraryItemSong *>(item)->song();
-            song.file=path()+song.file;
-            tracks.append(song);
-        }
-    }
-
-    if (!tracks.isEmpty()) {
-        emit invalid(tracks);
-    }
-
     #ifdef CDDB_FOUND
     if (cddb) {
         cddb->deleteLater();
@@ -163,6 +139,26 @@ AudioCdDevice::~AudioCdDevice()
         mb=0;
     }
     #endif
+    // Remove any downloaded cover image...
+    if (!coverImage.fileName.isEmpty() && coverImage.fileName.startsWith(Utils::cacheDir(Covers::constCddaCoverDir, false))) {
+        QFile::remove(coverImage.fileName);
+    }
+}
+
+void AudioCdDevice::dequeue()
+{
+    QList<Song> tracks;
+    for (const MusicLibraryItem *item: childItems()) {
+        if (MusicLibraryItem::Type_Song==item->itemType()) {
+            Song song=static_cast<const MusicLibraryItemSong *>(item)->song();
+            song.file=path()+song.file;
+            tracks.append(song);
+        }
+    }
+
+    if (!tracks.isEmpty()) {
+        PlayQueueModel::self()->remove(tracks);
+    }
 }
 
 bool AudioCdDevice::isAudioDevice(const QString &dev) const
@@ -182,7 +178,7 @@ void AudioCdDevice::connectService(bool useCddb)
         mb=0;
     }
     #else
-    Q_UNUSED(useCddb);
+    Q_UNUSED(useCddb)
     #endif
 
     #ifdef CDDB_FOUND
@@ -228,6 +224,7 @@ void AudioCdDevice::toggle()
     if (drive) {
         stop();
         drive->eject();
+        PlayQueueModel::self()->removeCantataStreams(true);
     }
 }
 
@@ -251,7 +248,7 @@ void AudioCdDevice::copySongTo(const Song &s, const QString &musicPath, bool ove
         if (needToFixVa) {
             Device::fixVariousArtists(QString(), check, false);
         }
-        if (MusicLibraryModel::self()->songExists(check)) {
+        if (MpdLibraryModel::self()->songExists(check)) {
             emit actionStatus(SongExists);
             return;
         }
@@ -286,7 +283,7 @@ quint32 AudioCdDevice::totalTime()
 {
     if (0xFFFFFFFF==time) {
         time=0;
-        foreach (MusicLibraryItem *i, childItems()) {
+        for (MusicLibraryItem *i: childItems()) {
             time+=static_cast<MusicLibraryItemSong *>(i)->song().time;
         }
     }
@@ -321,7 +318,7 @@ void AudioCdDevice::copySongToResult(int status)
     } else {
         currentSong.file=currentDestFile.mid(MPDConnection::self()->getDetails().dir.length());
         QString origPath;
-        if (MPDConnection::self()->isMopdidy()) {
+        if (MPDConnection::self()->isMopidy()) {
             origPath=currentSong.file;
             currentSong.file=Song::encodePath(currentSong.file);
         }
@@ -329,9 +326,9 @@ void AudioCdDevice::copySongToResult(int status)
             currentSong.revertVariousArtists();
         }
         Utils::setFilePerms(currentDestFile);
-        MusicLibraryModel::self()->addSongToList(currentSong);
-        DirViewModel::self()->addFileToList(origPath.isEmpty() ? currentSong.file : origPath,
-                                            origPath.isEmpty() ? QString() : currentSong.file);
+//        MusicLibraryModel::self()->addSongToList(currentSong);
+//        DirViewModel::self()->addFileToList(origPath.isEmpty() ? currentSong.file : origPath,
+//                                            origPath.isEmpty() ? QString() : currentSong.file);
         emit actionStatus(Ok, job && job->coverCopied());
     }
 }
@@ -351,13 +348,13 @@ void AudioCdDevice::setDetails(const CdAlbum &a)
     disc=a.disc;
     update=new MusicLibraryItemRoot();
     int totalDuration=0;
-    foreach (Song s, a.tracks) {
+    for (Song s: a.tracks) {
         totalDuration+=s.time;
         s.size=s.time*constBytesPerSecond;
         update->append(new MusicLibraryItemSong(s, update));
     }
     setStatusMessage(QString());
-    detailsString=Plurals::tracksWithDuration(a.tracks.count(), Utils::formatTime(totalDuration));
+    detailsString=tr("%n Tracks (%1)", "", a.tracks.count()).arg(Utils::formatTime(totalDuration));
     emit updating(id(), false);
     if (differentAlbum && !a.isDefault) {
         Song s;
@@ -397,6 +394,15 @@ void AudioCdDevice::setCover(const Covers::Image &img)
     updateStatus();
 }
 
+void AudioCdDevice::scaleCoverPix(int size) const
+{
+    if (!coverImage.img.isNull()) {
+        if (scaledCover.width()!=size && scaledCover.height()!=size) {
+            scaledCover=QPixmap::fromImage(coverImage.img.scaled(size, size, Qt::KeepAspectRatio, Qt::SmoothTransformation));
+        }
+    }
+}
+
 void AudioCdDevice::setCover(const Song &song, const QImage &img, const QString &file)
 {
     if (song.isCdda() && song.albumartist==artist && song.album==album) {
@@ -416,7 +422,7 @@ void AudioCdDevice::autoplay()
 void AudioCdDevice::playTracks()
 {
     QList<Song> tracks;
-    foreach (const MusicLibraryItem *item, childItems()) {
+    for (const MusicLibraryItem *item: childItems()) {
         if (MusicLibraryItem::Type_Song==item->itemType()) {
             Song song=static_cast<const MusicLibraryItemSong *>(item)->song();
             song.file=path()+song.file;
@@ -432,7 +438,7 @@ void AudioCdDevice::playTracks()
 void AudioCdDevice::updateDetails()
 {
     QList<Song> tracks;
-    foreach (const MusicLibraryItem *item, childItems()) {
+    for (const MusicLibraryItem *item: childItems()) {
         if (MusicLibraryItem::Type_Song==item->itemType()) {
             Song song=static_cast<const MusicLibraryItemSong *>(item)->song();
             song.file=path()+song.file;
@@ -444,3 +450,5 @@ void AudioCdDevice::updateDetails()
         emit updatedDetails(tracks);
     }
 }
+
+#include "moc_audiocddevice.cpp"
