@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,7 +26,7 @@
 
 #include <QString>
 #include <QList>
-#include "song.h"
+#include "mpd-interface/song.h"
 
 struct CdAlbum {
     CdAlbum() : isDefault(false), year(0), disc(0)  { }
