@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,21 +25,18 @@
 #define DEVICE_H
 
 #include "models/musiclibraryitemroot.h"
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
+#include "gui/covers.h"
 #include "config.h"
 #ifdef ENABLE_DEVICES_SUPPORT
 #include "deviceoptions.h"
-#ifdef ENABLE_KDE_SUPPORT
-#include <solid/device.h>
-#else
 #include "solid-lite/device.h"
 #endif
-#endif
 
 class QWidget;
 class QImage;
 class QTemporaryFile;
-class MusicModel;
+class MusicLibraryModel;
 
 // MOC requires the QObject class to be first. But due to models storing void pointers, and
 // needing to cast these - the model prefers MusicLibraryItemRoot to be first!
@@ -55,10 +52,11 @@ public:
     #ifdef ENABLE_DEVICES_SUPPORT
     static const QLatin1String constNoCover;
     static const QLatin1String constEmbedCover;
-    static Device * create(MusicModel *m, const QString &id);
+    static Device * create(MusicLibraryModel *m, const QString &id);
     static bool fixVariousArtists(const QString &file, Song &song, bool applyFix);
     static void embedCover(const QString &file, Song &song, unsigned int coverMaxSize);
     static QTemporaryFile * copySongToTemp(Song &song);
+    static bool isLossless(const QString &file);
     #endif
     static void moveDir(const QString &from, const QString &to, const QString &base, const QString &coverFile);
     static void cleanDir(const QString &dir, const QString &base, const QString &coverFile, int level=0);
@@ -94,53 +92,24 @@ public:
     };
 
     #ifndef ENABLE_DEVICES_SUPPORT
-    Device(MusicModel *m, const QString &name, const QString &id)
+    Device(MusicLibraryModel *m, const QString &name, const QString &id)
         : MusicLibraryItemRoot(name)
         , update(0)
         , needToFixVa(false)
         , jobAbortRequested(false)
         , transcoding(false) {
-        setUseArtistImages(false);
-        setUseAlbumImages(true);
-        setLargeImages(false);
         Q_UNUSED(m)
         Q_UNUSED(id)
     }
     #else
-    Device(MusicModel *m, Solid::Device &dev, bool albumArtistSupport=true, bool flat=false)
-        : MusicLibraryItemRoot(dev.product().startsWith(dev.vendor()) ? dev.product() : (dev.vendor()+QChar(' ')+dev.product()), albumArtistSupport, flat)
-        , configured(false)
-        , solidDev(dev)
-        , deviceId(dev.udi())
-        , update(0)
-        , needToFixVa(false)
-        , jobAbortRequested(false)
-        , transcoding(false) {
-        m_model=m;
-        setUseArtistImages(false);
-        setUseAlbumImages(true);
-        setLargeImages(false);
-        icn=Icon(solidDev.isValid() ? solidDev.icon() : QLatin1String("inode-directory"));
-    }
-    Device(MusicModel *m, const QString &name, const QString &id)
-        : MusicLibraryItemRoot(name)
-        , configured(false)
-        , deviceId(id)
-        , update(0)
-        , needToFixVa(false)
-        , jobAbortRequested(false)
-        , transcoding(false) {
-        m_model=m;
-        setUseArtistImages(false);
-        setUseAlbumImages(true);
-        icn=Icon(solidDev.isValid() ? solidDev.icon() : QLatin1String("inode-directory"));
-    }
+    Device(MusicLibraryModel *m, Solid::Device &dev, bool albumArtistSupport=true, bool flat=false);
+    Device(MusicLibraryModel *m, const QString &name, const QString &id);
     #endif
 
-    virtual ~Device() { }
+    ~Device() override { }
 
-    Icon icon() const { return icn; }
-    Song fixPath(const Song &orig, bool fullPath) const;
+    QIcon icon() const override { return icn; }
+    Song fixPath(const Song &orig, bool fullPath) const override;
     virtual QString coverFile() const { return QString(); }
     virtual bool isConnected() const=0;
     virtual void rescan(bool full=true)=0;
@@ -155,20 +124,19 @@ public:
     virtual void copySongTo(const Song &s, const QString &musicPath, bool overwrite, bool copyCover)=0;
     virtual void removeSong(const Song &s)=0;
     virtual void cleanDirs(const QSet<QString> &dirs)=0;
-    virtual void requestCover(const Song &) { }
+    virtual Covers::Image requestCover(const Song &) { return Covers::Image(); }
     virtual double usedCapacity()=0;
     virtual QString capacityString()=0;
     virtual qint64 freeSpace()=0;
     virtual DevType devType() const=0;
     virtual void removeCache() { }
-    virtual bool isDevice() const { return true; }
+    bool isDevice() const override { return true; }
 
     #ifdef ENABLE_DEVICES_SUPPORT
-    void toggleGrouping();
     virtual void saveCache();
-    const QString & id() const { return deviceId; }
+    const QString & id() const override { return deviceId; }
     void applyUpdate();
-    bool haveUpdate() const { return 0!=update; }
+    bool haveUpdate() const { return nullptr!=update; }
     int newRows() const { return update ? update->childCount() : 0; }
     const DeviceOptions & options() const { return opts; }
     void setOptions(const DeviceOptions &o) { opts=o; saveOptions(); }
@@ -177,11 +145,11 @@ public:
     bool isConfigured() { return configured; }
     virtual void abortJob() { jobAbortRequested=true; }
     bool abortRequested() const { return jobAbortRequested; }
-    virtual bool canPlaySongs() const { return false; }
+    bool canPlaySongs() const override { return false; }
     virtual bool supportsDisconnect() const { return false; }
     virtual bool isStdFs() const { return false; }
     virtual QString subText() { return QString(); }
-    QModelIndex index() const;
+    QModelIndex index() const override;
     void updateStatus();
 
 public Q_SLOTS:
@@ -200,7 +168,6 @@ Q_SIGNALS:
     void cover(const Song &song, const QImage &img);
     void cacheSaved();
     void configurationChanged();
-    void invalid(const QList<Song> &songs);
     void play(const QList<Song> &songs);
     void updatedDetails(const QList<Song> &songs);
     void renamed();
@@ -219,7 +186,7 @@ protected:
     bool needToFixVa;
     bool jobAbortRequested;
     bool transcoding;
-    Icon icn;
+    QIcon icn;
 };
 
 #endif
