@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,12 +24,11 @@
 #include "devicepropertieswidget.h"
 #include "filenameschemedialog.h"
 #include "gui/covers.h"
-#include "support/localize.h"
 #include "widgets/icons.h"
 #include "support/utils.h"
 #include <QValidator>
 #include <QTabWidget>
-#include <QApplication>
+#include <QTimer>
 
 class CoverNameValidator : public QValidator
 {
@@ -37,7 +36,7 @@ class CoverNameValidator : public QValidator
 
     CoverNameValidator(QObject *parent) : QValidator(parent) { }
 
-    State validate(QString &input, int &) const
+    State validate(QString &input, int &) const override
     {
         int dotCount(0);
 
@@ -80,23 +79,23 @@ class CoverNameValidator : public QValidator
 
 DevicePropertiesWidget::DevicePropertiesWidget(QWidget *parent)
     : QWidget(parent)
-    , schemeDlg(0)
-    , noCoverText(i18n("Don't copy covers"))
-    , embedCoverText(i18n("Embed cover within each file"))
+    , schemeDlg(nullptr)
+    , noCoverText(tr("Don't copy covers"))
+    , embedCoverText(tr("Embed cover within each file"))
     , modified(false)
     , saveable(false)
 {
     setupUi(this);
     configFilename->setIcon(Icons::self()->configureIcon);
-    coverMaxSize->insertItems(0, QStringList() << i18n("No maximum size") << i18n("400 pixels") << i18n("300 pixels") << i18n("200 pixels") << i18n("100 pixels"));
-    fixVariousArtists->setToolTip(i18n("<p>When copying tracks to a device, and the 'Album Artist' is set to 'Various Artists', "
+    coverMaxSize->insertItems(0, QStringList() << tr("No maximum size") << tr("400 pixels") << tr("300 pixels") << tr("200 pixels") << tr("100 pixels"));
+    fixVariousArtists->setToolTip(tr("<p>When copying tracks to a device, and the 'Album Artist' is set to 'Various Artists', "
                                        "then Cantata will set the 'Artist' tag of all tracks to 'Various Artists' and the "
                                        "track 'Title' tag to 'TrackArtist - TrackTitle'.<hr/> When copying from a device, Cantata "
                                        "will check if 'Album Artist' and 'Artist' are both set to 'Various Artists'. If so, it "
                                        "will attempt to extract the real artist from the 'Title' tag, and remove the artist name "
                                        "from the 'Title' tag.</p>"));
 
-    useCache->setToolTip(i18n("<p>If you enable this, then Cantata will create a cache of the device's music library. "
+    useCache->setToolTip(tr("<p>If you enable this, then Cantata will create a cache of the device's music library. "
                               "This will help to speed up subsequent library scans (as the cache file will be used instead of "
                               "having to read the tags of each file.)<hr/><b>NOTE:</b> If you use another application to update "
                               "the device's library, then this cache will become out-of-date. To rectify this, simply "
@@ -139,17 +138,17 @@ void DevicePropertiesWidget::update(const QString &path, const DeviceOptions &op
         replaceSpaces->setChecked(opts.replaceSpaces);
     } else {
         REMOVE(filenamesGroupBox)
-        filenameScheme=0;
-        vfatSafe=0;
-        asciiOnly=0;
-        ignoreThe=0;
-        replaceSpaces=0;
-        configFilename=0;
+        filenameScheme=nullptr;
+        vfatSafe=nullptr;
+        asciiOnly=nullptr;
+        ignoreThe=nullptr;
+        replaceSpaces=nullptr;
+        configFilename=nullptr;
     }
     origOpts=opts;
 
     if (props&Prop_Folder) {
-        musicFolder->setText(Utils::convertDirForDisplay(path));
+        musicFolder->setText(Utils::convertPathForDisplay(path));
         connect(musicFolder, SIGNAL(textChanged(const QString &)), this, SLOT(checkSaveable()));
         if (disabledProps&Prop_Folder) {
             musicFolder->setDisabled(true);
@@ -217,15 +216,24 @@ void DevicePropertiesWidget::update(const QString &path, const DeviceOptions &op
         bool transcode=props&Prop_Transcoder;
         transcoderName->clear();
         if (transcode) {
-            transcoderName->addItem(i18n("Do not transcode"), QString());
+            transcoderName->addItem(tr("Do not transcode"), QString());
             transcoderName->setCurrentIndex(0);
             transcoderValue->setVisible(false);
-            transcoderWhenDifferent->setVisible(false);
-            transcoderWhenDifferent->setChecked(opts.transcoderWhenDifferent);
-            connect(transcoderWhenDifferent, SIGNAL(stateChanged(int)), this, SLOT(checkSaveable()));
+            transcoderWhen->addItem(tr("Always transcode"), DeviceOptions::TW_Always);
+            transcoderWhen->addItem(tr("Only transcode if source file is of a different format"), DeviceOptions::TW_IfDifferent);
+            transcoderWhen->addItem(tr("Only transcode if source is FLAC/WAV"), DeviceOptions::TW_IfLossess);
+            transcoderWhen->setVisible(false);
+            transcoderWhen->setCurrentIndex(0);
+            for (int i=0; i<transcoderWhen->count(); ++i) {
+                if (transcoderWhen->itemData(i).toInt()==opts.transcoderWhen) {
+                    transcoderWhen->setCurrentIndex(i);
+                    break;
+                }
+            }
+            connect(transcoderWhen, SIGNAL(currentIndexChanged(int)), this, SLOT(checkSaveable()));
         } else {
-            transcoderFrame->setTitle(i18n("Encoder"));
-            REMOVE(transcoderWhenDifferent);
+            transcoderFrame->setTitle(tr("Encoder"));
+            REMOVE(transcoderWhen);
         }
 
         QList<Encoders::Encoder> encs=Encoders::getAvailable();
@@ -233,13 +241,13 @@ void DevicePropertiesWidget::update(const QString &path, const DeviceOptions &op
         if (encs.isEmpty()) {
             transcoderFrame->setVisible(false);
         } else {
-            foreach (const Encoders::Encoder &e, encs) {
+            for (const Encoders::Encoder &e: encs) {
                 if (!transcode || e.transcoder) {
                     QString name=e.name;
                     if (transcode && name.endsWith(QLatin1String(" (ffmpeg)"))) {
                         name=name.left(name.length()-9);
                     }
-                    transcoderName->addItem(transcode ? i18n("Transcode to %1", name) : name, e.codec);
+                    transcoderName->addItem(transcode ? tr("Transcode to %1").arg(name) : name, e.codec);
                 }
             }
 
@@ -269,9 +277,9 @@ void DevicePropertiesWidget::update(const QString &path, const DeviceOptions &op
         REMOVE(defaultVolume);
         REMOVE(defaultVolumeLabel);
     } else {
-        foreach (const DeviceStorage &ds, storage) {
-            defaultVolume->addItem(i18nc("name (size free)", "%1 (%2 free)",
-                                   ds.description, Utils::formatByteSize(ds.size-ds.used)), ds.volumeIdentifier);
+        for (const DeviceStorage &ds: storage) {
+            defaultVolume->addItem(tr("%1 (%2 free)", "name (size free)")
+                                   .arg(ds.description).arg(Utils::formatByteSize(ds.size-ds.used)), ds.volumeIdentifier);
         }
 
         for (int i=0; i<defaultVolume->count(); ++i) {
@@ -296,6 +304,7 @@ void DevicePropertiesWidget::update(const QString &path, const DeviceOptions &op
     if (albumCovers) {
         albumCoversChanged();
     }
+    QTimer::singleShot(0, this, SLOT(setSize()));
 }
 
 void DevicePropertiesWidget::transcoderChanged()
@@ -304,14 +313,14 @@ void DevicePropertiesWidget::transcoderChanged()
     if (codec.isEmpty()) {
         transcoderName->setToolTip(QString());
         transcoderValue->setVisible(false);
-        if (transcoderWhenDifferent) {
-            transcoderWhenDifferent->setVisible(false);
+        if (transcoderWhen) {
+            transcoderWhen->setVisible(false);
         }
     } else {
         Encoders::Encoder enc=Encoders::getEncoder(codec);
         transcoderName->setToolTip(enc.description);
-        if (transcoderWhenDifferent) {
-            transcoderWhenDifferent->setVisible(true);
+        if (transcoderWhen) {
+            transcoderWhen->setVisible(true);
         }
         if (enc.values.count()) {
             transcoderValue->setValues(enc);
@@ -321,17 +330,7 @@ void DevicePropertiesWidget::transcoderChanged()
         }
     }
 
-    QWidget *win=window();
-    if (win) {
-        QSize was=win->size();
-        QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
-        win->setMinimumSize(QSize(0, 0));
-        win->adjustSize();
-        QSize now=win->size();
-        win->setMinimumSize(now);
-        win->resize(qMax(was.width(), now.width()), now.height());
-    }
-
+    Utils::resizeWindow(this, true, false);
     checkSaveable();
 }
 
@@ -398,7 +397,7 @@ DeviceOptions DevicePropertiesWidget::settings()
     opts.autoScan=autoScan ? autoScan->isChecked() : false;
     opts.transcoderCodec=QString();
     opts.transcoderValue=0;
-    opts.transcoderWhenDifferent=false;
+    opts.transcoderWhen=DeviceOptions::TW_Always;
     opts.coverName=cover();
     opts.coverMaxSize=(!coverMaxSize || 0==coverMaxSize->currentIndex()) ? 0 : ((coverMaxSize->count()-coverMaxSize->currentIndex())*100);
     opts.volumeId=defaultVolume && defaultVolume->isVisible() ? defaultVolume->itemData(defaultVolume->currentIndex()).toString() : QString();
@@ -408,7 +407,9 @@ DeviceOptions DevicePropertiesWidget::settings()
         if (!opts.transcoderCodec.isEmpty()) {
             Encoders::Encoder enc=Encoders::getEncoder(opts.transcoderCodec);
 
-            opts.transcoderWhenDifferent=transcoderWhenDifferent ? transcoderWhenDifferent->isChecked() : false;
+            if (transcoderWhen) {
+                opts.transcoderWhen=(DeviceOptions::TranscodeWhen)transcoderWhen->itemData(transcoderWhen->currentIndex()).toUInt();
+            }
             if (!enc.isNull() && transcoderValue->value()<enc.values.count()) {
                 opts.transcoderValue=enc.values.at(transcoderValue->value()).value;
             }
@@ -426,3 +427,10 @@ QString DevicePropertiesWidget::cover() const
                 ? Device::constEmbedCover
                 : coverName;
 }
+
+void DevicePropertiesWidget::setSize()
+{
+    Utils::resizeWindow(this, true, false);
+}
+
+#include "moc_devicepropertieswidget.cpp"
