@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -52,12 +52,12 @@ public:
         Prop_All         = 0x03FF
     };
     DevicePropertiesWidget(QWidget *parent);
-    virtual ~DevicePropertiesWidget() { }
+    ~DevicePropertiesWidget() override { }
     void update(const QString &path, const DeviceOptions &opts, const QList<DeviceStorage> &storage, int props, int disabledProps);
     DeviceOptions settings();
     bool isModified() const { return modified; }
     bool isSaveable() const { return saveable; }
-    QString music() const { return musicFolder && musicFolder->isEnabled() ? Utils::convertDirFromDisplay(musicFolder->text()) : origMusicFolder; }
+    QString music() const { return musicFolder && musicFolder->isEnabled() ? Utils::convertPathFromDisplay(musicFolder->text()) : origMusicFolder; }
     QString cover() const;
     void showRemoteConnectionNote(bool v) { remoteDeviceNote->setVisible(v); }
 
@@ -69,6 +69,7 @@ private Q_SLOTS:
     void checkSaveable();
     void transcoderChanged();
     void albumCoversChanged();
+    void setSize();
 
 private:
     FilenameSchemeDialog *schemeDlg;
