@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -29,16 +29,17 @@
 #include "models/devicesmodel.h"
 #include "gui/settings.h"
 #include "support/messagebox.h"
-#include "support/localize.h"
+#include "support/configuration.h"
 #include "widgets/icons.h"
+#include "widgets/menubutton.h"
 #include "support/action.h"
-#include "support/actioncollection.h"
+#include "support/monoicon.h"
 #include "gui/stdactions.h"
+#include "syncdialog.h"
 #ifdef ENABLE_REMOTE_DEVICES
 #include "remotedevicepropertiesdialog.h"
 #include "devicepropertieswidget.h"
 #endif
-#include "syncdialog.h"
 #ifdef ENABLE_REPLAYGAIN_SUPPORT
 #include "replaygain/rgdialog.h"
 #endif
@@ -47,7 +48,7 @@
 #include "tags/trackorganiser.h"
 #include "gui/preferencesdialog.h"
 #include "gui/coverdialog.h"
-#include "mpd/mpdconnection.h"
+#include "mpd-interface/mpdconnection.h"
 #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
 #include "audiocddevice.h"
 #include "albumdetailsdialog.h"
@@ -56,47 +57,24 @@
 #include <QMenu>
 
 DevicesPage::DevicesPage(QWidget *p)
-    : QWidget(p)
+    : SinglePageWidget(p)
 {
-    setupUi(this);
-    copyAction = ActionCollection::get()->createAction("copytolibrary", i18n("Copy To Library"), Icons::self()->importIcon);
+    copyAction = new Action(Icons::self()->downloadIcon, tr("Copy To Library"), this);
+    ToolButton *copyToLibraryButton=new ToolButton(this);
     copyToLibraryButton->setDefaultAction(copyAction);
-    syncAction = ActionCollection::get()->createAction("syncdevice", i18n("Sync"), "folder-sync");
+    syncAction = new Action(MonoIcon::icon(FontAwesome::exchange, Utils::monoIconColor()), tr("Synchronise"), this);
+    syncAction->setEnabled(false);
     connect(syncAction, SIGNAL(triggered()), this, SLOT(sync()));
     #ifdef ENABLE_REMOTE_DEVICES
-    forgetDeviceAction=ActionCollection::get()->createAction("forgetdevice", i18n("Forget Device"), "list-remove");
+    forgetDeviceAction=new Action(tr("Forget Device"), this);
     connect(forgetDeviceAction, SIGNAL(triggered()), this, SLOT(forgetRemoteDevice()));
     #endif
     connect(DevicesModel::self()->connectAct(), SIGNAL(triggered()), this, SLOT(toggleDevice()));
     connect(DevicesModel::self()->disconnectAct(), SIGNAL(triggered()), this, SLOT(toggleDevice()));
-    copyToLibraryButton->setEnabled(false);
-    syncAction->setEnabled(false);
-    view->addAction(copyAction);
-    view->addAction(syncAction);
-    view->addAction(StdActions::self()->organiseFilesAction);
-    view->addAction(StdActions::self()->editTagsAction);
-    #ifdef ENABLE_REPLAYGAIN_SUPPORT
-    view->addAction(StdActions::self()->replaygainAction);
-    #endif
-    #ifdef ENABLE_REMOTE_DEVICES
-    QAction *sepA=new QAction(this);
-    sepA->setSeparator(true);
-    view->addAction(sepA);
-    view->addAction(forgetDeviceAction);
-    #endif
-    QAction *sep=new QAction(this);
-    sep->setSeparator(true);
-    view->addAction(sep);
-    view->addAction(StdActions::self()->deleteSongsAction);
-    connect(this, SIGNAL(add(const QStringList &, bool, quint8)), MPDConnection::self(), SLOT(add(const QStringList &, bool, quint8)));
-    connect(this, SIGNAL(addSongsToPlaylist(const QString &, const QStringList &)), MPDConnection::self(), SLOT(addToPlaylist(const QString &, const QStringList &)));
-    connect(genreCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(searchItems()));
-    connect(DevicesModel::self(), SIGNAL(updateGenres(const QSet<QString> &)), genreCombo, SLOT(update(const QSet<QString> &)));
     connect(DevicesModel::self(), SIGNAL(updated(QModelIndex)), this, SLOT(updated(QModelIndex)));
     connect(view, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(itemDoubleClicked(const QModelIndex &)));
     connect(view, SIGNAL(searchItems()), this, SLOT(searchItems()));
     connect(view, SIGNAL(itemsSelected(bool)), SLOT(controlActions()));
-    connect(view, SIGNAL(rootIndexSet(QModelIndex)), this, SLOT(updateGenres(QModelIndex)));
     connect(copyAction, SIGNAL(triggered()), this, SLOT(copyToLibrary()));
     connect(DevicesModel::self()->configureAct(), SIGNAL(triggered()), this, SLOT(configureDevice()));
     connect(DevicesModel::self()->refreshAct(), SIGNAL(triggered()), this, SLOT(refreshDevice()));
@@ -105,39 +83,47 @@ DevicesPage::DevicesPage(QWidget *p)
     connect(DevicesModel::self(), SIGNAL(matches(const QString &, const QList<CdAlbum> &)),
             SLOT(cdMatches(const QString &, const QList<CdAlbum> &)));
     #endif
-    QMenu *menu=new QMenu(this);
-    #ifdef ENABLE_REMOTE_DEVICES
-    Action *addRemote=ActionCollection::get()->createAction("adddevice", i18n("Add Device"), "network-server");
-    connect(addRemote, SIGNAL(triggered()), this, SLOT(addRemoteDevice()));
-    menu->addAction(addRemote);
-    menu->addAction(forgetDeviceAction);
-    menu->addSeparator();
-    #endif
-    menu->addAction(DevicesModel::self()->configureAct());
-    menu->addAction(DevicesModel::self()->refreshAct());
-    menu->addSeparator();
-    menu->addAction(StdActions::self()->organiseFilesAction);
-    menu->addAction(StdActions::self()->editTagsAction);
-    #ifdef ENABLE_REPLAYGAIN_SUPPORT
-    menu->addAction(StdActions::self()->replaygainAction);
-    #endif
-    menuButton->setMenu(menu);
     proxy.setSourceModel(DevicesModel::self());
     view->setModel(&proxy);
     view->setRootIsDecorated(false);
     view->setSearchResetLevel(1);
-    view->load(metaObject()->className());
+    Configuration config(metaObject()->className());
+    view->load(config);
+    MenuButton *menu=new MenuButton(this);
+    menu->addAction(createViewMenu(QList<ItemView::Mode>() << ItemView::Mode_BasicTree << ItemView::Mode_SimpleTree
+                                                           << ItemView::Mode_DetailedTree << ItemView::Mode_List));
+    menu->addSeparator();
+    menu->addAction(DevicesModel::self()->configureAct());
+    menu->addAction(DevicesModel::self()->refreshAct());
+    #ifdef ENABLE_REMOTE_DEVICES
+    menu->addSeparator();
+    Action *addRemote=new Action(tr("Add Device"), this);
+    connect(addRemote, SIGNAL(triggered()), this, SLOT(addRemoteDevice()));
+    menu->addAction(addRemote);
+    menu->addAction(forgetDeviceAction);
+    #endif
+    init(ReplacePlayQueue|AppendToPlayQueue, QList<QWidget *>() << menu, QList<QWidget *>() << copyToLibraryButton);
+
+    view->addAction(copyAction);
+    view->addAction(syncAction);
+    view->addAction(StdActions::self()->organiseFilesAction);
+    view->addAction(StdActions::self()->editTagsAction);
+    #ifdef ENABLE_REPLAYGAIN_SUPPORT
+    view->addAction(StdActions::self()->replaygainAction);
+    #endif
+    #ifdef ENABLE_REMOTE_DEVICES
+    view->addSeparator();
+    view->addAction(forgetDeviceAction);
+    #endif
+    view->addSeparator();
+    view->addAction(StdActions::self()->deleteSongsAction);
+    view->setInfoText(tr("Any supported devices will appear here when attached to your computer."));
 }
 
 DevicesPage::~DevicesPage()
 {
-    view->save(metaObject()->className());
-}
-
-void DevicesPage::showEvent(QShowEvent *e)
-{
-    view->focusView();
-    QWidget::showEvent(e);
+    Configuration config(metaObject()->className());
+    view->save(config);
 }
 
 void DevicesPage::clear()
@@ -157,12 +143,12 @@ Device * DevicesPage::activeFsDevice() const
     const QModelIndexList selected = view->selectedIndexes(false); // Dont need sorted selection here...
 
     if (selected.isEmpty()) {
-        return 0;
+        return nullptr;
     }
 
     QString udi;
-    Device *activeDev=0;
-    foreach (const QModelIndex &idx, selected) {
+    Device *activeDev=nullptr;
+    for (const QModelIndex &idx: selected) {
         QModelIndex index = proxy.mapToSource(idx);
         MusicLibraryItem *item=static_cast<MusicLibraryItem *>(index.internalPointer());
 
@@ -175,10 +161,10 @@ Device * DevicesPage::activeFsDevice() const
         if (item && MusicLibraryItem::Type_Root==item->itemType()) {
             Device *dev=static_cast<Device *>(item);
             if (Device::Ums!=dev->devType() && Device::RemoteFs!=dev->devType()) {
-                return 0;
+                return nullptr;
             }
             if (activeDev) {
-                return 0;
+                return nullptr;
             }
             activeDev=dev;
         }
@@ -195,7 +181,7 @@ QStringList DevicesPage::playableUrls() const
     }
 
     QModelIndexList mapped;
-    foreach (const QModelIndex &idx, selected) {
+    for (const QModelIndex &idx: selected) {
         mapped.append(proxy.mapToSource(idx));
     }
 
@@ -211,7 +197,7 @@ QList<Song> DevicesPage::selectedSongs(bool allowPlaylists) const
     }
 
     // Ensure all songs are from UMS/Remote devices...
-    foreach (const QModelIndex &idx, selected) {
+    for (const QModelIndex &idx: selected) {
         MusicLibraryItem *item=static_cast<MusicLibraryItem *>(proxy.mapToSource(idx).internalPointer());
         if (item && MusicLibraryItem::Type_Root!=item->itemType()) {
             while(item->parentItem()) {
@@ -227,17 +213,15 @@ QList<Song> DevicesPage::selectedSongs(bool allowPlaylists) const
         }
     }
 
-    return DevicesModel::self()->songs(proxy.mapToSource(selected, proxy.enabled() && Settings::self()->filteredOnly()));
+    return DevicesModel::self()->songs(proxy.mapToSource(selected));
 }
 
-void DevicesPage::addSelectionToPlaylist(const QString &name, bool replace, quint8 priorty, bool randomAlbums)
+void DevicesPage::addSelectionToPlaylist(const QString &name, int action, quint8 priority, bool decreasePriority)
 {
-    Q_UNUSED(randomAlbums)
     QStringList files=playableUrls();
-
     if (!files.isEmpty()) {
         if (name.isEmpty()) {
-            emit add(files, replace, priorty);
+            emit add(files, action, priority, decreasePriority);
         } else {
             emit addSongsToPlaylist(name, files);
         }
@@ -271,7 +255,7 @@ void DevicesPage::itemDoubleClicked(const QModelIndex &)
 void DevicesPage::searchItems()
 {
     QString text=view->searchText().trimmed();
-    proxy.update(text, genreCombo->currentIndex()<=0 ? QString() : genreCombo->currentText());
+    proxy.update(text);
     if (proxy.enabled() && !proxy.filterText().isEmpty()) {
         view->expandAll();
     }
@@ -293,7 +277,7 @@ void DevicesPage::controlActions()
     bool canPlay=false;
     QString udi;
 
-    foreach (const QModelIndex &idx, selected) {
+    for (const QModelIndex &idx: selected) {
         MusicLibraryItem *item=static_cast<MusicLibraryItem *>(proxy.mapToSource(idx).internalPointer());
 
         if (item && MusicLibraryItem::Type_Root==item->itemType()) {
@@ -346,16 +330,13 @@ void DevicesPage::controlActions()
     StdActions::self()->replaygainAction->setEnabled(!busyDevice && haveTracks && onlyFs && singleUdi && !deviceSelected);
     #endif
     StdActions::self()->organiseFilesAction->setEnabled(!busyDevice && haveTracks && onlyFs && singleUdi && !deviceSelected);
-    StdActions::self()->addToPlayQueueAction->setEnabled(canPlay && !selected.isEmpty() && singleUdi && !busyDevice && haveTracks && (audioCd || !deviceSelected));
-    StdActions::self()->addWithPriorityAction->setEnabled(StdActions::self()->addToPlayQueueAction->isEnabled());
-    StdActions::self()->replacePlayQueueAction->setEnabled(StdActions::self()->addToPlayQueueAction->isEnabled());
+    StdActions::self()->enableAddToPlayQueue(canPlay && !selected.isEmpty() && singleUdi && !busyDevice && haveTracks && (audioCd || !deviceSelected));
     #ifdef ENABLE_REMOTE_DEVICES
     forgetDeviceAction->setEnabled(singleUdi && remoteDev);
     #endif
     #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
     DevicesModel::self()->editAct()->setEnabled(!AlbumDetailsDialog::instanceCount() && !busyDevice && 1==selected.count() && audioCd && haveTracks && deviceSelected);
     #endif
-    menuButton->controlState();
 }
 
 void DevicesPage::copyToLibrary()
@@ -367,7 +348,7 @@ void DevicesPage::copyToLibrary()
     }
 
     QModelIndexList mapped;
-    foreach (const QModelIndex &idx, selected) {
+    for (const QModelIndex &idx: selected) {
         mapped.append(proxy.mapToSource(idx));
     }
 
@@ -425,8 +406,8 @@ void DevicesPage::refreshDevice()
         if (Device::AudioCd==dev->devType()) {
             // Bit hacky - we use 'full' to determine CDDB/MusicBrainz!
             #if defined CDDB_FOUND && defined MUSICBRAINZ5_FOUND
-            switch (MessageBox::questionYesNoCancel(this, i18n("Lookup album and track details?"),
-                                                    i18n("Refresh"), GuiItem(i18n("Via CDDB")), GuiItem(i18n("Via MusicBrainz")))) {
+            switch (MessageBox::questionYesNoCancel(this, tr("Lookup album and track details?"),
+                                                    tr("Refresh"), GuiItem(tr("Via CDDB")), GuiItem(tr("Via MusicBrainz")))) {
             case MessageBox::Yes:
                 full=true; // full==true => CDDB
                 break;
@@ -437,18 +418,20 @@ void DevicesPage::refreshDevice()
                 return;
             }
             #else
-            if (MessageBox::No==MessageBox::questionYesNo(this, i18n("Lookup album and track details?"),
-                                                          i18n("Refresh"), GuiItem(i18n("Refresh")), StdGuiItem::cancel())) {
+            if (MessageBox::No==MessageBox::questionYesNo(this, tr("Lookup album and track details?"),
+                                                          tr("Refresh"), GuiItem(tr("Refresh")), StdGuiItem::cancel())) {
                 return;
             }
             #endif
             dev=DevicesModel::self()->device(udi);
         } else {
             if (dev->childCount() && Device::Mtp!=dev->devType()) {
-                switch (MessageBox::questionYesNoCancel(this, i18n("<p>Which type of refresh do you wish to perform?<ul>"
-                                                                   "<li>Partial - Only new songs are scanned <i>(quick)</i></li>"
-                                                                   "<li>Full - All songs are rescanned <i>(slow)</i></li></ul></p>"),
-                                                        i18n("Refresh"), GuiItem(i18n("Partial")), GuiItem(i18n("Full")))) {
+                static const QChar constBullet(0x2022);
+
+                switch (MessageBox::questionYesNoCancel(this, tr("Which type of refresh do you wish to perform?")+QLatin1String("\n\n")+
+                                                              constBullet+QLatin1Char(' ')+tr("Partial - Only new songs are scanned (quick)")+QLatin1Char('\n')+
+                                                              constBullet+QLatin1Char(' ')+tr("Full - All songs are rescanned (slow)"),
+                                                        tr("Refresh"), GuiItem(tr("Partial")), GuiItem(tr("Full")))) {
                 case MessageBox::Yes:
                     full=false;
                 case MessageBox::No:
@@ -475,7 +458,7 @@ void DevicesPage::deleteSongs()
     }
 
     QModelIndexList mapped;
-    foreach (const QModelIndex &idx, selected) {
+    for (const QModelIndex &idx: selected) {
         mapped.append(proxy.mapToSource(idx));
     }
 
@@ -495,8 +478,8 @@ void DevicesPage::deleteSongs()
     QList<Song> songs=DevicesModel::self()->songs(mapped);
 
     if (!songs.isEmpty()) {
-        if (MessageBox::Yes==MessageBox::warningYesNo(this, i18n("Are you sure you wish to delete the selected songs?\nThis cannot be undone."),
-                                                      i18n("Delete Songs"), StdGuiItem::del(), StdGuiItem::cancel())) {
+        if (MessageBox::Yes==MessageBox::warningYesNo(this, tr("Are you sure you wish to delete the selected songs?\n\nThis cannot be undone."),
+                                                      tr("Delete Songs"), StdGuiItem::del(), StdGuiItem::cancel())) {
             emit deleteSongs(udi, songs);
         }
         view->clearSelection();
@@ -522,7 +505,7 @@ void DevicesPage::forgetRemoteDevice()
     }
     QString udi=dev->id();
     QString devName=dev->data();
-    if (MessageBox::Yes==MessageBox::warningYesNo(this, i18n("Are you sure you wish to forget <b>%1</b>?", devName))) {
+    if (MessageBox::Yes==MessageBox::warningYesNo(this, tr("Are you sure you wish to forget '%1'?").arg(devName))) {
         DevicesModel::self()->removeRemoteDevice(udi);
     }
     #endif
@@ -541,10 +524,10 @@ void DevicesPage::toggleDevice()
         Device *dev=static_cast<Device *>(item);
         if (dev->isConnected() &&
             (Device::AudioCd==dev->devType()
-                ? MessageBox::No==MessageBox::warningYesNo(this, i18n("Are you sure you wish to eject Audio CD <b>%1 - %2</b>?", dev->data(), dev->subText()),
-                                                           i18n("Eject"), GuiItem(i18n("Eject")), StdGuiItem::cancel())
-                : MessageBox::No==MessageBox::warningYesNo(this, i18n("Are you sure you wish to disconnect <b>%1</b>?", dev->data()),
-                                                           i18n("Disconnect"), GuiItem(i18n("Disconnect")), StdGuiItem::cancel()))) {
+                ? MessageBox::No==MessageBox::warningYesNo(this, tr("Are you sure you wish to eject Audio CD '%1 - %2'?").arg(dev->data()).arg(dev->subText()),
+                                                           tr("Eject"), GuiItem(tr("Eject")), StdGuiItem::cancel())
+                : MessageBox::No==MessageBox::warningYesNo(this, tr("Are you sure you wish to disconnect '%1'?").arg(dev->data()),
+                                                           tr("Disconnect"), GuiItem(tr("Disconnect")), StdGuiItem::cancel()))) {
             return;
         }
         static_cast<Device *>(item)->toggle();
@@ -563,7 +546,7 @@ void DevicesPage::sync()
         || 0!=RgDialog::instanceCount()
         #endif
         ) {
-        MessageBox::error(this, i18n("Please close other dialogs first."));
+        MessageBox::error(this, tr("Please close other dialogs first."));
         return;
     }
 
@@ -581,28 +564,6 @@ void DevicesPage::sync()
     }
 }
 
-void DevicesPage::updateGenres(const QModelIndex &idx)
-{
-    if (idx.isValid()) {
-        QModelIndex m=proxy.mapToSource(idx);
-        if (m.isValid()) {
-            MusicLibraryItem *item=static_cast<MusicLibraryItem *>(m.internalPointer());
-            MusicLibraryItem::Type itemType=item->itemType();
-            if (itemType==MusicLibraryItem::Type_Root) {
-                genreCombo->update(static_cast<MusicLibraryItemRoot *>(item)->genres());
-                return;
-            } else if (itemType==MusicLibraryItem::Type_Artist) {
-                genreCombo->update(static_cast<MusicLibraryItemArtist *>(item)->genres());
-                return;
-            } else if (itemType==MusicLibraryItem::Type_Album) {
-                genreCombo->update(static_cast<MusicLibraryItemAlbum *>(item)->genres());
-                return;
-            }
-        }
-    }
-    genreCombo->update(DevicesModel::self()->genres());
-}
-
 void DevicesPage::updated(const QModelIndex &idx)
 {
     view->setExpanded(proxy.mapFromSource(idx));
@@ -658,3 +619,5 @@ void DevicesPage::editDetails()
     }
     #endif
 }
+
+#include "moc_devicespage.cpp"
