@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,9 +24,8 @@
 #ifndef DEVICESPAGE_H
 #define DEVICESPAGE_H
 
-#include "ui_devicespage.h"
 #include "device.h"
-#include "gui/page.h"
+#include "widgets/singlepagewidget.h"
 #ifdef ENABLE_REMOTE_DEVICES
 #include "remotefsdevice.h"
 #endif
@@ -35,37 +34,35 @@
 
 class Action;
 
-class DevicesPage : public QWidget, public Ui::DevicesPage, public Page
+class DevicesPage : public SinglePageWidget
 {
     Q_OBJECT
 
 public:
     DevicesPage(QWidget *p);
-    virtual ~DevicesPage();
+    ~DevicesPage() override;
 
     void clear();
     QString activeFsDeviceUdi() const;
     QStringList playableUrls() const;
-    QList<Song> selectedSongs(bool allowPlaylists=false) const;
-    void addSelectionToPlaylist(const QString &name=QString(), bool replace=false, quint8 priorty=0, bool randomAlbums=false);
-    void setView(int v) { view->setMode((ItemView::Mode)v); }
-    void focusSearch() { view->focusSearch(); }
-    void refresh();
-    void showEvent(QShowEvent *e);
+    QList<Song> selectedSongs(bool allowPlaylists=false) const override;
+    void addSelectionToPlaylist(const QString &name=QString(), int action=MPDConnection::Append, quint8 priority=0, bool decreasePriority=false) override;
+    void focusSearch() override { view->focusSearch(); }
+    void refresh() override;
+    void resort() { proxy.sort(); }
 
 public Q_SLOTS:
     void itemDoubleClicked(const QModelIndex &);
     void searchItems();
-    void controlActions();
+    void controlActions() override;
     void copyToLibrary();
     void configureDevice();
     void refreshDevice();
-    void deleteSongs();
+    void deleteSongs() override;
     void addRemoteDevice();
     void forgetRemoteDevice();
-    void toggleDevice();
     void sync();
-    void updateGenres(const QModelIndex &);
+    void toggleDevice();
     void updated(const QModelIndex &idx);
     void cdMatches(const QString &udi, const QList<CdAlbum> &albums);
     void editDetails();
@@ -74,21 +71,16 @@ private:
     Device * activeFsDevice() const;
 
 Q_SIGNALS:
-    // These are for communicating with MPD object (which is in its own thread, so need to talk via signal/slots)
-    void add(const QStringList &files, bool replace, quint8 priorty);
-    void addSongsToPlaylist(const QString &name, const QStringList &files);
-
     void addToDevice(const QString &from, const QString &to, const QList<Song> &songs);
     void deleteSongs(const QString &from, const QList<Song> &songs);
 
 private:
     MusicLibraryProxyModel proxy;
     Action *copyAction;
+    Action *syncAction;
     #ifdef ENABLE_REMOTE_DEVICES
     Action *forgetDeviceAction;
     #endif
-    Action *syncAction;
-    QSet<QString> genres;
 };
 
 #endif
