@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,7 +26,7 @@
 #include "tags/tags.h"
 #include "cdparanoia.h"
 #include "gui/covers.h"
-#include "mpd/mpdconnection.h"
+#include "mpd-interface/mpdconnection.h"
 #include "gui/settings.h"
 #include <QStringList>
 #include <QProcess>
@@ -87,7 +87,7 @@ void ExtractJob::run()
         emit result(Device::Cancelled);
     } else {
         QStringList encParams=encoder.params(value, encoder.transcoder ? "pipe:" : "-", destFile);
-        CdParanoia cdparanoia(srcFile, Settings::self()->paranoiaFull(), Settings::self()->paranoiaNeverSkip());
+        CdParanoia cdparanoia(srcFile, Settings::self()->paranoiaFull(), Settings::self()->paranoiaNeverSkip(), false, Settings::self()->paranoiaOffset());
 
         if (!cdparanoia) {
             emit result(Device::FailedToLockDevice);
@@ -153,13 +153,11 @@ void ExtractJob::run()
         Tags::update(destFile, Song(), song, 3);
 
         if (!stopRequested && !coverFile.isEmpty()) {
-            QString mpdCover=MPDConnection::self()->getDetails().coverName;
-            if (mpdCover.isEmpty()) {
-                mpdCover=Covers::constFileName;
-            }
-            copiedCover=Covers::copyImage(Utils::getDir(coverFile), Utils::getDir(destFile), Utils::getFile(coverFile), mpdCover+coverFile.mid(coverFile.length()-4), 0);
+            copiedCover=Covers::copyImage(Utils::getDir(coverFile), Utils::getDir(destFile), Utils::getFile(coverFile), Covers::albumFileName(song)+Utils::getExtension(coverFile), 0);
         }
 
         emit result(Device::Ok);
     }
 }
+
+#include "moc_extractjob.cpp"
