@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,8 +25,8 @@
 #define FILENAMESCHEMEDIALOG_H
 
 #include "ui_filenameschemedialog.h"
-#include "dialog.h"
-#include "song.h"
+#include "support/dialog.h"
+#include "mpd-interface/song.h"
 #include "deviceoptions.h"
 
 class FilenameSchemeDialog : public Dialog, Ui::FilenameSchemeDialog
@@ -47,7 +47,7 @@ private Q_SLOTS:
     void updateExample();
 
 private:
-    void slotButtonClicked(int button);
+    void slotButtonClicked(int button) override;
 
 private:
     DeviceOptions origOpts;
