@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,21 +23,19 @@
 
 #include "umsdevice.h"
 #include "tags/tags.h"
-#include "models/musiclibrarymodel.h"
 #include "models/musiclibraryitemsong.h"
 #include "models/musiclibraryitemalbum.h"
 #include "models/musiclibraryitemartist.h"
 #include "models/musiclibraryitemroot.h"
-#include "models/dirviewmodel.h"
+#include "models/mpdlibrarymodel.h"
 #include "devicepropertiesdialog.h"
 #include "devicepropertieswidget.h"
 #include "support/utils.h"
-#include "mpd/mpdparseutils.h"
-#include "mpd/mpdconnection.h"
+#include "mpd-interface/mpdparseutils.h"
+#include "mpd-interface/mpdconnection.h"
 #include "encoders.h"
 #include "transcodingjob.h"
 #include "actiondialog.h"
-#include "support/localize.h"
 #include "gui/covers.h"
 #include "support/thread.h"
 #include <QDir>
@@ -61,12 +59,12 @@ const QLatin1String FsDevice::constUseCacheKey("use_cache"); // Cantata extensio
 const QLatin1String FsDevice::constDefCoverFileName("cover.jpg");
 const QLatin1String FsDevice::constAutoScanKey("auto_scan"); // Cantata extension!
 
-MusicScanner::MusicScanner()
-    : QObject(0)
+MusicScanner::MusicScanner(const QString &id)
+    : QObject(nullptr)
     , stopRequested(false)
     , count(0)
 {
-    thread=new Thread(metaObject()->className());
+    thread=new Thread(metaObject()->className()+QLatin1String("::")+id);
     moveToThread(thread);
     thread->start();
 }
@@ -80,10 +78,8 @@ void MusicScanner::scan(const QString &folder, const QString &cacheFile, bool re
 {
     if (!cacheFile.isEmpty() && readCache) {
         MusicLibraryItemRoot *lib=new MusicLibraryItemRoot;
-        MusicLibraryModel::convertCache(cacheFile);
         readProgress(0.0);
-        if (lib->fromXML(cacheFile, QDateTime(), 0, folder)) {
-            lib->applyGrouping();
+        if (lib->fromXML(cacheFile, folder)) {
             if (!stopRequested) {
                 emit libraryUpdated(lib);
             } else {
@@ -106,10 +102,9 @@ void MusicScanner::scan(const QString &folder, const QString &cacheFile, bool re
     scanFolder(library, topLevel, topLevel, existing, 0);
 
     if (!stopRequested) {
-        library->applyGrouping();
         if (!cacheFile.isEmpty()) {
             writeProgress(0.0);
-            library->toXML(cacheFile, QDateTime(), false, this);
+            library->toXML(cacheFile, this);
         }
         emit libraryUpdated(library);
     } else {
@@ -120,7 +115,7 @@ void MusicScanner::scan(const QString &folder, const QString &cacheFile, bool re
 void MusicScanner::saveCache(const QString &cache, MusicLibraryItemRoot *lib)
 {
     writeProgress(0.0);
-    lib->toXML(cache, QDateTime(), false, this);
+    lib->toXML(cache, this);
     emit cacheSaved();
 }
 
@@ -128,7 +123,7 @@ void MusicScanner::stop()
 {
     stopRequested=true;
     thread->stop();
-    thread=0;
+    thread=nullptr;
 }
 
 void MusicScanner::scanFolder(MusicLibraryItemRoot *library, const QString &topLevel, const QString &f,
@@ -140,9 +135,9 @@ void MusicScanner::scanFolder(MusicLibraryItemRoot *library, const QString &topL
     if (level<4) {
         QDir d(f);
         QFileInfoList entries=d.entryInfoList(QDir::Files|QDir::NoSymLinks|QDir::Dirs|QDir::NoDotAndDotDot);
-        MusicLibraryItemArtist *artistItem = 0;
-        MusicLibraryItemAlbum *albumItem = 0;
-        foreach (const QFileInfo &info, entries) {
+        MusicLibraryItemArtist *artistItem = nullptr;
+        MusicLibraryItemAlbum *albumItem = nullptr;
+        for (const QFileInfo &info: entries) {
             if (stopRequested) {
                 return;
             }
@@ -175,6 +170,7 @@ void MusicScanner::scanFolder(MusicLibraryItemRoot *library, const QString &topL
                 }
 
                 song.fillEmptyFields();
+                song.populateSorts();
                 song.size=info.size();
                 if (!artistItem || song.artistOrComposer()!=artistItem->data()) {
                     artistItem = library->artist(song);
@@ -182,12 +178,7 @@ void MusicScanner::scanFolder(MusicLibraryItemRoot *library, const QString &topL
                 if (!albumItem || albumItem->parentItem()!=artistItem || song.albumName()!=albumItem->data()) {
                     albumItem = artistItem->album(song);
                 }
-                MusicLibraryItemSong *songItem = new MusicLibraryItemSong(song, albumItem);
-                const QSet<QString> &songGenres=songItem->allGenres();
-                albumItem->append(songItem);
-                albumItem->addGenres(songGenres);
-                artistItem->addGenres(songGenres);
-                library->addGenres(songGenres);
+                albumItem->append(new MusicLibraryItemSong(song, albumItem));
             }
         }
     }
@@ -221,10 +212,27 @@ bool FsDevice::readOpts(const QString &fileName, DeviceOptions &opts, bool readA
                 opts.fixVariousArtists=QLatin1String("true")==line.section('=', 1, 1);
             } else if (line.startsWith(constTranscoderKey+"="))  {
                 QStringList parts=line.section('=', 1, 1).split(',');
-                if (3==parts.size()) {
+                if (parts.size()>=3) {
                     opts.transcoderCodec=parts.at(0);
                     opts.transcoderValue=parts.at(1).toInt();
-                    opts.transcoderWhenDifferent=QLatin1String("true")==parts.at(2);
+                    if (parts.size()>=4) {
+                        if (QLatin1String("true")==parts.at(3)) {
+                            opts.transcoderWhen=DeviceOptions::TW_IfLossess;
+                        } else if (QLatin1String("true")==parts.at(2)) {
+                            opts.transcoderWhen=DeviceOptions::TW_IfDifferent;
+                        } else {
+                            opts.transcoderWhen=DeviceOptions::TW_Always;
+                        }
+                    } else {
+                        const QString &val = parts.at(2);
+                        if (QLatin1String("true")==val) {
+                            opts.transcoderWhen=DeviceOptions::TW_IfDifferent;
+                        } else if (QLatin1String("false")==val) {
+                            opts.transcoderWhen=DeviceOptions::TW_Always;
+                        } else {
+                            opts.transcoderWhen=(DeviceOptions::TranscodeWhen)val.toInt();
+                        }
+                    }
                 }
             } else if (line.startsWith(constUseCacheKey+"=")) {
                 opts.useCache=QLatin1String("true")==line.section('=', 1, 1);
@@ -306,7 +314,8 @@ void FsDevice::writeOpts(const QString &fileName, const DeviceOptions &opts, boo
             out << constVariousArtistsFixKey << '=' << toString(opts.fixVariousArtists) << '\n';
         }
         if (!opts.transcoderCodec.isEmpty()) {
-            out << constTranscoderKey << '=' << opts.transcoderCodec << ',' << opts.transcoderValue << ',' << toString(opts.transcoderWhenDifferent) << '\n';
+            out << constTranscoderKey << '=' << opts.transcoderCodec << ',' << opts.transcoderValue
+                << ',' << opts.transcoderWhen << '\n';
         }
         if (opts.useCache!=def.useCache) {
             out << constUseCacheKey << '=' << toString(opts.useCache) << '\n';
@@ -317,21 +326,21 @@ void FsDevice::writeOpts(const QString &fileName, const DeviceOptions &opts, boo
     }
 }
 
-FsDevice::FsDevice(MusicModel *m, Solid::Device &dev)
+FsDevice::FsDevice(MusicLibraryModel *m, Solid::Device &dev)
     : Device(m, dev)
     , state(Idle)
     , scanned(false)
     , cacheProgress(-1)
-    , scanner(0)
+    , scanner(nullptr)
 {
 }
 
-FsDevice::FsDevice(MusicModel *m, const QString &name, const QString &id)
+FsDevice::FsDevice(MusicLibraryModel *m, const QString &name, const QString &id)
     : Device(m, name, id)
     , state(Idle)
     , scanned(false)
     , cacheProgress(-1)
-    , scanner(0)
+    , scanner(nullptr)
 {
 }
 
@@ -355,7 +364,7 @@ void FsDevice::rescan(bool full)
 
 void FsDevice::stop()
 {
-     if (0!=scanner) {
+     if (nullptr!=scanner) {
          stopScanner();
      }
 }
@@ -390,13 +399,21 @@ void FsDevice::addSong(const Song &s, bool overwrite, bool copyCover)
     currentDestFile=audioFolder+opts.createFilename(s);
     Encoders::Encoder encoder;
 
+    transcoding = false;
     if (!opts.transcoderCodec.isEmpty()) {
         encoder=Encoders::getEncoder(opts.transcoderCodec);
         if (encoder.codec.isEmpty()) {
             emit actionStatus(CodecNotAvailable);
             return;
         }
-        currentDestFile=encoder.changeExtension(currentDestFile);
+
+        transcoding = !opts.transcoderCodec.isEmpty() &&
+                         (DeviceOptions::TW_IfDifferent!=opts.transcoderWhen || encoder.isDifferent(s.file)) &&
+                         (DeviceOptions::TW_IfLossess!=opts.transcoderWhen || Device::isLossless(s.file));
+
+        if (transcoding) {
+            currentDestFile=encoder.changeExtension(currentDestFile);
+        }
     }
 
     if (!overwrite && QFile::exists(currentDestFile)) {
@@ -410,19 +427,19 @@ void FsDevice::addSong(const Song &s, bool overwrite, bool copyCover)
         return;
     }
     currentSong=s;
-    if (encoder.codec.isEmpty() || (opts.transcoderWhenDifferent && !encoder.isDifferent(s.file))) {
-        transcoding=false;
-        CopyJob *job=new CopyJob(s.file, currentDestFile, copyCover ? opts : DeviceOptions(Device::constNoCover),
-                                 (needToFixVa ? CopyJob::OptsApplyVaFix : CopyJob::OptsNone)|(Device::RemoteFs==devType() ? CopyJob::OptsFixLocal : CopyJob::OptsNone),
-                                 currentSong);
+
+    if (transcoding) {
+        TranscodingJob *job=new TranscodingJob(encoder, opts.transcoderValue, s.file, currentDestFile, copyCover ? opts : DeviceOptions(Device::constNoCover),
+                                               (needToFixVa ? CopyJob::OptsApplyVaFix : CopyJob::OptsNone)|
+                                                   (Device::RemoteFs==devType() ? CopyJob::OptsFixLocal : CopyJob::OptsNone),
+                                               currentSong);
         connect(job, SIGNAL(result(int)), SLOT(addSongResult(int)));
         connect(job, SIGNAL(percent(int)), SLOT(percent(int)));
         job->start();
     } else {
-        transcoding=true;
-        TranscodingJob *job=new TranscodingJob(encoder, opts.transcoderValue, s.file, currentDestFile, copyCover ? opts : DeviceOptions(Device::constNoCover),
-                                               (needToFixVa ? CopyJob::OptsApplyVaFix : CopyJob::OptsNone)|
-                                               (Device::RemoteFs==devType() ? CopyJob::OptsFixLocal : CopyJob::OptsNone));
+        CopyJob *job=new CopyJob(s.file, currentDestFile, copyCover ? opts : DeviceOptions(Device::constNoCover),
+                                 (needToFixVa ? CopyJob::OptsApplyVaFix : CopyJob::OptsNone)|(Device::RemoteFs==devType() ? CopyJob::OptsFixLocal : CopyJob::OptsNone),
+                                 currentSong);
         connect(job, SIGNAL(result(int)), SLOT(addSongResult(int)));
         connect(job, SIGNAL(percent(int)), SLOT(percent(int)));
         job->start();
@@ -445,7 +462,7 @@ void FsDevice::copySongTo(const Song &s, const QString &musicPath, bool overwrit
         if (needToFixVa) {
             Device::fixVariousArtists(QString(), check, false);
         }
-        if (MusicLibraryModel::self()->songExists(check)) {
+        if (MpdLibraryModel::self()->songExists(check)) {
             emit actionStatus(SongExists);
             return;
         }
@@ -507,8 +524,9 @@ void FsDevice::cleanDirs(const QSet<QString> &dirs)
     job->start();
 }
 
-void FsDevice::requestCover(const Song &s)
+Covers::Image FsDevice::requestCover(const Song &s)
 {
+    Covers::Image i;
     QString songFile=audioFolder+s.file;
     QString dirName=Utils::getDir(songFile);
 
@@ -516,19 +534,20 @@ void FsDevice::requestCover(const Song &s)
         QImage img(dirName+opts.coverName);
         if (!img.isNull()) {
             emit cover(s, img);
-            return;
+            return Covers::Image(img, dirName+opts.coverName);
         }
     }
 
     QStringList files=QDir(dirName).entryList(QStringList() << QLatin1String("*.jpg") << QLatin1String("*.png"), QDir::Files|QDir::Readable);
-    foreach (const QString &fileName, files) {
+    for (const QString &fileName: files) {
         QImage img(dirName+fileName);
 
         if (!img.isNull()) {
             emit cover(s, img);
-            return;
+            return Covers::Image(img, dirName+fileName);
         }
     }
+    return Covers::Image();
 }
 
 void FsDevice::percent(int pc)
@@ -583,7 +602,7 @@ void FsDevice::copySongToResult(int status)
     } else {
         currentSong.file=currentDestFile.mid(MPDConnection::self()->getDetails().dir.length());
         QString origPath;
-        if (MPDConnection::self()->isMopdidy()) {
+        if (MPDConnection::self()->isMopidy()) {
             origPath=currentSong.file;
             currentSong.file=Song::encodePath(currentSong.file);
         }
@@ -591,9 +610,9 @@ void FsDevice::copySongToResult(int status)
             currentSong.revertVariousArtists();
         }
         Utils::setFilePerms(currentDestFile);
-        MusicLibraryModel::self()->addSongToList(currentSong);
-        DirViewModel::self()->addFileToList(origPath.isEmpty() ? currentSong.file : origPath,
-                                            origPath.isEmpty() ? QString() : currentSong.file);
+//        MusicLibraryModel::self()->addSongToList(currentSong);
+//        DirViewModel::self()->addFileToList(origPath.isEmpty() ? currentSong.file : origPath,
+//                                            origPath.isEmpty() ? QString() : currentSong.file);
         emit actionStatus(Ok, job && job->coverCopied());
     }
 }
@@ -632,7 +651,7 @@ void FsDevice::initScaner()
              qRegisterMetaType<QSet<FileOnlySong> >("QSet<FileOnlySong>");
              registeredTypes=true;
         }
-        scanner=new MusicScanner();
+        scanner=new MusicScanner(data());
         connect(scanner, SIGNAL(libraryUpdated(MusicLibraryItemRoot *)), this, SLOT(libraryUpdated(MusicLibraryItemRoot *)));
         connect(scanner, SIGNAL(songCount(int)), this, SLOT(songCount(int)));
         connect(scanner, SIGNAL(cacheSaved()), this, SLOT(savedCache()));
@@ -651,13 +670,13 @@ void FsDevice::startScanner(bool fullScan)
     if (!fullScan) {
         QSet<Song> songs=allSongs();
 
-        foreach (const Song &s, songs) {
+        for (const Song &s: songs) {
             existingSongs.insert(FileOnlySong(s));
         }
     }
     state=Updating;
     emit scan(audioFolder, opts.useCache ? cacheFileName() : QString(), !scanned, existingSongs);
-    setStatusMessage(i18n("Updating..."));
+    setStatusMessage(tr("Updating..."));
     emit updating(id(), true);
 }
 
@@ -673,7 +692,7 @@ void FsDevice::stopScanner()
     disconnect(scanner, SIGNAL(savingCache(int)), this, SLOT(savingCache(int)));
     disconnect(scanner, SIGNAL(readingCache(int)), this, SLOT(readingCache(int)));
     scanner->deleteLater();
-    scanner=0;
+    scanner=nullptr;
 }
 
 void FsDevice::clear() const
@@ -703,7 +722,7 @@ QString FsDevice::cacheFileName() const
     if (audioFolder.isEmpty()) {
         setAudioFolder();
     }
-    return audioFolder+constCantataCacheFile+MusicLibraryModel::constLibraryCompressedExt;
+    return audioFolder+constCantataCacheFile+".xml.gz";
 }
 
 void FsDevice::saveCache()
@@ -729,29 +748,24 @@ void FsDevice::removeCache()
     if (QFile::exists(cacheFile)) {
         QFile::remove(cacheFile);
     }
-
-    // Remove old (non-compressed) cache file as well...
-    QString oldCache=cacheFile;
-    oldCache.replace(MusicLibraryModel::constLibraryCompressedExt, MusicLibraryModel::constLibraryExt);
-    if (oldCache!=cacheFile && QFile::exists(oldCache)) {
-        QFile::remove(oldCache);
-    }
 }
 
 void FsDevice::readingCache(int pc)
 {
-    cacheStatus(i18n("Reading cache"), pc);
+    cacheStatus(tr("Reading cache"), pc);
 }
 
 void FsDevice::savingCache(int pc)
 {
-    cacheStatus(i18n("Saving cache"), pc);
+    cacheStatus(tr("Saving cache"), pc);
 }
 
 void FsDevice::cacheStatus(const QString &msg, int prog)
 {
     if (prog!=cacheProgress) {
         cacheProgress=prog;
-        setStatusMessage(i18nc("Message percent", "%1 %2%", msg, cacheProgress));
+        setStatusMessage(tr("%1 %2%","Message percent").arg(msg).arg(cacheProgress));
     }
 }
+
+#include "moc_fsdevice.cpp"
