@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,42 +22,40 @@
  */
 
 #include "mtpdevice.h"
-#include "models/musiclibrarymodel.h"
 #include "models/musiclibraryitemsong.h"
 #include "models/musiclibraryitemalbum.h"
 #include "models/musiclibraryitemartist.h"
 #include "models/musiclibraryitemroot.h"
-#include "models/dirviewmodel.h"
+#include "models/mpdlibrarymodel.h"
+#include "models/devicesmodel.h"
 #include "devicepropertiesdialog.h"
 #include "devicepropertieswidget.h"
 #include "gui/covers.h"
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 #include "encoders.h"
 #include "transcodingjob.h"
 #include "support/utils.h"
-#include "mpd/mpdparseutils.h"
-#include "mpd/mpdconnection.h"
-#include "support/localize.h"
+#include "mpd-interface/mpdparseutils.h"
+#include "mpd-interface/mpdconnection.h"
 #include "filejob.h"
 #include "support/configuration.h"
 #include "support/thread.h"
+#include "support/monoicon.h"
 #include <QTimer>
 #include <QDir>
 #include <QTemporaryFile>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KMimeType>
-#include <solid/genericinterface.h>
-#else
-#include "solid-lite/genericinterface.h"
-#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 //#define TIME_MTP_OPERATIONS
 #ifdef TIME_MTP_OPERATIONS
-#include <QDebug>
 #include <QElapsedTimer>
 #endif
+#include <QDebug>
+
+#define COVER_DBUG if (Covers::debugEnabled()) qWarning() << metaObject()->className() << __FUNCTION__
+#define DBUG_CLASS(CLASS) if (DevicesModel::debugEnabled()) qWarning() << CLASS << QThread::currentThread()->objectName() << __FUNCTION__
+#define DBUG DBUG_CLASS(metaObject()->className())
 
 // Enable the following #define to have Cantata attempt to ascertain the AlbumArtist tag by
 // looking at the file path
@@ -71,6 +69,8 @@ static const QLatin1String constMtpDefaultCover("AlbumArt.jpg");
 static const uint32_t constRootFolder=0xffffffffU;
 static const QString constMusicFolder=QLatin1String("Music");
 
+static const quint16 constOrigFileName = Song::Performer;
+
 static int progressMonitor(uint64_t const processed, uint64_t const total, void const * const data)
 {
     const MtpConnection *con=static_cast<const MtpConnection *>(data);
@@ -94,7 +94,7 @@ static uint16_t fileReceiver(void *params, void *priv, uint32_t sendlen, unsigne
     return LIBMTP_HANDLER_RETURN_OK;
 }
 
-MtpConnection::MtpConnection(unsigned int bus, unsigned int dev, bool aaSupport)
+MtpConnection::MtpConnection(const QString &id, unsigned int bus, unsigned int dev, bool aaSupport)
     : device(0)
     #ifdef MTP_CLEAN_ALBUMS
     , albums(0)
@@ -109,7 +109,7 @@ MtpConnection::MtpConnection(unsigned int bus, unsigned int dev, bool aaSupport)
     size=0;
     used=0;
     LIBMTP_Init();
-    thread=new Thread(metaObject()->className());
+    thread=new Thread(metaObject()->className()+QLatin1String("::")+id);
     moveToThread(thread);
     thread->start();
 }
@@ -153,9 +153,9 @@ void MtpConnection::connectToDevice()
     defaultMusicFolder=0;
     LIBMTP_raw_device_t *rawDevices=0;
     int numDev=-1;
-    emit statusMessage(i18n("Connecting to device..."));
+    emit statusMessage(tr("Connecting to device..."));
     if (LIBMTP_ERROR_NONE!=LIBMTP_Detect_Raw_Devices(&rawDevices, &numDev) || numDev<=0) {
-        emit statusMessage(i18n("No devices found"));
+        emit statusMessage(tr("No devices found"));
         return;
     }
     #ifdef TIME_MTP_OPERATIONS
@@ -167,23 +167,13 @@ void MtpConnection::connectToDevice()
     for (int i = 0; i < numDev; i++) {
         if (0!=busNum && 0!=devNum) {
             if (rawDevices[i].bus_location==busNum && rawDevices[i].devnum==devNum) {
-                #ifdef ENABLE_UNCACHED_MTP
                 mptDev = LIBMTP_Open_Raw_Device_Uncached(&rawDevices[i]);
-                #else
-                mptDev = LIBMTP_Open_Raw_Device(&rawDevices[i]);
-                #endif
                 break;
             }
         } else {
-            #ifdef ENABLE_UNCACHED_MTP
             if ((mptDev = LIBMTP_Open_Raw_Device_Uncached(&rawDevices[i]))) {
                 break;
             }
-            #else
-            if ((mptDev = LIBMTP_Open_Raw_Device(&rawDevices[i]))) {
-                break;
-            }
-            #endif
         }
     }
 
@@ -202,7 +192,7 @@ void MtpConnection::connectToDevice()
 
     free(rawDevices);
     if (!device) {
-        emit statusMessage(i18n("No devices found"));
+        emit statusMessage(tr("No devices found"));
         #ifdef TIME_MTP_OPERATIONS
         qWarning() << "TOTAL connect:" <<totalTimer.elapsed();
         #endif
@@ -217,7 +207,7 @@ void MtpConnection::connectToDevice()
     }
 
     defaultMusicFolder=device->default_music_folder;
-    emit statusMessage(i18n("Connected to device"));
+    emit statusMessage(tr("Connected to device"));
     #ifdef TIME_MTP_OPERATIONS
     qWarning() << "TOTAL connect:" <<totalTimer.elapsed();
     #endif
@@ -230,7 +220,7 @@ void MtpConnection::disconnectFromDevice(bool showStatus)
         LIBMTP_Release_Device(device);
         device=0;
         if (showStatus) {
-            emit statusMessage(i18n("Disconnected from device"));
+            emit statusMessage(tr("Disconnected from device"));
         }
     }
 }
@@ -303,15 +293,11 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
     #endif
 
     library = new MusicLibraryItemRoot;
-    emit statusMessage(i18n("Updating folders..."));
-    #ifdef ENABLE_UNCACHED_MTP
+    emit statusMessage(tr("Updating folders..."));
     updateFilesAndFolders();
     if (abortRequested) {
         return;
     }
-    #else
-    updateFolders();
-    #endif
     #ifdef TIME_MTP_OPERATIONS
     qWarning() << "Folder update:" << timer.elapsed();
     timer.restart();
@@ -321,14 +307,6 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
         emit libraryUpdated();
         return;
     }
-    #ifndef ENABLE_UNCACHED_MTP
-    emit statusMessage(i18n("Updating files..."));
-    updateFiles();
-    #ifdef TIME_MTP_OPERATIONS
-    qWarning() << "Files update:" << timer.elapsed();
-    timer.restart();
-    #endif
-    #endif
     #ifdef MTP_CLEAN_ALBUMS
     updateAlbums();
     #ifdef TIME_MTP_OPERATIONS
@@ -336,7 +314,7 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
     timer.restart();
     #endif
     #endif
-    emit statusMessage(i18n("Updating tracks..."));
+    emit statusMessage(tr("Updating tracks..."));
     lastListPercent=-1;
     LIBMTP_track_t *tracks=LIBMTP_Get_Tracklisting_With_Callback(device, &trackListMonitor, this);
     QMap<uint32_t, Folder>::ConstIterator folderEnd=folderMap.constEnd();
@@ -388,11 +366,12 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
         }
         s.year=QString::fromUtf8(track->date).mid(0, 4).toUInt();
         s.title=QString::fromUtf8(track->title);
-        s.genre=QString::fromUtf8(track->genre);
+        s.genres[0]=QString::fromUtf8(track->genre);
         s.track=track->tracknumber;
         s.time=(track->duration/1000.0)+0.5;
         s.size=track->filesize;
         s.fillEmptyFields();
+        s.populateSorts();
         #ifdef MTP_FAKE_ALBUMARTIST_SUPPORT
         if (getAlbumArtistFromPath) {
             QStringList folderParts=(*folder).path.split('/', QString::SkipEmptyParts);
@@ -430,11 +409,7 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
             albumItem = artistItem->album(s);
         }
         MusicLibraryItemSong *songItem = new MusicLibraryItemSong(s, albumItem);
-        const QSet<QString> &songGenres=songItem->allGenres();
         albumItem->append(songItem);
-        albumItem->addGenres(songGenres);
-        artistItem->addGenres(songGenres);
-        library->addGenres(songGenres);
 
         #ifdef MTP_FAKE_ALBUMARTIST_SUPPORT
         // Store AlbumName->Artists/Songs mapping
@@ -469,7 +444,7 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
             QSet<quint16> tracks;
             QString shortestArtist;
             bool duplicateTrackNumbers=false;
-            foreach (MusicLibraryItemSong *s, (*it).songs) {
+            for (const MusicLibraryItemSong *s: (*it).songs) {
                 if (tracks.contains(s->track())) {
                     duplicateTrackNumbers=true;
                     break;
@@ -487,7 +462,7 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
                 // Now, check to see if all artists contain 'shortestArtist'. If so then use 'shortestArtist' as the album
                 // artist. This is probably due to songs which have artist set to '${artist} and somebodyelse'
                 QString albumArtist=shortestArtist;
-                foreach (const QString &artist, (*it).artists) {
+                for (const QString &artist: (*it).artists) {
                     if (!artist.contains(shortestArtist)) {
                         // Found an artist that did not contain 'shortestArtist', so use 'Various Artists' for album artist
                         albumArtist=!f.artist.isEmpty() && f.album==it.key() ? f.artist : Song::variousArtists();
@@ -496,7 +471,7 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
                 }
 
                 // Now move songs to correct artist/album...
-                foreach (MusicLibraryItemSong *s, (*it).songs) {
+                for (MusicLibraryItemSong *s: (*it).songs) {
                     if (s->song().albumartist==albumArtist) {
                         continue;
                     }
@@ -506,8 +481,6 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
                     albumItem=artistItem->album(song);
                     MusicLibraryItemSong *songItem = new MusicLibraryItemSong(song, albumItem);
                     albumItem->append(songItem);
-                    albumItem->updateGenres();
-                    artistItem->updateGenres();
                     MusicLibraryItemAlbum *prevAlbum=(MusicLibraryItemAlbum *)s->parentItem();
                     prevAlbum->remove(s);
                     if (0==prevAlbum->childCount()) {
@@ -531,7 +504,6 @@ void MtpConnection::updateLibrary(const DeviceOptions &opts)
     if (abortRequested) {
         return;
     }
-    library->applyGrouping();
     #ifdef TIME_MTP_OPERATIONS
     qWarning() << "Grouping:" << timer.elapsed();
     qWarning() << "TOTAL update:" <<totalTimer.elapsed();
@@ -552,11 +524,10 @@ void MtpConnection::setMusicFolder(Storage &store)
     }
 }
 
-#ifdef ENABLE_UNCACHED_MTP
 void MtpConnection::updateFilesAndFolders()
 {
     folderMap.clear();
-    foreach (const Storage st, storage) {
+    for (const Storage &st: storage) {
         if (abortRequested) {
             return;
         }
@@ -600,43 +571,6 @@ void MtpConnection::listFolder(uint32_t storage, uint32_t parentDir, Folder *f)
         LIBMTP_destroy_file_t(file);
     }
 }
-#else
-void MtpConnection::updateFolders()
-{
-    folderMap.clear();
-    LIBMTP_folder_t *folders=LIBMTP_Get_Folder_List(device);
-    parseFolder(folders);
-    if (folders) {
-        // LIBMTP_destroy_folder_t is recursive
-        LIBMTP_destroy_folder_t(folders);
-    }
-}
-
-void MtpConnection::updateFiles()
-{
-    LIBMTP_file_t *files=LIBMTP_Get_Filelisting_With_Callback(device, 0, 0);
-    QSet<uint32_t> folders=folderMap.keys().toSet();
-    while (files) {
-        LIBMTP_file_t *file=files;
-        files=files->next;
-
-        if (folders.contains(file->parent_id)) {
-            Folder &folder=folderMap[file->parent_id];
-            if (LIBMTP_FILETYPE_FOLDER==file->filetype) {
-                folder.folders.insert(file->item_id);
-            } else {
-                QString name=QString::fromUtf8(file->filename);
-                if (name.endsWith(".jpg", Qt::CaseInsensitive) || name.endsWith(".png", Qt::CaseInsensitive) || QLatin1String("albumart.pamp")==name) {
-                    folder.covers.insert(file->item_id, File(name, file->filesize, file->item_id));
-                } else {
-                    folder.files.insert(file->item_id, File(name, file->filesize, file->item_id));
-                }
-            }
-        }
-        LIBMTP_destroy_file_t(file);
-    }
-}
-#endif
 
 void MtpConnection::updateStorage()
 {
@@ -646,6 +580,12 @@ void MtpConnection::updateStorage()
         LIBMTP_devicestorage_struct *s=device->storage;
         while (s) {
             QString volumeIdentifier=QString::fromUtf8(s->VolumeIdentifier);
+            if (volumeIdentifier.isEmpty()) {
+                volumeIdentifier=QString::fromUtf8(s->StorageDescription);
+            }
+            if (volumeIdentifier.isEmpty()) {
+                volumeIdentifier="ID:"+QString::number(s->id);
+            }
             QList<Storage>::Iterator it=storage.begin();
             QList<Storage>::Iterator end=storage.end();
             for ( ;it!=end; ++it) {
@@ -760,7 +700,7 @@ uint32_t MtpConnection::checkFolderStructure(const QStringList &dirs, Storage &s
     QString path;
     uint32_t parentId=store.musicFolderId;
 
-    foreach (const QString &d, dirs) {
+    for (const QString &d: dirs) {
         path+=d+QChar('/');
         uint32_t folderId=getFolder(path, store.id);
         if (0==folderId) {
@@ -777,37 +717,6 @@ uint32_t MtpConnection::checkFolderStructure(const QStringList &dirs, Storage &s
     return parentId;
 }
 
-#ifndef ENABLE_UNCACHED_MTP
-void MtpConnection::parseFolder(LIBMTP_folder_t *folder)
-{
-    if (!folder) {
-        return;
-    }
-
-    QMap<uint32_t, Folder>::ConstIterator it=folderMap.find(folder->parent_id);
-    QString path;
-    if (it!=folderMap.constEnd()) {
-        path=it.value().path+QString::fromUtf8(folder->name)+QChar('/');
-    } else {
-        path=QString::fromUtf8(folder->name)+QChar('/');
-    }
-    bool isMusic=path.startsWith(constMusicFolder+QLatin1Char('/'), Qt::CaseInsensitive);
-    if (isMusic) {
-        folderMap.insert(folder->folder_id, Folder(path, folder->folder_id, folder->parent_id, folder->storage_id));
-        if (folderMap.contains(folder->parent_id)) {
-            folderMap[folder->parent_id].folders.insert(folder->folder_id);
-        }
-    }
-    // Only recurse into music folder...
-    if (folder->child && isMusic) {
-        parseFolder(folder->child);
-    }
-    if (folder->sibling) {
-        parseFolder(folder->sibling);
-    }
-}
-#endif
-
 static char * createString(const QString &str)
 {
     return str.isEmpty() ? qstrdup("") : qstrdup(str.toUtf8());
@@ -815,37 +724,6 @@ static char * createString(const QString &str)
 
 static LIBMTP_filetype_t mtpFileType(const QString &f)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    KMimeType::Ptr mime=KMimeType::findByPath(f);
-
-    if (mime->is("audio/mpeg")) {
-        return LIBMTP_FILETYPE_MP3;
-    }
-    if (mime->is("audio/ogg")) {
-        return LIBMTP_FILETYPE_OGG;
-    }
-    if (mime->is("audio/x-ms-wma")) {
-        return LIBMTP_FILETYPE_WMA;
-    }
-    if (mime->is("audio/mp4")) {
-        return LIBMTP_FILETYPE_M4A; // LIBMTP_FILETYPE_MP4
-    }
-    if (mime->is("audio/aac")) {
-        return LIBMTP_FILETYPE_AAC;
-    }
-    if (mime->is("audio/flac")) {
-        return LIBMTP_FILETYPE_FLAC;
-    }
-    if (mime->is("audio/x-wav")) {
-        return LIBMTP_FILETYPE_WAV;
-    }
-    if (mime->is("image/jpeg")) {
-        return LIBMTP_FILETYPE_JPEG;
-    }
-    if (mime->is("image/png")) {
-        return LIBMTP_FILETYPE_PNG;
-    }
-    #else
     if (f.endsWith(".mp3", Qt::CaseInsensitive)) {
         return LIBMTP_FILETYPE_MP3;
     }
@@ -873,7 +751,6 @@ static LIBMTP_filetype_t mtpFileType(const QString &f)
     if (f.endsWith(".png", Qt::CaseInsensitive)) {
         return LIBMTP_FILETYPE_PNG;
     }
-    #endif
     return LIBMTP_FILETYPE_UNDEF_AUDIO;
 }
 
@@ -883,9 +760,9 @@ static QTemporaryFile * saveImageToTemp(const QImage &img, const QString &name)
 
     int index=name.lastIndexOf('.');
     if (index>0) {
-        temp=new QTemporaryFile(QDir::tempPath()+"/cantata_XXXXXX"+name.mid(index));
+        temp=new QTemporaryFile("cantata_XXXXXX"+name.mid(index));
     } else {
-        temp=new QTemporaryFile(QDir::tempPath()+"/cantata_XXXXXX");
+        temp=new QTemporaryFile("cantata_XXXXXX");
     }
     img.save(temp);
     temp->close();
@@ -966,7 +843,7 @@ void MtpConnection::putSong(const Song &s, bool fixVa, const DeviceOptions &opts
             meta->title=createString(song.title);
             meta->artist=createString(song.artist);
             meta->composer=createString(song.composer());
-            meta->genre=createString(song.genre);
+            meta->genre=createString(song.genres[0]);
             meta->album=createString(song.album);
             meta->date=createString(QString().sprintf("%4d0101T0000.0", song.year));
             meta->filename=createString(destName);
@@ -1008,7 +885,13 @@ void MtpConnection::putSong(const Song &s, bool fixVa, const DeviceOptions &opts
         // Send cover, as a plain file...
         if (copyCover) {
             QString srcFile;
-            Covers::Image coverImage=Covers::self()->getImage(s);
+
+            // If we have transcoded a song, the song.file will point to /tmp!!!
+            Song orig = s;
+            if (orig.hasExtraField(constOrigFileName)) {
+                orig.file=orig.extraField(constOrigFileName);
+            }
+            Covers::Image coverImage=Covers::self()->getImage(orig);
             QTemporaryFile *temp=0;
             if (!coverImage.img.isNull() && !coverImage.fileName.isEmpty()) {
                 if (opts.coverMaxSize && (coverImage.img.width()>(int)opts.coverMaxSize || coverImage.img.height()>(int)opts.coverMaxSize)) {
@@ -1089,11 +972,7 @@ void MtpConnection::getSong(const Song &song, const QString &dest, bool fixVa, b
             File cover=getCoverDetils(song);
 
             if (0!=cover.id) {
-                QString mpdCover=MPDConnection::self()->getDetails().coverName;
-                if (mpdCover.isEmpty()) {
-                    mpdCover="cover";
-                }
-                QString fileName=QString(destDir+mpdCover+(cover.name.endsWith(".jpg", Qt::CaseInsensitive) ? ".jpg" : ".png"));
+                QString fileName=QString(destDir+Covers::albumFileName(song)+(cover.name.endsWith(".jpg", Qt::CaseInsensitive) ? ".jpg" : ".png"));
                 QByteArray fileNameUtf8=fileName.toUtf8();
                 copiedCover=0==LIBMTP_Get_File_To_File(device, cover.id, fileNameUtf8.constData(), 0, 0);
                 if (copiedCover) {
@@ -1158,7 +1037,7 @@ bool MtpConnection::removeFolder(uint32_t folderId)
     if (folderMap.end()!=folder && (*folder).folders.isEmpty() && (*folder).files.isEmpty()) {
         // Delete any cover files...
         QList<uint32_t> toRemove=(*folder).covers.keys();
-        foreach (uint32_t cover, toRemove) {
+        for (uint32_t cover: toRemove) {
             if (0==LIBMTP_Delete_Object(device, cover)) {
                 (*folder).covers.remove(cover);
             }
@@ -1178,7 +1057,7 @@ bool MtpConnection::removeFolder(uint32_t folderId)
 
 void MtpConnection::cleanDirs(const QSet<QString> &dirs)
 {
-    foreach (const QString &d, dirs) {
+    for (const QString &d: dirs) {
         Path path=decodePath(d);
         Storage &store=getStorage(path.storage);
         if (0==store.musicFolderId) {
@@ -1208,11 +1087,14 @@ void MtpConnection::getCover(const Song &song)
 {
     File c=getCoverDetils(song);
 
+    COVER_DBUG << c.name << c.id;
+
     if (0!=c.id) {
         QByteArray data;
         if (0==LIBMTP_Get_File_To_Handler(device, c.id, fileReceiver, &data, 0, 0)) {
             QImage img;
             if (img.loadFromData(data)) {
+                COVER_DBUG << "loaded cover";
                 emit cover(song, img);
             }
         }
@@ -1292,7 +1174,7 @@ QString cfgKey(Solid::Device &dev, const QString &serial)
     return key;
 }
 
-MtpDevice::MtpDevice(MusicModel *m, Solid::Device &dev)
+MtpDevice::MtpDevice(MusicLibraryModel *m, Solid::Device &dev, unsigned int busNum, unsigned int devNum)
     : Device(m, dev,
              #ifdef MTP_FAKE_ALBUMARTIST_SUPPORT
              true
@@ -1311,16 +1193,7 @@ MtpDevice::MtpDevice(MusicModel *m, Solid::Device &dev)
         registeredTypes=true;
     }
 
-    Solid::GenericInterface *iface = dev.as<Solid::GenericInterface>();
-    unsigned int busNum(0);
-    unsigned int devNum(0);
-    if (iface) {
-        QMap<QString, QVariant> properties = iface->allProperties();
-        busNum = properties.value(QLatin1String("BUSNUM")).toInt();
-        devNum = properties.value(QLatin1String("DEVNUM")).toInt();
-    }
-
-    connection=new MtpConnection(busNum, devNum, supportsAlbumArtistTag());
+    connection=new MtpConnection(data(), busNum, devNum, supportsAlbumArtistTag());
     connect(this, SIGNAL(updateLibrary(const DeviceOptions &)), connection, SLOT(updateLibrary(const DeviceOptions &)));
     connect(connection, SIGNAL(libraryUpdated()), this, SLOT(libraryUpdated()));
     connect(connection, SIGNAL(progress(int)), this, SLOT(emitProgress(int)));
@@ -1342,8 +1215,10 @@ MtpDevice::MtpDevice(MusicModel *m, Solid::Device &dev)
     QTimer::singleShot(0, this, SLOT(rescan(bool)));
     defaultName=data();
     if (!opts.name.isEmpty()) {
+        DBUG << "setName" << opts.name;
         setData(opts.name);
     }
+    icn=MonoIcon::icon(FontAwesome::mobilephone, Utils::monoIconColor());
 }
 
 MtpDevice::~MtpDevice()
@@ -1353,12 +1228,14 @@ MtpDevice::~MtpDevice()
 
 void MtpDevice::deviceDetails(const QString &s)
 {
+    DBUG << s;
     if ((s!=serial || serial.isEmpty()) && solidDev.isValid()) {
         serial=s;
         QString configKey=cfgKey(solidDev, serial);
         opts.load(configKey);
         configured=Configuration().hasGroup(configKey);
         if (!opts.name.isEmpty() && opts.name!=defaultName) {
+            DBUG << "setName" << opts.name;
             setData(opts.name);
             emit renamed();
         }
@@ -1451,41 +1328,48 @@ void MtpDevice::addSong(const Song &s, bool overwrite, bool copyCover)
         return;
     }
     currentSong=s;
+
+    Encoders::Encoder encoder;
+
     if (!opts.transcoderCodec.isEmpty()) {
-        Encoders::Encoder encoder=Encoders::getEncoder(opts.transcoderCodec);
+        encoder=Encoders::getEncoder(opts.transcoderCodec);
         if (encoder.codec.isEmpty()) {
             emit actionStatus(CodecNotAvailable);
             return;
         }
+    }
 
-        if (!opts.transcoderWhenDifferent || encoder.isDifferent(s.file)) {
+    transcoding = !opts.transcoderCodec.isEmpty() &&
+            (DeviceOptions::TW_IfDifferent!=opts.transcoderWhen || encoder.isDifferent(s.file)) &&
+            (DeviceOptions::TW_IfLossess!=opts.transcoderWhen || Device::isLossless(s.file));
+
+    if (transcoding) {
+        deleteTemp();
+        tempFile=new QTemporaryFile("cantata_XXXXXX."+encoder.extension);
+        tempFile->setAutoRemove(false);
+
+        if (!tempFile->open()) {
             deleteTemp();
-            tempFile=new QTemporaryFile(QDir::tempPath()+"/cantata_XXXXXX."+encoder.extension);
-            tempFile->setAutoRemove(false);
-
-            if (!tempFile->open()) {
-                deleteTemp();
-                emit actionStatus(FailedToCreateTempFile);
-                return;
-            }
-            QString destFile=tempFile->fileName();
-            tempFile->close();
-            if (QFile::exists(destFile)) {
-                QFile::remove(destFile);
-            }
-            transcoding=true;
-            TranscodingJob *job=new TranscodingJob(encoder, opts.transcoderValue, s.file, destFile);
-            job->setProperty("overwrite", overwrite);
-            job->setProperty("copyCover", copyCover);
-            connect(job, SIGNAL(result(int)), SLOT(transcodeSongResult(int)));
-            connect(job, SIGNAL(percent(int)), SLOT(transcodePercent(int)));
-            job->start();
-            currentSong.file=destFile;
+            emit actionStatus(FailedToCreateTempFile);
             return;
         }
+        QString destFile=tempFile->fileName();
+        tempFile->close();
+        if (QFile::exists(destFile)) {
+            QFile::remove(destFile);
+        }
+        transcoding=true;
+        TranscodingJob *job=new TranscodingJob(encoder, opts.transcoderValue, s.file, destFile);
+        job->setProperty("overwrite", overwrite);
+        job->setProperty("copyCover", copyCover);
+        connect(job, SIGNAL(result(int)), SLOT(transcodeSongResult(int)));
+        connect(job, SIGNAL(percent(int)), SLOT(transcodePercent(int)));
+        job->start();
+        currentSong.setExtraField(constOrigFileName, currentSong.file);
+        currentSong.file=destFile;
+    } else {
+        emit putSong(currentSong, needToFixVa, opts, overwrite, copyCover);
     }
-    transcoding=false;
-    emit putSong(currentSong, needToFixVa, opts, overwrite, copyCover);
 }
 
 void MtpDevice::copySongTo(const Song &s, const QString &musicPath, bool overwrite, bool copyCover)
@@ -1505,7 +1389,7 @@ void MtpDevice::copySongTo(const Song &s, const QString &musicPath, bool overwri
         if (needToFixVa) {
             Device::fixVariousArtists(QString(), check, false);
         }
-        if (MusicLibraryModel::self()->songExists(check)) {
+        if (MpdLibraryModel::self()->songExists(check)) {
             emit actionStatus(SongExists);
             return;
         }
@@ -1555,12 +1439,15 @@ void MtpDevice::cleanDirs(const QSet<QString> &dirs)
     emit cleanMusicDirs(dirs);
 }
 
-void MtpDevice::requestCover(const Song &song)
+Covers::Image MtpDevice::requestCover(const Song &song)
 {
+    COVER_DBUG << song.file;
     requestAbort(false);
     if (isConnected()) {
+        COVER_DBUG << "Get cover from connection";
         emit getCover(song);
     }
+    return Covers::Image();
 }
 
 void MtpDevice::putSongStatus(int status, const QString &file, bool fixedVa, bool copiedCover)
@@ -1634,7 +1521,7 @@ void MtpDevice::getSongStatus(bool ok, bool copiedCover)
     } else {
         currentSong.file=currentDestFile.mid(MPDConnection::self()->getDetails().dir.length());
         QString origPath;
-        if (MPDConnection::self()->isMopdidy()) {
+        if (MPDConnection::self()->isMopidy()) {
             origPath=currentSong.file;
             currentSong.file=Song::encodePath(currentSong.file);
         }
@@ -1642,9 +1529,9 @@ void MtpDevice::getSongStatus(bool ok, bool copiedCover)
             currentSong.revertVariousArtists();
         }
         Utils::setFilePerms(currentDestFile);
-        MusicLibraryModel::self()->addSongToList(currentSong);
-        DirViewModel::self()->addFileToList(origPath.isEmpty() ? currentSong.file : origPath,
-                                            origPath.isEmpty() ? QString() : currentSong.file);
+//        MusicLibraryModel::self()->addSongToList(currentSong);
+//        DirViewModel::self()->addFileToList(origPath.isEmpty() ? currentSong.file : origPath,
+//                                            origPath.isEmpty() ? QString() : currentSong.file);
         emit actionStatus(Ok, copiedCover);
     }
 }
@@ -1679,10 +1566,10 @@ double MtpDevice::usedCapacity()
 QString MtpDevice::capacityString()
 {
     if (!isConnected()) {
-        return i18n("Not Connected");
+        return tr("Not Connected");
     }
 
-    return i18n("%1 free", Utils::formatByteSize(connection->capacity()-connection->usedSpace()));
+    return tr("%1 free").arg(Utils::formatByteSize(connection->capacity()-connection->usedSpace()));
 }
 
 qint64 MtpDevice::freeSpace()
@@ -1715,6 +1602,7 @@ void MtpDevice::saveProperties(const QString &, const DeviceOptions &newOpts)
     bool diffName=opts.name!=newName;
     opts=newOpts;
     if (diffName) {
+        DBUG << "setName" << newName;
         setData(newName);
     }
     saveProperties();
@@ -1747,3 +1635,5 @@ void MtpDevice::deleteTemp()
         tempFile=0;
     }
 }
+
+#include "moc_mtpdevice.cpp"
