@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,13 +25,9 @@
 #define MTPDEVICE_H
 
 #include "fsdevice.h"
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 #include "config.h"
-#ifdef ENABLE_KDE_SUPPORT
-#include <solid/portablemediaplayer.h>
-#else
 #include "solid-lite/portablemediaplayer.h"
-#endif
 #include <libmtp.h>
 
 class MusicLibraryItemRoot;
@@ -72,7 +68,7 @@ public:
         QString musicPath;
     };
 
-    MtpConnection(unsigned int bus, unsigned int dev, bool aaSupport);
+    MtpConnection(const QString &id, unsigned int bus, unsigned int dev, bool aaSupport);
     virtual ~MtpConnection();
 
     bool isConnected() const { return 0!=device; }
@@ -111,13 +107,8 @@ Q_SIGNALS:
 private:
     File getCoverDetils(const Song &s);
     bool removeFolder(uint32_t folderId);
-    #ifdef ENABLE_UNCACHED_MTP
     void updateFilesAndFolders();
     void listFolder(uint32_t storage, uint32_t parentDir, Folder *f=0);
-    #else
-    void updateFolders();
-    void updateFiles();
-    #endif
     void updateStorage();
     Storage & getStorage(const QString &volumeIdentifier);
     Storage & getStorage(uint32_t id);
@@ -125,9 +116,6 @@ private:
     uint32_t getFolder(const QString &path, uint32_t storageId);
     QString getPath(uint32_t folderId);
     uint32_t checkFolderStructure(const QStringList &dirs, Storage &store);
-    #ifndef ENABLE_UNCACHED_MTP
-    void parseFolder(LIBMTP_folder_t *folder);
-    #endif
     void setMusicFolder(Storage &store);
     #ifdef MTP_CLEAN_ALBUMS
     void updateAlbums();
@@ -159,7 +147,7 @@ class MtpDevice : public Device
     Q_OBJECT
 
 public:
-    MtpDevice(MusicModel *m, Solid::Device &dev);
+    MtpDevice(MusicLibraryModel *m, Solid::Device &dev, unsigned int busNum, unsigned int devNum);
     virtual ~MtpDevice();
 
     bool isConnected() const;
@@ -171,7 +159,7 @@ public:
     void copySongTo(const Song &s, const QString &musicPath, bool overwrite, bool copyCover);
     void removeSong(const Song &s);
     void cleanDirs(const QSet<QString> &dirs);
-    void requestCover(const Song &song);
+    Covers::Image requestCover(const Song &song);
     double usedCapacity();
     QString capacityString();
     qint64 freeSpace();
