@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -44,7 +44,6 @@
 #include <QRegExp>
 #include "config.h"
 #include "support/thread.h"
-#include "support/localize.h"
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
@@ -77,6 +76,10 @@ struct Track {
 
 static QString calculateDiscId(const QList<Track> &tracks)
 {
+    if (tracks.isEmpty()) {
+        return QString();
+    }
+
     // Code based on libmusicbrainz/lib/diskid.cpp
     int numTracks = tracks.count()-1;
     QCryptographicHash sha(QCryptographicHash::Sha1);
@@ -88,7 +91,7 @@ static QString calculateDiscId(const QList<Track> &tracks)
     sha.addData(temp, strlen(temp));
 
     for(int i = 0; i < 100; i++) {
-        long offset;
+        int offset;
         if (0==i) {
             offset = tracks[numTracks].offset;
         } else if (i <= numTracks) {
@@ -97,7 +100,7 @@ static QString calculateDiscId(const QList<Track> &tracks)
             offset = 0;
         }
 
-        sprintf(temp, "%08lX", offset);
+        sprintf(temp, "%08X", offset);
         sha.addData(temp, strlen(temp));
     }
 
@@ -147,7 +150,7 @@ void MusicBrainz::readDisc()
 {
     int fd=open(dev.toLocal8Bit(), O_RDONLY | O_NONBLOCK);
     if (fd < 0) {
-        emit error(i18n("Failed to open CD device"));
+        emit error(tr("Failed to open CD device"));
         return;
     }
     QList<Track> tracks;
@@ -189,7 +192,7 @@ void MusicBrainz::readDisc()
         }
         te.cdte_track = CDROM_LEADOUT;
         if (0==ioctl(fd, CDROMREADTOCENTRY, &te)) {
-            tracks.append((te.cdte_addr.lba+secondsToFrames(2))/secondsToFrames(1));
+            tracks.append((te.cdte_addr.lba+secondsToFrames(2)));
         }
     }
     #endif
@@ -209,11 +212,12 @@ void MusicBrainz::readDisc()
             const Track &next=tracks.at(i+1);
             Song s;
             s.track=i+1;
-            s.title=i18n("Track %1", s.track).toUtf8();
+            s.title=tr("Track %1").arg(s.track).toUtf8();
             s.artist=Song::unknown();
             s.albumartist=initial.artist;
             s.album=initial.name;
             s.id=s.track;
+
             s.time=framesToSeconds((next.offset-trk.offset)-(next.isData ? constDataTrackAdjust : 0));
             s.file=QString("%1.wav").arg(s.track);
             s.year=initial.year;
@@ -244,10 +248,10 @@ void MusicBrainz::lookup(bool full)
     }
     DBUG << "Should lookup " << discId;
 
-    MusicBrainz5::CQuery Query("cantata-"PACKAGE_VERSION_STRING);
+    MusicBrainz5::CQuery Query("cantata-" PACKAGE_VERSION_STRING);
     QList<CdAlbum> m;
     QList<QNetworkProxy> proxies=NetworkProxyFactory::self()->queryProxy(QNetworkProxyQuery(QUrl("http://musicbrainz.org")));
-    foreach (const QNetworkProxy &p, proxies) {
+    for (const QNetworkProxy &p: proxies) {
         if (QNetworkProxy::HttpProxy==p.type() && 0!=p.port()) {
             Query.SetProxyHost(p.hostName().toLatin1().constData());
             Query.SetProxyPort(p.port());
@@ -258,11 +262,7 @@ void MusicBrainz::lookup(bool full)
     // Code adapted from libmusicbrainz/examples/cdlookup.cc
 
     try {
-        #if QT_VERSION < 0x050000
-        MusicBrainz5::CMetadata Metadata=Query.Query("discid", discId.toAscii().constData());
-        #else
         MusicBrainz5::CMetadata Metadata=Query.Query("discid", discId.toLatin1().constData());
-        #endif
 
         if (Metadata.Disc() && Metadata.Disc()->ReleaseList()) {
             MusicBrainz5::CReleaseList *releaseList=Metadata.Disc()->ReleaseList();
@@ -284,12 +284,7 @@ void MusicBrainz::lookup(bool full)
 
                     //However, these releases will include information for all media in the release
                     //So we need to filter out the only the media we want.
-
-                    #if QT_VERSION < 0x050000
-                    MusicBrainz5::CMediumList mediaList=fullRelease->MediaMatchingDiscID(discId.toAscii().constData());
-                    #else
                     MusicBrainz5::CMediumList mediaList=fullRelease->MediaMatchingDiscID(discId.toLatin1().constData());
-                    #endif
 
                     if (mediaList.NumItems() > 0) {
                         DBUG << "Found " << mediaList.NumItems() << " media item(s)";
@@ -303,7 +298,7 @@ void MusicBrainz::lookup(bool full)
                             album.name=QString::fromUtf8(fullRelease->Title().c_str());
 
                             if (fullRelease->MediumList()->NumItems() > 1) {
-                                album.name = i18n("%1 (Disc %2)", album.name, medium->Position());
+                                album.name = tr("%1 (Disc %2)").arg(album.name).arg(medium->Position());
                                 album.disc=medium->Position();
                             }
                             album.artist=artistFromCreditList(fullRelease->ArtistCredit());
@@ -333,7 +328,7 @@ void MusicBrainz::lookup(bool full)
 
                                     song.albumartist=album.artist;
                                     song.album=album.name;
-                                    song.genre=album.genre;
+                                    song.genres[0]=album.genre;
                                     song.id=song.track=track->Position();
                                     song.time=track->Length()/1000;
                                     song.disc=album.disc;
@@ -369,29 +364,23 @@ void MusicBrainz::lookup(bool full)
                 }
             }
         }
-    }
-    catch (MusicBrainz5::CConnectionError &e) {
+    } catch (MusicBrainz5::CConnectionError &e) {
         DBUG << "MusicBrainz error" << e.what();
-    }
-    catch (MusicBrainz5::CTimeoutError &e) {
+    } catch (MusicBrainz5::CTimeoutError &e) {
         DBUG << "MusicBrainz error - %1" << e.what();
-    }
-    catch (MusicBrainz5::CAuthenticationError &e) {
+    } catch (MusicBrainz5::CAuthenticationError &e) {
         DBUG << "MusicBrainz error - %1" << e.what();
-    }
-    catch (MusicBrainz5::CFetchError &e) {
+    } catch (MusicBrainz5::CFetchError &e) {
         DBUG << "MusicBrainz error - %1" << e.what();
-    }
-    catch (MusicBrainz5::CRequestError &e) {
+    } catch (MusicBrainz5::CRequestError &e) {
         DBUG << "MusicBrainz error - %1" << e.what();
-    }
-    catch (MusicBrainz5::CResourceNotFoundError &e) {
+    } catch (MusicBrainz5::CResourceNotFoundError &e) {
         DBUG << "MusicBrainz error - %1" << e.what();
     }
 
     if (m.isEmpty()) {
         if (!isInitial) {
-            emit error(i18n("No matches found in MusicBrainz"));
+            emit error(tr("No matches found in MusicBrainz"));
         }
     } else if (isInitial) {
         emit initialDetails(m.first());
@@ -399,3 +388,5 @@ void MusicBrainz::lookup(bool full)
         emit matches(m);
     }
 }
+
+#include "moc_musicbrainz.cpp"
