@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,34 +26,24 @@
 #include "remotedevicepropertiesdialog.h"
 #include "devicepropertieswidget.h"
 #include "actiondialog.h"
-#include "inputdialog.h"
-#include "utils.h"
-#include "httpserver.h"
-#include "localize.h"
-#include "settings.h"
+#include "support/inputdialog.h"
+#include "support/utils.h"
+#include "support/monoicon.h"
+#include "http/httpserver.h"
+#include "support/configuration.h"
 #include "mountpoints.h"
-#include "mounterinterface.h"
-#include "avahi/avahi.h"
-#include "avahi/avahiservice.h"
 #include <QDBusConnection>
 #include <QTimer>
 #include <QProcess>
+#include <QProcessEnvironment>
 #include <QDir>
 #include <QFile>
-#ifdef ENABLE_KDE_SUPPORT
-#include <kmountpoint.h>
-#include <kde_file.h>
-#endif
 #include <stdio.h>
 #include <unistd.h>
 
 const QLatin1String RemoteFsDevice::constPromptPassword("-");
 const QLatin1String RemoteFsDevice::constSshfsProtocol("sshfs");
 const QLatin1String RemoteFsDevice::constFileProtocol("file");
-const QLatin1String RemoteFsDevice::constDomainQuery("domain");
-const QLatin1String RemoteFsDevice::constSambaProtocol("smb");
-const QLatin1String RemoteFsDevice::constServiceNameQuery("name");
-const QLatin1String RemoteFsDevice::constSambaAvahiProtocol("smb-avahi");
 
 static const QLatin1String constCfgPrefix("RemoteFsDevice-");
 static const QLatin1String constCfgKey("remoteFsDevices");
@@ -68,96 +58,73 @@ static QString mountPoint(const RemoteFsDevice::Details &details, bool create)
 
 void RemoteFsDevice::Details::load(const QString &group)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    KConfigGroup cfg(KGlobal::config(), constCfgPrefix+group);
-    #else
-    QSettings cfg;
-    cfg.beginGroup(constCfgPrefix+group);
-    #endif
+    Configuration cfg(constCfgPrefix+group);
     name=group;
-    QString u=GET_STRING("url", QString());
+    QString u=cfg.get("url", QString());
 
     if (u.isEmpty()) {
-        QString protocol=GET_STRING("protocol", QString());
-        QString host=GET_STRING("host", QString());
-        QString user=GET_STRING("user", QString());
-        QString path=GET_STRING("path", QString());
-        int port=GET_INT("port", 0);
+        QString protocol=cfg.get("protocol", QString());
+        QString host=cfg.get("host", QString());
+        QString user=cfg.get("user", QString());
+        QString path=cfg.get("path", QString());
+        int port=cfg.get("port", 0);
         u=(protocol.isEmpty() ? "file" : protocol+"://")+user+(!user.isEmpty() && !host.isEmpty() ? "@" : "")+host+(0==port ? "" : (":"+QString::number(port)))+path;
     }
     if (!u.isEmpty()) {
         url=QUrl(u);
     }
-    extraOptions=GET_STRING("extraOptions", QString());
-    configured=GET_BOOL("configured", configured);
+    extraOptions=cfg.get("extraOptions", QString());
+    configured=cfg.get("configured", configured);
 }
 
 void RemoteFsDevice::Details::save() const
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    KConfigGroup cfg(KGlobal::config(), constCfgPrefix+name);
-    #else
-    QSettings cfg;
-    cfg.beginGroup(constCfgPrefix+name);
-    #endif
-    SET_VALUE("url", url.toString());
-    SET_VALUE("extraOptions", extraOptions);
-    SET_VALUE("configured", configured);
-    CFG_SYNC;
+    Configuration cfg(constCfgPrefix+name);
+
+    cfg.set("url", url.toString());
+    cfg.set("extraOptions", extraOptions);
+    cfg.set("configured", configured);
 }
 
 static inline bool isValid(const RemoteFsDevice::Details &d)
 {
-    return d.isLocalFile() || RemoteFsDevice::constSshfsProtocol==d.url.scheme() ||
-           RemoteFsDevice::constSambaProtocol==d.url.scheme() || RemoteFsDevice::constSambaAvahiProtocol==d.url.scheme();
+    return d.isLocalFile() || RemoteFsDevice::constSshfsProtocol==d.url.scheme();
 }
 
 static inline bool isMountable(const RemoteFsDevice::Details &d)
 {
-    return RemoteFsDevice::constSshfsProtocol==d.url.scheme() ||
-           RemoteFsDevice::constSambaProtocol==d.url.scheme() || RemoteFsDevice::constSambaAvahiProtocol==d.url.scheme();
+    return RemoteFsDevice::constSshfsProtocol==d.url.scheme();
 }
 
-QList<Device *> RemoteFsDevice::loadAll(MusicModel *m)
+QList<Device *> RemoteFsDevice::loadAll(MusicLibraryModel *m)
 {
     QList<Device *> devices;
-    #ifdef ENABLE_KDE_SUPPORT
-    KConfigGroup cfg(KGlobal::config(), "General");
-    #else
-    QSettings cfg;
-    #endif
-    QStringList names=GET_STRINGLIST(constCfgKey, QStringList());
-    foreach (const QString &n, names) {
+    Configuration cfg;
+    QStringList names=cfg.get(constCfgKey, QStringList());
+    for (const QString &n: names) {
         Details d;
         d.load(n);
         if (d.isEmpty()) {
-            REMOVE_GROUP(constCfgPrefix+n);
+            cfg.removeGroup(constCfgPrefix+n);
         } else if (isValid(d)) {
             devices.append(new RemoteFsDevice(m, d));
         }
     }
-    if (devices.count()!=names.count()) {
-        CFG_SYNC;
-    }
     return devices;
 }
 
-Device * RemoteFsDevice::create(MusicModel *m, const DeviceOptions &options, const Details &d)
+Device * RemoteFsDevice::create(MusicLibraryModel *m, const DeviceOptions &options, const Details &d)
 {
     if (d.isEmpty()) {
         return 0;
     }
-    #ifdef ENABLE_KDE_SUPPORT
-    KConfigGroup cfg(KGlobal::config(), "General");
-    #else
-    QSettings cfg;
-    #endif
-    QStringList names=GET_STRINGLIST(constCfgKey, QStringList());
+    Configuration cfg;
+    QStringList names=cfg.get(constCfgKey, QStringList());
     if (names.contains(d.name)) {
         return 0;
     }
     names.append(d.name);
-    SET_VALUE(constCfgKey, names);
+    cfg.set(constCfgKey, names);
     d.save();
     if (isValid(d)) {
         return new RemoteFsDevice(m, options, d);
@@ -168,17 +135,12 @@ Device * RemoteFsDevice::create(MusicModel *m, const DeviceOptions &options, con
 void RemoteFsDevice::destroy(bool removeFromConfig)
 {
     if (removeFromConfig) {
-        #ifdef ENABLE_KDE_SUPPORT
-        KConfigGroup cfg(KGlobal::config(), "General");
-        #else
-        QSettings cfg;
-        #endif
-        QStringList names=GET_STRINGLIST(constCfgKey, QStringList());
+        Configuration cfg;
+        QStringList names=cfg.get(constCfgKey, QStringList());
         if (names.contains(details.name)) {
             names.removeAll(details.name);
-            REMOVE_GROUP(id());
-            SET_VALUE(constCfgKey, names);
-            CFG_SYNC;
+            cfg.removeGroup(id());
+            cfg.set(constCfgKey, names);
         }
     }
     stopScanner();
@@ -197,22 +159,6 @@ void RemoteFsDevice::destroy(bool removeFromConfig)
     deleteLater();
 }
 
-void RemoteFsDevice::serviceAdded(const QString &name)
-{
-    if (name==details.serviceName && constSambaAvahiProtocol==details.url.scheme()) {
-        sub=i18n("Available");
-        updateStatus();
-    }
-}
-
-void RemoteFsDevice::serviceRemoved(const QString &name)
-{
-    if (name==details.serviceName && constSambaAvahiProtocol==details.url.scheme()) {
-        sub=i18n("Not Available");
-        updateStatus();
-    }
-}
-
 QString RemoteFsDevice::createUdi(const QString &n)
 {
     return constCfgPrefix+n;
@@ -220,51 +166,52 @@ QString RemoteFsDevice::createUdi(const QString &n)
 
 void RemoteFsDevice::renamed(const QString &oldName, const QString &newName)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    KConfigGroup cfg(KGlobal::config(), "General");
-    #else
-    QSettings cfg;
-    #endif
-    QStringList names=GET_STRINGLIST(constCfgKey, QStringList());
+    Configuration cfg;
+    QStringList names=cfg.get(constCfgKey, QStringList());
     if (names.contains(oldName)) {
         names.removeAll(oldName);
-        REMOVE_GROUP(createUdi(oldName));
+        cfg.removeGroup(createUdi(oldName));
     }
     if (!names.contains(newName)) {
         names.append(newName);
     }
-    SET_VALUE(constCfgKey, names);
-    CFG_SYNC;
+    cfg.set(constCfgKey, names);
 }
 
-RemoteFsDevice::RemoteFsDevice(MusicModel *m, const DeviceOptions &options, const Details &d)
+RemoteFsDevice::RemoteFsDevice(MusicLibraryModel *m, const DeviceOptions &options, const Details &d)
     : FsDevice(m, d.name, createUdi(d.name))
     , mountToken(0)
     , currentMountStatus(false)
     , details(d)
     , proc(0)
-    , mounterIface(0)
     , messageSent(false)
 {
     opts=options;
 //    details.path=Utils::fixPath(details.path);
     load();
     mount();
-    icn=Icon(details.isLocalFile() ? "inode-directory" : (constSshfsProtocol==details.url.scheme() ? "utilities-terminal" : "network-server"));
+    icn=MonoIcon::icon(details.isLocalFile()
+                       ? FontAwesome::foldero
+                       : constSshfsProtocol==details.url.scheme()
+                         ? FontAwesome::linux_os
+                         : FontAwesome::windows, Utils::monoIconColor());
 }
 
-RemoteFsDevice::RemoteFsDevice(MusicModel *m, const Details &d)
+RemoteFsDevice::RemoteFsDevice(MusicLibraryModel *m, const Details &d)
     : FsDevice(m, d.name, createUdi(d.name))
     , mountToken(0)
     , currentMountStatus(false)
     , details(d)
     , proc(0)
-    , mounterIface(0)
     , messageSent(false)
 {
 //    details.path=Utils::fixPath(details.path);
     setup();
-    icn=Icon(details.isLocalFile() ? "inode-directory" : (constSshfsProtocol==details.url.scheme() ? "utilities-terminal" : "network-server"));
+    icn=MonoIcon::icon(details.isLocalFile()
+                       ? FontAwesome::foldero
+                       : constSshfsProtocol==details.url.scheme()
+                         ? FontAwesome::linux_os
+                         : FontAwesome::windows, Utils::monoIconColor());
 }
 
 RemoteFsDevice::~RemoteFsDevice() {
@@ -280,20 +227,6 @@ void RemoteFsDevice::toggle()
     }
 }
 
-ComGooglecodeCantataMounterInterface * RemoteFsDevice::mounter()
-{
-    if (!mounterIface) {
-        if (!QDBusConnection::systemBus().interface()->isServiceRegistered(ComGooglecodeCantataMounterInterface::staticInterfaceName())) {
-            QDBusConnection::systemBus().interface()->startService(ComGooglecodeCantataMounterInterface::staticInterfaceName());
-        }
-        mounterIface=new ComGooglecodeCantataMounterInterface(ComGooglecodeCantataMounterInterface::staticInterfaceName(),
-                                                              "/Mounter", QDBusConnection::systemBus(), this);
-        connect(mounterIface, SIGNAL(mountStatus(const QString &, int, int)), SLOT(mountStatus(const QString &, int, int)));
-        connect(mounterIface, SIGNAL(umountStatus(const QString &, int, int)), SLOT(umountStatus(const QString &, int, int)));
-    }
-    return mounterIface;
-}
-
 void RemoteFsDevice::mount()
 {
     if (details.isLocalFile()) {
@@ -306,56 +239,15 @@ void RemoteFsDevice::mount()
     if (messageSent) {
         return;
     }
-    if (constSambaAvahiProtocol==details.url.scheme()) {
-        Details det=details;
-        AvahiService *srv=Avahi::self()->getService(det.serviceName);
-        if (!srv || srv->getHost().isEmpty() || 0==srv->getPort()) {
-            emit error(i18n("Failed to resolve connection details for %1", details.name));
-            return;
-        }
-        if (constPromptPassword==det.url.password()) {
-            bool ok=false;
-            QString passwd=InputDialog::getPassword(QString(), &ok, QApplication::activeWindow());
-            if (!ok) {
-                return;
-            }
-            det.url.setPassword(passwd);
-        }
-        det.url.setScheme(constSambaProtocol);
-        det.url.setHost(srv->getHost());
-        det.url.setPort(srv->getPort());
-        mounter()->mount(det.url.toString(), mountPoint(details, true), getuid(), getgid(), getpid());
-        setStatusMessage(i18n("Connecting..."));
-        messageSent=true;
-        return;
-    }
-    if (constSambaProtocol==details.url.scheme()) {
-        Details det=details;
-        if (constPromptPassword==det.url.password()) {
-            bool ok=false;
-            QString passwd=InputDialog::getPassword(QString(), &ok, QApplication::activeWindow());
-            if (!ok) {
-                return;
-            }
-            det.url.setPassword(passwd);
-        }
-        mounter()->mount(det.url.toString(), mountPoint(details, true), getuid(), getgid(), getpid());
-        setStatusMessage(i18n("Connecting..."));
-        messageSent=true;
-        return;
-    }
 
     QString cmd;
     QStringList args;
+    QString askPass;
     if (!details.isLocalFile() && !details.isEmpty()) {
         // If user has added 'IdentityFile' to extra options, then no password prompting is required...
         bool needAskPass=!details.extraOptions.contains("IdentityFile=");
 
         if (needAskPass) {
-            if (ttyname(0)) {
-                emit error(i18n("Password prompting does not work when cantata is started from the commandline."));
-                return;
-            }
             QStringList askPassList;
             if (Utils::KDE==Utils::currentDe()) {
                 askPassList << QLatin1String("ksshaskpass") << QLatin1String("ssh-askpass") << QLatin1String("ssh-askpass-gnome");
@@ -363,8 +255,7 @@ void RemoteFsDevice::mount()
                 askPassList << QLatin1String("ssh-askpass-gnome") << QLatin1String("ssh-askpass") << QLatin1String("ksshaskpass");
             }
 
-            QString askPass;
-            foreach (const QString &ap, askPassList) {
+            for (const QString &ap: askPassList) {
                 askPass=Utils::findExe(ap);
                 if (!askPass.isEmpty()) {
                     break;
@@ -372,18 +263,27 @@ void RemoteFsDevice::mount()
             }
 
             if (askPass.isEmpty()) {
-                emit error(i18n("No suitable ssh-askpass application installed! This is required for entering passwords."));
+                emit error(tr("No suitable ssh-askpass application installed! This is required for entering passwords."));
                 return;
             }
         }
-        cmd=Utils::findExe("sshfs");
+        QString sshfs=Utils::findExe("sshfs");
+        if (sshfs.isEmpty()) {
+            emit error(tr("\"sshfs\" is not installed!"));
+            return;
+        }
+        cmd=Utils::findExe("setsid");
         if (!cmd.isEmpty()) {
-            if (!QDir(mountPoint(details, true)).entryList(QDir::NoDot|QDir::NoDotDot|QDir::AllEntries|QDir::Hidden).isEmpty()) {
-                emit error(i18n("Mount point (\"%1\") is not empty!", mountPoint(details, true)));
+            QString mp=mountPoint(details, true);
+            if (mp.isEmpty()) {
+                emit error("Failed to determine mount point"); // TODO: 2.4 make translatable. For now, error should never happen!
+            }
+            if (!QDir(mp).entryList(QDir::NoDot|QDir::NoDotDot|QDir::AllEntries|QDir::Hidden).isEmpty()) {
+                emit error(tr("Mount point (\"%1\") is not empty!").arg(mp));
                 return;
             }
 
-            args << details.url.userName()+QChar('@')+details.url.host()+QChar(':')+details.url.path()<< QLatin1String("-p")
+            args << sshfs << details.url.userName()+QChar('@')+details.url.host()+QChar(':')+details.url.path()<< QLatin1String("-p")
                  << QString::number(details.url.port()) << mountPoint(details, true)
                  << QLatin1String("-o") << QLatin1String("ServerAliveInterval=15");
             //<< QLatin1String("-o") << QLatin1String("Ciphers=arcfour");
@@ -391,14 +291,20 @@ void RemoteFsDevice::mount()
                 args << details.extraOptions.split(' ', QString::SkipEmptyParts);
             }
         } else {
-            emit error(i18n("\"sshfs\" is not installed!"));
+            emit error(tr("\"sshfs\" is not installed!").replace("sshfs", "setsid")); // TODO: 2.4 use correct string!
         }
     }
 
     if (!cmd.isEmpty()) {
-        setStatusMessage(i18n("Connecting..."));
+        setStatusMessage(tr("Connecting..."));
         proc=new QProcess(this);
         proc->setProperty("mount", true);
+
+        if (!askPass.isEmpty()) {
+            QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
+            env.insert("SSH_ASKPASS", askPass);
+            proc->setProcessEnvironment(env);
+        }
         connect(proc, SIGNAL(finished(int)), SLOT(procFinished(int)));
         proc->start(cmd, args, QIODevice::ReadOnly);
     }
@@ -417,12 +323,6 @@ void RemoteFsDevice::unmount()
     if (messageSent) {
         return;
     }
-    if (constSambaProtocol==details.url.scheme() || constSambaAvahiProtocol==details.url.scheme()) {
-        mounter()->umount(mountPoint(details, false), getpid());
-        setStatusMessage(i18n("Disconnecting..."));
-        messageSent=true;
-        return;
-    }
 
     QString cmd;
     QStringList args;
@@ -433,13 +333,13 @@ void RemoteFsDevice::unmount()
             if (!cmd.isEmpty()) {
                 args << QLatin1String("-u") << QLatin1String("-z") << mp;
             } else {
-                emit error(i18n("\"fusermount\" is not installed!"));
+                emit error(tr("\"fusermount\" is not installed!"));
             }
         }
     }
 
     if (!cmd.isEmpty()) {
-        setStatusMessage(i18n("Disconnecting..."));
+        setStatusMessage(tr("Disconnecting..."));
         proc=new QProcess(this);
         proc->setProperty("unmount", true);
         connect(proc, SIGNAL(finished(int)), SLOT(procFinished(int)));
@@ -454,11 +354,11 @@ void RemoteFsDevice::procFinished(int exitCode)
     proc=0;
 
     if (0!=exitCode) {
-        emit error(wasMount ? i18n("Failed to connect to \"%1\"", details.name)
-                            : i18n("Failed to disconnect from \"%1\"", details.name));
+        emit error(wasMount ? tr("Failed to connect to \"%1\"").arg(details.name)
+                            : tr("Failed to disconnect from \"%1\"").arg(details.name));
         setStatusMessage(QString());
     } else if (wasMount) {
-        setStatusMessage(i18n("Updating tracks..."));
+        setStatusMessage(tr("Updating tracks..."));
         load();
         emit connectionStateHasChanged(id(), true);
     } else {
@@ -475,10 +375,10 @@ void RemoteFsDevice::mountStatus(const QString &mp, int pid, int st)
     if (pid==getpid() && mp==mountPoint(details, false)) {
         messageSent=false;
         if (0!=st) {
-            emit error(i18n("Failed to connect to \"%1\"", details.name));
+            emit error(tr("Failed to connect to \"%1\"").arg(details.name));
             setStatusMessage(QString());
         } else {
-            setStatusMessage(i18n("Updating tracks..."));
+            setStatusMessage(tr("Updating tracks..."));
             load();
             emit connectionStateHasChanged(id(), true);
         }
@@ -490,7 +390,7 @@ void RemoteFsDevice::umountStatus(const QString &mp, int pid, int st)
     if (pid==getpid() && mp==mountPoint(details, false)) {
         messageSent=false;
         if (0!=st) {
-            emit error(i18n("Failed to disconnect from \"%1\"", details.name));
+            emit error(tr("Failed to disconnect from \"%1\"").arg(details.name));
             setStatusMessage(QString());
         } else {
             setStatusMessage(QString());
@@ -561,14 +461,14 @@ QString RemoteFsDevice::capacityString()
         return statusMessage();
     }
     if (!isConnected()) {
-        return i18n("Not Connected");
+        return tr("Not Connected");
     }
 
     spaceInfo.setPath(mountPoint(details, false));
     if (isOldSshfs()) {
-        return i18n("Capacity Unknown");
+        return tr("Capacity Unknown");
     }
-    return i18n("%1 free", Utils::formatByteSize(spaceInfo.size()-spaceInfo.used()));
+    return tr("%1 free").arg(Utils::formatByteSize(spaceInfo.size()-spaceInfo.used()));
 }
 
 qint64 RemoteFsDevice::freeSpace()
@@ -586,32 +486,6 @@ qint64 RemoteFsDevice::freeSpace()
 
 void RemoteFsDevice::load()
 {
-    if (RemoteFsDevice::constSambaAvahiProtocol==details.url.scheme()) {
-        // Start Avahi listener...
-        Avahi::self();
-        #if QT_VERSION < 0x050000
-        if (details.url.hasQueryItem(constServiceNameQuery)) {
-            details.serviceName=details.url.queryItemValue(constServiceNameQuery);
-        }
-        #else // QT_VERSION
-        QUrlQuery q(details.url);
-        if (q.hasQueryItem(constServiceNameQuery)) {
-            details.serviceName=q.queryItemValue(constServiceNameQuery);
-        }
-        #endif // QT_VERSION
-
-        if (!details.serviceName.isEmpty()) {
-            AvahiService *srv=Avahi::self()->getService(details.serviceName);
-            if (!srv || srv->getHost().isEmpty()) {
-                sub=i18n("Not Available");
-            } else {
-                sub=i18n("Available");
-            }
-        }
-        connect(Avahi::self(), SIGNAL(serviceAdded(QString)), SLOT(serviceAdded(QString)));
-        connect(Avahi::self(), SIGNAL(serviceRemoved(QString)), SLOT(serviceRemoved(QString)));
-    }
-
     if (isConnected()) {
         setAudioFolder();
         readOpts(settingsFileName(), opts, true);
@@ -647,8 +521,8 @@ void RemoteFsDevice::configure(QWidget *parent)
         connect(dlg, SIGNAL(cancelled()), SLOT(saveProperties()));
     }
     dlg->show(opts, details,
-              DevicePropertiesWidget::Prop_All-(DevicePropertiesWidget::Prop_Folder+DevicePropertiesWidget::Prop_AutoScan),
-              false, isConnected());
+              DevicePropertiesWidget::Prop_All-(DevicePropertiesWidget::Prop_Name+DevicePropertiesWidget::Prop_Folder+DevicePropertiesWidget::Prop_AutoScan),
+              0, false, isConnected());
 }
 
 bool RemoteFsDevice::canPlaySongs() const
@@ -736,3 +610,5 @@ QString RemoteFsDevice::settingsFileName() const
     }
     return audioFolder+constCantataSettingsFile;
 }
+
+#include "moc_remotefsdevice.cpp"
