@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,20 +23,22 @@
 
 #include "application_mac.h"
 #include "support/utils.h"
-#include <QIcon>
+#include "config.h"
+#include <stdlib.h>
+#include <QDir>
 
 Application::Application(int &argc, char **argv)
     : SingleApplication(argc, argv)
 {
-    QCoreApplication::setAttribute(Qt::AA_DontShowIconsInMenus, true);
+    setAttribute(Qt::AA_DontShowIconsInMenus, true);
 
-    // Setup icon path...
-    QStringList paths=QIcon::themeSearchPaths();
-    QString path=Utils::systemDir("icons");
-    if (!paths.contains(path)) {
-        QIcon::setThemeSearchPaths(QStringList() << path << paths);
+    // Set DYLD_LIBRARY_PATH so that Qt finds our openSSL libs
+    QDir dir(argv[0]);
+    dir.cdUp();
+
+    QByteArray ldPath = qgetenv("DYLD_LIBRARY_PATH");
+    if (!ldPath.isEmpty()) {
+        ldPath=':'+ldPath;
     }
-
-    QIcon::setThemeName(QLatin1String("oxygen"));
+    ldPath = dir.absolutePath().toLocal8Bit()+ldPath;
 }
-
