@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,19 +22,16 @@
  */
 
 #include "application_win.h"
-#include <QIcon>
+#include "config.h"
 #include <windows.h>
 
 Application::Application(int &argc, char **argv)
     : SingleApplication(argc, argv)
 {
-    #if QT_VERSION >= 0x050000
     installNativeEventFilter(this);
-    #endif
-    QIcon::setThemeName(QLatin1String("oxygen"));
+    setAttribute(Qt::AA_DontShowIconsInMenus, true);
 }
 
-#if QT_VERSION >= 0x050000
 bool Application::nativeEventFilter(const QByteArray &, void *message, long *result)
 {
     MSG *msg = static_cast<MSG *>(message);
@@ -43,13 +40,4 @@ bool Application::nativeEventFilter(const QByteArray &, void *message, long *res
     }
     return false;
 }
-#else
-bool Application::winEventFilter(MSG *msg, long *result)
-{
-    if (msg && WM_POWERBROADCAST==msg->message && PBT_APMRESUMEAUTOMATIC==msg->wParam) {
-        emit reconnect();
-    }
-    return QCoreApplication::winEventFilter(msg, result);
-}
-#endif
 
