@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,23 +25,16 @@
 #define APPLICATION_WIN_H
 
 #include "singleapplication.h"
-
-#if QT_VERSION >= 0x050000
 #include <QAbstractNativeEventFilter>
+
 class Application : public SingleApplication, public QAbstractNativeEventFilter
-#else
-class Application : public SingleApplication
-#endif
 {
 public:
-    static void initObjects();
+    static void init();
+    static void fixSize(QWidget *widget);
     Application(int &argc, char **argv);
     virtual ~Application() { }
-    #if QT_VERSION >= 0x050000
     bool nativeEventFilter(const QByteArray &, void *message, long *result);
-    #else
-    bool winEventFilter(MSG *msg, long *result);
-    #endif
 };
 
 #endif
