@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,26 +22,20 @@
  */
 
 #include "cachesettings.h"
-#include "support/localize.h"
 #include "context/artistview.h"
 #include "context/albumview.h"
 #include "context/songview.h"
 #include "context/contextwidget.h"
 #include "context/wikipediasettings.h"
 #include "covers.h"
-#include "models/musiclibrarymodel.h"
 #include "support/utils.h"
 #include "support/messagebox.h"
 #include "config.h"
 #include "support/thread.h"
 #include "settings.h"
 #include "widgets/basicitemdelegate.h"
-#ifdef ENABLE_STREAMS
 #include "models/streamsmodel.h"
-#endif
-#ifdef ENABLE_ONLINE_SERVICES
-#include "online//podcastsearchdialog.h"
-#endif
+#include "online/podcastsearchdialog.h"
 #include "support/squeezedtextlabel.h"
 #include "scrobbling/scrobbler.h"
 #include <QLabel>
@@ -61,13 +55,13 @@ static void calculate(const QString &d, const QStringList &types, int &items, qu
         QDir dir(d);
         if (dir.exists()) {
             QFileInfoList files=dir.entryInfoList(types, QDir::Files|QDir::NoDotAndDotDot);
-            foreach (const QFileInfo &file, files) {
+            for (const QFileInfo &file: files) {
                 space+=file.size();
             }
             items+=files.count();
 
             QFileInfoList dirs=dir.entryInfoList(QDir::Dirs|QDir::NoDotAndDotDot);
-            foreach (const QFileInfo &subDir, dirs) {
+            for (const QFileInfo &subDir: dirs) {
                 calculate(subDir.absoluteFilePath(), types, items, space, level+1);
             }
         }
@@ -80,12 +74,12 @@ static void deleteAll(const QString &d, const QStringList &types, int level=0)
         QDir dir(d);
         if (dir.exists()) {
             QFileInfoList dirs=dir.entryInfoList(QDir::Dirs|QDir::NoDotAndDotDot);
-            foreach (const QFileInfo &subDir, dirs) {
+            for (const QFileInfo &subDir: dirs) {
                 deleteAll(subDir.absoluteFilePath(), types, level+1);
             }
 
             QFileInfoList files=dir.entryInfoList(types, QDir::Files|QDir::NoDotAndDotDot);
-            foreach (const QFileInfo &file, files) {
+            for (const QFileInfo &file: files) {
                 QFile::remove(file.absoluteFilePath());
             }
             if (0!=level) {
@@ -172,7 +166,7 @@ void CacheItem::setStatus(const QString &str)
 
 void CacheItem::clean()
 {
-    setStatus(i18n("Deleting..."));
+    setStatus(tr("Deleting..."));
     emit deleteAll();
     switch (type) {
     case Type_Covers:       Covers::self()->clearNameCache(); break;
@@ -183,24 +177,20 @@ void CacheItem::clean()
 
 void CacheItem::calculate()
 {
-    setStatus(i18n("Calculating..."));
+    setStatus(tr("Calculating..."));
     emit getCount();
 }
 
 static inline void setResizeMode(QHeaderView *hdr, int idx, QHeaderView::ResizeMode mode)
 {
-    #if QT_VERSION < 0x050000
-    hdr->setResizeMode(idx, mode);
-    #else
     hdr->setSectionResizeMode(idx, mode);
-    #endif
 }
 
 CacheTree::CacheTree(QWidget *parent)
     : QTreeWidget(parent)
     , calculated(false)
 {
-    setHeaderLabels(QStringList() << i18n("Name") << i18n("Item Count") << i18n("Space Used"));
+    setHeaderLabels(QStringList() << tr("Name") << tr("Item Count") << tr("Space Used"));
     setAllColumnsShowFocus(true);
     setSelectionMode(QAbstractItemView::ExtendedSelection);
     setRootIsDecorated(false);
@@ -230,21 +220,25 @@ void CacheTree::showEvent(QShowEvent *e)
     QTreeWidget::showEvent(e);
 }
 
-class SpaceLabel : public SqueezedTextLabel
-{
-public:
-    SpaceLabel(QWidget *p)
-        : SqueezedTextLabel(p)
-    {
-        QFont f=font();
-        f.setItalic(true);
-        setFont(f);
-        update(i18n("Calculating..."));
-    }
 
-    void update(int space) { update(Utils::formatByteSize(space)); }
-    void update(const QString &text) { setText(i18n("Total space used: %1", text)); }
-};
+SpaceLabel::SpaceLabel(QWidget *p)
+    : SqueezedTextLabel(p)
+{
+    QFont f=font();
+    f.setItalic(true);
+    setFont(f);
+    update(tr("Calculating..."));
+}
+
+void SpaceLabel::update(int space)
+{
+    update(Utils::formatByteSize(space));
+}
+
+void SpaceLabel::update(const QString &text)
+{
+    setText(tr("Total space used: %1").arg(text));
+}
 
 CacheSettings::CacheSettings(QWidget *parent)
     : QWidget(parent)
@@ -254,10 +248,11 @@ CacheSettings::CacheSettings(QWidget *parent)
     layout->setMargin(0);
     int row=0;
     int col=0;
-    QLabel *label=new QLabel(i18n("Cantata caches a local copy of the music listing to speed up loading of the library. Cantata might also have cached "
-                                  "covers, lyrics, or artist images, if these have been downloaded and could not be saved into the music folder (because "
-                                  "Cantata cannot access it, or you have configured Cantata to not save these items there). Below is a summary of Cantata's "
+    QLabel *label=new QLabel(tr("Cantata caches various pieces of information (covers, lyrics, etc). Below is a summary of Cantata's "
                                   "current cache usage."), this);
+    QFont f(Utils::smallFont(label->font()));
+    f.setItalic(true);
+    label->setFont(f);
     label->setWordWrap(true);
     layout->addWidget(label, row++, col, 1, 2);
     layout->addItem(new QSpacerItem(spacing, spacing, QSizePolicy::Fixed, QSizePolicy::Fixed), row++, 0);
@@ -265,35 +260,27 @@ CacheSettings::CacheSettings(QWidget *parent)
     tree=new CacheTree(this);
     layout->addWidget(tree, row++, col, 1, 2);
 
-    new CacheItem(i18n("Music Library"), Utils::cacheDir(MusicLibraryModel::constLibraryCache, false),
-                  QStringList() << "*"+MusicLibraryModel::constLibraryExt << "*"+MusicLibraryModel::constLibraryCompressedExt, tree);
-    new CacheItem(i18n("Covers"), Utils::cacheDir(Covers::constCoverDir, false), QStringList() << "*.jpg" << "*.png", tree,
+    new CacheItem(tr("Covers"), Utils::cacheDir(Covers::constCoverDir, false), QStringList() << "*.jpg" << "*.png", tree,
                   CacheItem::Type_Covers);
-    new CacheItem(i18n("Scaled Covers"), Utils::cacheDir(Covers::constScaledCoverDir, false), QStringList() << "*.jpg" << "*.png", tree,
+    new CacheItem(tr("Scaled Covers"), Utils::cacheDir(Covers::constScaledCoverDir, false), QStringList() << "*.jpg" << "*.png", tree,
                   CacheItem::Type_ScaledCovers);
-    new CacheItem(i18n("Backdrops"), Utils::cacheDir(ContextWidget::constCacheDir, false), QStringList() << "*.jpg" << "*.png", tree);
-    new CacheItem(i18n("Lyrics"), Utils::cacheDir(SongView::constLyricsDir, false), QStringList() << "*"+SongView::constExtension, tree);
-    new CacheItem(i18n("Artist Information"), Utils::cacheDir(ArtistView::constCacheDir, false), QStringList() << "*"+ArtistView::constInfoExt
+    new CacheItem(tr("Backdrops"), Utils::cacheDir(ContextWidget::constCacheDir, false), QStringList() << "*.jpg" << "*.png", tree);
+    new CacheItem(tr("Lyrics"), Utils::cacheDir(SongView::constLyricsDir, false), QStringList() << "*"+SongView::constExtension, tree);
+    new CacheItem(tr("Artist Information"), Utils::cacheDir(ArtistView::constCacheDir, false), QStringList() << "*"+ArtistView::constInfoExt
                   << "*"+ArtistView::constSimilarInfoExt << "*.json.gz" << "*.jpg" << "*.png", tree);
-    new CacheItem(i18n("Album Information"), Utils::cacheDir(AlbumView::constCacheDir, false), QStringList() << "*"+AlbumView::constInfoExt << "*.jpg" << "*.png", tree);
-    new CacheItem(i18n("Track Information"), Utils::cacheDir(SongView::constCacheDir, false), QStringList() << "*"+AlbumView::constInfoExt, tree);
-    #ifdef ENABLE_STREAMS
-    new CacheItem(i18n("Stream Listings"), Utils::cacheDir(StreamsModel::constSubDir, false), QStringList() << "*"+StreamsModel::constCacheExt, tree);
-    #endif
-    #ifdef ENABLE_ONLINE_SERVICES
-    new CacheItem(i18n("Jamendo"), Utils::cacheDir("jamendo", false), QStringList() << "*"+MusicLibraryModel::constLibraryCompressedExt << "*.jpg" << "*.png", tree);
-    new CacheItem(i18n("Magnatune"), Utils::cacheDir("magnatune", false), QStringList() << "*"+MusicLibraryModel::constLibraryCompressedExt << "*.jpg" << "*.png", tree);
-    new CacheItem(i18n("Podcast Directories"), Utils::cacheDir(PodcastSearchDialog::constCacheDir, false), QStringList() << "*"+PodcastSearchDialog::constExt, tree);
-    #endif
-    new CacheItem(i18n("Wikipedia Languages"), Utils::cacheDir(WikipediaSettings::constSubDir, false), QStringList() << "*.xml.gz", tree);
-    new CacheItem(i18n("Scrobble Tracks"), Utils::cacheDir(Scrobbler::constCacheDir, false), QStringList() << "*.xml.gz", tree);
+    new CacheItem(tr("Album Information"), Utils::cacheDir(AlbumView::constCacheDir, false), QStringList() << "*"+AlbumView::constInfoExt << "*.jpg" << "*.png", tree);
+    new CacheItem(tr("Track Information"), Utils::cacheDir(SongView::constCacheDir, false), QStringList() << "*"+AlbumView::constInfoExt, tree);
+    new CacheItem(tr("Stream Listings"), Utils::cacheDir(StreamsModel::constSubDir, false), QStringList() << "*"+StreamsModel::constCacheExt, tree);
+    new CacheItem(tr("Podcast Directories"), Utils::cacheDir(PodcastSearchDialog::constCacheDir, false), QStringList() << "*"+PodcastSearchDialog::constExt, tree);
+    new CacheItem(tr("Wikipedia Languages"), Utils::cacheDir(WikipediaSettings::constSubDir, false), QStringList() << "*.xml.gz", tree);
+    new CacheItem(tr("Scrobble Tracks"), Utils::cacheDir(Scrobbler::constCacheDir, false), QStringList() << "*.xml.gz", tree);
 
     for (int i=0; i<tree->topLevelItemCount(); ++i) {
         connect(static_cast<CacheItem *>(tree->topLevelItem(i)), SIGNAL(updated()), this, SLOT(updateSpace()));
     }
 
     spaceLabel=new SpaceLabel(this);
-    button=new QPushButton(i18n("Delete All"), this);
+    button=new QPushButton(tr("Delete All"), this);
     layout->addWidget(spaceLabel, row, 0, 1, 1);
     layout->addWidget(button, row++, 1, 1, 1);
     button->setEnabled(false);
@@ -332,22 +319,24 @@ void CacheSettings::deleteAll()
 
     if (!toDelete.isEmpty()) {
         if (1==toDelete.count()) {
-            if (MessageBox::Yes==MessageBox::warningYesNo(this, i18n("Delete all '%1' items?", toDelete.at(0)->name()),
-                                                          i18n("Delete Cache Items"), StdGuiItem::del(), StdGuiItem::cancel())) {
+            if (MessageBox::Yes==MessageBox::warningYesNo(this, tr("Delete all '%1' items?").arg(toDelete.at(0)->name()),
+                                                          tr("Delete Cache Items"), StdGuiItem::del(), StdGuiItem::cancel())) {
                 toDelete.first()->clean();
             }
         } else if (toDelete.count()>1) {
+            static const QChar constBullet(0x2022);
+
             QString items;
-            foreach (CacheItem *i, toDelete) {
-                items+="<li>"+i->name()+"</li>";
+            for (CacheItem *i: toDelete) {
+                items+=constBullet+QLatin1Char(' ')+i->name()+QLatin1Char('\n');
             }
 
-            if (MessageBox::No==MessageBox::warningYesNo(this, i18n("<p>Delete all from the following?<ul>%1</ul></p>", items),
-                                                         i18n("Delete Cache Items"), StdGuiItem::del(), StdGuiItem::cancel())) {
+            if (MessageBox::No==MessageBox::warningYesNo(this, tr("Delete items from all selected categories?")+QLatin1String("\n\n")+items,
+                                                         tr("Delete Cache Items"), StdGuiItem::del(), StdGuiItem::cancel())) {
                 return;
             }
 
-            foreach (CacheItem *i, toDelete) {
+            for (CacheItem *i: toDelete) {
                 i->clean();
             }
         }
@@ -362,3 +351,5 @@ void CacheSettings::updateSpace()
     }
     spaceLabel->update(space);
 }
+
+#include "moc_cachesettings.cpp"
