@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,18 +22,17 @@
  */
 
 #include "coverdialog.h"
-#include "messagebox.h"
-#include "localize.h"
-#include "listview.h"
-#include "networkaccessmanager.h"
+#include "apikeys.h"
+#include "support/messagebox.h"
+#include "widgets/listview.h"
+#include "network/networkaccessmanager.h"
 #include "settings.h"
-#include "mpdconnection.h"
-#include "utils.h"
-#include "spinner.h"
-#include "messageoverlay.h"
-#include "icon.h"
-#include "icons.h"
-#include "qjson/parser.h"
+#include "mpd-interface/mpdconnection.h"
+#include "support/utils.h"
+#include "support/spinner.h"
+#include "widgets/messageoverlay.h"
+#include "support/icon.h"
+#include "widgets/icons.h"
 #include "config.h"
 #include <QVBoxLayout>
 #include <QLabel>
@@ -51,24 +50,20 @@
 #include <QScrollBar>
 #include <QMenu>
 #include <QAction>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KFileDialog>
-#else
 #include <QFileDialog>
-#endif
 #include <QFileInfo>
 #include <QUrl>
-#if QT_VERSION >= 0x050000
 #include <QUrlQuery>
 #include <QMimeData>
-#endif
+#include <QJsonDocument>
 #include <QTemporaryFile>
 #include <QDir>
 #include <QXmlStreamReader>
 #include <QDomDocument>
 #include <QDomElement>
-#include <QDebug>
+#include <QSslSocket>
 
+#include <QDebug>
 #define DBUG if (Covers::debugEnabled()) qWarning() << "CoverDialog" << __FUNCTION__
 
 static int iCount=0;
@@ -97,13 +92,12 @@ bool canUse(int w, int h)
 enum Providers {
     Prov_LastFm   = 0x0001,
     Prov_Google   = 0x0002,
-    Prov_DiscoGs  = 0x0004,
-    Prov_CoverArt = 0x0008,
-    Prov_Deezer   = 0x0010,
-    Prov_Spotify  = 0x0020,
-    Prov_ITunes   = 0x0040,
+    Prov_CoverArt = 0x0004,
+    Prov_Deezer   = 0x0008,
+    Prov_Spotify  = 0x0010,
+    Prov_ITunes   = 0x0020,
 
-    Prov_All      = 0x007F
+    Prov_All      = 0x003F
 };
 
 class CoverItem : public QListWidgetItem
@@ -121,9 +115,9 @@ public:
             setImage(img);
         }
         if (sz>0) {
-            setText(i18nc("name\nwidth x height (file size)", "%1\n%2 x %3 (%4)", text, w, h, Utils::formatByteSize(sz)));
+            setText(QObject::tr("%1\n%2 x %3 (%4)", "name\nwidth x height (file size)").arg(text).arg(w).arg(h).arg(Utils::formatByteSize(sz)));
         } else if (w>0 && h>0) {
-            setText(i18nc("name\nwidth x height", "%1\n%2 x %3", text, w, h));
+            setText(QObject::tr("%1\n%2 x %3", "name\nwidth x height").arg(text).arg(w).arg(h));
         } else {
             setText(text);
         }
@@ -159,13 +153,13 @@ class ExistingCover : public CoverItem
 {
 public:
     ExistingCover(const Covers::Image &i, QListWidget *parent)
-        : CoverItem(i.fileName, QString(), i.img, i18n("Current Cover"), parent, i.img.width(), i.img.height(), QFileInfo(i.fileName).size())
+        : CoverItem(i.fileName, QString(), i.img, QObject::tr("Current Cover"), parent, i.img.width(), i.img.height(), QFileInfo(i.fileName).size())
         , img(i.img) {
         QFont f(font());
         f.setBold((true));
         setFont(f);
     }
-    bool isExisting() const { return true; }
+    bool isExisting() const override { return true; }
     const QImage & image() const { return img; }
 private:
     QImage img;
@@ -175,9 +169,9 @@ class LocalCover : public CoverItem
 {
 public:
     LocalCover(const QString &u, const QImage &i, QListWidget *parent)
-        : CoverItem(u, QString(), i, Utils::getFile(u), parent, img.width(), img.height(), QFileInfo(u).size())
+        : CoverItem(u, QString(), i, Utils::getFile(u), parent, i.width(), i.height(), QFileInfo(u).size())
         , img(i) { }
-    bool isLocal() const { return true; }
+    bool isLocal() const override { return true; }
     const QImage & image() const { return img; }
 private:
     QImage img;
@@ -197,18 +191,11 @@ public:
         : CoverItem(u, tu, img, QLatin1String("Google"), parent, w, h, size*1024) { }
 };
 
-class DiscogsCover : public CoverItem
-{
-public:
-    DiscogsCover(const QString &u, const QString &tu, const QImage &img, int w, int h, QListWidget *parent)
-        : CoverItem(u, tu, img, QLatin1String("Discogs"), parent, w, h) { }
-};
-
 class CoverArtArchiveCover : public CoverItem
 {
 public:
     CoverArtArchiveCover(const QString &u, const QString &tu, const QImage &img, QListWidget *parent)
-        : CoverItem(u, tu, img, i18n("CoverArt Archive"), parent)  { }
+        : CoverItem(u, tu, img, QObject::tr("CoverArt Archive"), parent)  { }
 };
 
 class DeezerCover : public CoverItem
@@ -239,10 +226,10 @@ CoverPreview::CoverPreview(QWidget *p)
       imgH(0)
 {
     setButtons(Close);
-    setWindowTitle(i18n("Image"));
+    setWindowTitle(tr("Image"));
     QWidget *mw=new QWidget(this);
     QVBoxLayout *layout=new QVBoxLayout(mw);
-    loadingLabel=new QLabel(i18n("Downloading..."), mw);
+    loadingLabel=new QLabel(tr("Downloading..."), mw);
     pbar=new QProgressBar(mw);
     imageLabel=new QLabel;
     pbar->setRange(0, 100);
@@ -272,7 +259,7 @@ void CoverPreview::showImage(const QImage &img, const QString &u)
         scrollArea->show();
         QApplication::processEvents();
         adjustSize();
-        QStyleOptionFrameV3 opt;
+        QStyleOptionFrame opt;
         opt.init(scrollArea);
         int fw=style()->pixelMetric(QStyle::PM_DefaultFrameWidth, &opt, scrollArea);
         if (fw<0) {
@@ -287,7 +274,7 @@ void CoverPreview::showImage(const QImage &img, const QString &u)
         imgW=img.width();
         imgH=img.height();
         resize(lrPad+qMax(100, qMin(maxWidth, imgW+fw)), tbPad+qMax(100, qMin(maxHeight, imgH+fw)));
-        setWindowTitle(i18nc("Image (width x height zoom%)", "Image (%1 x %2 %3%)", imgW, imgH, zoom*100));
+        setWindowTitle(tr("Image (%1 x %2 %3%)", "Image (width x height zoom%)").arg(imgW).arg(imgH).arg(zoom*100));
         show();
     }
 }
@@ -322,7 +309,7 @@ void CoverPreview::scaleImage(int adjust)
     }
     zoom=newZoom;
     imageLabel->resize(zoom * imageLabel->pixmap()->size());
-    setWindowTitle(i18nc("Image (width x height zoom%)", "Image (%1 x %2 %3%)", imgW, imgH, zoom*100));
+    setWindowTitle(tr("Image (%1 x %2 %3%)", "Image (width x height zoom%)").arg(imgW).arg(imgH).arg(zoom*100));
 }
 
 void CoverPreview::wheelEvent(QWheelEvent *event)
@@ -341,25 +328,20 @@ void CoverPreview::wheelEvent(QWheelEvent *event)
 
 CoverDialog::CoverDialog(QWidget *parent)
     : Dialog(parent, "CoverDialog")
-    , existing(0)
+    , existing(nullptr)
     , currentQueryProviders(0)
-    , preview(0)
+    , preview(nullptr)
     , saving(false)
     , isArtist(false)
-    , spinner(0)
-    , msgOverlay(0)
+    , spinner(nullptr)
+    , msgOverlay(nullptr)
     , page(0)
-    , menu(0)
-    , showAction(0)
-    , removeAction(0)
+    , menu(nullptr)
+    , showAction(nullptr)
+    , removeAction(nullptr)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    KConfigGroup cfg(KGlobal::config(), "CoverDialog");
-    #else
-    QSettings cfg;
-    cfg.beginGroup("CoverDialog");
-    #endif
-    enabledProviders=GET_INT("enabledProviders", (int)Prov_All);
+    Configuration cfg(objectName());
+    enabledProviders=cfg.get("enabledProviders", (int)Prov_All);
 
     iCount++;
     QWidget *mainWidet = new QWidget(this);
@@ -397,10 +379,10 @@ CoverDialog::CoverDialog(QWidget *parent)
     list->setSpacing(spacing);
     list->setViewMode(QListView::IconMode);
     list->setResizeMode(QListView::Adjust);
-    list->setMinimumSize((list->gridSize().width()*4)+(spacing*5), list->gridSize().height()*3);
+    list->setMinimumSize((list->gridSize().width()*3)+style()->pixelMetric(QStyle::PM_ScrollBarExtent)+spacing, list->gridSize().height()*2);
     list->setSortingEnabled(false);
 
-    addFileButton->setIcon(Icon("document-open"));
+    addFileButton->setIcon(Icons::self()->folderListIcon);
     addFileButton->setAutoRaise(true);
 
     configureButton->setIcon(Icons::self()->configureIcon);
@@ -408,9 +390,8 @@ CoverDialog::CoverDialog(QWidget *parent)
 
     QMenu *configMenu=new QMenu(configureButton);
     addProvider(configMenu, QLatin1String("Last.fm"), Prov_LastFm, enabledProviders);
-    addProvider(configMenu, i18n("CoverArt Archive"), Prov_CoverArt, enabledProviders);
+    addProvider(configMenu, tr("CoverArt Archive"), Prov_CoverArt, enabledProviders);
     addProvider(configMenu, QLatin1String("Google"), Prov_Google, enabledProviders);
-    addProvider(configMenu, QLatin1String("Discogs"), Prov_DiscoGs, enabledProviders);
     addProvider(configMenu, QLatin1String("Deezer"), Prov_Deezer, enabledProviders);
     addProvider(configMenu, QLatin1String("Spotify"), Prov_Spotify, enabledProviders);
     addProvider(configMenu, QLatin1String("iTunes"), Prov_ITunes, enabledProviders);
@@ -421,13 +402,8 @@ CoverDialog::CoverDialog(QWidget *parent)
 
 CoverDialog::~CoverDialog()
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    KConfigGroup cfg(KGlobal::config(), "CoverDialog");
-    #else
-    QSettings cfg;
-    cfg.beginGroup("CoverDialog");
-    #endif
-    SET_VALUE("enabledProviders", enabledProviders);
+    Configuration cfg(objectName());
+    cfg.set("enabledProviders", enabledProviders);
 
     iCount--;
     cancelQuery();
@@ -440,25 +416,44 @@ void CoverDialog::show(const Song &s, const Covers::Image &current)
     isArtist=song.isArtistImageRequest();
     Covers::Image img=current.img.isNull() ? Covers::locateImage(song) : current;
 
-    if (!img.fileName.isEmpty() && !QFileInfo(img.fileName).isWritable()) {
+    if (img.validFileName() && !QFileInfo(img.fileName).isWritable()) {
         MessageBox::error(parentWidget(),
-                          isArtist
-                            ? i18n("<p>An image already exists for this artist, and the file is not writeable.<p></p><i>%1</i></p>", img.fileName)
-                            : i18n("<p>A cover already exists for this album, and the file is not writeable.<p></p><i>%1</i></p>", img.fileName));
+                          (isArtist
+                            ? tr("An image already exists for this artist, and the file is not writeable.")
+                            : tr("A cover already exists for this album, and the file is not writeable."))+
+                          QString("<br/><br/><a href=\"%1\">%1</a>").arg(img.fileName));
         deleteLater();
         return;
     }
+    note->setVisible(!isArtist);
     if (isArtist) {
-        setCaption(i18n("'%1' Artist Image", song.albumartist));
+        setCaption(tr("'%1' Artist Image").arg(song.albumArtist()));
     } else {
-        setCaption(i18nc("'Artist - Album' Album Cover", "'%1 - %2' Album Cover", song.albumArtist(), song.album));
+        setCaption(tr("'%1 - %2' Album Cover", "'Artist - Album' Album Cover").arg(song.albumArtist()).arg(song.album));
     }
     if (!img.img.isNull()) {
         existing=new ExistingCover(isArtist ? Covers::Image(cropImage(img.img, true), img.fileName) : img, list);
         list->addItem(existing);
     }
-    query->setText(isArtist ? song.albumartist : QString(song.albumArtist()+QLatin1String(" ")+song.album));
-    adjustSize();
+    /*
+    Add other images in the source folder? #716
+    if (!isArtist) {
+        QString dirName=MPDConnection::self()->getDetails().dir+Utils::getDir(s.file);
+        QStringList files=QDir(dirName).entryList(QStringList() << QLatin1String("*.jpg") << QLatin1String("*.jpeg") << QLatin1String("*.png"), QDir::Files|QDir::Readable);
+        qWarning() << files;
+        for (const QString &f: files) {
+            QString fileName=dirName+f;
+            if (fileName!=img.fileName) {
+                QImage i(fileName);
+                if (!i.isNull()) {
+                    currentLocalCovers.insert(fileName);
+                    insertItem(new LocalCover(fileName, i, list));
+                }
+            }
+        }
+    }
+    */
+    query->setText(isArtist ? song.albumArtist() : QString(song.albumArtist()+QLatin1String(" ")+song.album));
     Dialog::show();
     sendQuery();
 }
@@ -472,7 +467,6 @@ static const char * constSizeProperty="sz";
 static const char * constTypeProperty="type";
 static const char * constLastFmHost="ws.audioscrobbler.com";
 static const char * constGoogleHost="images.google.com";
-static const char * constDiscogsHost="api.discogs.com";
 static const char * constCoverArtArchiveHost="coverartarchive.org";
 static const char * constSpotifyHost="ws.spotify.com";
 static const char * constITunesHost="itunes.apple.com";
@@ -498,8 +492,6 @@ void CoverDialog::queryJobFinished()
             parseLastFmQueryResponse(resp);
         } else if (constGoogleHost==host) {
             parseGoogleQueryResponse(resp);
-        } else if (constDiscogsHost==host) {
-            parseDiscogsQueryResponse(resp);
         } else if (constCoverArtArchiveHost==host) {
             parseCoverArtArchiveQueryResponse(resp);
         } else if (constSpotifyHost==host) {
@@ -551,10 +543,10 @@ void CoverDialog::downloadJobFinished()
         QImage img=QImage::fromData(data, format);
         if (!img.isNull()) {
             bool isLarge=reply->property(constThumbProperty).toString().isEmpty();
-            QTemporaryFile *temp=0;
+            QTemporaryFile *temp=nullptr;
 
             if (isLarge || (reply->property(constThumbProperty).toString()==reply->property(constLargeProperty).toString())) {
-                temp=new QTemporaryFile(QDir::tempPath()+"/cantata_XXXXXX."+(format ? QString(QLatin1String(format)).toLower() : "png"));
+                temp=new QTemporaryFile("cantata_XXXXXX."+(format ? QString(QLatin1String(format)).toLower() : "png"));
 
                 if (temp->open()) {
                     if (!format) {
@@ -572,7 +564,7 @@ void CoverDialog::downloadJobFinished()
                     tempFiles.prepend(temp);
                 } else {
                     delete temp;
-                    temp=0;
+                    temp=nullptr;
                 }
             }
             if (isLarge) {
@@ -580,22 +572,19 @@ void CoverDialog::downloadJobFinished()
                     previewDialog()->showImage(cropImage(img, isArtist), reply->property(constLargeProperty).toString());
                 } else if (DL_LargeSave==dlType) {
                     if (!temp) {
-                        MessageBox::error(this, i18n("Failed to set cover!\nCould not download to temporary file!"));
+                        MessageBox::error(this, tr("Failed to set cover!\n\nCould not download to temporary file!"));
                     } else if (saveCover(temp->fileName(), img)) {
                         accept();
                     }
                 }
             } else {
-                CoverItem *item=0;
+                CoverItem *item=nullptr;
                 img=cropImage(img, isArtist);
                 if (constLastFmHost==host) {
                     item=new LastFmCover(reply->property(constLargeProperty).toString(), url, img, list);
                 } else if (constGoogleHost==host) {
                     item=new GoogleCover(reply->property(constLargeProperty).toString(), url, img, reply->property(constWidthProperty).toInt(),
                                          reply->property(constHeightProperty).toInt(), reply->property(constSizeProperty).toInt(), list);
-                } else if (constDiscogsHost==host) {
-                    item=new DiscogsCover(reply->property(constLargeProperty).toString(), url, img, reply->property(constWidthProperty).toInt(),
-                                          reply->property(constHeightProperty).toInt(), list);
                 } else if (constCoverArtArchiveHost==host) {
                     item=new CoverArtArchiveCover(reply->property(constLargeProperty).toString(), url, img, list);
                 } else if (constSpotifyHost==host) {
@@ -614,7 +603,7 @@ void CoverDialog::downloadJobFinished()
         if (preview && preview->aboutToShow(reply->property(constLargeProperty).toString())) {
             preview->hide();
         }
-        MessageBox::error(this, i18n("Failed to download image!"));
+        MessageBox::error(this, tr("Failed to download image!"));
     }
     if (currentQuery.isEmpty()) {
         setSearching(false);
@@ -685,7 +674,7 @@ void CoverDialog::sendQuery()
             }
         }
 
-        foreach (CoverItem *item, keep) {
+        for (CoverItem *item: keep) {
             list->addItem(item);
         }
 
@@ -699,9 +688,6 @@ void CoverDialog::sendQuery()
     if (enabledProviders&Prov_Google) {
         sendGoogleQuery(fixedQuery, page);
     }
-    if (enabledProviders&Prov_DiscoGs) {
-        sendDiscoGsQuery(fixedQuery, page);
-    }
     if (page==0) {
         if (enabledProviders&Prov_Spotify) {
             sendSpotifyQuery(fixedQuery);
@@ -720,33 +706,23 @@ void CoverDialog::sendQuery()
 void CoverDialog::sendLastFmQuery(const QString &fixedQuery, int page)
 {
     QUrl url;
-    #if QT_VERSION < 0x050000
-    QUrl &query=url;
-    #else
     QUrlQuery query;
-    #endif
     url.setScheme("https");
     url.setHost(constLastFmHost);
     url.setPath("/2.0/");
-    query.addQueryItem("api_key", Covers::constLastFmApiKey);
+    ApiKeys::self()->addKey(query, ApiKeys::LastFm);
     query.addQueryItem("limit", QString::number(20));
-    query.addQueryItem("page", QString::number(page));
+    query.addQueryItem("page", QString::number(page+1));
     query.addQueryItem(isArtist ? "artist" : "album", fixedQuery);
     query.addQueryItem("method", isArtist ? "artist.search" : "album.search");
-    #if QT_VERSION >= 0x050000
     url.setQuery(query);
-    #endif
     sendQueryRequest(url);
 }
 
 void CoverDialog::sendGoogleQuery(const QString &fixedQuery, int page)
 {
     QUrl url;
-    #if QT_VERSION < 0x050000
-    QUrl &query=url;
-    #else
     QUrlQuery query;
-    #endif
     url.setScheme("http");
     url.setHost(constGoogleHost);
     url.setPath("/images");
@@ -754,53 +730,26 @@ void CoverDialog::sendGoogleQuery(const QString &fixedQuery, int page)
     query.addQueryItem("gbv", QChar('1'));
     query.addQueryItem("filter", QChar('1'));
     query.addQueryItem("start", QString::number(20 * page));
-    #if QT_VERSION >= 0x050000
     url.setQuery(query);
-    #endif
-    sendQueryRequest(url);
-}
-
-void CoverDialog::sendDiscoGsQuery(const QString &fixedQuery, int page)
-{
-    QUrl url;
-    #if QT_VERSION < 0x050000
-    QUrl &query=url;
-    #else
-    QUrlQuery query;
-    #endif
-    url.setScheme("http");
-    url.setHost(constDiscogsHost);
-    url.setPath("/search");
-    query.addQueryItem("page", QString::number(page + 1));
-    query.addQueryItem("per_page", QString::number(20));
-    query.addQueryItem("type", isArtist ? "artist" : "release");
-    query.addQueryItem("q", fixedQuery);
-    query.addQueryItem("f", "json");
-    #if QT_VERSION >= 0x050000
-    url.setQuery(query);
-    #endif
     sendQueryRequest(url);
 }
 
 void CoverDialog::sendSpotifyQuery(const QString &fixedQuery)
 {
-    #ifdef ENABLE_HTTPS_SUPPORT
-    QUrl url;
-    #if QT_VERSION < 0x050000
-    QUrl &query=url;
+    #ifdef QT_NO_SSL
+    return;
     #else
+    if (!QSslSocket::supportsSsl()) {
+        return;
+    }
+    QUrl url;
     QUrlQuery query;
-    #endif
     url.setScheme("http");
     url.setHost(constSpotifyHost);
     url.setPath(isArtist ? "/search/1/artist.json" : "/search/1/album.json");
     query.addQueryItem("q", fixedQuery);
-    #if QT_VERSION >= 0x050000
     url.setQuery(query);
-    #endif
     sendQueryRequest(url);
-    #else
-    Q_UNUSED(fixedQuery)
     #endif
 }
 
@@ -811,11 +760,7 @@ void CoverDialog::sendITunesQuery(const QString &fixedQuery)
     }
 
     QUrl url;
-    #if QT_VERSION < 0x050000
-    QUrl &query=url;
-    #else
     QUrlQuery query;
-    #endif
     url.setScheme("http");
     url.setHost(constITunesHost);
     url.setPath("/search");
@@ -823,29 +768,21 @@ void CoverDialog::sendITunesQuery(const QString &fixedQuery)
     query.addQueryItem("limit", QString::number(10));
     query.addQueryItem("media", "music");
     query.addQueryItem("entity", "album");
-    #if QT_VERSION >= 0x050000
     url.setQuery(query);
-    #endif
     sendQueryRequest(url);
 }
 
 void CoverDialog::sendDeezerQuery(const QString &fixedQuery)
 {
     QUrl url;
-    #if QT_VERSION < 0x050000
-    QUrl &query=url;
-    #else
     QUrlQuery query;
-    #endif
     url.setScheme("http");
     url.setHost(constDeezerHost);
     url.setPath(isArtist ? "/search/artist" : "/search/album");
     query.addQueryItem("q", fixedQuery);
     query.addQueryItem("nb_items", QString::number(10));
     query.addQueryItem("output", "json");
-    #if QT_VERSION >= 0x050000
     url.setQuery(query);
-    #endif
     sendQueryRequest(url);
 }
 
@@ -857,7 +794,7 @@ void CoverDialog::checkStatus()
 
 void CoverDialog::cancelQuery()
 {
-    foreach (NetworkJob *job, currentQuery) {
+    for (NetworkJob *job: currentQuery) {
         job->cancelAndDelete();
     }
     currentQuery.clear();
@@ -866,19 +803,15 @@ void CoverDialog::cancelQuery()
 
 void CoverDialog::addLocalFile()
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    QString fileName=KFileDialog::getOpenFileName(KUrl(), "image/jpeg image/png", this, i18n("Load Local Cover"));
-    #else
-    QString fileName=QFileDialog::getOpenFileName(this, i18n("Load Local Cover"), QDir::homePath(), i18n("Images (*.png *.jpg)"));
-    #endif
+    QString fileName=QFileDialog::getOpenFileName(this, tr("Load Local Cover"), QDir::homePath(), tr("Images (*.png *.jpg *.jpeg)"));
 
     if (!fileName.isEmpty()) {
         if (currentLocalCovers.contains(fileName)) {
-            MessageBox::error(this, i18n("File is already in list!"));
+            MessageBox::error(this, tr("File is already in list!"));
         } else {
             QImage img(fileName);
             if (img.isNull()) {
-                MessageBox::error(this, i18n("Failed to read image!"));
+                MessageBox::error(this, tr("Failed to read image!"));
             } else {
                 currentLocalCovers.insert(fileName);
                 insertItem(new LocalCover(fileName, img, list));
@@ -891,17 +824,17 @@ void CoverDialog::menuRequested(const QPoint &pos)
 {
     if (!menu) {
         menu=new QMenu(list);
-        showAction=menu->addAction(Icon("zoom-original"), i18n("Display"));
-        removeAction=menu->addAction(Icon("list-remove"), i18n("Remove"));
-        connect(showAction, SIGNAL(triggered(bool)), SLOT(showImage()));
-        connect(removeAction, SIGNAL(triggered(bool)), SLOT(removeImages()));
+        showAction=menu->addAction(tr("Display"));
+        removeAction=menu->addAction(tr("Remove"));
+        connect(showAction, SIGNAL(triggered()), SLOT(showImage()));
+        connect(removeAction, SIGNAL(triggered()), SLOT(removeImages()));
     }
 
     QList<QListWidgetItem*> items=list->selectedItems();
     showAction->setEnabled(1==items.count());
     removeAction->setEnabled(!items.isEmpty());
     if (removeAction->isEnabled()) {
-        foreach (QListWidgetItem *i, items) {
+        for (QListWidgetItem *i: items) {
             if (static_cast<CoverItem *>(i)->isExisting()) {
                 removeAction->setEnabled(false);
             }
@@ -921,7 +854,7 @@ void CoverDialog::showImage()
 void CoverDialog::removeImages()
 {
     QList<QListWidgetItem*> items=list->selectedItems();
-    foreach (QListWidgetItem *i, items) {
+    for (QListWidgetItem *i: items) {
         delete i;
     }
 }
@@ -939,7 +872,7 @@ void CoverDialog::updateProviders()
         }
     }
 
-    foreach (const QAction *act, configureButton->menu()->actions()) {
+    for (const QAction *act: configureButton->menu()->actions()) {
         if (act->isChecked()) {
             enabledProviders|=act->data().toUInt();
         }
@@ -949,7 +882,7 @@ void CoverDialog::updateProviders()
 
 void CoverDialog::clearTempFiles()
 {
-    foreach (QTemporaryFile *file, tempFiles) {
+    for (QTemporaryFile *file: tempFiles) {
         file->remove();
         delete file;
     }
@@ -970,11 +903,11 @@ NetworkJob * CoverDialog::downloadImage(const QString &url, DownloadType dlType)
     DBUG << url << dlType;
     if (DL_Thumbnail==dlType) {
         if (currentUrls.contains(url)) {
-            return 0;
+            return nullptr;
         }
         currentUrls.insert(url);
     } else {
-        foreach (QTemporaryFile *tmp, tempFiles) {
+        for (QTemporaryFile *tmp: tempFiles) {
             if (tmp->property(constLargeProperty).toString()==url) {
                 QImage img;
                 if (img.load(tmp->fileName())) {
@@ -986,7 +919,7 @@ NetworkJob * CoverDialog::downloadImage(const QString &url, DownloadType dlType)
                             accept();
                         }
                     }
-                    return 0;
+                    return nullptr;
                 }
                 tmp->remove();
                 delete tmp;
@@ -1043,6 +976,7 @@ void CoverDialog::parseLastFmQueryResponse(const QByteArray &resp)
     QList<SizeMap> entries;
     QStringList musibBrainzIds;
 
+    doc.setNamespaceProcessing(false);
     while (!doc.atEnd()) {
         doc.readNext();
 
@@ -1074,18 +1008,18 @@ void CoverDialog::parseLastFmQueryResponse(const QByteArray &resp)
     QStringList largeUrls=QStringList() << "extralarge" << "large";
     QStringList thumbUrls=QStringList() << "large" << "medium" << "small";
 
-    foreach (const SizeMap &urls, entries) {
+    for (const SizeMap &urls: entries) {
         QString largeUrl;
         QString thumbUrl;
 
-        foreach (const QString &u, largeUrls) {
+        for (const QString &u: largeUrls) {
             if (urls.contains(u)) {
                 largeUrl=urls[u];
                 break;
             }
         }
 
-        foreach (const QString &u, thumbUrls) {
+        for (const QString &u: thumbUrls) {
             if (urls.contains(u)) {
                 thumbUrl=urls[u];
                 break;
@@ -1095,7 +1029,7 @@ void CoverDialog::parseLastFmQueryResponse(const QByteArray &resp)
     }
 
     if (enabledProviders&Prov_CoverArt) {
-        foreach (const QString &id, musibBrainzIds) {
+        for (const QString &id: musibBrainzIds) {
             QUrl coverartUrl;
             coverartUrl.setScheme("http");
             coverartUrl.setHost(constCoverArtArchiveHost);
@@ -1117,12 +1051,8 @@ void CoverDialog::parseGoogleQueryResponse(const QByteArray &resp)
     QString html = xml.replace(QLatin1String("&amp;"), QLatin1String("&"));
 
     while (-1!=(pos=rx.indexIn(html, pos))) {
-        #if QT_VERSION < 0x050000
-        QUrl url("http://www.google.com"+rx.cap(1));
-        #else
         QUrl u("http://www.google.com"+rx.cap(1));
         QUrlQuery url(u);
-        #endif
         int width=url.queryItemValue("w").toInt();
         int height=url.queryItemValue("h").toInt();
         if (canUse(width, height)) {
@@ -1132,73 +1062,15 @@ void CoverDialog::parseGoogleQueryResponse(const QByteArray &resp)
     }
 }
 
-void CoverDialog::parseDiscogsQueryResponse(const QByteArray &resp)
-{
-    QJson::Parser parser;
-    bool ok=false;
-    QVariantMap parsed=parser.parse(resp, &ok).toMap();
-    if (ok && parsed.contains("resp")) {
-        QVariantMap response=parsed["resp"].toMap();
-        if (response.contains("search")) {
-            QVariantMap search=response["search"].toMap();
-            if (search.contains("searchresults")) {
-                QVariantMap searchresults=search["searchresults"].toMap();
-                if (searchresults.contains("results")) {
-                    QVariantList results=searchresults["results"].toList();
-                    foreach (const QVariant &r, results) {
-                        QVariantMap rm=r.toMap();
-                        if (!isArtist && rm.contains("uri")) {
-                            QStringList parts=rm["uri"].toString().split("/", QString::SkipEmptyParts);
-                            if (!parts.isEmpty()) {
-                                QUrl discogsUrl;
-                                #if QT_VERSION < 0x050000
-                                QUrl &discogsQuery=discogsUrl;
-                                #else
-                                QUrlQuery discogsQuery;
-                                #endif
-                                discogsUrl.setScheme("http");
-                                discogsUrl.setHost(constDiscogsHost);
-                                discogsUrl.setPath("/release/"+parts.last());
-                                discogsQuery.addQueryItem("f", "json");
-                                #if QT_VERSION >= 0x050000
-                                discogsUrl.setQuery(discogsQuery);
-                                #endif
-                                sendQueryRequest(discogsUrl);
-                            }
-                        } else if (isArtist && rm.contains("thumb")) {
-                            QString thumbUrl=rm["thumb"].toString();
-                            if (thumbUrl.contains("/image/A-150-")) {
-                                downloadThumbnail(thumbUrl, QString(thumbUrl).replace("image/A-150-", "/image/A-"), constDiscogsHost);
-                            }
-                        }
-                    }
-                }
-            }
-        } else if (response.contains("release")) {
-            QVariantMap release=response["release"].toMap();
-            if (release.contains("images")) {
-                QVariantList images=release["images"].toList();
-                foreach (const QVariant &i, images) {
-                    QVariantMap im=i.toMap();
-                    if (im.contains("uri") && im.contains("uri150") && im.contains("width") && im.contains("height") &&
-                        canUse(im["width"].toString().toInt(), im["height"].toString().toInt())) {
-                        downloadThumbnail(im["uri150"].toString(), im["uri"].toString(), constDiscogsHost,
-                                          im["width"].toString().toInt(), im["height"].toString().toInt());
-                    }
-                }
-            }
-        }
-    }
-}
-
 void CoverDialog::parseCoverArtArchiveQueryResponse(const QByteArray &resp)
 {
-    QJson::Parser parser;
-    bool ok=false;
-    QVariantMap parsed=parser.parse(resp, &ok).toMap();
+    QJsonParseError jsonParseError;
+    QVariantMap parsed=QJsonDocument::fromJson(resp, &jsonParseError).toVariant().toMap();
+    bool ok=QJsonParseError::NoError==jsonParseError.error;
+
     if (ok && parsed.contains("images")) {
         QVariantList images=parsed["images"].toList();
-        foreach (const QVariant &i, images) {
+        for (const QVariant &i: images) {
             QVariantMap im=i.toMap();
             if (im.contains("front") && im["front"].toBool() && im.contains("image") && im.contains("thumbnails")) {
                 QVariantMap thumb=im["thumbnails"].toMap();
@@ -1217,9 +1089,10 @@ void CoverDialog::parseCoverArtArchiveQueryResponse(const QByteArray &resp)
 
 void CoverDialog::parseSpotifyQueryResponse(const QByteArray &resp)
 {
-    QJson::Parser parser;
-    bool ok=false;
-    QVariantMap parsed=parser.parse(resp, &ok).toMap();
+    QJsonParseError jsonParseError;
+    QVariantMap parsed=QJsonDocument::fromJson(resp, &jsonParseError).toVariant().toMap();
+    bool ok=QJsonParseError::NoError==jsonParseError.error;
+
     if (ok) {
         if (parsed.contains("info")) { // Initial query response...
             const QString key=QLatin1String(isArtist ? "artists" : "albums");
@@ -1227,7 +1100,7 @@ void CoverDialog::parseSpotifyQueryResponse(const QByteArray &resp)
             const QString baseUrl=QLatin1String("https://embed.spotify.com/oembed/?url=http://open.spotify.com/")+QLatin1String(isArtist ? "artist/" : "album/");
             if (parsed.contains(key)) {
                 QVariantList results=parsed[key].toList();
-                foreach (const QVariant &res, results) {
+                for (const QVariant &res: results) {
                     QVariantMap item=res.toMap();
                     if (item.contains("href")) {
                         QString url=item["href"].toString();
@@ -1247,12 +1120,13 @@ void CoverDialog::parseSpotifyQueryResponse(const QByteArray &resp)
 
 void CoverDialog::parseITunesQueryResponse(const QByteArray &resp)
 {
-    QJson::Parser parser;
-    bool ok=false;
-    QVariantMap parsed=parser.parse(resp, &ok).toMap();
+    QJsonParseError jsonParseError;
+    QVariantMap parsed=QJsonDocument::fromJson(resp, &jsonParseError).toVariant().toMap();
+    bool ok=QJsonParseError::NoError==jsonParseError.error;
+
     if (ok && parsed.contains("results")) {
         QVariantList results=parsed["results"].toList();
-        foreach (const QVariant &res, results) {
+        for (const QVariant &res: results) {
             QVariantMap item=res.toMap();
             if (item.contains("artworkUrl100")) {
                 QString thumbUrl=item["artworkUrl100"].toString();
@@ -1265,12 +1139,13 @@ void CoverDialog::parseITunesQueryResponse(const QByteArray &resp)
 void CoverDialog::parseDeezerQueryResponse(const QByteArray &resp)
 {
     const QString key=QLatin1String(isArtist ? "picture" : "cover");
-    QJson::Parser parser;
-    bool ok=false;
-    QVariantMap parsed=parser.parse(resp, &ok).toMap();
+    QJsonParseError jsonParseError;
+    QVariantMap parsed=QJsonDocument::fromJson(resp, &jsonParseError).toVariant().toMap();
+    bool ok=QJsonParseError::NoError==jsonParseError.error;
+
     if (ok && parsed.contains("data")) {
         QVariantList results=parsed["data"].toList();
-        foreach (const QVariant &res, results) {
+        for (const QVariant &res: results) {
             QVariantMap item=res.toMap();
             if (item.contains(key)) {
                 QString thumbUrl=item[key].toString();
@@ -1313,10 +1188,14 @@ void CoverDialog::slotButtonClicked(int button)
 bool CoverDialog::saveCover(const QString &src, const QImage &img)
 {
     QString filePath=song.filePath();
+    QString ext = Utils::getExtension(src);
+    if (0==ext.compare(".jpeg", Qt::CaseInsensitive)) {
+        ext = ".jpg";
+    }
     if (song.isCdda()) {
         QString dir = Utils::cacheDir(Covers::constCddaCoverDir, true);
         if (!dir.isEmpty()) {
-            QString destName=dir+filePath.mid(7)+src.mid(src.length()-4);
+            QString destName=dir+filePath.mid(Song::constCddaProtocol.length())+ext;
             if (QFile::exists(destName)) {
                 QFile::remove(destName);
             }
@@ -1325,16 +1204,16 @@ bool CoverDialog::saveCover(const QString &src, const QImage &img)
                 return true;
             }
         }
-        MessageBox::error(this, i18n("Failed to set cover!\nCould not make copy!"));
+        MessageBox::error(this, tr("Failed to set cover!\n\nCould not make copy!"));
         return false;
     }
     QString existingBackup;
 
-    if (existing && !existing->url().isEmpty()) {
+    if (existing && !existing->url().isEmpty() && !existing->url().startsWith(Covers::constCoverInTagPrefix)) {
         static const QLatin1String constBakExt(".bak");
         existingBackup=existing->url()+constBakExt;
         if (!QFile::rename(existing->url(), existingBackup)) {
-            MessageBox::error(this, i18n("Failed to set cover!\nCould not backup original!"));
+            MessageBox::error(this, tr("Failed to set cover!\n\nCould not backup original!"));
             return false;
         }
     }
@@ -1342,7 +1221,7 @@ bool CoverDialog::saveCover(const QString &src, const QImage &img)
     QString destName;
     QString mpdDir;
     QString dirName;
-    bool saveInMpd=Settings::self()->storeCoversInMpdDir();
+    bool saveInMpd=!isArtist && Settings::self()->storeCoversInMpdDir();
 
     if (saveInMpd) {
         bool haveAbsPath=filePath.startsWith('/');
@@ -1351,22 +1230,23 @@ bool CoverDialog::saveCover(const QString &src, const QImage &img)
             dirName=filePath.endsWith('/') ? (haveAbsPath ? QString() : mpdDir)+filePath
                                             : Utils::getDir((haveAbsPath ? QString() : mpdDir)+filePath);
         }
+        saveInMpd=QDir(dirName).exists();
     }
 
     if (isArtist) {
         if (saveInMpd && !mpdDir.isEmpty() && dirName.startsWith(mpdDir) && 2==dirName.mid(mpdDir.length()).split('/', QString::SkipEmptyParts).count()) {
             QDir d(dirName);
             d.cdUp();
-            destName=d.absolutePath()+'/'+Covers::artistFileName(song)+src.mid(src.length()-4);
+            destName=d.absolutePath()+'/'+Covers::constArtistImage+ext;
         } else {
-            destName=Utils::cacheDir(Covers::constCoverDir, true)+Covers::encodeName(song.albumartist)+src.mid(src.length()-4);
+            destName=Utils::cacheDir(Covers::constCoverDir, true)+Covers::encodeName(song.albumArtist())+ext;
         }
     } else {
         if (saveInMpd) {
-            destName=dirName+Covers::albumFileName(song)+src.mid(src.length()-4);
+            destName=dirName+Covers::albumFileName(song)+ext;
         } else { // Save to cache dir...
             QString dir(Utils::cacheDir(Covers::constCoverDir+Covers::encodeName(song.albumArtist()), true));
-            destName=dir+Covers::encodeName(song.album)+src.mid(src.length()-4);
+            destName=dir+Covers::encodeName(song.album)+ext;
         }
     }
 
@@ -1375,24 +1255,20 @@ bool CoverDialog::saveCover(const QString &src, const QImage &img)
         if (!existingBackup.isEmpty() && QFile::exists(existingBackup)) {
             QFile::remove(existingBackup);
         }
-        Covers::self()->emitCoverUpdated(song, img, destName);
+        Covers::self()->updateCover(song, img, destName);
         return true;
     } else {
         if (existing && !existingBackup.isEmpty()) {
             QFile::rename(existingBackup, existing->url());
         }
-        MessageBox::error(this, i18n("Failed to set cover!\nCould not copy file to '%1'!", destName));
+        MessageBox::error(this, tr("Failed to set cover!\n\nCould not copy file to '%1'!").arg(destName));
         return false;
     }
 }
 
 static bool hasMimeType(QDropEvent *event)
 {
-    #if QT_VERSION < 0x050000
-    return event->provides("text/uri-list");
-    #else
     return event->mimeData()->formats().contains(QLatin1String("text/uri-list"));
-    #endif
 }
 
 void CoverDialog::dragEnterEvent(QDragEnterEvent *event)
@@ -1408,10 +1284,10 @@ void CoverDialog::dropEvent(QDropEvent *event)
         event->acceptProposedAction();
 
         QList<QUrl> urls(event->mimeData()->urls());
-        foreach (const QUrl &url, urls) {
+        for (const QUrl &url: urls) {
             if (url.scheme().isEmpty() || "file"==url.scheme()) {
                 QString path=url.path();
-                if (!currentLocalCovers.contains(path) && (path.endsWith(".jpg", Qt::CaseInsensitive) || path.endsWith(".png", Qt::CaseInsensitive))) {
+                if (!currentLocalCovers.contains(path) && (path.endsWith(".jpg", Qt::CaseInsensitive) || path.endsWith(".jpeg", Qt::CaseInsensitive) || path.endsWith(".png", Qt::CaseInsensitive))) {
                     QImage img(path);
                     if (!img.isNull()) {
                         currentLocalCovers.insert(path);
@@ -1439,7 +1315,7 @@ void CoverDialog::setSearching(bool s)
     }
     if (s) {
         spinner->start();
-        msgOverlay->setText(i18n("Searching..."));
+        msgOverlay->setText(tr("Searching..."));
     } else {
         spinner->stop();
         msgOverlay->setText(QString());
@@ -1455,3 +1331,5 @@ void CoverDialog::addProvider(QMenu *mnu, const QString &name, int bit, int valu
     connect(act, SIGNAL(toggled(bool)), this, SLOT(updateProviders()));
     providers.insert(bit, act);
 }
+
+#include "moc_coverdialog.cpp"
