@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -32,7 +32,7 @@
 #include <QPixmap>
 #include <QMutex>
 #include <QCache>
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 #include "config.h"
 
 class QString;
@@ -40,9 +40,6 @@ class Thread;
 class NetworkJob;
 class QMutex;
 class QTimer;
-#ifdef ENABLE_KDE_SUPPORT
-class KUrl;
-#endif
 class NetworkAccessManager;
 
 class CoverDownloader : public QObject
@@ -51,25 +48,25 @@ class CoverDownloader : public QObject
 
 public:
     enum JobType {
+        JobMpd,
         JobHttpJpg,
         JobHttpPng,
-        JobLastFm
+        JobRemote
     };
 
     struct Job
     {
-        Job(const Song &s, const QString &d, bool a=false)
-            : song(s), filePath(s.filePath()), dir(d), isArtist(a), type(JobLastFm), level(0) { }
+        Job(const Song &s, const QString &d)
+            : song(s), filePath(s.filePath()), dir(d), type(JobRemote), level(0) { }
         Song song;
         QString filePath;
         QString dir;
-        bool isArtist;
         JobType type;
         int level;
     };
 
     CoverDownloader();
-    ~CoverDownloader() { }
+    ~CoverDownloader() override { }
 
     void stop();
 
@@ -79,14 +76,19 @@ public Q_SLOTS:
 Q_SIGNALS:
     void cover(const Song &song, const QImage &img, const QString &file);
     void artistImage(const Song &song, const QImage &img, const QString &file);
+    void composerImage(const Song &song, const QImage &img, const QString &file);
+    void mpdCover(const Song &song);
 
 private:
+    void downloadViaMpd(Job &job);
     bool downloadViaHttp(Job &job, JobType type);
-    void downloadViaLastFm(Job &job);
+    void downloadViaRemote(Job &job);
 
 private Q_SLOTS:
-    void lastFmCallFinished();
+    void mpdAlbumArt(const Song &song, const QByteArray &data);
+    void remoteCallFinished();
     void jobFinished();
+    void onlineJobFinished();
 
 private:
     void failed(const Job &job);
@@ -96,6 +98,7 @@ private:
 
 private:
     QHash<NetworkJob *, Job> jobs;
+    QHash<QString, Job> mpdJobs;
 
 private:
     Thread *thread;
@@ -116,7 +119,7 @@ class CoverLocator : public QObject
     Q_OBJECT
 public:
     CoverLocator();
-    ~CoverLocator() { }
+    ~CoverLocator() override { }
 
     void stop();
 
@@ -149,7 +152,7 @@ class CoverLoader : public QObject
     Q_OBJECT
 public:
     CoverLoader();
-    ~CoverLoader() { }
+    ~CoverLoader() override { }
 
     void stop();
 
@@ -180,21 +183,23 @@ public:
             : img(i)
             , fileName(f) {
         }
-
+        bool validFileName() const;
         QImage img;
         QString fileName;
     };
 
-    static void enableDebug();
+    static void enableDebug(bool verbose);
     static bool debugEnabled();
+    static bool verboseDebugEnabled();
 
     static const QSize constMaxSize;
-    static const QLatin1String constLastFmApiKey;
     static const QLatin1String constCoverDir;
     static const QLatin1String constScaledCoverDir;
     static const QLatin1String constFileName;
     static const QLatin1String constCddaCoverDir;
     static const QLatin1String constArtistImage;
+    static const QLatin1String constComposerImage;
+    static const QString constCoverInTagPrefix;
 
     static Covers * self();
 //    static bool isCoverFile(const QString &file);
@@ -203,18 +208,13 @@ public:
     static const QStringList &standardNames();
     static QString encodeName(QString name);
     static QString albumFileName(const Song &song);
-    static QString artistFileName(const Song &song);
     static QString fixArtist(const QString &artist);
     static bool isJpg(const QByteArray &data);
     static bool isPng(const QByteArray &data);
     static const char * imageFormat(const QByteArray &data);
 
     Covers();
-    #ifdef ENABLE_UBUNTU
-    void setFetchCovers(bool f);
-    #else
     void readConfig();
-    #endif
     void stop();
 
     void clearNameCache();
@@ -223,7 +223,7 @@ public:
     QPixmap * saveScaledCover(const QImage &img, const Song &song, int size);
     // Get cover image of specified size. If this is not found 0 will be returned, and the cover
     // will be downloaded.
-    QPixmap * get(const Song &song, int size);
+    QPixmap * get(const Song &song, int size, bool urgent=false);
     // Get QImage and filename associated with Song request. If this is not found, then the cover
     // will NOT be downloaded. 'emitResult' controls whether 'cover()/artistImage()' is emitted if
     // a cover is found.
@@ -232,7 +232,7 @@ public:
     // will be downloaded. If more than 5 covers have been requested in an event-loop iteration, then
     // the cover requests are placed on a queue.
     Image requestImage(const Song &song, bool urgent=false);
-    void emitCoverUpdated(const Song &song, const QImage &img, const QString &file);
+    void updateCover(const Song &song, const QImage &img, const QString &file);
 
     #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
     void cleanCdda();
@@ -248,23 +248,26 @@ Q_SIGNALS:
     void cover(const Song &song, const QImage &img, const QString &file);
     void coverUpdated(const Song &song, const QImage &img, const QString &file);
     void artistImage(const Song &song, const QImage &img, const QString &file);
-    void coverRetrieved(const Song &song);
+    void composerImage(const Song &song, const QImage &img, const QString &file);
 
 private Q_SLOTS:
     void located(const QList<LocatedCover> &covers);
     void loaded(const QList<LoadedCover> &covers);
     void coverDownloaded(const Song &song, const QImage &img, const QString &file);
     void artistImageDownloaded(const Song &song, const QImage &img, const QString &file);
+    void composerImageDownloaded(const Song &song, const QImage &img, const QString &file);
 
 private:
+    QPixmap * defaultPix(const Song &song, int size, int origSize);
     void tryToLocate(const Song &song);
     void tryToDownload(const Song &song);
     void tryToLoad(const Song &song);
     Image findImage(const Song &song, bool emitResult);
-    void updateCache(const Song &song, const QImage &img, bool dummyEntriesOnly);
+    bool updateCache(const Song &song, const QImage &img, bool dummyEntriesOnly);
     void gotAlbumCover(const Song &song, const QImage &img, const QString &fileName, bool emitResult=true);
     void gotArtistImage(const Song &song, const QImage &img, const QString &fileName, bool emitResult=true);
-    QString getFilename(const Song &s, bool isArtist);
+    void gotComposerImage(const Song &song, const QImage &img, const QString &fileName, bool emitResult=true);
+    QString getFilename(const Song &s);
 
 private:
     QSet<QString> currentImageRequests;
