@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,72 +26,69 @@
 #include "settings.h"
 #include "support/utils.h"
 #include "support/icon.h"
+#include "support/monoicon.h"
 #include "widgets/icons.h"
-#include "mpd/mpduser.h"
-#include <QDir>
-#if QT_VERSION > 0x050000
-#include <QStandardPaths>
-#else
-#include <QDesktopServices>
+#include "models/mpdlibrarymodel.h"
+#ifdef ENABLE_SIMPLE_MPD_SUPPORT
+#include "mpd-interface/mpduser.h"
 #endif
+#ifdef AVAHI_FOUND
+#include "findmpddialog.h"
+#endif
+#include <QDir>
+#include <QStandardPaths>
+#include <QTimer>
 
 enum Pages {
     PAGE_INTRO,
     PAGE_CONNECTION,
-    PAGE_FILES,
+    PAGE_COVERS,
     PAGE_END
 };
 
 InitialSettingsWizard::InitialSettingsWizard(QWidget *p)
     : QWizard(p)
-    , singleUserSupported(MPDUser::self()->isSupported())
 {
     setupUi(this);
     connect(this, SIGNAL(currentIdChanged(int)), SLOT(pageChanged(int)));
-    connect(this, SIGNAL(setDetails(const MPDConnectionDetails &)), MPDConnection::self(), SLOT(setDetails(const MPDConnectionDetails &)));
+    connect(this, SIGNAL(setDetails(MPDConnectionDetails)), MPDConnection::self(), SLOT(setDetails(MPDConnectionDetails)));
     connect(MPDConnection::self(), SIGNAL(stateChanged(bool)), SLOT(mpdConnectionStateChanged(bool)));
-    connect(MPDConnection::self(), SIGNAL(error(const QString &, bool)), SLOT(showError(const QString &, bool)));
+    connect(MPDConnection::self(), SIGNAL(error(QString, bool)), SLOT(showError(QString)));
+    connect(MpdLibraryModel::self(), SIGNAL(error(QString)), SLOT(dbError(QString)));
     connect(connectButton, SIGNAL(clicked(bool)), SLOT(connectToMpd()));
     connect(basicDir, SIGNAL(textChanged(QString)), SLOT(controlNextButton()));
     MPDConnection::self()->start();
-    statusLabel->setText(i18n("Not Connected"));
+    statusLabel->setText(tr("Not Connected"));
 
     MPDConnectionDetails det=Settings::self()->connectionDetails();
     host->setText(det.hostname);
     port->setValue(det.port);
     password->setText(det.password);
     dir->setText(det.dir);
-    #if defined Q_OS_WIN || defined Q_OS_MAC
-    bool showGroupWarning=false;
-    #else
-    bool showGroupWarning=0==Utils::getGroupId();
-    #endif
-    groupWarningLabel->setVisible(showGroupWarning);
-    groupWarningIcon->setVisible(showGroupWarning);
-    int iconSize=Icon::dlgIconSize();
-    groupWarningIcon->setPixmap(Icon("dialog-warning").pixmap(iconSize, iconSize));
-    storeCoversInMpdDir->setChecked(Settings::self()->storeCoversInMpdDir());
-    storeLyricsInMpdDir->setChecked(Settings::self()->storeLyricsInMpdDir());
-    storeBackdropsInMpdDir->setChecked(Settings::self()->storeBackdropsInMpdDir());
-    #ifdef ENABLE_KDE_SUPPORT
-    introPage->setBackground(Icon("cantata"));
-    #else
     introPage->setBackground(Icons::self()->appIcon);
-    #endif
-    connectionPage->setBackground(Icons::self()->audioFileIcon);
-    filesPage->setBackground(Icons::self()->filesIcon);
-    finishedPage->setBackground(Icon("dialog-ok"));
+    connectionPage->setBackground(Icons::self()->audioListIcon);
+    coversPage->setBackground(Icons::self()->albumMonoIcon);
+    finishedPage->setBackground(MonoIcon::icon(FontAwesome::check, Utils::monoIconColor()));
+    fetchCovers->setChecked(Settings::self()->fetchCovers());
+    storeCoversInMpdDir->setChecked(Settings::self()->storeCoversInMpdDir());
 
-    introStack->setCurrentIndex(singleUserSupported ? 1 : 0);
-    basic->setChecked(false); //singleUserSupported);
-    advanced->setChecked(true); // !singleUserSupported);
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
+    introStack->setCurrentIndex(MPDUser::self()->isSupported() ? 1 : 0);
+    basic->setChecked(false);
+    advanced->setChecked(true);
+    #else
+    introStack->setCurrentIndex(0);
+    basic->setChecked(false);
+    advanced->setChecked(true);
+    #endif
 
     #ifndef Q_OS_WIN
-    QSize sz=size();
+    ensurePolished();
+    QSize sz=size()+QSize(64, 32);
     // Adjust size for high-DPI setups...
     bool highDpi=fontMetrics().height()>20;
     if (highDpi) {
-        foreach (int id, pageIds()) {
+        for (int id: pageIds()) {
             QWizardPage *p=QWizard::page(id);
             p->adjustSize();
             QSize ps=p->size();
@@ -110,19 +107,41 @@ InitialSettingsWizard::InitialSettingsWizard(QWidget *p)
     resize(sz);
     setMinimumSize(sz);
     #endif
-    httpNote->setOn(true);
+
+    #ifdef AVAHI_FOUND
+    discoveryButton = new QPushButton(tr("Discover..."), this);
+    hostLayout->insertWidget(hostLayout->count(), discoveryButton);
+    connect(discoveryButton, &QPushButton::clicked, this, &InitialSettingsWizard::detectMPDs);
+    #endif
 }
 
 InitialSettingsWizard::~InitialSettingsWizard()
 {
 }
 
+#ifdef AVAHI_FOUND
+void InitialSettingsWizard::adoptServerSettings(QString ip, QString p)
+{
+    host->setText(ip);
+    port->setValue(p.toInt());
+}
+
+void InitialSettingsWizard::detectMPDs()
+{
+    FindMpdDialog findMpdDlg(this);
+    QObject::connect(&findMpdDlg, &FindMpdDialog::serverChosen, this, &InitialSettingsWizard::adoptServerSettings);
+    findMpdDlg.exec();
+}
+#endif
+
 MPDConnectionDetails InitialSettingsWizard::getDetails()
 {
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     if (basic->isChecked()) {
         MPDUser::self()->setMusicFolder(basicDir->text().trimmed());
         return MPDUser::self()->details(true);
     }
+    #endif
     MPDConnectionDetails det;
     det.hostname=host->text().trimmed();
     det.port=port->value();
@@ -139,28 +158,29 @@ void InitialSettingsWizard::connectToMpd()
 
 void InitialSettingsWizard::mpdConnectionStateChanged(bool c)
 {
-    statusLabel->setText(c ? i18n("Connection Established") : i18n("Connection Failed"));
+    statusLabel->setText(c ? tr("Connection Established") : tr("Connection Failed"));
     if (PAGE_CONNECTION==currentId()) {
         controlNextButton();
     }
 }
 
-void InitialSettingsWizard::showError(const QString &message, bool showActions)
+void InitialSettingsWizard::showError(const QString &message)
 {
-    Q_UNUSED(showActions)
     MessageBox::error(this, message);
 }
 
+void InitialSettingsWizard::dbError(const QString &message)
+{
+    MessageBox::error(this, message+QLatin1String("<br/><br/>")+tr("Cantata will now terminate"));
+    reject();
+}
+
 void InitialSettingsWizard::pageChanged(int p)
 {
     if (PAGE_CONNECTION==p) {
         connectionStack->setCurrentIndex(basic->isChecked() ? 1 : 0);
         if (basic->isChecked() && basicDir->text().isEmpty()) {
-            #if QT_VERSION > 0x050000
             QString dir=QStandardPaths::writableLocation(QStandardPaths::MusicLocation);
-            #else
-            QString dir=QDesktopServices::storageLocation(QDesktopServices::MusicLocation);
-            #endif
             if (dir.isEmpty()) {
                 QString dir=QDir::homePath()+"/Music";
                 dir=dir.replace("//", "/");
@@ -170,17 +190,13 @@ void InitialSettingsWizard::pageChanged(int p)
         controlNextButton();
         return;
     }
-    if (PAGE_FILES==p) {
+    if (PAGE_COVERS==p) {
         if (dir->text().trimmed().startsWith(QLatin1String("http:/"))) {
             storeCoversInMpdDir->setChecked(false);
-            storeLyricsInMpdDir->setChecked(false);
-            storeBackdropsInMpdDir->setChecked(false);
-            httpNote->setVisible(true);
+            fetchCovers->setChecked(true);
         } else {
             storeCoversInMpdDir->setChecked(Settings::self()->storeCoversInMpdDir());
-            storeLyricsInMpdDir->setChecked(Settings::self()->storeLyricsInMpdDir());
-            storeBackdropsInMpdDir->setChecked(Settings::self()->storeBackdropsInMpdDir());
-            httpNote->setVisible(false);
+            fetchCovers->setChecked(Settings::self()->fetchCovers());
         }
     }
     button(NextButton)->setEnabled(PAGE_END!=p);
@@ -212,9 +228,9 @@ void InitialSettingsWizard::controlNextButton()
 void InitialSettingsWizard::accept()
 {
     Settings::self()->saveConnectionDetails(getDetails());
+    Settings::self()->saveFetchCovers(fetchCovers->isChecked());
     Settings::self()->saveStoreCoversInMpdDir(storeCoversInMpdDir->isChecked());
-    Settings::self()->saveStoreLyricsInMpdDir(storeLyricsInMpdDir->isChecked());
-
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     if (basic->isChecked()) {
         Settings::self()->saveCurrentConnection(MPDUser::constName);
         Settings::self()->saveStopOnExit(true);
@@ -222,7 +238,8 @@ void InitialSettingsWizard::accept()
     } else {
         MPDUser::self()->cleanup();
     }
-    Settings::self()->save(true);
+    #endif
+    Settings::self()->save();
     QDialog::accept();
 }
 
@@ -230,6 +247,9 @@ void InitialSettingsWizard::reject()
 {
     // Clear version number - so that wizard is shown next time Cantata is started.
     Settings::self()->clearVersion();
+    ThreadCleaner::self()->stopAll();
+    QTimer::singleShot(0, qApp, SLOT(quit()));
     QDialog::reject();
 }
 
+#include "moc_initialsettingswizard.cpp"
