@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,74 +23,65 @@
 
 #include "interfacesettings.h"
 #include "settings.h"
-#include "widgets/itemview.h"
-#include "support/localize.h"
-#include "models/musiclibraryitemalbum.h"
-#include "models/albumsmodel.h"
+#include "models/sqllibrarymodel.h"
+#include "support/utils.h"
 #include "support/fancytabwidget.h"
+#include "support/pathrequester.h"
 #include "widgets/basicitemdelegate.h"
 #include "widgets/playqueueview.h"
-#include "support/pathrequester.h"
+#include "widgets/itemview.h"
+#include "db/librarydb.h"
 #include <QComboBox>
-#ifndef ENABLE_KDE_SUPPORT
 #include <QDir>
 #include <QMap>
 #include <QRegExp>
 #include <QSet>
-#endif
 #ifdef QT_QTDBUS_FOUND
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
 #endif
-#ifndef ENABLE_KDE_SUPPORT
 #include <QSystemTrayIcon>
-#endif
+#include <QSysInfo>
+#include <QStyleFactory>
 
 #define REMOVE(w) \
     w->setVisible(false); \
     w->deleteLater(); \
     w=0;
 
-static void addAlbumSorts(QComboBox *box)
-{
-    box->addItem(i18n("Album, Artist, Year"), AlbumsModel::Sort_AlbumArtist);
-    box->addItem(i18n("Album, Year, Artist"), AlbumsModel::Sort_AlbumYear);
-    box->addItem(i18n("Artist, Album, Year"), AlbumsModel::Sort_ArtistAlbum);
-    box->addItem(i18n("Artist, Year, Album"), AlbumsModel::Sort_ArtistYear);
-    box->addItem(i18n("Year, Album, Artist"), AlbumsModel::Sort_YearAlbum);
-    box->addItem(i18n("Year, Artist, Album"), AlbumsModel::Sort_YearArtist);
-}
-
-static void addImageSizes(QComboBox *box)
-{
-    box->addItem(i18n("None"), MusicLibraryItemAlbum::CoverNone);
-    box->addItem(i18n("Small"), MusicLibraryItemAlbum::CoverSmall);
-    box->addItem(i18n("Medium"), MusicLibraryItemAlbum::CoverMedium);
-    box->addItem(i18n("Large"), MusicLibraryItemAlbum::CoverLarge);
-    box->addItem(i18n("Extra Large"), MusicLibraryItemAlbum::CoverExtraLarge);
-}
-
 static QString viewTypeString(ItemView::Mode mode)
 {
     switch (mode) {
     default:
-    case ItemView::Mode_BasicTree:    return i18n("Basic Tree (No Icons)");
-    case ItemView::Mode_SimpleTree:   return i18n("Simple Tree");
-    case ItemView::Mode_DetailedTree: return i18n("Detailed Tree");
-    case ItemView::Mode_GroupedTree:  return i18n("Grouped Albums");
-    case ItemView::Mode_List:         return i18n("List");
-    case ItemView::Mode_IconTop:      return i18n("Icon/List");
-    case ItemView::Mode_Table:        return i18n("Table");
+    case ItemView::Mode_GroupedTree:  return QObject::tr("Grouped Albums");
+    case ItemView::Mode_Table:        return QObject::tr("Table");
     }
 }
 
 static void addViewTypes(QComboBox *box, QList<ItemView::Mode> modes)
 {
-    foreach (ItemView::Mode m, modes) {
+    for (ItemView::Mode m: modes) {
         box->addItem(viewTypeString(m), m);
     }
 }
 
+static QString cueSupportString(MPDParseUtils::CueSupport cs)
+{
+    switch (cs) {
+    default:
+    case MPDParseUtils::Cue_Parse:            return QObject::tr("Parse in Library view, and show in Folders view");
+    case MPDParseUtils::Cue_ListButDontParse: return QObject::tr("Only show in Folders view");
+    case MPDParseUtils::Cue_Ignore:           return QObject::tr("Do not list");
+    }
+}
+
+static void addCueSupportTypes(QComboBox *box)
+{
+    for (int i=0; i<MPDParseUtils::Cue_Count; ++i) {
+        box->addItem(cueSupportString((MPDParseUtils::CueSupport)i), i);
+    }
+}
+
 static void selectEntry(QComboBox *box, int v)
 {
     for (int i=1; i<box->count(); ++i) {
@@ -100,118 +91,93 @@ static void selectEntry(QComboBox *box, int v)
         }
     }
 }
-bool enableNotifications(true);
 
 static inline int getValue(QComboBox *box)
 {
     return box->itemData(box->currentIndex()).toInt();
 }
 
+static inline QString getStrValue(QComboBox *box)
+{
+    return box->itemData(box->currentIndex()).toString();
+}
+
 static const char * constValueProperty="value";
+static const char * constSep=",";
+
+class CoverNameValidator : public QValidator
+{
+    public:
+
+    CoverNameValidator(QObject *parent) : QValidator(parent) { }
+
+    State validate(QString &input, int &) const override
+    {
+        for (int i=0; i<input.length(); ++i) {
+            if (!input[i].isLetterOrNumber() && '%'!=input[i] && ' '!=input[i] && '-'!=input[i]) {
+                return Invalid;
+            }
+        }
+
+        return Acceptable;
+    }
+};
 
 InterfaceSettings::InterfaceSettings(QWidget *p)
     : QWidget(p)
-    #ifndef ENABLE_KDE_SUPPORT
-    , loadedLangs(false)
-    #endif
+    , loaded(false)
 {
+    bool mprisSettings=false;
+    bool enableTrayItem=Utils::useSystemTray();
+    #ifdef Q_OS_MAC
+    // OSX always displays an entry in the taskbar - and the tray seems to confuse things.
+    bool enableNotifications=QSysInfo::MacintoshVersion >= QSysInfo::MV_10_8;
+    #else
     #ifdef QT_QTDBUS_FOUND
     // We have dbus, check that org.freedesktop.Notifications exists
     bool enableNotifications=QDBusConnection::sessionBus().interface()->isServiceRegistered("org.freedesktop.Notifications");
-    #else
+    mprisSettings=true;
+    #else // QT_QTDBUS_FOUND
     bool enableNotifications=true;
-    #endif
-    bool enableTrayItem=true;
-    #ifndef ENABLE_KDE_SUPPORT
-    if (!QSystemTrayIcon::isSystemTrayAvailable()) {
-        enableTrayItem=false;
-        #ifndef QT_QTDBUS_FOUND
-        enableNotifications=false;
-        #endif
-    }
-    #endif
+    #endif // QT_QTDBUS_FOUND
+    #endif // Q_MAC_OS
 
     setupUi(this);
-    addImageSizes(libraryCoverSize);
-    QList<ItemView::Mode> standardViews=QList<ItemView::Mode>() << ItemView::Mode_BasicTree << ItemView::Mode_SimpleTree
-                                                                << ItemView::Mode_DetailedTree << ItemView::Mode_List;
-    addViewTypes(libraryView, QList<ItemView::Mode>() << standardViews << ItemView::Mode_IconTop);
-    addViewTypes(albumsView, QList<ItemView::Mode>() << standardViews << ItemView::Mode_IconTop);
-    addViewTypes(folderView, standardViews);
-    addViewTypes(playlistsView, QList<ItemView::Mode>() << standardViews << ItemView::Mode_GroupedTree << ItemView::Mode_Table);
-    addViewTypes(searchView, QList<ItemView::Mode>() << ItemView::Mode_List << ItemView::Mode_Table);
+    addCueSupportTypes(cueSupport);
     addViewTypes(playQueueView, QList<ItemView::Mode>() << ItemView::Mode_GroupedTree << ItemView::Mode_Table);
+    yearTag->addItem(tr("Use 'Year' tag to display & sort"));
+    yearTag->addItem(tr("Use 'Original Year' tag to display & sort, fallback to 'Year' if not set"));
 
-    addImageSizes(albumsCoverSize);
-    addAlbumSorts(albumSort);
-
-    addView(i18n("Play Queue"), QLatin1String("PlayQueuePage"));
-    addView(i18n("Artists"), QLatin1String("LibraryPage"));
-    addView(i18n("Albums"), QLatin1String("AlbumsPage"));
-    addView(i18n("Folders"), QLatin1String("FolderPage"));
-    addView(i18n("Playlists"), QLatin1String("PlaylistsPage"));
-    #ifdef ENABLE_DYNAMIC
-    addView(i18n("Dynamic Playlists"), QLatin1String("DynamicPage"));
-    #endif
-    #ifdef ENABLE_STREAMS
-    addViewTypes(streamsView, standardViews);
-    addView(i18n("Streams (e.g. Radio Stations)"), QLatin1String("StreamsPage"));
-    #else
-    REMOVE(streamsView)
-    REMOVE(streamsViewLabel)
-    #endif
-    #ifdef ENABLE_ONLINE_SERVICES
-    addViewTypes(onlineView, standardViews);
-    addView(i18n("Online Services - Jamendo, Maganatune, SoundCloud, and Podcasts"), QLatin1String("OnlineServicesPage"));
-    #else
-    REMOVE(onlineView)
-    REMOVE(onlineViewLabel)
-    #endif
+    addView(tr("Play Queue"), QLatin1String("PlayQueuePage"));
+    addView(tr("Library"), QLatin1String("LibraryPage"));
+    addView(tr("Folders"), QLatin1String("FolderPage"));
+    addView(tr("Playlists"), QLatin1String("PlaylistsPage"));
+    addView(tr("Internet - Streams, Jamendo, Maganatune, SoundCloud, and Podcasts"), QLatin1String("OnlineServicesPage"));
     #ifdef ENABLE_DEVICES_SUPPORT
-    addViewTypes(devicesView, standardViews);
-    addView(i18n("Devices - UMS, MTP (e.g. Android), and AudioCDs"), QLatin1String("DevicesPage"));
+    addView(tr("Devices - UMS, MTP (e.g. Android), and AudioCDs"), QLatin1String("DevicesPage"));
     #else
-    REMOVE(devicesView)
-    REMOVE(devicesViewLabel)
     REMOVE(showDeleteAction)
     #endif
-    addView(i18n("Search (via MPD)"), QLatin1String("SearchPage"));
-    addView(i18n("Info - Current song information (artist, album, and lyrics)"), QLatin1String("ContextPage"));
-
-    connect(libraryView, SIGNAL(currentIndexChanged(int)), SLOT(libraryViewChanged()));
-    connect(libraryCoverSize, SIGNAL(currentIndexChanged(int)), SLOT(libraryCoverSizeChanged()));
-    connect(albumsView, SIGNAL(currentIndexChanged(int)), SLOT(albumsViewChanged()));
-    connect(albumsCoverSize, SIGNAL(currentIndexChanged(int)), SLOT(albumsCoverSizeChanged()));
-    connect(playlistsView, SIGNAL(currentIndexChanged(int)), SLOT(playlistsViewChanged()));
+    addView(tr("Search (via MPD)"), QLatin1String("SearchPage"));
+    addView(tr("Info - Current song information (artist, album, and lyrics)"), QLatin1String("ContextPage"));
     connect(playQueueView, SIGNAL(currentIndexChanged(int)), SLOT(playQueueViewChanged()));
     connect(forceSingleClick, SIGNAL(toggled(bool)), SLOT(forceSingleClickChanged()));
-    connect(touchFriendly, SIGNAL(toggled(bool)), SLOT(touchFriendlyChanged()));
     connect(views, SIGNAL(itemChanged(QListWidgetItem*)), SLOT(viewItemChanged(QListWidgetItem*)));
 
-    #ifdef ENABLE_KDE_SUPPORT
-    REMOVE(lang)
-    REMOVE(langLabel)
-    REMOVE(langNoteLabel)
-    #endif
-
-    sbStyle->addItem(i18n("Large"), FancyTabWidget::Large);
-    sbStyle->addItem(i18n("Small"), FancyTabWidget::Small);
-    sbStyle->addItem(i18n("Tab-bar"), FancyTabWidget::Tab);
-    sbPosition->addItem(Qt::LeftToRight==layoutDirection() ? i18n("Left") : i18n("Right"), FancyTabWidget::Side);
-    sbPosition->addItem(i18n("Top"), FancyTabWidget::Top);
-    sbPosition->addItem(i18n("Bottom"), FancyTabWidget::Bot);
+    sbStyle->addItem(tr("Large"), FancyTabWidget::Large);
+    sbStyle->addItem(tr("Small"), FancyTabWidget::Small);
+    sbStyle->addItem(tr("Tab-bar"), FancyTabWidget::Tab);
+    sbPosition->addItem(Qt::LeftToRight==layoutDirection() ? tr("Left") : tr("Right"), FancyTabWidget::Side);
+    sbPosition->addItem(tr("Top"), FancyTabWidget::Top);
+    sbPosition->addItem(tr("Bottom"), FancyTabWidget::Bot);
     connect(sbAutoHide, SIGNAL(toggled(bool)), SLOT(sbAutoHideChanged()));
     views->setItemDelegate(new BasicItemDelegate(views));
     playQueueBackground_none->setProperty(constValueProperty, PlayQueueView::BI_None);
     playQueueBackground_cover->setProperty(constValueProperty, PlayQueueView::BI_Cover);
     playQueueBackground_custom->setProperty(constValueProperty, PlayQueueView::BI_Custom);
     playQueueBackgroundFile->setDirMode(false);
-    #ifdef ENABLE_KDE_SUPPORT
-    playQueueBackgroundFile->setFilter("image/jpeg image/png");
-    #else
-    playQueueBackgroundFile->setFilter(i18n("Images (*.png *.jpg)"));
-    #endif
-    int labelWidth=qMax(fontMetrics().width(QLatin1String("100%")), fontMetrics().width(i18nc("pixels", "10px")));
+    playQueueBackgroundFile->setFilter(tr("Images (*.png *.jpg)"));
+    int labelWidth=qMax(fontMetrics().width(QLatin1String("100%")), fontMetrics().width(tr("10px", "pixels")));
     playQueueBackgroundOpacityLabel->setFixedWidth(labelWidth);
     playQueueBackgroundBlurLabel->setFixedWidth(labelWidth);
     connect(playQueueBackgroundOpacity, SIGNAL(valueChanged(int)), SLOT(setPlayQueueBackgroundOpacityLabel()));
@@ -219,7 +185,7 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
     connect(playQueueBackground_none, SIGNAL(toggled(bool)), SLOT(enablePlayQueueBackgroundOptions()));
     connect(playQueueBackground_cover, SIGNAL(toggled(bool)), SLOT(enablePlayQueueBackgroundOptions()));
     connect(playQueueBackground_custom, SIGNAL(toggled(bool)), SLOT(enablePlayQueueBackgroundOptions()));
-
+    connect(storeCoversInMpdDir, SIGNAL(toggled(bool)), this, SLOT(storeCoversInMpdDirToggled()));
     if (!enableNotifications) {
         REMOVE(systemTrayPopup)
     }
@@ -233,8 +199,13 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
         REMOVE(startupState)
     }
 
-    if (!enableNotifications && !enableTrayItem) {
-        tabWidget->removeTab(6);
+    if (!enableNotifications && !enableTrayItem && !mprisSettings) {
+        tabWidget->removeTab(3);
+    } else if (!enableTrayItem && enableNotifications && !mprisSettings) {
+        tabWidget->setTabText(3, tr("Notifications"));
+    }
+    if (!mprisSettings) {
+        REMOVE(enableMpris)
     }
     #if defined Q_OS_WIN || defined Q_OS_MAC || !defined QT_QTDBUS_FOUND
     if (systemTrayPopup && systemTrayCheckBox) {
@@ -242,34 +213,35 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
         connect(systemTrayPopup, SIGNAL(toggled(bool)), SLOT(systemTrayPopupToggled()));
     }
     #endif
+    coverFilenameLabel->setToolTip(coverFilename->toolTip());
+    coverFilename->setValidator(new CoverNameValidator(this));
+    #ifdef ENABLE_DEVICES_SUPPORT
+    coverNameNoteLabel->setText(tr("If no setting is specified for 'Filename', then Cantata will use a default of "
+                                   "<code>cover</code>. This filename is used when downloading covers, or when adding "
+                                   "music to your library from devices."));
+    #else
+    coverNameNoteLabel->setText(tr("If no setting is specified for 'Filename', then Cantata will use a default of "
+                                   "<code>cover</code>. This filename is used when downloading covers."));
+    #endif
+
+    QFont f = Utils::smallFont(ignorePrefixesLabel->font());
+    f.setItalic(true);
+    ignorePrefixesLabel->setFont(f);
+    composerGenresLabel->setFont(f);
+    singleTracksFoldersLabel->setFont(f);
+    setMinimumSize(Utils::scaleForDpi(720), Utils::scaleForDpi(620));
 }
 
 void InterfaceSettings::load()
 {
-    libraryArtistImage->setChecked(Settings::self()->libraryArtistImage());
-    selectEntry(libraryView, Settings::self()->libraryView());
-    libraryCoverSize->setCurrentIndex(Settings::self()->libraryCoverSize());
-    libraryYear->setChecked(Settings::self()->libraryYear());
-    selectEntry(albumsView, Settings::self()->albumsView());
-    albumsCoverSize->setCurrentIndex(Settings::self()->albumsCoverSize());
-    selectEntry(albumSort, Settings::self()->albumSort());
-    selectEntry(folderView, Settings::self()->folderView());
-    selectEntry(playlistsView, Settings::self()->playlistsView());
-    playListsStartClosed->setChecked(Settings::self()->playListsStartClosed());
-    #ifdef ENABLE_STREAMS
-    selectEntry(streamsView, Settings::self()->streamsView());
-    #endif
-    #ifdef ENABLE_ONLINE_SERVICES
-    selectEntry(onlineView, Settings::self()->onlineView());
-    #endif
-    groupSingle->setChecked(Settings::self()->groupSingle());
-    useComposer->setChecked(Settings::self()->useComposer());
-    filteredOnly->setChecked(Settings::self()->filteredOnly());
+    ignorePrefixes->setText(QStringList(Settings::self()->ignorePrefixes().toList()).join(QString(constSep)));
+    composerGenres->setText(QStringList(Settings::self()->composerGenres().toList()).join(QString(constSep)));
+    singleTracksFolders->setText(QStringList(Settings::self()->singleTracksFolders().toList()).join(QString(constSep)));
+    selectEntry(cueSupport, Settings::self()->cueSupport());
+    yearTag->setCurrentIndex(Settings::self()->useOriginalYear() ? 1 : 0);
     #ifdef ENABLE_DEVICES_SUPPORT
     showDeleteAction->setChecked(Settings::self()->showDeleteAction());
-    selectEntry(devicesView, Settings::self()->devicesView());
     #endif
-    selectEntry(searchView, Settings::self()->searchView());
 
     selectEntry(playQueueView, Settings::self()->playQueueView());
     playQueueAutoExpand->setChecked(Settings::self()->playQueueAutoExpand());
@@ -282,17 +254,17 @@ void InterfaceSettings::load()
     playQueueBackground_custom->setChecked(pqBgnd==playQueueBackground_custom->property(constValueProperty).toInt());
     playQueueBackgroundOpacity->setValue(Settings::self()->playQueueBackgroundOpacity());
     playQueueBackgroundBlur->setValue(Settings::self()->playQueueBackgroundBlur());
-    playQueueBackgroundFile->setText(Settings::self()->playQueueBackgroundFile());
+    playQueueBackgroundFile->setText(Utils::convertPathForDisplay(Settings::self()->playQueueBackgroundFile(), false));
 
     playQueueConfirmClear->setChecked(Settings::self()->playQueueConfirmClear());
-    albumsViewChanged();
-    albumsCoverSizeChanged();
-    playlistsViewChanged();
+    playQueueSearch->setChecked(Settings::self()->playQueueSearch());
     playQueueViewChanged();
     forceSingleClick->setChecked(Settings::self()->forceSingleClick());
-    touchFriendly->setChecked(Settings::self()->touchFriendly());
+    infoTooltips->setChecked(Settings::self()->infoTooltips());
     showStopButton->setChecked(Settings::self()->showStopButton());
     showCoverWidget->setChecked(Settings::self()->showCoverWidget());
+    showRatingWidget->setChecked(Settings::self()->showRatingWidget());
+    showTechnicalInfo->setChecked(Settings::self()->showTechnicalInfo());
     if (systemTrayCheckBox) {
         systemTrayCheckBox->setChecked(Settings::self()->useSystemTray());
         if (minimiseOnClose) {
@@ -312,9 +284,9 @@ void InterfaceSettings::load()
     if (systemTrayPopup) {
         systemTrayPopup->setChecked(Settings::self()->showPopups());
     }
-
-    cacheScaledCovers->setChecked(Settings::self()->cacheScaledCovers());
     fetchCovers->setChecked(Settings::self()->fetchCovers());
+    storeCoversInMpdDir->setChecked(Settings::self()->storeCoversInMpdDir());
+    coverFilename->setText(Settings::self()->coverFilename());
 
     QStringList hiddenPages=Settings::self()->hiddenPages();
     for (int i=0; i<views->count(); ++i) {
@@ -325,42 +297,38 @@ void InterfaceSettings::load()
     selectEntry(sbStyle, sidebar&FancyTabWidget::Style_Mask);
     selectEntry(sbPosition, sidebar&FancyTabWidget::Position_Mask);
     sbIconsOnly->setChecked(sidebar&FancyTabWidget::IconOnly);
-    sbMonoIcons->setChecked(Settings::self()->monoSidebarIcons());
     sbAutoHide->setChecked(Settings::self()->splitterAutoHide());
     sbAutoHideChanged();
+    responsiveSidebar->setChecked(Settings::self()->responsiveSidebar());
     viewItemChanged(views->item(0));
     setPlayQueueBackgroundOpacityLabel();
     setPlayQueueBackgroundBlurLabel();
     enablePlayQueueBackgroundOptions();
-    libraryViewChanged();
+    if (enableMpris) {
+        enableMpris->setChecked(Settings::self()->mpris());
+    }
+}
+
+static QSet<QString> toSet(const QString &str)
+{
+    QStringList parts=str.split(constSep, QString::SkipEmptyParts);
+    QSet<QString> set;
+    for (QString s: parts) {
+        set.insert(s.trimmed());
+    }
+    return set;
 }
 
 void InterfaceSettings::save()
 {
-    Settings::self()->saveLibraryArtistImage(libraryArtistImage->isChecked());
-    Settings::self()->saveLibraryView(getValue(libraryView));
-    Settings::self()->saveLibraryCoverSize(libraryCoverSize->currentIndex());
-    Settings::self()->saveLibraryYear(libraryYear->isChecked());
-    Settings::self()->saveAlbumsView(getValue(albumsView));
-    Settings::self()->saveAlbumsCoverSize(albumsCoverSize->currentIndex());
-    Settings::self()->saveAlbumSort(getValue(albumSort));
-    Settings::self()->saveFolderView(getValue(folderView));
-    Settings::self()->savePlaylistsView(getValue(playlistsView));
-    Settings::self()->savePlayListsStartClosed(playListsStartClosed->isChecked());
-    #ifdef ENABLE_STREAMS
-    Settings::self()->saveStreamsView(getValue(streamsView));
-    #endif
-    #ifdef ENABLE_ONLINE_SERVICES
-    Settings::self()->saveOnlineView(getValue(onlineView));
-    #endif
-    Settings::self()->saveGroupSingle(groupSingle->isChecked());
-    Settings::self()->saveUseComposer(useComposer->isChecked());
-    Settings::self()->saveFilteredOnly(filteredOnly->isChecked());
+    Settings::self()->saveIgnorePrefixes(toSet(ignorePrefixes->text()));
+    Settings::self()->saveComposerGenres(toSet(composerGenres->text()));
+    Settings::self()->saveSingleTracksFolders(toSet(singleTracksFolders->text()));
+    Settings::self()->saveCueSupport((MPDParseUtils::CueSupport)(cueSupport->itemData(cueSupport->currentIndex()).toInt()));
+    Settings::self()->saveUseOriginalYear(1==yearTag->currentIndex());
     #ifdef ENABLE_DEVICES_SUPPORT
     Settings::self()->saveShowDeleteAction(showDeleteAction->isChecked());
-    Settings::self()->saveDevicesView(getValue(devicesView));
     #endif
-    Settings::self()->saveSearchView(getValue(searchView));
     Settings::self()->savePlayQueueView(getValue(playQueueView));
     Settings::self()->savePlayQueueAutoExpand(playQueueAutoExpand->isChecked());
     Settings::self()->savePlayQueueStartClosed(playQueueStartClosed->isChecked());
@@ -375,13 +343,16 @@ void InterfaceSettings::save()
     }
     Settings::self()->savePlayQueueBackgroundOpacity(playQueueBackgroundOpacity->value());
     Settings::self()->savePlayQueueBackgroundBlur(playQueueBackgroundBlur->value());
-    Settings::self()->savePlayQueueBackgroundFile(playQueueBackgroundFile->text().trimmed());
+    Settings::self()->savePlayQueueBackgroundFile(Utils::convertPathFromDisplay(playQueueBackgroundFile->text(), false));
 
     Settings::self()->savePlayQueueConfirmClear(playQueueConfirmClear->isChecked());
+    Settings::self()->savePlayQueueSearch(playQueueSearch->isChecked());
     Settings::self()->saveForceSingleClick(forceSingleClick->isChecked());
-    Settings::self()->saveTouchFriendly(touchFriendly->isChecked());
+    Settings::self()->saveInfoTooltips(infoTooltips->isChecked());
     Settings::self()->saveShowStopButton(showStopButton->isChecked());
     Settings::self()->saveShowCoverWidget(showCoverWidget->isChecked());
+    Settings::self()->saveShowRatingWidget(showRatingWidget->isChecked());
+    Settings::self()->saveShowTechnicalInfo(showTechnicalInfo->isChecked());
     Settings::self()->saveUseSystemTray(systemTrayCheckBox && systemTrayCheckBox->isChecked());
     Settings::self()->saveShowPopups(systemTrayPopup && systemTrayPopup->isChecked());
     Settings::self()->saveMinimiseOnClose(minimiseOnClose && minimiseOnClose->isChecked());
@@ -392,14 +363,15 @@ void InterfaceSettings::save()
     } else if (startupStateRestore->isChecked()) {
         Settings::self()->saveStartupState(Settings::SS_Previous);
     }
-    Settings::self()->saveCacheScaledCovers(cacheScaledCovers->isChecked());
     Settings::self()->saveFetchCovers(fetchCovers->isChecked());
-    #ifndef ENABLE_KDE_SUPPORT
-    if (loadedLangs && lang) {
+    Settings::self()->saveStoreCoversInMpdDir(storeCoversInMpdDir->isChecked());
+    Settings::self()->saveCoverFilename(coverFilename->text().trimmed());
+    if (loaded && lang) {
         Settings::self()->saveLang(lang->itemData(lang->currentIndex()).toString());
     }
-    #endif
-
+    if (loaded && styleOption) {
+        Settings::self()->saveStyle(0==styleOption->currentIndex() ? QString() : styleOption->currentText());
+    }
     QStringList hiddenPages;
     for (int i=0; i<views->count(); ++i) {
         QListWidgetItem *v=views->item(i);
@@ -413,11 +385,13 @@ void InterfaceSettings::save()
         sidebar|=FancyTabWidget::IconOnly;
     }
     Settings::self()->saveSidebar(sidebar);
-    Settings::self()->saveMonoSidebarIcons(sbMonoIcons->isChecked());
     Settings::self()->saveSplitterAutoHide(sbAutoHide->isChecked());
+    Settings::self()->saveResponsiveSidebar(responsiveSidebar->isChecked());
+    if (enableMpris) {
+        Settings::self()->saveMpris(enableMpris->isChecked());
+    }
 }
 
-#ifndef ENABLE_KDE_SUPPORT
 static bool localeAwareCompare(const QString &a, const QString &b)
 {
     return a.localeAwareCompare(b) < 0;
@@ -429,7 +403,7 @@ static QSet<QString> translationCodes(const QString &dir)
     QDir d(dir);
     QStringList installed(d.entryList(QStringList() << "*.qm"));
     QRegExp langRegExp("^cantata_(.*).qm$");
-    foreach (const QString &filename, installed) {
+    for (const QString &filename: installed) {
         if (langRegExp.exactMatch(filename)) {
             codes.insert(langRegExp.cap(1));
         }
@@ -439,8 +413,8 @@ static QSet<QString> translationCodes(const QString &dir)
 
 void InterfaceSettings::showEvent(QShowEvent *e)
 {
-    if (!loadedLangs) {
-        loadedLangs=true;
+    if (!loaded) {
+        loaded=true;
 
         QMap<QString, QString> langMap;
         QSet<QString> transCodes;
@@ -451,25 +425,23 @@ void InterfaceSettings::showEvent(QShowEvent *e)
         transCodes+=translationCodes(CANTATA_SYS_TRANS_DIR);
         #endif
 
-        foreach (const QString &code, transCodes) {
+        for (const QString &code: transCodes) {
             QString langName = QLocale::languageToString(QLocale(code).language());
-            #if QT_VERSION >= 0x040800
             QString nativeName = QLocale(code).nativeLanguageName();
             if (!nativeName.isEmpty()) {
                 langName = nativeName;
             }
-            #endif
             langMap[QString("%1 (%2)").arg(langName, code)] = code;
         }
 
-        langMap["English (en)"] = "en";
+        langMap[tr("English (en)")] = "en";
 
         QString current = Settings::self()->lang();
         QStringList names = langMap.keys();
         qStableSort(names.begin(), names.end(), localeAwareCompare);
-        lang->addItem(i18n("System default"), QString());
+        lang->addItem(tr("System default"), QString());
         lang->setCurrentIndex(0);
-        foreach (const QString &name, names) {
+        for (const QString &name: names) {
             lang->addItem(name, langMap[name]);
             if (langMap[name]==current) {
                 lang->setCurrentIndex(lang->count()-1);
@@ -483,10 +455,30 @@ void InterfaceSettings::showEvent(QShowEvent *e)
         } else {
             connect(lang, SIGNAL(currentIndexChanged(int)), SLOT(langChanged()));
         }
+
+        styleOption->addItem(tr("System default"));
+        styleOption->addItems(QStyleFactory::keys());
+
+        if (styleOption->count()<3) {
+            REMOVE(styleOption)
+            REMOVE(styleLabel)
+            REMOVE(styleNoteLabel)
+        } else {
+            QString selected = Settings::self()->style();
+            styleOption->setCurrentIndex(0);
+            if (!selected.isEmpty()) {
+                for (int i=1; i<styleOption->count(); ++i) {
+                    if (styleOption->itemText(i) == selected) {
+                        styleOption->setCurrentIndex(i);
+                        break;
+                    }
+                }
+            }
+            connect(styleOption, SIGNAL(currentIndexChanged(int)), SLOT(styleChanged()));
+        }
     }
     QWidget::showEvent(e);
 }
-#endif
 
 void InterfaceSettings::showPage(const QString &page)
 {
@@ -495,6 +487,16 @@ void InterfaceSettings::showPage(const QString &page)
     }
 }
 
+QSize InterfaceSettings::sizeHint() const
+{
+    QSize sz=QWidget::sizeHint();
+    #ifdef Q_OS_MAC
+    sz.setWidth(sz.width()+32);
+    sz.setHeight(qMin(sz.height(), 500));
+    #endif
+    return sz;
+}
+
 void InterfaceSettings::addView(const QString &v, const QString &prop)
 {
     QListWidgetItem *item=new QListWidgetItem(v, views);
@@ -502,47 +504,6 @@ void InterfaceSettings::addView(const QString &v, const QString &prop)
     item->setData(Qt::UserRole, prop);
 }
 
-void InterfaceSettings::libraryViewChanged()
-{
-    int vt=getValue(libraryView);
-    if (ItemView::Mode_IconTop==vt && 0==libraryCoverSize->currentIndex()) {
-        libraryCoverSize->setCurrentIndex(2);
-    }
-
-    bool isIcon=ItemView::Mode_IconTop==vt;
-    bool isSimpleTree=ItemView::Mode_SimpleTree==vt || ItemView::Mode_BasicTree==vt;
-    libraryArtistImage->setEnabled(!isIcon && !isSimpleTree);
-    if (isIcon) {
-        libraryArtistImage->setChecked(true);
-    } else if (isSimpleTree) {
-        libraryArtistImage->setChecked(false);
-    }
-}
-
-void InterfaceSettings::libraryCoverSizeChanged()
-{
-    if (ItemView::Mode_IconTop==getValue(libraryView) && 0==libraryCoverSize->currentIndex()) {
-        libraryView->setCurrentIndex(1);
-    }
-    if (0==libraryCoverSize->currentIndex()) {
-        libraryArtistImage->setChecked(false);
-    }
-}
-
-void InterfaceSettings::albumsViewChanged()
-{
-    if (ItemView::Mode_IconTop==getValue(albumsView) && 0==albumsCoverSize->currentIndex()) {
-        albumsCoverSize->setCurrentIndex(2);
-    }
-}
-
-void InterfaceSettings::albumsCoverSizeChanged()
-{
-    if (ItemView::Mode_IconTop==getValue(albumsView) && 0==albumsCoverSize->currentIndex()) {
-        albumsView->setCurrentIndex(1);
-    }
-}
-
 void InterfaceSettings::playQueueViewChanged()
 {
     bool grouped=ItemView::Mode_GroupedTree==getValue(playQueueView);
@@ -550,22 +511,11 @@ void InterfaceSettings::playQueueViewChanged()
     playQueueStartClosed->setEnabled(grouped);
 }
 
-void InterfaceSettings::playlistsViewChanged()
-{
-    bool grouped=ItemView::Mode_GroupedTree==getValue(playlistsView);
-    playListsStartClosed->setEnabled(grouped);
-}
-
 void InterfaceSettings::forceSingleClickChanged()
 {
     singleClickLabel->setOn(forceSingleClick->isChecked()!=Settings::self()->forceSingleClick());
 }
 
-void InterfaceSettings::touchFriendlyChanged()
-{
-    touchFriendlyNoteLabel->setOn(touchFriendly->isChecked()!=Settings::self()->touchFriendly());
-}
-
 void InterfaceSettings::enableStartupState()
 {
     if (systemTrayCheckBox && minimiseOnClose && startupState) {
@@ -575,9 +525,13 @@ void InterfaceSettings::enableStartupState()
 
 void InterfaceSettings::langChanged()
 {
-    #ifndef ENABLE_KDE_SUPPORT
     langNoteLabel->setOn(lang->itemData(lang->currentIndex()).toString()!=Settings::self()->lang());
-    #endif
+}
+
+void InterfaceSettings::styleChanged()
+{
+    QString st = 0==styleOption->currentIndex() ? QString() : styleOption->currentText();
+    styleNoteLabel->setOn(st!=Settings::self()->style());
 }
 
 void InterfaceSettings::viewItemChanged(QListWidgetItem *changedItem)
@@ -608,12 +562,12 @@ void InterfaceSettings::sbAutoHideChanged()
 
 void InterfaceSettings::setPlayQueueBackgroundOpacityLabel()
 {
-    playQueueBackgroundOpacityLabel->setText(i18nc("value%", "%1%", playQueueBackgroundOpacity->value()));
+    playQueueBackgroundOpacityLabel->setText(tr("%1%", "value%").arg(playQueueBackgroundOpacity->value()));
 }
 
 void InterfaceSettings::setPlayQueueBackgroundBlurLabel()
 {
-    playQueueBackgroundBlurLabel->setText(i18nc("pixels", "%1px", playQueueBackgroundBlur->value()));
+    playQueueBackgroundBlurLabel->setText(tr("%1 px", "pixels").arg(playQueueBackgroundBlur->value()));
 }
 
 void InterfaceSettings::enablePlayQueueBackgroundOptions()
@@ -637,3 +591,10 @@ void InterfaceSettings::systemTrayPopupToggled()
         systemTrayCheckBox->setChecked(true);
     }
 }
+
+void InterfaceSettings::storeCoversInMpdDirToggled()
+{
+    storeCoversInMpdDirLabel->setOn(storeCoversInMpdDir->isChecked() && storeCoversInMpdDir->isChecked()!=Settings::self()->storeCoversInMpdDir());
+}
+
+#include "moc_interfacesettings.cpp"
