@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -34,30 +34,24 @@ class InterfaceSettings : public QWidget, private Ui::InterfaceSettings
 
 public:
     InterfaceSettings(QWidget *p);
-    virtual ~InterfaceSettings() { }
+    ~InterfaceSettings() override { }
 
     void load();
     void save();
 
-    #ifndef ENABLE_KDE_SUPPORT
-    void showEvent(QShowEvent *e);
-    #endif
+    void showEvent(QShowEvent *e) override;
     void showPage(const QString &page);
+    QSize sizeHint() const override;
 
 private:
     void addView(const QString &v, const QString &prop);
 
 private Q_SLOTS:
-    void libraryViewChanged();
-    void libraryCoverSizeChanged();
-    void albumsViewChanged();
-    void albumsCoverSizeChanged();
-    void playlistsViewChanged();
     void playQueueViewChanged();
     void forceSingleClickChanged();
-    void touchFriendlyChanged();
     void enableStartupState();
     void langChanged();
+    void styleChanged();
     void viewItemChanged(QListWidgetItem *changedItem);
     void sbAutoHideChanged();
     void setPlayQueueBackgroundOpacityLabel();
@@ -65,11 +59,10 @@ private Q_SLOTS:
     void enablePlayQueueBackgroundOptions();
     void systemTrayCheckBoxToggled();
     void systemTrayPopupToggled();
+    void storeCoversInMpdDirToggled();
 
 private:
-    #ifndef ENABLE_KDE_SUPPORT
-    bool loadedLangs;
-    #endif
+    bool loaded;
 };
 
 #endif
