@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,60 +24,61 @@
 #ifndef LIBRARYPAGE_H
 #define LIBRARYPAGE_H
 
-#include "ui_librarypage.h"
-#include "models/musiclibraryproxymodel.h"
-#include "page.h"
+#include "widgets/singlepagewidget.h"
 
 class Action;
+class GenreCombo;
 
-class LibraryPage : public QWidget, public Ui::LibraryPage, public Page
+class LibraryPage : public SinglePageWidget
 {
     Q_OBJECT
 
 public:
     LibraryPage(QWidget *p);
-    virtual ~LibraryPage();
+    ~LibraryPage() override;
 
-    void refresh();
     void clear();
-    QStringList selectedFiles(bool allowPlaylists=false, bool randomAlbums=false) const;
-    QList<Song> selectedSongs(bool allowPlaylists=false) const;
-    Song coverRequest() const;
-    void addSelectionToPlaylist(const QString &name=QString(), bool replace=false, quint8 priorty=0, bool randomAlbums=false);
+    QStringList selectedFiles(bool allowPlaylists=false) const override;
+    QList<Song> selectedSongs(bool allowPlaylists=false) const override;
+    Song coverRequest() const override;
     #ifdef ENABLE_DEVICES_SUPPORT
-    void addSelectionToDevice(const QString &udi);
-    void deleteSongs();
+    void addSelectionToDevice(const QString &udi) override;
+    void deleteSongs() override;
     #endif
-    void setView(int v);
-    ItemView::Mode viewMode() const { return view->viewMode(); }
     void showSongs(const QList<Song> &songs);
     void showArtist(const QString &artist);
     void showAlbum(const QString &artist, const QString &album);
-    void focusSearch() { view->focusSearch(); }
-    void showEvent(QShowEvent *e);
 
 private:
     void setItemSize(int v);
 
 Q_SIGNALS:
-    // These are for communicating with MPD object (which is in its own thread, so need to talk via signal/slots)
-    void add(const QStringList &files, bool replace, quint8 priorty);
-    void addSongsToPlaylist(const QString &name, const QStringList &files);
-    void loadLibrary();
-
     void addToDevice(const QString &from, const QString &to, const QList<Song> &songs);
     void deleteSongs(const QString &from, const QList<Song> &songs);
 
 public Q_SLOTS:
     void itemDoubleClicked(const QModelIndex &);
-    void searchItems();
-    void controlActions();
-    void updateGenres(const QModelIndex &);
-    void showSpinner();
-    void hideSpinner();
+
+private Q_SLOTS:
+    void modelReset();
+    void groupByChanged();
+    void libraryAlbumSortChanged();
+    void albumAlbumSortChanged();
+    void showArtistImagesChanged(bool u);
+    void updateToPlayQueue(const QModelIndex &idx, bool replace);
+    void addRandomAlbum();
 
 private:
-    MusicLibraryProxyModel proxy;
+    void setView(int v) override;
+    void doSearch() override;
+    void controlActions() override;
+
+private:
+    GenreCombo *genreCombo;
+    QAction *viewAction;
+    QAction *showArtistImagesAction;
+    QAction *libraryAlbumSortAction;
+    QAction *albumAlbumSortAction;
 };
 
 #endif
