@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -28,20 +28,15 @@
 #define MAIN_WINDOW_H
 
 #include <qglobal.h>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KXmlGuiWindow>
-#define MAIN_WINDOW_BASE_CLASS KXmlGuiWindow
-class KToggleAction;
-#else
 #include <QMainWindow>
-#define MAIN_WINDOW_BASE_CLASS QMainWindow
-#endif
 #include <QToolButton>
 #include <QStringList>
 #include "ui_mainwindow.h"
-#include "models/playqueuemodel.h"
 #include "models/playqueueproxymodel.h"
-#include "mpd/song.h"
+#include "models/playqueuemodel.h"
+#include "mpd-interface/song.h"
+#include "mpd-interface/mpdstatus.h"
+#include "mpd-interface/mpdconnection.h"
 #include "config.h"
 
 class Action;
@@ -49,18 +44,9 @@ class ActionCollection;
 class MainWindow;
 class Page;
 class LibraryPage;
-class AlbumsPage;
 class FolderPage;
 class PlaylistsPage;
-#ifdef ENABLE_DYNAMIC
-class DynamicPage;
-#endif
-#ifdef ENABLE_STREAMS
-class StreamsPage;
-#endif
-#ifdef ENABLE_ONLINE_SERVICES
 class OnlineServicesPage;
-#endif
 #ifdef ENABLE_DEVICES_SUPPORT
 class DevicesPage;
 #endif
@@ -74,11 +60,18 @@ class QPropertyAnimation;
 class QActionGroup;
 class QDateTime;
 class QMenu;
+class QSessionManager;
 class TrayItem;
 class HttpStream;
 class MPDStatus;
 struct MPDConnectionDetails;
 struct Output;
+#if defined Q_OS_WIN
+class ThumbnailToolBar;
+#endif
+#ifdef Q_OS_MAC
+class DockMenu;
+#endif
 
 // Dummy classes so that when class name is saved to the config file, we get a more meaningful name than QWidget!!!
 class PlayQueuePage : public QWidget
@@ -94,27 +87,20 @@ public:
     ContextPage(QWidget *p) : QWidget(p) { }
 };
 
-class MainWindow : public MAIN_WINDOW_BASE_CLASS, private Ui::MainWindow
+class MainWindow : public QMainWindow, private Ui::MainWindow
 {
     Q_OBJECT
 
+    Q_PROPERTY( QStringList listActions READ listActions )
+
 public:
     enum Pages
     {
         PAGE_PLAYQUEUE,
         PAGE_LIBRARY,
-        PAGE_ALBUMS,
         PAGE_FOLDERS,
         PAGE_PLAYLISTS,
-        #ifdef ENABLE_DYNAMIC
-        PAGE_DYNAMIC,
-        #endif
-        #ifdef ENABLE_STREAMS
-        PAGE_STREAMS,
-        #endif
-        #ifdef ENABLE_ONLINE_SERVICES
         PAGE_ONLINE,
-        #endif
         #ifdef ENABLE_DEVICES_SUPPORT
         PAGE_DEVICES,
         #endif
@@ -122,18 +108,22 @@ public:
         PAGE_CONTEXT
     };
 
-    MainWindow(QWidget *parent = 0);
-    ~MainWindow();
+    MainWindow(QWidget *parent = nullptr);
+    ~MainWindow() override;
 
-    int mpdVolume() const { return volumeSlider->value(); }
+    QList<Song> selectedSongs() const;
+    QStringList listActions() const;
 
 protected:
-    void keyPressEvent(QKeyEvent *event);
-    void closeEvent(QCloseEvent *event);
+    void keyPressEvent(QKeyEvent *event) override;
+    void showEvent(QShowEvent *event) override;
+    void closeEvent(QCloseEvent *event) override;
+    void resizeEvent(QResizeEvent *event) override;
 
 private:
+    #if !defined Q_OS_WIN
     void addMenuAction(QMenu *menu, QAction *action);
-    void initSizes();
+    #endif
     void setupTrayIcon();
 
 Q_SIGNALS:
@@ -142,6 +132,7 @@ Q_SIGNALS:
     void pause(bool p);
     void play();
     void stop(bool afterCurrent=false);
+    void terminating();
     void getStatus();
     void playListInfo();
     void currentSong();
@@ -149,29 +140,27 @@ Q_SIGNALS:
     void startPlayingSongId(qint32);
     void setVolume(int);
     void outputs();
-    void enableOutput(int id, bool);
-    void setPriority(const QList<qint32> &ids, quint8 priority);
+    void enableOutput(quint32 id, bool);
+    void setPriority(const QList<qint32> &ids, quint8 priority, bool decreasePriority);
     void addSongsToPlaylist(const QString &name, const QStringList &files);
     void showPreferencesPage(const QString &page);
+    void playNext(const QList<quint32> &items, quint32 pos, quint32 size);
 
 public Q_SLOTS:
     void showError(const QString &message, bool showActions=false);
     void showInformation(const QString &message);
     void dynamicStatus(const QString &message);
+    void setCollection(const QString &collection);
+    void mpdConnectionName(const QString &name);
     void hideWindow();
     void restoreWindow();
-    void load(const QStringList &urls) { playQueueModel.load(urls); }
-    #ifdef ENABLE_KDE_SUPPORT
-    void configureShortcuts();
-    void saveShortcuts();
-    #else
+    void load(const QStringList &urls) { PlayQueueModel::self()->load(urls, MPDConnection::AppendAndPlay); }
     void showAboutDialog();
-    #endif
     void mpdConnectionStateChanged(bool connected);
     void playQueueItemsSelected(bool s);
-    void showSidebarPreferencesPage() { showPreferencesDialog("interface:sidebar"); }
     void showPreferencesDialog(const QString &page=QString());
     void quit();
+    void commitDataRequest(QSessionManager &mgr);
     void updateSettings();
     void toggleOutput();
     void changeConnection();
@@ -188,27 +177,28 @@ public Q_SLOTS:
     void searchPlayQueue();
     void realSearchPlayQueue();
     void playQueueSearchActivated(bool a);
-    void updatePlayQueue(const QList<Song> &songs);
-    void updateCurrentSong(const Song &song, bool wasEmpty=false);
+    void updatePlayQueue(const QList<Song> &songs, bool isComplete);
+    void updateCurrentSong(Song song, bool wasEmpty=false);
     void scrollPlayQueue(bool wasEmpty=false);
-    void updateStats();
     void updateStatus();
     void playQueueItemActivated(const QModelIndex &);
-    void promptClearPlayQueue();
     void clearPlayQueue();
     void centerPlayQueue();
-    void removeFromPlayQueue() { playQueueModel.remove(playQueueProxyModel.mapToSourceRows(playQueue->selectedIndexes())); }
-    void replacePlayQueue() { addToPlayQueue(true); }
-    void addToPlayQueue() { addToPlayQueue(false); }
-    void addRandomToPlayQueue() { addToPlayQueue(false, 0, true); }
+    void removeFromPlayQueue() { PlayQueueModel::self()->remove(playQueueProxyModel.mapToSourceRows(playQueue->selectedIndexes())); }
+    void replacePlayQueue() { appendToPlayQueue(MPDConnection::ReplaceAndplay); }
+    void appendToPlayQueue() { appendToPlayQueue(MPDConnection::Append); }
+    void appendToPlayQueueAndPlay() { appendToPlayQueue(MPDConnection::AppendAndPlay); }
+    void addToPlayQueueAndPlay() { appendToPlayQueue(MPDConnection::AddAndPlay); }
+    void insertIntoPlayQueue() { appendToPlayQueue(MPDConnection::AddAfterCurrent); }
     void addWithPriority();
     void addToNewStoredPlaylist();
     void addToExistingStoredPlaylist(const QString &name) { addToExistingStoredPlaylist(name, playQueue->hasFocus()); }
     void addToExistingStoredPlaylist(const QString &name, bool pq);
     void addStreamToPlayQueue();
+    void addLocalFilesToPlayQueue();
     void removeItems();
     void checkMpdAccessibility();
-    void cropPlayQueue() { playQueueModel.crop(playQueueProxyModel.mapToSourceRows(playQueue->selectedIndexes())); }
+    void cropPlayQueue() { PlayQueueModel::self()->crop(playQueueProxyModel.mapToSourceRows(playQueue->selectedIndexes())); }
     void updatePlayQueueStats(int songs, quint32 time);
     void expandOrCollapse(bool saveCurrentSize=true);
     void showSongInfo();
@@ -218,24 +208,9 @@ public Q_SLOTS:
     void tabToggled(int index);
     void showPlayQueue() { showTab(PAGE_PLAYQUEUE); }
     void showLibraryTab() { showTab(PAGE_LIBRARY); }
-    void showAlbumsTab() { showTab(PAGE_ALBUMS); }
     void showFoldersTab() { showTab(PAGE_FOLDERS); }
     void showPlaylistsTab() { showTab(PAGE_PLAYLISTS); }
-    void showDynamicTab() {
-        #ifdef ENABLE_DYNAMIC
-        showTab(PAGE_DYNAMIC);
-        #endif
-    }
-    void showStreamsTab() {
-        #ifdef ENABLE_STREAMS
-        showTab(PAGE_STREAMS);
-        #endif
-    }
-    void showOnlineTab() {
-        #ifdef ENABLE_ONLINE_SERVICES
-        showTab(PAGE_ONLINE); 
-        #endif
-    }
+    void showOnlineTab() { showTab(PAGE_ONLINE); }
     void showContextTab() { showTab(PAGE_CONTEXT); }
     void showDevicesTab() {
         #ifdef ENABLE_DEVICES_SUPPORT
@@ -244,13 +219,12 @@ public Q_SLOTS:
     }
     void showSearchTab() { showTab(PAGE_SEARCH); }
     void toggleSplitterAutoHide();
-    void toggleMonoIcons();
     void locateTracks(const QList<Song> &songs);
     void locateTrack();
+    void moveSelectionAfterCurrentSong();
     void locateArtist(const QString &artist);
     void locateAlbum(const QString &artist, const QString &album);
     void editTags();
-    void editPlayQueueTags();
     void organiseFiles();
     void addToDevice(const QString &udi);
     void deleteSongs();
@@ -258,6 +232,7 @@ public Q_SLOTS:
     void deleteSongs(const QString &from, const QList<Song> &songs);
     void replayGain();
     void setCover();
+    void showPlayQueueSearch();
     void showSearch();
     void expandAll();
     void collapseAll();
@@ -266,6 +241,8 @@ public Q_SLOTS:
     void updateConnectionsMenu();
     void controlConnectionsMenu(bool enable=true);
     void controlDynamicButton();
+    void setRating();
+    void triggerAction(const QString &name);
 
 private:
     bool canClose();
@@ -274,39 +251,33 @@ private:
     void enableStopActions(bool enable);
     void updateStatus(MPDStatus * const status);
     void readSettings();
-    void addToPlayQueue(bool replace, quint8 priority=0, bool randomAlbums=false);
-    #ifdef TAGLIB_FOUND
-    void editTags(const QList<Song> &songs, bool isPlayQueue);
-    #endif
-    bool currentIsStream() const { return playQueueModel.rowCount() && -1!=current.id && current.isStream(); }
+    void appendToPlayQueue(int action, quint8 priority=0, bool decreasePriority=false);
+    bool currentIsStream() const { return PlayQueueModel::self()->rowCount() && -1!=current.id && current.isStream(); }
     void updateWindowTitle();
     void showTab(int page) { tabWidget->setCurrentIndex(page); }
     void updateNextTrack(int nextTrackId);
     void updateActionToolTips();
-    void setPlaylistsEnabled(bool e);
-    void controlPlaylistActions();
     void startContextTimer();
     int calcMinHeight();
     int calcCollapsedSize();
     void setCollapsedSize();
+    void controlView(bool forceUpdate=false);
 
 private Q_SLOTS:
+    void controlPlayQueueButtons();
     void toggleContext();
+    void initMpris();
     void toggleMenubar();
+    void paletteChanged();
 
 private:
     int prevPage;
     MPDState lastState;
     qint32 lastSongId;
-    PlayQueueModel playQueueModel;
     PlayQueueProxyModel playQueueProxyModel;
     bool autoScrollPlayQueue;
-    #ifdef ENABLE_KDE_SUPPORT
-    KToggleAction *showMenuAction;
-    Action *shortcutsAction;
-    #else
-    Action *showMenuAction;
-    #endif
+    bool singlePane;
+    bool shown;
     Action *prefAction;
     Action *refreshDbAction;
     Action *doDbRefreshAction;
@@ -316,17 +287,17 @@ private:
     QActionGroup *connectionsGroup;
     Action *stopAfterTrackAction;
     Action *addPlayQueueToStoredPlaylistAction;
-    Action *promptClearPlayQueueAction;
+    Action *clearPlayQueueAction;
     Action *centerPlayQueueAction;
     Action *expandInterfaceAction;
     Action *cropPlayQueueAction;
     Action *addStreamToPlayQueueAction;
+    Action *addLocalFilesToPlayQueueAction;
     Action *randomPlayQueueAction;
     Action *repeatPlayQueueAction;
     Action *singlePlayQueueAction;
     Action *consumePlayQueueAction;
     Action *searchPlayQueueAction;
-    Action *setPriorityAction;
     #ifdef ENABLE_HTTP_STREAM_PLAYBACK
     HttpStream *httpStream;
     Action *streamPlayAction;
@@ -335,7 +306,11 @@ private:
     Action *fullScreenAction;
     Action *quitAction;
     Action *restoreAction;
+    Action *locateAction;
     Action *locateTrackAction;
+    Action *locateAlbumAction;
+    Action *locateArtistAction;
+    Action *playNextAction;
     #ifdef TAGLIB_FOUND
     Action *editPlayQueueTagsAction;
     #endif
@@ -343,40 +318,29 @@ private:
     Action *expandAllAction;
     Action *collapseAllAction;
     Action *serverInfoAction;
-    Action *clearPlayQueueAction;
     Action *cancelAction;
-    Action *clearNewStateAction;
+    Action *ratingAction;
     TrayItem *trayItem;
     QPoint lastPos;
     QSize expandedSize;
     QSize collapsedSize;
+    QSize previousSize;
     Song current;
     Page *currentPage;
     Action *showPlayQueueAction;
     QWidget *playQueuePage;
     Action *libraryTabAction;
     LibraryPage *libraryPage;
-    Action *albumsTabAction;
-    AlbumsPage *albumsPage;
     Action *foldersTabAction;
     FolderPage *folderPage;
     Action *playlistsTabAction;
     PlaylistsPage *playlistsPage;
-    #ifdef ENABLE_DYNAMIC
-    Action *dynamicTabAction;
-    DynamicPage *dynamicPage;
-    #endif
-    #ifdef ENABLE_STREAMS
-    Action *streamsTabAction;
-    StreamsPage *streamsPage;
-    #endif
-    #ifdef ENABLE_ONLINE_SERVICES
     Action *onlineTabAction;
     OnlineServicesPage *onlinePage;
-    #endif
     QWidget *contextPage;
     #ifdef ENABLE_DEVICES_SUPPORT
     Action *devicesTabAction;
+    Action *copyToDeviceAction;
     DevicesPage *devicesPage;
     #endif
     SearchPage *searchPage;
@@ -387,11 +351,19 @@ private:
     QTimer *playQueueSearchTimer;
     #if !defined Q_OS_WIN && !defined Q_OS_MAC
     QTimer *mpdAccessibilityTimer;
+    Action *showMenubarAction;
     #endif
     QTimer *contextTimer;
     int contextSwitchTime;
     enum { CS_Init, CS_Connected, CS_Disconnected } connectedState;
     bool stopAfterCurrent;
+    bool responsiveSidebar;
+    #if defined Q_OS_WIN
+    ThumbnailToolBar *thumbnailTooolbar;
+    #endif
+    #ifdef Q_OS_MAC
+    DockMenu *dockMenu;
+    #endif
     friend class TrayItem;
 };
 
