@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,7 +24,8 @@
 #ifndef PAGE_H
 #define PAGE_H
 
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
+#include "mpd-interface/mpdconnection.h"
 
 class Page
 {
@@ -33,8 +34,8 @@ public:
     virtual ~Page() { }
     virtual Song coverRequest() const { return Song(); }
     virtual QList<Song> selectedSongs(bool allowPlaylists=false) const { Q_UNUSED(allowPlaylists) return QList<Song>(); }
-    virtual void addSelectionToPlaylist(const QString &name=QString(), bool replace=false, quint8 priorty=0, bool randomAlbums=false) {
-        Q_UNUSED(name) Q_UNUSED(replace) Q_UNUSED(priorty) Q_UNUSED(randomAlbums)
+    virtual void addSelectionToPlaylist(const QString &name=QString(), int action=MPDConnection::Append, quint8 priority=0, bool decreasePriority=false) {
+        Q_UNUSED(name) Q_UNUSED(action) Q_UNUSED(priority) Q_UNUSED(decreasePriority)
     }
     #ifdef ENABLE_DEVICES_SUPPORT
     virtual void addSelectionToDevice(const QString &udi) { Q_UNUSED(udi) }
