@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,8 +23,8 @@
 
 #include "playbacksettings.h"
 #include "settings.h"
-#include "support/localize.h"
-#include "mpd/mpdconnection.h"
+#include "support/utils.h"
+#include "mpd-interface/mpdconnection.h"
 #include "support/icon.h"
 #include "widgets/basicitemdelegate.h"
 #include "support/messagebox.h"
@@ -43,37 +43,42 @@ PlaybackSettings::PlaybackSettings(QWidget *p)
     stopFadeDuration->setSingleStep(100);
 
     outputsView->setItemDelegate(new BasicItemDelegate(outputsView));
-    replayGain->addItem(i18n("None"), QVariant("off"));
-    replayGain->addItem(i18n("Track"), QVariant("track"));
-    replayGain->addItem(i18n("Album"), QVariant("album"));
-    replayGain->addItem(i18n("Auto"), QVariant("auto"));
+    replayGain->addItem(tr("None"), QVariant("off"));
+    replayGain->addItem(tr("Track"), QVariant("track"));
+    replayGain->addItem(tr("Album"), QVariant("album"));
+    replayGain->addItem(tr("Auto"), QVariant("auto"));
     connect(MPDConnection::self(), SIGNAL(replayGain(const QString &)), this, SLOT(replayGainSetting(const QString &)));
     connect(MPDConnection::self(), SIGNAL(outputsUpdated(const QList<Output> &)), this, SLOT(updateOutputs(const QList<Output> &)));
     connect(MPDConnection::self(), SIGNAL(stateChanged(bool)), this, SLOT(mpdConnectionStateChanged(bool)));
-    connect(this, SIGNAL(enableOutput(int, bool)), MPDConnection::self(), SLOT(enableOutput(int, bool)));
+    connect(this, SIGNAL(enableOutput(quint32, bool)), MPDConnection::self(), SLOT(enableOutput(quint32, bool)));
     connect(this, SIGNAL(outputs()), MPDConnection::self(), SLOT(outputs()));
     connect(this, SIGNAL(setReplayGain(const QString &)), MPDConnection::self(), SLOT(setReplayGain(const QString &)));
     connect(this, SIGNAL(setCrossFade(int)), MPDConnection::self(), SLOT(setCrossFade(int)));
     connect(this, SIGNAL(getReplayGain()), MPDConnection::self(), SLOT(getReplayGain()));
     connect(aboutReplayGain, SIGNAL(leftClickedUrl()), SLOT(showAboutReplayGain()));
     int iconSize=Icon::dlgIconSize();
-    messageIcon->setMinimumSize(iconSize, iconSize);
-    messageIcon->setMaximumSize(iconSize, iconSize);
+    messageIcon->setFixedSize(iconSize, iconSize);
     mpdConnectionStateChanged(MPDConnection::self()->isConnected());
-    #if defined Q_OS_WIN || defined Q_OS_MAC
+    #if defined Q_OS_WIN || (defined Q_OS_MAC && !defined IOKIT_FOUND)
     REMOVE(inhibitSuspend)
     #endif
     outputsView->setVisible(outputsView->count()>1);
     outputsViewLabel->setVisible(outputsView->count()>1);
+
+    #ifdef Q_OS_MAC
+    expandingSpacer->changeSize(0, 0, QSizePolicy::Fixed, QSizePolicy::Fixed);
+    #endif
+    volumeStep->setToolTip(volumeStepLabel->toolTip());
 }
 
 void PlaybackSettings::load()
 {
     stopOnExit->setChecked(Settings::self()->stopOnExit());
     stopFadeDuration->setValue(Settings::self()->stopFadeDuration());
-    #if !defined Q_OS_WIN && !defined Q_OS_MAC
+    #if (defined Q_OS_LINUX && defined QT_QTDBUS_FOUND) || (defined Q_OS_MAC && defined IOKIT_FOUND)
     inhibitSuspend->setChecked(Settings::self()->inhibitSuspend());
     #endif
+    volumeStep->setValue(Settings::self()->volumeStep());
 
     crossfading->setValue(MPDStatus::self()->crossFade());
     if (MPDConnection::self()->isConnected()) {
@@ -86,9 +91,10 @@ void PlaybackSettings::save()
 {
     Settings::self()->saveStopOnExit(stopOnExit->isChecked());
     Settings::self()->saveStopFadeDuration(stopFadeDuration->value());
-    #if !defined Q_OS_WIN && !defined Q_OS_MAC
+    #if (defined Q_OS_LINUX && defined QT_QTDBUS_FOUND) || (defined Q_OS_MAC && defined IOKIT_FOUND)
     Settings::self()->saveInhibitSuspend(inhibitSuspend->isChecked());
     #endif
+    Settings::self()->saveVolumeStep(volumeStep->value());
 
     if (MPDConnection::self()->isConnected()) {
         int crossFade=crossfading->value();
@@ -134,7 +140,7 @@ void PlaybackSettings::updateOutputs(const QList<Output> &outputs)
 {
     outputsView->clear();
     enabledOutputs.clear();
-    foreach(const Output &output, outputs) {
+    for (const Output &output: outputs) {
         QListWidgetItem *item=new QListWidgetItem(output.name, outputsView);
         item->setCheckState(output.enabled ? Qt::Checked : Qt::Unchecked);
         item->setData(Qt::UserRole, output.id);
@@ -155,25 +161,27 @@ void PlaybackSettings::mpdConnectionStateChanged(bool c)
     crossfadingLabel->setEnabled(c);
     replayGainLabel->setEnabled(rgSupported);
     replayGain->setEnabled(rgSupported);
-    messageIcon->setPixmap(Icon(c ? "dialog-information" : "dialog-warning").pixmap(messageIcon->minimumSize()));
+    messageIcon->setPixmap(style()->standardIcon(c ? QStyle::SP_MessageBoxInformation : QStyle::SP_MessageBoxWarning).pixmap(messageIcon->minimumSize()));
     if (c) {
-        messageLabel->setText(i18n("<i>Connected to %1<br/>The entries below apply to the currently connected MPD collection.</i>",
-                                   MPDConnection::self()->getDetails().description()));
+        messageLabel->setText(tr("<i>Connected to %1<br/>The entries below apply to the currently connected MPD collection.</i>")
+                                 .arg(MPDConnection::self()->getDetails().description()));
     } else {
-        messageLabel->setText(i18n("<i>Not Connected!<br/>The entries below cannot be modified, as Cantata is not connected to MPD.</i>"));
+        messageLabel->setText(tr("<i>Not Connected!<br/>The entries below cannot be modified, as Cantata is not connected to MPD.</i>"));
         outputsView->clear();
     }
 }
 
 void PlaybackSettings::showAboutReplayGain()
 {
-    MessageBox::information(this, i18n("<p>Replay Gain is a proposed standard published in 2001 to normalize the perceived loudness of computer "
+    MessageBox::information(this, tr("Replay Gain is a proposed standard published in 2001 to normalize the perceived loudness of computer "
                                        "audio formats such as MP3 and Ogg Vorbis. It works on a track/album basis, and is now supported in a "
-                                       "growing number of players.</p>"
-                                       "<p>The following ReplayGain settings may be used:<ul>"
+                                       "growing number of players."
+                                       "<br/><br/>The following ReplayGain settings may be used:<ul>"
                                        "<li><i>None</i> - No ReplayGain is applied.</li>"
                                        "<li><i>Track</i> - Volume will be adjusted using the track's ReplayGain tags.</li>"
                                        "<li><i>Album</i> - Volume will be adjusted using the albums's ReplayGain tags.</li>"
                                        "<li><i>Auto</i> - Volume will be adjusted using the track's ReplayGain tags if random play is activated, otherwise the album's tags will be used.</li>"
-                                       "</ul></p>"));
+                                       "</ul>"));
 }
+
+#include "moc_playbacksettings.cpp"
