@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -29,13 +29,15 @@ QxtMediaKeys::QxtMediaKeys(QObject *p)
 {
 }
 
-void QxtMediaKeys::activate(bool a)
+bool QxtMediaKeys::activate()
 {
-    if (a) {
-        createShortcuts();
-    } else {
-        clear();
-    }
+    createShortcuts();
+    return true; // Hmm... How to detect if this failed?
+}
+
+void QxtMediaKeys::deactivate()
+{
+    clear();
 }
 
 void QxtMediaKeys::createShortcuts()
@@ -64,3 +66,4 @@ void QxtMediaKeys::clear()
         shortcuts.clear();
     }
 }
+
