@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,14 +22,16 @@
  */
 
 #include "searchpage.h"
-#include "mpd/mpdconnection.h"
-#include "support/localize.h"
+#include "mpd-interface/mpdconnection.h"
 #include "settings.h"
 #include "stdactions.h"
+#include "customactions.h"
 #include "support/utils.h"
 #include "support/icon.h"
-#include "plurals.h"
+#include "support/messagebox.h"
 #include "widgets/tableview.h"
+#include "widgets/menubutton.h"
+#include "widgets/icons.h"
 
 class SearchTableView : public TableView
 {
@@ -41,67 +43,65 @@ public:
         setIndentation(0);
     }
 
-    virtual ~SearchTableView() { }
+    ~SearchTableView() override { }
 };
 
 SearchPage::SearchPage(QWidget *p)
-    : QWidget(p)
+    : SinglePageWidget(p)
     , state(-1)
     , model(this)
     , proxy(this)
 {
-    setupUi(this);
-    addToPlayQueue->setDefaultAction(StdActions::self()->addToPlayQueueAction);
-    replacePlayQueue->setDefaultAction(StdActions::self()->replacePlayQueueAction);
-    locateAction=new Action(Icon("edit-find"), i18n("Locate In Library"), this);
+    statsLabel=new SqueezedTextLabel(this);
+    locateAction=new Action(Icons::self()->searchIcon, tr("Locate In Library"), this);
     view->allowTableView(new SearchTableView(view));
-    view->addAction(StdActions::self()->addToPlayQueueAction);
-    view->addAction(StdActions::self()->replacePlayQueueAction);
-    view->addAction(StdActions::self()->addWithPriorityAction);
-    view->addAction(StdActions::self()->addToStoredPlaylistAction);
-    #ifdef TAGLIB_FOUND
-    #ifdef ENABLE_DEVICES_SUPPORT
-    view->addAction(StdActions::self()->copyToDeviceAction);
-    #endif
-    #endif // TAGLIB_FOUND
-    view->addAction(locateAction);
 
-    connect(this, SIGNAL(add(const QStringList &, bool, quint8)), MPDConnection::self(), SLOT(add(const QStringList &, bool, quint8)));
-    connect(this, SIGNAL(addSongsToPlaylist(const QString &, const QStringList &)), MPDConnection::self(), SLOT(addToPlaylist(const QString &, const QStringList &)));
     connect(&model, SIGNAL(searching()), view, SLOT(showSpinner()));
     connect(&model, SIGNAL(searched()), view, SLOT(hideSpinner()));
     connect(&model, SIGNAL(statsUpdated(int, quint32)), this, SLOT(statsUpdated(int, quint32)));
     connect(view, SIGNAL(itemsSelected(bool)), this, SLOT(controlActions()));
     connect(view, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(itemDoubleClicked(const QModelIndex &)));
-    connect(view, SIGNAL(searchItems()), this, SLOT(searchItems()));
     connect(MPDConnection::self(), SIGNAL(stateChanged(bool)), this, SLOT(setSearchCategories()));
-    connect(locateAction, SIGNAL(triggered(bool)), SLOT(locateSongs()));
+    connect(locateAction, SIGNAL(triggered()), SLOT(locateSongs()));
     proxy.setSourceModel(&model);
     view->setModel(&proxy);
-    view->setMode(ItemView::Mode_List);
     view->setPermanentSearch();
     setSearchCategories();
     view->setSearchCategory(Settings::self()->searchCategory());
-    statsUpdated(0, 0);
+    statsUpdated(0, 0);    
+    view->setMode(ItemView::Mode_List);
+    Configuration config(metaObject()->className());
+    view->load(config);
+    MenuButton *menu=new MenuButton(this);
+    menu->addActions(createViewActions(QList<ItemView::Mode>() << ItemView::Mode_List << ItemView::Mode_Table));
+    statsLabel->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
+    init(ReplacePlayQueue|AppendToPlayQueue, QList<QWidget *>() << menu << statsLabel);
+
+    view->addAction(StdActions::self()->addToStoredPlaylistAction);
+    view->addAction(CustomActions::self());
+    #ifdef TAGLIB_FOUND
+    #ifdef ENABLE_DEVICES_SUPPORT
+    view->addAction(StdActions::self()->copyToDeviceAction);
+    #endif
+    view->addAction(StdActions::self()->organiseFilesAction);
+    view->addAction(StdActions::self()->editTagsAction);
+    #ifdef ENABLE_REPLAYGAIN_SUPPORT
+    view->addAction(StdActions::self()->replaygainAction);
+    #endif
+    #ifdef ENABLE_DEVICES_SUPPORT
+    view->addSeparator();
+    view->addAction(StdActions::self()->deleteSongsAction);
+    #endif
+    #endif // TAGLIB_FOUND
+    view->addAction(locateAction);
+    view->setInfoText(tr("Use the text field above to search for music in your library via MPD's search mechanism. "
+                         "Clicking the label next to the field will produce a menu allowing you to control the search area."));
 }
 
 SearchPage::~SearchPage()
 {
-}
-
-void SearchPage::showEvent(QShowEvent *e)
-{
-    view->focusSearch();
-    QWidget::showEvent(e);
-}
-
-void SearchPage::saveConfig()
-{
-    Settings::self()->saveSearchCategory(view->searchCategory());
-    TableView *tv=qobject_cast<TableView *>(view->view());
-    if (tv) {
-        tv->saveHeader();
-    }
+    Configuration config(metaObject()->className());
+    view->save(config);
 }
 
 void SearchPage::refresh()
@@ -120,6 +120,12 @@ void SearchPage::setView(int mode)
     model.setMultiColumn(ItemView::Mode_Table==mode);
 }
 
+void SearchPage::showEvent(QShowEvent *e)
+{
+    SinglePageWidget::showEvent(e);
+    view->focusSearch();
+}
+
 QStringList SearchPage::selectedFiles(bool allowPlaylists) const
 {
     QModelIndexList selected = view->selectedIndexes();
@@ -128,7 +134,7 @@ QStringList SearchPage::selectedFiles(bool allowPlaylists) const
     }
 
     QModelIndexList mapped;
-    foreach (const QModelIndex &idx, selected) {
+    for (const QModelIndex &idx: selected) {
         mapped.append(proxy.mapToSource(idx));
     }
 
@@ -143,28 +149,13 @@ QList<Song> SearchPage::selectedSongs(bool allowPlaylists) const
     }
 
     QModelIndexList mapped;
-    foreach (const QModelIndex &idx, selected) {
+    for (const QModelIndex &idx: selected) {
         mapped.append(proxy.mapToSource(idx));
     }
 
     return model.songs(mapped, allowPlaylists);
 }
 
-void SearchPage::addSelectionToPlaylist(const QString &name, bool replace, quint8 priorty, bool randomAlbums)
-{
-    Q_UNUSED(randomAlbums)
-    QStringList files=selectedFiles(true);
-
-    if (!files.isEmpty()) {
-        if (name.isEmpty()) {
-            emit add(files, replace, priorty);
-        } else {
-            emit addSongsToPlaylist(name, files);
-        }
-        view->clearSelection();
-    }
-}
-
 #ifdef ENABLE_DEVICES_SUPPORT
 void SearchPage::addSelectionToDevice(const QString &udi)
 {
@@ -175,9 +166,22 @@ void SearchPage::addSelectionToDevice(const QString &udi)
         view->clearSelection();
     }
 }
+
+void SearchPage::deleteSongs()
+{
+    QList<Song> songs=selectedSongs();
+
+    if (!songs.isEmpty()) {
+        if (MessageBox::Yes==MessageBox::warningYesNo(this, tr("Are you sure you wish to delete the selected songs?\n\nThis cannot be undone."),
+                                                      tr("Delete Songs"), StdGuiItem::del(), StdGuiItem::cancel())) {
+            emit deleteSongs(QString(), songs);
+        }
+        view->clearSelection();
+    }
+}
 #endif
 
-void SearchPage::searchItems()
+void SearchPage::doSearch()
 {
     QString text=view->searchText().trimmed();
 
@@ -202,19 +206,36 @@ void SearchPage::controlActions()
     QModelIndexList selected=view->selectedIndexes(false); // Dont need sorted selection here...
     bool enable=selected.count()>0;
 
-    StdActions::self()->addToPlayQueueAction->setEnabled(enable);
-    StdActions::self()->addWithPriorityAction->setEnabled(enable);
-    StdActions::self()->replacePlayQueueAction->setEnabled(enable);
-    StdActions::self()->addToStoredPlaylistAction->setEnabled(enable);
-    StdActions::self()->addRandomToPlayQueueAction->setEnabled(false);
+    StdActions::self()->enableAddToPlayQueue(enable);
+    CustomActions::self()->setEnabled(enable);
     locateAction->setEnabled(enable);
+
+    StdActions::self()->addToStoredPlaylistAction->setEnabled(enable);
+    #ifdef TAGLIB_FOUND
+    StdActions::self()->organiseFilesAction->setEnabled(enable && MPDConnection::self()->getDetails().dirReadable);
+    StdActions::self()->editTagsAction->setEnabled(StdActions::self()->organiseFilesAction->isEnabled());
+    #ifdef ENABLE_REPLAYGAIN_SUPPORT
+    StdActions::self()->replaygainAction->setEnabled(StdActions::self()->organiseFilesAction->isEnabled());
+    #endif
+    #ifdef ENABLE_DEVICES_SUPPORT
+    StdActions::self()->deleteSongsAction->setEnabled(StdActions::self()->organiseFilesAction->isEnabled());
+    StdActions::self()->copyToDeviceAction->setEnabled(StdActions::self()->organiseFilesAction->isEnabled());
+    #endif
+    #endif // TAGLIB_FOUND
+}
+
+void SearchPage::setSearchCategory(const QString &cat)
+{
+    view->setSearchCategory(cat);
 }
 
 void SearchPage::setSearchCategories()
 {
     int newState=(MPDConnection::self()->composerTagSupported() ? State_ComposerSupported : 0)|
                  (MPDConnection::self()->commentTagSupported() ? State_CommmentSupported : 0)|
-                 (MPDConnection::self()->performerTagSupported() ? State_PerformerSupported : 0);;
+                 (MPDConnection::self()->performerTagSupported() ? State_PerformerSupported : 0)|
+                 (MPDConnection::self()->modifiedFindSupported() ? State_ModifiedSupported : 0)|
+                 (MPDConnection::self()->originalDateTagSupported() ? State_OrigDateSupported : 0);
 
     if (state==newState) {
         // No changes to be made!
@@ -222,31 +243,41 @@ void SearchPage::setSearchCategories()
     }
 
     state=newState;
-    QList<QPair<QString, QString> > categories;
+    QList<SearchWidget::Category> categories;
 
-    categories << QPair<QString, QString>(i18n("Artist:"), QLatin1String("artist"));
+    categories << SearchWidget::Category(tr("Artist:"), QLatin1String("artist"), tr("Search for songs by artist."));
 
     if (state&State_ComposerSupported) {
-        categories << QPair<QString, QString>(i18n("Composer:"), QLatin1String("composer"));
+        categories << SearchWidget::Category(tr("Composer:"), QLatin1String("composer"), tr("Search for songs by composer."));
     }
     if (state&State_PerformerSupported) {
-        categories << QPair<QString, QString>(i18n("Performer:"), QLatin1String("performer"));
+        categories << SearchWidget::Category(tr("Performer:"), QLatin1String("performer"), tr("Search for songs by performer."));
     }
-    categories << QPair<QString, QString>(i18n("Album:"), QLatin1String("album"))
-               << QPair<QString, QString>(i18n("Title:"), QLatin1String("title"))
-               << QPair<QString, QString>(i18n("Genre:"), QLatin1String("genre"));
+    categories << SearchWidget::Category(tr("Album:"), QLatin1String("album"), tr("Search for songs by album."))
+               << SearchWidget::Category(tr("Title:"), QLatin1String("title"), tr("Search for songs by title."))
+               << SearchWidget::Category(tr("Genre:"), QLatin1String("genre"), tr("Search for songs by genre."));
     if (state&State_CommmentSupported) {
-        categories << QPair<QString, QString>(i18n("Comment:"), QLatin1String("comment"));
+        categories << SearchWidget::Category(tr("Comment:"), QLatin1String("comment"), tr("Search for songs containing comment."));
     }
-    categories << QPair<QString, QString>(i18n("Date:"), QLatin1String("date"))
-               << QPair<QString, QString>(i18n("File:"), QLatin1String("file"))
-               << QPair<QString, QString>(i18n("Any:"), QLatin1String("any"));
+    categories << SearchWidget::Category(tr("Date:"), QLatin1String("date"),
+                                         tr("Find songs be searching the 'Date' tag.<br/><br/>Usually just entering the year should suffice."));
+    if (state&State_OrigDateSupported) {
+        categories << SearchWidget::Category(tr("Original Date:"), QLatin1String("originaldate"),
+                                             tr("Find songs be searching the 'Original Date' tag.<br/><br/>Usually just entering the year should suffice."));
+    }
+    if (state&State_ModifiedSupported) {
+        categories << SearchWidget::Category(tr("Modified:"), MPDConnection::constModifiedSince,
+                                             tr("Enter date (YYYY/MM/DD - e.g. 2015/01/31) to search for files modified since that date.<br/><br>"
+                                                  "Or enter a number of days to find files that were modified in the previous number of days."));
+    }
+    categories << SearchWidget::Category(tr("File:"), QLatin1String("file"), tr("Search for songs by filename or path."))
+               << SearchWidget::Category(tr("Any:"), QLatin1String("any"), tr("Search for songs by any matching metadata."));
     view->setSearchCategories(categories);
 }
 
 void SearchPage::statsUpdated(int songs, quint32 time)
 {
-    statsLabel->setText(0==songs ? i18n("No tracks found.") : Plurals::tracksWithDuration(songs, Utils::formatDuration(time)));
+    statsLabel->setText(0==songs ? tr("No tracks found.") : tr("%n Tracks (%1)", "", songs).arg(Utils::formatDuration(time)));
 }
 
 void SearchPage::locateSongs()
@@ -257,3 +288,5 @@ void SearchPage::locateSongs()
         emit locate(songs);
     }
 }
+
+#include "moc_searchpage.cpp"
