@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,61 +24,63 @@
 #ifndef SEARCHPAGE_H
 #define SEARCHPAGE_H
 
-#include "ui_searchpage.h"
-#include "page.h"
-#include "models/searchmodel.h"
+#include "widgets/singlepagewidget.h"
+#include "models/mpdsearchmodel.h"
 #include "models/searchproxymodel.h"
 
 class Action;
+class SqueezedTextLabel;
 
-class SearchPage : public QWidget, public Ui::SearchPage, public Page
+class SearchPage : public SinglePageWidget
 {
     Q_OBJECT
 
 public:
     SearchPage(QWidget *p);
-    virtual ~SearchPage();
+    ~SearchPage() override;
 
-    void saveConfig();
-    void refresh();
+    void refresh() override;
     void clear();
-    void setView(int mode);
-    QStringList selectedFiles(bool allowPlaylists=false) const;
-    QList<Song> selectedSongs(bool allowPlaylists=false) const;
-    void addSelectionToPlaylist(const QString &name=QString(), bool replace=false, quint8 priorty=0, bool randomAlbums=false);
+    void setView(int mode) override;
+    void showEvent(QShowEvent *e) override;
+    QStringList selectedFiles(bool allowPlaylists=false) const override;
+    QList<Song> selectedSongs(bool allowPlaylists=false) const override;
     #ifdef ENABLE_DEVICES_SUPPORT
-    void addSelectionToDevice(const QString &udi);
+    void addSelectionToDevice(const QString &udi) override;
+    void deleteSongs() override;
     #endif
-    void showEvent(QShowEvent *e);
+    void setSearchCategory(const QString &cat);
 
 Q_SIGNALS:
-    // These are for communicating with MPD object (which is in its own thread, so need to talk via signal/slots)
-    void add(const QStringList &files, bool replace, quint8 priorty);
-    void addSongsToPlaylist(const QString &name, const QStringList &files);
-
     void addToDevice(const QString &from, const QString &to, const QList<Song> &songs);
+    void deleteSongs(const QString &from, const QList<Song> &songs);
     void locate(const QList<Song> &songs);
 
-public Q_SLOTS:
-    void searchItems();
+public Q_SLOTS:   
     void itemDoubleClicked(const QModelIndex &);
-    void controlActions();
     void setSearchCategories();
     void statsUpdated(int songs, quint32 time);
     void locateSongs();
 
+private:
+    void doSearch() override;
+    void controlActions() override;
+
 private:
     enum State
     {
-        State_ComposerSupported=0x01,
-        State_CommmentSupported=0x02,
-        State_PerformerSupported=0x04
+        State_ComposerSupported  = 0x01,
+        State_CommmentSupported  = 0x02,
+        State_PerformerSupported = 0x04,
+        State_ModifiedSupported  = 0x08,
+        State_OrigDateSupported  = 0x10
     };
 
     int state;
-    SearchModel model;
+    MpdSearchModel model;
     SearchProxyModel proxy;
     Action *locateAction;
+    SqueezedTextLabel *statsLabel;
 };
 
 #endif
