@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,95 +23,105 @@
 
 #include "serversettings.h"
 #include "settings.h"
-#include "support/localize.h"
 #include "support/inputdialog.h"
 #include "support/messagebox.h"
 #include "widgets/icons.h"
-#include "models/musiclibrarymodel.h"
-#include "mpd/mpduser.h"
+#include "db/mpdlibrarydb.h"
+#ifdef ENABLE_SIMPLE_MPD_SUPPORT
+#include "mpd-interface/mpduser.h"
+#endif
 #include "support/utils.h"
+#ifdef AVAHI_FOUND
+#include "findmpddialog.h"
+#endif
 #include <QDir>
 #include <QComboBox>
 #include <QPushButton>
 #include <QValidator>
 #include <QStyle>
-#if QT_VERSION > 0x050000
 #include <QStandardPaths>
-#else
-#include <QDesktopServices>
-#endif
 
 #define REMOVE(w) \
     w->setVisible(false); \
     w->deleteLater(); \
     w=0;
 
-class CoverNameValidator : public QValidator
-{
-    public:
-
-    CoverNameValidator(QObject *parent) : QValidator(parent) { }
-
-    State validate(QString &input, int &) const
-    {
-        for (int i=0; i<input.length(); ++i) {
-            if (!input[i].isLetterOrNumber() && '%'!=input[i] && ' '!=input[i] && '-'!=input[i]) {
-                return Invalid;
-            }
-        }
-
-        return Acceptable;
-    }
-};
-
 class CollectionNameValidator : public QValidator
 {
     public:
 
     CollectionNameValidator(QObject *parent) : QValidator(parent) { }
 
-    State validate(QString &input, int &) const
+    State validate(QString &input, int &) const override
     {
-        return input.startsWith(MPDUser::constName) ? Invalid : Acceptable;
+        #ifdef ENABLE_SIMPLE_MPD_SUPPORT
+        if (input.startsWith(MPDUser::constName)) {
+            return Invalid;
+        }
+        #endif
+        if (-1!=input.indexOf("/") || -1!=input.indexOf("\\") || -1!=input.indexOf("*") || -1!=input.indexOf("?")) {
+            return Invalid;
+        }
+        return Acceptable;
     }
 };
 
 ServerSettings::ServerSettings(QWidget *p)
     : QWidget(p)
     , haveBasicCollection(false)
+    , isCurrentConnection(false)
+    , allOptions(true) // will be toggled
     , prevIndex(0)
 {
     setupUi(this);
     #if defined ENABLE_DEVICES_SUPPORT
-    musicFolderNoteLabel->setText(musicFolderNoteLabel->text()+
-                                  i18n("<i> This folder will also be used to locate music files "
-                                       "for transferring to (and from) devices.</i>"));
+    musicFolderNoteLabel->appendText(QLatin1String("<i> ")+tr("This folder will also be used to locate music files "
+                                     "for tag-editing, replay gain, and transferring to (and from) devices.")+QLatin1String("</i>"));
+    #else
+    musicFolderNoteLabel->appendText(QLatin1String("<i> ")+tr("This folder will also be used to locate music files "
+                                     "for tag-editing, replay gain, etc.")+QLatin1String("</i>"));
     #endif
+    musicFolderNoteLabel->appendText(QLatin1String("<br/><i><a href=\"https://github.com/CDrummond/cantata/wiki/Accessing-music-files,-and-covers\">")+
+                                     tr("More information")+QLatin1String("</a></i>"));
+
     connect(combo, SIGNAL(activated(int)), SLOT(showDetails(int)));
     connect(removeButton, SIGNAL(clicked(bool)), SLOT(remove()));
     connect(addButton, SIGNAL(clicked(bool)), SLOT(add()));
     connect(name, SIGNAL(textChanged(QString)), SLOT(nameChanged()));
     connect(basicDir, SIGNAL(textChanged(QString)), SLOT(basicDirChanged()));
-    addButton->setIcon(Icon("list-add"));
-    removeButton->setIcon(Icon("list-remove"));
+    addButton->setIcon(Icons::self()->addIcon);
+    removeButton->setIcon(Icons::self()->minusIcon);
     addButton->setAutoRaise(true);
     removeButton->setAutoRaise(true);
 
     #if defined Q_OS_WIN
-    hostLabel->setText(i18n("Host:"));
-    socketNoteLabel->setVisible(false);
+    hostLabel->setText(tr("Host:"));
     #endif
-    basicCoverName->setToolTip(coverName->toolTip());
-    basicCoverNameLabel->setToolTip(coverName->toolTip());
-    coverNameLabel->setToolTip(coverName->toolTip());
-    coverName->setValidator(new CoverNameValidator(this));
-    basicCoverName->setValidator(new CoverNameValidator(this));
+
     name->setValidator(new CollectionNameValidator(this));
     #ifndef ENABLE_HTTP_STREAM_PLAYBACK
     REMOVE(streamUrlLabel)
     REMOVE(streamUrl)
     REMOVE(streamUrlNoteLabel)
     #endif
+    #ifdef ENABLE_HTTP_SERVER
+    allowLocalStreamingLabel->setToolTip(allowLocalStreaming->toolTip());
+    #else
+    REMOVE(allowLocalStreaming)
+    REMOVE(allowLocalStreamingLabel)
+    REMOVE(allowLocalStreamingNoteLabel)
+    #endif
+    autoUpdate->setToolTip(autoUpdate->toolTip());
+
+    #ifdef Q_OS_MAC
+    expandingSpacer->changeSize(0, 0, QSizePolicy::Fixed, QSizePolicy::Fixed);
+    #endif
+
+    #ifdef AVAHI_FOUND
+    discoveryButton = new QPushButton(tr("Discover..."), this);
+    hostLayout->insertWidget(hostLayout->count(), discoveryButton);
+    connect(discoveryButton, &QPushButton::clicked, this, &ServerSettings::detectMPDs);
+    #endif
 }
 
 void ServerSettings::load()
@@ -123,17 +133,19 @@ void ServerSettings::load()
     combo->clear();
     int idx=0;
     haveBasicCollection=false;
-    foreach (MPDConnectionDetails d, all) {
+    for (MPDConnectionDetails d: all) {
         combo->addItem(d.getName(), d.name);
         if (d.name==currentCon) {
             prevIndex=idx;
         }
         idx++;
+        #ifdef ENABLE_SIMPLE_MPD_SUPPORT
         if (d.name==MPDUser::constName) {
             d.dir=MPDUser::self()->details().dir;
             haveBasicCollection=true;
             prevBasic=d;
         }
+        #endif
         DeviceOptions opts;
         opts.load(MPDConnectionDetails::configGroupName(d.name), true);
         collections.append(Collection(d, opts));
@@ -156,7 +168,7 @@ void ServerSettings::save()
     QList<MPDConnectionDetails> existingInConfig=Settings::self()->allConnections();
     QList<Collection> toAdd;
 
-    foreach (const Collection &c, collections) {
+    for (const Collection &c: collections) {
         bool found=false;
         for (int i=0; i<existingInConfig.count(); ++i) {
             MPDConnectionDetails e=existingInConfig.at(i);
@@ -164,10 +176,12 @@ void ServerSettings::save()
                 existingInConfig.removeAt(i);
                 found=true;
                 if (c.details.hostname!=e.hostname || c.details.port!=e.port || c.details.password!=e.password ||
-                    c.details.dir!=e.dir || c.details.coverName!=e.coverName
+                    c.details.dir!=e.dir
                     #ifdef ENABLE_HTTP_STREAM_PLAYBACK
                     || c.details.streamUrl!=e.streamUrl
                     #endif
+                    || c.details.allowLocalStreaming!=e.allowLocalStreaming
+                    || c.details.autoUpdate!=e.autoUpdate
                     ) {
                     toAdd.append(c);
                 }
@@ -176,20 +190,23 @@ void ServerSettings::save()
         if (!found) {
             toAdd.append(c);
         }
+        #ifdef ENABLE_SIMPLE_MPD_SUPPORT
         if (c.details.name==MPDUser::constName) {
             prevBasic=c;
         }
+        #endif
     }
 
-    foreach (const MPDConnectionDetails &c, existingInConfig) {
+    for (const MPDConnectionDetails &c: existingInConfig) {
         Settings::self()->removeConnectionDetails(c.name);
     }
 
-    foreach (const Collection &c, toAdd) {
+    for (const Collection &c: toAdd) {
         Settings::self()->saveConnectionDetails(c.details);
         c.namingOpts.save(MPDConnectionDetails::configGroupName(c.details.name), true);
     }
 
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     if (!haveBasicCollection && MPDUser::self()->isSupported()) {
         MPDUser::self()->cleanup();
     }
@@ -197,16 +214,19 @@ void ServerSettings::save()
     if (current.details.name==MPDUser::constName) {
         MPDUser::self()->setDetails(current.details);
     }
+    #endif
     Settings::self()->saveCurrentConnection(current.details.name);
-    MusicLibraryModel::cleanCache();
+    MpdLibraryDb::removeUnusedDbs();
 }
 
 void ServerSettings::cancel()
 {
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     // If we are canceling any changes, then we need to restore user settings...
     if (prevBasic.details.name==MPDUser::constName) {
         MPDUser::self()->setDetails(prevBasic.details);
     }
+    #endif
 }
 
 void ServerSettings::showDetails(int index)
@@ -217,7 +237,10 @@ void ServerSettings::showDetails(int index)
             details.name=generateName(prevIndex);
         }
         collections.replace(prevIndex, details);
-        if (details.name!=MPDUser::constName) {
+        #ifdef ENABLE_SIMPLE_MPD_SUPPORT
+        if (details.name!=MPDUser::constName)
+        #endif
+        {
             combo->setItemText(prevIndex, details.name);
         }
     }
@@ -227,34 +250,46 @@ void ServerSettings::showDetails(int index)
 
 void ServerSettings::add()
 {
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     bool addStandard=true;
 
     if (!haveBasicCollection && MPDUser::self()->isSupported()) {
+        static const QChar constBullet(0x2022);
+
         switch (MessageBox::questionYesNoCancel(this,
-                                   i18n("Which type of collection do you wish to connect to?<br/><ul>"
-                                   "<li>Standard - music collection may be shared, is on another machine, or is already setup</li>"
-                                   "<li>Basic - music collection is not shared with others, and Cantata will configure and control the MPD instance</li></ul>"),
-                                   i18n("Add Collection"), GuiItem(i18n("Standard")), GuiItem(i18n("Basic")))) {
+                                   QLatin1String("<p>")+
+                                   tr("Which type of collection do you wish to connect to?")+QLatin1String("<br/><br/>")+
+                                   constBullet+QLatin1Char(' ')+tr("Standard - music collection may be shared, is on another machine, is "
+                                                                     "already setup, or you wish to enable access from other clients (e.g. "
+                                                                     "MPDroid)")+QLatin1String("<br/><br/>")+
+                                   constBullet+QLatin1Char(' ')+tr("Basic - music collection is not shared with others, and Cantata will "
+                                                                     "configure and control the MPD instance. This setup will be exclusive "
+                                                                     "to Cantata, and will <b>not</b> be accessible to other MPD clients.")+
+                                                                     QLatin1String("<br/><br/>")+
+                                   tr("<i><b>NOTE:</b> %1</i>").arg(tr("If you wish to have an advanced MPD setup (e.g. multiple audio "
+                                        "outputs, full DSD support, etc) then you <b>must</b> choose 'Standard'")),
+                                   tr("Add Collection"), GuiItem(tr("Standard")), GuiItem(tr("Basic")))) {
         case MessageBox::Yes: addStandard=true; break;
         case MessageBox::No: addStandard=false; break;
         default: return;
         }
     }
-
+    #endif
     MPDConnectionDetails details;
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     if (addStandard) {
+    #endif
         details.name=generateName();
         details.port=6600;
         details.hostname=QLatin1String("localhost");
         details.dir=QLatin1String("/var/lib/mpd/music/");
+        details.allowLocalStreaming=true;
+        details.autoUpdate=false;
         combo->addItem(details.name);
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     } else {
         details=MPDUser::self()->details(true);
-        #if QT_VERSION > 0x050000
         QString dir=QStandardPaths::writableLocation(QStandardPaths::MusicLocation);
-        #else
-        QString dir=QDesktopServices::storageLocation(QDesktopServices::MusicLocation);
-        #endif
         if (dir.isEmpty()) {
             QString dir=QDir::homePath()+"/Music";
             dir=dir.replace("//", "/");
@@ -264,6 +299,7 @@ void ServerSettings::add()
         MPDUser::self()->setMusicFolder(dir);
         combo->addItem(MPDUser::translatedName());
     }
+    #endif
     removeButton->setEnabled(combo->count()>1);
     collections.append(Collection(details));
     combo->setCurrentIndex(combo->count()-1);
@@ -274,9 +310,13 @@ void ServerSettings::add()
 void ServerSettings::remove()
 {
     int index=combo->currentIndex();
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     QString cName=1==stackedWidget->currentIndex() ? MPDUser::translatedName() : name->text();
-    if (combo->count()>1 && MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Delete <b>%1</b>?", cName),
-                                                                       i18n("Delete"), StdGuiItem::del(), StdGuiItem::cancel())) {
+    #else
+    QString cName=name->text();
+    #endif
+    if (combo->count()>1 && MessageBox::Yes==MessageBox::questionYesNo(this, tr("Delete '%1'?").arg(cName),
+                                                                       tr("Delete"), StdGuiItem::del(), StdGuiItem::cancel())) {
         bool isLast=index==(combo->count()-1);
         combo->removeItem(index);
         combo->setCurrentIndex(isLast ? index-1 : index);
@@ -303,6 +343,21 @@ void ServerSettings::basicDirChanged()
     }
 }
 
+#ifdef AVAHI_FOUND
+void ServerSettings::adoptServerSettings(QString ip, QString p)
+{
+    host->setText(ip);
+    port->setValue(p.toInt());
+}
+
+void ServerSettings::detectMPDs()
+{
+    FindMpdDialog findMpdDlg(this);
+    QObject::connect(&findMpdDlg, &FindMpdDialog::serverChosen, this, &ServerSettings::adoptServerSettings);
+    findMpdDlg.exec();
+}
+#endif
+
 QString ServerSettings::generateName(int ignore) const
 {
     QString n;
@@ -314,7 +369,7 @@ QString ServerSettings::generateName(int ignore) const
     }
 
     for (int i=1; i<512; ++i) {
-        n=i18n("New Collection %1", i);
+        n=tr("New Collection %1").arg(i);
         if (!collectionNames.contains(n)) {
             break;
         }
@@ -325,22 +380,28 @@ QString ServerSettings::generateName(int ignore) const
 
 void ServerSettings::setDetails(const MPDConnectionDetails &details)
 {
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     if (details.name==MPDUser::constName) {
-        basicDir->setText(Utils::convertDirForDisplay(details.dir));
-        basicCoverName->setText(details.coverName);
+        basicDir->setText(Utils::convertPathForDisplay(details.dir));
         stackedWidget->setCurrentIndex(1);
     } else {
-        name->setText(details.name.isEmpty() ? i18n("Default") : details.name);
+    #endif
+        name->setText(details.name.isEmpty() ? tr("Default") : details.name);
         host->setText(details.hostname);
         port->setValue(details.port);
         password->setText(details.password);
-        dir->setText(Utils::convertDirForDisplay(details.dir));
-        coverName->setText(details.coverName);
+        dir->setText(Utils::convertPathForDisplay(details.dir));
         #ifdef ENABLE_HTTP_STREAM_PLAYBACK
         streamUrl->setText(details.streamUrl);
         #endif
+        #ifdef ENABLE_HTTP_SERVER
+        allowLocalStreaming->setChecked(details.allowLocalStreaming);
+        #endif
+        autoUpdate->setChecked(details.autoUpdate);
         stackedWidget->setCurrentIndex(0);
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
     }
+    #endif
 }
 
 MPDConnectionDetails ServerSettings::getDetails() const
@@ -348,23 +409,32 @@ MPDConnectionDetails ServerSettings::getDetails() const
     MPDConnectionDetails details;
     if (0==stackedWidget->currentIndex()) {
         details.name=name->text().trimmed();
+        #ifdef ENABLE_SIMPLE_MPD_SUPPORT
         if (details.name==MPDUser::constName) {
             details.name=QString();
         }
+        #endif
         details.hostname=host->text().trimmed();
         details.port=port->value();
         details.password=password->text();
-        details.dir=Utils::convertDirFromDisplay(dir->text());
-        details.coverName=coverName->text().trimmed();
+        details.dir=Utils::convertPathFromDisplay(dir->text());
         #ifdef ENABLE_HTTP_STREAM_PLAYBACK
         details.streamUrl=streamUrl->text().trimmed();
         #endif
-    } else {
+        #ifdef ENABLE_HTTP_SERVER
+        details.allowLocalStreaming=allowLocalStreaming->isChecked();
+        #endif
+        details.autoUpdate=autoUpdate->isChecked();
+    }
+    #ifdef ENABLE_SIMPLE_MPD_SUPPORT
+    else {
         details=MPDUser::self()->details(true);
-        details.dir=Utils::convertDirFromDisplay(basicDir->text());
-        details.coverName=basicCoverName->text().trimmed();
+        details.dir=Utils::convertPathFromDisplay(basicDir->text());
         MPDUser::self()->setMusicFolder(details.dir);
     }
+    #endif
     details.setDirReadable();
     return details;
 }
+
+#include "moc_serversettings.cpp"
