@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,19 +22,13 @@
  */
 
 #include "settings.h"
-#include "models/musiclibraryitemalbum.h"
+#include "models/sqllibrarymodel.h"
 #include "support/fancytabwidget.h"
-#include "models/albumsmodel.h"
 #include "widgets/itemview.h"
-#include "mpd/mpdparseutils.h"
+#include "mpd-interface/mpdparseutils.h"
 #include "support/utils.h"
 #include "support/globalstatic.h"
-#if defined ENABLE_KDE_SUPPORT && defined ENABLE_KWALLET
-#include <kwallet.h>
-#endif
-#ifndef ENABLE_KDE_SUPPORT
-#include "mediakeys.h"
-#endif
+#include "db/librarydb.h"
 #include <QFile>
 #include <QDir>
 #include <qglobal.h>
@@ -137,25 +131,19 @@ static Settings::StartupState getStartupState(const QString &str)
 
 Settings::Settings()
     : state(AP_Configured)
-    , timer(0)
     , ver(-1)
-    #if defined ENABLE_KDE_SUPPORT && defined ENABLE_KWALLET
-    , wallet(0)
-    #endif
 {
+    // Call 'version' so that it initialises 'ver' and 'state'
+    version();
     // Only need to read system defaults if we have not previously been configured...
-    if (version()<CANTATA_MAKE_VERSION(0, 8, 0)
-            ? cfg.get("connectionHost", QString()).isEmpty()
-            : !cfg.hasGroup(MPDConnectionDetails::configGroupName())) {
+    if (!cfg.hasGroup(MPDConnectionDetails::configGroupName())) {
         mpdDefaults.read();
     }
 }
 
 Settings::~Settings()
 {
-    #if defined ENABLE_KDE_SUPPORT && defined ENABLE_KWALLET
-    delete wallet;
-    #endif
+    save();
 }
 
 QString Settings::currentConnection()
@@ -166,68 +154,36 @@ QString Settings::currentConnection()
 MPDConnectionDetails Settings::connectionDetails(const QString &name)
 {
     MPDConnectionDetails details;
-    if (version()<CANTATA_MAKE_VERSION(0, 8, 0) || (name.isEmpty() && !cfg.hasGroup(MPDConnectionDetails::configGroupName(name)))) {
-        details.hostname=cfg.get("connectionHost", name.isEmpty() ? mpdDefaults.host : QString());
-        #if defined ENABLE_KDE_SUPPORT && defined ENABLE_KWALLET
-        if (cfg.get("connectionPasswd", false)) {
-            if (openWallet()) {
-                wallet->readPassword("mpd", details.password);
-            }
-        } else if (name.isEmpty()) {
-           details.password=mpdDefaults.passwd;
-        }
-        #else
-        details.password=cfg.get("connectionPasswd", name.isEmpty() ? mpdDefaults.passwd : QString());
-        #endif
-        details.port=cfg.get("connectionPort", name.isEmpty() ? mpdDefaults.port : 6600);
-        #ifdef Q_OS_WIN
-        details.dir=Utils::fixPath(QDir::fromNativeSeparators(cfg.get("mpdDir", mpdDefaults.dir)));
-        #else
-        details.dir=Utils::fixPath(cfg.get("mpdDir", mpdDefaults.dir));
+    QString n=MPDConnectionDetails::configGroupName(name);
+    details.name=name;
+    if (!cfg.hasGroup(n)) {
+        details.name=QString();
+        n=MPDConnectionDetails::configGroupName(details.name);
+    }
+    if (cfg.hasGroup(n)) {
+        Configuration grp(n);
+        details.hostname=grp.get("host", name.isEmpty() ? mpdDefaults.host : QString());
+        details.port=grp.get("port", name.isEmpty() ? mpdDefaults.port : 6600);
+        details.dir=grp.getDirPath("dir", name.isEmpty() ? mpdDefaults.dir : "/var/lib/mpd/music");
+        details.password=grp.get("passwd", name.isEmpty() ? mpdDefaults.passwd : QString());
+        #ifdef ENABLE_HTTP_STREAM_PLAYBACK
+        details.streamUrl=grp.get("streamUrl", QString());
         #endif
+        details.replayGain=grp.get("replayGain", QString());
+        // if the setting hasn't been set before, we set it to true to allow
+        // for easy migration of existing settings
+        details.allowLocalStreaming=grp.get("allowLocalStreaming", true);
+        details.autoUpdate=grp.get("autoUpdate", false);
     } else {
-        QString n=MPDConnectionDetails::configGroupName(name);
-        details.name=name;
-        if (!cfg.hasGroup(n)) {
-            details.name=QString();
-            n=MPDConnectionDetails::configGroupName(details.name);
-        }
-        if (cfg.hasGroup(n)) {
-            cfg.beginGroup(n);
-            details.hostname=cfg.get("host", name.isEmpty() ? mpdDefaults.host : QString());
-            details.port=cfg.get("port", name.isEmpty() ? mpdDefaults.port : 6600);
-            #ifdef Q_OS_WIN
-            details.dir=Utils::fixPath(QDir::fromNativeSeparators(cfg.get("dir", name.isEmpty() ? mpdDefaults.dir : "/var/lib/mpd/music")));
-            #else
-            details.dir=Utils::fixPath(cfg.get("dir", name.isEmpty() ? mpdDefaults.dir : "/var/lib/mpd/music"));
-            #endif
-            #if defined ENABLE_KDE_SUPPORT && defined ENABLE_KWALLET
-            if (KWallet::Wallet::isEnabled()) {
-                if (cfg.get("passwd", false)) {
-                    if (openWallet()) {
-                        wallet->readPassword(name.isEmpty() ? "mpd" : name, details.password);
-                    }
-                } else if (name.isEmpty()) {
-                    details.password=mpdDefaults.passwd;
-                }
-            } else
-            #endif // ENABLE_KWALLET
-            details.password=cfg.get("passwd", name.isEmpty() ? mpdDefaults.passwd : QString());
-            details.coverName=cfg.get("coverName", QString());
-            #ifdef ENABLE_HTTP_STREAM_PLAYBACK
-            details.streamUrl=cfg.get("streamUrl", QString());
-            #endif
-            cfg.endGroup();
-        } else {
-            details.hostname=mpdDefaults.host;
-            details.port=mpdDefaults.port;
-            details.dir=mpdDefaults.dir;
-            details.password=mpdDefaults.passwd;
-            details.coverName=QString();
-            #ifdef ENABLE_HTTP_STREAM_PLAYBACK
-            details.streamUrl=QString();
-            #endif
-        }
+        details.hostname=mpdDefaults.host;
+        details.port=mpdDefaults.port;
+        details.dir=mpdDefaults.dir;
+        details.password=mpdDefaults.passwd;
+        #ifdef ENABLE_HTTP_STREAM_PLAYBACK
+        details.streamUrl=QString();
+        #endif
+        details.allowLocalStreaming=true;
+        details.autoUpdate=false;
     }
     details.setDirReadable();
     return details;
@@ -235,13 +191,9 @@ MPDConnectionDetails Settings::connectionDetails(const QString &name)
 
 QList<MPDConnectionDetails> Settings::allConnections()
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    QStringList groups=KGlobal::config()->groupList();
-    #else
     QStringList groups=cfg.childGroups();
-    #endif
     QList<MPDConnectionDetails> connections;
-    foreach (const QString &grp, groups) {
+    for (const QString &grp: groups) {
         if (cfg.hasGroup(grp) && grp.startsWith("Connection")) {
             connections.append(connectionDetails(grp=="Connection" ? QString() : grp.mid(11)));
         }
@@ -254,33 +206,6 @@ QList<MPDConnectionDetails> Settings::allConnections()
     return connections;
 }
 
-#if defined ENABLE_KDE_SUPPORT && defined ENABLE_KWALLET
-bool Settings::openWallet()
-{
-    if (wallet) {
-        return true;
-    }
-
-    wallet=KWallet::Wallet::openWallet(KWallet::Wallet::LocalWallet(), QApplication::activeWindow() ? QApplication::activeWindow()->winId() : 0);
-    if (wallet) {
-        if (!wallet->hasFolder(PACKAGE_NAME)) {
-            wallet->createFolder(PACKAGE_NAME);
-        }
-        wallet->setFolder(PACKAGE_NAME);
-        return true;
-    }
-
-    return false;
-}
-#endif
-
-#ifndef ENABLE_KDE_SUPPORT
-QString Settings::iconTheme()
-{
-    return cfg.get("iconTheme", QString());
-}
-#endif
-
 bool Settings::showPlaylist()
 {
     return cfg.get("showPlaylist", true);
@@ -319,6 +244,11 @@ QSize Settings::mainWindowCollapsedSize()
     return cfg.get("mainWindowCollapsedSize", QSize());
 }
 
+bool Settings::maximized()
+{
+    return cfg.get("maximized", false);
+}
+
 bool Settings::useSystemTray()
 {
     return cfg.get("useSystemTray", false);
@@ -341,179 +271,68 @@ bool Settings::stopOnExit()
 
 bool Settings::storeCoversInMpdDir()
 {
-    return cfg.get("storeCoversInMpdDir", true);
+    return cfg.get("storeCoversInMpdDir", false);
 }
 
-bool Settings::storeLyricsInMpdDir()
+QString Settings::coverFilename()
 {
-    return cfg.get("storeLyricsInMpdDir", true);
-}
-
-bool Settings::storeStreamsInMpdDir()
-{
-    #ifdef Q_OS_WIN
-    bool def=false;
-    #else
-    bool def=version()>=CANTATA_MAKE_VERSION(0, 9, 50);
-    #endif
-    return cfg.get("storeStreamsInMpdDir", def);
-}
-
-bool Settings::storeBackdropsInMpdDir()
-{
-    return cfg.get("storeBackdropsInMpdDir", false);
-}
-
-#ifndef ENABLE_UBUNTU
-int Settings::libraryView()
-{
-    int v=version();
-    QString def=ItemView::modeStr(v<CANTATA_MAKE_VERSION(0, 9, 50) ? ItemView::Mode_SimpleTree : ItemView::Mode_DetailedTree);
-    return ItemView::toMode(cfg.get("libraryView", def));
-}
-
-int Settings::albumsView()
-{
-    return ItemView::toMode(cfg.get("albumsView", ItemView::modeStr(ItemView::Mode_IconTop)));
-}
-
-int Settings::folderView()
-{
-    int v=version();
-    QString def=ItemView::modeStr(v<CANTATA_MAKE_VERSION(1, 0, 51) ? ItemView::Mode_SimpleTree : ItemView::Mode_DetailedTree);
-    return ItemView::toMode(cfg.get("folderView", def));
-}
-
-int Settings::playlistsView()
-{
-    int v=version();
-    QString def=ItemView::modeStr(v<CANTATA_MAKE_VERSION(0, 9, 50)
-                                    ? ItemView::Mode_SimpleTree
-                                    : v<CANTATA_MAKE_VERSION(1, 0, 51)
-                                        ? ItemView::Mode_GroupedTree
-                                        : ItemView::Mode_DetailedTree);
-    return ItemView::toMode(cfg.get("playlistsView", def));
-}
-
-int Settings::streamsView()
-{
-    int v=version();
-    QString def=ItemView::modeStr(v<CANTATA_MAKE_VERSION(0, 9, 50) ? ItemView::Mode_SimpleTree : ItemView::Mode_DetailedTree);
-    return ItemView::toMode(cfg.get("streamsView", def));
-}
-
-int Settings::onlineView()
-{
-    return ItemView::toMode(cfg.get("onlineView", ItemView::modeStr(ItemView::Mode_DetailedTree)));
-}
-#endif
-
-bool Settings::libraryArtistImage()
-{
-    #ifndef ENABLE_UBUNTU
-    if (cfg.get("libraryArtistImage", false)) {
-        int view=libraryView();
-        return ItemView::Mode_SimpleTree!=view && ItemView::Mode_BasicTree!=view;
-    }
-    #endif
-    return false;
-}
-
-int Settings::libraryCoverSize()
-{
-    int size=cfg.get("libraryCoverSize", (int)(MusicLibraryItemAlbum::CoverMedium));
-    return size>MusicLibraryItemAlbum::CoverExtraLarge || size<0 ? MusicLibraryItemAlbum::CoverMedium : size;
-}
-
-int Settings::albumsCoverSize()
-{
-    int size=cfg.get("albumsCoverSize", (int)(MusicLibraryItemAlbum::CoverMedium));
-    return size>MusicLibraryItemAlbum::CoverExtraLarge || size<0 ? MusicLibraryItemAlbum::CoverMedium : size;
-}
-
-int Settings::albumSort()
-{
-    if (version()<CANTATA_MAKE_VERSION(1, 3, 52)) {
-        switch (cfg.get("albumSort", 0)) {
-        case 0: return AlbumsModel::Sort_AlbumArtist;
-        case 1: return AlbumsModel::Sort_ArtistAlbum;
-        case 2: return AlbumsModel::Sort_ArtistYear;
+    QString name=cfg.get("coverFilename", QString());
+    // name is empty, so for old configs try to get from MPD settings
+    if (name.isEmpty() && version()<CANTATA_MAKE_VERSION(2, 2, 51)) {
+        QStringList groups=cfg.childGroups();
+        QList<MPDConnectionDetails> connections;
+        for (const QString &grp: groups) {
+            if (cfg.hasGroup(grp) && grp.startsWith("Connection")) {
+                Configuration cfg(grp);
+                name=cfg.get("coverName", QString());
+                if (!name.isEmpty()) {
+                    // Use 1st non-empty
+                    saveCoverFilename(name);
+                    return name;
+                }
+            }
         }
     }
-    return AlbumsModel::toSort(cfg.get("albumSort", AlbumsModel::sortStr(AlbumsModel::Sort_AlbumArtist)));
+    return name;
 }
 
 int Settings::sidebar()
 {
     if (version()<CANTATA_MAKE_VERSION(1, 2, 52)) {
-        switch (cfg.get("sidebar", 1)) {
-        default:
-        case 1: return FancyTabWidget::Side|FancyTabWidget::Large;
-        case 2: return FancyTabWidget::Side|FancyTabWidget::Small;
-        case 3: return FancyTabWidget::Side|FancyTabWidget::Tab;
-        case 4: return FancyTabWidget::Top|FancyTabWidget::Tab;
-        case 5: return FancyTabWidget::Top|FancyTabWidget::Tab|FancyTabWidget::IconOnly;
-        case 6: return FancyTabWidget::Bot|FancyTabWidget::Tab;
-        case 7: return FancyTabWidget::Bot|FancyTabWidget::Tab|FancyTabWidget::IconOnly;
-        case 8: return FancyTabWidget::Side|FancyTabWidget::Large|FancyTabWidget::IconOnly;
-        case 9: return FancyTabWidget::Side|FancyTabWidget::Small|FancyTabWidget::IconOnly;
-        case 10: return FancyTabWidget::Side|FancyTabWidget::Tab|FancyTabWidget::IconOnly;
-        case 11: return FancyTabWidget::Bot|FancyTabWidget::Large;
-        case 12: return FancyTabWidget::Bot|FancyTabWidget::Large|FancyTabWidget::IconOnly;
-        case 13: return FancyTabWidget::Top|FancyTabWidget::Large;
-        case 14: return FancyTabWidget::Top|FancyTabWidget::Large|FancyTabWidget::IconOnly;
-        case 15: return FancyTabWidget::Bot|FancyTabWidget::Small;
-        case 16: return FancyTabWidget::Bot|FancyTabWidget::Small|FancyTabWidget::IconOnly;
-        case 17: return FancyTabWidget::Top|FancyTabWidget::Small;
-        case 18: return FancyTabWidget::Top|FancyTabWidget::Small|FancyTabWidget::IconOnly;
-        }
+        return (int)(FancyTabWidget::Side|FancyTabWidget::Large);
     } else {
         return cfg.get("sidebar", (int)(FancyTabWidget::Side|FancyTabWidget::Large))&FancyTabWidget::All_Mask;
     }
 }
 
-bool Settings::libraryYear()
+QSet<QString> Settings::composerGenres()
 {
-    return cfg.get("libraryYear", true);
+    return cfg.get("composerGenres", Song::composerGenres().toList()).toSet();
 }
 
-bool Settings::groupSingle()
+QSet<QString> Settings::singleTracksFolders()
 {
-    return cfg.get("groupSingle", MPDParseUtils::groupSingle());
+    return cfg.get("singleTracksFolders", QStringList()).toSet();
 }
 
-bool Settings::useComposer()
+MPDParseUtils::CueSupport Settings::cueSupport()
 {
-    return cfg.get("useComposer", Song::useComposer());
+    return MPDParseUtils::toCueSupport(cfg.get("cueSupport", MPDParseUtils::toStr(MPDParseUtils::Cue_Parse)));
 }
 
 QStringList Settings::lyricProviders()
 {
-    QStringList def;
-    def << "lyrics.wikia.com"
-        << "lyricstime.com"
-        << "lyricsreg.com"
-        << "lyricsmania.com"
-        << "metrolyrics.com"
-        << "azlyrics.com"
-        << "songlyrics.com"
-        << "elyrics.net"
-        << "lyricsdownload.com"
-        << "lyrics.com"
-        << "lyricsbay.com"
-        << "directlyrics.com"
-        << "loudson.gs"
-        << "teksty.org"
-        << "tekstowo.pl (POLISH)"
-        << "vagalume.uol.com.br"
-        << "vagalume.uol.com.br (PORTUGUESE)";
-    return cfg.get("lyricProviders", def);
+    return cfg.get("lyricProviders", QStringList() << "lyrics.wikia.com" << "lyricstime.com" << "lyricsreg.com"
+                                                   << "lyricsmania.com" << "metrolyrics.com" << "azlyrics.com"
+                                                   << "songlyrics.com" << "elyrics.net" << "lyricsdownload.com"
+                                                   << "lyrics.com" << "lyricsbay.com" << "directlyrics.com"
+                                                   << "loudson.gs" << "teksty.org" << "tekstowo.pl (POLISH)"
+                                                   << "vagalume.uol.com.br" << "vagalume.uol.com.br (PORTUGUESE)");
 }
 
 QStringList Settings::wikipediaLangs()
 {
-    QStringList def=QStringList() << "en:en";
-    return cfg.get("wikipediaLangs", def);
+    return cfg.get("wikipediaLangs", QStringList() << "en:en");
 }
 
 bool Settings::wikipediaIntroOnly()
@@ -523,7 +342,7 @@ bool Settings::wikipediaIntroOnly()
 
 int Settings::contextBackdrop()
 {
-    return getBoolAsInt("contextBackdrop", 1);
+    return getBoolAsInt("contextBackdrop", 0);
 }
 
 int Settings::contextBackdropOpacity()
@@ -538,7 +357,7 @@ int Settings::contextBackdropBlur()
 
 QString Settings::contextBackdropFile()
 {
-    return cfg.get("contextBackdropFile", QString());
+    return cfg.getFilePath("contextBackdropFile", QString());
 }
 
 bool Settings::contextDarkBackground()
@@ -588,11 +407,7 @@ QString Settings::page()
 
 QStringList Settings::hiddenPages()
 {
-    QStringList def=QStringList() << "PlayQueuePage" << "FolderPage" << "SearchPage" << "ContextPage";
-    QStringList config=cfg.get("hiddenPages", def);
-    if (version()<CANTATA_MAKE_VERSION(1, 2, 51) && !config.contains("SearchPage")) {
-        config.append("SearchPage");
-    }
+    QStringList config=cfg.get("hiddenPages", QStringList() << "PlayQueuePage" << "ContextPage");
     // If splitter auto-hide is enabled, then playqueue cannot be in sidebar!
     if (splitterAutoHide() && !config.contains("PlayQueuePage")) {
         config << "PlayQueuePage";
@@ -600,17 +415,6 @@ QStringList Settings::hiddenPages()
     return config;
 }
 
-#if !defined ENABLE_KDE_SUPPORT && !defined ENABLE_UBUNTU
-QString Settings::mediaKeysIface()
-{
-    #if defined Q_OS_WIN
-    return cfg.get("mediaKeysIface", MediaKeys::toString(MediaKeys::QxtInterface));
-    #else
-    return cfg.get("mediaKeysIface", MediaKeys::toString(MediaKeys::GnomeInteface));
-    #endif
-}
-#endif
-
 #ifdef ENABLE_DEVICES_SUPPORT
 bool Settings::overwriteSongs()
 {
@@ -621,24 +425,6 @@ bool Settings::showDeleteAction()
 {
     return cfg.get("showDeleteAction", false);
 }
-
-int Settings::devicesView()
-{
-    if (version()<CANTATA_MAKE_VERSION(1, 0, 51)) {
-        int v=cfg.get("devicesView", (int)ItemView::Mode_DetailedTree);
-        cfg.set("devicesView", ItemView::modeStr((ItemView::Mode)v));
-        return v;
-    } else {
-        return ItemView::toMode(cfg.get("devicesView", ItemView::modeStr(ItemView::Mode_DetailedTree)));
-    }
-}
-#endif
-
-#ifndef ENABLE_UBUNTU
-int Settings::searchView()
-{
-    return ItemView::toMode(cfg.get("searchView", ItemView::modeStr(ItemView::Mode_List)));
-}
 #endif
 
 int Settings::version()
@@ -671,15 +457,6 @@ QString Settings::httpInterface()
     return cfg.get("httpInterface", QString());
 }
 
-bool Settings::alwaysUseHttp()
-{
-    #ifdef ENABLE_HTTP_SERVER
-    return cfg.get("alwaysUseHttp", false);
-    #else
-    return false;
-    #endif
-}
-
 int Settings::playQueueView()
 {
     if (version()<CANTATA_MAKE_VERSION(1, 3, 53)) {
@@ -720,12 +497,17 @@ int Settings::playQueueBackgroundBlur()
 
 QString Settings::playQueueBackgroundFile()
 {
-    return cfg.get("playQueueBackgroundFile", QString());
+    return cfg.getFilePath("playQueueBackgroundFile", QString());
 }
 
 bool Settings::playQueueConfirmClear()
 {
-    return cfg.get("playQueueConfirmClear", true);
+    return cfg.get("playQueueConfirmClear", false);
+}
+
+bool Settings::playQueueSearch()
+{
+    return cfg.get("playQueueSearch", true);
 }
 
 bool Settings::playListsStartClosed()
@@ -755,6 +537,11 @@ bool Settings::paranoiaNeverSkip()
 {
     return cfg.get("paranoiaNeverSkip", true);
 }
+
+int Settings::paranoiaOffset()
+{
+    return cfg.get("paranoiaOffset", 0);
+}
 #endif
 
 #if defined CDDB_FOUND && defined MUSICBRAINZ5_FOUND
@@ -783,16 +570,7 @@ bool Settings::forceSingleClick()
 
 bool Settings::startHidden()
 {
-    return cfg.get("startHidden", false);
-}
-
-bool Settings::monoSidebarIcons()
-{
-    #ifdef Q_OS_WIN
-    return cfg.get("monoSidebarIcons", false);
-    #else
-    return cfg.get("monoSidebarIcons", true);
-    #endif
+    return useSystemTray() ? cfg.get("startHidden", false) : false;
 }
 
 bool Settings::showTimeRemaining()
@@ -810,7 +588,7 @@ QStringList Settings::hiddenOnlineProviders()
     return cfg.get("hiddenOnlineProviders", QStringList());
 }
 
-#if !defined Q_OS_WIN && !defined Q_OS_MAC
+#if (defined Q_OS_LINUX && defined QT_QTDBUS_FOUND) || (defined Q_OS_MAC && defined IOKIT_FOUND)
 bool Settings::inhibitSuspend()
 {
     return cfg.get("inhibitSuspend", false);
@@ -832,29 +610,12 @@ QString Settings::podcastDownloadPath()
     return Utils::fixPath(cfg.get("podcastDownloadPath", Utils::fixPath(QDir::homePath())+QLatin1String("Podcasts/")));
 }
 
-bool Settings::podcastAutoDownload()
+int Settings::podcastAutoDownloadLimit()
 {
-    return cfg.get("podcastAutoDownload", false);
-}
-
-int Settings::maxCoverUpdatePerIteration()
-{
-    return cfg.get("maxCoverUpdatePerIteration", 10, 1, 50);
-}
-
-int Settings::coverCacheSize()
-{
-    return cfg.get("coverCacheSize", 10, 1, 512);
-}
-
-QStringList Settings::cueFileCodecs()
-{
-    return cfg.get("cueFileCodecs", QStringList());
-}
-
-bool Settings::networkAccessEnabled()
-{
-    return cfg.get("networkAccessEnabled", true);
+    if (cfg.hasEntry("podcastAutoDownload")) {
+        return cfg.get("podcastAutoDownload", false) ? 1000 : 0;
+    }
+    return cfg.get("podcastAutoDownloadLimit", 0, 0, 1000);
 }
 
 int Settings::volumeStep()
@@ -867,75 +628,20 @@ Settings::StartupState Settings::startupState()
     return getStartupState(cfg.get("startupState", getStartupStateStr(SS_Previous)));
 }
 
-int Settings::undoSteps()
-{
-    return cfg.get("undoSteps", 10, 0, 20);
-}
-
 QString Settings::searchCategory()
 {
     return cfg.get("searchCategory", QString());
 }
 
-bool Settings::cacheScaledCovers()
-{
-    return cfg.get("cacheScaledCovers", true);
-}
-
 bool Settings::fetchCovers()
 {
     return cfg.get("fetchCovers", true);
 }
 
-int Settings::mpdPoll()
-{
-    return cfg.get("mpdPoll", 0, 0, 60);
-}
-
-int Settings::mpdListSize()
-{
-    return cfg.get("mpdListSize", 10000, 100, 65535);
-}
-
-#ifndef ENABLE_KDE_SUPPORT
 QString Settings::lang()
 {
     return cfg.get("lang", QString());
 }
-#endif
-
-bool Settings::alwaysUseLsInfo()
-{
-    return cfg.get("alwaysUseLsInfo", false);
-}
-
-bool Settings::showMenubar()
-{
-    return cfg.get("showMenubar", false);
-}
-
-int Settings::menu()
-{
-    #if defined Q_OS_WIN
-    int def=MC_Button;
-    #elif defined Q_OS_MAC
-    int def=MC_Bar;
-    #else
-    int def=Utils::Gnome==Utils::currentDe() ? MC_Button : Utils::Unity==Utils::currentDe() ? MC_Bar : (MC_Bar|MC_Button);
-    #endif
-    int v=cfg.get("menu", def)&(MC_Bar|MC_Button);
-    return 0==v ? MC_Bar : v;
-}
-
-bool Settings::touchFriendly()
-{
-    return cfg.get("touchFriendly", false);
-}
-
-bool Settings::filteredOnly()
-{
-    return cfg.get("filteredOnly", false);
-}
 
 bool Settings::showCoverWidget()
 {
@@ -947,6 +653,53 @@ bool Settings::showStopButton()
     return cfg.get("showStopButton", false);
 }
 
+bool Settings::showRatingWidget()
+{
+    return cfg.get("showRatingWidget", false);
+}
+
+bool Settings::showTechnicalInfo()
+{
+    return cfg.get("showTechnicalInfo", false);
+}
+
+bool Settings::infoTooltips()
+{
+    return cfg.get("infoTooltips", true);
+}
+
+QSet<QString> Settings::ignorePrefixes()
+{
+    return cfg.get("ignorePrefixes", Song::ignorePrefixes().toList()).toSet();
+}
+
+bool Settings::mpris()
+{
+    return cfg.get("mpris", true);
+}
+
+QString Settings::style()
+{
+    return cfg.get("style", QString());
+}
+
+#if !defined Q_OS_WIN && !defined Q_OS_MAC
+bool Settings::showMenubar()
+{
+    return cfg.get("showMenubar", false);
+}
+#endif
+
+bool Settings::useOriginalYear()
+{
+    return cfg.get("useOriginalYear", false);
+}
+
+bool Settings::responsiveSidebar()
+{
+    return cfg.get("responsiveSidebar", true);
+}
+
 void Settings::removeConnectionDetails(const QString &v)
 {
     if (v==currentConnection()) {
@@ -966,30 +719,16 @@ void Settings::saveConnectionDetails(const MPDConnectionDetails &v)
 
     QString n=MPDConnectionDetails::configGroupName(v.name);
 
-    cfg.beginGroup(n);
-    cfg.set("host", v.hostname);
-    cfg.set("port", (int)v.port);
-    cfg.set("dir", v.dir);
-    #if defined ENABLE_KDE_SUPPORT && defined ENABLE_KWALLET
-    if (KWallet::Wallet::isEnabled()) {
-        cfg.set("passwd", !v.password.isEmpty());
-        QString walletEntry=v.name.isEmpty() ? "mpd" : v.name;
-        if (v.password.isEmpty()) {
-            if (wallet) {
-                wallet->removeEntry(walletEntry);
-            }
-        }
-        else if (openWallet()) {
-            wallet->writePassword(walletEntry, v.password);
-        }
-    } else
-    #endif // ENABLE_KWALLET
-    cfg.set("passwd", v.password);
-    cfg.set("coverName", v.coverName);
+    Configuration grp(n);
+    grp.set("host", v.hostname);
+    grp.set("port", (int)v.port);
+    grp.setDirPath("dir", v.dir);
+    grp.set("passwd", v.password);
     #ifdef ENABLE_HTTP_STREAM_PLAYBACK
-    cfg.set("streamUrl", v.streamUrl);
+    grp.set("streamUrl", v.streamUrl);
     #endif
-    cfg.endGroup();
+    grp.set("allowLocalStreaming", v.allowLocalStreaming);
+    grp.set("autoUpdate", v.autoUpdate);
 }
 
 void Settings::saveCurrentConnection(const QString &v)
@@ -1024,9 +763,12 @@ void Settings::saveSplitterAutoHide(bool v)
 
 void Settings::saveMainWindowSize(const QSize &v)
 {
-    if (v.width()>16 && v.height()>16) {
-        cfg.set("mainWindowSize", v);
-    }
+    cfg.set("mainWindowSize", v);
+}
+
+void Settings::saveMaximized(bool v)
+{
+    cfg.set("maximized", v);
 }
 
 void Settings::saveMainWindowCollapsedSize(const QSize &v)
@@ -1061,66 +803,9 @@ void Settings::saveStoreCoversInMpdDir(bool v)
     cfg.set("storeCoversInMpdDir", v);
 }
 
-void Settings::saveStoreLyricsInMpdDir(bool v)
+void Settings::saveCoverFilename(const QString &v)
 {
-    cfg.set("storeLyricsInMpdDir", v);
-}
-
-void Settings::saveStoreBackdropsInMpdDir(bool v)
-{
-    cfg.set("storeBackdropsInMpdDir", v);
-}
-
-#ifndef ENABLE_UBUNTU
-void Settings::saveLibraryView(int v)
-{
-    cfg.set("libraryView", ItemView::modeStr((ItemView::Mode)v));
-}
-
-void Settings::saveAlbumsView(int v)
-{
-    cfg.set("albumsView", ItemView::modeStr((ItemView::Mode)v));
-}
-
-void Settings::saveFolderView(int v)
-{
-    cfg.set("folderView", ItemView::modeStr((ItemView::Mode)v));
-}
-
-void Settings::savePlaylistsView(int v)
-{
-    cfg.set("playlistsView", ItemView::modeStr((ItemView::Mode)v));
-}
-
-void Settings::saveStreamsView(int v)
-{
-    cfg.set("streamsView", ItemView::modeStr((ItemView::Mode)v));
-}
-
-void Settings::saveOnlineView(int v)
-{
-    cfg.set("onlineView", ItemView::modeStr((ItemView::Mode)v));
-}
-#endif
-
-void Settings::saveLibraryArtistImage(bool v)
-{
-    cfg.set("libraryArtistImage", v);
-}
-
-void Settings::saveLibraryCoverSize(int v)
-{
-    cfg.set("libraryCoverSize", v);
-}
-
-void Settings::saveAlbumsCoverSize(int v)
-{
-    cfg.set("albumsCoverSize", v);
-}
-
-void Settings::saveAlbumSort(int v)
-{
-    cfg.set("albumSort", AlbumsModel::sortStr((AlbumsModel::Sort)v));
+    cfg.set("coverFilename", v);
 }
 
 void Settings::saveSidebar(int v)
@@ -1128,19 +813,19 @@ void Settings::saveSidebar(int v)
     cfg.set("sidebar", v);
 }
 
-void Settings::saveLibraryYear(bool v)
+void Settings::saveComposerGenres(const QSet<QString> &v)
 {
-    cfg.set("libraryYear", v);
+    cfg.set("composerGenres", v.toList());
 }
 
-void Settings::saveGroupSingle(bool v)
+void Settings::saveSingleTracksFolders(const QSet<QString> &v)
 {
-    cfg.set("groupSingle", v);
+    cfg.set("singleTracksFolders", v.toList());
 }
 
-void Settings::saveUseComposer(bool v)
+void Settings::saveCueSupport(MPDParseUtils::CueSupport v)
 {
-    cfg.set("useComposer", v);
+    cfg.set("cueSupport", MPDParseUtils::toStr(v));
 }
 
 void Settings::saveLyricProviders(const QStringList &v)
@@ -1160,11 +845,6 @@ void Settings::saveWikipediaIntroOnly(bool v)
 
 void Settings::saveContextBackdrop(int v)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    if (cfg.get("contextBackdrop", QString()).length()>1) {
-        cfg.removeEntry("contextBackdrop");
-    }
-    #endif
     cfg.set("contextBackdrop", v);
 }
 
@@ -1180,7 +860,7 @@ void Settings::saveContextBackdropBlur(int v)
 
 void Settings::saveContextBackdropFile(const QString &v)
 {
-    cfg.set("contextBackdropFile", v);
+    cfg.setFilePath("contextBackdropFile", v);
 }
 
 void Settings::saveContextDarkBackground(bool v)
@@ -1233,13 +913,6 @@ void Settings::saveHiddenPages(const QStringList &v)
     cfg.set("hiddenPages", v);
 }
 
-#if !defined ENABLE_KDE_SUPPORT && !defined ENABLE_UBUNTU
-void Settings::saveMediaKeysIface(const QString &v)
-{
-    cfg.set("mediaKeysIface", v);
-}
-#endif
-
 #ifdef ENABLE_DEVICES_SUPPORT
 void Settings::saveOverwriteSongs(bool v)
 {
@@ -1250,18 +923,6 @@ void Settings::saveShowDeleteAction(bool v)
 {
     cfg.set("showDeleteAction", v);
 }
-
-void Settings::saveDevicesView(int v)
-{
-    cfg.set("devicesView", ItemView::modeStr((ItemView::Mode)v));
-}
-#endif
-
-#ifndef ENABLE_UBUNTU
-void Settings::saveSearchView(int v)
-{
-    cfg.set("searchView", ItemView::modeStr((ItemView::Mode)v));
-}
 #endif
 
 void Settings::saveStopFadeDuration(int v)
@@ -1301,11 +962,6 @@ void Settings::savePlayQueueScroll(bool v)
 
 void Settings::savePlayQueueBackground(int v)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    if (cfg.get("playQueueBackground", QString()).length()>1) {
-        cfg.removeEntry("playQueueBackground");
-    }
-    #endif
     cfg.set("playQueueBackground", v);
 }
 
@@ -1321,7 +977,7 @@ void Settings::savePlayQueueBackgroundBlur(int v)
 
 void Settings::savePlayQueueBackgroundFile(const QString &v)
 {
-    cfg.set("playQueueBackgroundFile", v);
+    cfg.setFilePath("playQueueBackgroundFile", v);
 }
 
 void Settings::savePlayQueueConfirmClear(bool v)
@@ -1329,6 +985,11 @@ void Settings::savePlayQueueConfirmClear(bool v)
     cfg.set("playQueueConfirmClear", v);
 }
 
+void Settings::savePlayQueueSearch(bool v)
+{
+    cfg.set("playQueueSearch", v);
+}
+
 void Settings::savePlayListsStartClosed(bool v)
 {
     cfg.set("playListsStartClosed", v);
@@ -1339,11 +1000,6 @@ void Settings::savePlayStream(bool v)
 {
     cfg.set("playStream", v);
 }
-
-bool Settings::stopHttpStreamOnPause()
-{
-    return cfg.get("stopHttpStreamOnPause", true);
-}
 #endif
 
 #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
@@ -1361,6 +1017,11 @@ void Settings::saveParanoiaNeverSkip(bool v)
 {
     cfg.set("paranoiaNeverSkip", v);
 }
+
+void Settings::saveParanoiaOffset(int v)
+{
+    cfg.set("paranoiaOffset", v);
+}
 #endif
 
 #if defined CDDB_FOUND && defined MUSICBRAINZ5_FOUND
@@ -1392,11 +1053,6 @@ void Settings::saveStartHidden(bool v)
     cfg.set("startHidden", v);
 }
 
-void Settings::saveMonoSidebarIcons(bool v)
-{
-    cfg.set("monoSidebarIcons", v);
-}
-
 void Settings::saveShowTimeRemaining(bool v)
 {
     cfg.set("showTimeRemaining", v);
@@ -1412,7 +1068,7 @@ void Settings::saveHiddenOnlineProviders(const QStringList &v)
     cfg.set("hiddenOnlineProviders", v);
 }
 
-#if !defined Q_OS_WIN && !defined Q_OS_MAC
+#if (defined Q_OS_LINUX && defined QT_QTDBUS_FOUND) || (defined Q_OS_MAC && defined IOKIT_FOUND)
 void Settings::saveInhibitSuspend(bool v)
 {
     cfg.set("inhibitSuspend", v);
@@ -1434,9 +1090,17 @@ void Settings::savePodcastDownloadPath(const QString &v)
     cfg.set("podcastDownloadPath", v);
 }
 
-void Settings::savePodcastAutoDownload(bool v)
+void Settings::savePodcastAutoDownloadLimit(int v)
 {
-    cfg.set("podcastAutoDownload", v);
+    if (cfg.hasEntry("podcastAutoDownload")) {
+        cfg.removeEntry("podcastAutoDownload");
+    }
+    cfg.set("podcastAutoDownloadLimit", v);
+}
+
+void Settings::saveVolumeStep(int v)
+{
+    cfg.set("volumeStep", v);
 }
 
 void Settings::saveStartupState(int v)
@@ -1449,68 +1113,89 @@ void Settings::saveSearchCategory(const QString &v)
     cfg.set("searchCategory", v);
 }
 
-void Settings::saveCacheScaledCovers(bool v)
-{
-    cfg.set("cacheScaledCovers", v);
-}
-
 void Settings::saveFetchCovers(bool v)
 {
     cfg.set("fetchCovers", v);
 }
 
-#ifndef ENABLE_KDE_SUPPORT
 void Settings::saveLang(const QString &v)
 {
     cfg.set("lang", v);
 }
-#endif
-
-void Settings::saveShowMenubar(bool v)
-{
-    cfg.set("showMenubar", v);
-}
-
-void Settings::saveTouchFriendly(bool v)
-{
-    cfg.set("touchFriendly", v);
-}
-
-void Settings::saveFilteredOnly(bool v)
-{
-    cfg.set("filteredOnly", v);
-}
 
 void Settings::saveShowCoverWidget(bool v)
 {
-    return cfg.set("showCoverWidget", v);
+    cfg.set("showCoverWidget", v);
 }
 
 void Settings::saveShowStopButton(bool v)
 {
-    return cfg.set("showStopButton", v);
+    cfg.set("showStopButton", v);
 }
 
-void Settings::save(bool force)
+void Settings::saveShowRatingWidget(bool v)
 {
-    if (force) {
-        if (AP_NotConfigured!=state) {
-            if (version()!=PACKAGE_VERSION || AP_FirstRun==state) {
-                cfg.set("version", PACKAGE_VERSION_STRING);
-                ver=PACKAGE_VERSION;
-            }
+    cfg.set("showRatingWidget", v);
+}
+
+void Settings::saveShowTechnicalInfo(bool v)
+{
+    cfg.set("showTechnicalInfo", v);
+}
+
+void Settings::saveInfoTooltips(bool v)
+{
+    cfg.set("infoTooltips", v);
+}
+
+void Settings::saveIgnorePrefixes(const QSet<QString> &v)
+{
+    cfg.set("ignorePrefixes", v.toList());
+}
+
+void Settings::saveMpris(bool v)
+{
+    cfg.set("mpris", v);
+}
+
+void Settings::saveReplayGain(const QString &conn, const QString &v)
+{
+    QString n=MPDConnectionDetails::configGroupName(conn);
+    Configuration grp(n);
+    grp.set("replayGain", v);
+}
+
+void Settings::saveStyle(const QString &v)
+{
+    cfg.set("style", v);
+}
+
+#if !defined Q_OS_WIN && !defined Q_OS_MAC
+void Settings::saveShowMenubar(bool v)
+{
+    cfg.set("showMenubar", v);
+}
+#endif
+
+void Settings::saveUseOriginalYear(bool v)
+{
+    cfg.set("useOriginalYear", v);
+}
+
+void Settings::saveResponsiveSidebar(bool v)
+{
+    cfg.set("responsiveSidebar", v);
+}
+
+void Settings::save()
+{
+    if (AP_NotConfigured!=state) {
+        if (version()!=PACKAGE_VERSION || AP_FirstRun==state) {
+            cfg.set("version", PACKAGE_VERSION_STRING);
+            ver=PACKAGE_VERSION;
         }
-        cfg.sync();
-        if (timer) {
-            timer->stop();
-        }
-    } else {
-        if (!timer) {
-            timer=new QTimer(this);
-            connect(timer, SIGNAL(timeout()), this, SLOT(actualSave()));
-        }
-        timer->start(30*1000);
     }
+    cfg.sync();
 }
 
 void Settings::clearVersion()
@@ -1519,11 +1204,6 @@ void Settings::clearVersion()
     state=AP_NotConfigured;
 }
 
-void Settings::actualSave()
-{
-    save(true);
-}
-
 int Settings::getBoolAsInt(const QString &key, int def)
 {
     // Old config, sometimes bool was used - which has now been converted
@@ -1537,4 +1217,3 @@ int Settings::getBoolAsInt(const QString &key, int def)
     }
     return v.toInt();
 }
-
