@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -34,8 +34,8 @@ public:
     SingleApplication(int &argc, char **argv);
     virtual ~SingleApplication() { }
 
-    bool start();
-    void loadFiles();
+    bool start(const QStringList &files);
+    void loadFiles(const QStringList &files);
 
 private:
     void load(const QStringList &files);
