@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -43,17 +43,16 @@ public:
     Action *increaseVolumeAction;
     Action *decreaseVolumeAction;
     Action *savePlayQueueAction;
-    Action *addToPlayQueueAction;
-    Action *addRandomToPlayQueueAction;
+    Action *appendToPlayQueueAction;
+    Action *appendToPlayQueueAndPlayAction;
+    Action *addToPlayQueueAndPlayAction;
+    Action *insertAfterCurrentAction;
     Action *replacePlayQueueAction;
     Action *addWithPriorityAction;
     Action *addToStoredPlaylistAction;
-    Action *addPrioHighestAction;  // 255
-    Action *addPrioHighAction;     // 200
-    Action *addPrioMediumAction;   // 125
-    Action *addPrioLowAction;      // 50
-    Action *addPrioDefaultAction;  // 0
-    Action *addPrioCustomAction;
+    Action *setPriorityAction;
+    Action *addToPlayQueueMenuAction;
+    Action *addRandomAlbumToPlayQueueAction;
     #ifdef TAGLIB_FOUND
     Action *editTagsAction;
     Action *organiseFilesAction;
@@ -68,6 +67,10 @@ public:
     Action *setCoverAction;
     Action *removeAction;
     Action *searchAction;
+    Action *zoomInAction;
+    Action *zoomOutAction;
+
+    void enableAddToPlayQueue(bool en);
 
 private:
     StdActions(const StdActions &o);
