@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,77 +25,67 @@
 #define TRAYITEM_H
 
 #include <QObject>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KStatusNotifierItem>
-class KMenu;
-#else
 #include <QSystemTrayIcon>
 #include "support/icon.h"
 class QMenu;
-#endif
 #include "config.h"
 
 #ifdef QT_QTDBUS_FOUND
 class Notify;
 #endif
 class MainWindow;
+class QImage;
 struct Song;
+class Action;
 
 class TrayItem : public QObject
 {
     Q_OBJECT
 public:
     TrayItem(MainWindow *p);
-    virtual ~TrayItem() { }
+    ~TrayItem() override { }
 
+    void showMessage(const QString &title, const QString &text, const QImage &img=QImage());
     void setup();
-    bool isActive() const { return 0!=trayItem; }
-    #ifdef ENABLE_KDE_SUPPORT
-    void setIconByName(const QString &name) {
-        if (trayItem) {
-            trayItem->setIconByName(name);
-        }
-    }
+    #ifdef Q_OS_MAC
+    bool isActive() const { return false; }
+    void setIcon(const QIcon &) { }
+    void setToolTip(const QString &, const QString &, const QString &) { }
     #else
+    bool isActive() const { return nullptr!=trayItem; }
     void setIcon(const QIcon &icon) {
         if (trayItem) {
             trayItem->setIcon(icon);
         }
     }
-    #endif
     void setToolTip(const QString &iconName, const QString &title, const QString &subTitle) {
         if (trayItem) {
-            #ifdef ENABLE_KDE_SUPPORT
-            trayItem->setToolTip(iconName, title, subTitle);
-            #else
             Q_UNUSED(iconName)
             Q_UNUSED(subTitle)
             trayItem->setToolTip(title);
-            #endif
         }
     }
+    #endif
     void songChanged(const Song &song, bool isPlaying);
+    void updateConnections();
+    void updateOutputs();
 
 private Q_SLOTS:
-    #ifdef ENABLE_KDE_SUPPORT
-    void clicked();
-    void trayItemScrollRequested(int delta, Qt::Orientation orientation);
-    #else
     void trayItemClicked(QSystemTrayIcon::ActivationReason reason);
-    #endif
 
 private:
+    #ifndef Q_OS_MAC
+
     MainWindow *mw;
-    #ifdef ENABLE_KDE_SUPPORT
-    KStatusNotifierItem *trayItem;
-    KMenu *trayItemMenu;
-    #else
     QSystemTrayIcon *trayItem;
     QMenu *trayItemMenu;
-    #endif
     #ifdef QT_QTDBUS_FOUND
     Notify *notification;
     #endif
+    Action *connectionsAction;
+    Action *outputsAction;
+
+    #endif
 };
 
 #endif
