@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -29,12 +29,11 @@
 #include "gui/settings.h"
 #include "support/thread.h"
 #include "support/globalstatic.h"
+#include "mpd-interface/mpdconnection.h"
 #include <QFile>
 #include <QUrl>
 #include <QTimer>
-#if QT_VERSION >= 0x050000
 #include <QUrlQuery>
-#endif
 
 #include <QDebug>
 static bool debugIsEnabled=false;
@@ -61,16 +60,21 @@ GLOBAL_STATIC(HttpServer, instance)
 
 #ifdef ENABLE_HTTP_SERVER
 HttpServer::HttpServer()
-    : QObject(0)
-    , thread(0)
-    , socket(0)
-    , closeTimer(0)
+    : QObject(nullptr)
+    , thread(nullptr)
+    , socket(nullptr)
+    , closeTimer(nullptr)
 {
-    force=Settings::self()->alwaysUseHttp();
-    connect(MPDConnection::self(), SIGNAL(socketAddress(QString)), this, SLOT(mpdAddress(QString)));
     connect(MPDConnection::self(), SIGNAL(cantataStreams(QList<Song>,bool)), this, SLOT(cantataStreams(QList<Song>,bool)));
     connect(MPDConnection::self(), SIGNAL(cantataStreams(QStringList)), this, SLOT(cantataStreams(QStringList)));
     connect(MPDConnection::self(), SIGNAL(removedIds(QSet<qint32>)), this, SLOT(removedIds(QSet<qint32>)));
+    connect(MPDConnection::self(), SIGNAL(ifaceIp(QString)), this, SLOT(ifaceIp(QString)));
+}
+
+bool HttpServer::isAlive() const
+{
+    // started on demand, but only start if allowed
+    return MPDConnection::self()->getDetails().allowLocalStreaming;
 }
 
 bool HttpServer::start()
@@ -87,12 +91,12 @@ bool HttpServer::start()
 
     DBUG << "open new socket";
     quint16 prevPort=Settings::self()->httpAllocatedPort();
-    bool newThread=0==thread;
+    bool newThread=nullptr==thread;
     if (newThread) {
         thread=new Thread("HttpServer");
     }
     socket=new HttpSocket(Settings::self()->httpInterface(), prevPort);
-    socket->mpdAddress(mpdAddr);
+    socket->mpdAddress(MPDConnection::self()->ipAddress());
     connect(this, SIGNAL(terminateSocket()), socket, SLOT(terminate()), Qt::QueuedConnection);
     if (socket->serverPort()!=prevPort) {
         Settings::self()->saveHttpAllocatedPort(socket->serverPort());
@@ -110,7 +114,7 @@ void HttpServer::stop()
     if (socket) {
         DBUG;
         emit terminateSocket();
-        socket=0;
+        socket=nullptr;
     }
 }
 
@@ -122,22 +126,36 @@ void HttpServer::readConfig()
         return;
     }
 
-    bool wasStarted=0!=socket;
+    bool wasStarted=nullptr!=socket;
     stop();
     if (wasStarted) {
         start();
     }
 }
 
+static inline QString serverUrl(const QString &ip, quint16 port)
+{
+    return QLatin1String("http://")+ip+QLatin1Char(':')+QString::number(port);
+}
+
 QString HttpServer::address() const
 {
-    return socket ? QLatin1String("http://")+socket->address()+QChar(':')+QString::number(socket->serverPort())
-                  : QLatin1String("http://127.0.0.1:*");
+    return socket ? serverUrl(currentIfaceIp, socket->serverPort()) : QLatin1String("http://127.0.0.1:*");
 }
 
 bool HttpServer::isOurs(const QString &url) const
 {
-    return 0!=socket ? url.startsWith(address()+"/") : false;
+    if (nullptr==socket || !url.startsWith(QLatin1String("http://"))) {
+        return false;
+    }
+
+    for (const QString &ip: ipAddresses) {
+        if (url.startsWith(serverUrl(ip, socket->serverPort()))) {
+            return true;
+        }
+    }
+
+    return false;
 }
 
 QByteArray HttpServer::encodeUrl(const Song &s)
@@ -146,17 +164,18 @@ QByteArray HttpServer::encodeUrl(const Song &s)
     if (!start()) {
         return QByteArray();
     }
-    #if QT_VERSION < 0x050000
-    QUrl url;
-    QUrl &query=url;
-    #else
     QUrl url;
     QUrlQuery query;
-    #endif
     url.setScheme("http");
-    url.setHost(socket->address());
+    url.setHost(currentIfaceIp);
     url.setPort(socket->serverPort());
+    #ifdef Q_OS_WIN
+    // Use a query item, as s.file might have a driver specifier
+    query.addQueryItem("file", s.file);
+    url.setPath("/"+Utils::getFile(s.file));
+    #else
     url.setPath(s.file);
+    #endif
     if (!s.album.isEmpty()) {
         query.addQueryItem("album", s.album);
     }
@@ -172,8 +191,8 @@ QByteArray HttpServer::encodeUrl(const Song &s)
     if (!s.title.isEmpty()) {
         query.addQueryItem("title", s.title);
     }
-    if (!s.genre.isEmpty()) {
-        query.addQueryItem("genre", s.genre);
+    if (!s.genres[0].isEmpty()) {
+        query.addQueryItem("genre", s.firstGenre());
     }
     if (s.disc) {
         query.addQueryItem("disc", QString::number(s.disc));
@@ -187,16 +206,12 @@ QByteArray HttpServer::encodeUrl(const Song &s)
     if (s.track) {
         query.addQueryItem("track", QString::number(s.track));
     }
-    #ifdef ENABLE_ONLINE_SERVICES
     if (s.isFromOnlineService()) {
         query.addQueryItem("onlineservice", s.onlineService());
     }
-    #endif
     query.addQueryItem("id", QString::number(s.id));
     query.addQueryItem("cantata", "song");
-    #if QT_VERSION >= 0x050000
     url.setQuery(query);
-    #endif
     DBUG << "encoded as" << url.toString();
     return url.toEncoded();
 }
@@ -226,28 +241,18 @@ QByteArray HttpServer::encodeUrl(const QString &file)
     #endif
     s.file=file;
     #endif
-    return encodeUrl(s);
+    return /*s.isEmpty() ? QByteArray() :*/ encodeUrl(s);
 }
 
 Song HttpServer::decodeUrl(const QString &url) const
 {
-    #if QT_VERSION >= 0x050000
     return decodeUrl(QUrl(url));
-    #else
-    QUrl u;
-    u.setEncodedUrl(url.toUtf8());
-    return decodeUrl(u);
-    #endif
 }
 
 Song HttpServer::decodeUrl(const QUrl &url) const
 {
     Song s;
-    #if QT_VERSION < 0x050000
-    const QUrl &q=url;
-    #else
     QUrlQuery q(url);
-    #endif
 
     if (q.hasQueryItem("cantata") && q.queryItemValue("cantata")=="song") {
         if (q.hasQueryItem("album")) {
@@ -266,7 +271,7 @@ Song HttpServer::decodeUrl(const QUrl &url) const
             s.title=q.queryItemValue("title");
         }
         if (q.hasQueryItem("genre")) {
-            s.genre=q.queryItemValue("genre");
+            s.addGenre(q.queryItemValue("genre"));
         }
         if (q.hasQueryItem("disc")) {
             s.disc=q.queryItemValue("disc").toInt();
@@ -283,19 +288,17 @@ Song HttpServer::decodeUrl(const QUrl &url) const
         if (q.hasQueryItem("id")) {
             s.id=q.queryItemValue("id").toInt();
         }
-        #ifdef ENABLE_ONLINE_SERVICES
         if (q.hasQueryItem("onlineservice")) {
             s.setIsFromOnlineService(q.queryItemValue("onlineservice"));
         }
-        #endif
-        s.file=url.path();
-        s.type=Song::CantataStream;
         #ifdef Q_OS_WIN
-        s.file=fixWindowsPath(s.file);
+        s.file=fixWindowsPath(q.queryItemValue("file"));
+        #else
+        s.file=url.path();
         #endif
+        s.type=Song::CantataStream;
         #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
-        if (s.file.startsWith(QChar('/')+Song::constCddaProtocol)) {
-            s.file=s.file.mid(1);
+        if (s.file.startsWith(Song::constCddaProtocol)) {
             s.type=Song::Cdda;
         }
         #endif
@@ -316,15 +319,10 @@ void HttpServer::startCloseTimer()
     closeTimer->start(1000);
 }
 
-void HttpServer::mpdAddress(const QString &a)
-{
-    mpdAddr=a;
-}
-
 void HttpServer::cantataStreams(const QStringList &files)
 {
     DBUG << files;
-    foreach (const QString &f, files) {
+    for (const QString &f: files) {
         Song s=HttpServer::self()->decodeUrl(f);
         if (s.isCantataStream() || s.isCdda()) {
             start();
@@ -340,7 +338,7 @@ void HttpServer::cantataStreams(const QList<Song> &songs, bool isUpdate)
         streamIds.clear();
     }
 
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         streamIds.insert(s.id);
     }
 
@@ -359,4 +357,16 @@ void HttpServer::removedIds(const QSet<qint32> &ids)
     }
 }
 
+void HttpServer::ifaceIp(const QString &ip)
+{
+    DBUG << "MPD interface ip" << ip;
+    if (ip.isEmpty()) {
+        return;
+    }
+    currentIfaceIp=ip;
+    ipAddresses.insert(ip);
+}
+
 #endif
+
+#include "moc_httpserver.cpp"
