@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -27,7 +27,7 @@
 #include <QObject>
 #include <QByteArray>
 #include <QSet>
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 #include "config.h"
 
 class HttpSocket;
@@ -37,7 +37,7 @@ class QTimer;
 
 class HttpServer : public QObject
 {
-    #if defined ENABLE_HTTP_SERVER || defined ENABLE_KDE_SUPPORT
+    #if defined ENABLE_HTTP_SERVER
     Q_OBJECT
     #endif
 
@@ -47,12 +47,11 @@ public:
 
     static HttpServer * self();
 
-    virtual ~HttpServer() { }
+    ~HttpServer() override { }
 
     #ifdef ENABLE_HTTP_SERVER
     HttpServer();
-    bool isAlive() const { return true; } // Started on-demamnd!
-    bool forceUsage() const { return force; }
+    bool isAlive() const; // Started on-demamnd!
     void readConfig();
     QString address() const;
     bool isOurs(const QString &url) const;
@@ -63,7 +62,6 @@ public:
     #else
     HttpServer() { }
     bool isAlive() const { return false; } // Not used!
-    bool forceUsage() const { return false; }
     void readConfig() { }
     QString address() const { return QString(); }
     bool isOurs(const QString &) const { return false; }
@@ -80,22 +78,23 @@ private:
 private Q_SLOTS:
     void stop();
     void startCloseTimer();
-    void mpdAddress(const QString &a);
     void cantataStreams(const QStringList &files);
     void cantataStreams(const QList<Song> &songs, bool isUpdate);
     void removedIds(const QSet<qint32> &ids);
+    void ifaceIp(const QString &ip);
 
 Q_SIGNALS:
     void terminateSocket();
 
 private:
-    bool force;
     Thread *thread;
     HttpSocket *socket;
 
-    QString mpdAddr;
     QSet<qint32> streamIds; // Currently playing MPD stream IDs
     QTimer *closeTimer;
+
+    QString currentIfaceIp;
+    QSet<QString> ipAddresses;
     #endif
 };
 
