@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -30,46 +30,27 @@
 #include "devices/extractjob.h"
 #endif
 #include <QTcpSocket>
-#include <QNetworkInterface>
 #include <QStringList>
 #include <QTextStream>
 #include <QFile>
 #include <QUrl>
 #include <QNetworkProxy>
-#if QT_VERSION >= 0x050000
 #include <QUrlQuery>
-#endif
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KMimeType>
-#endif
 #include <QFileInfo>
-#if defined TAGLIB_FOUND && !defined ENABLE_EXTERNAL_TAGS
-#include "tags/tags.h"
-#endif
 #include <QDebug>
 #define DBUG if (HttpServer::debugEnabled()) qWarning() << "HttpSocket" << __FUNCTION__
 
+static const quint64 constMaxBuffer = 32768;
+
 static QString detectMimeType(const QString &file)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    QString km=KMimeType::findByPath(file)->name();
-    if (!km.isEmpty() && (km.startsWith("audio/") || km.startsWith("application/"))) {
-        return QLatin1String("audio/x-ape")==km ? QLatin1String("audio/x-monkeys-audio") : km;
-    }
-    #endif
     QString suffix = QFileInfo(file).suffix().toLower();
     if (suffix == QLatin1String("mp3")) {
         return QLatin1String("audio/mpeg");
     }
-    #if defined TAGLIB_FOUND && !defined ENABLE_EXTERNAL_TAGS
-    if (suffix == QLatin1String("ogg")) {
-        return Tags::oggMimeType(file);
-    }
-    #else
     if (suffix == QLatin1String("ogg")) {
         return QLatin1String("audio/ogg");
     }
-    #endif
     if (suffix == QLatin1String("flac")) {
         return QLatin1String("audio/x-flac");
     }
@@ -141,18 +122,6 @@ static void writeMimeType(const QString &mimeType, QTcpSocket *socket, qint32 fr
     }
 }
 
-static QHostAddress getAddress(const QNetworkInterface &iface)
-{
-    QList<QNetworkAddressEntry> addresses=iface.addressEntries();
-    foreach (const QNetworkAddressEntry &addr, addresses) {
-        QHostAddress hostAddress=addr.ip();
-        if (QAbstractSocket::IPv4Protocol==hostAddress.protocol()) {
-            return hostAddress;
-        }
-    }
-    return QHostAddress();
-}
-
 static int getSep(const QByteArray &a, int pos)
 {
     for (int i=pos+1; i<a.length(); ++i) {
@@ -185,19 +154,9 @@ static QList<QByteArray> split(const QByteArray &a)
     return rv;
 }
 
-static bool isFromMpd(const QStringList &params)
-{
-    foreach (const QString &str, params) {
-        if (str.startsWith("User-Agent:") && str.contains("Music Player Daemon")) {
-            return true;
-        }
-    }
-    return false;
-}
-
 static void getRange(const QStringList &params, qint32 &from, qint32 &to)
 {
-    foreach (const QString &str, params) {
+    for (const QString &str: params) {
         if (str.startsWith("Range:")) {
             int start=str.indexOf("bytes=");
             if (start>0) {
@@ -215,78 +174,30 @@ static void getRange(const QStringList &params, qint32 &from, qint32 &to)
 }
 
 HttpSocket::HttpSocket(const QString &iface, quint16 port)
-    : QTcpServer(0)
+    : QTcpServer(nullptr)
     , cfgInterface(iface)
     , terminated(false)
 {
-    // Get network address...
-    QHostAddress a;
-    if (!iface.isEmpty()) {
-        // Try rquested interface...
-        QNetworkInterface netIface=QNetworkInterface::interfaceFromName(iface);
-        if (netIface.isValid()) {
-            a=getAddress(netIface);
-        }
+    if (!openPort(port)) {
+        openPort(0);
     }
 
-    if (a.isNull()) {
-        // Hmm... try first active interface...
-        QNetworkInterface loIface;
-        QList<QNetworkInterface> ifaces=QNetworkInterface::allInterfaces();
-        foreach (const QNetworkInterface &iface, ifaces) {
-            if (iface.flags()&QNetworkInterface::IsUp) {
-                if (QLatin1String("lo")==iface.name()) {
-                    loIface=iface;
-                } else {
-                    a=getAddress(iface);
-                    if (!a.isNull()) {
-                        break;
-                    }
-                }
-            }
-        }
-        if (a.isNull() && !loIface.isValid()) {
-            // Get address of 'loopback' interface...
-            a=getAddress(loIface);
-        }
-    }
-
-    if (!openPort(a, port)) {
-        openPort(a, 0);
-    }
-
-    if (isListening() && ifaceAddress.isEmpty()) {
-        ifaceAddress=QLatin1String("127.0.0.1");
-    }
-
-    DBUG << isListening() << ifaceAddress << serverPort();
+    DBUG << isListening() << serverPort();
 
     connect(MPDConnection::self(), SIGNAL(socketAddress(QString)), this, SLOT(mpdAddress(QString)));
     connect(MPDConnection::self(), SIGNAL(cantataStreams(QList<Song>,bool)), this, SLOT(cantataStreams(QList<Song>,bool)));
     connect(MPDConnection::self(), SIGNAL(cantataStreams(QStringList)), this, SLOT(cantataStreams(QStringList)));
     connect(MPDConnection::self(), SIGNAL(removedIds(QSet<qint32>)), this, SLOT(removedIds(QSet<qint32>)));
+    connect(this, SIGNAL(newConnection()), SLOT(handleNewConnection()));
 }
 
-bool HttpSocket::openPort(const QHostAddress &a, quint16 p)
+bool HttpSocket::openPort(quint16 p)
 {
-    if (!a.isNull() && listen(a, p)) {
-        ifaceAddress=a.toString();
-        return true;
-    }
-    // Listen probably failed due to proxy, so unset and try again!
     setProxy(QNetworkProxy::NoProxy);
-    if (!a.isNull() && listen(a, p)) {
-        ifaceAddress=a.toString();
-        return true;
-    }
-
     if (listen(QHostAddress::Any, p)) {
-        ifaceAddress=serverAddress().toString();
         return true;
     }
-
     if (listen(QHostAddress::LocalHost, p)) {
-        ifaceAddress=QLatin1String("127.0.0.1");
         return true;
     }
 
@@ -304,12 +215,33 @@ void HttpSocket::terminate()
     deleteLater();
 }
 
-void HttpSocket::incomingConnection(int socket)
+void HttpSocket::handleNewConnection()
 {
-    QTcpSocket *s = new QTcpSocket(this);
-    connect(s, SIGNAL(readyRead()), this, SLOT(readClient()));
-    connect(s, SIGNAL(disconnected()), this, SLOT(discardClient()));
-    s->setSocketDescriptor(socket);
+    DBUG;
+    while (hasPendingConnections()) {
+        QTcpSocket *socket = nextPendingConnection();
+
+        // prevent clients from sending too much data
+        socket->setReadBufferSize(constMaxBuffer);
+
+        static const QLatin1String constIpV6Prefix("::ffff:");
+
+        QString peer=socket->peerAddress().toString();
+        QString ifaceAddress=serverAddress().toString();
+        const bool hostOk=peer==ifaceAddress || peer==mpdAddr || peer==(constIpV6Prefix+mpdAddr) ||
+                          peer==QLatin1String("127.0.0.1") || peer==(constIpV6Prefix+QLatin1String("127.0.0.1"));
+
+        DBUG << "peer:" << peer << "mpd:" << mpdAddr << "iface:" << ifaceAddress << "ok:" << hostOk;
+        if (!hostOk) {
+            sendErrorResponse(socket, 400);
+            socket->close();
+            DBUG << "Not from valid host";
+            return;
+        }
+
+        connect(socket, SIGNAL(readyRead()), this, SLOT(readClient()));
+        connect(socket, SIGNAL(disconnected()), this, SLOT(discardClient()));
+    }
 }
 
 void HttpSocket::readClient()
@@ -319,36 +251,26 @@ void HttpSocket::readClient()
     }
 
     QTcpSocket *socket = static_cast<QTcpSocket *>(sender());
+    if (!socket) {
+        return;
+    }
+
+    if (socket->bytesAvailable() >= constMaxBuffer) {
+        // Request too large, reject
+        sendErrorResponse(socket, 400);
+        socket->close();
+        DBUG << "Request too large";
+        return;
+    }
+
     if (socket->canReadLine()) {
         QList<QByteArray> tokens = split(socket->readLine()); // QRegExp("[ \r\n][ \r\n]*"));
         if (tokens.length()>=2 && "GET"==tokens[0]) {
             QStringList params = QString(socket->readAll()).split(QRegExp("[\r\n][\r\n]*"));
 
             DBUG << "params" << params << "tokens" << tokens;
-            if (!isFromMpd(params)) {
-                sendErrorResponse(socket, 400);
-                socket->close();
-                DBUG << "Not from MPD";
-                return;
-            }
-
-            QString peer=socket->peerAddress().toString();
-            bool hostOk=peer==ifaceAddress || peer==mpdAddr || peer==QLatin1String("127.0.0.1");
-
-            DBUG << "peer:" << peer << "mpd:" << mpdAddr << "iface:" << ifaceAddress << "ok:" << hostOk;
-            if (!hostOk) {
-                sendErrorResponse(socket, 400);
-                socket->close();
-                DBUG << "Not from valid host";
-                return;
-            }
-
             QUrl url(QUrl::fromEncoded(tokens[1]));
-            #if QT_VERSION < 0x050000
-            QUrl &q=url;
-            #else
             QUrlQuery q(url);
-            #endif
             bool ok=false;
             qint32 readBytesFrom=0;
             qint32 readBytesTo=0;
@@ -370,7 +292,7 @@ void HttpSocket::readClient()
                     QStringList parts=song.file.split("/", QString::SkipEmptyParts);
                     if (parts.length()>=3) {
                         QString dev=QLatin1Char('/')+parts.at(1)+QLatin1Char('/')+parts.at(2);
-                        CdParanoia cdparanoia(dev, false, false, true);
+                        CdParanoia cdparanoia(dev, false, false, true, Settings::self()->paranoiaOffset());
 
                         if (cdparanoia) {
                             int firstSector = cdparanoia.firstSectorOfTrack(song.id);
@@ -464,7 +386,7 @@ void HttpSocket::readClient()
                             } while (readPos<totalBytes && !stop && !terminated);
                         }
                     } else {
-                        DBUG << "Filed to open" << song.file;
+                        DBUG << "Failed to open" << song.file;
                     }
                 }
             }
@@ -476,8 +398,14 @@ void HttpSocket::readClient()
             socket->close();
 
             if (QTcpSocket::UnconnectedState==socket->state()) {
-                delete socket;
+                socket->deleteLater();
             }
+        } else {
+            // Bad Request
+            sendErrorResponse(socket, 400);
+            socket->close();
+            DBUG << "Bad Request";
+            return;
         }
     }
 }
@@ -510,7 +438,7 @@ void HttpSocket::sendErrorResponse(QTcpSocket *socket, int code)
 void HttpSocket::cantataStreams(const QStringList &files)
 {
     DBUG << files;
-    foreach (const QString &f, files) {
+    for (const QString &f: files) {
         Song s=HttpServer::self()->decodeUrl(f);
         if (s.isCantataStream() || s.isCdda()) {
             DBUG << s.file;
@@ -526,16 +454,21 @@ void HttpSocket::cantataStreams(const QList<Song> &songs, bool isUpdate)
         streamIds.clear();
     }
 
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         DBUG << s.file;
-        streamIds.insert(s.id, s.file);
+        if (s.isCantataStream()) {
+            streamIds.insert(s.id, HttpServer::self()->decodeUrl(s.file).file);
+        } else {
+            streamIds.insert(s.id, s.file);
+        }
         newlyAddedFiles.remove(s.file);
     }
+    DBUG << streamIds;
 }
 
 void HttpSocket::removedIds(const QSet<qint32> &ids)
 {
-    foreach (qint32 id, ids) {
+    for (qint32 id: ids) {
         streamIds.remove(id);
     }
 }
@@ -565,3 +498,5 @@ bool HttpSocket::write(QTcpSocket *socket, char *buffer, qint32 bytesRead, bool
     }
     return true;
 }
+
+#include "moc_httpsocket.cpp"
