@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -39,22 +39,22 @@ class HttpSocket : public QTcpServer
 
 public:
     HttpSocket(const QString &iface, quint16 port);
-    virtual ~HttpSocket() { }
+    ~HttpSocket() override { }
 
-    void incomingConnection(int socket);
-    QString address() const { return ifaceAddress; }
     QString configuredInterface() { return cfgInterface; }
+    quint16 boundPort();
 
 public Q_SLOTS:
     void terminate();
     void mpdAddress(const QString &a);
 
 private:
-    bool openPort(const QHostAddress &a, quint16 p);
+    bool openPort(quint16 p);
     bool isCantataStream(const QString &file) const;
     void sendErrorResponse(QTcpSocket *socket, int code);
 
 private Q_SLOTS:
+    void handleNewConnection();
     void readClient();
     void discardClient();
     void cantataStreams(const QStringList &files);
@@ -63,12 +63,12 @@ private Q_SLOTS:
 
 private:
     bool write(QTcpSocket *socket, char *buffer, qint32 bytesRead, bool &stop);
+    void setUrlAddress();
 
 private:
     QSet<QString> newlyAddedFiles; // Holds cantata strema filenames as added to MPD via "add"
     QMap<qint32, QString> streamIds; // Maps MPD playqueue song ID to fileName
     QString cfgInterface;
-    QString ifaceAddress;
     QString mpdAddr;
     bool terminated;
 };
