@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,17 +26,18 @@
 
 #include <QAbstractItemModel>
 #include <QMetaType>
-#include "action.h"
+#include "support/action.h"
+#include "config.h"
 
 class ActionModel : public QAbstractItemModel
 {
     Q_OBJECT
 
 public:
-    ActionModel(QObject *p=0) : QAbstractItemModel(p) { }
-    virtual ~ActionModel() { }
+    ActionModel(QObject *p=nullptr) : QAbstractItemModel(p) { }
+    ~ActionModel() override { }
     
-    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
     virtual void resetModel() { beginResetModel(); endResetModel(); }
 };
 
