@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,41 +25,27 @@
 #include "musiclibraryitemartist.h"
 #include "musiclibraryitemsong.h"
 #include "musiclibraryitemroot.h"
-#include "musiclibrarymodel.h"
 #include "devicesmodel.h"
 #include "playqueuemodel.h"
-#include "settings.h"
-#include "itemview.h"
-#include "mpdparseutils.h"
-#include "mpdconnection.h"
-#include "umsdevice.h"
-#include "httpserver.h"
-#include "localize.h"
-#include "icons.h"
-#include "mountpoints.h"
-#include "stdactions.h"
-#include "actioncollection.h"
+#include "gui/settings.h"
+#include "roles.h"
+#include "mpd-interface/mpdparseutils.h"
+#include "mpd-interface/mpdconnection.h"
+#include "devices/umsdevice.h"
+#include "http/httpserver.h"
+#include "widgets/icons.h"
+#include "widgets/mirrormenu.h"
+#include "devices/mountpoints.h"
+#include "gui/stdactions.h"
+#include "support/action.h"
 #include "config.h"
 #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
-#include "audiocddevice.h"
+#include "devices/audiocddevice.h"
 #endif
-#include <QMenu>
+#include "support/globalstatic.h"
 #include <QStringList>
 #include <QMimeData>
 #include <QTimer>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KIcon>
-#include <KDE/KGlobal>
-#include <solid/device.h>
-#include <solid/deviceinterface.h>
-#include <solid/devicenotifier.h>
-#include <solid/portablemediaplayer.h>
-#include <solid/storageaccess.h>
-#include <solid/storagedrive.h>
-#include <solid/storagevolume.h>
-#include <solid/opticaldisc.h>
-K_GLOBAL_STATIC(DevicesModel, instance)
-#else
 #include "solid-lite/device.h"
 #include "solid-lite/deviceinterface.h"
 #include "solid-lite/devicenotifier.h"
@@ -68,14 +54,19 @@ K_GLOBAL_STATIC(DevicesModel, instance)
 #include "solid-lite/storagedrive.h"
 #include "solid-lite/storagevolume.h"
 #include "solid-lite/opticaldisc.h"
-#endif
-
-#if defined ENABLE_MODEL_TEST
-#include "modeltest.h"
-#endif
 
 #include <QDebug>
-#define DBUG qDebug()
+static bool debugIsEnabled=false;
+#define DBUG if (debugIsEnabled) qWarning() << metaObject()->className() << __FUNCTION__
+void DevicesModel::enableDebug()
+{
+    debugIsEnabled=true;
+}
+
+bool DevicesModel::debugEnabled()
+{
+    return debugIsEnabled;
+}
 
 QString DevicesModel::fixDevicePath(const QString &path)
 {
@@ -91,41 +82,228 @@ QString DevicesModel::fixDevicePath(const QString &path)
     return path;
 }
 
-DevicesModel * DevicesModel::self()
-{
-    #ifdef ENABLE_KDE_SUPPORT
-    return instance;
-    #else
-    static DevicesModel *instance=0;
-    if(!instance) {
-        instance=new DevicesModel;
-        #if defined ENABLE_MODEL_TEST
-        new ModelTest(instance, instance);
-        #endif
-    }
-    return instance;
-    #endif
-}
+GLOBAL_STATIC(DevicesModel, instance)
 
 DevicesModel::DevicesModel(QObject *parent)
-    : MultiMusicModel(parent)
-    , itemMenu(0)
+    : MusicLibraryModel(parent)
+    , itemMenu(nullptr)
     , enabled(false)
     , inhibitMenuUpdate(false)
 {
-    configureAction = ActionCollection::get()->createAction("configuredevice", i18n("Configure Device"), Icons::self()->configureIcon);
-    refreshAction = ActionCollection::get()->createAction("refreshdevice", i18n("Refresh Device"), "view-refresh");
-    connectAction = ActionCollection::get()->createAction("connectdevice", i18n("Connect Device"), Icons::self()->connectIcon);
-    disconnectAction = ActionCollection::get()->createAction("disconnectdevice", i18n("Disconnect Device"), Icons::self()->disconnectIcon);
+    configureAction = new Action(Icons::self()->configureIcon, tr("Configure Device"), this);
+    refreshAction = new Action(Icons::self()->reloadIcon, tr("Refresh Device"), this);
+    connectAction = new Action(Icons::self()->connectIcon, tr("Connect Device"), this);
+    disconnectAction = new Action(Icons::self()->disconnectIcon, tr("Disconnect Device"), this);
     #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
-    editAction = ActionCollection::get()->createAction("editcd", i18n("Edit CD Details"), Icons::self()->editIcon);
+    editAction = new Action(Icons::self()->editIcon, tr("Edit CD Details"), this);
     #endif
     updateItemMenu();
-    connect(this, SIGNAL(add(const QStringList &, bool, quint8)), MPDConnection::self(), SLOT(add(const QStringList &, bool, quint8)));
+    connect(this, SIGNAL(add(const QStringList &, int, quint8, bool)), MPDConnection::self(), SLOT(add(const QStringList &, int, quint8, bool)));
 }
 
 DevicesModel::~DevicesModel()
 {
+    qDeleteAll(collections);
+}
+
+QModelIndex DevicesModel::index(int row, int column, const QModelIndex &parent) const
+{
+    if (!hasIndex(row, column, parent)) {
+        return QModelIndex();
+    }
+
+    if (parent.isValid()) {
+        MusicLibraryItem *p=static_cast<MusicLibraryItem *>(parent.internalPointer());
+
+        if (p) {
+            return row<p->childCount() ? createIndex(row, column, p->childItem(row)) : QModelIndex();
+        }
+    } else {
+        return row<collections.count() ? createIndex(row, column, collections.at(row)) : QModelIndex();
+    }
+
+    return QModelIndex();
+}
+
+QModelIndex DevicesModel::parent(const QModelIndex &index) const
+{
+    if (!index.isValid()) {
+        return QModelIndex();
+    }
+
+    MusicLibraryItem *childItem = static_cast<MusicLibraryItem *>(index.internalPointer());
+    MusicLibraryItem *parentItem = childItem->parentItem();
+
+    if (parentItem) {
+        return createIndex(parentItem->parentItem() ? parentItem->row() : row(parentItem), 0, parentItem);
+    } else {
+        return QModelIndex();
+    }
+}
+
+int DevicesModel::rowCount(const QModelIndex &parent) const
+{
+    if (parent.column() > 0) {
+        return 0;
+    }
+
+    return parent.isValid() ? static_cast<MusicLibraryItem *>(parent.internalPointer())->childCount() : collections.count();
+}
+
+void DevicesModel::getDetails(QSet<QString> &artists, QSet<QString> &albumArtists, QSet<QString> &composers, QSet<QString> &albums, QSet<QString> &genres)
+{
+    for (MusicLibraryItemRoot *col: collections) {
+        col->getDetails(artists, albumArtists, composers, albums, genres);
+    }
+}
+
+int DevicesModel::indexOf(const QString &id)
+{
+    int i=0;
+    for (MusicLibraryItemRoot *col: collections) {
+        if (col->id()==id) {
+            return i;
+        }
+        i++;
+    }
+    return -1;
+}
+
+QList<Song> DevicesModel::songs(const QModelIndexList &indexes, bool playableOnly, bool fullPath) const
+{
+    QMap<MusicLibraryItem *, QList<Song> > colSongs;
+    QMap<MusicLibraryItem *, QSet<QString> > colFiles;
+
+    for (QModelIndex index: indexes) {
+        MusicLibraryItem *item = static_cast<MusicLibraryItem *>(index.internalPointer());
+        MusicLibraryItem *p=item;
+
+        while (p->parentItem()) {
+            p=p->parentItem();
+        }
+
+        if (!p) {
+            continue;
+        }
+
+        MusicLibraryItemRoot *parent=static_cast<MusicLibraryItemRoot *>(p);
+
+        if (playableOnly && !parent->canPlaySongs()) {
+            continue;
+        }
+
+        switch (item->itemType()) {
+        case MusicLibraryItem::Type_Root: {
+            if (static_cast<MusicLibraryItemRoot *>(parent)->flat()) {
+                for (const MusicLibraryItem *song: static_cast<const MusicLibraryItemContainer *>(item)->childItems()) {
+                    if (MusicLibraryItem::Type_Song==song->itemType() && !colFiles[parent].contains(static_cast<const MusicLibraryItemSong*>(song)->file())) {
+                        colSongs[parent] << parent->fixPath(static_cast<const MusicLibraryItemSong*>(song)->song(), fullPath);
+                        colFiles[parent] << static_cast<const MusicLibraryItemSong*>(song)->file();
+                    }
+                }
+            } else {
+                // First, sort all artists as they would appear in UI...
+                QList<MusicLibraryItem *> artists=static_cast<const MusicLibraryItemContainer *>(item)->childItems();
+                if (artists.isEmpty()) {
+                    break;
+                }
+
+                qSort(artists.begin(), artists.end(), MusicLibraryItemArtist::lessThan);
+
+                for (MusicLibraryItem *a: artists) {
+                    const MusicLibraryItemContainer *artist=static_cast<const MusicLibraryItemContainer *>(a);
+                    // Now sort all albums as they would appear in UI...
+                    QList<MusicLibraryItem *> artistAlbums=artist->childItems();
+                    qSort(artistAlbums.begin(), artistAlbums.end(), MusicLibraryItemAlbum::lessThan);
+                    for (MusicLibraryItem *i: artistAlbums) {
+                        const MusicLibraryItemContainer *album=static_cast<const MusicLibraryItemContainer *>(i);
+                        for (const MusicLibraryItem *song: album->childItems()) {
+                            if (MusicLibraryItem::Type_Song==song->itemType() && !colFiles[parent].contains(static_cast<const MusicLibraryItemSong*>(song)->file())) {
+                                colSongs[parent] << parent->fixPath(static_cast<const MusicLibraryItemSong*>(song)->song(), fullPath);
+                                colFiles[parent] << static_cast<const MusicLibraryItemSong*>(song)->file();
+                            }
+                        }
+                    }
+                }
+            }
+            break;
+        }
+        case MusicLibraryItem::Type_Artist: {
+            // First, sort all albums as they would appear in UI...
+            QList<MusicLibraryItem *> artistAlbums=static_cast<const MusicLibraryItemContainer *>(item)->childItems();
+            qSort(artistAlbums.begin(), artistAlbums.end(), MusicLibraryItemAlbum::lessThan);
+
+            for (MusicLibraryItem *i: artistAlbums) {
+                const MusicLibraryItemContainer *album=static_cast<const MusicLibraryItemContainer *>(i);
+                for (const MusicLibraryItem *song: album->childItems()) {
+                    if (MusicLibraryItem::Type_Song==song->itemType() && !colFiles[parent].contains(static_cast<const MusicLibraryItemSong*>(song)->file())) {
+                        colSongs[parent] << parent->fixPath(static_cast<const MusicLibraryItemSong*>(song)->song(), fullPath);
+                        colFiles[parent] << static_cast<const MusicLibraryItemSong*>(song)->file();
+                    }
+                }
+            }
+            break;
+        }
+        case MusicLibraryItem::Type_Album:
+            for (const MusicLibraryItem *song: static_cast<const MusicLibraryItemContainer *>(item)->childItems()) {
+                if (MusicLibraryItem::Type_Song==song->itemType() && !colFiles[parent].contains(static_cast<const MusicLibraryItemSong*>(song)->file())) {
+                    colSongs[parent] << parent->fixPath(static_cast<const MusicLibraryItemSong*>(song)->song(), fullPath);
+                    colFiles[parent] << static_cast<const MusicLibraryItemSong*>(song)->file();
+                }
+            }
+            break;
+        case MusicLibraryItem::Type_Song:
+            if (!colFiles[parent].contains(static_cast<const MusicLibraryItemSong*>(item)->file())) {
+                colSongs[parent] << parent->fixPath(static_cast<const MusicLibraryItemSong*>(item)->song(), fullPath);
+                colFiles[parent] << static_cast<const MusicLibraryItemSong*>(item)->file();
+            }
+            break;
+        default:
+            break;
+        }
+    }
+
+    QList<Song> songs;
+    QMap<MusicLibraryItem *, QList<Song> >::Iterator it(colSongs.begin());
+    QMap<MusicLibraryItem *, QList<Song> >::Iterator end(colSongs.end());
+
+    for (; it!=end; ++it) {
+        songs.append(it.value());
+    }
+
+    return songs;
+}
+
+QStringList DevicesModel::filenames(const QModelIndexList &indexes, bool playableOnly, bool fullPath) const
+{
+    QList<Song> songList=songs(indexes, playableOnly, fullPath);
+    QStringList fnames;
+    for (const Song &s: songList) {
+        fnames.append(s.file);
+    }
+    return fnames;
+}
+
+bool DevicesModel::setData(const QModelIndex &index, const QVariant &value, int role)
+{
+    if (!index.isValid()) {
+        return false;
+    }
+
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    if (Cantata::Role_Image==role) {
+        MusicLibraryItem *item = static_cast<MusicLibraryItem *>(index.internalPointer());
+
+        if (MusicLibraryItem::Type_Root==item->itemType()) {
+            Device *dev=static_cast<Device *>(item);
+            if (Device::AudioCd==dev->devType()) {
+                static_cast<AudioCdDevice *>(dev)->scaleCoverPix(value.toInt());
+                return true;
+            }
+        }
+    }
+    #endif
+   return MusicLibraryModel::setData(index, value, role);
 }
 
 QVariant DevicesModel::data(const QModelIndex &index, int role) const
@@ -141,12 +319,21 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
         if (MusicLibraryItem::Type_Song==item->itemType()) {
             MusicLibraryItemSong *song = static_cast<MusicLibraryItemSong *>(item);
             if (MusicLibraryItem::Type_Root==song->parentItem()->itemType()) {
-                Device *dev=static_cast<Device *>(song->parentItem());
-                return song->song().trackAndTitleStr(Device::AudioCd!=dev->devType() || !song->song().albumartist.startsWith(dev->data()));
+                return song->song().trackAndTitleStr();
             }
         }
         break;
-    case ItemView::Role_SubText:
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    case Cantata::Role_Image:
+        if (MusicLibraryItem::Type_Root==item->itemType()) {
+            Device *dev=static_cast<Device *>(item);
+            if (Device::AudioCd==dev->devType()) {
+                return static_cast<AudioCdDevice *>(dev)->coverPix();
+            }
+        }
+        break;
+    #endif
+    case Cantata::Role_SubText:
         if (MusicLibraryItem::Type_Root==item->itemType()) {
             Device *dev=static_cast<Device *>(item);
             if (!dev->statusMessage().isEmpty()) {
@@ -154,26 +341,26 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
             }
             if (!dev->isConnected()) {
                 QString sub=dev->subText();
-                return i18n("Not Connected")+(sub.isEmpty() ? QString() : (QString(" - ")+sub));
+                return tr("Not Connected")+(sub.isEmpty() ? QString() : (QString(" – ")+sub));
             }
             if (Device::AudioCd==dev->devType()) {
                 return dev->subText();
             }
         }
         break;
-    case ItemView::Role_Capacity:
+    case Cantata::Role_Capacity:
         if (MusicLibraryItem::Type_Root==item->itemType()) {
             return static_cast<Device *>(item)->usedCapacity();
         }
         return QVariant();
-    case ItemView::Role_CapacityText:
+    case Cantata::Role_CapacityText:
         if (MusicLibraryItem::Type_Root==item->itemType()) {
             return static_cast<Device *>(item)->capacityString();
         }
         return QVariant();
-    case ItemView::Role_Actions:
+    case Cantata::Role_Actions: {
+        QVariant v;
         if (MusicLibraryItem::Type_Root==item->itemType()) {
-            QVariant v;
             QList<Action *> actions;
             if (Device::AudioCd!=static_cast<Device *>(item)->devType()) {
                 actions << configureAction;
@@ -190,23 +377,17 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
             }
             #endif
             v.setValue<QList<Action *> >(actions);
-            return v;
-        } /*else if (HttpServer::self()->isAlive()) {
-            MusicLibraryItem *root=static_cast<MusicLibraryItem *>(item);
-            while (root && MusicLibraryItem::Type_Root!=root->itemType()) {
-                root=root->parentItem();
-            }
-            if (root && static_cast<Device *>(root)->canPlaySongs()) {
-                QVariant v;
-                v.setValue<QList<Action *> >(QList<Action *>() << StdActions::self()->replacePlayQueueAction << StdActions::self()->addToPlayQueueAction);
-                return v;
-            }
-        } */
-        break;
+        } else if (root(item)->canPlaySongs() && HttpServer::self()->isAlive()) {
+            v.setValue<QList<Action *> >(QList<Action *>() << StdActions::self()->replacePlayQueueAction << StdActions::self()->appendToPlayQueueAction);
+        }
+        return v;
+    }
+    case Cantata::Role_ListImage:
+        return MusicLibraryItem::Type_Album==item->itemType();
     default:
         break;
     }
-    return MultiMusicModel::data(index, role);
+    return MusicLibraryModel::data(index, role);
 }
 
 void DevicesModel::clear(bool clearConfig)
@@ -214,7 +395,7 @@ void DevicesModel::clear(bool clearConfig)
     inhibitMenuUpdate=true;
     QSet<QString> remoteUdis;
     QSet<QString> udis;
-    foreach (MusicLibraryItemRoot *col, collections) {
+    for (MusicLibraryItemRoot *col: collections) {
         Device *dev=static_cast<Device *>(col);
         if (Device::RemoteFs==dev->devType()) {
             remoteUdis.insert(dev->id());
@@ -223,10 +404,10 @@ void DevicesModel::clear(bool clearConfig)
         }
     }
 
-    foreach (const QString &u, udis) {
+    for (const QString &u: udis) {
         deviceRemoved(u);
     }
-    foreach (const QString &u, remoteUdis) {
+    for (const QString &u: remoteUdis) {
         removeRemoteDevice(u, clearConfig);
     }
 
@@ -270,7 +451,7 @@ void DevicesModel::setEnabled(bool e)
 
 void DevicesModel::stop()
 {
-    foreach (MusicLibraryItemRoot *col, collections) {
+    for (MusicLibraryItemRoot *col: collections) {
         static_cast<Device *>(col)->stop();
     }
 
@@ -289,17 +470,20 @@ void DevicesModel::stop()
 Device * DevicesModel::device(const QString &udi)
 {
     int idx=indexOf(udi);
-    return idx<0 ? 0 : static_cast<Device *>(collections.at(idx));
+    return idx<0 ? nullptr : static_cast<Device *>(collections.at(idx));
 }
 
 void DevicesModel::setCover(const Song &song, const QImage &img, const QString &file)
 {
     #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    DBUG << "Set CDDA cover" << song.file << img.isNull() << file << song.isCdda();
     if (song.isCdda()) {
         int idx=indexOf(song.title);
         if (idx>=0) {
             Device *dev=static_cast<Device *>(collections.at(idx));
             if (Device::AudioCd==dev->devType()) {
+                DBUG << "Set cover of CD";
+                Covers::self()->updateCover(song, img, file);
                 static_cast<AudioCdDevice *>(dev)->setCover(song, img, file);
             }
         }
@@ -313,10 +497,7 @@ void DevicesModel::setCover(const Song &song, const QImage &img, const QString &
 
 void DevicesModel::setCover(const Song &song, const QImage &img)
 {
-    if (MusicLibraryItemAlbum::CoverNone==MusicLibraryItemAlbum::currentCoverSize()) {
-        return;
-    }
-
+    DBUG << "Set album cover" << song.file << img.isNull();
     if (img.isNull()) {
         return;
     }
@@ -332,7 +513,9 @@ void DevicesModel::setCover(const Song &song, const QImage &img)
     MusicLibraryItemArtist *artistItem = dev->artist(song, false);
     if (artistItem) {
         MusicLibraryItemAlbum *albumItem = artistItem->album(song, false);
-        if (albumItem && albumItem->setCover(img)) {
+        if (albumItem) {
+            DBUG << "Set cover of album";
+            Covers::self()->updateCover(song, img, QString());
             QModelIndex idx=index(albumItem->row(), 0, index(artistItem->row(), 0, index(i, 0, QModelIndex())));
             emit dataChanged(idx, idx);
         }
@@ -354,7 +537,6 @@ void DevicesModel::deviceUpdating(const QString &udi, bool state)
             }
             QModelIndex modelIndex=createIndex(idx, 0, dev);
             emit dataChanged(modelIndex, modelIndex);
-            MultiMusicModel::updateGenres();
             emit updated(modelIndex);
         }
     }
@@ -372,8 +554,11 @@ QStringList DevicesModel::playableUrls(const QModelIndexList &indexes) const
 {
     QList<Song> songList=songs(indexes, true, true);
     QStringList urls;
-    foreach (const Song &s, songList) {
-        urls.append(HttpServer::self()->encodeUrl(s));
+    for (const Song &s: songList) {
+        QByteArray encoded=HttpServer::self()->encodeUrl(s);
+        if (!encoded.isEmpty()) {
+            urls.append(encoded);
+        }
     }
     return urls;
 }
@@ -395,7 +580,7 @@ void DevicesModel::deviceAdded(const QString &udi)
 
     Solid::Device device(udi);
     DBUG << "Solid device added udi:" << device.udi() << "product:" << device.product() << "vendor:" << device.vendor();
-    Solid::StorageAccess *ssa =0;
+    Solid::StorageAccess *ssa =nullptr;
     #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
     Solid::OpticalDisc * opt = device.as<Solid::OpticalDisc>();
 
@@ -438,9 +623,9 @@ void DevicesModel::addLocalDevice(const QString &udi)
         connect(dev, SIGNAL(updating(const QString &, bool)), SLOT(deviceUpdating(const QString &, bool)));
         connect(dev, SIGNAL(error(const QString &)), SIGNAL(error(const QString &)));
         connect(dev, SIGNAL(cover(const Song &, const QImage &)), SLOT(setCover(const Song &, const QImage &)));
-        connect(dev, SIGNAL(invalid(QList<Song>)), SIGNAL(invalid(QList<Song>)));
         connect(dev, SIGNAL(updatedDetails(QList<Song>)), SIGNAL(updatedDetails(QList<Song>)));
         connect(dev, SIGNAL(play(QList<Song>)), SLOT(play(QList<Song>)));
+        connect(dev, SIGNAL(renamed()), this, SLOT(updateItemMenu()));
         #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
         if (Device::AudioCd==dev->devType()) {
             connect(static_cast<AudioCdDevice *>(dev), SIGNAL(matches(const QString &, const QList<CdAlbum> &)),
@@ -470,10 +655,15 @@ void DevicesModel::deviceRemoved(const QString &udi)
         }
 
         beginRemoveRows(QModelIndex(), idx, idx);
-        static_cast<Device *>(collections.takeAt(idx))->deleteLater();
+        Device *dev=static_cast<Device *>(collections.takeAt(idx));
+        dev->deleteLater();
+        #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+        if (Device::AudioCd==dev->devType()) {
+            static_cast<AudioCdDevice *>(dev)->dequeue();
+        }
+        #endif
         endRemoveRows();
         updateItemMenu();
-        MultiMusicModel::updateGenres();
     }
 }
 
@@ -505,8 +695,6 @@ void DevicesModel::addRemoteDevice(const DeviceOptions &opts, RemoteFsDevice::De
         connect(dev, SIGNAL(cover(const Song &, const QImage &)), SLOT(setCover(const Song &, const QImage &)));
         if (Device::RemoteFs==dev->devType()) {
             connect(static_cast<RemoteFsDevice *>(dev), SIGNAL(udiChanged()), SLOT(remoteDeviceUdiChanged()));
-            connect(static_cast<RemoteFsDevice *>(dev), SIGNAL(connectionStateHasChanged(QString, bool)),
-                    SLOT(removeDeviceConnectionStateChanged(QString, bool)));
         }
         updateItemMenu();
     }
@@ -532,7 +720,6 @@ void DevicesModel::removeRemoteDevice(const QString &udi, bool removeFromConfig)
         collections.takeAt(idx);
         endRemoveRows();
         updateItemMenu();
-        MultiMusicModel::updateGenres();
         RemoteFsDevice *rfs=qobject_cast<RemoteFsDevice *>(dev);
         if (rfs) {
             // Destroy will stop device, and delete it (via deleteLater())
@@ -552,26 +739,10 @@ void DevicesModel::remoteDeviceUdiChanged()
     #endif
 }
 
-void DevicesModel::removeDeviceConnectionStateChanged(const QString &udi, bool state)
-{
-    #ifdef ENABLE_REMOTE_DEVICES
-    if (!state) {
-        int idx=indexOf(udi);
-        if (idx<0) {
-            return;
-        }
-        MultiMusicModel::updateGenres();
-    }
-    #else
-    Q_UNUSED(udi)
-    Q_UNUSED(state)
-    #endif
-}
-
 void DevicesModel::mountsChanged()
 {
     #ifdef ENABLE_REMOTE_DEVICES
-    foreach (MusicLibraryItemRoot *col, collections) {
+    for (MusicLibraryItemRoot *col: collections) {
         Device *dev=static_cast<Device *>(col);
         if (Device::RemoteFs==dev->devType() && ((RemoteFsDevice *)dev)->getDetails().isLocalFile()) {
             if (0==dev->childCount()) {
@@ -593,7 +764,7 @@ void DevicesModel::loadLocal()
 {
     // Build set of currently known MTP/UMS collections...
     QSet<QString> existingUdis;
-    foreach (MusicLibraryItemRoot *col, collections) {
+    for (MusicLibraryItemRoot *col: collections) {
         Device *dev=static_cast<Device *>(col);
         if (Device::Mtp==dev->devType() || Device::Ums==dev->devType()) {
             existingUdis.insert(dev->id());
@@ -601,7 +772,7 @@ void DevicesModel::loadLocal()
     }
 
     QList<Solid::Device> deviceList = Solid::Device::listFromType(Solid::DeviceInterface::PortableMediaPlayer);
-    foreach (const Solid::Device &device, deviceList) {
+    for (const Solid::Device &device: deviceList) {
         if (existingUdis.contains(device.udi())) {
             existingUdis.remove(device.udi());
             continue;
@@ -614,8 +785,7 @@ void DevicesModel::loadLocal()
         addLocalDevice(device.udi());
     }
     deviceList = Solid::Device::listFromType(Solid::DeviceInterface::StorageAccess);
-    foreach (const Solid::Device &device, deviceList)
-    {
+    for (const Solid::Device &device: deviceList) {
         if (existingUdis.contains(device.udi())) {
             existingUdis.remove(device.udi());
             continue;
@@ -639,7 +809,7 @@ void DevicesModel::loadLocal()
 
     #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
     deviceList = Solid::Device::listFromType(Solid::DeviceInterface::OpticalDisc);
-    foreach (const Solid::Device &device, deviceList) {
+    for (const Solid::Device &device: deviceList) {
         if (existingUdis.contains(device.udi())) {
             existingUdis.remove(device.udi());
             continue;
@@ -656,7 +826,7 @@ void DevicesModel::loadLocal()
 
     // Remove any previous MTP/UMS devices that were not listed above.
     // This is to fix BUG:127
-    foreach (const QString &udi, existingUdis) {
+    for (const QString &udi: existingUdis) {
         deviceRemoved(udi);
     }
 }
@@ -667,15 +837,13 @@ void DevicesModel::loadRemote()
     QList<Device *> rem=RemoteFsDevice::loadAll(this);
     if (rem.count()) {
         beginInsertRows(QModelIndex(), collections.count(), collections.count()+(rem.count()-1));
-        foreach (Device *dev, rem) {
+        for (Device *dev: rem) {
             collections.append(dev);
             connect(dev, SIGNAL(updating(const QString &, bool)), SLOT(deviceUpdating(const QString &, bool)));
             connect(dev, SIGNAL(error(const QString &)), SIGNAL(error(const QString &)));
             connect(dev, SIGNAL(cover(const Song &, const QImage &)), SLOT(setCover(const Song &, const QImage &)));
             if (Device::RemoteFs==dev->devType()) {
                 connect(static_cast<RemoteFsDevice *>(dev), SIGNAL(udiChanged()), SLOT(remoteDeviceUdiChanged()));
-                connect(static_cast<RemoteFsDevice *>(dev), SIGNAL(connectionStateHasChanged(QString, bool)),
-                        SLOT(removeDeviceConnectionStateChanged(QString, bool)));
             }
         }
         endInsertRows();
@@ -685,7 +853,7 @@ void DevicesModel::loadRemote()
 
 void DevicesModel::unmountRemote()
 {
-    foreach (MusicLibraryItemRoot *col, collections) {
+    for (MusicLibraryItemRoot *col: collections) {
         Device *dev=static_cast<Device *>(col);
         if (Device::RemoteFs==dev->devType()) {
             static_cast<RemoteFsDevice *>(dev)->unmount();
@@ -697,7 +865,7 @@ void DevicesModel::unmountRemote()
 #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
 void DevicesModel::playCd(const QString &dev)
 {
-    foreach (MusicLibraryItemRoot *col, collections) {
+    for (MusicLibraryItemRoot *col: collections) {
         Device *d=static_cast<Device *>(col);
         if (Device::AudioCd==d->devType() && static_cast<AudioCdDevice *>(d)->isAudioDevice(dev)) {
             static_cast<AudioCdDevice *>(d)->autoplay();
@@ -721,17 +889,15 @@ void DevicesModel::updateItemMenu()
     }
 
     if (!itemMenu) {
-        itemMenu = new QMenu(0);
+        itemMenu = new MirrorMenu(nullptr);
     }
 
     itemMenu->clear();
 
-    if (collections.isEmpty()) {
-        itemMenu->addAction(i18n("No Devices Attached"))->setEnabled(false);
-    } else {
+    if (!collections.isEmpty()) {
         QMap<QString, const MusicLibraryItemRoot *> items;
 
-        foreach (const MusicLibraryItemRoot *d, collections) {
+        for (const MusicLibraryItemRoot *d: collections) {
             if (Device::AudioCd!=static_cast<const Device *>(d)->devType()) {
                 items.insert(d->data(), d);
             }
@@ -740,18 +906,22 @@ void DevicesModel::updateItemMenu()
         QStringList keys=items.keys();
         qSort(keys.begin(), keys.end(), lessThan);
 
-        foreach (const QString &k, keys) {
+        for (const QString &k: keys) {
             const MusicLibraryItemRoot *d=items[k];
             QAction *act=itemMenu->addAction(d->icon(), k, this, SLOT(emitAddToDevice()));
             act->setData(d->id());
             Action::initIcon(act);
         }
     }
+
+    if (itemMenu->isEmpty()) {
+        itemMenu->addAction(tr("No Devices Attached"))->setEnabled(false);
+    }
 }
 
 QMimeData * DevicesModel::mimeData(const QModelIndexList &indexes) const
 {
-    QMimeData *mimeData=0;
+    QMimeData *mimeData=nullptr;
     QStringList paths=playableUrls(indexes);
 
     if (!paths.isEmpty()) {
@@ -765,12 +935,14 @@ void DevicesModel::play(const QList<Song> &songs)
 {
     QStringList paths;
     if (HttpServer::self()->isAlive()) {
-        foreach (const Song &s, songs) {
+        for (const Song &s: songs) {
             paths.append(HttpServer::self()->encodeUrl(s));
         }
 
         if (!paths.isEmpty()) {
-            emit add(paths, true, 0);
+            emit add(paths, MPDConnection::ReplaceAndplay, 0, false);
         }
     }
 }
+
+#include "moc_devicesmodel.cpp"
