@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,38 +24,50 @@
 #define DEVICES_MODEL_H
 
 #include <QSet>
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 #include "config.h"
 #include "devices/remotefsdevice.h"
-#include "multimusicmodel.h"
+#include "musiclibrarymodel.h"
 #include "devices/cdalbum.h"
 #include "musiclibraryproxymodel.h"
 
 class QMimeData;
 class Device;
-class QMenu;
+class MirrorMenu;
 
-class DevicesModel : public MultiMusicModel
+class DevicesModel : public MusicLibraryModel
 {
     Q_OBJECT
 
 public:
     static DevicesModel * self();
 
+    static void enableDebug();
+    static bool debugEnabled();
     static QString fixDevicePath(const QString &path);
 
-    DevicesModel(QObject *parent = 0);
-    ~DevicesModel();
-    QVariant data(const QModelIndex &, int) const;
-    Qt::ItemFlags flags(const QModelIndex &index) const;
+    DevicesModel(QObject *parent = nullptr);
+    ~DevicesModel() override;
+    QModelIndex index(int row, int column, const QModelIndex &parent=QModelIndex()) const override;
+    QModelIndex parent(const QModelIndex &index) const override;
+    QVariant headerData(int, Qt::Orientation, int = Qt::DisplayRole) const override { return QVariant(); }
+    int columnCount(const QModelIndex & = QModelIndex()) const override { return 1; }
+    int rowCount(const QModelIndex &parent=QModelIndex()) const override;
+    void getDetails(QSet<QString> &artists, QSet<QString> &albumArtists, QSet<QString> &composers, QSet<QString> &albums, QSet<QString> &genres);
+    QList<Song> songs(const QModelIndexList &indexes, bool playableOnly=false, bool fullPath=false) const;
+    QStringList filenames(const QModelIndexList &indexes, bool playableOnly=false, bool fullPath=false) const;
+    int row(void *i) const override { return collections.indexOf(static_cast<MusicLibraryItemRoot *>(i)); }
+    bool setData(const QModelIndex &index, const QVariant &value, int role) override;
+    QVariant data(const QModelIndex &, int) const override;
+    Qt::ItemFlags flags(const QModelIndex &index) const override;
     QStringList playableUrls(const QModelIndexList &indexes) const;
     void clear(bool clearConfig=true);
-    QMenu * menu() { return itemMenu; }
+    MirrorMenu * menu() { return itemMenu; }
     Device * device(const QString &udi);
     bool isEnabled() const { return enabled; }
     void setEnabled(bool e);
     void stop();
-    QMimeData * mimeData(const QModelIndexList &indexes) const;
+    QMimeData * mimeData(const QModelIndexList &indexes) const override;
     #ifdef ENABLE_REMOTE_DEVICES
     void unmountRemote();
     #endif
@@ -69,6 +81,9 @@ public:
     void playCd(const QString &dev);
     #endif
 
+private:
+    int indexOf(const QString &id);
+
 public Q_SLOTS:
     void setCover(const Song &song, const QImage &img, const QString &file);
     void setCover(const Song &song, const QImage &img);
@@ -80,7 +95,6 @@ public Q_SLOTS:
     void addRemoteDevice(const DeviceOptions &opts, RemoteFsDevice::Details details);
     void removeRemoteDevice(const QString &udi, bool removeFromConfig=true);
     void remoteDeviceUdiChanged();
-    void removeDeviceConnectionStateChanged(const QString &udi, bool state);
     void mountsChanged();
 
 private:
@@ -90,14 +104,13 @@ private:
     #endif
 
 Q_SIGNALS:
-    void updateGenres(const QSet<QString> &genres);
     void addToDevice(const QString &udi);
     void error(const QString &text);
     void updated(const QModelIndex &idx);
     void matches(const QString &udi, const QList<CdAlbum> &albums);
     void invalid(const QList<Song> &songs);
     void updatedDetails(const QList<Song> &songs);
-    void add(const QStringList &files, bool replace, quint8 priorty); // add songs to MPD playqueue
+    void add(const QStringList &files, int action, quint8 priority, bool decreasePriority); // add songs to MPD playqueue
 
 private Q_SLOTS:
     void play(const QList<Song> &songs);
@@ -105,8 +118,9 @@ private Q_SLOTS:
     void updateItemMenu();
 
 private:
+    QList<MusicLibraryItemRoot *> collections;
     QSet<QString> volumes;
-    QMenu *itemMenu;
+    MirrorMenu *itemMenu;
     bool enabled;
     bool inhibitMenuUpdate;
     Action *configureAction;
