@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,14 +24,9 @@
 #include "digitallyimported.h"
 #include "support/configuration.h"
 #include "network/networkaccessmanager.h"
-#include "support/localize.h"
 #include "support/globalstatic.h"
 #include <QNetworkRequest>
-#if QT_VERSION >= 0x050000
 #include <QJsonDocument>
-#else
-#include "qjson/parser.h"
-#endif
 #include <QTime>
 #include <QTimer>
 
@@ -45,9 +40,9 @@ const QString DigitallyImported::constPublicValue=QLatin1String("public3");
 GLOBAL_STATIC(DigitallyImported, instance)
 
 DigitallyImported::DigitallyImported()
-    : job(0)
+    : job(nullptr)
     , streamType(0)
-    , timer(0)
+    , timer(nullptr)
 {
     load();
 }
@@ -60,7 +55,7 @@ void DigitallyImported::login()
 {
     if (job) {
         job->deleteLater();
-        job=0;
+        job=nullptr;
     }
     QNetworkRequest req(constAuthUrl);
     addAuthHeader(req);
@@ -72,7 +67,7 @@ void DigitallyImported::logout()
 {
     if (job) {
         job->deleteLater();
-        job=0;
+        job=nullptr;
     }
     listenHash=QString();
     expires=QDateTime();
@@ -81,11 +76,7 @@ void DigitallyImported::logout()
 
 void DigitallyImported::addAuthHeader(QNetworkRequest &req) const
 {
-    #if QT_VERSION < 0x050000
-    req.setRawHeader("Authorization", "Basic "+QString("%1:%2").arg(constApiUserName, constApiPassword).toAscii().toBase64());
-    #else
     req.setRawHeader("Authorization", "Basic "+QString("%1:%2").arg(constApiUserName, constApiPassword).toLatin1().toBase64());
-    #endif
 }
 
 void DigitallyImported::load()
@@ -98,7 +89,7 @@ void DigitallyImported::load()
     streamType=cfg.get("streamType", streamType);
     QString ex=cfg.get("expires", QString());
 
-    status=i18n("Not logged in");
+    status=tr("Not logged in");
     if (ex.isEmpty()) {
         listenHash=QString();
     } else {
@@ -107,7 +98,7 @@ void DigitallyImported::load()
         if (QDateTime::currentDateTime().secsTo(expires)<(5*60)) {
             listenHash=QString();
         } else if (!listenHash.isEmpty()) {
-            status=i18n("Logged in");
+            status=tr("Logged in");
         }
     }
     controlTimer();
@@ -125,6 +116,22 @@ void DigitallyImported::save()
     emit updated();
 }
 
+bool DigitallyImported::isDiUrl(const QString &u) const
+{
+    if (!u.startsWith(QLatin1String("http://"))) {
+        return false;
+    }
+    QUrl url(u);
+    if (!url.host().startsWith(QLatin1String("listen."))) {
+        return false;
+    }
+    QStringList pathParts=url.path().split(QLatin1Char('/'), QString::SkipEmptyParts);
+    if (2!=pathParts.count()) {
+        return false;
+    }
+    return pathParts.at(0)==constPublicValue;
+}
+
 QString DigitallyImported::modifyUrl(const QString &u) const
 {
     if (listenHash.isEmpty()) {
@@ -147,7 +154,7 @@ void DigitallyImported::loginResponse()
     if (reply!=job) {
         return;
     }
-    job=0;
+    job=nullptr;
 
     status=listenHash=QString();
     const int httpStatus = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
@@ -156,32 +163,28 @@ void DigitallyImported::loginResponse()
         emit loginStatus(false, status);
         return;
     } else if (200!=httpStatus) {
-        status=i18n("Unknown error");
+        status=tr("Unknown error");
         emit loginStatus(false, status);
         return;
     }
 
-    #if QT_VERSION >= 0x050000
     QVariantMap data=QJsonDocument::fromJson(reply->readAll()).toVariant().toMap();
-    #else
-    QVariantMap data = QJson::Parser().parse(reply->readAll()).toMap();
-    #endif
 
     if (!data.contains("subscriptions")) {
-        status=i18n("No subscriptions");
+        status=tr("No subscriptions");
         emit loginStatus(false, status);
         return;
     }
 
     QVariantList subscriptions = data.value("subscriptions", QVariantList()).toList();
     if (subscriptions.isEmpty() || QLatin1String("active")!=subscriptions[0].toMap().value("status").toString()) {
-        status=i18n("You do not have an active subscription");
+        status=tr("You do not have an active subscription");
         emit loginStatus(false, status);
         return;
     }
 
     if (!subscriptions[0].toMap().contains("expires_on") || !data.contains("listen_key")) {
-        status=i18n("Unknown error");
+        status=tr("Unknown error");
         emit loginStatus(false, status);
         return;
     }
@@ -194,7 +197,7 @@ void DigitallyImported::loginResponse()
         listenHash=lh;
         save();
     }
-    status=i18n("Logged in (expiry:%1)", expires.toString(Qt::ISODate));
+    status=tr("Logged in (expiry:%1)").arg(expires.toString(Qt::ISODate));
     controlTimer();
     emit loginStatus(true, status);
 }
@@ -202,7 +205,7 @@ void DigitallyImported::loginResponse()
 void DigitallyImported::timeout()
 {
     listenHash=QString();
-    emit loginStatus(false, i18n("Session expired"));
+    emit loginStatus(false, tr("Session expired"));
 }
 
 void DigitallyImported::controlTimer()
@@ -228,3 +231,5 @@ void DigitallyImported::controlTimer()
         }
     }
 }
+
+#include "moc_digitallyimported.cpp"
