@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -31,9 +31,8 @@
 #include <QVariant>
 #include <QHash>
 #include "musiclibraryitem.h"
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 
-class QPixmap;
 class MusicLibraryItemRoot;
 class MusicLibraryItemAlbum;
 
@@ -42,46 +41,25 @@ class MusicLibraryItemArtist : public MusicLibraryItemContainer
 public:
     static bool lessThan(const MusicLibraryItem *a, const MusicLibraryItem *b);
 
-    MusicLibraryItemArtist(const QString &data, const QString &artistName, MusicLibraryItemContainer *parent = 0);
-    virtual ~MusicLibraryItemArtist() { }
+    MusicLibraryItemArtist(const Song &song, MusicLibraryItemContainer *parent=nullptr);
+    ~MusicLibraryItemArtist() override { }
 
     MusicLibraryItemAlbum * album(const Song &s, bool create=true);
     MusicLibraryItemAlbum * createAlbum(const Song &s);
-    const QString & baseArtist() const;
-    bool isVarious() const { return m_various; }
-    bool allSingleTrack() const;
-    void addToSingleTracks(MusicLibraryItemArtist *other);
-    bool isFromSingleTracks(const Song &s) const;
+    const QString & sortString() const { return m_sortString.isEmpty() ? m_itemData : m_sortString; }
     void remove(MusicLibraryItemAlbum *album);
-    void updateIndexes();
-    Type itemType() const { return Type_Artist; }
-    static void clearDefaultCover();
-    #ifdef ENABLE_UBUNTU
-    const QString & cover() const;
-    #else
-    const QPixmap & cover() const;
-    #endif
+    Type itemType() const override { return Type_Artist; }
     // 'data' could be 'Composer' if we are set to use that, but need to save real artist...
     const QString & actualArtist() const { return m_actualArtist; }
-    #ifdef ENABLE_UBUNTU
-    void setCover(const QString &c) { m_coverName="file://"+c; m_coverRequested=false; }
-    const QString & coverName() { return m_coverName; }
-    #endif
- 
-private:
-    bool largeImages() const;
+    Song coverSong() const;
 
 private:
-    #ifdef ENABLE_UBUNTU
-    mutable QString m_coverName;
-    mutable bool m_coverRequested;
-    #else
-    mutable Song m_coverSong;
-    #endif
-    bool m_various;
-    QString m_nonTheArtist;
+    MusicLibraryItemAlbum * getAlbum(const QString &key) const;
+
+private:
+    QString m_sortString; // Do we have an actual artist-sort, or is m_sortString just "Artist, The" ??? - needed for cache saving
     QString m_actualArtist;
-    QHash<QString, int> m_indexes;
+    mutable QHash<QString, int> m_indexes;
 };
 
 #endif
