@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -26,7 +26,6 @@
 
 #include <QModelIndex>
 #include <QMimeData>
-#include <QMenu>
 #include <QFont>
 #include "config.h"
 #include "playlistsmodel.h"
@@ -34,25 +33,21 @@
 #include "playqueuemodel.h"
 #include "widgets/groupedview.h"
 #include "roles.h"
-#include "gui/plurals.h"
-#include "support/localize.h"
+#include "gui/covers.h"
+#include "gui/settings.h"
 #include "support/utils.h"
+#include "support/monoicon.h"
 #include "support/globalstatic.h"
-#include "mpd/mpdparseutils.h"
-#include "mpd/mpdstats.h"
-#include "mpd/mpdconnection.h"
+#include "mpd-interface/mpdparseutils.h"
+#include "mpd-interface/mpdstats.h"
+#include "mpd-interface/mpdconnection.h"
 #include "playqueuemodel.h"
-#include "streamsmodel.h"
 #include "widgets/icons.h"
+#include "widgets/mirrormenu.h"
 #ifdef ENABLE_HTTP_SERVER
 #include "http/httpserver.h"
 #endif
 
-#if defined ENABLE_MODEL_TEST
-#include "modeltest.h"
-#endif
-
-#ifndef ENABLE_UBUNTU
 QString PlaylistsModel::headerText(int col)
 {
     switch (col) {
@@ -61,25 +56,25 @@ QString PlaylistsModel::headerText(int col)
     case COL_ALBUM:     return PlayQueueModel::headerText(PlayQueueModel::COL_ALBUM);
     case COL_LENGTH:    return PlayQueueModel::headerText(PlayQueueModel::COL_LENGTH);
     case COL_YEAR:      return PlayQueueModel::headerText(PlayQueueModel::COL_YEAR);
+    case COL_ORIGYEAR:  return PlayQueueModel::headerText(PlayQueueModel::COL_ORIGYEAR);
     case COL_GENRE:     return PlayQueueModel::headerText(PlayQueueModel::COL_GENRE);
     case COL_COMPOSER:  return PlayQueueModel::headerText(PlayQueueModel::COL_COMPOSER);
     case COL_PERFORMER: return PlayQueueModel::headerText(PlayQueueModel::COL_PERFORMER);
+    case COL_FILENAME:  return PlayQueueModel::headerText(PlayQueueModel::COL_FILENAME);
+    case COL_PATH:      return PlayQueueModel::headerText(PlayQueueModel::COL_PATH);
     default:            return QString();
     }
 }
-#endif
 
 GLOBAL_STATIC(PlaylistsModel, instance)
 
 PlaylistsModel::PlaylistsModel(QObject *parent)
     : ActionModel(parent)
-    , enabled(true)
     , multiCol(false)
-    #ifndef ENABLE_UBUNTU
-    , itemMenu(0)
+    , itemMenu(nullptr)
     , dropAdjust(0)
-    #endif
 {
+    icn=Icons::self()->playlistListIcon;
     connect(MPDConnection::self(), SIGNAL(stateChanged(bool)), SLOT(mpdConnectionStateChanged(bool)));
     connect(MPDConnection::self(), SIGNAL(playlistsRetrieved(const QList<Playlist> &)), this, SLOT(setPlaylists(const QList<Playlist> &)));
     connect(MPDConnection::self(), SIGNAL(playlistInfoRetrieved(const QString &, const QList<Song> &)), this, SLOT(playlistInfoRetrieved(const QString &, const QList<Song> &)));
@@ -94,24 +89,36 @@ PlaylistsModel::PlaylistsModel(QObject *parent)
     connect(this, SIGNAL(addToPlaylist(const QString &, const QStringList, quint32, quint32)), MPDConnection::self(), SLOT(addToPlaylist(const QString &, const QStringList, quint32, quint32)));
     connect(this, SIGNAL(moveInPlaylist(const QString &, const QList<quint32> &, quint32, quint32)), MPDConnection::self(), SLOT(moveInPlaylist(const QString &, const
     QList<quint32> &, quint32, quint32)));
-    #ifndef ENABLE_UBUNTU
-    newAction=new QAction(Icon("document-new"), i18n("New Playlist..."), this);
-    connect(newAction, SIGNAL(triggered(bool)), this, SIGNAL(addToNew()));
+    connect(Covers::self(), SIGNAL(loaded(Song,int)), this, SLOT(coverLoaded(Song,int)));
+    newAction=new QAction(MonoIcon::icon(FontAwesome::asterisk, Utils::monoIconColor()), tr("New Playlist..."), this);
+    connect(newAction, SIGNAL(triggered()), this, SIGNAL(addToNew()));
     Action::initIcon(newAction);
-    #endif
-    #if defined ENABLE_MODEL_TEST
-    new ModelTest(this, this);
-    #endif
+    alignments[COL_TITLE]=alignments[COL_ARTIST]=alignments[COL_ALBUM]=alignments[COL_GENRE]=alignments[COL_COMPOSER]=
+            alignments[COL_PERFORMER]=alignments[COL_FILENAME]=alignments[COL_PATH]=int(Qt::AlignVCenter|Qt::AlignLeft);
+    alignments[COL_LENGTH]=alignments[COL_YEAR]=alignments[COL_ORIGYEAR]=int(Qt::AlignVCenter|Qt::AlignRight);
 }
 
 PlaylistsModel::~PlaylistsModel()
 {
-    #ifndef ENABLE_UBUNTU
     if (itemMenu) {
         itemMenu->deleteLater();
-        itemMenu=0;
+        itemMenu=nullptr;
     }
-    #endif
+}
+
+QString PlaylistsModel::name() const
+{
+    return QLatin1String("playlists");
+}
+
+QString PlaylistsModel::title() const
+{
+    return tr("Stored Playlists");
+}
+
+QString PlaylistsModel::descr() const
+{
+    return tr("Standard playlists");
 }
 
 int PlaylistsModel::rowCount(const QModelIndex &index) const
@@ -197,39 +204,33 @@ QModelIndex PlaylistsModel::index(int row, int col, const QModelIndex &parent) c
     return row<items.count() ? createIndex(row, col, items.at(row)) : QModelIndex();
 }
 
-#ifndef ENABLE_UBUNTU
 QVariant PlaylistsModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
     if (Qt::Horizontal==orientation) {
         switch (role) {
         case Qt::DisplayRole:
+        case Cantata::Role_ContextMenuText:
             return headerText(section);
         case Qt::TextAlignmentRole:
-            switch (section) {
-            case COL_TITLE:
-            case COL_ARTIST:
-            case COL_ALBUM:
-            case COL_GENRE:
-            case COL_COMPOSER:
-            case COL_PERFORMER:
-            default:
-                return int(Qt::AlignVCenter|Qt::AlignLeft);
-            case COL_LENGTH:
-            case COL_YEAR:
-                return int(Qt::AlignVCenter|Qt::AlignRight);
-            }
+            return alignments[section];
+        case Cantata::Role_InitiallyHidden:
+            return COL_YEAR==section || COL_ORIGYEAR==section || COL_GENRE==section || COL_COMPOSER==section || COL_PERFORMER==section ||
+                   COL_FILENAME==section || COL_PATH==section;
         case Cantata::Role_Hideable:
-            return COL_YEAR==section || COL_GENRE==section || COL_COMPOSER==section || COL_PERFORMER==section ? true : false;
+            return COL_LENGTH!=section;
         case Cantata::Role_Width:
             switch (section) {
             case COL_TITLE:     return 0.4;
             case COL_ARTIST:    return 0.15;
             case COL_ALBUM:     return 0.15;
             case COL_YEAR:      return 0.05;
+            case COL_ORIGYEAR:  return 0.05;
             case COL_GENRE:     return 0.125;
             case COL_LENGTH:    return 0.125;
             case COL_COMPOSER:  return 0.15;
             case COL_PERFORMER: return 0.15;
+            case COL_FILENAME:  return 0.15;
+            case COL_PATH:      return 0.15;
             }
         default:
             break;
@@ -238,28 +239,35 @@ QVariant PlaylistsModel::headerData(int section, Qt::Orientation orientation, in
 
     return QVariant();
 }
-#endif
+
+bool PlaylistsModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role)
+{
+    if (Qt::Horizontal==orientation && Qt::TextAlignmentRole==role && section>=0) {
+        int al=value.toInt()|Qt::AlignVCenter;
+        if (al!=alignments[section]) {
+            alignments[section]=al;
+            return true;
+        }
+    }
+    return false;
+}
 
 QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid()) {
+        switch (role) {
+        case Cantata::Role_TitleText:
+            return title();
+        case Cantata::Role_SubText:
+            return descr();
+        case Qt::DecorationRole:
+            return icon();
+        }
         return QVariant();
     }
 
     if (Qt::TextAlignmentRole==role) {
-        switch (index.column()) {
-        case COL_TITLE:
-        case COL_ARTIST:
-        case COL_ALBUM:
-        case COL_GENRE:
-        case COL_COMPOSER:
-        case COL_PERFORMER:
-        default:
-            return int(Qt::AlignVCenter|Qt::AlignLeft);
-        case COL_LENGTH:
-        case COL_YEAR:
-            return int(Qt::AlignVCenter|Qt::AlignRight);
-        }
+        return alignments[index.column()];
     }
 
     Item *item=static_cast<Item *>(index.internalPointer());
@@ -268,16 +276,15 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
         PlaylistItem *pl=static_cast<PlaylistItem *>(item);
 
         switch(role) {
-        #ifdef ENABLE_UBUNTU
-        case Cantata::Role_Image:
-            return QString();
-        #endif
+        case Cantata::Role_ListImage:
+            return false;
         case Cantata::Role_IsCollection:
             return true;
         case Cantata::Role_CollectionId:
             return pl->key;
         case Cantata::Role_Key:
             return 0;
+        case Cantata::Role_SongWithRating:
         case Cantata::Role_Song: {
             QVariant var;
             var.setValue<Song>(Song());
@@ -318,6 +325,7 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
                     }
                     return pl->loaded && !pl->isSmartPlaylist ? Utils::formatTime(pl->totalTime()) : QVariant();
                 case COL_YEAR:
+                case COL_ORIGYEAR:
                 case COL_GENRE:
                     return QVariant();
                 default:
@@ -326,26 +334,29 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
             }
             return pl->visibleName();
         case Qt::ToolTipRole:
+            if (!Settings::self()->infoTooltips()) {
+                return QVariant();
+            }
             if (!pl->loaded) {
                 pl->loaded=true;
                 emit playlistInfo(pl->name);
             }
             return 0==pl->songs.count()
                 ? pl->visibleName()
-                : pl->visibleName()+"\n"+Plurals::tracksWithDuration(pl->songs.count(), Utils::formatTime(pl->totalTime()));
-        #ifndef ENABLE_UBUNTU
+                : pl->visibleName()+"\n"+tr("%n Tracks (%1)", "", pl->songs.count()).arg(Utils::formatTime(pl->totalTime()));
         case Qt::DecorationRole:
-            return multiCol ? QVariant() : (pl->isSmartPlaylist ? Icons::self()->dynamicRuleIcon : Icons::self()->playlistIcon);
-        #endif
+            return multiCol ? QVariant() : (pl->isSmartPlaylist ? Icons::self()->smartPlaylistIcon : Icons::self()->playlistListIcon);
         case Cantata::Role_SubText:
             if (!pl->loaded) {
                 pl->loaded=true;
                 emit playlistInfo(pl->name);
             }
             if (pl->isSmartPlaylist) {
-                return i18n("Smart Playlist");
+                return tr("Smart Playlist");
             }
-            return Plurals::tracksWithDuration(pl->songs.count(), Utils::formatTime(pl->totalTime()));
+            return tr("%n Tracks (%1)", "", pl->songs.count()).arg(Utils::formatTime(pl->totalTime()));
+        case Cantata::Role_TitleActions:
+            return true;
         default:
             return ActionModel::data(index, role);
         }
@@ -353,16 +364,16 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
         SongItem *s=static_cast<SongItem *>(item);
 
         switch (role) {
-        #ifdef ENABLE_UBUNTU
-        case Cantata::Role_Image:
-            return QString();
-        #endif
+        case Cantata::Role_ListImage:
+            return true;
         case Cantata::Role_IsCollection:
             return false;
         case Cantata::Role_CollectionId:
             return s->parent->key;
         case Cantata::Role_Key:
             return s->key;
+        case Cantata::Role_CoverSong:
+        case Cantata::Role_SongWithRating:
         case Cantata::Role_Song: {
             QVariant var;
             var.setValue<Song>(*s);
@@ -433,27 +444,41 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
                         return QVariant();
                     }
                     return s->year;
+                case COL_ORIGYEAR:
+                    if (s->origYear <= 0) {
+                        return QVariant();
+                    }
+                    return s->origYear;
                 case COL_GENRE:
                     return s->displayGenre();
                 case COL_COMPOSER:
                     return s->composer();
                 case COL_PERFORMER:
                     return s->performer();
+                case COL_FILENAME:
+                    return Utils::getFile(QUrl(s->file).path());
+                case COL_PATH: {
+                    QString dir=Utils::getDir(QUrl(s->file).path());
+                    return dir.endsWith("/") ? dir.left(dir.length()-1) : dir;
+                }
                 default:
                     break;
                 }
             }
             return s->entryName();
         case Qt::ToolTipRole:
+            if (!Settings::self()->infoTooltips()) {
+                return QVariant();
+            }
             return s->toolTip();
-        #ifndef ENABLE_UBUNTU
         case Qt::DecorationRole:
-            return multiCol ? QVariant() : (s->title.isEmpty() ? Icons::self()->streamIcon : Icons::self()->audioFileIcon);
-        #endif
+            return multiCol ? QVariant() : (s->title.isEmpty() ? Icons::self()->streamIcon : Icons::self()->audioListIcon);
         case Cantata::Role_MainText:
-            return s->title.isEmpty() ? s->file : s->title;
+            return s->title.isEmpty() ? s->name().isEmpty() ? s->file : s->name() : s->title;
         case Cantata::Role_SubText:
-            return s->artist+QLatin1String(" - ")+s->displayAlbum();
+            return (s->artist.isEmpty() ? QString() : (s->artist+QString(" – ")))+
+                   (s->displayAlbum().isEmpty() ? QString() : (s->displayAlbum()+QString(" – ")))+
+                   (s->time>0 ? Utils::formatTime(s->time) : QString());
         default:
             return ActionModel::data(index, role);
         }
@@ -462,7 +487,6 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
-#ifndef ENABLE_UBUNTU
 bool PlaylistsModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
     if (Cantata::Role_DropAdjust==role) {
@@ -472,7 +496,6 @@ bool PlaylistsModel::setData(const QModelIndex &index, const QVariant &value, in
         return QAbstractItemModel::setData(index, value, role);
     }
 }
-#endif
 
 Qt::ItemFlags PlaylistsModel::flags(const QModelIndex &index) const
 {
@@ -487,23 +510,31 @@ Qt::DropActions PlaylistsModel::supportedDropActions() const
     return Qt::CopyAction | Qt::MoveAction;
 }
 
+static QString songFileName(const Song &song)
+{
+    if (song.isStream() && song.title.isEmpty()) {
+        return MPDParseUtils::addStreamName(song.file, song.name());
+    }
+    return song.file;
+}
+
 QStringList PlaylistsModel::filenames(const QModelIndexList &indexes, bool filesOnly) const
 {
     QStringList fnames;
     QSet<Item *> selectedPlaylists;
-    foreach(QModelIndex index, indexes) {
+    for(QModelIndex index: indexes) {
         Item *item=static_cast<Item *>(index.internalPointer());
 
         if (item->isPlaylist()) {
             selectedPlaylists.insert(item);
-            foreach (const SongItem *s, static_cast<PlaylistItem*>(item)->songs) {
+            for (const SongItem *s: static_cast<PlaylistItem*>(item)->songs) {
                 if (!filesOnly || !s->file.contains(':')) {
                     fnames << s->file;
                 }
             }
         } else if (!selectedPlaylists.contains(static_cast<SongItem*>(item)->parent)) {
             if (!filesOnly || !static_cast<SongItem*>(item)->file.contains(':')) {
-                fnames << static_cast<SongItem*>(item)->file;
+                fnames << songFileName(* static_cast<SongItem*>(item));
             }
         }
     }
@@ -516,17 +547,17 @@ QList<Song> PlaylistsModel::songs(const QModelIndexList &indexes) const
     QList<Song> songs;
     QSet<Item *> selectedPlaylists;
 
-    foreach (const QModelIndex &idx, indexes) {
+    for (const QModelIndex &idx: indexes) {
         Item *item=static_cast<Item *>(idx.internalPointer());
         if (item->isPlaylist()) {
             PlaylistItem *playlist=static_cast<PlaylistItem *>(item);
-            foreach (const SongItem *song, playlist->songs) {
+            for (const SongItem *song: playlist->songs) {
                 selectedPlaylists.insert(item);
                 songs.append(*song);
             }
         }
     }
-    foreach (const QModelIndex &idx, indexes) {
+    for (const QModelIndex &idx: indexes) {
         PlaylistsModel::Item *item=static_cast<PlaylistsModel::Item *>(idx.internalPointer());
         if (!item->isPlaylist()) {
             SongItem *song=static_cast<SongItem *>(item);
@@ -538,7 +569,6 @@ QList<Song> PlaylistsModel::songs(const QModelIndexList &indexes) const
     return songs;
 }
 
-#ifndef ENABLE_UBUNTU
 static const QLatin1String constPlaylistNameMimeType("cantata/playlistnames");
 static const QLatin1String constPositionsMimeType("cantata/positions");
 static const char *constPlaylistProp="hasPlaylist";
@@ -550,7 +580,7 @@ QMimeData * PlaylistsModel::mimeData(const QModelIndexList &indexes) const
     QStringList playlists;
     QList<quint32> positions;
     QSet<Item *> selectedPlaylists;
-    foreach(QModelIndex index, indexes) {
+    for (QModelIndex index: indexes) {
         if (!index.isValid() || 0!=index.column()) {
             continue;
         }
@@ -558,12 +588,19 @@ QMimeData * PlaylistsModel::mimeData(const QModelIndexList &indexes) const
         Item *item=static_cast<Item *>(index.internalPointer());
 
         if (item->isPlaylist()) {
+            PlaylistItem *playlist=static_cast<PlaylistItem*>(item);
             int pos=0;
             selectedPlaylists.insert(item);
-            foreach (const SongItem *s, static_cast<PlaylistItem*>(item)->songs) {
-                filenames << s->file;
-                playlists << static_cast<PlaylistItem*>(item)->name;
+            if (playlist->songs.isEmpty()) {
+                filenames << MPDConnection::constPlaylistPrefix+playlist->name;
+                playlists << playlist->name;
                 positions << pos++;
+            } else {
+                for (const SongItem *s: playlist->songs) {
+                    filenames << s->file;
+                    playlists << playlist->name;
+                    positions << pos++;
+               }
             }
             mimeData->setProperty(constPlaylistProp, true);
         } else if (!selectedPlaylists.contains(static_cast<SongItem*>(item)->parent)) {
@@ -629,7 +666,7 @@ bool PlaylistsModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
 
             bool fromThis=false;
             bool fromOthers=false;
-            foreach (const QString &p, playlists) {
+            for (const QString &p: playlists) {
                 if (p==pl->name) {
                     fromThis=true;
                 } else {
@@ -661,14 +698,10 @@ QStringList PlaylistsModel::mimeTypes() const
     types << PlayQueueModel::constFileNameMimeType;
     return types;
 }
-#endif
 
 void PlaylistsModel::getPlaylists()
 {
-// ALWAYS need to send listPlaylists() -> as it is now used for favourite streams!
-//    if (enabled) {
-        emit listPlaylists();
-//    }
+    emit listPlaylists();
 }
 
 void PlaylistsModel::clear()
@@ -679,56 +712,28 @@ void PlaylistsModel::clear()
     endResetModel();
 }
 
-void PlaylistsModel::setEnabled(bool e)
-{
-    if (e==enabled) {
-        return;
-    }
-
-    enabled=e;
-    if (enabled) {
-        getPlaylists();
-    } else {
-        clear();
-    }
-}
-
-#ifndef ENABLE_UBUNTU
-QMenu * PlaylistsModel::menu()
+MirrorMenu * PlaylistsModel::menu()
 {
     if (!itemMenu) {
         updateItemMenu(true);
     }
     return itemMenu;
 }
-#endif
 
 void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
 {
-    if (!enabled) {
-        return;
-    }
-
     if (items.isEmpty()) {
         if (playlists.isEmpty()) {
             return;
         }
         beginResetModel();
-        foreach (const Playlist &p, playlists) {
-            if (p.name!=StreamsModel::constPlayListName) {
-                items.append(new PlaylistItem(p, allocateKey()));
-            }
+        for (const Playlist &p: playlists) {
+            items.append(new PlaylistItem(p, allocateKey()));
         }
         endResetModel();
         updateItemMenu();
-        #ifdef ENABLE_UBUNTU
-        emit updated();
-        #endif
     } else if (playlists.isEmpty()) {
         clear();
-        #ifdef ENABLE_UBUNTU
-        emit updated();
-        #endif
     } else {
         QModelIndex parent=QModelIndex();
         QSet<QString> existing;
@@ -736,14 +741,11 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
         QSet<QString> removed;
         QSet<QString> added;
 
-        foreach (PlaylistItem *p, items) {
+        for (PlaylistItem *p: items) {
             existing.insert(p->name);
         }
 
-        foreach (const Playlist &p, playlists) {
-            if (p.name==StreamsModel::constPlayListName) {
-                continue;
-            }
+        for (const Playlist &p: playlists) {
             retreived.insert(p.name);
             PlaylistItem *pl=getPlaylist(p.name);
 
@@ -759,7 +761,7 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
         added=retreived-existing;
 
         if (removed.count()) {
-            foreach (const QString &p, removed) {
+            for (const QString &p: removed) {
                 PlaylistItem *pl=getPlaylist(p);
                 if (pl) {
                     int index=items.indexOf(pl);
@@ -773,7 +775,7 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
         }
         if (added.count()) {
             beginInsertRows(parent, items.count(), items.count()+added.count()-1);
-            foreach (const QString &p, added) {
+            for (const QString &p: added) {
                 int idx=playlists.indexOf(Playlist(p));
                 if (-1!=idx) {
                     items.append(new PlaylistItem(playlists.at(idx), allocateKey()));
@@ -784,9 +786,6 @@ void PlaylistsModel::setPlaylists(const QList<Playlist> &playlists)
 
         if (added.count() || removed.count()) {
             updateItemMenu();
-            #ifdef ENABLE_UBUNTU
-            emit updated();
-            #endif
         }
     }
 }
@@ -797,63 +796,35 @@ void PlaylistsModel::playlistInfoRetrieved(const QString &name, const QList<Song
 
     if (pl) {
         QModelIndex idx=createIndex(items.indexOf(pl), 0, pl);
-        if (pl->songs.isEmpty()) {
-            if (songs.isEmpty()) {
-                return;
-            }
-            beginInsertRows(idx, 0, songs.count()-1);
-            foreach (const Song &s, songs) {
-                pl->songs.append(new SongItem(s, pl));
-            }
-            endInsertRows();
-        } else if (songs.isEmpty()) {
+
+        // If Cantata moves songs, then movedInPlaylist() is called. Likwise if
+        // Cantata deltes songs, removedFromPlaylist() is called. Therefore, no
+        // need to do partial updates here. If another client does a move whilst
+        // a playlist has duplicate songs, then it can fail. #873
+        if (!pl->songs.isEmpty()) {
             beginRemoveRows(idx, 0, pl->songs.count()-1);
             pl->clearSongs();
             endRemoveRows();
-        } else {
-            for (qint32 i=0; i<songs.count(); ++i) {
-                Song s=songs.at(i);
-                SongItem *si=i<pl->songs.count() ? pl->songs.at(i) : 0;
-                if (i>=pl->songs.count() || !(s==*static_cast<Song *>(si))) {
-                    si=i<pl->songs.count() ? pl->getSong(s, i) : 0;
-                    if (!si) {
-                        beginInsertRows(idx, i, i);
-                        pl->songs.insert(i, new SongItem(s, pl));
-                        endInsertRows();
-                    } else {
-                        int existing=pl->songs.indexOf(si);
-                        beginMoveRows(idx, existing, existing, idx, i>existing ? i+1 : i);
-                        SongItem *si=pl->songs.takeAt(existing);
-                        si->key=s.key;
-                        pl->songs.insert(i, si);
-                        endMoveRows();
-                    }
-                } else if (si) {
-                    si->key=s.key;
-                }
-            }
-
-            if (pl->songs.count()>songs.count()) {
-                int toRemove=pl->songs.count()-songs.count();
-                beginRemoveRows(idx, pl->songs.count()-toRemove, pl->songs.count()-1);
-                for (int i=0; i<toRemove; ++i) {
-                    delete pl->songs.takeLast();
-                }
-                endRemoveRows();
-            }
         }
-        pl->updateGenres();
+
+        if (!songs.isEmpty()) {
+            beginInsertRows(idx, 0, songs.count()-1);
+            for (const Song &s: songs) {
+                pl->songs.append(new SongItem(s, pl));
+            }
+            endInsertRows();
+        }
+
         emit updated(idx);
         emit dataChanged(idx, idx);
     } else {
         emit listPlaylists();
     }
-    updateGenreList();
 }
 
 void PlaylistsModel::removedFromPlaylist(const QString &name, const QList<quint32> &positions)
 {
-    PlaylistItem *pl=0;
+    PlaylistItem *pl=nullptr;
     if (0==positions.count() || !(pl=getPlaylist(name))) {
         emit listPlaylists();
         return;
@@ -883,21 +854,19 @@ void PlaylistsModel::removedFromPlaylist(const QString &name, const QList<quint3
         adjust+=(rowEnd-rowBegin)+1;
         endRemoveRows();
         it++;
-        pl->updateGenres();
     }
     emit updated(parent);
-    updateGenreList();
 }
 
 void PlaylistsModel::movedInPlaylist(const QString &name, const QList<quint32> &idx, quint32 pos)
 {
-    PlaylistItem *pl=0;
+    PlaylistItem *pl=nullptr;
     if (!(pl=getPlaylist(name)) || idx.count()>pl->songs.count()) {
         emit listPlaylists();
         return;
     }
 
-    foreach (quint32 i, idx) {
+    for (quint32 i: idx) {
         if (i>=(quint32)pl->songs.count()) {
             emit listPlaylists();
             return;
@@ -946,65 +915,72 @@ void PlaylistsModel::mpdConnectionStateChanged(bool connected)
     }
 }
 
+void PlaylistsModel::coverLoaded(const Song &song, int s)
+{
+    Q_UNUSED(s)
+    if (!song.isArtistImageRequest() && !song.isComposerImageRequest()) {
+        int plRow=0;
+        for (const PlaylistItem *pl: items) {
+            QModelIndex plIdx;
+            int songRow=0;
+            for (const SongItem *s: pl->songs) {
+                if (s->albumArtist()==song.albumArtist() && s->album==song.album) {
+                    if (!plIdx.isValid()) {
+                        plIdx=index(plRow, 0, QModelIndex());
+                    }
+                    QModelIndex idx=index(songRow, 0, plIdx);
+                    emit dataChanged(idx, idx);
+                }
+                songRow++;
+            }
+            plRow++;
+        }
+    }
+}
+
 void PlaylistsModel::updateItemMenu(bool create)
 {
-    #ifndef ENABLE_UBUNTU
     if (!itemMenu) {
         if (!create) {
             return;
         }
-        itemMenu = new QMenu(0);
+        itemMenu = new MirrorMenu(nullptr);
     }
 
     itemMenu->clear();
     itemMenu->addAction(newAction);
     QStringList names;
-    foreach (const PlaylistItem *p, items) {
+    for (const PlaylistItem *p: items) {
         if (!p->isSmartPlaylist) {
             names << p->name;
         }
     }
     qSort(names.begin(), names.end(), PlaylistsProxyModel::compareNames);
-    foreach (const QString &n, names) {
+    for (const QString &n: names) {
         itemMenu->addAction(n, this, SLOT(emitAddToExisting()));
     }
-    #endif
 }
 
 PlaylistsModel::PlaylistItem * PlaylistsModel::getPlaylist(const QString &name)
 {
-    foreach (PlaylistItem *p, items) {
+    for (PlaylistItem *p: items) {
         if (p->name==name) {
             return p;
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 void PlaylistsModel::clearPlaylists()
 {
-    foreach (PlaylistItem *p, items) {
+    for (PlaylistItem *p: items) {
         usedKeys.remove(p->key);
         emit playlistRemoved(p->key);
     }
 
     qDeleteAll(items);
     items.clear();
-    updateGenreList();
-}
-
-void PlaylistsModel::updateGenreList()
-{
-    QSet<QString> newGenres;
-    foreach (PlaylistItem *p, items) {
-        newGenres+=p->genres;
-    }
-
-    if (newGenres!=plGenres) {
-        plGenres=newGenres;
-        emit updateGenres(plGenres);
-    }
 }
 
 quint32 PlaylistsModel::allocateKey()
@@ -1019,25 +995,13 @@ quint32 PlaylistsModel::allocateKey()
     return 0xFFFFFFFF;
 }
 
-PlaylistsModel::SongItem::SongItem(const Song &s, PlaylistItem *p)
-    : Song(s)
-    , parent(p)
-    , genreSet(0)
-{
-    QStringList g=genres();
-    if (g.count()>1) {
-        genreSet=new QSet<QString>();
-        *genreSet=g.toSet();
-    }
-}
-
 PlaylistsModel::PlaylistItem::PlaylistItem(const Playlist &pl, quint32 k)
     : name(pl.name)
     , time(0)
     , key(k)
     , lastModified(pl.lastModified)
 {
-    loaded=isSmartPlaylist=MPDConnection::self()->isMopdidy() && name.startsWith("Smart Playlist:");
+    loaded=isSmartPlaylist=MPDConnection::self()->isMopidy() && name.startsWith("Smart Playlist:");
     if (isSmartPlaylist) {
         shortName=name.mid(16);
     }
@@ -1048,27 +1012,16 @@ PlaylistsModel::PlaylistItem::~PlaylistItem()
     clearSongs();
 }
 
-void PlaylistsModel::PlaylistItem::updateGenres()
-{
-    genres.clear();
-    foreach (const SongItem *s, songs) {
-        if (!s->genre.isEmpty()) {
-            genres.insert(s->genre);
-        }
-    }
-}
-
 void PlaylistsModel::PlaylistItem::clearSongs()
 {
     qDeleteAll(songs);
     songs.clear();
-    updateGenres();
 }
 
 PlaylistsModel::SongItem * PlaylistsModel::PlaylistItem::getSong(const Song &song, int offset)
 {
     int i=0;
-    foreach (SongItem *s, songs) {
+    for (SongItem *s: songs) {
         if (++i<offset) {
             continue;
         }
@@ -1077,15 +1030,17 @@ PlaylistsModel::SongItem * PlaylistsModel::PlaylistItem::getSong(const Song &son
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 quint32 PlaylistsModel::PlaylistItem::totalTime()
 {
     if (0==time) {
-        foreach (SongItem *s, songs) {
+        for (SongItem *s: songs) {
             time+=s->time;
         }
     }
     return time;
 }
+
+#include "moc_playlistsmodel.cpp"
