@@ -1,8 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
- * Copyright (c) 2014 Niklas Wenzel <nikwen.developer@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -28,14 +27,16 @@
 #ifndef PLAYQUEUEMODEL_H
 #define PLAYQUEUEMODEL_H
 
-#include "mpd/song.h"
-#include "mpd/mpdstatus.h"
+#include "mpd-interface/mpdconnection.h"
+#include "mpd-interface/song.h"
+#include "mpd-interface/mpdstatus.h"
 #include "config.h"
 #include <QStringList>
 #include <QAbstractItemModel>
 #include <QList>
 #include <QSet>
 #include <QStack>
+#include <QMap>
 
 class StreamFetcher;
 class Action;
@@ -54,10 +55,14 @@ public:
         COL_ALBUM,
         COL_LENGTH,
         COL_YEAR,
+        COL_ORIGYEAR,
         COL_GENRE,
         COL_PRIO,
         COL_COMPOSER,
         COL_PERFORMER,
+        COL_RATING,
+        COL_FILENAME,
+        COL_PATH,
 
         COL_COUNT
     };
@@ -70,34 +75,29 @@ public:
         QList<quint8> priority;
     };
 
-    #ifndef ENABLE_UBUNTU
     static const QLatin1String constMoveMimeType;
     static const QLatin1String constFileNameMimeType;
     static const QLatin1String constUriMimeType;
+    static QSet<QString> constFileExtensions;
 
     static void encode(QMimeData &mimeData, const QString &mime, const QStringList &values);
     static void encode(QMimeData &mimeData, const QString &mime, const QList<quint32> &values);
     static QStringList decode(const QMimeData &mimeData, const QString &mime);
     static QList<quint32> decodeInts(const QMimeData &mimeData, const QString &mime);
     static QString headerText(int col);
-    #endif
 
-    PlayQueueModel(QObject *parent = 0);
-    ~PlayQueueModel();
-    QModelIndex index(int row, int column, const QModelIndex &parent=QModelIndex()) const;
-    QModelIndex parent(const QModelIndex &idx) const;
-    #ifndef ENABLE_UBUNTU
-    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
-    #endif
-    int rowCount(const QModelIndex &parent = QModelIndex()) const;
-    int columnCount(const QModelIndex &) const { return COL_COUNT; }
-    #ifdef ENABLE_UBUNTU
-    QHash<int, QByteArray> roleNames() const;
-    #endif
-    QVariant data(const QModelIndex &, int) const;
-    #ifndef ENABLE_UBUNTU
-    bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);
-    #endif
+    static PlayQueueModel * self();
+
+    PlayQueueModel(QObject *parent = nullptr);
+    ~PlayQueueModel() override;
+    QModelIndex index(int row, int column, const QModelIndex &parent=QModelIndex()) const override;
+    QModelIndex parent(const QModelIndex &idx) const override;
+    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;
+    bool setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role = Qt::EditRole) override;
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    int columnCount(const QModelIndex &) const override { return COL_COUNT; }
+    QVariant data(const QModelIndex &, int) const override;
+    bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
     void updateCurrentSong(quint32 id);
     qint32 getIdByRow(qint32 row) const;
     qint32 getSongId(const QString &file) const;
@@ -105,47 +105,46 @@ public:
     qint32 getRowById(qint32 id) const;
     Song getSongByRow(const qint32 row) const;
     Song getSongById(qint32 id) const;
-    Qt::DropActions supportedDropActions() const;
-    Qt::ItemFlags flags(const QModelIndex &index) const;
-    #ifndef ENABLE_UBUNTU
-    QStringList mimeTypes() const;
-    QMimeData *mimeData(const QModelIndexList &indexes) const;
-    bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent);
-    #endif
+    Qt::DropActions supportedDropActions() const override;
+    Qt::ItemFlags flags(const QModelIndex &index) const override;
+    QStringList mimeTypes() const override;
+    QMimeData *mimeData(const QModelIndexList &indexes) const override;
+    bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent) override;
     QStringList filenames();
     void clear();
     qint32 currentSong() const { return currentSongId; }
     qint32 currentSongRow() const;
     void setState(MPDState st);
-    void update(const QList<Song> &songList);
+    void update(const QList<Song> &songList, bool isComplete);
     void setStopAfterTrack(qint32 track);
     void clearStopAfterTrack() { setStopAfterTrack(-1); }
-    bool removeCantataStreams();
+    bool removeCantataStreams(bool cdOnly=false);
     void removeAll();
     void remove(const QList<int> &rowsToRemove);
     void crop(const QList<int> &rowsToKeep);
-    #ifndef ENABLE_UBUNTU
+    void setRating(const QList<int> &rows, quint8 rating) const;
     Action * shuffleAct() { return shuffleAction; }
     Action * removeDuplicatesAct() { return removeDuplicatesAction; }
     Action * sortAct() { return sortAction; }
     Action * undoAct() { return undoAction; }
     Action * redoAct() { return redoAction; }
-    #endif
     void enableUndo(bool e);
     bool lastCommandWasUnodOrRedo() const { return Cmd_Other!=lastCommand; }
     qint32 totalTime() const { return time; }
+    void remove(const QList<Song> &rem);
 
 private:
     void saveHistory(const QList<Song> &prevList);
     void controlActions();
     void addSortAction(const QString &name, const QString &key);
+    QStringList parseUrls(const QStringList &urls);
 
 public Q_SLOTS:
-    void load(const QStringList &urls);
-    void addItems(const QStringList &items, int row, bool replace, quint8 priority);
-    void addItems(const QStringList &items, bool replace, quint8 priority) { addItems(items, -1, replace, priority); }
-    void addFiles(const QStringList &filenames, int row, bool replace, quint8 priority);
-    void prioritySet(const QList<qint32> &ids, quint8 priority);
+    void load(const QStringList &urls, int action=MPDConnection::Append, quint8 priority=0, bool decreasePriority=false);
+    void addItems(const QStringList &items, int row, int action, quint8 priority, bool decreasePriority);
+    void addItems(const QStringList &items, int action, quint8 priority, bool decreasePriority) { addItems(items, -1, action, priority, decreasePriority); }
+    void addFiles(const QStringList &filenames, int row, int action, quint8 priority, bool decreasePriority=false);
+    void prioritySet(const QMap<qint32, quint8> &prio);
     void stats();
     void cancelStreamFetch();
     void shuffleAlbums();
@@ -154,20 +153,22 @@ public Q_SLOTS:
 private Q_SLOTS:
     void sortBy();
     void stopAfterCurrentChanged(bool afterCurrent);
-    void remove(const QList<Song> &rem);
     void updateDetails(const QList<Song> &updated);
     void undo();
     void redo();
     void removeDuplicates();
-    // Touch version...
-    void setCover(const Song &song, const QImage &img, const QString &file);
+    void ratingResult(const QString &file, quint8 r);
+    void stickerDbChanged();
 
 Q_SIGNALS:
     void stop(bool afterCurrent);
     void clearStopAfter();
-    void filesAdded(const QStringList filenames, const quint32 row, const quint32 size, int action, quint8 priority);
+    void filesAdded(const QStringList filenames, const quint32 row, const quint32 size, int action, quint8 priority, bool decreasePriority);
     void populate(const QStringList &items, const QList<quint8> &priority);
     void move(const QList<quint32> &items, const quint32 row, const quint32 size);
+    void setOrder(const QList<quint32> &items);
+    void getRating(const QString &file) const;
+    void setRating(const QStringList &files, quint8 rating) const;
     void statsUpdated(int songs, quint32 time);
     void fetchingStreams();
     void streamsFetched();
@@ -177,6 +178,8 @@ Q_SIGNALS:
     void clearEntries();
     void addAndPlay(const QString &file);
     void startPlayingSongId(qint32 id);
+    void currentSongRating(const QString &file, quint8 r);
+    void error(const QString &str);
 
 private:
     QList<Song> songs;
@@ -201,17 +204,13 @@ private:
     Command lastCommand;
     QStack<UndoItem> undoStack;
     QStack<UndoItem> redoStack;
-    #ifdef ENABLE_UBUNTU
-    mutable QMap<quint16, QString> covers;
-    mutable QMap<quint16, Song> coverRequests;
-    #else
     quint32 dropAdjust;
     Action *removeDuplicatesAction;
     Action *undoAction;
     Action *redoAction;
     Action *shuffleAction;
     Action *sortAction;
-    #endif
+    QMap<int, int> alignments;
 };
 
 #endif
