@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,7 +26,7 @@
 
 #include <QSortFilterProxyModel>
 #include <QStringList>
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 #include "config.h"
 
 class QMimeData;
@@ -34,25 +34,24 @@ class QMimeData;
 class ProxyModel : public QSortFilterProxyModel
 {
 public:
-    ProxyModel(QObject *parent) : QSortFilterProxyModel(parent), isSorted(false), filterEnabled(false), filter(0) { }
-    virtual ~ProxyModel() { }
+    ProxyModel(QObject *parent) : QSortFilterProxyModel(parent), isSorted(false), filterEnabled(false), filter(nullptr) { }
+    ~ProxyModel() override { }
 
-    bool update(const QString &text, const QString &genre=QString());
+    bool update(const QString &text);
     const void * filterItem() const { return filter; }
     void setFilterItem(void *f) { filter=f; }
     void setRootIndex(const QModelIndex &idx) { rootIndex=idx.isValid() ? mapToSource(idx) : idx; }
     bool isChildOfRoot(const QModelIndex &idx) const;
-    bool isEmpty() const { return filterGenre.isEmpty() && filterStrings.isEmpty() && 0==filter; }
+    bool isEmpty() const { return filterStrings.isEmpty() && nullptr==filter; }
     bool enabled() const { return filterEnabled; }
     const QString & filterText() const { return origFilterText; }
+    void resort();
     void sort() { isSorted=false; sort(0); }
-    void sort(int column, Qt::SortOrder order = Qt::AscendingOrder);
+    void sort(int column, Qt::SortOrder order = Qt::AscendingOrder) override;
     QList<int> mapToSourceRows(const QModelIndexList &list) const;
-    QModelIndex mapToSource(const QModelIndex &idx) const { return QSortFilterProxyModel::mapToSource(idx); }
-    QModelIndexList mapToSource(const QModelIndexList &list, bool leavesOnly) const;
-    #ifndef ENABLE_UBUNTU
-    QMimeData * mimeData(const QModelIndexList &indexes) const;
-    #endif
+    QModelIndex mapToSource(const QModelIndex &idx) const override { return QSortFilterProxyModel::mapToSource(idx); }
+    QModelIndexList mapToSource(const QModelIndexList &list, bool leavesOnly=true) const;
+    QMimeData * mimeData(const QModelIndexList &indexes) const override;
     QModelIndexList leaves(const QModelIndexList &list) const;
 
 protected:
@@ -65,12 +64,13 @@ private:
 protected:
     bool isSorted;
     bool filterEnabled;
-    QString filterGenre;
     QModelIndex rootIndex;
     QString origFilterText;
     QStringList filterStrings;
     uint unmatchedStrings;
     const void *filter;
+    quint16 yearFrom;
+    quint16 yearTo;
 };
 
 #endif
