@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,45 +24,40 @@
 #include "searchmodel.h"
 #include "widgets/icons.h"
 #include "roles.h"
-#include "mpd/mpdconnection.h"
 #include "playqueuemodel.h"
+#include "gui/settings.h"
 #include <QString>
 #include <QVariant>
 #include <QMimeData>
 #include <QLocale>
 #include <QUrl>
-#if QT_VERSION >= 0x050000
-#include <QUrlQuery>
-#endif
-#if defined ENABLE_MODEL_TEST
-#include "modeltest.h"
-#endif
 
-#ifndef ENABLE_UBUNTU
 QString SearchModel::headerText(int col)
 {
     switch (col) {
+    case COL_TRACK:     return PlayQueueModel::headerText(PlayQueueModel::COL_TRACK);
     case COL_DISC:      return PlayQueueModel::headerText(PlayQueueModel::COL_DISC);
     case COL_TITLE:     return PlayQueueModel::headerText(PlayQueueModel::COL_TITLE);
     case COL_ARTIST:    return PlayQueueModel::headerText(PlayQueueModel::COL_ARTIST);
     case COL_ALBUM:     return PlayQueueModel::headerText(PlayQueueModel::COL_ALBUM);
     case COL_LENGTH:    return PlayQueueModel::headerText(PlayQueueModel::COL_LENGTH);
     case COL_YEAR:      return PlayQueueModel::headerText(PlayQueueModel::COL_YEAR);
+    case COL_ORIGYEAR:  return PlayQueueModel::headerText(PlayQueueModel::COL_ORIGYEAR);
     case COL_GENRE:     return PlayQueueModel::headerText(PlayQueueModel::COL_GENRE);
     case COL_COMPOSER:  return PlayQueueModel::headerText(PlayQueueModel::COL_COMPOSER);
     case COL_PERFORMER: return PlayQueueModel::headerText(PlayQueueModel::COL_PERFORMER);
+    case COL_RATING:    return PlayQueueModel::headerText(PlayQueueModel::COL_RATING);
     default:            return QString();
     }
 }
-#endif
 
 SearchModel::SearchModel(QObject *parent)
     : ActionModel(parent)
     , multiCol(false)
-    , currentId(0)
 {
-    connect(this, SIGNAL(search(QString,QString,int)), MPDConnection::self(), SLOT(search(QString,QString,int)));
-    connect(MPDConnection::self(), SIGNAL(searchResponse(int,QList<Song>)), this, SLOT(searchFinished(int,QList<Song>)));
+    alignments[COL_TITLE]=alignments[COL_ARTIST]=alignments[COL_ALBUM]=alignments[COL_GENRE]=alignments[COL_COMPOSER]=alignments[COL_PERFORMER]=int(Qt::AlignVCenter|Qt::AlignLeft);
+    alignments[COL_TRACK]=alignments[COL_LENGTH]=alignments[COL_DISC]=alignments[COL_YEAR]=alignments[COL_ORIGYEAR]=int(Qt::AlignVCenter|Qt::AlignRight);
+    alignments[COL_RATING]=int(Qt::AlignVCenter|Qt::AlignHCenter);
 }
 
 SearchModel::~SearchModel()
@@ -85,41 +80,35 @@ QModelIndex SearchModel::parent(const QModelIndex &index) const
     return QModelIndex();
 }
 
-#ifndef ENABLE_UBUNTU
 QVariant SearchModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
     if (Qt::Horizontal==orientation) {
         switch (role) {
-        case Qt::DisplayRole:
+        case Qt::DisplayRole:            
             return headerText(section);
+        case Cantata::Role_ContextMenuText:
+            return COL_TRACK==section ? tr("# (Track Number)") : headerText(section);
         case Qt::TextAlignmentRole:
-            switch (section) {
-            case COL_TITLE:
-            case COL_ARTIST:
-            case COL_ALBUM:
-            case COL_GENRE:
-            case COL_COMPOSER:
-            case COL_PERFORMER:
-            default:
-                return int(Qt::AlignVCenter|Qt::AlignLeft);
-            case COL_LENGTH:
-            case COL_DISC:
-            case COL_YEAR:
-                return int(Qt::AlignVCenter|Qt::AlignRight);
-            }
+            return alignments[section];
+        case Cantata::Role_InitiallyHidden:
+            return COL_YEAR==section || COL_ORIGYEAR==section || COL_DISC==section || COL_GENRE==section ||
+                   COL_COMPOSER==section || COL_PERFORMER==section || COL_RATING==section;
         case Cantata::Role_Hideable:
-            return COL_YEAR==section || COL_DISC==section || COL_GENRE==section || COL_COMPOSER==section || COL_PERFORMER==section ? true : false;
+            return COL_TITLE!=section && COL_ARTIST!=section;
         case Cantata::Role_Width:
             switch (section) {
+            case COL_TRACK:     return 0.075;
             case COL_DISC:      return 0.03;
-            case COL_TITLE:     return 0.375;
+            case COL_TITLE:     return 0.3;
             case COL_ARTIST:    return 0.27;
             case COL_ALBUM:     return 0.27;
             case COL_LENGTH:    return 0.05;
             case COL_YEAR:      return 0.05;
-            case COL_GENRE:     return 0.115;
-            case COL_COMPOSER:  return 0.20;
-            case COL_PERFORMER: return 0.20;
+            case COL_ORIGYEAR:  return 0.05;
+            case COL_GENRE:     return 0.1;
+            case COL_COMPOSER:  return 0.2;
+            case COL_PERFORMER: return 0.2;
+            case COL_RATING:    return 0.08;
             }
         default:
             break;
@@ -127,9 +116,18 @@ QVariant SearchModel::headerData(int section, Qt::Orientation orientation, int r
     }
     return QVariant();
 }
-#endif
-
 
+bool SearchModel::setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role)
+{
+    if (Qt::Horizontal==orientation && Qt::TextAlignmentRole==role && section>=0) {
+        int al=value.toInt()|Qt::AlignVCenter;
+        if (al!=alignments[section]) {
+            alignments[section]=al;
+            return true;
+        }
+    }
+    return false;
+}
 
 int SearchModel::rowCount(const QModelIndex &parent) const
 {
@@ -138,6 +136,10 @@ int SearchModel::rowCount(const QModelIndex &parent) const
 
 QVariant SearchModel::data(const QModelIndex &index, int role) const
 {
+    if (!index.isValid() && Cantata::Role_RatingCol==role) {
+        return COL_RATING;
+    }
+
     const Song *song = toSong(index);
 
     if (!song) {
@@ -145,37 +147,22 @@ QVariant SearchModel::data(const QModelIndex &index, int role) const
     }
     
     switch (role) {
-    #ifndef ENABLE_UBUNTU
     case Qt::DecorationRole:
         if (multiCol) {
             return QVariant();
         }
         return Song::Playlist==song->type
-                ? Icons::self()->playlistIcon
+                ? Icons::self()->playlistListIcon
                 : song->isStream()
                   ? Icons::self()->streamIcon
-                  : Icons::self()->audioFileIcon;
+                  : Icons::self()->audioListIcon;
     case Qt::TextAlignmentRole:
-        switch (index.column()) {
-        case COL_TITLE:
-        case COL_ARTIST:
-        case COL_ALBUM:
-        case COL_GENRE:
-        case COL_COMPOSER:
-        case COL_PERFORMER:
-        default:
-            return int(Qt::AlignVCenter|Qt::AlignLeft);
-        case COL_LENGTH:
-        case COL_DISC:
-        case COL_YEAR:
-            return int(Qt::AlignVCenter|Qt::AlignRight);
-        }
-    #endif
+        return alignments[index.column()];
     case Qt::DisplayRole:
         if (multiCol) {
             switch (index.column()) {
             case COL_TITLE:
-                return song->title.isEmpty() ? Utils::getFile(song->file) : song->trackAndTitleStr(false);
+                return song->title.isEmpty() ? Utils::getFile(song->file) : song->title;
             case COL_ARTIST:
                 return song->artist.isEmpty() ? Song::unknown() : song->artist;
             case COL_ALBUM:
@@ -186,6 +173,11 @@ QVariant SearchModel::data(const QModelIndex &index, int role) const
                     }
                 }
                 return song->album;
+            case COL_TRACK:
+                if (song->track <= 0) {
+                    return QVariant();
+                }
+                return song->track;
             case COL_LENGTH:
                 return Utils::formatTime(song->time);
             case COL_DISC:
@@ -198,6 +190,11 @@ QVariant SearchModel::data(const QModelIndex &index, int role) const
                     return QVariant();
                 }
                 return song->year;
+            case COL_ORIGYEAR:
+                if (song->origYear <= 0) {
+                    return QVariant();
+                }
+                return song->origYear;
             case COL_GENRE:
                 return song->displayGenre();
             case COL_COMPOSER:
@@ -211,11 +208,41 @@ QVariant SearchModel::data(const QModelIndex &index, int role) const
         }
         return song->entryName();
     case Qt::ToolTipRole:
+        if (!Settings::self()->infoTooltips()) {
+            return QVariant();
+        }
         return song->toolTip();
     case Cantata::Role_MainText:
         return song->title.isEmpty() ? song->file : song->trackAndTitleStr();
-    case Cantata::Role_SubText:
-        return song->artist+QLatin1String(" - ")+song->displayAlbum();
+    case Cantata::Role_SubText: {
+        QString resp=song->artist;
+        QString al=song->displayAlbum();
+        if (!al.isEmpty()) {
+            if (!resp.isEmpty()) {
+                resp+=QString(" – ");
+            }
+            resp+=al;
+        }
+        if (song->time>0) {
+            if (!resp.isEmpty()) {
+                resp+=QString(" – ");
+            }
+            resp+=Utils::formatTime(song->time);
+        }
+        return resp;
+    }
+    case Cantata::Role_ListImage:
+        return true;
+    case Cantata::Role_CoverSong: {
+        QVariant v;
+        v.setValue<Song>(*song);
+        return v;
+    }
+    case Cantata::Role_Song: {
+        QVariant var;
+        var.setValue<Song>(*song);
+        return var;
+    }
     default:
         return ActionModel::data(index, role);
     }
@@ -235,7 +262,7 @@ QStringList SearchModel::filenames(const QModelIndexList &indexes, bool allowPla
 {
     QList<Song> list=songs(indexes, allowPlaylists);
     QStringList fnames;
-    foreach (const Song &s, list) {
+    for (const Song &s: list) {
         fnames.append(s.file);
     }
     return fnames;
@@ -245,20 +272,21 @@ QList<Song> SearchModel::songs(const QModelIndexList &indexes, bool allowPlaylis
 {
     QList<Song> list;
     QSet<QString> files;
-    foreach(QModelIndex index, indexes) {
+    for (const QModelIndex &index: indexes) {
         if (!index.isValid() || 0!=index.column()) {
             continue;
         }
         Song *song=static_cast<Song *>(index.internalPointer());
         if ((allowPlaylists || Song::Playlist!=song->type) && !files.contains(song->file)) {
-            list << *song;
-            files << song->file;
+            Song s=*song;
+            fixPath(s);
+            list << s;
+            files << s.file;
         }
     }
     return list;
 }
 
-#ifndef ENABLE_UBUNTU
 QMimeData * SearchModel::mimeData(const QModelIndexList &indexes) const
 {
     QMimeData *mimeData = new QMimeData();
@@ -272,7 +300,6 @@ QStringList SearchModel::mimeTypes() const
     types << PlayQueueModel::constFileNameMimeType;
     return types;
 }
-#endif
 
 void SearchModel::refresh()
 {
@@ -290,38 +317,22 @@ void SearchModel::clear()
         endRemoveRows();
     }
     currentKey=currentValue=QString();
-    currentId++;
     emit statsUpdated(0, 0);
 }
 
-void SearchModel::search(const QString &key, const QString &value)
+void SearchModel::results(const QList<Song> &songs)
 {
-    if (key==currentKey && value==currentValue) {
-        return;
-    }
-    emit searching();
-    clear();
-    currentKey=key;
-    currentValue=value;
-    currentId++;
-    emit search(key, value, currentId);
-}
-
-void SearchModel::searchFinished(int id, const QList<Song> &result)
-{
-    if (id!=currentId) {
-        return;
-    }
-
     beginResetModel();
     songList.clear();
-    songList=result;
+    songList=songs;
     endResetModel();
     quint32 time=0;
-    foreach (const Song &s, songList) {
+    for (const Song &s: songList) {
         time+=s.time;
     }
 
     emit statsUpdated(songList.size(), time);
     emit searched();
 }
+
+#include "moc_searchmodel.cpp"
