@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,9 +24,10 @@
 #ifndef SEARCH_MODEL_H
 #define SEARCH_MODEL_H
 
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 #include "actionmodel.h"
 #include <QList>
+#include <QMap>
 
 class SearchModel : public ActionModel
 {
@@ -36,43 +37,43 @@ public:
 
     enum Columns
     {
+        COL_TRACK,
+        COL_DISC,
         COL_TITLE,
         COL_ARTIST,
         COL_ALBUM,
-        COL_DISC,
         COL_LENGTH,
         COL_YEAR,
+        COL_ORIGYEAR,
         COL_GENRE,
         COL_COMPOSER,
         COL_PERFORMER,
+        COL_RATING,
 
         COL_COUNT
     };
 
     static QString headerText(int col);
 
-    SearchModel(QObject *parent = 0);
-    ~SearchModel();
-    QModelIndex index(int, int, const QModelIndex & = QModelIndex()) const;
-    QModelIndex parent(const QModelIndex &) const;
-    #ifndef ENABLE_UBUNTU
-    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
-    #endif
-    int rowCount(const QModelIndex &parent = QModelIndex()) const;
-    int columnCount(const QModelIndex &) const { return COL_COUNT; }
-    QVariant data(const QModelIndex &, int) const;
-    Qt::ItemFlags flags(const QModelIndex &index) const;
+    SearchModel(QObject *parent = nullptr);
+    ~SearchModel() override;
+    QModelIndex index(int, int, const QModelIndex & = QModelIndex()) const override;
+    QModelIndex parent(const QModelIndex &) const override;
+    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;
+    bool setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role = Qt::EditRole) override;
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    int columnCount(const QModelIndex &) const override { return COL_COUNT; }
+    QVariant data(const QModelIndex &, int) const override;
+    Qt::ItemFlags flags(const QModelIndex &index) const override;
 
     QStringList filenames(const QModelIndexList &indexes, bool allowPlaylists=false) const;
     QList<Song> songs(const QModelIndexList &indexes, bool allowPlaylists=false) const;
-    #ifndef ENABLE_UBUNTU
-    QMimeData * mimeData(const QModelIndexList &indexes) const;
-    QStringList mimeTypes() const;
-    #endif
+    QMimeData * mimeData(const QModelIndexList &indexes) const override;
+    QStringList mimeTypes() const override;
 
     void refresh();
-    void clear();
-    void search(const QString &key, const QString &value);
+    virtual void clear();
+    virtual void search(const QString &key, const QString &value)=0;
     void setMultiColumn(bool m) { multiCol=m; }
 
 Q_SIGNALS:
@@ -80,21 +81,17 @@ Q_SIGNALS:
     void searched();
     void statsUpdated(int songs, quint32 time);
 
-    void search(const QString &field, const QString &value, int id);
+protected:
+    virtual Song & fixPath(Song &s) const { return s; }
+    void results(const QList<Song> &songs);
+    const Song * toSong(const QModelIndex &index) const { return index.isValid() ? static_cast<const Song *>(index.internalPointer()) : nullptr; }
 
-private Q_SLOTS:
-    void searchFinished(int id, const QList<Song> &result);
-
-private:
-    void clearItems();
-    const Song * toSong(const QModelIndex &index) const { return index.isValid() ? static_cast<const Song *>(index.internalPointer()) : 0; }
-
-private:
+protected:
     bool multiCol;
     QList<Song> songList;
-    int currentId;
     QString currentKey;
     QString currentValue;
+    QMap<int, int> alignments;
 };
 
 #endif
