@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,31 +24,29 @@
 #include "streamsearchmodel.h"
 #include "widgets/icons.h"
 #include "roles.h"
-#include "support/localize.h"
-#include "gui/plurals.h"
 #include "playqueuemodel.h"
 #include "network/networkaccessmanager.h"
 #include "gui/stdactions.h"
+#include "gui/settings.h"
+#include "support/monoicon.h"
+#include "support/utils.h"
 #include <QString>
 #include <QVariant>
 #include <QXmlStreamReader>
 #include <QMimeData>
 #include <QLocale>
 #include <QUrl>
-#if QT_VERSION >= 0x050000
 #include <QUrlQuery>
-#endif
 
 StreamSearchModel::StreamSearchModel(QObject *parent)
     : ActionModel(parent)
-    , category(TuneIn)
     , root(new StreamsModel::CategoryItem(QString(), "root"))
-    , filterRoot(0)
-    , unmatchedStrings(0)
-    , radioTimeSearchUrl("http://opml.radiotime.com/Search.ashx")
-    , shoutCastSearchUrl(QLatin1String("http://")+StreamsModel::constShoutCastHost+QLatin1String("/legacy/genrelist"))
-    , dirbleSearchUrl(QLatin1String("http://")+StreamsModel::constDirbleHost+QLatin1String("/v1/search/apikey/")+StreamsModel::constDirbleApiKey+QLatin1String("/search/"))
 {
+    // ORDER *MUST* MATCH Category ENUM!!!!!
+    root->children.append(new StreamsModel::CategoryItem("http://opml.radiotime.com/Search.ashx", tr("TuneIn"), root, MonoIcon::icon(":tunein.svg", Utils::monoIconColor())));
+    root->children.append(new StreamsModel::CategoryItem(QLatin1String("http://")+StreamsModel::constShoutCastHost+QLatin1String("/legacy/genrelist"), tr("ShoutCast"), root, MonoIcon::icon(":shoutcast.svg", Utils::monoIconColor())));
+    root->children.append(new StreamsModel::CategoryItem(QLatin1String("http://")+StreamsModel::constDirbleHost+QLatin1String("/v2/search/"), tr("Dirble"), root, MonoIcon::icon(":station.svg", Utils::monoIconColor())));
+    icon = MonoIcon::icon(FontAwesome::search, Utils::monoIconColor());
 }
 
 StreamSearchModel::~StreamSearchModel()
@@ -64,7 +62,7 @@ QModelIndex StreamSearchModel::index(int row, int column, const QModelIndex &par
     }
 
     const StreamsModel::CategoryItem * p = parent.isValid() ? static_cast<StreamsModel::CategoryItem *>(parent.internalPointer()) : root;
-    const StreamsModel::Item * c = row<p->children.count() ? p->children.at(row) : 0;
+    const StreamsModel::Item * c = row<p->children.count() ? p->children.at(row) : nullptr;
     return c ? createIndex(row, column, (void *)c) : QModelIndex();
 }
 
@@ -103,25 +101,43 @@ int StreamSearchModel::columnCount(const QModelIndex &) const
 
 QVariant StreamSearchModel::data(const QModelIndex &index, int role) const
 {
+    if (!index.isValid()) {
+        switch (role) {
+        case Cantata::Role_TitleText:
+            return tr("Stream Search");
+        case Cantata::Role_SubText:
+            return tr("Search for radio streams");
+        case Qt::DecorationRole:
+            return icon;
+        }
+        return QVariant();
+    }
+
     const StreamsModel::Item *item = toItem(index);
 
     switch (role) {
     case Qt::DecorationRole:
-        return item->isCategory() ? Icons::self()->streamCategoryIcon : Icons::self()->radioStreamIcon;
+        if (item->parent==root && item->isCategory()) {
+            return static_cast<const StreamsModel::CategoryItem *>(item) ->icon;
+        }
+        return item->isCategory() ? Icons::self()->streamCategoryIcon : Icons::self()->streamListIcon;
     case Qt::DisplayRole:
         return item->name;
     case Qt::ToolTipRole:
+        if (!Settings::self()->infoTooltips()) {
+            return QVariant();
+        }
         return item->isCategory() ? item->name : (item->name+QLatin1String("<br><small><i>")+item->url+QLatin1String("</i></small>"));
     case Cantata::Role_SubText:
         if (item->isCategory()) {
             const StreamsModel::CategoryItem *cat=static_cast<const StreamsModel::CategoryItem *>(item);
             switch (cat->state) {
             case StreamsModel::CategoryItem::Initial:
-                return i18n("Not Loaded");
+                return root==item->parent ? tr("Enter string to search") : tr("Not Loaded");
             case StreamsModel::CategoryItem::Fetching:
-                return i18n("Loading...");
+                return tr("Loading...");
             default:
-                return Plurals::entries(cat->children.count());
+                return tr("%n Entry(s)", "", cat->children.count());
             }
         } else {
             return item->subText.isEmpty() ? QLatin1String("-") : item->subText;
@@ -197,7 +213,7 @@ void StreamSearchModel::fetchMore(const QModelIndex &index)
 QStringList StreamSearchModel::filenames(const QModelIndexList &indexes, bool addPrefix) const
 {
     QStringList fnames;
-    foreach(QModelIndex index, indexes) {
+    for (const QModelIndex &index: indexes) {
         StreamsModel::Item *item=static_cast<StreamsModel::Item *>(index.internalPointer());
         if (!item->isCategory() && !fnames.contains(item->url)) {
             fnames << StreamsModel::modifyUrl(item->url, addPrefix, item->name);
@@ -224,39 +240,19 @@ QStringList StreamSearchModel::mimeTypes() const
 void StreamSearchModel::clear()
 {
     cancelAll();
-    if (!root->children.isEmpty()) {
-        beginRemoveRows(QModelIndex(), 0, root->children.count()-1);
-        if (Filter!=category) {
-            qDeleteAll(root->children);
-        }
-        root->children.clear();
-        endRemoveRows();
+    for (StreamsModel::Item *item: root->children) {
+        StreamsModel::CategoryItem *cat=static_cast<StreamsModel::CategoryItem *>(item);
+		if (cat->children.count()) {
+			QModelIndex index = createIndex(root->children.indexOf(cat), 0, (void *)cat);
+			beginRemoveRows(index, 0, cat->children.count() - 1);
+			qDeleteAll(cat->children);
+			cat->children.clear();
+			endRemoveRows();
+		}
     }
     currentSearch=QString();
 }
 
-static int getParam(const QString &key, QString &query)
-{
-    int index=query.indexOf(" "+key+"=");
-    int val=0;
-    if (-1!=index) {
-        int endPos=query.indexOf(" ", index+3);
-        int end=endPos;
-        if (-1==end) {
-            end=query.length()-1;
-        }
-        int start=index+key.length()+2;
-        val=query.mid(start, (end+1)-start).toInt();
-        if (endPos>start) {
-            query=query.left(index)+query.mid(endPos);
-            query=query.trimmed();
-        } else {
-            query=query.left(index);
-        }
-    }
-    return val;
-}
-
 void StreamSearchModel::search(const QString &searchTerm, bool stationsOnly)
 {
     if (searchTerm==currentSearch) {
@@ -265,76 +261,50 @@ void StreamSearchModel::search(const QString &searchTerm, bool stationsOnly)
     clear();
     currentSearch=searchTerm;
 
-    QUrl searchUrl;
-    #if QT_VERSION < 0x050000
-    QUrl &query=searchUrl;
-    #else
-    QUrlQuery query;
-    #endif
-
-    switch (category) {
-    case Filter:
-        unmatchedStrings=0;
-        filterStrings=searchTerm.split(" ", QString::SkipEmptyParts);
-        if (!filterStrings.isEmpty()) {
-            QList<StreamsModel::Item *> newItems=getStreams(filterRoot);
-            if (!newItems.isEmpty()) {
-                beginInsertRows(QModelIndex(), 0, newItems.count()-1);
-                root->children+=newItems;
-                endInsertRows();
+    for (StreamsModel::Item *item: root->children) {
+        QUrl searchUrl;
+        QUrlQuery query;
+        switch (root->children.indexOf(item)) {
+        case TuneIn: {
+            searchUrl=QUrl(item->url);
+            if (stationsOnly) {
+                query.addQueryItem("types", "station");
             }
+            query.addQueryItem("query", searchTerm);
+            QString locale=QLocale::system().name();
+            if (!locale.isEmpty()) {
+                query.addQueryItem("locale", locale);
+            }
+            break;
         }
-        emit loaded();
-        return;
-    case TuneIn: {
-        searchUrl=QUrl(radioTimeSearchUrl);
-        if (stationsOnly) {
-            query.addQueryItem("types", "station");
+        case ShoutCast: {
+            searchUrl=QUrl(item->url);
+            ApiKeys::self()->addKey(query, ApiKeys::ShoutCast);
+            query.addQueryItem("search", searchTerm);
+            query.addQueryItem("limit", QString::number(200));
+            break;
         }
-        query.addQueryItem("query", searchTerm);
-        QString locale=QLocale::system().name();
-        if (!locale.isEmpty()) {
-            query.addQueryItem("locale", locale);
+        case Dirble:
+            searchUrl=QUrl(item->url+searchTerm);
+            ApiKeys::self()->addKey(query, ApiKeys::Dirble);
+            break;
         }
-        break;
-    }
-    case ShoutCast: {
-        searchUrl=QUrl(shoutCastSearchUrl);
-        QString search=searchTerm;
-        int limit=getParam("limit", search);
-        int bitrate=getParam("br", search);
-        if (0==bitrate) {
-            bitrate=getParam("bitrate", search);
-        }
-        query.addQueryItem("k", StreamsModel::constShoutCastApiKey);
-        query.addQueryItem("search", search);
-        query.addQueryItem("limit", QString::number(limit<1 ? 100 : limit));
-        if (bitrate>=32 && bitrate<=512) {
-            query.addQueryItem("br", QString::number(bitrate));
-        }
-        break;
-    }
-    case Dirble:
-        searchUrl=QUrl(dirbleSearchUrl+searchTerm);
-        break;
-    }
 
-    #if QT_VERSION >= 0x050000
-    searchUrl.setQuery(query);
-    #endif
-    NetworkJob *job=NetworkAccessManager::self()->get(searchUrl);
-    if (jobs.isEmpty()) {
-        emit loading();
+        searchUrl.setQuery(query);
+        NetworkJob *job=NetworkAccessManager::self()->get(searchUrl);
+        if (jobs.isEmpty()) {
+            emit loading();
+        }
+        jobs.insert(job, static_cast<StreamsModel::CategoryItem *>(item));
+        connect(job, SIGNAL(finished()), this, SLOT(jobFinished()));
     }
-    jobs.insert(job, root);
-    connect(job, SIGNAL(finished()), this, SLOT(jobFinished()));
 }
 
 void StreamSearchModel::cancelAll()
 {
     if (!jobs.isEmpty()) {
         QList<NetworkJob *> jobList=jobs.keys();
-        foreach (NetworkJob *j, jobList) {
+        for (NetworkJob *j: jobList) {
             j->cancelAndDelete();
         }
         jobs.clear();
@@ -342,15 +312,6 @@ void StreamSearchModel::cancelAll()
     }
 }
 
-void StreamSearchModel::setCat(Category c)
-{
-    clear();
-    category=c;
-    if (c!=Filter) {
-        filterRoot=0;
-    }
-}
-
 void StreamSearchModel::jobFinished()
 {
     NetworkJob *job=dynamic_cast<NetworkJob *>(sender());
@@ -364,12 +325,14 @@ void StreamSearchModel::jobFinished()
         StreamsModel::CategoryItem *cat=jobs[job];
         cat->state=StreamsModel::CategoryItem::Fetched;
         jobs.remove(job);
-
         QModelIndex index=cat==root ? QModelIndex() : createIndex(cat->parent->children.indexOf(cat), 0, (void *)cat);
+        StreamsModel::Item *i=cat;
+        while (i->parent && i->parent!=root) {
+            i=i->parent;
+        }
         if (job->ok()) {
             QList<StreamsModel::Item *> newItems;
-
-            switch(category) {
+            switch(root->children.indexOf(i)) {
             case TuneIn:    newItems=StreamsModel::parseRadioTimeResponse(job->actualJob(), cat, true); break;
             case ShoutCast: newItems=StreamsModel::parseShoutCastSearchResponse(job->actualJob(), cat); break;
             case Dirble:    newItems=StreamsModel::parseDirbleStations(job->actualJob(), cat); break;
@@ -380,6 +343,12 @@ void StreamSearchModel::jobFinished()
                 cat->children+=newItems;
                 endInsertRows();
             }
+        } else {
+            switch(root->children.indexOf(i)) {
+            case ShoutCast: ApiKeys::self()->isLimitReached(job->actualJob(), ApiKeys::ShoutCast); break;
+            case Dirble:    ApiKeys::self()->isLimitReached(job->actualJob(), ApiKeys::Dirble); break;
+            default : break;
+            }
         }
         emit dataChanged(index, index);
         if (jobs.isEmpty()) {
@@ -388,38 +357,19 @@ void StreamSearchModel::jobFinished()
     }
 }
 
-bool StreamSearchModel::matchesFilter(const QString &str) const
-{
-    if (filterStrings.isEmpty()) {
-        return true;
-    }
-
-    uint ums = unmatchedStrings;
-    int numStrings = filterStrings.count();
-
-    for (int i = 0; i < numStrings; ++i) {
-        if (str.contains(filterStrings.at(i), Qt::CaseInsensitive)) {
-            ums &= ~(1<<i);
-            if (0==ums) {
-                return true;
-            }
-        }
-    }
-
-    return false;
-}
-
 QList<StreamsModel::Item *> StreamSearchModel::getStreams(StreamsModel::CategoryItem *cat)
 {
     QList<StreamsModel::Item *> streams;
     if (cat) {
-        foreach (StreamsModel::Item *i, cat->children) {
+        for (StreamsModel::Item *i: cat->children) {
             if (i->isCategory()) {
                 streams+=getStreams(static_cast<StreamsModel::CategoryItem *>(i));
-            } else if (matchesFilter(i->name)) {
+            } else {
                 streams.append(i);
             }
         }
     }
     return streams;
 }
+
+#include "moc_streamsearchmodel.cpp"
