@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -40,35 +40,33 @@ class StreamSearchModel : public ActionModel
 
 public:
     enum Category {
+        // DO NOT CHANGE ORDER!
         TuneIn,
         ShoutCast,
         Dirble,
-        Filter
+        NumCategories
     };
 
-    StreamSearchModel(QObject *parent = 0);
-    ~StreamSearchModel();
-    QModelIndex index(int, int, const QModelIndex & = QModelIndex()) const;
-    QModelIndex parent(const QModelIndex &) const;
-    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
-    int rowCount(const QModelIndex &parent = QModelIndex()) const;
-    int columnCount(const QModelIndex &) const;
-    QVariant data(const QModelIndex &, int) const;
-    Qt::ItemFlags flags(const QModelIndex &index) const;
-    bool hasChildren(const QModelIndex &index) const;
-    bool canFetchMore(const QModelIndex &index) const;
-    void fetchMore(const QModelIndex &index);
+    StreamSearchModel(QObject *parent = nullptr);
+    ~StreamSearchModel() override;
+    QModelIndex index(int, int, const QModelIndex & = QModelIndex()) const override;
+    QModelIndex parent(const QModelIndex &) const override;
+    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    int columnCount(const QModelIndex &) const override;
+    QVariant data(const QModelIndex &, int) const override;
+    Qt::ItemFlags flags(const QModelIndex &index) const override;
+    bool hasChildren(const QModelIndex &index) const override;
+    bool canFetchMore(const QModelIndex &index) const override;
+    void fetchMore(const QModelIndex &index) override;
 
     QStringList filenames(const QModelIndexList &indexes, bool addPrefix) const;
-    QMimeData * mimeData(const QModelIndexList &indexes) const;
-    QStringList mimeTypes() const;
+    QMimeData * mimeData(const QModelIndexList &indexes) const override;
+    QStringList mimeTypes() const override;
 
     void clear();
     void search(const QString &searchTerm, bool stationsOnly);
     void cancelAll();
-    Category cat() const { return category; }
-    void setCat(Category c);
-    void setFilterRoot(StreamsModel::CategoryItem *fr) { filterRoot=fr; }
  
 Q_SIGNALS:
     void loading();
@@ -79,7 +77,6 @@ private Q_SLOTS:
     void jobFinished();
 
 private:
-    bool matchesFilter(const QString &str) const;
     QList<StreamsModel::Item *> getStreams(StreamsModel::CategoryItem *cat);
     StreamsModel::Item * toItem(const QModelIndex &index) const { return index.isValid() ? static_cast<StreamsModel::Item*>(index.internalPointer()) : root; }
     QList<StreamsModel::Item *> parseRadioTimeResponse(QIODevice *dev, StreamsModel::CategoryItem *cat);
@@ -90,14 +87,7 @@ private:
     QMap<NetworkJob *, StreamsModel::CategoryItem *> jobs;
     StreamsModel::CategoryItem *root;
     QString currentSearch;
-
-    StreamsModel::CategoryItem *filterRoot;
-    QStringList filterStrings;
-    uint unmatchedStrings;
-
-    QString radioTimeSearchUrl;
-    QString shoutCastSearchUrl;
-    QString dirbleSearchUrl;
+    QIcon icon;
 };
 
 #endif
