@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,45 +23,42 @@
 
 #include "magnatuneservice.h"
 #include "magnatunesettingsdialog.h"
-#include "models/musiclibraryitemartist.h"
-#include "models/musiclibraryitemalbum.h"
-#include "models/musiclibraryitemsong.h"
-#include "mpd/song.h"
+#include "db/onlinedb.h"
+#include "models/roles.h"
+#include "support/icon.h"
 #include "support/configuration.h"
-#include "support/localize.h"
+#include "support/monoicon.h"
 #include <QXmlStreamReader>
+#include <QUrl>
 
-MagnatuneMusicLoader::MagnatuneMusicLoader(const QUrl &src)
-    : OnlineMusicLoader(src)
-{
-}
-
-bool MagnatuneMusicLoader::parse(QXmlStreamReader &xml)
+int MagnatuneXmlParser::parse(QXmlStreamReader &xml)
 {
+    QList<Song> *songList=new QList<Song>();
     while (!xml.atEnd()) {
-        if (wasStopped()) {
-            delete library;
-            library=0;
-            return false;
-        }
         xml.readNext();
         if (QXmlStreamReader::StartElement==xml.tokenType() && QLatin1String("Track")==xml.name()) {
-            parseSong(xml);
+            songList->append(parseSong(xml));
+            if (songList->count()>500) {
+                emit songs(songList);
+                songList=new QList<Song>();
+            }
         }
     }
-    return true;
+    if (songList->isEmpty()) {
+        delete songList;
+    } else {
+        emit songs(songList);
+    }
+    return artists.count();
 }
 
-void MagnatuneMusicLoader::parseSong(QXmlStreamReader &xml)
+Song MagnatuneXmlParser::parseSong(QXmlStreamReader &xml)
 {
     Song s;
 //    QString artistImg;
     QString albumImg;
 
     while (!xml.atEnd()) {
-        if (wasStopped()) {
-            return;
-        }
         xml.readNext();
 
         if (QXmlStreamReader::StartElement==xml.tokenType()) {
@@ -80,9 +77,10 @@ void MagnatuneMusicLoader::parseSong(QXmlStreamReader &xml)
                 s.year=value.toInt();
             } else  if (QLatin1String("magnatunegenres")==name) {
                 QStringList genres=value.split(',', QString::SkipEmptyParts);
-                foreach (const QString &g, genres) {
-                    s.addGenre(g);
-                }
+//                for (const QString &g: genres) {
+//                    s.addGenre(g);
+//                }
+                s.genres[0]=genres.first();
             } else if (QLatin1String("seconds")==name) {
                  s.time=value.toInt();
 //            } else if (QLatin1String("cover_small")==name) {
@@ -99,70 +97,29 @@ void MagnatuneMusicLoader::parseSong(QXmlStreamReader &xml)
         }
     }
 
-    if (!s.title.isEmpty()) {
-        s.fillEmptyFields();
-        MusicLibraryItemArtist *artist = library->artist(s);
-        MusicLibraryItemAlbum *album = artist->album(s);
-        MusicLibraryItemSong *song=new MusicLibraryItemSong(s, album);
-        QSet<QString> songGenres=song->allGenres();
-        album->append(song);
-        album->addGenres(songGenres);
-        artist->addGenres(songGenres);
-        library->addGenres(songGenres);
-//        if (!artistImg.isEmpty() && artist->imageUrl().isEmpty()) {
-//            artist->setImageUrl(artistImg);
-//        }
-        if (!albumImg.isEmpty() && album->imageUrl().isEmpty()) {
-            album->setImageUrl(albumImg);
+    if (!albumImg.isEmpty()) {
+        QString key=s.artistOrComposer()+"-"+s.albumId();
+        if (!albumUrls.contains(key)) {
+            albumUrls.insert(key);
+            emit coverUrl(s.artistOrComposer(), s.album, albumImg);
         }
     }
+    artists.insert(s.artistOrComposer());
+    return s;
 }
 
-const QLatin1String MagnatuneService::constName("Magnatune");
+static const QLatin1String constListingUrl("http://magnatune.com/info/song_info_xml.gz");
+static const QLatin1String constName("magnatune");
 static const char * constStreamingHostname = "streaming.magnatune.com";
 static const char * constDownloadHostname = "download.magnatune.com";
 
-void MagnatuneService::createLoader()
-{
-    loader=new MagnatuneMusicLoader(QUrl("http://magnatune.com/info/song_info_xml.gz"));
-}
-
-Song MagnatuneService::fixPath(const Song &orig, bool) const
-{
-    Song s=orig;
-    if (MB_None!=membership) {
-        QUrl url;
-        #if QT_VERSION < 0x050000
-        url.setEncodedUrl(orig.file.toLocal8Bit());
-        #else
-        url=QUrl(orig.file);
-        #endif
-        url.setScheme("http");
-        url.setHost(MB_Streaming==membership ? constStreamingHostname : constDownloadHostname);
-        url.setUserName(username);
-        url.setPassword(password);
-
-        // And remove the commercial
-        QString path = url.path();
-        path.insert(path.lastIndexOf('.'), "_nospeech");
-        url.setPath(path);
-        s.file=url.toString();
-    // TODO: Magnatune downloads!
-    //    if (MB_Download==membership) {
-    //        s.genre=downloadTypeStr(download);
-    //    }
-    }
-    s.setIsFromOnlineService(constName);
-    return encode(s);
-}
-
 QString MagnatuneService::membershipStr(MemberShip f, bool trans)
 {
     switch (f) {
     default:
-    case MB_None :      return trans ? i18n("None") : QLatin1String("none");
-    case MB_Streaming : return trans ? i18n("Streaming") : QLatin1String("streaming");
-//    case MB_Download :  return trans ? i18n("Download") : QLatin1String("download"); // TODO: Magnatune downloads!
+    case MB_None :      return trans ? tr("None") : QLatin1String("none");
+    case MB_Streaming : return trans ? tr("Streaming") : QLatin1String("streaming");
+//    case MB_Download :  return trans ? tr("Download") : QLatin1String("download"); // TODO: Magnatune downloads!
     }
 }
 
@@ -180,11 +137,11 @@ QString MagnatuneService::downloadTypeStr(DownloadType f, bool trans)
 {
     switch (f) {
     default:
-    case DL_Mp3 :    return trans ? i18n("MP3 128k") : QLatin1String("mp3");
-    case DL_Mp3Vbr : return trans ? i18n("MP3 VBR") : QLatin1String("vbr");
-    case DL_Ogg :    return trans ? i18n("Ogg Vorbis") : QLatin1String("ogg");
-    case DL_Flac :   return trans ? i18n("FLAC"): QLatin1String("flac");
-    case DL_Wav :    return trans ? i18n("WAV") : QLatin1String("wav");
+    case DL_Mp3 :    return trans ? tr("MP3 128k") : QLatin1String("mp3");
+    case DL_Mp3Vbr : return trans ? tr("MP3 VBR") : QLatin1String("vbr");
+    case DL_Ogg :    return trans ? tr("Ogg Vorbis") : QLatin1String("ogg");
+    case DL_Flac :   return trans ? tr("FLAC"): QLatin1String("flac");
+    case DL_Wav :    return trans ? tr("WAV") : QLatin1String("wav");
     }
 }
 
@@ -198,24 +155,98 @@ static MagnatuneService::DownloadType toDownloadType(const QString &f)
     return MagnatuneService::DL_Mp3;
 }
 
-void MagnatuneService::loadConfig()
+MagnatuneService::MagnatuneService(QObject *p)
+    : OnlineDbService(new OnlineDb(constName, p), p)
 {
+    icn=MonoIcon::icon(":magnatune.svg", Utils::monoIconColor());
+    useCovers(name());
     Configuration cfg(constName);
-
-    membership=toMembership(cfg.get("membership", membershipStr(membership)));
-    download=toDownloadType(cfg.get("download", downloadTypeStr(download)));
+    membership=toMembership(cfg.get("membership", membershipStr(MB_None)));
+    download=toDownloadType(cfg.get("download", downloadTypeStr(DL_Mp3)));
     username=cfg.get("username", username);
     password=cfg.get("username", password);
 }
 
-void MagnatuneService::saveConfig()
+QVariant MagnatuneService::data(const QModelIndex &index, int role) const
 {
-    Configuration cfg(constName);
+    if (index.isValid()) {
+        switch (role) {
+        case Cantata::Role_CoverSong: {
+            QVariant v;
+            Item *item = static_cast<Item *>(index.internalPointer());
+            switch (item->getType()) {
+            case T_Album:
+                if (item->getSong().isEmpty()) {
+                    Song song;
+                    song.artist=item->getParent()->getId();
+                    song.album=item->getId();
+                    song.setIsFromOnlineService(constName);
+                    song.file=constName; // Just so that isEmpty() is false!
+                    QString url=static_cast<OnlineDb *>(db)->getCoverUrl(/*T_Album==topLevel() ? static_cast<AlbumItem *>(item)->getArtistId() : */item->getParent()->getId(), item->getId());
+                    song.setExtraField(Song::OnlineImageUrl, url);
+                    item->setSong(song);
+                }
+                v.setValue<Song>(item->getSong());
+                break;
+            case T_Artist:
+                break;
+            default:
+                break;
+            }
+            return v;
+        }
+        }
+    }
+    return OnlineDbService::data(index, role);
+}
 
-    cfg.set("membership",  membershipStr(membership));
-    cfg.set("download",  downloadTypeStr(download));
-    cfg.set("username", username);
-    cfg.set("password", password);
+QString MagnatuneService::name() const
+{
+    return constName;
+}
+
+QString MagnatuneService::title() const
+{
+    return QLatin1String("Magnatune");
+}
+
+QString MagnatuneService::descr() const
+{
+    return tr("Online music from magnatune.com");
+}
+
+OnlineXmlParser * MagnatuneService::createParser()
+{
+    return new MagnatuneXmlParser();
+}
+
+QUrl MagnatuneService::listingUrl() const
+{
+    return QUrl(constListingUrl);
+}
+
+Song & MagnatuneService::fixPath(Song &s) const
+{
+    s.type=Song::OnlineSvrTrack;
+    if (MB_None!=membership) {
+        QUrl url(s.file);
+        url.setScheme("http");
+        url.setHost(MB_Streaming==membership ? constStreamingHostname : constDownloadHostname);
+        url.setUserName(username);
+        url.setPassword(password);
+
+        // And remove the commercial
+        QString path = url.path();
+        path.insert(path.lastIndexOf('.'), "_nospeech");
+        url.setPath(path);
+        s.file=url.toString();
+    // TODO: Magnatune downloads!
+    //    if (MB_Download==membership) {
+    //        ???
+    //    }
+    }
+    s.setIsFromOnlineService(name());
+    return encode(s);
 }
 
 void MagnatuneService::configure(QWidget *p)
@@ -227,6 +258,12 @@ void MagnatuneService::configure(QWidget *p)
         password=dlg.password();
         membership=(MemberShip)dlg.membership();
         download=(DownloadType)dlg.download();
-        saveConfig();
-    }
+        Configuration cfg(constName);
+
+        cfg.set("membership",  membershipStr(membership));
+        cfg.set("download",  downloadTypeStr(download));
+        cfg.set("username", username);
+        cfg.set("password", password);    }
 }
+
+#include "moc_magnatuneservice.cpp"
