@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -29,13 +29,15 @@
 #include "widgets/icons.h"
 #include "support/spinner.h"
 #include "widgets/basicitemdelegate.h"
-#include "models/onlineservicesmodel.h"
 #include "podcastservice.h"
 #include "support/utils.h"
 #include "support/action.h"
+#include "support/monoicon.h"
 #include "widgets/textbrowser.h"
 #include "support/messagewidget.h"
+#include "gui/covers.h"
 #include "rssparser.h"
+#include "podcastservice.h"
 #include "config.h"
 #include <QLabel>
 #include <QPushButton>
@@ -53,12 +55,8 @@
 #include <QCoreApplication>
 #include <QDesktopServices>
 #include <QNetworkRequest>
-#if QT_VERSION >= 0x050000
 #include <QUrlQuery>
 #include <QJsonDocument>
-#else
-#include "qjson/parser.h"
-#endif
 
 static int iCount=0;
 
@@ -90,16 +88,16 @@ public:
     ITunesSearchPage(QWidget *p)
         : PodcastSearchPage(p,
                             QLatin1String("iTunes"),
-                            QLatin1String("itunes"),
+                            FontAwesome::apple,
                             QUrl(QLatin1String("http://ax.phobos.apple.com.edgesuite.net/WebObjects/MZStoreServices.woa/wa/wsSearch")),
                             QLatin1String("term"),
                             QStringList() << QLatin1String("country") << QLatin1String("US") << QLatin1String("media") << QLatin1String("podcast"))
     {
     }
 
-    void parse(const QVariant &data)
+    void parse(const QVariant &data) override
     {
-        foreach (const QVariant &resultVariant, data.toMap()[QLatin1String("results")].toList()) {
+        for (const QVariant &resultVariant: data.toMap()[QLatin1String("results")].toList()) {
             QVariantMap result(resultVariant.toMap());
             if (result[QLatin1String("kind")].toString() != QLatin1String("podcast")) {
                 continue;
@@ -110,7 +108,7 @@ public:
                        result[QLatin1String("artworkUrl100")].toUrl(),
                        QString(),
                        result[QLatin1String("collectionViewUrl")].toString(),
-                       0);
+                       nullptr);
         }
     }
 };
@@ -121,23 +119,23 @@ public:
     GPodderSearchPage(QWidget *p)
         : PodcastSearchPage(p,
                             QLatin1String("GPodder"),
-                            QLatin1String("gpodder"),
+                            FontAwesome::podcast,
                             QUrl(QLatin1String("http://gpodder.net/search.json")),
                             QLatin1String("q"))
     {
     }
 
-    void parse(const QVariant &data)
+    void parse(const QVariant &data) override
     {
         QVariantList list=data.toList();
-        foreach (const QVariant &var, list) {
+        for (const QVariant &var: list) {
             QVariantMap map=var.toMap();
             addPodcast(map[QLatin1String("title")].toString(),
                        map[QLatin1String("url")].toUrl(),
                        map[QLatin1String("logo_url")].toUrl(),
                        map[QLatin1String("description")].toString(),
                        map[QLatin1String("website")].toString(),
-                       0);
+                       nullptr);
         }
     }
 };
@@ -145,8 +143,8 @@ public:
 PodcastPage::PodcastPage(QWidget *p, const QString &n)
     : QWidget(p)
     , pageName(n)
-    , job(0)
-    , imageJob(0)
+    , job(nullptr)
+    , imageJob(nullptr)
 {
     tree = new QTreeWidget(this);
     tree->setItemDelegate(new BasicItemDelegate(tree));
@@ -192,7 +190,7 @@ void PodcastPage::cancel()
     spinner->stop();
     if (job) {
         job->cancelAndDelete();
-        job=0;
+        job=nullptr;
     }
 }
 
@@ -201,7 +199,7 @@ void PodcastPage::cancelImage()
     imageSpinner->stop();
     if (imageJob) {
         imageJob->cancelAndDelete();
-        imageJob=0;
+        imageJob=nullptr;
     }
 }
 
@@ -219,7 +217,7 @@ void PodcastPage::addPodcast(const QString &title, const QUrl &url, const QUrl &
     podItem->setData(0, ImageUrlRole, image);
     podItem->setData(0, DescriptionRole, description);
     podItem->setData(0, WebPageUrlRole, webPage);
-    podItem->setIcon(0, Icons::self()->audioFileIcon);
+    podItem->setIcon(0, Icons::self()->audioListIcon);
 }
 
 static QString encode(const QImage &img)
@@ -253,16 +251,16 @@ void PodcastPage::updateText()
             if (!descr.isEmpty()) {
                 str+="<p>"+descr+"</p><br/>";
             }
-            str+="<table><tr><td><b>"+i18n("RSS:")+"</b></td><td><a href=\""+url.toString()+"\">"+url.toString()+"</a></td></tr>";
+            str+="<table><tr><td><b>"+tr("RSS:")+"</b></td><td><a href=\""+url.toString()+"\">"+url.toString()+"</a></td></tr>";
             if (!web.isEmpty()) {
-                str+="<tr><td><b>"+i18n("Website:")+"</b></td><td><a href=\""+web+"\">"+web+"</a></td></tr>";
+                str+="<tr><td><b>"+tr("Website:")+"</b></td><td><a href=\""+web+"\">"+web+"</a></td></tr>";
             }
             str+="</table>";
             text->setHtml(str);
             return;
         }
     }
-    text->setHtml("<b>"+i18n("Podcast details")+"</b><p><i>"+i18n("Select a podcast to display its details")+"</i></p>");
+    text->setHtml("<b>"+tr("Podcast details")+"</b><p><i>"+tr("Select a podcast to display its details")+"</i></p>");
 }
 
 void PodcastPage::selectionChanged()
@@ -295,7 +293,7 @@ void PodcastPage::imageJobFinished()
         imageCache.insert(imageJob->property(constOrigUrlProperty).toUrl(), new QImage(img), img.byteCount());
         updateText();
     }
-    imageJob=0;
+    imageJob=nullptr;
 }
 
 void PodcastPage::jobFinished()
@@ -311,8 +309,8 @@ void PodcastPage::jobFinished()
     if (spinner) {
         spinner->stop();
     }
-    parseResonse(j->ok() ? j->actualJob() : 0);
-    job=0;
+    parseResonse(j->ok() ? j->actualJob() : nullptr);
+    job=nullptr;
 }
 
 void PodcastPage::openLink(const QUrl &url)
@@ -320,7 +318,7 @@ void PodcastPage::openLink(const QUrl &url)
     QDesktopServices::openUrl(url);
 }
 
-PodcastSearchPage::PodcastSearchPage(QWidget *p, const QString &n, const QString &i, const QUrl &qu, const QString &qk, const QStringList &other)
+PodcastSearchPage::PodcastSearchPage(QWidget *p, const QString &n, int i, const QUrl &qu, const QString &qk, const QStringList &other)
     : PodcastPage(p, n)
     , queryUrl(qu)
     , queryKey(qk)
@@ -333,8 +331,8 @@ PodcastSearchPage::PodcastSearchPage(QWidget *p, const QString &n, const QString
     viewLayout->setMargin(0);
     mainLayout->setMargin(0);
     search=new LineEdit(p);
-    search->setPlaceholderText(i18n("Enter search term..."));
-    searchButton=new QPushButton(i18n("Search"), p);
+    search->setPlaceholderText(tr("Enter search term..."));
+    searchButton=new QPushButton(tr("Search"), p);
     QWidget::setTabOrder(search, searchButton);
     QWidget::setTabOrder(searchButton, tree);
     searchLayout->addWidget(search);
@@ -345,7 +343,7 @@ PodcastSearchPage::PodcastSearchPage(QWidget *p, const QString &n, const QString
     mainLayout->addLayout(viewLayout);
     connect(search, SIGNAL(returnPressed()), SLOT(doSearch()));
     connect(searchButton, SIGNAL(clicked()), SLOT(doSearch()));
-    icn.addFile(":"+i);
+    icn=MonoIcon::icon((FontAwesome::icon)i, Utils::monoIconColor());
 }
 
 void PodcastSearchPage::showEvent(QShowEvent *e)
@@ -363,42 +361,32 @@ void PodcastSearchPage::doSearch()
 
     currentSearch=text;
     QUrl url=queryUrl;
-    #if QT_VERSION < 0x050000
-    QUrl &query=url;
-    #else
     QUrlQuery query;
-    #endif
     query.addQueryItem(queryKey, text);
     if (otherArgs.size()>1) {
         for (int i=0; i<otherArgs.size()-1; i+=2) {
             query.addQueryItem(otherArgs.at(i), otherArgs.at(i+1));
         }
     }
-    #if QT_VERSION >= 0x050000
     url.setQuery(query);
-    #endif
     fetch(url);
 }
 
 void PodcastSearchPage::parseResonse(QIODevice *dev)
 {
     if (!dev) {
-        emit error(i18n("Failed to fetch podcasts from %1", name()));
+        emit error(tr("Failed to fetch podcasts from %1").arg(name()));
         return;
     }
-    #if QT_VERSION >= 0x050000
     QVariant data=QJsonDocument::fromJson(dev->readAll()).toVariant();
-    #else
-    QVariant data = QJson::Parser().parse(dev->readAll());
-    #endif
     if (data.isNull()) {
-        emit error(i18n("There was a problem parsing the response from %1", name()));
+        emit error(tr("There was a problem parsing the response from %1").arg(name()));
         return;
     }
     parse(data);
 }
 
-OpmlBrowsePage::OpmlBrowsePage(QWidget *p, const QString &n, const QString &i, const QUrl &u)
+OpmlBrowsePage::OpmlBrowsePage(QWidget *p, const QString &n, const QIcon &i, const QUrl &u)
     : PodcastPage(p, n)
     , loaded(false)
     , url(u)
@@ -407,11 +395,11 @@ OpmlBrowsePage::OpmlBrowsePage(QWidget *p, const QString &n, const QString &i, c
     mainLayout->setMargin(0);
     mainLayout->addWidget(tree, 1);
     mainLayout->addWidget(text, 0);
-    Action *act=new Action(i18n("Reload"), this);
+    Action *act=new Action(tr("Reload"), this);
     tree->addAction(act);
-    connect(act, SIGNAL(triggered(bool)), this, SLOT(reload()));
+    connect(act, SIGNAL(triggered()), this, SLOT(reload()));
     tree->setContextMenuPolicy(Qt::ActionsContextMenu);
-    icn.addFile(i.isEmpty() || !QFile::exists(i) ? ":podcasts" : i);
+    icn=i;
 }
 
 void OpmlBrowsePage::showEvent(QShowEvent *e)
@@ -424,6 +412,7 @@ void OpmlBrowsePage::showEvent(QShowEvent *e)
                 parseResonse(&f);
                 if (tree->topLevelItemCount()>0) {
                     Utils::touchFile(cacheFile);
+                    loaded=true;
                     return;
                 }
             }
@@ -449,7 +438,7 @@ void OpmlBrowsePage::parseResonse(QIODevice *dev)
 
     if (!dev) {
         if (!isLoadingFromCache) {
-            emit error(i18n("Failed to download directory listing"));
+            emit error(tr("Failed to download directory listing"));
         }
         return;
     }
@@ -457,7 +446,7 @@ void OpmlBrowsePage::parseResonse(QIODevice *dev)
     OpmlParser::Category parsed=OpmlParser::parse(data);
     if (parsed.categories.isEmpty() && parsed.podcasts.isEmpty()) {
         if (!isLoadingFromCache) {
-            emit error(i18n("Failed to parse directory listing"));
+            emit error(tr("Failed to parse directory listing"));
         }
         return;
     }
@@ -465,11 +454,11 @@ void OpmlBrowsePage::parseResonse(QIODevice *dev)
     if (1==parsed.categories.count() && parsed.podcasts.isEmpty()) {
         parsed=parsed.categories.at(0);
     }
-    foreach (const OpmlParser::Category &cat, parsed.categories) {
-        addCategory(cat, 0);
+    for (const OpmlParser::Category &cat: parsed.categories) {
+        addCategory(cat, nullptr);
     }
-    foreach (const OpmlParser::Podcast &pod, parsed.podcasts) {
-        addPodcast(pod, 0);
+    for (const OpmlParser::Podcast &pod: parsed.podcasts) {
+        addPodcast(pod, nullptr);
     }
     if (!isLoadingFromCache && tree->topLevelItemCount()>0) {
         QString cacheFile=generateCacheFileName(url, true);
@@ -492,12 +481,12 @@ void OpmlBrowsePage::addCategory(const OpmlParser::Category &cat, QTreeWidgetIte
                                : new QTreeWidgetItem(tree, QStringList() << cat.name);
 
     catItem->setData(0, IsPodcastRole, false);
-    catItem->setIcon(0, Icons::self()->folderIcon);
-    foreach (const OpmlParser::Podcast &pod, cat.podcasts) {
+    catItem->setIcon(0, Icons::self()->folderListIcon);
+    for (const OpmlParser::Podcast &pod: cat.podcasts) {
         addPodcast(pod, catItem);
     }
-    foreach (const OpmlParser::Category &cat, cat.categories) {
-        addCategory(cat, catItem);
+    for (const OpmlParser::Category &subCat: cat.categories) {
+        addCategory(subCat, catItem);
     }
 }
 
@@ -507,7 +496,7 @@ void OpmlBrowsePage::addPodcast(const OpmlParser::Podcast &pod, QTreeWidgetItem
 }
 
 PodcastUrlPage::PodcastUrlPage(QWidget *p)
-    : PodcastPage(p, i18n("URL"))
+    : PodcastPage(p, tr("URL"))
 {
     QBoxLayout *searchLayout=new QBoxLayout(QBoxLayout::LeftToRight);
     QBoxLayout *viewLayout=new QBoxLayout(QBoxLayout::LeftToRight);
@@ -516,20 +505,20 @@ PodcastUrlPage::PodcastUrlPage(QWidget *p)
     viewLayout->setMargin(0);
     mainLayout->setMargin(0);
     urlEntry=new LineEdit(p);
-    urlEntry->setPlaceholderText(i18n("Enter podcast URL..."));
-    loadButton=new QPushButton(i18n("Load"), p);
+    urlEntry->setPlaceholderText(tr("Enter podcast URL..."));
+    loadButton=new QPushButton(tr("Load"), p);
     QWidget::setTabOrder(urlEntry, loadButton);
     QWidget::setTabOrder(loadButton, tree);
     searchLayout->addWidget(urlEntry);
     searchLayout->addWidget(loadButton);
     viewLayout->addWidget(tree, 1);
     viewLayout->addWidget(text, 0);
-    mainLayout->addWidget(new QLabel(i18n("Enter podcast URL below, and press 'Load'"), this));
+    mainLayout->addWidget(new QLabel(tr("Enter podcast URL below, and press 'Load'"), this));
     mainLayout->addLayout(searchLayout);
     mainLayout->addLayout(viewLayout);
     connect(urlEntry, SIGNAL(returnPressed()), SLOT(loadUrl()));
     connect(loadButton, SIGNAL(clicked()), SLOT(loadUrl()));
-    icn.addFile(":podcasts");
+    icn=Icons::self()->rssListIcon;
 }
 
 void PodcastUrlPage::showEvent(QShowEvent *e)
@@ -551,7 +540,7 @@ void PodcastUrlPage::loadUrl()
     }
 
     if (!PodcastService::isUrlOk(url)) {
-        emit error(i18n("Invalid URL!"));
+        emit error(tr("Invalid URL!"));
     } else {
         currentUrl=url;
         fetch(url);
@@ -561,19 +550,19 @@ void PodcastUrlPage::loadUrl()
 void PodcastUrlPage::parseResonse(QIODevice *dev)
 {
     if (!dev) {
-        emit error(i18n("Failed to fetch podcast!"));
+        emit error(tr("Failed to fetch podcast!"));
         return;
     }
     RssParser::Channel ch=RssParser::parse(dev, false, true);
     if (!ch.isValid()) {
-        emit error(i18n("Failed to parse podcast."));
+        emit error(tr("Failed to parse podcast."));
         return;
     }
     if (!ch.isValid()) {
-        emit error(i18n("Cantata only supports audio podcasts! The URL entered contains only video podcasts."));
+        emit error(tr("Cantata only supports audio podcasts! The URL entered contains only video podcasts."));
         return;
     }
-    addPodcast(ch.name, currentUrl, ch.image, ch.description, QString(), 0);
+    addPodcast(ch.name, currentUrl, ch.image, ch.description, QString(), nullptr);
 }
 
 int PodcastSearchDialog::instanceCount()
@@ -581,14 +570,15 @@ int PodcastSearchDialog::instanceCount()
     return iCount;
 }
 
-PodcastSearchDialog::PodcastSearchDialog(QWidget *parent)
+PodcastSearchDialog::PodcastSearchDialog(PodcastService *s, QWidget *parent)
     : Dialog(parent, "PodcastSearchDialog", QSize(800, 600))
+    , service(s)
 {
     Utils::clearOldCache(constCacheDir, 2);
 
     iCount++;
     setButtons(User1|Close);
-    setButtonText(User1, i18n("Subscribe"));
+    setButtonText(User1, tr("Subscribe"));
 
     QWidget *mainWidget = new QWidget(this);
     messageWidget = new MessageWidget(mainWidget);
@@ -606,27 +596,27 @@ PodcastSearchDialog::PodcastSearchDialog(QWidget *parent)
     layout->addWidget(pageWidget);
 
     PodcastUrlPage *urlPage=new PodcastUrlPage(pageWidget);
-    pageWidget->addPage(urlPage, i18n("Enter URL"), urlPage->icon(), i18n("Manual podcast URL"));
+    pageWidget->addPage(urlPage, tr("Enter URL"), urlPage->icon(), tr("Manual podcast URL"));
     pages << urlPage;
 
     ITunesSearchPage *itunes=new ITunesSearchPage(pageWidget);
-    pageWidget->addPage(itunes, i18n("Search %1", itunes->name()), itunes->icon(), i18n("Search for podcasts on %1", itunes->name()));
+    pageWidget->addPage(itunes, tr("Search %1").arg(itunes->name()), itunes->icon(), tr("Search for podcasts on %1").arg(itunes->name()));
     pages << itunes;
 
     GPodderSearchPage *gpodder=new GPodderSearchPage(pageWidget);
-    pageWidget->addPage(gpodder, i18n("Search %1", gpodder->name()), gpodder->icon(), i18n("Search for podcasts on %1", gpodder->name()));
+    pageWidget->addPage(gpodder, tr("Search %1").arg(gpodder->name()), gpodder->icon(), tr("Search for podcasts on %1").arg(gpodder->name()));
     pages << gpodder;
 
     QSet<QString> loaded;
     pages << loadDirectories(Utils::dataDir(), false, loaded);
-    pages << loadDirectories(CANTATA_SYS_CONFIG_DIR, true, loaded);
+    pages << loadDirectories(QString(), true, loaded);
 
-    foreach (PodcastPage *p, pages) {
+    for (PodcastPage *p: pages) {
         connect(p, SIGNAL(rssSelected(QUrl)), SLOT(rssSelected(QUrl)));
         connect(p, SIGNAL(error(QString)), SLOT(showError(QString)));
     }
 
-    setCaption(i18n("Add Podcast Subscription"));
+    setCaption(tr("Add Podcast Subscription"));
     setMainWidget(mainWidget);
     setAttribute(Qt::WA_DeleteOnClose);
     enableButton(User1, false);
@@ -635,7 +625,7 @@ PodcastSearchDialog::PodcastSearchDialog(QWidget *parent)
     if (-1==maxImageSize) {
         maxImageSize=fontMetrics().height()*8;
     }
-    connect(OnlineServicesModel::self(), SIGNAL(podcastError(QString)), this, SLOT(showError(QString)));
+    connect(service, SIGNAL(newError(QString)), this, SLOT(showError(QString)));
     connect(messageWidget, SIGNAL(visible(bool)), SLOT(msgWidgetVisible(bool)));
     connect(pageWidget, SIGNAL(currentPageChanged()), this, SLOT(pageChanged()));
     messageWidget->hide();
@@ -643,7 +633,6 @@ PodcastSearchDialog::PodcastSearchDialog(QWidget *parent)
 
 PodcastSearchDialog::~PodcastSearchDialog()
 {
-    disconnect(OnlineServicesModel::self(), SIGNAL(podcastError(QString)), this, SLOT(showError(QString)));
     iCount--;
     imageCache.clear();
 }
@@ -672,7 +661,7 @@ void PodcastSearchDialog::msgWidgetVisible(bool v)
 void PodcastSearchDialog::pageChanged()
 {
     PageWidgetItem *pwi=pageWidget->currentPage();
-    PodcastPage *page=pwi ? qobject_cast<PodcastPage *>(pwi->widget()) : 0;
+    PodcastPage *page=pwi ? qobject_cast<PodcastPage *>(pwi->widget()) : nullptr;
     rssSelected(page ? page->currentRss() : QUrl());
 }
 
@@ -680,11 +669,11 @@ QList<PodcastPage *> PodcastSearchDialog::loadDirectories(const QString &dir, bo
 {
     QList<PodcastPage *> pages;
 
-    if (dir.isEmpty()) {
+    if (dir.isEmpty() && !isSystem) {
         return pages;
     }
 
-    QFile file(dir+QLatin1String("/podcast_directories.xml"));
+    QFile file(isSystem ? QLatin1String(":podcast_directories.xml") : (dir+QLatin1String("/podcast_directories.xml")));
 
     if (file.open(QIODevice::ReadOnly)) {
         QXmlStreamReader reader(&file);
@@ -693,15 +682,26 @@ QList<PodcastPage *> PodcastSearchDialog::loadDirectories(const QString &dir, bo
             if (reader.isStartElement() && QLatin1String("directory")==reader.name()) {
                 QString url=reader.attributes().value(QLatin1String("url")).toString();
                 if (!loaded.contains(url)) {
-                    QString icon=reader.attributes().value(QLatin1String("icon")).toString();
-                    if (!icon.isEmpty()) {
-                        icon=dir+(isSystem ? "../icons/" : "")+icon;
+                    QString iconName=reader.attributes().value(QLatin1String("icon")).toString();
+                    QIcon icon;
+
+                    if (iconName.isEmpty()) {
+                        icon=MonoIcon::icon(FontAwesome::rsssquare, Utils::monoIconColor());
+                    } else if (iconName.startsWith(":")) {
+                        icon= MonoIcon::icon(iconName, Utils::monoIconColor());
+                    } else {
+                        icon.addFile(iconName);
                     }
+
+                    if (icon.isNull()) {
+                        icon=MonoIcon::icon(FontAwesome::rsssquare, Utils::monoIconColor());
+                    }
+
                     OpmlBrowsePage *page=new OpmlBrowsePage(pageWidget,
                                                             reader.attributes().value(QLatin1String("name")).toString(),
                                                             icon,
                                                             QUrl(url));
-                    pageWidget->addPage(page, i18n("Browse %1", page->name()), page->icon(), i18n("Browse %1 podcasts", page->name()));
+                    pageWidget->addPage(page, tr("Browse %1").arg(page->name()), page->icon(), tr("Browse %1 podcasts").arg(page->name()));
                     pages << page;
                     loaded.insert(url);
                 }
@@ -716,10 +716,11 @@ void PodcastSearchDialog::slotButtonClicked(int button)
     switch (button) {
     case User1: {
         QUrl fixed=PodcastService::fixUrl(currentUrl);
-        if (OnlineServicesModel::self()->subscribePodcast(fixed)) {
-            showInfo(i18n("Subscription added"));
+        if (service->subscribedToUrl(fixed)) {
+            showError(tr("You are already subscribed to this podcast!"));
         } else {
-            showError(i18n("You are already subscribed to this podcast!"));
+            service->addUrl(fixed);
+            showInfo(tr("Subscription added"));
         }
         break;
     }
@@ -732,3 +733,5 @@ void PodcastSearchDialog::slotButtonClicked(int button)
         break;
     }
 }
+
+#include "moc_podcastsearchdialog.cpp"
