@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,7 +25,6 @@
 #include "support/buddylabel.h"
 #include "support/pathrequester.h"
 #include "gui/settings.h"
-#include "support/localize.h"
 #include "support/utils.h"
 #include <QComboBox>
 #include <QCheckBox>
@@ -56,57 +55,70 @@ PodcastSettingsDialog::PodcastSettingsDialog(QWidget *p)
 {
     QWidget *mw=new QWidget(this);
     QFormLayout * lay=new QFormLayout(mw);
-    BuddyLabel * updateLabel=new BuddyLabel(i18n("Check for new episodes:"), mw);
-    BuddyLabel * downloadLabel=new BuddyLabel(i18n("Download episodes to:"), mw);
+    BuddyLabel * updateLabel=new BuddyLabel(tr("Check for new episodes:"), mw);
+    BuddyLabel * downloadLabel=new BuddyLabel(tr("Download episodes to:"), mw);
+    BuddyLabel * autoDownloadLabel=new BuddyLabel(tr("Download automatically:"), mw);
 
     updateCombo = new QComboBox(this);
     updateLabel->setBuddy(updateCombo);
     downloadPath = new PathRequester(this);
     downloadLabel->setBuddy(downloadPath);
     downloadPath->setDirMode(true);
-    autoDownload = new QCheckBox(i18n("Automatically download new episodes"), this);
+    autoDownloadCombo = new QComboBox(this);
+    autoDownloadLabel->setBuddy(autoDownloadCombo);
 
     int row=0;
-    lay->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
     lay->setMargin(0);
+    lay->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
     lay->setWidget(row, QFormLayout::LabelRole, updateLabel);
     lay->setWidget(row++, QFormLayout::FieldRole, updateCombo);
     lay->setWidget(row, QFormLayout::LabelRole, downloadLabel);
     lay->setWidget(row++, QFormLayout::FieldRole, downloadPath);
     lay->setWidget(row, QFormLayout::LabelRole, downloadLabel);
     lay->setWidget(row++, QFormLayout::FieldRole, downloadPath);
-    lay->setWidget(row++, QFormLayout::SpanningRole, autoDownload);
+    lay->setWidget(row, QFormLayout::LabelRole, autoDownloadLabel);
+    lay->setWidget(row++, QFormLayout::FieldRole, autoDownloadCombo);
 
     setButtons(Ok|Cancel);
     setMainWidget(mw);
-    setCaption(i18n("Podcast Settings"));
+    setCaption(tr("Podcast Settings"));
 
-    updateCombo->addItem(i18n("Manually"), 0);
-    updateCombo->addItem(i18n("Every 15 minutes"), 15);
-    updateCombo->addItem(i18n("Every 30 minutes"), 30);
-    updateCombo->addItem(i18n("Every hour"), 60);
-    updateCombo->addItem(i18n("Every 2 hours"), 2*60);
-    updateCombo->addItem(i18n("Every 6 hours"), 6*60);
-    updateCombo->addItem(i18n("Every 12 hours"), 12*60);
-    updateCombo->addItem(i18n("Every day"), 24*60);
-    updateCombo->addItem(i18n("Every week"), 7*24*60);
+    updateCombo->addItem(tr("Manually"), 0);
+    updateCombo->addItem(tr("Every 15 minutes"), 15);
+    updateCombo->addItem(tr("Every 30 minutes"), 30);
+    updateCombo->addItem(tr("Every hour"), 60);
+    updateCombo->addItem(tr("Every 2 hours"), 2*60);
+    updateCombo->addItem(tr("Every 6 hours"), 6*60);
+    updateCombo->addItem(tr("Every 12 hours"), 12*60);
+    updateCombo->addItem(tr("Every day"), 24*60);
+    updateCombo->addItem(tr("Every week"), 7*24*60);
+
+    autoDownloadCombo->addItem(tr("Don't automatically download episodes"), 0);
+    autoDownloadCombo->addItem(tr("Latest episode"), 1);
+    autoDownloadCombo->addItem(tr("Latest %1 episodes").arg(2), 2);
+    autoDownloadCombo->addItem(tr("Latest %1 episodes").arg(5), 5);
+    autoDownloadCombo->addItem(tr("Latest %1 episodes").arg(10), 10);
+    autoDownloadCombo->addItem(tr("Latest %1 episodes").arg(15), 15);
+    autoDownloadCombo->addItem(tr("Latest %1 episodes").arg(20), 20);
+    autoDownloadCombo->addItem(tr("Latest %1 episodes").arg(50), 50);
+    autoDownloadCombo->addItem(tr("All episodes"), 1000);
 
     origRssUpdate=Settings::self()->rssUpdate();
     setIndex(updateCombo, origRssUpdate);
     connect(updateCombo, SIGNAL(currentIndexChanged(int)), SLOT(checkSaveable()));
-    origPodcastDownloadPath=Utils::convertDirForDisplay(Settings::self()->podcastDownloadPath());
-    origPodcastAutoDownload=Settings::self()->podcastAutoDownload();
+    origPodcastDownloadPath=Utils::convertPathForDisplay(Settings::self()->podcastDownloadPath());
+    origPodcastAutoDownload=Settings::self()->podcastAutoDownloadLimit();
+    setIndex(autoDownloadCombo, origPodcastAutoDownload);
     downloadPath->setText(origPodcastDownloadPath);
-    autoDownload->setChecked(origPodcastAutoDownload);
     connect(downloadPath, SIGNAL(textChanged(QString)), SLOT(checkSaveable()));
-    connect(autoDownload, SIGNAL(toggled(bool)), SLOT(checkSaveable()));
+    connect(autoDownloadCombo, SIGNAL(currentIndexChanged(int)), SLOT(checkSaveable()));
     enableButton(Ok, false);
     changed=0;
 }
 
 void PodcastSettingsDialog::checkSaveable()
 {
-    enableButton(Ok, origPodcastAutoDownload!=autoDownload->isChecked() ||
+    enableButton(Ok, autoDownloadCombo->itemData(autoDownloadCombo->currentIndex()).toInt()!=origPodcastAutoDownload ||
                      updateCombo->itemData(updateCombo->currentIndex()).toInt()!=origRssUpdate ||
                      downloadPath->text().trimmed()!=origPodcastDownloadPath);
 }
@@ -121,11 +133,11 @@ void PodcastSettingsDialog::slotButtonClicked(int button)
         }
         if (downloadPath->text().trimmed()!=origPodcastDownloadPath) {
             changed|=DownloadPath;
-            Settings::self()->savePodcastDownloadPath(Utils::convertDirFromDisplay(downloadPath->text().trimmed()));
+            Settings::self()->savePodcastDownloadPath(Utils::convertPathFromDisplay(downloadPath->text().trimmed()));
         }
-        if (origPodcastAutoDownload!=autoDownload->isChecked()) {
+        if (autoDownloadCombo->itemData(autoDownloadCombo->currentIndex()).toInt()!=origPodcastAutoDownload) {
             changed|=AutoDownload;
-            Settings::self()->savePodcastAutoDownload(autoDownload->isChecked());
+            Settings::self()->savePodcastAutoDownloadLimit(autoDownloadCombo->itemData(autoDownloadCombo->currentIndex()).toInt());
         }
         accept();
     case Close:
@@ -138,3 +150,5 @@ void PodcastSettingsDialog::slotButtonClicked(int button)
         break;
     }
 }
+
+#include "moc_podcastsettingsdialog.cpp"
