@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -41,7 +41,7 @@ public:
     };
 
     PodcastSettingsDialog(QWidget *p);
-    virtual ~PodcastSettingsDialog() { }
+    ~PodcastSettingsDialog() override { }
 
     int changes() const { return changed; }
 
@@ -49,15 +49,15 @@ private Q_SLOTS:
     void checkSaveable();
 
 private:
-    void slotButtonClicked(int button);
+    void slotButtonClicked(int button) override;
 
 private:
     QComboBox *updateCombo;
     int origRssUpdate;
     PathRequester *downloadPath;
-    QCheckBox *autoDownload;
+    QComboBox *autoDownloadCombo;
     QString origPodcastDownloadPath;
-    bool origPodcastAutoDownload;
+    int origPodcastAutoDownload;
     int changed;
 };
 
