@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -54,7 +54,17 @@ static void consumeCurrentElement(QXmlStreamReader &reader)
     }
 }
 
-static QDateTime parseRfc822DateTime(const QString& text)
+static QString capitaliseWord(QString str)
+{
+    if (str.isEmpty() || !str[0].isLetter()) {
+        return str;
+    }
+    str=str.toLower();
+    str[0]=str[0].toUpper();
+    return str;
+}
+
+static QDateTime parseRfc822DateTime(const QString &text)
 {
     // This sucks but we need it because some podcasts don't quite follow the
     // spec properly - they might have 1-digit hour numbers for example.
@@ -64,7 +74,13 @@ static QDateTime parseRfc822DateTime(const QString& text)
         return QDateTime();
     }
 
-    return QDateTime(QDate::fromString(QString("%1 %2 %3 %4").arg(re.cap(1), re.cap(3), re.cap(2), re.cap(4)), Qt::TextDate),
+    QDateTime dt(QDate::fromString(QString("%1 %2 %3 %4").arg(re.cap(1), re.cap(3), re.cap(2), re.cap(4)), Qt::TextDate),
+                 QTime(re.cap(5).toInt(), re.cap(6).toInt(), re.cap(7).toInt()));
+
+    if (dt.isValid()) {
+        return dt;
+    }
+    return QDateTime(QDate::fromString(QString("%1 %2 %3 %4").arg(capitaliseWord(re.cap(1)), capitaliseWord(re.cap(3)), re.cap(2), re.cap(4)), Qt::TextDate),
                      QTime(re.cap(5).toInt(), re.cap(6).toInt(), re.cap(7).toInt()));
 }
 
@@ -89,14 +105,15 @@ static QUrl parseImage(QXmlStreamReader &reader)
 static Episode parseEpisode(QXmlStreamReader &reader)
 {
     Episode ep;
+    bool isAudio=false;
+    QUrl guidUrl;
 
     while (!reader.atEnd()) {
         reader.readNext();
         const QStringRef name = reader.name();
-
         if (reader.isStartElement()) {
             if (QLatin1String("title")==name) {
-                ep.name=reader.readElementText();
+                ep.name=reader.readElementText().simplified();
             } else if (QLatin1String("duration")==name && constITunesNameSpace==reader.namespaceUri()) {
                 QStringList parts = reader.readElementText().split(':');
                 if (2==parts.count()) {
@@ -116,13 +133,28 @@ static Episode parseEpisode(QXmlStreamReader &reader)
                 }
                 QString type=reader.attributes().value(QLatin1String("type")).toString();
                 if (type.startsWith(QLatin1String("audio/")) || audioFormats.contains(type)) {
+                    isAudio=true;
                     ep.url=QUrl::fromEncoded(reader.attributes().value(QLatin1String("url")).toString().toLatin1());
                 } else if (type.startsWith(QLatin1String("video/")) ) {
-                    ep.video=true;
+                    // At least one broken feed (BUG: 588) has the audio podcast listed as video/mp4,
+                    // ...but the path ends in .mp3 !!!
+                    QUrl url=QUrl::fromEncoded(reader.attributes().value(QLatin1String("url")).toString().toLatin1());
+                    QString path=url.path();
+                    if (path.endsWith(QLatin1String(".mp3"), Qt::CaseInsensitive) ||
+                        path.endsWith(QLatin1String(".ogg"), Qt::CaseInsensitive) ||
+                        path.endsWith(QLatin1String(".wma"), Qt::CaseInsensitive)) {
+                        ep.url=url;
+                    } else {
+                        ep.video=true;
+                    }
                 }
                 consumeCurrentElement(reader);
+            } else if (QLatin1String("guid")==name) {
+                guidUrl=QUrl(reader.readElementText());
             } else if (QLatin1String("pubDate")==name) {
                  ep.publicationDate=parseRfc822DateTime(reader.readElementText());
+            } else if (QLatin1String("description")==name) {
+                ep.description=reader.readElementText();
             } else {
                 consumeCurrentElement(reader);
             }
@@ -131,6 +163,11 @@ static Episode parseEpisode(QXmlStreamReader &reader)
         }
     }
 
+    // Sometimes the url entry in 'enclusure' is empty, but there is a url in 'guid' - so use
+    // that if present (BUG: 602)
+    if (isAudio && ep.url.isEmpty() && !guidUrl.isEmpty()) {
+        ep.url=guidUrl;
+    }
     return ep;
 }
 
@@ -145,7 +182,7 @@ Channel RssParser::parse(QIODevice *dev, bool getEpisodes, bool getDescription)
             if (reader.isStartElement()) {
                 const QStringRef name = reader.name();
                 if (ch.name.isEmpty() && QLatin1String("title")==name) {
-                    ch.name=reader.readElementText();
+                    ch.name=reader.readElementText().simplified();
                 } else if (QLatin1String("image")==name && ch.image.isEmpty()) {
                     if (constITunesNameSpace==reader.namespaceUri()) {
                         ch.image=reader.attributes().value(QLatin1String("href")).toString();
@@ -176,6 +213,5 @@ Channel RssParser::parse(QIODevice *dev, bool getEpisodes, bool getDescription)
     if (ch.video && !ch.episodes.isEmpty()) {
         ch.video=false;
     }
-
     return ch;
 }
