@@ -1,14 +1,13 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /* This class is a C++/Qt version of input_ffmpeg.c from libebur128
  */
 
 /* See LICENSE file for copyright and license details. */
-#define _POSIX_C_SOURCE 1
 #ifdef __cplusplus
 #define __STDC_CONSTANT_MACROS
 #ifdef _STDINT_H
@@ -22,7 +21,7 @@ extern "C" {
 #include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
 #if LIBAVFORMAT_VERSION_MAJOR >= 54
-#include <libavutil/audioconvert.h>
+#include <libavutil/channel_layout.h>
 #endif
 #ifdef __cplusplus
 }
@@ -41,26 +40,80 @@ static QMutex mutex;
 #define AVCODEC_MAX_AUDIO_FRAME_SIZE 192000 // 1 second of 48khz 32bit audio
 #endif
 
-#define BUFFER_SIZE ((((AVCODEC_MAX_AUDIO_FRAME_SIZE * 3) / 2) * sizeof(int16_t)) + FF_INPUT_BUFFER_PADDING_SIZE)
+#if defined FF_INPUT_BUFFER_PADDING_SIZE
+    #define FFMPEG_INPUT_BUFFER_PADDING_SIZE FF_INPUT_BUFFER_PADDING_SIZE
+#elif defined AV_INPUT_BUFFER_PADDING_SIZE
+    #define FFMPEG_INPUT_BUFFER_PADDING_SIZE AV_INPUT_BUFFER_PADDING_SIZE
+#else
+    #define FFMPEG_INPUT_BUFFER_PADDING_SIZE 32
+#endif
+
+#define BUFFER_SIZE ((((AVCODEC_MAX_AUDIO_FRAME_SIZE * 3) / 2) * sizeof(int16_t)) + FFMPEG_INPUT_BUFFER_PADDING_SIZE)
+
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 89, 100) // Not 100% of version here!
+#define GET_CODEC_TYPE(A) A->codecpar->codec_type
+#else
+#define GET_CODEC_TYPE(A) A->codec->codec_type
+#endif
+
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 24, 0)
+#define AV_FREE(PKT) av_packet_unref(PKT)
+#else
+#define AV_FREE(PKT) av_free_packet(PKT)
+#endif
 
 struct FfmpegInput::Handle {
     Handle()
         : formatContext(0)
         , codecContext(0)
         , codec(0)
+        #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 35, 0)
+        , frame(0)
+        , gotFrame(0)
+        , packetLeft(false)
+        , flushing(false)
+        #endif
         , audioStream(0)
         , currentBytes(0) {
         av_init_packet(&packet);
         audioBuffer = (int16_t*)av_malloc(BUFFER_SIZE);
+        #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 35, 0)
+        #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(55, 39, 101)
+        frame = avcodec_alloc_frame();
+        #else
+        frame = av_frame_alloc();
+        packet.data = NULL;
+        origPacket.size = 0;
+        origPacket.data=NULL;
+        #endif
+        #endif
     }
     ~Handle() {
         if (audioBuffer) {
             av_free(audioBuffer);
         }
+        #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 35, 0)
+        if (frame) {
+            #if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(54, 23, 100)
+            av_free(&frame);
+            #elif LIBAVCODEC_VERSION_INT < AV_VERSION_INT(55, 39, 101)
+            avcodec_free_frame(&frame);
+            #else
+            av_frame_free(&frame);
+            #endif
+        }
+        #endif
     }
     AVFormatContext *formatContext;
     AVCodecContext *codecContext;
     AVCodec *codec;
+    #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 35, 0)
+    AVFrame *frame;
+    int gotFrame;
+    bool packetLeft;
+    bool flushing;
+    AVPacket origPacket;
+    #endif
     AVPacket packet;
     int audioStream;
     int16_t *audioBuffer;
@@ -112,7 +165,7 @@ FfmpegInput::FfmpegInput(const QString &fileName)
     if (ok) {
         handle->audioStream = -1;
         for (size_t j = 0; j < handle->formatContext->nb_streams; ++j) {
-            if (handle->formatContext->streams[j]->codec->codec_type ==
+            if (GET_CODEC_TYPE(handle->formatContext->streams[j]) ==
                     #if LIBAVCODEC_VERSION_MAJOR >= 53 || \
                         (LIBAVCODEC_VERSION_MAJOR == 52 && LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(52, 64, 0))
                     AVMEDIA_TYPE_AUDIO
@@ -130,6 +183,13 @@ FfmpegInput::FfmpegInput(const QString &fileName)
     }
 
     if (ok) {
+        #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 89, 100) // Not 100% of version here!
+        handle->codec=avcodec_find_decoder(handle->formatContext->streams[handle->audioStream]->codecpar->codec_id);
+        handle->codecContext=avcodec_alloc_context3(handle->codec);
+        if (avcodec_parameters_to_context(handle->codecContext, handle->formatContext->streams[handle->audioStream]->codecpar)<0) {
+            ok = false;
+        }
+        #else
         // Get a pointer to the codec context for the audio stream
         handle->codecContext = handle->formatContext->streams[handle->audioStream]->codec;
 
@@ -141,21 +201,31 @@ FfmpegInput::FfmpegInput(const QString &fileName)
         #endif
         // Find the decoder for the video stream, and open codec...
         handle->codec = avcodec_find_decoder(handle->codecContext->codec_id);
+        #endif
 
-        QString floatCodec=QLatin1String(handle->codec->name)+QLatin1String("float");
-        AVCodec *possibleFloatCodec = avcodec_find_decoder_by_name(floatCodec.toLatin1().constData());
-        if (possibleFloatCodec) {
-            handle->codec = possibleFloatCodec;
-        }
+        if (ok) {
+            QString floatCodec=QLatin1String(handle->codec->name)+QLatin1String("float");
+            AVCodec *possibleFloatCodec = avcodec_find_decoder_by_name(floatCodec.toLatin1().constData());
+            if (possibleFloatCodec) {
+                handle->codec = possibleFloatCodec;
+                #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 89, 100) // Not 100% of version here!
+                avcodec_free_context(&handle->codecContext);
+                handle->codecContext=avcodec_alloc_context3(handle->codec);
+                if (avcodec_parameters_to_context(handle->codecContext, handle->formatContext->streams[handle->audioStream]->codecpar)<0) {
+                    ok = false;
+                }
+                #endif
+            }
 
-        if (!handle->codec ||
-            #if LIBAVCODEC_VERSION_MAJOR >= 53
-            avcodec_open2(handle->codecContext, handle->codec, NULL) < 0)
-            #else
-            avcodec_open(handle->codecContext, handle->codec) < 0)
-            #endif
+            if (!handle->codec ||
+                    #if LIBAVCODEC_VERSION_MAJOR >= 53
+                    avcodec_open2(handle->codecContext, handle->codec, NULL) < 0)
+                    #else
+                    avcodec_open(handle->codecContext, handle->codec) < 0)
+                    #endif
             {
-            ok=false;
+                ok=false;
+            }
         }
     }
 
@@ -303,17 +373,183 @@ size_t FfmpegInput::readFrames()
     return bufferPosition / sizeof(float) / channels();
 }
 
+#if LIBAVCODEC_VERSION_MAJOR >= 54
+static int decodePacket(FfmpegInput::Handle *handle)
+{
+    #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 89, 100) // Not 100% of version here!
+    int ret = avcodec_receive_frame(handle->codecContext, handle->frame);
+    if (0 == ret) {
+        handle->gotFrame = true;
+    }
+    if (AVERROR(EAGAIN) == ret) {
+        ret = 0;
+    }
+    if (0 == ret) {
+        ret = avcodec_send_packet(handle->codecContext, &handle->packet);
+    }
+    if (AVERROR(EAGAIN) == ret) {
+        ret = 0;
+    }
+    return ret<0 ? ret : handle->packet.size;
+    #else
+    int ret = avcodec_decode_audio4(handle->codecContext, handle->frame, &handle->gotFrame, &handle->packet);
+    if (ret < 0) {
+        return ret;
+    }
+    return FFMIN(ret, handle->packet.size);
+    #endif
+}
+
+size_t FfmpegInput::readOnePacket()
+{
+    if (!handle) {
+        return 0;
+    }
+
+start:
+    if (handle->flushing) {
+        handle->packet.data = NULL;
+        handle->packet.size = 0;
+        decodePacket(handle);
+        if (!handle->gotFrame) {
+            return 0;
+        } else {
+            goto write_to_buffer;
+        }
+    }
+
+    if (handle->packetLeft) {
+        goto packetLeft;
+    }
+
+//next_frame:
+    for (;;) {
+        if (av_read_frame(handle->formatContext, &handle->packet) < 0) {
+            handle->flushing = true;
+            goto start;
+        }
+        if (handle->packet.stream_index != handle->audioStream) {
+            AV_FREE(&handle->packet);
+            continue;
+        } else {
+            break;
+        }
+    }
+
+    int ret;
+    handle->origPacket = handle->packet;
+packetLeft:
+    ret = decodePacket(handle);
+    if (ret < 0) {
+        goto free_packet;
+    }
+    handle->packet.data += ret;
+    handle->packet.size -= ret;
+    if (handle->packet.size > 0) {
+        handle->packetLeft = true;
+    } else {
+free_packet:
+        AV_FREE(&handle->origPacket);
+        handle->packetLeft = false;
+    }
+
+    if (!handle->gotFrame) {
+        goto start;
+    }
+
+write_to_buffer: ;
+    size_t numberRead=handle->frame->nb_samples;
+    /* TODO: fix this */
+    int numChannels = handle->codecContext->channels;
+    // channels = handle->frame->channels;
+
+    if (handle->frame->nb_samples * numChannels > (int)sizeof handle->buffer) {
+        return 0;
+    }
+
+    switch (handle->codecContext->sample_fmt) {
+    case AV_SAMPLE_FMT_S16: {
+        int16_t *dataShort = (int16_t*)handle->frame->extended_data[0];
+        for (int i = 0; i < handle->frame->nb_samples * numChannels; ++i) {
+            handle->buffer[i] = ((float) dataShort[i])/qMax(-(float) SHRT_MIN, (float) SHRT_MAX);
+        }
+        break;
+    }
+    case AV_SAMPLE_FMT_S32: {
+        int32_t *dataInt = (int32_t*)handle->frame->extended_data[0];
+        for (int i = 0; i < handle->frame->nb_samples * numChannels; ++i) {
+            handle->buffer[i] = ((float) dataInt[i])/qMax(-(float) INT_MIN, (float) INT_MAX);
+        }
+        break;
+    }
+    case AV_SAMPLE_FMT_FLT: {
+        float *dataFloat = (float*)handle->frame->extended_data[0];
+        for (int i = 0; i < handle->frame->nb_samples * numChannels; ++i) {
+            handle->buffer[i] = dataFloat[i];
+        }
+        break;
+    }
+    case AV_SAMPLE_FMT_DBL: {
+        double *dataDouble = (double*)handle->frame->extended_data[0];
+        for (int i = 0; i < handle->frame->nb_samples * numChannels; ++i) {
+            handle->buffer[i] = (float)dataDouble[i];
+        }
+        break;
+    }
+    case AV_SAMPLE_FMT_S16P: {
+        uint8_t **ed = handle->frame->extended_data;
+        for (int i = 0; i < handle->frame->nb_samples * numChannels; ++i) {
+            int currentChannel = i / handle->frame->nb_samples;
+            int currentSample = i % handle->frame->nb_samples;
+            handle->buffer[currentSample * numChannels + currentChannel] = ((float)((int16_t*) ed[currentChannel])[currentSample])/qMax(-(float)SHRT_MIN, (float)SHRT_MAX);
+        }
+        break;
+    }
+    case AV_SAMPLE_FMT_S32P: {
+        uint8_t **ed = handle->frame->extended_data;
+        for (int i = 0; i < handle->frame->nb_samples * numChannels; ++i) {
+            int currentChannel = i / handle->frame->nb_samples;
+            int currentSample = i % handle->frame->nb_samples;
+            handle->buffer[currentSample * numChannels + currentChannel] = ((float) ((int32_t*) ed[currentChannel])[currentSample])/qMax(-(float)INT_MIN, (float)INT_MAX);
+        }
+        break;
+    }
+    case AV_SAMPLE_FMT_FLTP: {
+        uint8_t **ed = handle->frame->extended_data;
+        for (int i = 0; i < handle->frame->nb_samples * numChannels; ++i) {
+            int currentChannel = i / handle->frame->nb_samples;
+            int currentSample = i % handle->frame->nb_samples;
+            handle->buffer[currentSample * numChannels + currentChannel] = ((float*) ed[currentChannel])[currentSample];
+        }
+        break;
+    }
+    case AV_SAMPLE_FMT_DBLP:{
+        uint8_t **ed = handle->frame->extended_data;
+        for (int i = 0; i < handle->frame->nb_samples * numChannels; ++i) {
+            int currentChannel = i / handle->frame->nb_samples;
+            int currentSample = i % handle->frame->nb_samples;
+            handle->buffer[currentSample * numChannels + currentChannel] = ((double*) ed[currentChannel])[currentSample];
+        }
+        break;
+    }
+    case AV_SAMPLE_FMT_U8:
+    case AV_SAMPLE_FMT_NONE:
+    case AV_SAMPLE_FMT_NB:
+    default:
+        numberRead=0;
+        goto out;
+    }
+
+    out:
+    return numberRead;
+}
+#else
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(53, 35, 0)
 static int decodeAudio(FfmpegInput::Handle *handle, int *frame_size_ptr)
 {
-    #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(55, 39, 101)
-    AVFrame *frame = avcodec_alloc_frame();
-    #else
-    AVFrame *frame = av_frame_alloc();
-    #endif
     int ret, got_frame = 0;
 
-    if (!frame) {
+    if (!handle->frame) {
          return AVERROR(ENOMEM);
     }
 
@@ -324,23 +560,23 @@ static int decodeAudio(FfmpegInput::Handle *handle, int *frame_size_ptr)
     }
     #endif
 
-    ret = avcodec_decode_audio4(handle->codecContext, frame, &got_frame, &handle->packet);
+    ret = avcodec_decode_audio4(handle->codecContext, handle->frame, &got_frame, &handle->packet);
 
     if (ret >= 0 && got_frame) {
         int ch, plane_size;
         int planar = av_sample_fmt_is_planar(handle->codecContext->sample_fmt);
         int data_size = av_samples_get_buffer_size(&plane_size, handle->codecContext->channels,
-                                                   frame->nb_samples, handle->codecContext->sample_fmt, 1);
+                                                   handle->frame->nb_samples, handle->codecContext->sample_fmt, 1);
         if (*frame_size_ptr < data_size) {
             return AVERROR(EINVAL);
         }
 
-        memcpy(handle->audioBuffer, frame->extended_data[0], plane_size);
+        memcpy(handle->audioBuffer, handle->frame->extended_data[0], plane_size);
 
         if (planar && handle->codecContext->channels > 1) {
             uint8_t *out = ((uint8_t *)(handle->audioBuffer)) + plane_size;
             for (ch = 1; ch < handle->codecContext->channels; ch++) {
-                memcpy(out, frame->extended_data[ch], plane_size);
+                memcpy(out, handle->frame->extended_data[ch], plane_size);
                 out += plane_size;
             }
         }
@@ -349,13 +585,6 @@ static int decodeAudio(FfmpegInput::Handle *handle, int *frame_size_ptr)
         *frame_size_ptr = 0;
     }
 
-    #if LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(54, 23, 100)
-    av_free(&frame);
-    #elif LIBAVCODEC_VERSION_INT < AV_VERSION_INT(55, 39, 101)
-    avcodec_free_frame(&frame);
-    #else
-    av_frame_free(&frame);
-    #endif
     return ret;
 }
 #endif
@@ -374,7 +603,7 @@ size_t FfmpegInput::readOnePacket()
         if (handle->packet.stream_index == handle->audioStream) {
             break;
         }
-        av_free_packet(&handle->packet);
+        AV_FREE(&handle->packet);
     }
 
     size_t numberRead=0;
@@ -387,24 +616,17 @@ size_t FfmpegInput::readOnePacket()
     #else
     int len = avcodec_decode_audio2(handle->codecContext, handle->audioBuffer, &dataSize, handle->packet.data, handle->packet.size);
     #endif
-
     if (len < 0) {
         goto out;
     }
 
     /* No data used, (happens with metadata frames for example) */
     if (len <= 0) {
-        av_free_packet(&handle->packet);
+        AV_FREE(&handle->packet);
         goto next_frame;
     }
-
     switch (handle->codecContext->sample_fmt) {
-    #if LIBAVCODEC_VERSION_MAJOR >= 54
-    case AV_SAMPLE_FMT_S16:
-    #else
-    case SAMPLE_FMT_S16:
-    #endif
-    {
+    case SAMPLE_FMT_S16: {
         int16_t *dataShort = (int16_t*) handle->audioBuffer;
         numberRead = (size_t) dataSize / sizeof(int16_t) / (size_t) handle->codecContext->channels;
         for (unsigned int i = 0; i < (size_t) dataSize / sizeof(int16_t); ++i) {
@@ -412,12 +634,7 @@ size_t FfmpegInput::readOnePacket()
         }
         break;
     }
-    #if LIBAVCODEC_VERSION_MAJOR >= 54
-    case AV_SAMPLE_FMT_S32:
-    #else
-    case SAMPLE_FMT_S32:
-    #endif
-    {
+    case SAMPLE_FMT_S32: {
         int32_t *dataInt = (int32_t*) handle->audioBuffer;
         numberRead = (size_t) dataSize / sizeof(int32_t) / (size_t) handle->codecContext->channels;
         for (unsigned int i = 0; i < (size_t) dataSize / sizeof(int32_t); ++i) {
@@ -425,12 +642,7 @@ size_t FfmpegInput::readOnePacket()
         }
         break;
     }
-    #if LIBAVCODEC_VERSION_MAJOR >= 54
-    case AV_SAMPLE_FMT_FLT:
-    #else
-    case SAMPLE_FMT_FLT:
-    #endif
-    {
+    case SAMPLE_FMT_FLT: {
         float *dataFloat = (float*) handle->audioBuffer;
         numberRead = (size_t) dataSize / sizeof(float) / (size_t) handle->codecContext->channels;
         for (unsigned int i = 0; i < (size_t) dataSize / sizeof(float); ++i) {
@@ -438,12 +650,7 @@ size_t FfmpegInput::readOnePacket()
         }
         break;
     }
-    #if LIBAVCODEC_VERSION_MAJOR >= 54
-    case AV_SAMPLE_FMT_DBL:
-    #else
-    case SAMPLE_FMT_DBL:
-    #endif
-    {
+    case SAMPLE_FMT_DBL: {
         double *dataDouble = (double*) handle->audioBuffer;
         numberRead = (size_t) dataSize / sizeof(double) / (size_t) handle->codecContext->channels;
         for (unsigned int i = 0; i < (size_t) dataSize / sizeof(double); ++i) {
@@ -451,24 +658,19 @@ size_t FfmpegInput::readOnePacket()
         }
         break;
     }
-    #if LIBAVCODEC_VERSION_MAJOR >= 54
-    case AV_SAMPLE_FMT_U8:
-    case AV_SAMPLE_FMT_NONE:
-    case AV_SAMPLE_FMT_NB:
-    #else
     case SAMPLE_FMT_U8:
     case SAMPLE_FMT_NONE:
     case SAMPLE_FMT_NB:
-    #endif
     default:
         numberRead=0;
         goto out;
     }
 
     out:
-    av_free_packet(&handle->packet);
+    AV_FREE(&handle->packet);
     return numberRead;
 }
+#endif
 
 bool FfmpegInput::isFloatCodec() const
 {
