@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -99,12 +99,12 @@ void JobController::startJobs()
 }
 
 void JobController::cancel() {
-    foreach (Job *j, active) {
+    for (Job *j: active) {
         disconnect(j, SIGNAL(done()), this, SLOT(jobDone()));
         j->stop();
     }
     active.clear();
-    foreach (Job *j, jobs) {
+    for (Job *j: jobs) {
         j->deleteLater();
     }
     jobs.clear();
@@ -119,3 +119,5 @@ void JobController::jobDone()
     }
     startJobs();
 }
+
+#include "moc_jobcontroller.cpp"
