@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -36,7 +36,7 @@ int main(int argc, char *argv[])
 
     QStringList fileNames;
     for (int i=0; i<argc-1; ++i) {
-        fileNames.append(QFile::decodeName(argv[i+1]));
+        fileNames.append(QString::fromUtf8(argv[i+1]));
     }
 
     QCoreApplication app(argc, argv);
