@@ -1,14 +1,13 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /* This class is a C++/Qt version of input_ffmpeg.c from libebur128
  */
 
 /* See LICENSE file for copyright and license details. */
-#define _POSIX_C_SOURCE 1
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -133,7 +132,7 @@ float * Mpg123Input::buffer() const
 
 bool Mpg123Input::setChannelMap(int *st) const
 {
-    Q_UNUSED(st);
+    Q_UNUSED(st)
     return false;
 }
 
