@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -30,12 +30,12 @@
 #include "tags/tags.h"
 #include "tagreader.h"
 #include "support/utils.h"
-#include "support/localize.h"
 #include "support/messagebox.h"
+#include "support/monoicon.h"
 #include "jobcontroller.h"
 #include "widgets/basicitemdelegate.h"
 #include "support/action.h"
-#include "mpd/cuefile.h"
+#include "mpd-interface/cuefile.h"
 #include <QComboBox>
 #include <QTreeWidget>
 #include <QLabel>
@@ -66,11 +66,7 @@ int RgDialog::instanceCount()
 
 static inline void setResizeMode(QHeaderView *hdr, int idx, QHeaderView::ResizeMode mode)
 {
-    #if QT_VERSION < 0x050000
-    hdr->setResizeMode(idx, mode);
-    #else
     hdr->setSectionResizeMode(idx, mode);
-    #endif
 }
 
 RgDialog::RgDialog(QWidget *parent)
@@ -82,7 +78,7 @@ RgDialog::RgDialog(QWidget *parent)
 {
     iCount++;
     setButtons(User1|Ok|Cancel);
-    setCaption(i18n("ReplayGain"));
+    setCaption(tr("ReplayGain"));
     setAttribute(Qt::WA_DeleteOnClose);
     QWidget *mainWidget = new QWidget(this);
     QBoxLayout *layout=new QBoxLayout(QBoxLayout::TopToBottom, mainWidget);
@@ -92,25 +88,25 @@ RgDialog::RgDialog(QWidget *parent)
     statusLabel->setVisible(false);
     progress = new QProgressBar(this);
     progress->setVisible(false);
-    combo->addItem(i18n("Show All Tracks"), true);
-    combo->addItem(i18n("Show Untagged Tracks"), false);
+    combo->addItem(tr("Show All Tracks"), true);
+    combo->addItem(tr("Show Untagged Tracks"), false);
     view->setRootIsDecorated(false);
     view->setAllColumnsShowFocus(true);
     view->setItemDelegate(new BasicItemDelegate(view));
     view->setAlternatingRowColors(false);
     view->setContextMenuPolicy(Qt::ActionsContextMenu);
     view->setSelectionMode(QAbstractItemView::ExtendedSelection);
-    removeAct=new Action(i18n("Remove From List"), view);
+    removeAct=new Action(tr("Remove From List"), view);
     removeAct->setEnabled(false);
     view->addAction(removeAct);
     QTreeWidgetItem *hdr = view->headerItem();
-    hdr->setText(COL_ARTIST, i18n("Artist"));
-    hdr->setText(COL_ALBUM, i18n("Album"));
-    hdr->setText(COL_TITLE, i18n("Title"));
-    hdr->setText(COL_ALBUMGAIN, i18n("Album Gain"));
-    hdr->setText(COL_TRACKGAIN, i18n("Track Gain"));
-    hdr->setText(COL_ALBUMPEAK, i18n("Album Peak"));
-    hdr->setText(COL_TRACKPEAK, i18n("Track Peak"));
+    hdr->setText(COL_ARTIST, tr("Artist"));
+    hdr->setText(COL_ALBUM, tr("Album"));
+    hdr->setText(COL_TITLE, tr("Title"));
+    hdr->setText(COL_ALBUMGAIN, tr("Album Gain"));
+    hdr->setText(COL_TRACKGAIN, tr("Track Gain"));
+    hdr->setText(COL_ALBUMPEAK, tr("Album Peak"));
+    hdr->setText(COL_TRACKPEAK, tr("Track Peak"));
 
     QHeaderView *hv=view->header();
     setResizeMode(hv, COL_ARTIST, QHeaderView::ResizeToContents);
@@ -130,13 +126,13 @@ RgDialog::RgDialog(QWidget *parent)
     setMainWidget(mainWidget);
     setButtonGuiItem(Ok, StdGuiItem::save());
     setButtonGuiItem(Cancel, StdGuiItem::close());
-    setButtonGuiItem(User1, GuiItem(i18n("Scan"), "edit-find"));
+    setButtonGuiItem(User1, GuiItem(tr("Scan"), FontAwesome::search));
     enableButton(Ok, false);
     enableButton(User1, false);
     qRegisterMetaType<Tags::ReplayGain>("Tags::ReplayGain");
     connect(combo, SIGNAL(currentIndexChanged(int)), SLOT(toggleDisplay()));
     connect(view, SIGNAL(itemSelectionChanged()), SLOT(controlRemoveAct()));
-    connect(removeAct, SIGNAL(triggered(bool)), SLOT(removeItems()));
+    connect(removeAct, SIGNAL(triggered()), SLOT(removeItems()));
 
     italic=font();
     italic.setItalic(true);
@@ -156,7 +152,7 @@ void RgDialog::show(const QList<Song> &songs, const QString &udi, bool autoScan)
         return;
     }
 
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         if (!CueFile::isCue(s.file)) {
            origSongs.append(s);
         }
@@ -194,8 +190,18 @@ void RgDialog::show(const QList<Song> &songs, const QString &udi, bool autoScan)
     state=State_Idle;
     enableButton(User1, origSongs.count());
     view->clear();
-    foreach (const Song &s, origSongs) {
-        new QTreeWidgetItem(view, QStringList() << s.albumArtist() << s.album << s.title);
+    for (const Song &s: origSongs) {
+        if (s.isLocalFile()) {
+            base = QString();
+        }
+        QString aa = s.albumArtist();
+        const QString &al = s.album;
+        const QString &t = s.title;
+        if (aa.isEmpty() && al.isEmpty() && t.isEmpty()) {
+            new QTreeWidgetItem(view, QStringList() << "-" << "-" << Utils::getFile(s.file));
+        } else {
+            new QTreeWidgetItem(view, QStringList() << s.albumArtist() << s.album << s.title);
+        }
     }
     Dialog::show();
     startReadingTags();
@@ -209,13 +215,13 @@ void RgDialog::slotButtonClicked(int button)
 
     switch (button) {
     case Ok:
-        if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Update ReplayGain tags in tracks?"), i18n("Update Tags"),
-                                                       GuiItem(i18n("Update Tags")), StdGuiItem::cancel())) {
+        //if (MessageBox::Yes==MessageBox::questionYesNo(this, tr("Update ReplayGain tags in tracks?"), tr("Update Tags"),
+        //                                               GuiItem(tr("Update Tags")), StdGuiItem::cancel())) {
             if (saveTags()) {
                 stopScanning();
                 accept();
             }
-        }
+        //}
         break;
     case User1:
         startScanning();
@@ -223,8 +229,8 @@ void RgDialog::slotButtonClicked(int button)
     case Cancel:
         switch (state) {
         case State_ScanningFiles:
-            if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Abort scanning of tracks?"), i18n("Abort"),
-                                                           GuiItem(i18n("Abort")), StdGuiItem::no())) {
+            if (MessageBox::Yes==MessageBox::questionYesNo(this, tr("Abort scanning of tracks?"), tr("Abort"),
+                                                           GuiItem(tr("Abort")), StdGuiItem::no())) {
                 stopScanning();
                 // Need to call this - if not, when dialog is closed by window X control, it is not deleted!!!!
                 Dialog::slotButtonClicked(button);
@@ -232,8 +238,8 @@ void RgDialog::slotButtonClicked(int button)
             }
             break;
         case State_ScanningTags:
-            if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Abort reading of existing tags?"), i18n("Abort"),
-                                                           GuiItem(i18n("Abort")), StdGuiItem::no())) {
+            if (MessageBox::Yes==MessageBox::questionYesNo(this, tr("Abort reading of existing tags?"), tr("Abort"),
+                                                           GuiItem(tr("Abort")), StdGuiItem::no())) {
                 stopReadingTags();
                 // Need to call this - if not, when dialog is closed by window X control, it is not deleted!!!!
                 Dialog::slotButtonClicked(button);
@@ -258,10 +264,10 @@ void RgDialog::startScanning()
 {
     bool all=origTags.isEmpty() ||
              (origTags.count()==origSongs.count()
-                ? MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Scan <b>all</b> tracks?<br><br><i>NOTE: All tracks have existing ReplyGain tags.</i>"), QString(),
-                                                             GuiItem(i18n("Scan")), StdGuiItem::cancel())
-                : MessageBox::No==MessageBox::questionYesNo(this, i18n("Do you wish to scan all tracks, or only tracks without existing tags?"), QString(),
-                                                            GuiItem(i18n("Untagged Tracks")), GuiItem(i18n("All Tracks"))));
+                ? MessageBox::Yes==MessageBox::questionYesNo(this, tr("Scan <b>all</b> tracks?<br/><br/><i>All tracks have existing ReplayGain tags.</i>"), QString(),
+                                                             GuiItem(tr("Scan")), StdGuiItem::cancel())
+                : MessageBox::No==MessageBox::questionYesNo(this, tr("Do you wish to scan all tracks, or only tracks without existing tags?"), QString(),
+                                                            GuiItem(tr("Untagged Tracks")), GuiItem(tr("All Tracks"))));
     if (!all && origTags.count()==origSongs.count()) {
         return;
     }
@@ -271,7 +277,7 @@ void RgDialog::startScanning()
     enableButton(User1, false);
     progress->setValue(0);
     progress->setVisible(true);
-    statusLabel->setText(i18n("Scanning tracks..."));
+    statusLabel->setText(tr("Scanning tracks..."));
     statusLabel->setVisible(true);
     clearScanners();
     totalToScan=0;
@@ -279,7 +285,9 @@ void RgDialog::startScanning()
     for (int i=0; i<origSongs.count(); ++i) {
         if (!removedItems.contains(i) && (all || !origTags.contains(i))) {
             const Song &sng=origSongs.at(i);
-            groupedTracks[sng.albumArtist()+" -- "+sng.album].append(i);
+            QString aa = sng.albumArtist();
+            const QString &al = sng.album;
+            groupedTracks[aa.isEmpty() && al.isEmpty() ? (Utils::getFile(sng.file)+" -- "+QString::number(i)) : (sng.albumArtist()+" -- "+sng.album)].append(i);
         }
     }
     QMap<QString, QList<int> >::ConstIterator it(groupedTracks.constBegin());
@@ -307,8 +315,8 @@ void RgDialog::stopScanning()
 void RgDialog::createScanner(const QList<int> &indexes)
 {
     QMap<int, QString> fileMap;
-    foreach (int i, indexes) {
-        fileMap[i]=base+origSongs.at(i).filePath();
+    for (int i: indexes) {
+        fileMap[i]=origSongs.at(i).filePath(base);
     }
 
     AlbumScanner *s=new AlbumScanner(fileMap);
@@ -321,7 +329,7 @@ void RgDialog::createScanner(const QList<int> &indexes)
 void RgDialog::clearScanners()
 {
     QList<AlbumScanner *> scannerList=scanners.keys();
-    foreach (AlbumScanner *sc, scannerList) {
+    for (AlbumScanner *sc: scannerList) {
         sc->stop();
     }
     scanners.clear();
@@ -338,7 +346,7 @@ void RgDialog::startReadingTags()
     enableButton(User1, false);
     progress->setRange(0, origSongs.count());
     progress->setVisible(true);
-    statusLabel->setText(i18n("Reading existing tags..."));
+    statusLabel->setText(tr("Reading existing tags..."));
     statusLabel->setVisible(true);
     tagReader=new TagReader();
     tagReader->setDetails(origSongs, base);
@@ -383,13 +391,13 @@ bool RgDialog::saveTags()
     QMap<int, Tags::ReplayGain>::ConstIterator end=tagsToSave.constEnd();
 
     for (; it!=end; ++it) {
-        QString filePath=origSongs.at(it.key()).filePath();
-        switch (Tags::updateReplaygain(base+filePath, it.value())) {
+        QString filePath=origSongs.at(it.key()).filePath(base);
+        switch (Tags::updateReplaygain(filePath, it.value())) {
         case Tags::Update_Failed:
             failed.append(filePath);
             break;
         case Tags::Update_BadFile:
-            failed.append(i18nc("filename (Corrupt tags?)", "%1 (Corrupt tags?)", filePath));
+            failed.append(tr("%1 (Corrupt tags?)", "filename (Corrupt tags?)").arg(filePath));
             break;
         default:
             break;
@@ -402,7 +410,7 @@ bool RgDialog::saveTags()
     }
 
     if (failed.count()) {
-        MessageBox::errorListEx(this, i18n("Failed to update the tags of the following tracks:"), failed);
+        MessageBox::errorListEx(this, tr("Failed to update the tags of the following tracks:"), failed);
     }
 
     return !someTimedout;
@@ -438,17 +446,17 @@ Device * RgDialog::getDevice(const QString &udi, QWidget *p)
 {
     Device *dev=DevicesModel::self()->device(udi);
     if (!dev) {
-        MessageBox::error(p ? p : this, i18n("Device has been removed!"));
+        MessageBox::error(p ? p : this, tr("Device has been removed!"));
         reject();
         return 0;
     }
     if (!dev->isConnected()) {
-        MessageBox::error(p ? p : this, i18n("Device is not connected."));
+        MessageBox::error(p ? p : this, tr("Device is not connected."));
         reject();
         return 0;
     }
     if (!dev->isIdle()) {
-        MessageBox::error(p ? p : this, i18n("Device is busy?"));
+        MessageBox::error(p ? p : this, tr("Device is busy?"));
         reject();
         return 0;
     }
@@ -483,18 +491,22 @@ void RgDialog::scannerDone()
             Tags::ReplayGain updatedTags(it.value().gain, s->albumValues().gain, it.value().peak, s->albumValues().peak);
             QTreeWidgetItem *item=view->topLevelItem(it.key());
             if (it.value().ok) {
-                item->setText(COL_TRACKGAIN, i18n("%1 dB", Utils::formatNumber(updatedTags.trackGain, 2)));
-                item->setText(COL_TRACKPEAK, Utils::formatNumber(updatedTags.trackPeak, 6));
+                item->setText(COL_TRACKGAIN, tr("%1 dB").arg(Utils::formatNumber(updatedTags.trackGain, 2)));
+                if (!Utils::equal(updatedTags.trackPeak, 0.0)) {
+                    item->setText(COL_TRACKPEAK, Utils::formatNumber(updatedTags.trackPeak, 6));
+                }
             } else {
-                item->setText(COL_TRACKGAIN, i18n("Failed"));
-                item->setText(COL_TRACKPEAK, i18n("Failed"));
+                item->setText(COL_TRACKGAIN, tr("Failed"));
+                item->setText(COL_TRACKPEAK, tr("Failed"));
             }
             if (s->albumValues().ok) {
-                item->setText(COL_ALBUMGAIN, i18n("%1 dB", Utils::formatNumber(updatedTags.albumGain, 2)));
-                item->setText(COL_ALBUMPEAK, Utils::formatNumber(updatedTags.albumPeak, 6));
+                item->setText(COL_ALBUMGAIN, tr("%1 dB").arg(Utils::formatNumber(updatedTags.albumGain, 2)));
+                if (!Utils::equal(updatedTags.albumPeak, 0.0)) {
+                    item->setText(COL_ALBUMPEAK, Utils::formatNumber(updatedTags.albumPeak, 6));
+                }
             } else {
-                item->setText(COL_ALBUMGAIN, i18n("Failed"));
-                item->setText(COL_ALBUMPEAK, i18n("Failed"));
+                item->setText(COL_ALBUMGAIN, tr("Failed"));
+                item->setText(COL_ALBUMPEAK, tr("Failed"));
             }
 
             if (it.value().ok && origTags.contains(it.key())) {
@@ -503,22 +515,22 @@ void RgDialog::scannerDone()
                 bool diffAlbum=false;
                 if (!Utils::equal(t.trackGain, updatedTags.trackGain, 0.01)) {
                     item->setFont(COL_TRACKGAIN, italic);
-                    item->setToolTip(COL_TRACKGAIN, i18n("Original: %1 dB", Utils::formatNumber(t.trackGain, 2)));
+                    item->setToolTip(COL_TRACKGAIN, tr("Original: %1 dB").arg(Utils::formatNumber(t.trackGain, 2)));
                     diff=true;
                 }
-                if (!Utils::equal(t.trackPeak, updatedTags.trackPeak, 0.000001)) {
+                if (!Utils::equal(updatedTags.trackPeak, 0.0) && !Utils::equal(t.trackPeak, updatedTags.trackPeak, 0.000001)) {
                     item->setFont(COL_TRACKPEAK, italic);
-                    item->setToolTip(COL_TRACKPEAK, i18n("Original: %1", Utils::formatNumber(t.trackPeak, 6)));
+                    item->setToolTip(COL_TRACKPEAK, tr("Original: %1").arg(Utils::formatNumber(t.trackPeak, 6)));
                     diff=true;
                 }
                 if (!Utils::equal(t.albumGain, updatedTags.albumGain, 0.01)) {
                     item->setFont(COL_ALBUMGAIN, italic);
-                    item->setToolTip(COL_ALBUMGAIN, i18n("Original: %1 dB", Utils::formatNumber(t.albumGain, 2)));
+                    item->setToolTip(COL_ALBUMGAIN, tr("Original: %1 dB").arg(Utils::formatNumber(t.albumGain, 2)));
                     diffAlbum=true;
                 }
-                if (!Utils::equal(t.albumPeak, updatedTags.albumPeak, 0.000001)) {
+                if (!Utils::equal(updatedTags.albumPeak, 0.0) && !Utils::equal(t.albumPeak, updatedTags.albumPeak, 0.000001)) {
                     item->setFont(COL_ALBUMPEAK, italic);
-                    item->setToolTip(COL_ALBUMPEAK, i18n("Original: %1", Utils::formatNumber(t.albumPeak, 6)));
+                    item->setToolTip(COL_ALBUMPEAK, tr("Original: %1").arg(Utils::formatNumber(t.albumPeak, 6)));
                     diffAlbum=true;
                 }
                 if (diff || diffAlbum) {
@@ -549,10 +561,10 @@ void RgDialog::scannerDone()
     } else {
         for(; it!=end; ++it) {
             QTreeWidgetItem *item=view->topLevelItem(it.key());
-            item->setText(COL_TRACKGAIN, i18n("Failed"));
-            item->setText(COL_TRACKPEAK, i18n("Failed"));
-            item->setText(COL_ALBUMGAIN, i18n("Failed"));
-            item->setText(COL_ALBUMPEAK, i18n("Failed"));
+            item->setText(COL_TRACKGAIN, tr("Failed"));
+            item->setText(COL_TRACKPEAK, tr("Failed"));
+            item->setText(COL_ALBUMGAIN, tr("Failed"));
+            item->setText(COL_ALBUMPEAK, tr("Failed"));
             tagsToSave.remove(it.key());
         }
     }
@@ -573,10 +585,14 @@ void RgDialog::songTags(int index, Tags::ReplayGain tags)
             if (!item) {
                 return;
             }
-            item->setText(COL_TRACKGAIN, i18n("%1 dB", Utils::formatNumber(tags.trackGain, 2)));
-            item->setText(COL_TRACKPEAK, Utils::formatNumber(tags.trackPeak, 6));
-            item->setText(COL_ALBUMGAIN, i18n("%1 dB", Utils::formatNumber(tags.albumGain, 2)));
-            item->setText(COL_ALBUMPEAK, Utils::formatNumber(tags.albumPeak, 6));
+            item->setText(COL_TRACKGAIN, tr("%1 dB").arg(Utils::formatNumber(tags.trackGain, 2)));
+            if (!Utils::equal(tags.trackPeak, 0.0)) {
+                item->setText(COL_TRACKPEAK, Utils::formatNumber(tags.trackPeak, 6));
+            }
+            item->setText(COL_ALBUMGAIN, tr("%1 dB").arg(Utils::formatNumber(tags.albumGain, 2)));
+            if (!Utils::equal(tags.albumPeak, 0.0)) {
+                item->setText(COL_ALBUMPEAK, Utils::formatNumber(tags.albumPeak, 6));
+            }
         }
     }
 }
@@ -625,10 +641,10 @@ void RgDialog::removeItems()
         return;
     }
 
-    if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Remove the selected tracks from the list?"),
-                                                   i18n("Remove Tracks"), StdGuiItem::remove(), StdGuiItem::cancel())) {
+    if (MessageBox::Yes==MessageBox::questionYesNo(this, tr("Remove the selected tracks from the list?"),
+                                                   tr("Remove Tracks"), StdGuiItem::remove(), StdGuiItem::cancel())) {
         QList<QTreeWidgetItem *> selection=view->selectedItems();
-        foreach (QTreeWidgetItem *item, selection) {
+        for (QTreeWidgetItem *item: selection) {
             int index=view->indexOfTopLevelItem(item);
             view->setItemHidden(item, true);
             removedItems.insert(index);
@@ -650,3 +666,5 @@ void RgDialog::closeEvent(QCloseEvent *event)
         Dialog::closeEvent(event);
     }
 }
+
+#include "moc_rgdialog.cpp"
