@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,7 +22,6 @@
  */
 
 #include "tagreader.h"
-#include "tags.h"
 
 void TagReader::setDetails(const QList<Song> &s, const QString &dir)
 {
@@ -42,3 +41,5 @@ void TagReader::run()
     }
     setFinished(true);
 }
+
+#include "moc_tagreader.cpp"
