@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,8 +25,8 @@
 #define _TAGREADER_H_
 
 #include "jobcontroller.h"
-#include "song.h"
-#include "tags.h"
+#include "mpd-interface/song.h"
+#include "tags/tags.h"
 
 class TagReader : public StandardJob
 {
