@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * This file contains code from QMPDClient:
  * Copyright (C) 2005-2008 Håvard Tautra Knutsen <havtknut@tihlde.org>
@@ -28,10 +28,9 @@
 #include "scrobbler.h"
 #include "pausabletimer.h"
 #include "config.h"
-#include "gui/covers.h"
+#include "gui/apikeys.h"
 #include "network/networkaccessmanager.h"
-#include "mpd/mpdconnection.h"
-#include "support/localize.h"
+#include "mpd-interface/mpdconnection.h"
 #include "support/globalstatic.h"
 #include "support/utils.h"
 #include "support/configuration.h"
@@ -89,20 +88,20 @@ enum LastFmErrors
 static QString errorString(int code, const QString &msg)
 {
     switch (code) {
-    case InvalidService: return i18n("Invalid service");
-    case InvalidMethod: return i18n("Invalid method");
-    case AuthenticationFailed: return i18n("Authentication failed");
-    case InvalidFormat: return i18n("Invalid format");
-    case InvalidParameters: return i18n("Invalid parameters");
-    case InvalidResourceSpecified: return i18n("Invalid resource specified");
-    case OperationFailed: return i18n("Operation failed");
-    case InvalidSessionKey: return i18n("Invalid session key");
-    case InvalidApiKey: return i18n("Invalid API key");
-    case ServiceOffline: return i18n("Service offline");
-    case TryAgainLater: return i18n("Last.fm is currently busy, please try again in a few minutes");
-    case RateLimitExceeded: return i18n("Rate-limit exceeded");
+    case InvalidService: return QObject::tr("Invalid service");
+    case InvalidMethod: return QObject::tr("Invalid method");
+    case AuthenticationFailed: return QObject::tr("Authentication failed");
+    case InvalidFormat: return QObject::tr("Invalid format");
+    case InvalidParameters: return QObject::tr("Invalid parameters");
+    case InvalidResourceSpecified: return QObject::tr("Invalid resource specified");
+    case OperationFailed: return QObject::tr("Operation failed");
+    case InvalidSessionKey: return QObject::tr("Invalid session key");
+    case InvalidApiKey: return QObject::tr("Invalid API key");
+    case ServiceOffline: return QObject::tr("Service offline");
+    case TryAgainLater: return QObject::tr("Last.fm is currently busy, please try again in a few minutes");
+    case RateLimitExceeded: return QObject::tr("Rate-limit exceeded");
     default:
-        return msg.isEmpty() ? i18n("Unknown error") : msg.trimmed();
+        return msg.isEmpty() ? QObject::tr("Unknown error") : msg.trimmed();
     }
 }
 
@@ -114,10 +113,10 @@ static QString md5(const QString &s)
 static void sign(QMap<QString, QString> &params)
 {
     QString s;
-    params[QLatin1String("api_key")] = Covers::constLastFmApiKey;
+    params[QLatin1String("api_key")] = ApiKeys::self()->get(ApiKeys::LastFm);
     QStringList keys=params.keys();
     keys.sort();
-    foreach (const QString &k, keys) {
+    for (const QString &k: keys) {
         s += k+params[k];
     }
     s += constSecretKey;
@@ -147,9 +146,15 @@ static QString cacheName(bool createDir)
 
 Scrobbler::Track::Track(const Song &s)
 {
-    title=s.title;
-    artist=s.artist;
-    album=s.album;
+    if (!(s.blank&Song::BlankTitle)) {
+        title=s.title;
+    }
+    if (!(s.blank&Song::BlankArtist)) {
+        artist=s.artist;
+    }
+    if (!(s.blank&Song::BlankAlbum)) {
+        album=s.album;
+    }
     albumartist=s.albumartist;
     track=s.track;
     length=s.time;
@@ -157,7 +162,7 @@ Scrobbler::Track::Track(const Song &s)
 }
 
 Scrobbler::Scrobbler()
-    : QObject(0)
+    : QObject(nullptr)
     , scrobblingEnabled(false)
     , loveIsEnabled(false)
     , scrobbler("Last.fm")
@@ -171,8 +176,8 @@ Scrobbler::Scrobbler()
     , scrobbleViaMpd(false)
     , failedCount(0)
     , lastState(MPDState_Inactive)
-    , authJob(0)
-    , scrobbleJob(0)
+    , authJob(nullptr)
+    , scrobbleJob(nullptr)
 {
     hardFailTimer = new QTimer(this);
     hardFailTimer->setInterval(60*1000);
@@ -300,12 +305,17 @@ void Scrobbler::setDetails(const QString &s, const QString &u, const QString &p)
 
 void Scrobbler::love()
 {
+    if (lovedTrack()) {
+        return;
+    }
+
     lovePending=false;
     if (!loveIsEnabled) {
         return;
     }
 
-    if (currentSong.title.isEmpty() || currentSong.artist.isEmpty() || loveSent) {
+    const Track &song=isEnabled() ? currentSong : inactiveSong;
+    if (song.title.isEmpty() || song.artist.isEmpty() || loveSent) {
         return;
     }
 
@@ -322,13 +332,15 @@ void Scrobbler::love()
 
     QMap<QString, QString> params;
     params["method"] = "track.love";
-    params["track"] = currentSong.title;
-    params["artist"] = currentSong.artist;
+    params["track"] = song.title;
+    params["artist"] = song.artist;
     params["sk"] = sessionKey;
     sign(params);
-    DBUG << currentSong.title << currentSong.artist;
+    DBUG << song.title << song.artist;
     loveSent=true;
-    if (!fakeScrobbling) {
+    if (fakeScrobbling) {
+        DBUG << "MSG" << params;
+    } else {
         QNetworkReply *job=NetworkAccessManager::self()->postFormData(QUrl(scrobblerUrl()), format(params));
         connect(job, SIGNAL(finished()), this, SLOT(handleResp()));
     }
@@ -379,16 +391,19 @@ void Scrobbler::calcScrobbleIntervals()
 
 void Scrobbler::setSong(const Song &s)
 {
-    DBUG << isEnabled() << s.isStandardStream() << s.time << s.file << s.title << s.artist << s.album << s.albumartist;
+    DBUG << isEnabled() << s.isStandardStream() << s.time << s.file << s.title << s.artist << s.album << s.albumartist << s.blank;
     if (!scrobbleViaMpd && !isEnabled()) {
-        inactiveSong=s;
+        if (inactiveSong.artist != s.artist || inactiveSong.title!=s.title || inactiveSong.album!=s.album) {
+            emit songChanged(!s.isStandardStream() && !s.isEmpty());
+        }
+        inactiveSong=Track(s);
         return;
     }
 
     inactiveSong.clear();
     if (currentSong.artist != s.artist || currentSong.title!=s.title || currentSong.album!=s.album) {
         nowPlayingSent=scrobbledCurrent=loveSent=lovePending=nowPlayingIsPending=false;
-        currentSong=s;
+        currentSong=Track(s);
         lastNowPlaying=0;
         emit songChanged(!s.isStandardStream() && !s.isEmpty());
         if (scrobbleViaMpd || !isEnabled() || s.isStandardStream() || s.time<30) {
@@ -432,8 +447,10 @@ void Scrobbler::scrobbleNowPlaying()
     sign(params);
     DBUG << currentSong.title << currentSong.artist << currentSong.albumartist << currentSong.album << currentSong.track << currentSong.length;
     nowPlayingSent=true;
-    lastNowPlaying=time(NULL);
-    if (!fakeScrobbling) {
+    lastNowPlaying=time(nullptr);
+    if (fakeScrobbling) {
+        DBUG << "MSG" << params;
+    } else {
         QNetworkReply *job=NetworkAccessManager::self()->postFormData(QUrl(scrobblerUrl()), format(params));
         connect(job, SIGNAL(finished()), this, SLOT(handleResp()));
     }
@@ -500,6 +517,7 @@ void Scrobbler::scrobbleQueued()
         params["sk"] = sessionKey;
         sign(params);
         if (fakeScrobbling) {
+            DBUG << "MSG" << params;
             lastScrobbledSongs.clear();
         } else {
             scrobbleJob=NetworkAccessManager::self()->postFormData(scrobblerUrl(), format(params));
@@ -519,7 +537,7 @@ void Scrobbler::scrobbleFinished()
     if (job==scrobbleJob) {
         QByteArray data=job->readAll();
         DBUG << job->errorString() << data << songQueue.size() << lastScrobbledSongs.size();
-        scrobbleJob=0;
+        scrobbleJob=nullptr;
 
         int errorCode=NoError;
         QXmlStreamReader reader(data);
@@ -536,7 +554,7 @@ void Scrobbler::scrobbleFinished()
                                 if (QLatin1String("error")==reader.name().toString()) {
                                     errorCode=reader.attributes().value(QLatin1String("code")).toString().toInt();
                                     QString errorStr=errorString(errorCode, reader.readElementText());
-                                    emit error(i18n("%1 error: %2", scrobbler, errorStr));
+                                    emit error(tr("%1 error: %2").arg(scrobbler).arg(errorStr));
                                     DBUG << errorStr;
                                     break;
                                 }
@@ -603,7 +621,12 @@ void Scrobbler::authenticate()
     QMap<QString, QString> params;
     params["method"] = "auth.getMobileSession";
     params["username"] = userName;
-    if (QSslSocket::supportsSsl()) {
+
+    bool supportsSsl = false;
+    #ifndef QT_NO_SSL
+    supportsSsl = QSslSocket::supportsSsl();
+    #endif
+    if (supportsSsl) {
         params["password"] = password;
         url.setScheme("https"); // Use HTTPS to authenticate
     } else {
@@ -627,7 +650,7 @@ void Scrobbler::authResp()
     if (job!=authJob) {
         return;
     }
-    authJob=0;
+    authJob=nullptr;
     sessionKey.clear();
 
     QByteArray data=job->readAll();
@@ -651,7 +674,7 @@ void Scrobbler::authResp()
                 break;
             } else if (QLatin1String("error")==element) {
                 int code=reader.attributes().value(QLatin1String("code")).toString().toInt();
-                emit error(i18n("%1 error: %2", scrobbler, errorString(code, reader.readElementText())));
+                emit error(tr("%1 error: %2").arg(scrobbler).arg(errorString(code, reader.readElementText())));
                 break;
             }
         }
@@ -727,7 +750,7 @@ void Scrobbler::saveCache()
         writer.writeAttribute("version", "1");
 
         const QQueue<Track> &queue=lastScrobbledSongs.isEmpty() ? songQueue : lastScrobbledSongs;
-        foreach (const Track &t, queue) {
+        for (const Track &t: queue) {
             writer.writeEmptyElement("track");
             writer.writeAttribute(QLatin1String("artist"), t.artist);
             writer.writeAttribute(QLatin1String("album"), t.album);
@@ -759,7 +782,7 @@ void Scrobbler::mpdStateUpdated(bool songChanged)
             nowPlayingTimer->pause();
             break;
         case MPDState_Playing: {
-            time_t now=time(NULL);
+            time_t now=time(nullptr);
             currentSong.timestamp = now-MPDStatus::self()->timeElapsed();
             DBUG << "Timestamp:" << currentSong.timestamp << "scrobbledCurrent:" << scrobbledCurrent << "nowPlayingSent:" << nowPlayingSent
                  << "now:" << now << "lastNowPlaying:" << lastNowPlaying << "isRepeat:" << isRepeat;
@@ -822,14 +845,14 @@ void Scrobbler::cancelJobs()
         authJob->close();
         authJob->abort();
         authJob->deleteLater();
-        authJob=0;
+        authJob=nullptr;
     }
     if (scrobbleJob) {
         disconnect(scrobbleJob, SIGNAL(finished()), this, SLOT(scrobbleFinished()));
         authJob->close();
         authJob->abort();
         authJob->deleteLater();
-        scrobbleJob=0;
+        scrobbleJob=nullptr;
     }
 }
 
@@ -844,15 +867,19 @@ void Scrobbler::reset()
 void Scrobbler::loadScrobblers()
 {
     if (scrobblers.isEmpty()) {
-        QStringList dirs=QStringList() << Utils::dataDir() << CANTATA_SYS_CONFIG_DIR;
-        foreach (const QString &dir, dirs) {
-            if (dir.isEmpty()) {
-                continue;
-            }
+        QStringList files;
+        QString userDir=Utils::dataDir();
 
-            QFile f(dir+"scrobblers.xml");
-            if (f.open(QIODevice::ReadOnly)) {
-                QXmlStreamReader doc(&f);
+        if (!userDir.isEmpty()) {
+            files.append(Utils::fixPath(userDir)+QLatin1String("scrobblers.xml"));
+        }
+
+        files.append(":scrobblers.xml");
+
+        for (const auto &f: files) {
+            QFile file(f);
+            if (file.open(QIODevice::ReadOnly)) {
+                QXmlStreamReader doc(&file);
                 while (!doc.atEnd()) {
                     doc.readNext();
                     if (doc.isStartElement() && QLatin1String("scrobbler")==doc.name()) {
@@ -880,3 +907,5 @@ QString Scrobbler::scrobblerUrl()
     }
     return scrobblers[scrobbler];
 }
+
+#include "moc_scrobbler.cpp"
