@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,8 +25,8 @@
 #include "models/streamsmodel.h"
 #include "streamproviderlistdialog.h"
 #include "widgets/basicitemdelegate.h"
+#include "widgets/icons.h"
 #include "support/icon.h"
-#include "support/localize.h"
 #include "tar.h"
 #include "support/messagebox.h"
 #include "support/utils.h"
@@ -34,11 +34,8 @@
 #include <QListWidget>
 #include <QMenu>
 #include <QFileInfo>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KFileDialog>
-#else
 #include <QFileDialog>
-#endif
+#include <QTimer>
 
 enum Roles {
     KeyRole = Qt::UserRole,
@@ -51,7 +48,7 @@ static bool removeDir(const QString &d, const QStringList &types)
     QDir dir(d);
     if (dir.exists()) {
         QFileInfoList files=dir.entryInfoList(types, QDir::Files|QDir::NoDotAndDotDot);
-        foreach (const QFileInfo &file, files) {
+        for (const QFileInfo &file: files) {
             if (!QFile::remove(file.absoluteFilePath())) {
                 return false;
             }
@@ -62,25 +59,31 @@ static bool removeDir(const QString &d, const QStringList &types)
 }
 
 StreamsSettings::StreamsSettings(QWidget *p)
-    : QWidget(p)
-    , providerDialog(0)
+    : Dialog(p, "StreamsDialog", QSize(400, 500))
+    , providerDialog(nullptr)
 {
-    setupUi(this);
+    setCaption(tr("Configure Streams"));
+    QWidget *mw=new QWidget(this);
+    setupUi(mw);
+    setMainWidget(mw);
     categories->setItemDelegate(new BasicItemDelegate(categories));
     categories->setSortingEnabled(true);
     int iSize=Icon::stdSize(QApplication::fontMetrics().height()*1.25);
     QMenu *installMenu=new QMenu(this);
-    QAction *installFromFileAct=installMenu->addAction(i18n("From File..."));
-    QAction *installFromWebAct=installMenu->addAction(i18n("Download..."));
-    installButton->setMenu(installMenu);
+    QAction *installFromFileAct=installMenu->addAction(tr("From File..."));
+    QAction *installFromWebAct=installMenu->addAction(tr("Download..."));
     categories->setIconSize(QSize(iSize, iSize));
     connect(categories, SIGNAL(currentRowChanged(int)), SLOT(currentCategoryChanged(int)));
     connect(installFromFileAct, SIGNAL(triggered()), this, SLOT(installFromFile()));
     connect(installFromWebAct, SIGNAL(triggered()), this, SLOT(installFromWeb()));
-    connect(removeButton, SIGNAL(clicked()), this, SLOT(remove()));
-    connect(configureButton, SIGNAL(clicked()), this, SLOT(configure()));
-    removeButton->setEnabled(false);
-    configureButton->setEnabled(false);
+
+    setButtons(Close|User1|User2|User3);
+    setButtonGuiItem(User1, GuiItem(tr("Configure Provider")));
+    setButtonGuiItem(User2, GuiItem(tr("Install")));
+    setButtonGuiItem(User3, GuiItem(tr("Remove")));
+    setButtonMenu(User2, installMenu, InstantPopup);
+    enableButton(User3, false);
+    enableButton(User1, false);
 }
 
 void StreamsSettings::load()
@@ -88,7 +91,8 @@ void StreamsSettings::load()
     QList<StreamsModel::Category> cats=StreamsModel::self()->getCategories();
     QFont f(font());
     f.setItalic(true);
-    foreach (const StreamsModel::Category &cat, cats) {
+    categories->clear();
+    for (const StreamsModel::Category &cat: cats) {
         QListWidgetItem *item=new QListWidgetItem(cat.name, categories);
         item->setCheckState(cat.hidden ? Qt::Unchecked : Qt::Checked);
         item->setData(KeyRole, cat.key);
@@ -123,17 +127,13 @@ void StreamsSettings::currentCategoryChanged(int row)
         enableRemove=!item->data(BuiltInRole).toBool();
         enableConfigure=item->data(ConfigurableRole).toBool();
     }
-    removeButton->setEnabled(enableRemove);
-    configureButton->setEnabled(enableConfigure);
+    enableButton(User3, enableRemove);
+    enableButton(User1, enableConfigure);
 }
 
 void StreamsSettings::installFromFile()
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    QString fileName=KFileDialog::getOpenFileName(KUrl(), i18n("*.streams|Cantata Streams"), this, i18n("Install Streams"));
-    #else
-    QString fileName=QFileDialog::getOpenFileName(this, i18n("Install Streams"), QDir::homePath(), i18n("Cantata Streams (*.streams)"));
-    #endif
+    QString fileName=QFileDialog::getOpenFileName(this, tr("Install Streams"), QDir::homePath(), tr("Cantata Streams (*.streams)"));
     if (fileName.isEmpty()) {
         return;
     }
@@ -147,7 +147,7 @@ void StreamsSettings::installFromFile()
     name=name.replace("\\", "_");
     #endif
 
-    if (get(name) && MessageBox::No==MessageBox::warningYesNo(this, i18n("A category named <b>%1</b> already exists!<br/>Overwrite?", name))) {
+    if (get(name) && MessageBox::No==MessageBox::warningYesNo(this, tr("A category named '%1' already exists!\n\nOverwrite?").arg(name))) {
         return;
     }
     install(fileName, name);
@@ -166,6 +166,11 @@ void StreamsSettings::installFromWeb()
     }
 
     providerDialog->show(installed);
+    #ifdef Q_OS_MAC
+    // Under OSX when stream providers are installed/updated, and the dialog closed, it
+    // puts the pref dialog below the main window! This hack fixes this...
+    QTimer::singleShot(0, this, SLOT(raiseWindow()));
+    #endif
 }
 
 bool StreamsSettings::install(const QString &fileName, const QString &name, bool showErrors)
@@ -173,7 +178,7 @@ bool StreamsSettings::install(const QString &fileName, const QString &name, bool
     Tar tar(fileName);
     if (!tar.open()) {
         if (showErrors) {
-            MessageBox::error(this, i18n("Failed top open package file"));
+            MessageBox::error(this, tr("Failed to open package file."));
         }
         return false;
     }
@@ -193,7 +198,7 @@ bool StreamsSettings::install(const QString &fileName, const QString &name, bool
 
     if (streamFile.isEmpty()) {
         if (showErrors) {
-            MessageBox::error(this, i18n("Invalid file format!"));
+            MessageBox::error(this, tr("Invalid file format!"));
         }
         return false;
     }
@@ -202,7 +207,7 @@ bool StreamsSettings::install(const QString &fileName, const QString &name, bool
     QString dir=streamsDir+name;
     if (!QDir(dir).exists() && !QDir(dir).mkpath(dir)) {
         if (showErrors) {
-            MessageBox::error(this, i18n("Failed to create stream category folder!"));
+            MessageBox::error(this, tr("Failed to create stream category folder!"));
         }
         return false;
     }
@@ -210,7 +215,7 @@ bool StreamsSettings::install(const QString &fileName, const QString &name, bool
     QFile streamsFile(dir+Utils::constDirSep+streamsName);
     if (!streamsFile.open(QIODevice::WriteOnly)) {
         if (showErrors) {
-            MessageBox::error(this, i18n("Failed to save stream list!"));
+            MessageBox::error(this, tr("Failed to save stream list!"));
         }
         return false;
     }
@@ -242,7 +247,7 @@ bool StreamsSettings::install(const QString &fileName, const QString &name, bool
     StreamsModel::CategoryItem *cat=StreamsModel::self()->addInstalledProvider(name, icn, dir+Utils::constDirSep+streamsName, true);
     if (!cat) {
         if (showErrors) {
-            MessageBox::error(this, i18n("Invalid file format!"));
+            MessageBox::error(this, tr("Invalid file format!"));
         }
         return false;
     }
@@ -255,7 +260,7 @@ bool StreamsSettings::install(const QString &fileName, const QString &name, bool
     item->setCheckState(Qt::Checked);
     item->setData(KeyRole, cat->configName);
     item->setData(BuiltInRole, false);
-    item->setData(ConfigurableRole, cat->canConfigure());
+    item->setData(ConfigurableRole, cat->isDi());
     item->setIcon(icn);
     return true;
 }
@@ -268,14 +273,14 @@ void StreamsSettings::remove()
     }
 
     QListWidgetItem *item=categories->item(row);
-    if (!item->data(BuiltInRole).toBool() && MessageBox::No==MessageBox::warningYesNo(this, i18n("Are you sure you wish to remove <b>%1</b>?", item->text()))) {
+    if (!item->data(BuiltInRole).toBool() && MessageBox::No==MessageBox::warningYesNo(this, tr("Are you sure you wish to remove '%1'?").arg(item->text()))) {
         return;
     }
 
     QString dir=Utils::dataDir(StreamsModel::constSubDir);
     if (!dir.isEmpty() && !removeDir(dir+item->text(), QStringList() << StreamsModel::constXmlFile << StreamsModel::constCompressedXmlFile
                                                                      << StreamsModel::constSettingsFile << "*.png" << "*.svg")) {
-        MessageBox::error(this, i18n("Failed to remove streams folder!"));
+        MessageBox::error(this, tr("Failed to remove streams folder!"));
         return;
     }
 
@@ -299,6 +304,31 @@ void StreamsSettings::configure()
     DigitallyImportedSettings(this).show();
 }
 
+void StreamsSettings::slotButtonClicked(int button)
+{
+    switch (button) {
+    case User1:
+        configure();
+        break;
+    case User3:
+        remove();
+        break;
+    case Close:
+        save();
+        reject();
+        // Need to call this - if not, when dialog is closed by window X control, it is not deleted!!!!
+        Dialog::slotButtonClicked(button);
+        break;
+    default:
+        break;
+    }
+}
+
+void StreamsSettings::raiseWindow()
+{
+    Utils::raiseWindow(topLevelWidget());
+}
+
 QListWidgetItem *  StreamsSettings::get(const QString &name)
 {
     for (int i=0; i<categories->count(); ++i) {
@@ -307,5 +337,7 @@ QListWidgetItem *  StreamsSettings::get(const QString &name)
             return item;
         }
     }
-    return 0;
+    return nullptr;
 }
+
+#include "moc_streamssettings.cpp"
