@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -27,17 +27,13 @@
 #include <QAbstractItemView>
 #include <QDesktopWidget>
 #include <QApplication>
-#include "gtkstyle.h"
+#include <QStyleOptionComboBox>
 
 // Max number of items before we try to force a scrollbar in popup menu...
 static int maxPopupItemCount=-1;
 
 ComboBox::ComboBox(QWidget *p)
-    #ifdef ENABLE_KDE_SUPPORT
-    : KComboBox(p)
-    #else
     : QComboBox(p)
-    #endif
     , toggleState(false)
 {
     #if !defined Q_OS_WIN && !defined Q_OS_MAC
@@ -49,9 +45,7 @@ ComboBox::ComboBox(QWidget *p)
         }
     }
     #endif
-    #if QT_VERSION >= 0x050000
     connect(this, SIGNAL(editTextChanged(QString)), this, SIGNAL(textChanged(QString)));
-    #endif
 }
 
 void ComboBox::setEditable(bool editable)
@@ -66,7 +60,11 @@ void ComboBox::setEditable(bool editable)
 #if !defined Q_OS_WIN && !defined Q_OS_MAC
 void ComboBox::showPopup()
 {
-    if (GtkStyle::isActive() && count()>(maxPopupItemCount-2)) {
+    QStyleOptionComboBox opt;
+    initStyleOption(&opt);
+    toggleState=false;
+    bool hack=0!=style()->styleHint(QStyle::SH_ComboBox_Popup, &opt, this, nullptr);
+    if (hack && count()>(maxPopupItemCount-2)) {
         toggleState=!isEditable();
 
         // Hacky, but if we set the combobox as editable - the style gives the
@@ -78,12 +76,8 @@ void ComboBox::showPopup()
         }
     }
     QComboBox::showPopup();
-//    if (Utils::touchFriendly()) {
-//        view()->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-//        view()->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-//    }
 
-    if (GtkStyle::isActive() && parentWidget() && view()->parentWidget() && count()>maxPopupItemCount) {
+    if (hack && view()->parentWidget() && count()>maxPopupItemCount) {
         // Also, if the size of the popup is more than required for 32 items, then
         // restrict its height...
         int maxHeight=maxPopupItemCount*view()->sizeHintForRow(0);
@@ -107,22 +101,23 @@ void ComboBox::showPopup()
         }
         view()->parentWidget()->setGeometry(geo);
 
-        #ifndef ENABLE_KDE_SUPPORT
         // Hide scrollers - these look ugly...
-        foreach (QObject *c, view()->parentWidget()->children()) {
+        for (QObject *c: view()->parentWidget()->children()) {
             if (0==qstrcmp("QComboBoxPrivateScroller", c->metaObject()->className())) {
                 static_cast<QWidget *>(c)->setMaximumHeight(0);
             }
         }
-        #endif
     }
 }
 
 void ComboBox::hidePopup()
 {
-    if (GtkStyle::isActive() && count()>(maxPopupItemCount-2) && toggleState) {
+    if (toggleState) {
         QComboBox::setEditable(false);
+        toggleState=false;
     }
     QComboBox::hidePopup();
 }
 #endif
+
+#include "moc_combobox.cpp"
