@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,34 +26,22 @@
 
 #include <QComboBox>
 
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KComboBox>
-#endif
-class ComboBox
-    #ifdef ENABLE_KDE_SUPPORT
-    : public KComboBox
-    #else
-    : public QComboBox
-    #endif
+class ComboBox : public QComboBox
 {
-    #if QT_VERSION >= 0x050000
     Q_OBJECT
-    #endif
 
 public:
     ComboBox(QWidget *p);
-    virtual ~ComboBox() { }
+    ~ComboBox() override { }
 
     void setEditable(bool editable);
     #if !defined Q_OS_WIN && !defined Q_OS_MAC
-    void showPopup();
-    void hidePopup();
+    void showPopup() override;
+    void hidePopup() override;
     #endif
 
-#if QT_VERSION >= 0x050000
 Q_SIGNALS:
     void textChanged(const QString &t);
-#endif
 
 private:
     bool toggleState;
