@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  */
 
 /**************************************************************************
@@ -34,23 +34,24 @@
 **************************************************************************/
 
 #include "fancytabwidget.h"
-#include "localize.h"
 #include "icon.h"
-#include "gtkstyle.h"
 #include "action.h"
 #include "utils.h"
+#include "config.h"
+#ifdef Q_OS_MAC
+#include "osxstyle.h"
+#endif
+#include <QStyleOption>
 #include <QHBoxLayout>
 #include <QMouseEvent>
+#include <QWheelEvent>
 #include <QPainter>
 #include <QSplitter>
 #include <QStackedWidget>
-#include <QStyleOption>
 #include <QToolButton>
 #include <QToolTip>
 #include <QVBoxLayout>
 #include <QApplication>
-#include <QAnimationGroup>
-#include <QPropertyAnimation>
 #include <QSignalMapper>
 #include <QMenu>
 
@@ -73,7 +74,7 @@ static int smallIconSize=16;
 void FancyTabWidget::setup()
 {
     largeIconSize=Icon::stdSize(Utils::scaleForDpi(32));
-    smallIconSize=Icon::stdSize(Utils::scaleForDpi(Utils::touchFriendly() ? 22 : 16));
+    smallIconSize=Icon::stdSize(Utils::scaleForDpi(16));
 }
 
 int FancyTabWidget::iconSize(bool large)
@@ -83,8 +84,14 @@ int FancyTabWidget::iconSize(bool large)
 
 static void drawIcon(const QIcon &icon, const QRect &r, QPainter *p, const QSize &iconSize, bool selected)
 {
+    #ifdef Q_OS_WIN
+    Q_UNUSED(selected);
+    QPixmap px = icon.pixmap(iconSize, QIcon::Normal);
+    #else
     QPixmap px = icon.pixmap(iconSize, selected ? QIcon::Selected : QIcon::Normal);
-    p->drawPixmap(r.x()+(r.width()-px.width())/2.0, r.y()+(r.height()-px.height())/2.0, px.width(), px.height(), px);
+    #endif
+    QSize layoutSize = px.size() / px.DEVICE_PIXEL_RATIO();
+    p->drawPixmap(r.x()+(r.width()-layoutSize.width())/2.0, r.y()+(r.height()-layoutSize.height())/2.0, layoutSize.width(), layoutSize.height(), px);
 }
 
 void FancyTabProxyStyle::drawPrimitive(PrimitiveElement element, const QStyleOption *option, QPainter *p, const QWidget *widget) const
@@ -94,31 +101,21 @@ void FancyTabProxyStyle::drawPrimitive(PrimitiveElement element, const QStyleOpt
     }
 }
 
-int FancyTabProxyStyle::styleHint(StyleHint hint, const QStyleOption *option, const QWidget *widget, QStyleHintReturn *returnData) const
-{
-    if (SH_TabBar_Alignment==hint && widget && qobject_cast<const QTabBar *>(widget)) {
-        QTabBar::Shape shape=static_cast<const QTabBar *>(widget)->shape();
-        if (QTabBar::RoundedNorth==shape || QTabBar::RoundedSouth==shape) {
-            return (int)Qt::AlignCenter;
-        }
-    }
-    return QProxyStyle::styleHint(hint, option, widget, returnData);
-}
-
 void FancyTabProxyStyle::drawControl(ControlElement element, const QStyleOption *option, QPainter *p, const QWidget *widget) const
 {
+    const QStyleOptionTab *tabOpt = qstyleoption_cast<const QStyleOptionTab*>(option);
 
-    const QStyleOptionTabV3 *v3Opt = qstyleoption_cast<const QStyleOptionTabV3*>(option);
-
-    if (element != CE_TabBarTab || !v3Opt) {
+    if (element != CE_TabBarTab || !tabOpt) {
         QProxyStyle::drawControl(element, option, p, widget);
         return;
     }
 
-    const QRect rect = v3Opt->rect;
-    const bool selected = v3Opt->state  &State_Selected;
-    const bool verticalTabs = v3Opt->shape == QTabBar::RoundedWest;
-    const QString text = v3Opt->text;
+    const QRect rect = tabOpt->rect;
+    const bool selected = tabOpt->state&State_Selected;
+    const bool underMouse = tabOpt->state&State_MouseOver;
+    bool active = tabOpt->state&State_Active;
+    const bool verticalTabs = tabOpt->shape == QTabBar::RoundedWest;
+    const QString text = tabOpt->text;
 
     QTransform m;
     if (verticalTabs) {
@@ -130,88 +127,50 @@ void FancyTabProxyStyle::drawControl(ControlElement element, const QStyleOption
 
     const QRect draw_rect(QPoint(0, 0), m.mapRect(rect).size());
 
-    if (!selected && GtkStyle::isActive()) {
-        p->fillRect(option->rect, option->palette.background());
-    }
-
     p->save();
     p->setTransform(m);
 
-    QRect iconRect(QPoint(8, 0), v3Opt->iconSize);
+    QRect iconRect(QPoint(8, 0), tabOpt->iconSize);
     QRect textRect(iconRect.topRight() + QPoint(4, 0), draw_rect.size());
     textRect.setRight(draw_rect.width());
     iconRect.translate(0, (draw_rect.height() - iconRect.height()) / 2);
 
-    QStyleOptionViewItemV4 styleOpt;
-    styleOpt.palette=option->palette;
-    styleOpt.rect=draw_rect;
-    if (QStyleOptionTab::Beginning==v3Opt->position) {
-        styleOpt.rect.adjust(0, 0, -1, 0);
-    }
-    styleOpt.state=option->state;
-    styleOpt.state&=~(QStyle::State_Selected|QStyle::State_MouseOver);
-    styleOpt.state|=QStyle::State_Selected|QStyle::State_Enabled;
-    styleOpt.viewItemPosition = QStyleOptionViewItemV4::OnlyOne;
-    styleOpt.showDecorationSelected=true;
-    bool drawBgnd=true;
-    int fader = 1;
+    if (selected || underMouse) {
+        #ifdef Q_OS_MAC
+        QColor col = OSXStyle::self()->viewPalette().highlight().color();
+        #elif defined Q_OS_WIN
+        QColor col = active ? option->palette.highlight().color() : QColor(96, 96, 96);
+        col.setAlphaF(0.25);
+        #else
+        QColor col = option->palette.highlight().color();
+        #endif
 
-    if (!selected && drawBgnd) {
-        const QString faderKey = "tab_" + text + "_fader";
-        const QString animationKey = "tab_" + text + "_animation";
-
-        const QString tab_hover = widget->property("tab_hover").toString();
-        fader=widget->property(faderKey.toUtf8().constData()).toInt();
-        QPropertyAnimation *animation = widget->property(animationKey.toUtf8().constData()).value<QPropertyAnimation*>();
-
-        if (!animation) {
-            QWidget* mut_widget = const_cast<QWidget*>(widget);
-            fader = 0;
-            mut_widget->setProperty(faderKey.toUtf8().constData(), fader);
-            animation = new QPropertyAnimation(mut_widget, faderKey.toUtf8(), mut_widget);
-            connect(animation, SIGNAL(valueChanged(QVariant)), mut_widget, SLOT(update()));
-            mut_widget->setProperty(animationKey.toUtf8().constData(), QVariant::fromValue(animation));
-        }
-
-        if (text == tab_hover) {
-            if (animation->state() != QAbstractAnimation::Running && fader != 40) {
-                animation->stop();
-                animation->setDuration(80);
-                animation->setEndValue(50);
-                animation->start();
+        if (selected) {
+            #ifndef Q_OS_MAC
+            if (!active && option->palette.highlight().color()==QApplication::palette().color(QPalette::Active, QPalette::Highlight)) {
+                active = true;
             }
+            #endif
         } else {
-            if (animation->state() != QAbstractAnimation::Running && fader != 0) {
-                animation->stop();
-                animation->setDuration(160);
-                animation->setEndValue(0);
-                animation->start();
-            }
-        }
-
-        if (fader<1) {
-            drawBgnd=false;
-        } else {
-            QColor col(styleOpt.palette.highlight().color());
-            col.setAlpha(fader);
-            styleOpt.palette.setColor(styleOpt.palette.currentColorGroup(), QPalette::Highlight, col);
-        }
-    }
-
-    if (drawBgnd) {
-        if (!selected && GtkStyle::isActive()) {
-            GtkStyle::drawSelection(styleOpt, p, (fader*1.0)/150.0);
-        } else {
-            QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &styleOpt, p, 0);
+            #if defined Q_OS_WIN
+            col.setAlphaF(0.1);
+            #else
+            col.setAlphaF(0.2);
+            #endif
         }
+        p->fillRect(draw_rect, col);
     }
 
     int textFlags = Qt::AlignTop | Qt::AlignVCenter;
-    p->setPen(selected && option->state&State_Active
-              ? QApplication::palette().highlightedText().color() : QApplication::palette().foreground().color());
+    #ifdef Q_OS_MAC
+    p->setPen(selected && active ? OSXStyle::self()->viewPalette().highlightedText().color() : OSXStyle::self()->viewPalette().foreground().color());
+    #elif defined Q_OS_WIN
+    p->setPen(QApplication::palette().foreground().color());
+    #else
+    p->setPen(selected && active ? QApplication::palette().highlightedText().color() : QApplication::palette().foreground().color());
+    #endif
 
-    drawIcon(v3Opt->icon, iconRect, p, v3Opt->iconSize,
-             selected && option->state&State_Active);
+    drawIcon(tabOpt->icon, iconRect, p, tabOpt->iconSize, selected && active);
 
     QString txt=text;
     txt.replace("&", "");
@@ -241,6 +200,7 @@ void FancyTabProxyStyle::polish(QPalette &palette)
 
 bool FancyTabProxyStyle::eventFilter(QObject* o, QEvent* e)
 {
+    #ifndef Q_OS_MAC
     QTabBar *bar = qobject_cast<QTabBar*>(o);
     if (bar && (e->type() == QEvent::MouseMove || e->type() == QEvent::Leave)) {
         QMouseEvent *event = static_cast<QMouseEvent*>(e);
@@ -252,38 +212,16 @@ bool FancyTabProxyStyle::eventFilter(QObject* o, QEvent* e)
             bar->update();
         }
     }
+    #endif
 
     return false;
 }
 
 FancyTab::FancyTab(FancyTabBar* tabbar)
-    : QWidget(tabbar), tabbar(tabbar), faderValue(0)
+    : QWidget(tabbar), tabbar(tabbar), underMouse(false)
 {
-    animator.setPropertyName("fader");
-    animator.setTargetObject(this);
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
-}
-
-void FancyTab::fadeIn()
-{
-    animator.stop();
-    animator.setDuration(80);
-    animator.setEndValue(50);
-    animator.start();
-}
-
-void FancyTab::fadeOut()
-{
-    animator.stop();
-    animator.setDuration(160);
-    animator.setEndValue(0);
-    animator.start();
-}
-
-void FancyTab::setFader(float value)
-{
-    faderValue = value;
-    tabbar->update();
+    setAttribute(Qt::WA_Hover, true);
 }
 
 FancyTabBar::FancyTabBar(QWidget *parent, bool text, int iSize, Pos pos)
@@ -298,7 +236,7 @@ FancyTabBar::FancyTabBar(QWidget *parent, bool text, int iSize, Pos pos)
     setMouseTracking(true); // Needed for hover events
     triggerTimer.setSingleShot(true);
 
-    QBoxLayout* layout=0;
+    QBoxLayout* layout=nullptr;
 
     if (Side!=pos) {
         setMinimumHeight(tabSizeHint().height());
@@ -328,28 +266,30 @@ QSize FancyTab::sizeHint() const
     int iconSize = tabbar->iconSize();
     bool withText = tabbar->showText();
     int spacing = sidebarSpacing(withText);
+    int padding = FancyTabBar::Side==tabbar->position() ? Utils::scaleForDpi(12) : 0;
     if (withText) {
         QFontMetrics fm(font());
         int textWidth = fm.width(text)*1.1;
         int width = qMax(iconSize, qMin(3*iconSize, textWidth)) + spacing;
-        return QSize(width, iconSize + spacing + fm.height());
+        return QSize(width, iconSize + spacing + fm.height() + padding);
     } else {
-        return QSize(iconSize + spacing, iconSize + spacing);
+        return QSize(iconSize + spacing + padding, iconSize + spacing + padding);
     }
 }
 
 QSize FancyTabBar::tabSizeHint() const
 {
     int spacing = sidebarSpacing(withText);
+    int padding = Side==pos ? Utils::scaleForDpi(12) : 0;
     if (withText) {
         QFontMetrics fm(font());
         int maxTw=0;
-        foreach (FancyTab *tab, tabs) {
+        for (FancyTab *tab: tabs) {
             maxTw=qMax(maxTw, tab->sizeHint().width());
         }
-        return QSize(qMax(icnSize + spacing, maxTw), icnSize + spacing + fm.height());
+        return QSize(qMax(icnSize + spacing, maxTw), icnSize + spacing + fm.height() + padding);
     } else {
-        return QSize(icnSize + spacing, icnSize + spacing);
+        return QSize(icnSize + spacing + padding, icnSize + spacing + padding);
     }
 }
 
@@ -357,28 +297,27 @@ void FancyTabBar::paintEvent(QPaintEvent *event)
 {
     Q_UNUSED(event)
     QPainter p(this);
-    bool gtkStyle=GtkStyle::isActive();
 
     for (int i = 0; i < count(); ++i) {
         if (i != currentIndex()) {
-            paintTab(&p, i, gtkStyle);
+            paintTab(&p, i);
         }
     }
 
     // paint active tab last, since it overlaps the neighbors
     if (currentIndex() != -1) {
-        paintTab(&p, currentIndex(), gtkStyle);
+        paintTab(&p, currentIndex());
     }
 }
 
 void FancyTab::enterEvent(QEvent*)
 {
-    fadeIn();
+    underMouse = true;
 }
 
 void FancyTab::leaveEvent(QEvent*)
 {
-    fadeOut();
+    underMouse = false;
 }
 
 QSize FancyTabBar::sizeHint() const
@@ -436,6 +375,24 @@ void FancyTabBar::mousePressEvent(QMouseEvent *e)
     }
 }
 
+void FancyTabBar::wheelEvent(QWheelEvent *ev)
+{
+    int numDegrees = ev->delta() / 8;
+    int numSteps = numDegrees / -15;
+    int prevIndex = currentIdx;
+
+    if (numSteps>0) {
+        currentIdx=qMin(currentIdx+numSteps, tabs.size()-1);
+    } else if (numSteps<0) {
+        currentIdx=qMax(currentIdx+numSteps, 0);
+    }
+    if (currentIdx!=prevIndex) {
+        update();
+        triggerTimer.start(0);
+    }
+    ev->accept();
+}
+
 void FancyTabBar::addTab(const QIcon &icon, const QString &label, const QString &tt)
 {
     FancyTab *tab = new FancyTab(this);
@@ -456,7 +413,7 @@ void FancyTabBar::addSpacer(int size)
                                                            new QSpacerItem(0, size, QSizePolicy::Fixed, QSizePolicy::Maximum));
 }
 
-void FancyTabBar::paintTab(QPainter *painter, int tabIndex, bool gtkStyle) const
+void FancyTabBar::paintTab(QPainter *painter, int tabIndex) const
 {
     if (!validIndex(tabIndex)) {
         qWarning("invalid index");
@@ -466,53 +423,53 @@ void FancyTabBar::paintTab(QPainter *painter, int tabIndex, bool gtkStyle) const
 
     QRect rect = tabRect(tabIndex);
     bool selected = (tabIndex == currentIdx);
+    bool underMouse = tabs[tabIndex]->underMouse;
 
-    QStyleOptionViewItemV4 styleOpt;
-    styleOpt.initFrom(this);
-    styleOpt.state&=~(QStyle::State_Selected|QStyle::State_MouseOver);
-    styleOpt.state|=QStyle::State_Selected|QStyle::State_Enabled;
-    styleOpt.rect=rect;
-    styleOpt.viewItemPosition = QStyleOptionViewItemV4::OnlyOne;
-    styleOpt.showDecorationSelected=true;
-    bool drawBgnd=true;
+    if (selected || underMouse) {
+        #ifdef Q_OS_MAC
+        QColor col = OSXStyle::self()->viewPalette().highlight().color();
+        #elif defined Q_OS_WIN
+        QColor col = palette().currentColorGroup()==QPalette::Active ? palette().highlight().color() : QColor(96, 96, 96);
+        col.setAlphaF(0.25);
+        #else
+        QColor col = palette().highlight().color();
+        #endif
 
-    if (!selected && drawBgnd) {
-        int fader=int(tabs[tabIndex]->fader());
-        if (fader<1) {
-            drawBgnd=false;
-        } else  {
-            QColor col(styleOpt.palette.highlight().color());
-            col.setAlpha(fader);
-            styleOpt.palette.setColor(styleOpt.palette.currentColorGroup(), QPalette::Highlight, col);
-        }
-    }
-    if (drawBgnd) {
-        if (!selected && gtkStyle) {
-            GtkStyle::drawSelection(styleOpt, painter, tabs[tabIndex]->fader()/150.0);
-        } else {
-            QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &styleOpt, painter, 0);
+        if (!selected) {
+            #if defined Q_OS_WIN
+            col.setAlphaF(0.1);
+            #else
+            col.setAlphaF(0.2);
+            #endif
         }
+        painter->fillRect(rect, col);
     }
 
+    selected = selected && (palette().currentColorGroup()==QPalette::Active ||
+                           (palette().highlightedText().color()==palette().brush(QPalette::Active, QPalette::HighlightedText).color()));
+
     if (withText) {
         QString tabText(painter->fontMetrics().elidedText(this->tabText(tabIndex), elideMode(), width()));
         QRect tabTextRect(tabRect(tabIndex));
         QRect tabIconRect(tabTextRect);
         tabIconRect.adjust(+4, +4, -4, -4);
         tabTextRect.translate(0, -2);
-        painter->setPen(selected ? palette().highlightedText().color() : palette().foreground().color());
+
+        #ifdef Q_OS_MAC
+        painter->setPen(selected ? OSXStyle::self()->viewPalette().highlightedText().color() : OSXStyle::self()->viewPalette().foreground().color());
+        #elif defined Q_OS_WIN
+        painter->setPen(QApplication::palette().foreground().color());
+        #else
+        painter->setPen(selected ? QApplication::palette().highlightedText().color() : QApplication::palette().foreground().color());
+        #endif
         int textFlags = Qt::AlignCenter | Qt::AlignBottom;
         painter->drawText(tabTextRect, textFlags, tabText);
 
         const int textHeight = painter->fontMetrics().height();
         tabIconRect.adjust(0, 4, 0, -textHeight);
-        drawIcon(tabIcon(tabIndex), tabIconRect, painter, QSize(icnSize, icnSize),
-                 selected && (palette().currentColorGroup()==QPalette::Active ||
-                              (palette().highlightedText().color()==palette().brush(QPalette::Active, QPalette::HighlightedText).color())));
+        drawIcon(tabIcon(tabIndex), tabIconRect, painter, QSize(icnSize, icnSize), selected);
     } else {
-        drawIcon(tabIcon(tabIndex), rect, painter, QSize(icnSize, icnSize),
-                 selected && (palette().currentColorGroup()==QPalette::Active ||
-                              (palette().highlightedText().color()==palette().brush(QPalette::Active, QPalette::HighlightedText).color())));
+        drawIcon(tabIcon(tabIndex), rect, painter, QSize(icnSize, icnSize), selected);
     }
 
     painter->restore();
@@ -525,17 +482,16 @@ void FancyTabBar::setCurrentIndex(int index)
     emit currentChanged(currentIdx);
 }
 
-FancyTabWidget::FancyTabWidget(QWidget *parent, bool allowContextMenu)
+FancyTabWidget::FancyTabWidget(QWidget *parent)
     : QWidget(parent)
     , styleSetting(0)
-    , tabBar(NULL)
+    , tabBar(nullptr)
     , stack_(new QStackedWidget(this))
     , sideWidget(new QWidget)
     , sideLayout(new QVBoxLayout)
     , topLayout(new QVBoxLayout)
-    , menu(0)
+    , menu(nullptr)
     , proxyStyle(new FancyTabProxyStyle)
-    , allowContext(allowContextMenu)
 {
     sideLayout->setSpacing(0);
     sideLayout->setMargin(0);
@@ -586,7 +542,7 @@ int FancyTabWidget::count() const
 int FancyTabWidget::visibleCount() const
 {
     int c=0;
-    foreach (const Item &i, items) {
+    for (const Item &i: items) {
         if (i.enabled) {
             c++;
         }
@@ -625,50 +581,6 @@ void FancyTabWidget::showWidget(int index)
     emit currentChanged(idx);
 }
 
-void FancyTabWidget::contextMenuEvent(QContextMenuEvent *e)
-{
-    if (!allowContext) {
-        return;
-    }
-
-    // Check we are over tab space...
-    if (Tab==(styleSetting&Style_Mask)) {
-        if (QApplication::widgetAt(e->globalPos())!=tabBar) {
-            return;
-        }
-    }
-    else {
-        switch (styleSetting&Position_Mask) {
-        case Bot:
-            if (e->pos().y()<=(sideWidget->pos().y()+(sideWidget->height()-tabBar->height()))) {
-                return;
-            }
-            break;
-        case Top:
-            if (e->pos().y()>(sideWidget->pos().y()+tabBar->height())) {
-                return;
-            }
-            break;
-        default:
-            if (QApplication::isRightToLeft()) {
-                if (e->pos().x()<=sideWidget->pos().x()) {
-                    return;
-                }
-            } else if (e->pos().x()>=sideWidget->rect().right()) {
-                return;
-            }
-        }
-    }
-
-    if (!menu) {
-        menu = new QMenu(this);
-        QAction *act=new QAction(i18n("Configure..."), this);
-        connect(act, SIGNAL(triggered()), SIGNAL(configRequested()));
-        menu->addAction(act);
-    }
-    menu->popup(e->globalPos());
-}
-
 void FancyTabWidget::setStyle(int s)
 {
     if (s==styleSetting && tabBar) {
@@ -676,7 +588,7 @@ void FancyTabWidget::setStyle(int s)
     }
     // Remove previous tab bar
     delete tabBar;
-    tabBar = NULL;
+    tabBar = nullptr;
 
     //   use_background_ = false;
     // Create new tab bar
@@ -884,3 +796,5 @@ QStringList FancyTabWidget::hiddenPages() const
     }
     return pages;
 }
+
+#include "moc_fancytabwidget.cpp"
