@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /**************************************************************************
@@ -40,7 +40,6 @@
 #include <QProxyStyle>
 #include <QTabBar>
 #include <QWidget>
-#include <QPropertyAnimation>
 #include <QTimer>
 #include <QScopedPointer>
 #include <QStringList>
@@ -57,45 +56,33 @@ class FancyTabProxyStyle : public QProxyStyle
     Q_OBJECT
 
 public:
-    void drawPrimitive(PrimitiveElement element, const QStyleOption *option, QPainter *p, const QWidget *widget) const;
-    void drawControl(ControlElement element, const QStyleOption *option, QPainter *p, const QWidget *widget) const;
-    int styleHint(StyleHint hint, const QStyleOption *option, const QWidget *widget, QStyleHintReturn *returnData) const;
-    void polish(QWidget *widget);
-    void polish(QApplication *app);
-    void polish(QPalette &palette);
+    void drawPrimitive(PrimitiveElement element, const QStyleOption *option, QPainter *p, const QWidget *widget) const override;
+    void drawControl(ControlElement element, const QStyleOption *option, QPainter *p, const QWidget *widget) const override;
+    void polish(QWidget *widget) override;
+    void polish(QApplication *app) override;
+    void polish(QPalette &palette) override;
 
 protected:
-    bool eventFilter(QObject *o, QEvent *e);
+    bool eventFilter(QObject *o, QEvent *e) override;
 };
 
 class FancyTabBar;
 class FancyTab : public QWidget
 {
-    Q_OBJECT
-
-    Q_PROPERTY(float fader READ fader WRITE setFader)
-
 public:
     FancyTab(FancyTabBar *tabbar);
-    float fader() { return faderValue; }
-    void setFader(float value);
-
-    QSize sizeHint() const;
-
-    void fadeIn();
-    void fadeOut();
 
+    QSize sizeHint() const override;
     QIcon icon;
     QString text;
+    bool underMouse;
 
 protected:
-    void enterEvent(QEvent *);
-    void leaveEvent(QEvent *);
+    void enterEvent(QEvent *) override;
+    void leaveEvent(QEvent *) override;
 
 private:
-    QPropertyAnimation animator;
     FancyTabBar *tabbar;
-    float faderValue;
 };
 
 class FancyTabBar : public QWidget
@@ -110,15 +97,16 @@ public:
     };
 
     FancyTabBar(QWidget *parent, bool text, int iSize, Pos pos);
-    ~FancyTabBar();
+    ~FancyTabBar() override;
 
-    void paintEvent(QPaintEvent *event);
-    void paintTab(QPainter *painter, int tabIndex, bool gtkStyle) const;
-    void mousePressEvent(QMouseEvent *);
+    void paintEvent(QPaintEvent *event) override;
+    void paintTab(QPainter *painter, int tabIndex) const;
+    void mousePressEvent(QMouseEvent *) override;
+    void wheelEvent(QWheelEvent *ev) override;
     bool validIndex(int index) const { return index >= 0 && index < tabs.count(); }
 
-    QSize sizeHint() const;
-    QSize minimumSizeHint() const;
+    QSize sizeHint() const override;
+    QSize minimumSizeHint() const override;
 
     void addTab(const QIcon &icon, const QString &label, const QString &tt);
     void addSpacer(int size = 40);
@@ -136,6 +124,7 @@ public:
     bool showText() const { return withText; }
     int iconSize() const { return icnSize; }
     QSize tabSizeHint() const;
+    Pos position() const { return pos; }
 
 Q_SIGNALS:
     void currentChanged(int);
@@ -160,7 +149,7 @@ public:
     static void setup();
     static int iconSize(bool large=true);
 
-    FancyTabWidget(QWidget *parent, bool allowContextMenu=true);
+    FancyTabWidget(QWidget *parent);
 
     enum Style {
         Side     = 0x0001,
@@ -222,13 +211,11 @@ Q_SIGNALS:
     void currentChanged(int index);
     void styleChanged(int styleSetting);
     void tabToggled(int index);
-    void configRequested();
 
 private Q_SLOTS:
     void showWidget(int index);
 
 private:
-    void contextMenuEvent(QContextMenuEvent *e);
     void makeTabBar(QTabBar::Shape shape, bool text, bool icons, bool fancy);
     int tabToIndex(int tab) const;
     int IndexToTab(int index) const { return index>=0 && index<items.count() ? items[index].index : 0; }
@@ -243,9 +230,7 @@ private:
     QVBoxLayout *topLayout;
     QMenu *menu;
     QScopedPointer<FancyTabProxyStyle> proxyStyle;
-    bool allowContext;
 };
 
-Q_DECLARE_METATYPE(QPropertyAnimation*);
 
 #endif // FANCYTABWIDGET_H
