@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,25 +25,15 @@
 #define GTKSTYLE_H
 
 #include <QStyleOption>
+#include <QString>
+
 class QPainter;
 class QWidget;
 
-#include <QString>
-
 namespace GtkStyle
 {
-    extern const char * constHideFrameProp;
-
     extern bool isActive();
-    extern void drawSelection(const QStyleOptionViewItemV4 &opt, QPainter *painter, double opacity);
-    extern QString themeName();
-    extern QString iconTheme();
-    extern void setThemeName(const QString &n);
-    extern void setIconTheme(const QString &n);
-    extern void applyTheme(QWidget *widget);
-    extern void registerWidget(QWidget *widget);
-    extern bool useSymbolicIcons();
-    extern QColor symbolicColor();
+    extern void drawSelection(const QStyleOptionViewItem &opt, QPainter *painter, double opacity);
 }
 
 #endif
