@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,6 +26,7 @@
 
 #include <QThread>
 
+class QTimer;
 // ThreadCleaner *needs* to reside in the GUI thread. When a 'Thread' is created it will connect
 // its finished signal to threadFinished(), this then calls deleteLater() to ensure that the
 // thread is finished before it is deleted - and is deleted in the gui thread.
@@ -37,7 +38,7 @@ public:
     static void enableDebug();
     static ThreadCleaner * self();
     ThreadCleaner() { }
-    ~ThreadCleaner() { }
+    ~ThreadCleaner() override { }
 
     // This function must *ONLY* be called from GUI thread...
     void stopAll();
@@ -57,13 +58,19 @@ class Thread : public QThread
 {
     Q_OBJECT
 public:
-    Thread(const QString &name, QObject *p=0);
-    virtual ~Thread();
+    Thread(const QString &name, QObject *p=nullptr);
+    ~Thread() override;
+
+    // Make QThread::msleep accessible!
+    using QThread::msleep;
+
+    void run() override;
+
+    QTimer * createTimer(QObject *parent=nullptr);
+    void deleteTimer(QTimer *timer);
 
-    virtual void run();
 public Q_SLOTS:
     void stop() { quit(); }
 };
 
 #endif
-
