@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,27 +23,25 @@
 
 #include "utils.h"
 #include "config.h"
-#include "localize.h"
 #include <QFile>
 #include <QFileInfo>
 #include <QDir>
 #include <QProcess>
-#include <QSet>
-#include <QThread>
 #include <QApplication>
 #include <QDateTime>
 #include <QTime>
 #include <QWidget>
 #include <QStyle>
-#if QT_VERSION >= 0x050000
+#include <QDesktopWidget>
+#include <QEventLoop>
 #include <QStandardPaths>
-#else
-#include <QDesktopServices>
-#endif
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KStandardDirs>
-#endif
+#include <QSet>
+#ifndef _MSC_VER 
 #include <unistd.h>
+#include <utime.h>
+#else
+#include <sys/utime.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifndef Q_OS_WIN
@@ -51,13 +49,143 @@
 #include <pwd.h>
 #endif
 #include <sys/types.h>
-#include <utime.h>
+#if QT_QTDBUS_FOUND && !defined Q_OS_MAC && !defined Q_OS_WIN
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#endif
 
 const QLatin1Char Utils::constDirSep('/');
 const QLatin1String Utils::constDirSepStr("/");
 const char * Utils::constDirSepCharStr="/";
 
-static const QLatin1String constHttp("http://");
+static const QLatin1String constHttp("http:");
+
+QString Utils::fixPath(const QString &dir, bool ensureEndsInSlash)
+{
+    QString d(dir);
+
+    if (!d.isEmpty() && !d.startsWith(constHttp)) {
+        #ifdef Q_OS_WIN
+        // Windows shares can be \\host\share (which gets converted to //host/share)
+        // so if th epath starts with // we need to keep this double slash.
+        bool startsWithDoubleSlash=d.length()>2 && d.startsWith(QLatin1String("//"));
+        #endif
+        d.replace(QLatin1String("//"), constDirSepStr);
+        #ifdef Q_OS_WIN
+        if (startsWithDoubleSlash) { // Re add first slash
+            d=QLatin1Char('/')+d;
+        }
+        #endif
+    }
+    d.replace(QLatin1String("/./"), constDirSepStr);
+    if (ensureEndsInSlash && !d.isEmpty() && !d.endsWith(constDirSep)) {
+        d+=constDirSep;
+    }
+    return d;
+}
+
+#ifndef Q_OS_WIN
+static const QLatin1String constTilda("~");
+QString Utils::homeToTilda(const QString &s)
+{
+    QString hp=QDir::homePath();
+    if (s==hp) {
+        return constTilda;
+    }
+    if (s.startsWith(hp+constDirSepStr)) {
+        return constTilda+fixPath(s.mid(hp.length()), false);
+    }
+    return s;
+}
+
+QString Utils::tildaToHome(const QString &s)
+{
+    if (s==constTilda) {
+        return fixPath(QDir::homePath());
+    }
+    if (s.startsWith(constTilda+constDirSep)) {
+        return fixPath(QDir::homePath()+constDirSepStr+s.mid(1), false);
+    }
+    return s;
+}
+#endif
+
+QString Utils::getDir(const QString &file)
+{
+    bool isCueFile=file.contains("/cue:///") && file.contains("?pos=");
+    QString d(file);
+    int slashPos(d.lastIndexOf(constDirSep));
+
+    if(slashPos!=-1) {
+        d.remove(slashPos+1, d.length());
+    }
+
+    if (isCueFile) {
+        d.remove("cue:///");
+    }
+    return fixPath(d);
+}
+
+QString Utils::getFile(const QString &file)
+{
+    QString f(file);
+    int slashPos=f.lastIndexOf(constDirSep);
+
+    if (-1!=slashPos) {
+        f.remove(0, slashPos+1);
+    }
+
+    return f;
+}
+
+QString Utils::getExtension(const QString &file)
+{
+    QString f(file);
+    int dotPos=f.lastIndexOf('.');
+
+    if (-1!=dotPos) {
+        return f.mid(dotPos);
+    }
+
+    return QString();
+}
+
+QString Utils::changeExtension(const QString &file, const QString &extension)
+{
+    if (extension.isEmpty()) {
+        return file;
+    }
+
+    QString f(file);
+    int pos=f.lastIndexOf('.');
+    if (pos>1) {
+        f=f.left(pos+1);
+    }
+
+    if (f.endsWith('.')) {
+        return f+(extension.startsWith('.') ? extension.mid(1) : extension);
+    }
+    return f+(extension.startsWith('.') ? extension : (QChar('.')+extension));
+}
+
+bool Utils::isDirReadable(const QString &dir)
+{
+    #ifdef Q_OS_WIN
+    if (dir.isEmpty()) {
+        return false;
+    } else {
+        QDir d(dir);
+        bool dirReadable=d.isReadable();
+        // Handle cases where dir is set to \\server\ (i.e. no shared folder is set in path)
+        if (!dirReadable && dir.startsWith(QLatin1String("//")) && d.isRoot() && (dir.length()-1)==dir.indexOf(Utils::constDirSep, 2)) {
+            dirReadable=true;
+        }
+        return dirReadable;
+    }
+    #else
+    return dir.isEmpty() ? false : QDir(dir).isReadable();
+    #endif
+}
 
 QString Utils::strippedText(QString s)
 {
@@ -95,100 +223,33 @@ QString Utils::stripAcceleratorMarkers(QString label)
     return label;
 }
 
-QString Utils::fixPath(const QString &dir)
+QString Utils::convertPathForDisplay(const QString &path, bool isFolder)
 {
-    QString d(dir);
+    if (path.isEmpty() || path.startsWith(constHttp)) {
+        return path;
+    }
 
-    if (!d.isEmpty() && !d.startsWith(constHttp)) {
-        #ifdef Q_OS_WIN
-        // Windows shares can be \\host\share (which gets converted to //host/share)
-        // so if th epath starts with // we need to keep this double slash.
-        bool startsWithDoubleSlash=d.length()>2 && d.startsWith(QLatin1String("//"));
-        #endif
-        d.replace(QLatin1String("//"), constDirSepStr);
-        #ifdef Q_OS_WIN
-        if (startsWithDoubleSlash) { // Re add first slash
-            d=QLatin1Char('/')+d;
-        }
-        #endif
+    QString p(path);
+    if (p.endsWith(constDirSep)) {
+        p=p.left(p.length()-1);
     }
-    d.replace(QLatin1String("/./"), constDirSepStr);
-    if (!d.isEmpty() && !d.endsWith(constDirSep)) {
-        d+=constDirSep;
-    }
-    return d;
+    /* TODO: Display ~/Music or /home/user/Music / /Users/user/Music ???
+    p=homeToTilda(QDir::toNativeSeparators(p));
+    */
+    return QDir::toNativeSeparators(isFolder && p.endsWith(constDirSep) ? p.left(p.length()-1) : p);
 }
 
-QString Utils::convertDirForDisplay(const QString &dir)
+QString Utils::convertPathFromDisplay(const QString &path, bool isFolder)
 {
-    if (dir.isEmpty() || dir.startsWith(constHttp)) {
-        return dir;
+    QString p=path.trimmed();
+    if (p.isEmpty()) {
+        return p;
     }
 
-    QString d(dir);
-    if (d.endsWith(constDirSep)) {
-        d=d.left(d.length()-1);
+    if (p.startsWith(constHttp)) {
+        return fixPath(p);
     }
-    return QDir::toNativeSeparators(d);
-}
-
-QString Utils::convertDirFromDisplay(const QString &dir)
-{
-    QString d=dir.trimmed();
-    if (d.isEmpty()) {
-        return d;
-    }
-
-    if (d.startsWith(constHttp)) {
-        return fixPath(d);
-    }
-    return fixPath(QDir::fromNativeSeparators(d));
-}
-
-QString Utils::getDir(const QString &file)
-{
-    bool isCueFile=file.contains("/cue:///") && file.contains("?pos=");
-    QString d(file);
-    int slashPos(d.lastIndexOf(constDirSep));
-
-    if(slashPos!=-1) {
-        d.remove(slashPos+1, d.length());
-    }
-
-    if (isCueFile) {
-        d.remove("cue:///");
-    }
-    return fixPath(d);
-}
-
-QString Utils::getFile(const QString &file)
-{
-    QString d(file);
-    int slashPos=d.lastIndexOf(constDirSep);
-
-    if (-1!=slashPos) {
-        d.remove(0, slashPos+1);
-    }
-
-    return d;
-}
-
-QString Utils::changeExtension(const QString &file, const QString &extension)
-{
-    if (extension.isEmpty()) {
-        return file;
-    }
-
-    QString f(file);
-    int pos=f.lastIndexOf('.');
-    if (pos>1) {
-        f=f.left(pos+1);
-    }
-
-    if (f.endsWith('.')) {
-        return f+(extension.startsWith('.') ? extension.mid(1) : extension);
-    }
-    return f+(extension.startsWith('.') ? extension : (QChar('.')+extension));
+    return tildaToHome(fixPath(QDir::fromNativeSeparators(p), isFolder));
 }
 
 #ifndef Q_OS_WIN
@@ -246,7 +307,7 @@ void Utils::setFilePerms(const QString &file, const char *groupName)
     ::chmod(fn.constData(), 0==gid ? 0644 : 0664);
     if (0!=gid) {
         int rv=::chown(fn.constData(), geteuid(), gid);
-        Q_UNUSED(rv);
+        Q_UNUSED(rv)
     }
     // Reset umask
     ::umask(oldMask);
@@ -254,8 +315,8 @@ void Utils::setFilePerms(const QString &file, const char *groupName)
 #else
 void Utils::setFilePerms(const QString &file, const char *groupName)
 {
-    Q_UNUSED(file);
-    Q_UNUSED(groupName);
+    Q_UNUSED(file)
+    Q_UNUSED(groupName)
 }
 #endif
 
@@ -266,8 +327,8 @@ void Utils::setFilePerms(const QString &file, const char *groupName)
 bool Utils::createWorldReadableDir(const QString &dir, const QString &base, const char *groupName)
 {
     #ifdef Q_OS_WIN
-    Q_UNUSED(base);
-    Q_UNUSED(groupName);
+    Q_UNUSED(base)
+    Q_UNUSED(groupName)
     return makeDir(dir, 0775);
     #else
     //
@@ -280,10 +341,10 @@ bool Utils::createWorldReadableDir(const QString &dir, const QString &base, cons
         QStringList parts=dir.mid(base.length()).split(constDirSep);
         QString d(base);
 
-        foreach (const QString &p, parts) {
+        for (const QString &p: parts) {
             d+=constDirSep+p;
             int rv=::chown(QFile::encodeName(d).constData(), geteuid(), gid);
-            Q_UNUSED(rv);
+            Q_UNUSED(rv)
         }
     }
     // Reset umask
@@ -292,17 +353,6 @@ bool Utils::createWorldReadableDir(const QString &dir, const QString &base, cons
     #endif
 }
 
-struct Thread : public QThread
-{
-    using QThread::msleep;
-};
-
-void Utils::msleep(int msecs)
-{
-    Thread::msleep(msecs);
-}
-
-#ifndef ENABLE_KDE_SUPPORT
 // Copied from KDE... START
 #include <QLocale>
 #include <fcntl.h>
@@ -394,19 +444,19 @@ QString Utils::formatByteSize(double size)
 
     if (useSiUnites) {
         switch(unit) {
-        case 0: return i18n("%1 B", size);
-        case 1: return i18n("%1 kB", locale.toString(size, 'f', 1));
-        case 2: return i18n("%1 MB", locale.toString(size, 'f', 1));
+        case 0: return QObject::tr("%1 B").arg(size);
+        case 1: return QObject::tr("%1 kB").arg(locale.toString(size, 'f', 1));
+        case 2: return QObject::tr("%1 MB").arg(locale.toString(size, 'f', 1));
         default:
-        case 3: return i18n("%1 GB", locale.toString(size, 'f', 1));
+        case 3: return QObject::tr("%1 GB").arg(locale.toString(size, 'f', 1));
         }
     } else {
         switch(unit) {
-        case 0: return i18n("%1 B", size);
-        case 1: return i18n("%1 KiB", locale.toString(size, 'f', 1));
-        case 2: return i18n("%1 MiB", locale.toString(size, 'f', 1));
+        case 0: return QObject::tr("%1 B").arg(size);
+        case 1: return QObject::tr("%1 KiB").arg(locale.toString(size, 'f', 1));
+        case 2: return QObject::tr("%1 MiB").arg(locale.toString(size, 'f', 1));
         default:
-        case 3: return i18n("%1 GiB", locale.toString(size, 'f', 1));
+        case 3: return QObject::tr("%1 GiB").arg(locale.toString(size, 'f', 1));
         }
     }
 }
@@ -563,7 +613,7 @@ QString Utils::findExe(const QString &appname, const QString &pstr)
     QStringList executable_extensions = executableExtensions();
     if (!executable_extensions.contains(appname.section(QLatin1Char('.'), -1, -1, QString::SectionIncludeLeadingSep), Qt::CaseInsensitive)) {
         QString found_exe;
-        foreach (const QString& extension, executable_extensions) {
+        for (const QString& extension: executable_extensions) {
             found_exe = findExe(appname + extension, pstr);
             if (!found_exe.isEmpty()) {
                 return found_exe;
@@ -588,8 +638,6 @@ QString Utils::findExe(const QString &appname, const QString &pstr)
 }
 // Copied from KDE... END
 
-#endif
-
 QString Utils::formatDuration(const quint32 totalseconds)
 {
     //Get the days,hours,minutes and seconds out of the total seconds
@@ -603,21 +651,15 @@ QString Utils::formatDuration(const quint32 totalseconds)
     //Convert hour,minutes and seconds to a QTime for easier parsing
     QTime time(hours, minutes, seconds);
 
-    #ifdef ENABLE_KDE_SUPPORT
-    return 0==days
-            ? time.toString("h:mm:ss")
-            : i18np("1 day %2", "%1 days %2", days, time.toString("h:mm:ss"));
-    #else
     return 0==days
             ? time.toString("h:mm:ss")
             : QString("%1:%2").arg(days).arg(time.toString("hh:mm:ss"));
-    #endif
 }
 
 QString Utils::formatTime(const quint32 seconds, bool zeroIsUnknown)
 {
     if (0==seconds && zeroIsUnknown) {
-        return i18n("Unknown");
+        return QObject::tr("Unknown");
     }
 
     static const quint32 constHour=60*60;
@@ -654,27 +696,19 @@ static QString userDir(const QString &mainDir, const QString &sub, bool create)
 
 QString Utils::dataDir(const QString &sub, bool create)
 {
-    #if defined Q_OS_WIN || defined Q_OS_MAC || defined ENABLE_UBUNTU
+    #if defined Q_OS_WIN || defined Q_OS_MAC
 
-    #if QT_VERSION >= 0x050000
     return userDir(QStandardPaths::writableLocation(QStandardPaths::DataLocation)+constDirSep, sub, create);
-    #else
-    return userDir(QDesktopServices::storageLocation(QDesktopServices::DataLocation)+constDirSep, sub, create);
-    #endif
 
     #else
 
     static QString location;
     if (location.isEmpty()) {
-        #if QT_VERSION >= 0x050000
         location=QStandardPaths::writableLocation(QStandardPaths::DataLocation);
-        location=location.replace(QCoreApplication::applicationName()+Utils::constDirSep+QCoreApplication::applicationName(),
-                                  QCoreApplication::applicationName());
-        #else
-        location=QDesktopServices::storageLocation(QDesktopServices::DataLocation);
-        location=location.replace(QLatin1String("data")+Utils::constDirSep+QCoreApplication::applicationName()+Utils::constDirSep+QCoreApplication::applicationName(),
-                                  QCoreApplication::applicationName());
-        #endif
+        if (QCoreApplication::organizationName()==QCoreApplication::applicationName()) {
+            location=location.replace(QCoreApplication::organizationName()+Utils::constDirSep+QCoreApplication::applicationName(),
+                                      QCoreApplication::applicationName());
+        }
     }
     return userDir(location+constDirSep, sub, create);
 
@@ -683,25 +717,19 @@ QString Utils::dataDir(const QString &sub, bool create)
 
 QString Utils::cacheDir(const QString &sub, bool create)
 {
-    #if defined Q_OS_WIN || defined Q_OS_MAC || defined ENABLE_UBUNTU
+    #if defined Q_OS_WIN || defined Q_OS_MAC
 
-    #if QT_VERSION >= 0x050000
     return userDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation)+constDirSep, sub, create);
-    #else
-    return userDir(QDesktopServices::storageLocation(QDesktopServices::CacheLocation)+constDirSep, sub, create);
-    #endif
 
     #else
 
     static QString location;
     if (location.isEmpty()) {
-        #if QT_VERSION >= 0x050000
         location=QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
-        #else
-        location=QDesktopServices::storageLocation(QDesktopServices::CacheLocation);
-        #endif
-        location=location.replace(QCoreApplication::applicationName()+Utils::constDirSep+QCoreApplication::applicationName(),
-                                  QCoreApplication::applicationName());
+        if (QCoreApplication::organizationName()==QCoreApplication::applicationName()) {
+            location=location.replace(QCoreApplication::organizationName()+Utils::constDirSep+QCoreApplication::applicationName(),
+                                      QCoreApplication::applicationName());
+        }
     }
     return userDir(location+constDirSep, sub, create);
 
@@ -713,20 +741,25 @@ QString Utils::systemDir(const QString &sub)
     #if defined Q_OS_WIN
     return fixPath(QCoreApplication::applicationDirPath())+(sub.isEmpty() ? QString() : (sub+constDirSep));
     #elif defined Q_OS_MAC
-    return fixPath(QCoreApplication::applicationDirPath())+QLatin1String("Contents/Resources/")+(sub.isEmpty() ? QString() : (sub+constDirSep));
+    return fixPath(QCoreApplication::applicationDirPath())+QLatin1String("../Resources/")+(sub.isEmpty() ? QString() : (sub+constDirSep));
     #else
-    return fixPath(QString(INSTALL_PREFIX "/share/")+QCoreApplication::applicationName()+constDirSep+(sub.isEmpty() ? QString() : sub));
+    return fixPath(QString(SHARE_INSTALL_PREFIX"/")+QCoreApplication::applicationName()+constDirSep+(sub.isEmpty() ? QString() : sub));
     #endif
 }
 
 QString Utils::helper(const QString &app)
 {
     #if defined Q_OS_WIN
-    return systemDir(QLatin1String("helpers"))+app+QLatin1String(".exe");
+    return fixPath(QCoreApplication::applicationDirPath())+app+QLatin1String(".exe");
     #elif defined Q_OS_MAC
-    return systemDir(QLatin1String("helpers"))+app;
+    return fixPath(QCoreApplication::applicationDirPath())+app;
     #else
-    return QString(INSTALL_PREFIX "/lib/")+QCoreApplication::applicationName()+constDirSep+app;
+    // Check for helpers in same folder as main exe, so that can test dev versions without install.
+    QString local = fixPath(QCoreApplication::applicationDirPath())+app;
+    if (QFile::exists(local)) {
+        return local;
+    }
+    return QString(INSTALL_PREFIX "/" LINUX_LIB_DIR "/")+QCoreApplication::applicationName()+constDirSep+app;
     #endif
 }
 
@@ -752,7 +785,7 @@ void Utils::moveDir(const QString &from, const QString &to)
     }
 
     QFileInfoList files=f.entryInfoList(QStringList() << "*", QDir::Files|QDir::Dirs|QDir::NoDotAndDotDot);
-    foreach (const QFileInfo &file, files) {
+    for (const QFileInfo &file: files) {
         if (file.isDir()) {
             QString dest=to+file.fileName()+constDirSep;
             if (!QDir(dest).exists()) {
@@ -784,7 +817,7 @@ void Utils::clearOldCache(const QString &sub, int maxAge)
         QFileInfoList files=dir.entryInfoList(QDir::Files|QDir::NoDotAndDotDot);
         if (files.count()) {
             QDateTime now=QDateTime::currentDateTime();
-            foreach (const QFileInfo &f, files) {
+            for (const QFileInfo &f: files) {
                 if (f.lastModified().daysTo(now)>maxAge) {
                     QFile::remove(f.absoluteFilePath());
                 }
@@ -795,26 +828,7 @@ void Utils::clearOldCache(const QString &sub, int maxAge)
 
 void Utils::touchFile(const QString &fileName)
 {
-    ::utime(QFile::encodeName(fileName).constData(), 0);
-}
-
-bool Utils::isDirReadable(const QString &dir)
-{
-    #ifdef Q_OS_WIN
-    if (dir.isEmpty()) {
-        return false;
-    } else {
-        QDir d(dir);
-        bool dirReadable=d.isReadable();
-        // Handle cases where dir is set to \\server\ (i.e. no shared folder is set in path)
-        if (!dirReadable && dir.startsWith(QLatin1String("//")) && d.isRoot() && (dir.length()-1)==dir.indexOf(Utils::constDirSep, 2)) {
-            dirReadable=true;
-        }
-        return dirReadable;
-    }
-    #else
-    return dir.isEmpty() ? false : QDir(dir).isReadable();
-    #endif
+    ::utime(QFile::encodeName(fileName).constData(), nullptr);
 }
 
 double Utils::smallFontFactor(const QFont &f)
@@ -844,18 +858,53 @@ int Utils::layoutSpacing(QWidget *w)
     return spacing;
 }
 
-bool Utils::isHighDpi()
+double Utils::screenDpiScale()
 {
-    static int fontHeight=-1;
-    if (-1==fontHeight) {
-        fontHeight=QApplication::fontMetrics().height();
+    static double scaleFactor=-1.0;
+    if (scaleFactor<0) {
+        QWidget *dw=QApplication::desktop();
+        if (!dw) {
+            return 1.0;
+        }
+        scaleFactor=dw->logicalDpiX()>120 ? qMin(qMax(dw->logicalDpiX()/96.0, 1.0), 4.0) : 1.0;
     }
-    return fontHeight>22;
+    return scaleFactor;
 }
 
-int Utils::scaleForDpi(int v)
+bool Utils::limitedHeight(QWidget *w)
 {
-    return isHighDpi() ? v*2 : v;
+    static bool init=false;
+    static bool limited=false;
+    if (!init) {
+        limited=!qgetenv("CANTATA_NETBOOK").isEmpty();
+        if (!limited) {
+            QDesktopWidget *dw=QApplication::desktop();
+            if (dw) {
+                limited=dw->availableGeometry(w).size().height()<=800;
+            }
+        }
+    }
+    return limited;
+}
+
+void Utils::resizeWindow(QWidget *w, bool preserveWidth, bool preserveHeight)
+{
+    QWidget *window=w ? w->window() : nullptr;
+    if (window) {
+        QSize was=window->size();
+        QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+        window->setMinimumSize(QSize(0, 0));
+        window->adjustSize();
+        QSize now=window->size();
+        window->setMinimumSize(now);
+        if (preserveWidth && preserveHeight) {
+            window->resize(qMax(was.width(), now.width()), qMax(was.height(), now.height()));
+        } else if (preserveWidth) {
+            window->resize(qMax(was.width(), now.width()), now.height());
+        } else if (preserveHeight) {
+            window->resize(now.width(), qMax(was.height(), now.height()));
+        }
+    }
 }
 
 Utils::Desktop Utils::currentDe()
@@ -864,12 +913,12 @@ Utils::Desktop Utils::currentDe()
     static int de=-1;
     if (-1==de) {
         de=Other;
-        QByteArray desktop=qgetenv("XDG_CURRENT_DESKTOP").toLower();
-        if ("unity"==desktop) {
+        QSet<QByteArray> desktop=qgetenv("XDG_CURRENT_DESKTOP").toLower().split(':').toSet();
+        if (desktop.contains("unity")) {
             de=Unity;
-        } else if ("kde"==desktop) {
+        } else if (desktop.contains("kde")) {
             de=KDE;
-        } else if ("gnome"==desktop || "pantheon"==desktop) {
+        } else if (desktop.contains("gnome") || desktop.contains("pantheon")) {
             de=Gnome;
         } else {
             QByteArray kde=qgetenv("KDE_FULL_SESSION");
@@ -883,15 +932,17 @@ Utils::Desktop Utils::currentDe()
     return Other;
 }
 
-static bool isTouchFriendly=false;
-void Utils::setTouchFriendly(bool t)
+bool Utils::useSystemTray()
 {
-    isTouchFriendly=t;
-}
-
-bool Utils::touchFriendly()
-{
-    return isTouchFriendly;
+    #if defined Q_OS_MAC
+    return false;
+    #elif defined Q_OS_WIN
+    return true;
+    #elif QT_QTDBUS_FOUND
+    return Gnome==currentDe() ? QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.StatusNotifierWatcher") : true;
+    #else
+    return Gnome!=currentDe();
+    #endif
 }
 
 QPainterPath Utils::buildPath(const QRectF &r, double radius)
@@ -906,3 +957,58 @@ QPainterPath Utils::buildPath(const QRectF &r, double radius)
     path.arcTo(r.x()+r.width()-diameter, r.y()+r.height()-diameter, diameter, diameter, 270, 90);
     return path;
 }
+
+QColor Utils::clampColor(const QColor &col)
+{
+    static const int constMin=64;
+    static const int constMax=240;
+
+    if (col.value()<constMin) {
+        return QColor(constMin, constMin, constMin);
+    } else if (col.value()>constMax) {
+        return QColor(constMax, constMax, constMax);
+    }
+    return col;
+}
+
+QColor Utils::monoIconColor()
+{
+    return clampColor(QApplication::palette().color(QPalette::Active, QPalette::WindowText));
+}
+
+#ifdef Q_OS_WIN
+// This is down here, because windows.h includes ALL windows stuff - and we get conflicts with MessageBox :-(
+#include <windows.h>
+#endif
+
+void Utils::raiseWindow(QWidget *w)
+{
+    if (!w) {
+        return;
+    }
+
+    #ifdef Q_OS_WIN
+    ::SetWindowPos(reinterpret_cast<HWND>(w->effectiveWinId()), HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
+    ::SetWindowPos(reinterpret_cast<HWND>(w->effectiveWinId()), HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
+    #elif !defined Q_OS_WIN
+    bool wasHidden=w->isHidden();
+    #endif
+
+    w->raise();
+    w->showNormal();
+    w->activateWindow();
+    #ifdef Q_OS_MAC
+    w->raise();
+    #endif
+    #if !defined Q_OS_WIN && !defined Q_OS_MAC
+    // This section seems to be required for compiz, so that MPRIS.Raise actually shows the window, and not just highlight launcher.
+    QString wmctrl=Utils::findExe(QLatin1String("wmctrl"));
+    if (!wmctrl.isEmpty()) {
+        if (wasHidden) {
+            QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+        }
+        QProcess::execute(wmctrl, QStringList() << QLatin1String("-i") << QLatin1String("-a") << QString::number(w->effectiveWinId()));
+    }
+    #endif
+}
+
