@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,6 +24,7 @@
 #ifndef UTILS_H
 #define UTILS_H
 
+#include "thread.h"
 #include <math.h>
 #include <sys/types.h>
 #include <QtGlobal>
@@ -32,16 +33,8 @@
 #include <QDir>
 #include <QFont>
 #include <QPainterPath>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KGlobal>
-#include <KDE/KLocale>
-#include <KDE/KStandardDirs>
-#endif
 #include <stdlib.h>
-#ifdef Q_OS_WIN
-#include <time.h>
-#endif
-#include <sys/time.h>
+#include <QTime>
 
 class QString;
 class QWidget;
@@ -59,40 +52,43 @@ namespace Utils
         return (fabs(d1 - d2) < precision);
     }
     inline int random(int max=0) { return max ? (qrand()%max) : qrand(); }
-    inline void initRand() { timeval time; gettimeofday(&time, NULL); qsrand((time.tv_sec*1000)+(time.tv_usec/1000)); }
+	inline void initRand() { QTime time = QTime::currentTime();	qsrand((time.second() * 1000) + (time.msec()));	}
+
+    extern QString fixPath(const QString &d, bool ensureEndsInSlash=true);
+    #ifdef Q_OS_WIN
+    inline QString homeToTilda(const QString &s) { return s; }
+    inline QString tildaToHome(const QString &s) { return s; }
+    #else
+    extern QString homeToTilda(const QString &s);
+    extern QString tildaToHome(const QString &s);
+    #endif
+
+    extern QString getDir(const QString &file);
+    extern QString getFile(const QString &file);
+    extern QString getExtension(const QString &file);
+    extern QString changeExtension(const QString &file, const QString &extension);
+    extern bool isDirReadable(const QString &dir);
+
+    inline void msleep(int msecs) { Thread::msleep(msecs); }
+    inline void sleep() { msleep(100); }
 
     extern QString strippedText(QString s);
     extern QString stripAcceleratorMarkers(QString label);
-    extern QString fixPath(const QString &d);
-    // Convert directory to a format suitable fo rUI - e.g. use native separators, and remove any traling separator
-    extern QString convertDirForDisplay(const QString &dir);
-    // Convert directory from a UI field - convert to / separators, and add a trailing separator
-    extern QString convertDirFromDisplay(const QString &dir);
-    extern QString getDir(const QString &file);
-    extern QString getFile(const QString &file);
-    extern QString changeExtension(const QString &file, const QString &extension);
+
+    // Convert path to a format suitable fo rUI - e.g. use native separators, and remove any trailing separator
+    extern QString convertPathForDisplay(const QString &dir, bool isFolder=true);
+    // Convert path from a UI field - convert to / separators, and add a trailing separator (if isFolder)
+    extern QString convertPathFromDisplay(const QString &dir, bool isFolder=true);
     #ifndef Q_OS_WIN
     extern gid_t getGroupId(const char *groupName="users"); // Return 0 if user is not in group, otherwise returns group ID
     #endif
     extern void setFilePerms(const QString &file, const char *groupName="users");
-    #ifdef ENABLE_KDE_SUPPORT
-    inline bool makeDir(const QString &dir, int mode) { return KStandardDirs::makeDir(dir, mode); }
-    #else
     extern bool makeDir(const QString &dir, int mode);
-    #endif
     extern bool createWorldReadableDir(const QString &dir, const QString &base, const char *groupName="users");
-    extern void msleep(int msecs);
-    inline void sleep() { msleep(100); }
 
-    #ifdef ENABLE_KDE_SUPPORT
-    inline QString findExe(const QString &appname, const QString &pathstr=QString()) { return KStandardDirs::findExe(appname, pathstr); }
-    inline QString formatByteSize(double size) { return KGlobal::locale()->formatByteSize(size, 1); }
-    inline QString formatNumber(double number, int precision) { return KGlobal::locale()->formatNumber(number, precision); }
-    #else
     extern QString findExe(const QString &appname, const QString &pathstr=QString());
     extern QString formatByteSize(double size);
     inline QString formatNumber(double number, int precision) { return QString::number(number, 'f', precision); }
-    #endif
     extern QString formatDuration(const quint32 totalseconds);
     extern QString formatTime(const quint32 seconds, bool zeroIsUnknown=false);
 
@@ -105,13 +101,16 @@ namespace Utils
     extern void moveDir(const QString &from, const QString &to);
     extern void clearOldCache(const QString &sub, int maxAge);
     extern void touchFile(const QString &fileName);
-    extern bool isDirReadable(const QString &dir);
 
     extern double smallFontFactor(const QFont &f);
     extern QFont smallFont(QFont f);
     extern int layoutSpacing(QWidget *w);
-    extern bool isHighDpi();
-    extern int scaleForDpi(int v);
+    extern double screenDpiScale();
+    inline bool isHighDpi() { return screenDpiScale()>1.35; }
+    inline int scaleForDpi(int v) { return qRound(screenDpiScale()*v); }
+    extern bool limitedHeight(QWidget *w);
+    extern void resizeWindow(QWidget *w, bool preserveWidth=true, bool preserveHeight=true);
+    extern void raiseWindow(QWidget *w);
 
     enum Desktop {
         KDE,
@@ -120,9 +119,10 @@ namespace Utils
         Other
     };
     extern Desktop currentDe();
-    extern void setTouchFriendly(bool t);
-    extern bool touchFriendly();
+    extern bool useSystemTray();
     extern QPainterPath buildPath(const QRectF &r, double radius);
+    extern QColor clampColor(const QColor &col);
+    extern QColor monoIconColor();
 }
 
 #endif
