@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,15 +24,14 @@
 #include "tageditor.h"
 #include "tags.h"
 #include "widgets/tagspinbox.h"
-#include "models/musiclibrarymodel.h"
-#include "mpd/mpdconnection.h"
+#include "mpd-interface/mpdconnection.h"
 #include "gui/settings.h"
 #include "support/messagebox.h"
 #include "support/inputdialog.h"
-#include "support/localize.h"
 #include "trackorganiser.h"
-#include "mpd/cuefile.h"
+#include "mpd-interface/cuefile.h"
 #include "support/utils.h"
+#include "support/monoicon.h"
 #ifdef ENABLE_DEVICES_SUPPORT
 #include "models/devicesmodel.h"
 #include "devices/device.h"
@@ -50,13 +49,13 @@
     w=0;
 
 //
-// NOTE: Cantata does NOT read, and store, comments from MPD. For comment support, Cantata will read these from the files when the tag editor is opened
-//       These are placed into the Song's 'name' field (as this is not currently used in this editor)
+// NOTE: Cantata does NOT read, and store, comments from MPD. For comment support, Cantata will read these from the
+// files when the tag editor is opened.
 //
 static bool equalTags(const Song &a, const Song &b, bool compareCommon, bool composerSupport, bool commentSupport)
 {
     return (compareCommon || a.track==b.track) && a.year==b.year && a.disc==b.disc &&
-           a.artist==b.artist && a.genre==b.genre && a.album==b.album &&
+           a.artist==b.artist && a.genres[0]==b.genres[0] && a.album==b.album &&
            a.albumartist==b.albumartist && (!composerSupport || a.composer()==b.composer()) &&
            (!commentSupport || a.comment()==b.comment()) && (compareCommon || a.title==b.title);
 }
@@ -77,6 +76,14 @@ static QString trim(QString str) {
     return str;
 }
 
+// Split genres back out
+static void splitGenres(Song &song) {
+    QStringList genres=song.genres[0].split(",", QString::SkipEmptyParts);
+    for (int i=0; i<Song::constNumGenres; ++i) {
+        song.genres[i]=i<genres.count() ? genres.at(i).trimmed() : QString();
+    }
+}
+
 static int iCount=0;
 
 int TagEditor::instanceCount()
@@ -84,6 +91,16 @@ int TagEditor::instanceCount()
     return iCount;
 }
 
+static inline bool nullRating(const quint8 &rating)
+{
+    return rating>Song::Rating_Max;
+}
+
+static inline bool nullRating(const Song &s)
+{
+    return nullRating(s.rating);
+}
+
 TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
                      const QSet<QString> &existingArtists, const QSet<QString> &existingAlbumArtists, const QSet<QString> &existingComposers,
                      const QSet<QString> &existingAlbums, const QSet<QString> &existingGenres, const QString &udi)
@@ -101,18 +118,21 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     , haveGenres(false)
     , haveDiscs(false)
     , haveYears(false)
+    , haveRatings(false)
     , saving(false)
     , composerSupport(false)
     , commentSupport(false)
+    , readRatingsAct(nullptr)
+    , writeRatingsAct(nullptr)
 {
     iCount++;
-    bool isMopidy=false;
+    bool ratingsSupport=false;
     #ifdef ENABLE_DEVICES_SUPPORT
     if (deviceUdi.isEmpty()) {
         baseDir=MPDConnection::self()->getDetails().dir;
         composerSupport=MPDConnection::self()->composerTagSupported();
         commentSupport=MPDConnection::self()->commentTagSupported();
-        isMopidy=MPDConnection::self()->isMopdidy();
+        ratingsSupport=MPDConnection::self()->stickersSupported();
     } else {
         Device *dev=getDevice(udi, parentWidget());
 
@@ -127,18 +147,28 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     baseDir=MPDConnection::self()->getDetails().dir;
     composerSupport=MPDConnection::self()->composerTagSupported();
     commentSupport=MPDConnection::self()->commentTagSupported();
-    isMopidy=MPDConnection::self()->isMopdidy();
+    ratingsSupport=MPDConnection::self()->stickersSupported();
     #endif
 
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         if (CueFile::isCue(s.file)) {
             continue;
         }
         Song song(s);
+        song.rating=Song::Rating_Null;
         if (s.guessed) {
             song.revertGuessedTags();
         }
+        // Store all Genres's in 1st genre
+        song.genres[0]=song.displayGenre();
+        song.genres[1]=QString();
         original.append(song);
+        if (s.isLocalFile()) {
+            ratingsSupport = false;
+            composerSupport = true;
+            commentSupport = false;
+            baseDir = QString();
+        }
     }
 
     if (original.isEmpty()) {
@@ -153,10 +183,18 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
 
     QWidget *mainWidet = new QWidget(this);
     setupUi(mainWidet);
-    if (isMopidy) {
-        connect(mopidyNote, SIGNAL(leftClickedUrl()), SLOT(showMopidyMessage()));
+    if (!ratingsSupport) {
+        REMOVE(ratingWidget);
+        REMOVE(ratingLabel);
+        REMOVE(ratingVarious);
+        REMOVE(ratingNoteLabel);
     } else {
-        REMOVE(mopidyNote);
+        connect(this, SIGNAL(getRating(QString)), MPDConnection::self(), SLOT(getRating(QString)));
+        connect(this, SIGNAL(setRating(QString,quint8)), MPDConnection::self(), SLOT(setRating(QString,quint8)));
+        connect(MPDConnection::self(), SIGNAL(rating(QString,quint8)), this, SLOT(rating(QString,quint8)));
+        ratingWidget->setShowZeroForNull(true);
+        QColor col=palette().color(QPalette::WindowText);
+        ratingVarious->setStyleSheet(QString("QLabel{color:rgba(%1,%2,%3,128);}").arg(col.red()).arg(col.green()).arg(col.blue()));
     }
     setMainWidget(mainWidet);
     ButtonCodes buttons=Ok|Cancel|Reset|User3;
@@ -164,8 +202,7 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         buttons|=User2|User1;
     }
     setButtons(buttons);
-    setCaption(i18n("Tags"));
-    progress->setVisible(false);
+    setCaption(tr("Tags"));
     if (songs.count()>1) {
         setButtonGuiItem(User2, StdGuiItem::back(true));
         setButtonGuiItem(User1,StdGuiItem::forward(true));
@@ -173,13 +210,19 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         enableButton(User2, false);
     }
     setButtonGuiItem(Ok, StdGuiItem::save());
-    setButtonGuiItem(User3, GuiItem(i18n("Tools"), "tools-wizard"));
+    setButtonGuiItem(User3, GuiItem(tr("Tools"), FontAwesome::magic));
     QMenu *toolsMenu=new QMenu(this);
-    toolsMenu->addAction(i18n("Apply \"Various Artists\" Workaround"), this, SLOT(applyVa()));
-    toolsMenu->addAction(i18n("Revert \"Various Artists\" Workaround"), this, SLOT(revertVa()));
-    toolsMenu->addAction(i18n("Set 'Album Artist' from 'Artist'"), this, SLOT(setAlbumArtistFromArtist()));
-    toolsMenu->addAction(i18n("Capitalize"), this, SLOT(capitalise()));
-    toolsMenu->addAction(i18n("Adjust Track Numbers"), this, SLOT(adjustTrackNumbers()));
+    toolsMenu->addAction(tr("Apply \"Various Artists\" Workaround"), this, SLOT(applyVa()));
+    toolsMenu->addAction(tr("Revert \"Various Artists\" Workaround"), this, SLOT(revertVa()));
+    toolsMenu->addAction(tr("Set 'Album Artist' from 'Artist'"), this, SLOT(setAlbumArtistFromArtist()));
+    toolsMenu->addAction(tr("Capitalize"), this, SLOT(capitalise()));
+    toolsMenu->addAction(tr("Adjust Track Numbers"), this, SLOT(adjustTrackNumbers()));
+    if (ratingsSupport) {
+        readRatingsAct=toolsMenu->addAction(tr("Read Ratings from File"), this, SLOT(readRatings()));
+        writeRatingsAct=toolsMenu->addAction(tr("Write Ratings to File"), this, SLOT(writeRatings()));
+        readRatingsAct->setEnabled(false);
+        writeRatingsAct->setEnabled(false);
+    }
     setButtonMenu(User3, toolsMenu, InstantPopup);
     enableButton(Ok, false);
     enableButton(Reset, false);
@@ -238,11 +281,13 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         QSet<int> songYears;
         QSet<int> songDiscs;
 
-        foreach (const Song &s, original) {
+        for (const Song &s: original) {
             songArtists.insert(s.artist);
             songAlbumArtists.insert(s.albumartist);
             songAlbums.insert(s.album);
-            songGenres.insert(s.genre);
+            for (int i=0; i<Song::constNumGenres && !s.genres[i].isEmpty(); ++i) {
+                songGenres.insert(s.genres[i]);
+            }
             if (composerSupport) {
                 songComposers.insert(s.composer());
             }
@@ -268,9 +313,10 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         all.setComment(1==songComments.count() ? *(songComments.begin()) : QString());
         all.albumartist=1==songAlbumArtists.count() ? *(songAlbumArtists.begin()) : QString();
         all.album=1==songAlbums.count() ? *(songAlbums.begin()) : QString();
-        all.genre=1==songGenres.count() ? *(songGenres.begin()) : QString();
+        all.genres[0]=1==songGenres.count() ? *(songGenres.begin()) : QString();
         all.year=1==songYears.count() ? *(songYears.begin()) : 0;
         all.disc=1==songDiscs.count() ? *(songDiscs.begin()) : 0;
+        all.rating=Song::Rating_Null;
         original.prepend(all);
         artist->setFocus();
         haveArtists=songArtists.count()>1;
@@ -285,13 +331,25 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     }
     edited=original;
     setIndex(0);
+
+    if (ratingsSupport || commentSupport) {
+        progress->setVisible(true);
+        progress->setRange(0, (original.count()>1 ? (original.count()-1) : 1)*(commentSupport && ratingsSupport ? 2 : 1));
+        progress->setValue(0);
+    } else {
+        progress->setVisible(false);
+    }
+
     bool first=original.count()>1;
-    foreach (const Song &s, original) {
+    for (const Song &s: original) {
         if (first) {
-            trackName->insertItem(trackName->count(), i18n("All tracks"));
+            trackName->insertItem(trackName->count(), tr("All tracks"));
             first=false;
         } else {
             trackName->insertItem(trackName->count(), s.filePath());
+            if (ratingsSupport) {
+                emit getRating(s.file);
+            }
         }
     }
     connect(title, SIGNAL(textChanged(const QString &)), SLOT(checkChanged()));
@@ -314,7 +372,10 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     connect(genre, SIGNAL(editTextChanged(const QString &)), SLOT(checkChanged()));
     connect(year, SIGNAL(valueChanged(int)), SLOT(checkChanged()));
     connect(trackName, SIGNAL(activated(int)), SLOT(setIndex(int)));
-    connect(this, SIGNAL(update()), MPDConnection::self(), SLOT(update()));
+    connect(this, SIGNAL(update()), MPDConnection::self(), SLOT(updateMaybe()));
+    if (ratingWidget) {
+        connect(ratingWidget, SIGNAL(valueChanged(int)), SLOT(checkRating()));
+    }
     adjustSize();
     int w=Utils::scaleForDpi(600);
     if (width()<w) {
@@ -323,6 +384,7 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     if (commentSupport) {
         QTimer::singleShot(0, this, SLOT(readComments()));
     }
+    trackName->lineEdit()->setReadOnly(true);
 }
 
 TagEditor::~TagEditor()
@@ -359,15 +421,18 @@ void TagEditor::fillSong(Song &s, bool isAll, bool skipEmpty) const
     if (!isAll || 0!=disc->value()) {
         s.disc=disc->value();
     }
-    setString(s.genre, trim(genre->text()), skipEmpty && (!haveAll || all.genre.isEmpty()));
+    setString(s.genres[0], trim(genre->text()), skipEmpty && (!haveAll || all.genres[0].isEmpty()));
     if (!isAll || 0!=year->value()) {
         s.year=year->value();
     }
+    if (ratingWidget) {
+        s.rating=ratingWidget->value();
+    }
 }
 
 void TagEditor::setVariousHint()
 {
-    if(0==currentSongIndex && original.count()>1) {
+    if (0==currentSongIndex && original.count()>1) {
         Song all=original.at(0);
         artist->setPlaceholderText(all.artist.isEmpty() && haveArtists ? TagSpinBox::variousStr() : QString());
         album->setPlaceholderText(all.album.isEmpty() && haveAlbums ? TagSpinBox::variousStr() : QString());
@@ -378,9 +443,12 @@ void TagEditor::setVariousHint()
         if (commentSupport) {
             comment->setPlaceholderText(all.comment().isEmpty() && haveComments ? TagSpinBox::variousStr() : QString());
         }
-        genre->setPlaceholderText(all.genre.isEmpty() && haveGenres ? TagSpinBox::variousStr() : QString());
+        genre->setPlaceholderText(all.genres[0].isEmpty() && haveGenres ? TagSpinBox::variousStr() : QString());
         disc->setVarious(0==all.disc && haveDiscs);
         year->setVarious(0==all.year && haveYears);
+        if (ratingVarious) {
+            ratingVarious->setVisible(nullRating(all) && haveRatings);
+        }
     } else {
         artist->setPlaceholderText(QString());
         album->setPlaceholderText(QString());
@@ -394,6 +462,9 @@ void TagEditor::setVariousHint()
         genre->setPlaceholderText(QString());
         disc->setVarious(false);
         year->setVarious(false);
+        if (ratingVarious) {
+            ratingVarious->setVisible(false);
+        }
     }
 }
 
@@ -423,19 +494,26 @@ void TagEditor::setLabelStates()
     albumLabel->setOn(o.album!=e.album);
     trackLabel->setOn(!isAll && o.track!=e.track);
     discLabel->setOn(o.disc!=e.disc);
-    genreLabel->setOn(o.genre!=e.genre);
+    genreLabel->setOn(o.genres[0]!=e.genres[0]);
     yearLabel->setOn(o.year!=e.year);
+    if (ratingLabel) {
+        ratingLabel->setOn(o.rating<=Song::Rating_Max &&
+                           e.rating<=Song::Rating_Max &&
+                           o.rating!=e.rating);
+    }
 }
 
 void TagEditor::readComments()
 {
-    progress->setVisible(true);
-    progress->setRange(0, original.count());
     bool haveMultiple=original.count()>1;
     bool updated=false;
+    bool multipleComments=false;
+    QString allComment;
 
     for (int i=0; i<original.count(); ++i) {
-        progress->setValue(i+1);
+        if (0!=i || !haveMultiple) {
+            progress->setValue(progress->value()+1);
+        }
         if (i && 0==i%10) {
             QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
         }
@@ -444,18 +522,32 @@ void TagEditor::readComments()
             continue;
         }
         Song song=original.at(i);
-        QString comment=Tags::readComment(baseDir+song.file);
+        QString comment=Tags::readComment(song.filePath(baseDir));
         if (!comment.isEmpty()) {
             song.setComment(comment);
             original.replace(i, song);
             haveComments=true;
             updated=true;
+            if (haveMultiple) {
+                if (allComment.isEmpty()) {
+                    allComment=comment;
+                } else if (comment!=allComment) {
+                    multipleComments=true;
+                }
+            }
         }
     }
     if (updated) {
         edited=original;
+        if (haveMultiple && !allComment.isEmpty() && !multipleComments) {
+            edited[0].setComment(allComment);
+            original[0].setComment(allComment);
+            if (0==currentSongIndex) {
+                comment->setText(allComment);
+            }
+        }
     }
-    progress->setVisible(false);
+    controlInitialActionsState();
     if (haveMultiple) {
         setVariousHint();
     } else {
@@ -467,12 +559,12 @@ void TagEditor::applyVa()
 {
     bool isAll=0==currentSongIndex && original.count()>1;
 
-    if (MessageBox::No==MessageBox::questionYesNo(this, (isAll ? i18n("Apply \"Various Artists\" workaround to <b>all</b> tracks?")
-                                                               : i18n("Apply \"Various Artists\" workaround?"))+
-                                                           QLatin1String("<br/><hr/><br/>")+
-                                                           i18n("<i>This will set 'Album artist' and 'Artist' to "
+    if (MessageBox::No==MessageBox::questionYesNo(this, (isAll ? tr("Apply \"Various Artists\" workaround to <b>all</b> tracks?")
+                                                               : tr("Apply \"Various Artists\" workaround?"))+
+                                                           QLatin1String("<br/><br/>")+
+                                                           tr("<i>This will set 'Album artist' and 'Artist' to "
                                                                 "\"Various Artists\", and set 'Title' to "
-                                                                "\"TrackArtist - TrackTitle\"</i>"), i18n("Apply \"Various Artists\" Workaround"),
+                                                                "\"TrackArtist - TrackTitle\"</i>"), tr("Apply \"Various Artists\" Workaround"),
                                                   StdGuiItem::apply(), StdGuiItem::cancel())) {
         return;
     }
@@ -509,16 +601,16 @@ void TagEditor::revertVa()
 {
     bool isAll=0==currentSongIndex && original.count()>1;
 
-    if (MessageBox::No==MessageBox::questionYesNo(this, (isAll ? i18n("Revert \"Various Artists\" workaround on <b>all</b> tracks?")
-                                                               : i18n("Revert \"Various Artists\" workaround"))+
-                                                           QLatin1String("<br/><hr/><br/>")+
-                                                           i18n("<i>Where the 'Album artist' is the same as 'Artist' "
+    if (MessageBox::No==MessageBox::questionYesNo(this, (isAll ? tr("Revert \"Various Artists\" workaround on <b>all</b> tracks?")
+                                                               : tr("Revert \"Various Artists\" workaround"))+
+                                                           QLatin1String("<br/><br/>")+
+                                                           tr("<i>Where the 'Album artist' is the same as 'Artist' "
                                                                 "and the 'Title' is of the format \"TrackArtist - TrackTitle\", "
                                                                 "'Artist' will be taken from 'Title' and 'Title' itself will be "
                                                                 "set to just the title. e.g. <br/><br/>"
                                                                 "If 'Title' is \"Wibble - Wobble\", then 'Artist' will be set to "
-                                                                "\"Wibble\" and 'Title' will be set to \"Wobble\"</i>"), i18n("Revert \"Various Artists\" Workaround"),
-                                                  GuiItem(i18n("Revert")), StdGuiItem::cancel())) {
+                                                                "\"Wibble\" and 'Title' will be set to \"Wobble\"</i>"), tr("Revert \"Various Artists\" Workaround"),
+                                                  GuiItem(tr("Revert")), StdGuiItem::cancel())) {
         return;
     }
 
@@ -559,9 +651,9 @@ void TagEditor::setAlbumArtistFromArtist()
 {
     bool isAll=0==currentSongIndex && original.count()>1;
 
-    if (MessageBox::No==MessageBox::questionYesNo(this, isAll ? i18n("Set 'Album Artist' from 'Artist' (if 'Album Artist' is empty) for <b>all</b> tracks?")
-                                                              : i18n("Set 'Album Artist' from 'Artist' (if 'Album Artist' is empty)?"),
-                                                  i18n("Album Artist from Artist"), StdGuiItem::apply(), StdGuiItem::cancel())) {
+    if (MessageBox::No==MessageBox::questionYesNo(this, isAll ? tr("Set 'Album Artist' from 'Artist' (if 'Album Artist' is empty) for <b>all</b> tracks?")
+                                                              : tr("Set 'Album Artist' from 'Artist' (if 'Album Artist' is empty)?"),
+                                                  tr("Album Artist from Artist"), StdGuiItem::apply(), StdGuiItem::cancel())) {
         return;
     }
 
@@ -594,11 +686,10 @@ void TagEditor::capitalise()
 {
     bool isAll=0==currentSongIndex && original.count()>1;
 
-    if (MessageBox::No==MessageBox::questionYesNo(this, isAll ? i18n("Capitalize the first letter of 'Title', 'Artist', 'Album artist', and "
-                                                                     "'Album' of <b>all</b> tracks?")
-                                                              : i18n("Capitalize the first letter of 'Title', 'Artist', 'Album artist', and "
-                                                                     "'Album'"),
-                                                  i18n("Capitalize"), GuiItem(i18n("Capitalize")), StdGuiItem::cancel())) {
+    if (MessageBox::No==MessageBox::questionYesNo(this, isAll ? tr("Capitalize the first letter of text fields (e.g. 'Title', 'Artist', etc) "
+                                                                     "of <b>all</b> tracks?")
+                                                              : tr("Capitalize the first letter of text fields (e.g. 'Title', 'Artist', etc)?"),
+                                                  tr("Capitalize"), GuiItem(tr("Capitalize")), StdGuiItem::cancel())) {
         return;
     }
 
@@ -627,8 +718,8 @@ void TagEditor::adjustTrackNumbers()
 {
     bool isAll=0==currentSongIndex && original.count()>1;
     bool ok=false;
-    int adj=InputDialog::getInteger(i18n("Adjust Track Numbers"), isAll ? i18n("Adjust the value of each track number by:")
-                                                                        : i18n("Adjust track number by:"),
+    int adj=InputDialog::getInteger(tr("Adjust Track Numbers"), isAll ? tr("Adjust the value of each track number by:")
+                                                                        : tr("Adjust track number by:"),
                                     0, -500, 500, 1, 10, &ok, this);
 
     if (!ok || 0==adj) {
@@ -654,6 +745,118 @@ void TagEditor::adjustTrackNumbers()
     }
 }
 
+void TagEditor::readRatings()
+{
+    bool isAll=0==currentSongIndex && original.count()>1;
+
+    if (MessageBox::No==MessageBox::questionYesNo(this, isAll ? tr("Read ratings for all tracks from the music files?")
+                                                              : tr("Read rating from music file?"),
+                                                  tr("Ratings"),
+                                                  isAll ? GuiItem(tr("Read Ratings")) : GuiItem(tr("Read Rating")),
+                                                  StdGuiItem::cancel())) {
+        return;
+    }
+
+    if (isAll) {
+        progress->setVisible(true);
+        progress->setRange(0, original.count());
+        QStringList updated;
+        for (int i=1; i<original.count(); ++i) {
+            progress->setValue(i+1);
+            if (i && 0==i%10) {
+                QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+            }
+            Song s=edited.at(i);
+            int r=Tags::readRating(s.filePath(baseDir));
+            if (r>=0 && r<=Song::Rating_Max && s.rating!=r) {
+                s.rating=r;
+                edited.replace(i, s);
+                updated.append(s.file);
+                editedIndexes.insert(i);
+                updateTrackName(i, true);
+                if (i==currentSongIndex) {
+                    setSong(s);
+                }
+            }
+        }
+        progress->setVisible(false);
+        if (!updated.isEmpty()) {
+            MessageBox::informationList(this, tr("Read, and updated, ratings from the following tracks:"), updated);
+        }
+    } else {
+        Song s=edited.at(currentSongIndex);
+        int r=Tags::readRating(s.filePath(baseDir));
+        if (r>=0 && r<=Song::Rating_Max && s.rating!=r) {
+            s.rating=r;
+            edited.replace(currentSongIndex, s);
+            setSong(s);
+        }
+    }
+    enableOkButton();
+}
+
+void TagEditor::writeRatings()
+{
+    bool isAll=0==currentSongIndex && original.count()>1;
+
+    if (isAll) {
+        for (int i=1; i<original.count(); ++i) {
+            if (nullRating(original.at(i))) {
+                MessageBox::error(this, tr("Not all Song ratings have been read from MPD!")+QLatin1String("\n\n")+
+                                  tr("Song ratings are not stored in the song files, but within MPD's 'sticker' database. "
+                                       "In order to save these into the actual file, Cantata must first read them from MPD."));
+                return;
+            }
+        }
+    } else {
+        if (nullRating(original.at(currentSongIndex))) {
+            MessageBox::error(this, tr("Song rating has not been read from MPD!")+QLatin1String("\n\n")+
+                              tr("Song ratings are not stored in the song files, but within MPD's 'sticker' database. "
+                                   "In order to save these into the actual file, Cantata must first read them from MPD."));
+            return;
+        }
+    }
+
+    if (MessageBox::No==MessageBox::questionYesNo(this, isAll ? tr("Write ratings for all tracks to the music files?")
+                                                              : tr("Write rating to music file?"),
+                                                  tr("Ratings"),
+                                                  isAll ? GuiItem(tr("Write Ratings")) : GuiItem(tr("Write Rating")),
+                                                  StdGuiItem::cancel())) {
+        return;
+    }
+
+    if (isAll) {
+        progress->setVisible(true);
+        progress->setRange(0, edited.count());
+        QStringList failed;
+        for (int i=1; i<edited.count(); ++i) {
+            progress->setValue(i+1);
+            if (i && 0==i%10) {
+                QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+            }
+            Song s=edited.at(i);
+            if (s.rating<=Song::Rating_Max) {
+                Tags::Update status=Tags::updateRating(s.filePath(baseDir), s.rating);
+                if (Tags::Update_Failed==status || Tags::Update_BadFile==status){
+                    failed.append(s.file);
+                }
+            }
+        }
+        progress->setVisible(false);
+        if (!failed.isEmpty()) {
+            MessageBox::errorList(this, tr("Failed to write ratings of the following tracks:"), failed);
+        }
+    } else {
+        Song s=edited.at(currentSongIndex);
+        if (s.rating<=Song::Rating_Max) {
+            Tags::Update status=Tags::updateRating(s.filePath(baseDir), s.rating);
+            if (Tags::Update_Failed==status || Tags::Update_BadFile==status){
+                MessageBox::error(this, tr("Failed to write rating to music file!"));
+            }
+        }
+    }
+}
+
 void TagEditor::checkChanged()
 {
     if (updating) {
@@ -685,13 +888,13 @@ void TagEditor::updateTrackName(int index, bool edited)
 
     if (edited) {
         if (isAll) {
-            trackName->setItemText(index, i18n("All tracks [modified]"));
+            trackName->setItemText(index, tr("All tracks [modified]"));
         } else {
-            trackName->setItemText(index, i18n("%1 [modified]", original.at(index).filePath()));
+            trackName->setItemText(index, tr("%1 [modified]").arg(original.at(index).filePath()));
         }
     } else {
         if (isAll) {
-            trackName->setItemText(index, i18n("All tracks"));
+            trackName->setItemText(index, tr("All tracks"));
         } else {
             trackName->setItemText(index, original.at(index).filePath());
         }
@@ -702,7 +905,7 @@ void TagEditor::updateEditedStatus(int index)
 {
     bool isAll=0==index && original.count()>1;
     Song s=edited.at(index);
-    if (equalTags(s, original.at(index), isAll, composerSupport, commentSupport)) {
+    if (equalTags(s, original.at(index), isAll, composerSupport, commentSupport) && s.rating==original.at(index).rating) {
         if (editedIndexes.contains(index)) {
             editedIndexes.remove(index);
             updateTrackName(index, false);
@@ -741,12 +944,12 @@ void TagEditor::updateEdited(bool isFromAll)
         if (all.album.isEmpty() && s.album.isEmpty() && !o.album.isEmpty()) {
             s.album=o.album;
         }
-        if (all.genre.isEmpty() && s.genre.isEmpty() && !o.genre.isEmpty()) {
-            s.genre=o.genre;
+        if (all.genres[0].isEmpty() && s.genres[0].isEmpty() && !o.genres[0].isEmpty()) {
+            s.genres[0]=o.genres[0];
         }
     }
 
-    if (equalTags(s, original.at(currentSongIndex), isFromAll || isAll, composerSupport, commentSupport)) {
+    if (equalTags(s, original.at(currentSongIndex), isFromAll || isAll, composerSupport, commentSupport) && s.rating==original.at(currentSongIndex).rating) {
         if (editedIndexes.contains(currentSongIndex)) {
             editedIndexes.remove(currentSongIndex);
             updateTrackName(currentSongIndex, false);
@@ -763,6 +966,7 @@ void TagEditor::updateEdited(bool isFromAll)
 
 void TagEditor::setSong(const Song &s)
 {
+    blockSignals(true);
     title->setText(s.title);
     artist->setText(s.artist);
     albumArtist->setText(s.albumartist);
@@ -775,8 +979,13 @@ void TagEditor::setSong(const Song &s)
     album->setText(s.album);
     track->setValue(s.track);
     disc->setValue(s.disc);
-    genre->setText(s.genre);
+    genre->setText(s.genres[0]);
     year->setValue(s.year);
+    if (ratingWidget) {
+        ratingWidget->setValue(s.rating);
+    }
+    blockSignals(false);
+    checkChanged();
 }
 
 void TagEditor::setIndex(int idx)
@@ -800,6 +1009,7 @@ void TagEditor::setIndex(int idx)
     titleLabel->setEnabled(!isMultiple);
     track->setEnabled(!isMultiple);
     trackLabel->setEnabled(!isMultiple);
+
     if (isMultiple) {
         title->setText(QString());
         track->setValue(0);
@@ -816,13 +1026,95 @@ void TagEditor::setIndex(int idx)
     updating=false;
 }
 
-void TagEditor::showMopidyMessage()
+void TagEditor::rating(const QString &f, quint8 r)
 {
-    MessageBox::information(this, i18n("Cantata has detected that you are connected to a Mopidy server.\n\n"
-                                       "Currently it is not possible for Cantata to force Mopidy to refresh its local "
-                                       "music listing. Therefore, you will need to stop Cantata, manually refresh "
-                                       "Mopidy's database, and restart Cantata for any changes to be active."),
-                            QLatin1String("Mopidy"));
+    if (!ratingWidget) {
+        return;
+    }
+    for (int i=original.count()>1 ? 1 : 0; i<original.count(); ++i) {
+        Song s=original.at(i);
+        if (nullRating(s) && s.rating!=r && s.file==f) {
+            progress->setValue(progress->value()+1);
+            controlInitialActionsState();
+            s.rating=r;
+            original.replace(i, s);
+            s=edited.at(i);
+            if (nullRating(s)) {
+                s.rating=r;
+                edited.replace(i, s);
+            }
+            if (i==currentSongIndex && ratingWidget->value()>Song::Rating_Max) {
+                ratingWidget->setValue(r);
+            }
+        }
+    }
+
+    if (original.count()>1 && !haveRatings) {
+        quint8 rating=Song::Rating_Null;
+        bool first=true;
+        for (int i=1; i<original.count() && !haveRatings; ++i) {
+            quint8 r=original.at(i).rating;
+            if (nullRating(r)) {
+                continue;
+            }
+            if (first) {
+                rating=r;
+                first=false;
+            } else if (r!=rating) {
+                haveRatings=true;
+                rating=Song::Rating_Null;
+            }
+        }
+        Song s=original.at(0);
+        if (s.rating!=rating) {
+            s.rating=rating;
+            original.replace(0, s);
+        }
+        s=edited.at(0);
+        if (s.rating!=rating) {
+            s.rating=rating;
+            edited.replace(0, s);
+        }
+        if (0==currentSongIndex) {
+            ratingWidget->setValue(rating);
+        }
+        setVariousHint();
+    }
+}
+
+void TagEditor::checkRating()
+{
+    if (!ratingWidget) {
+        return;
+    }
+    checkChanged();
+    if (original.count()>1 && 0==currentSongIndex) {
+        quint8 rating=Song::Rating_Null;
+        bool first=true;
+        haveRatings=false;
+        for (int i=1; i<edited.count() && !haveRatings; ++i) {
+            quint8 r=edited.at(i).rating;
+            if (nullRating(r)) {
+                continue;
+            }
+            if (first) {
+                rating=r;
+                first=false;
+            } else if (r!=rating) {
+                haveRatings=true;
+                rating=Song::Rating_Null;
+            }
+        }
+        Song s=edited.at(0);
+        if (s.rating!=rating) {
+            s.rating=rating;
+            edited.replace(0, s);
+            if (0==currentSongIndex) {
+                ratingWidget->setValue(rating);
+            }
+        }
+        setVariousHint();
+    }
 }
 
 bool TagEditor::applyUpdates()
@@ -832,7 +1124,7 @@ bool TagEditor::applyUpdates()
     DeviceOptions opts;
     QString udi;
     #ifdef ENABLE_DEVICES_SUPPORT
-    Device * dev=0;
+    Device * dev=nullptr;
     if (!deviceUdi.isEmpty()) {
         dev=getDevice(deviceUdi, this);
         if (!dev) {
@@ -860,7 +1152,8 @@ bool TagEditor::applyUpdates()
 
     int count=0;
     bool someTimedout=false;
-    foreach (int idx, editedIndexes) {
+    bool isLocal=false;
+    for (int idx: editedIndexes) {
         progress->setValue(progress->value()+1);
 
         if (0==count++%10) {
@@ -872,12 +1165,23 @@ bool TagEditor::applyUpdates()
         }
         Song orig=original.at(idx);
         Song edit=edited.at(idx);
+
+        if (orig.isLocalFile()) {
+            isLocal = true;
+        }
+        if (ratingWidget && orig.rating!=edit.rating && edit.rating<=Song::Rating_Max) {
+            emit setRating(orig.file, edit.rating);
+        }
+
         if (equalTags(orig, edit, false, composerSupport, commentSupport)) {
             continue;
         }
 
         QString file=orig.filePath();
-        switch(Tags::update(baseDir+file, orig, edit, -1, commentSupport)) {
+        QString afile=orig.filePath(baseDir);
+        splitGenres(orig);
+        splitGenres(edit);
+        switch(Tags::update(afile, orig, edit, -1, commentSupport)) {
         case Tags::Update_Modified:
             edit.setComment(QString());
             #ifdef ENABLE_DEVICES_SUPPORT
@@ -889,13 +1193,13 @@ bool TagEditor::applyUpdates()
             } else
             #endif
             {
-                if (!MusicLibraryModel::self()->updateSong(orig, edit)) {
-                    MusicLibraryModel::self()->removeSongFromList(orig);
-                    MusicLibraryModel::self()->addSongToList(edit);
-                }
+//                if (!MusicLibraryModel::self()->updateSong(orig, edit)) {
+//                    MusicLibraryModel::self()->removeSongFromList(orig);
+//                    MusicLibraryModel::self()->addSongToList(edit);
+//                }
             }
             updatedSongs.append(edit);
-            if (!renameFiles && file!=opts.createFilename(edit)) {
+            if (!renameFiles && !isLocal && file!=opts.createFilename(edit)) {
                 renameFiles=true;
             }
             break;
@@ -903,7 +1207,7 @@ bool TagEditor::applyUpdates()
             failed.append(file);
             break;
         case Tags::Update_BadFile:
-            failed.append(i18nc("filename (Corrupt tags?)", "%1 (Corrupt tags?)", file));
+            failed.append(tr("%1 (Corrupt tags?)", "filename (Corrupt tags?)").arg(file));
             break;
         default:
             break;
@@ -912,14 +1216,14 @@ bool TagEditor::applyUpdates()
     saving=false;
 
     if (failed.count()) {
-        MessageBox::errorListEx(this, i18n("Failed to update the tags of the following tracks:"), failed);
+        MessageBox::errorListEx(this, tr("Failed to update the tags of the following tracks:"), failed);
     }
 
     if (updatedSongs.count()) {
         // If we call tag-editor, no need to do MPD update - as this will be done from that dialog...
         if (renameFiles &&
-            MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Would you also like to rename your song files, so as to match your tags?"),
-                                                       i18n("Rename Files"), GuiItem(i18n("Rename")), StdGuiItem::cancel())) {
+            MessageBox::Yes==MessageBox::questionYesNo(this, tr("Would you also like to rename your song files, so as to match your tags?"),
+                                                       tr("Rename Files"), GuiItem(tr("Rename")), StdGuiItem::cancel())) {
             TrackOrganiser *dlg=new TrackOrganiser(parentWidget());
             dlg->show(updatedSongs, udi, true);
         } else {
@@ -928,7 +1232,7 @@ bool TagEditor::applyUpdates()
                 dev->saveCache();
             } else
             #endif
-            {
+            if (!isLocal) {
             //             MusicLibraryModel::self()->removeCache();
                 emit update();
             }
@@ -955,9 +1259,11 @@ void TagEditor::slotButtonClicked(int button)
             }
             editedIndexes.clear();
             setSong(original.at(currentSongIndex));
+            checkRating();
         } else {
             setSong(original.at(currentSongIndex));
         }
+        setLabelStates();
         enableOkButton();
         break;
     case User1: // Next
@@ -981,19 +1287,19 @@ Device * TagEditor::getDevice(const QString &udi, QWidget *p)
 {
     Device *dev=DevicesModel::self()->device(udi);
     if (!dev) {
-        MessageBox::error(p ? p : this, i18n("Device has been removed!"));
+        MessageBox::error(p ? p : this, tr("Device has been removed!"));
         reject();
-        return 0;
+        return nullptr;
     }
     if (!dev->isConnected()) {
-        MessageBox::error(p ? p : this, i18n("Device is not connected."));
+        MessageBox::error(p ? p : this, tr("Device is not connected."));
         reject();
-        return 0;
+        return nullptr;
     }
     if (!dev->isIdle()) {
-        MessageBox::error(p ? p : this, i18n("Device is busy?"));
+        MessageBox::error(p ? p : this, tr("Device is busy?"));
         reject();
-        return 0;
+        return nullptr;
     }
     return dev;
 }
@@ -1007,3 +1313,18 @@ void TagEditor::closeEvent(QCloseEvent *event)
         Dialog::closeEvent(event);
     }
 }
+
+void TagEditor::controlInitialActionsState()
+{
+    if (progress->value()>=progress->maximum()) {
+        progress->setVisible(false);
+        if (readRatingsAct) {
+            readRatingsAct->setEnabled(true);
+        }
+        if (writeRatingsAct) {
+            writeRatingsAct->setEnabled(true);
+        }
+    }
+}
+
+#include "moc_tageditor.cpp"
