@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -44,25 +44,28 @@ public:
     TagEditor(QWidget *parent, const QList<Song> &songs,
               const QSet<QString> &existingArtists, const QSet<QString> &existingAlbumArtists, const QSet<QString> &existingComposers,
               const QSet<QString> &existingAlbums, const QSet<QString> &existingGenres, const QString &udi);
-    virtual ~TagEditor();
+    ~TagEditor() override;
 
 Q_SIGNALS:
     // These are for communicating with MPD object (which is in its own thread, so need to talk via signal/slots)
     void update();
+    void getRating(const QString &f);
+    void setRating(const QString &f, quint8 r);
 
 private:
     void enableOkButton();
     void setLabelStates();
     void setVariousHint();
     void fillSong(Song &s, bool isAll, bool skipEmpty) const;
-    void slotButtonClicked(int button);
+    void slotButtonClicked(int button) override;
     void updateTrackName(int index, bool edited);
     void updateEditedStatus(int index);
     bool applyUpdates();
     #ifdef ENABLE_DEVICES_SUPPORT
     Device * getDevice(const QString &udi, QWidget *p);
     #endif
-    void closeEvent(QCloseEvent *event);
+    void closeEvent(QCloseEvent *event) override;
+    void controlInitialActionsState();
 
 private Q_SLOTS:
     void readComments();
@@ -72,10 +75,13 @@ private Q_SLOTS:
     void capitalise();
     void checkChanged();
     void adjustTrackNumbers();
+    void readRatings();
+    void writeRatings();
     void updateEdited(bool isFromAll=false);
     void setSong(const Song &s);
     void setIndex(int idx);
-    void showMopidyMessage();
+    void rating(const QString &f, quint8 r);
+    void checkRating();
 
 private:
     QString baseDir;
@@ -95,9 +101,12 @@ private:
     bool haveGenres;
     bool haveDiscs;
     bool haveYears;
+    bool haveRatings;
     bool saving;
     bool composerSupport;
     bool commentSupport;
+    QAction *readRatingsAct;
+    QAction *writeRatingsAct;
 };
 
 #endif
