@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -27,17 +27,15 @@
 #include "models/devicesmodel.h"
 #endif
 #include "devices/device.h"
-#include "models/musiclibrarymodel.h"
-#include "models/dirviewmodel.h"
 #include "gui/settings.h"
-#include "mpd/mpdconnection.h"
+#include "mpd-interface/mpdconnection.h"
 #include "support/utils.h"
 #include "context/songview.h"
-#include "support/localize.h"
 #include "support/messagebox.h"
+#include "support/action.h"
 #include "widgets/icons.h"
 #include "widgets/basicitemdelegate.h"
-#include "mpd/cuefile.h"
+#include "mpd-interface/cuefile.h"
 #include "gui/covers.h"
 #include "context/contextwidget.h"
 #include <QTimer>
@@ -58,7 +56,7 @@ int TrackOrganiser::instanceCount()
 
 TrackOrganiser::TrackOrganiser(QWidget *parent)
     : SongDialog(parent, "TrackOrganiser",  QSize(800, 500))
-    , schemeDlg(0)
+    , schemeDlg(nullptr)
     , autoSkip(false)
     , paused(false)
     , updated(false)
@@ -66,24 +64,24 @@ TrackOrganiser::TrackOrganiser(QWidget *parent)
 {
     iCount++;
     setButtons(Ok|Cancel);
-    setCaption(i18n("Organize Files"));
+    setCaption(tr("Organize Files"));
     setAttribute(Qt::WA_DeleteOnClose);
     QWidget *mainWidet = new QWidget(this);
     setupUi(mainWidet);
     setMainWidget(mainWidet);
     configFilename->setIcon(Icons::self()->configureIcon);
-    setButtonGuiItem(Ok, GuiItem(i18n("Rename"), "edit-rename"));
-    connect(this, SIGNAL(update()), MPDConnection::self(), SLOT(update()));
+    setButtonGuiItem(Ok, GuiItem(tr("Rename")));
+    connect(this, SIGNAL(update()), MPDConnection::self(), SLOT(updateMaybe()));
     progress->setVisible(false);
     files->setItemDelegate(new BasicItemDelegate(files));
     files->setAlternatingRowColors(false);
     files->setContextMenuPolicy(Qt::ActionsContextMenu);
     files->setSelectionMode(QAbstractItemView::ExtendedSelection);
-    removeAct=new Action(i18n("Remove From List"), files);
+    removeAct=new Action(tr("Remove From List"), files);
     removeAct->setEnabled(false);
     files->addAction(removeAct);
     connect(files, SIGNAL(itemSelectionChanged()), SLOT(controlRemoveAct()));
-    connect(removeAct, SIGNAL(triggered(bool)), SLOT(removeItems()));
+    connect(removeAct, SIGNAL(triggered()), SLOT(removeItems()));
 }
 
 TrackOrganiser::~TrackOrganiser()
@@ -96,7 +94,7 @@ void TrackOrganiser::show(const QList<Song> &songs, const QString &udi, bool for
     // If we are called from the TagEditor dialog, then forceUpdate will be true. This is so that we dont do 2
     // MPD updates (one from TagEditor, and one from here!)
     alwaysUpdate=forceUpdate;
-    foreach (const Song &s, songs) {
+    for (const Song &s: songs) {
         if (!CueFile::isCue(s.file)) {
            origSongs.append(s);
         }
@@ -111,12 +109,10 @@ void TrackOrganiser::show(const QList<Song> &songs, const QString &udi, bool for
     }
 
     QString musicFolder;
-    bool isMopidy=false;
     #ifdef ENABLE_DEVICES_SUPPORT
     if (udi.isEmpty()) {
         musicFolder=MPDConnection::self()->getDetails().dir;
         opts.load(MPDConnectionDetails::configGroupName(MPDConnection::self()->getDetails().name), true);
-        isMopidy=MPDConnection::self()->isMopdidy();
     } else {
         deviceUdi=udi;
         Device *dev=getDevice(parentWidget());
@@ -132,7 +128,6 @@ void TrackOrganiser::show(const QList<Song> &songs, const QString &udi, bool for
     #else
     opts.load(MPDConnectionDetails::configGroupName(MPDConnection::self()->getDetails().name), true);
     musicFolder=MPDConnection::self()->getDetails().dir;
-    isMopidy=MPDConnection::self()->isMopdidy();
     #endif
     qSort(origSongs);
 
@@ -152,11 +147,7 @@ void TrackOrganiser::show(const QList<Song> &songs, const QString &udi, bool for
     if (!songsOk(origSongs, musicFolder, udi.isEmpty())) {
         return;
     }
-    if (isMopidy) {
-        connect(mopidyNote, SIGNAL(leftClickedUrl()), SLOT(showMopidyMessage()));
-    } else {
-        REMOVE(mopidyNote);
-    }
+    connect(ratingsNote, SIGNAL(leftClickedUrl()), SLOT(showRatingsMessage()));
     Dialog::show();
     enableButtonOk(false);
     updateView();
@@ -171,7 +162,7 @@ void TrackOrganiser::slotButtonClicked(int button)
     case Cancel:
         if (!optionsBox->isEnabled()) {
             paused=true;
-            if (MessageBox::No==MessageBox::questionYesNo(this, i18n("Abort renaming of files?"), i18n("Abort"), GuiItem(i18n("Abort")), StdGuiItem::cancel())) {
+            if (MessageBox::No==MessageBox::questionYesNo(this, tr("Abort renaming of files?"), tr("Abort"), GuiItem(tr("Abort")), StdGuiItem::cancel())) {
                 paused=false;
                 QTimer::singleShot(0, this, SLOT(renameFile()));
                 return;
@@ -212,10 +203,23 @@ void TrackOrganiser::updateView()
     files->clear();
     bool different=false;
     readOptions();
-    foreach (const Song &s, origSongs) {
-        QString modified=opts.createFilename(s);
+
+    QString musicFolder;
+    #ifdef ENABLE_DEVICES_SUPPORT
+    if (!deviceUdi.isEmpty()) {
+        Device *dev=getDevice();
+        if (!dev) {
+            return;
+        }
+        musicFolder=dev->path();
+    } else
+    #endif
+        musicFolder=MPDConnection::self()->getDetails().dir;
+
+    for (const Song &s: origSongs) {
+        QString modified=musicFolder + opts.createFilename(s);
         //different=different||(modified!=s.file);
-        QString orig=s.filePath();
+        QString orig=s.filePath(musicFolder);
         bool diff=modified!=orig;
         different|=diff;
         QTreeWidgetItem *item=new QTreeWidgetItem(files, QStringList() << orig << modified);
@@ -267,17 +271,16 @@ void TrackOrganiser::renameFile()
     #endif
         musicFolder=MPDConnection::self()->getDetails().dir;
 
-    QString orig=s.filePath();
-    if (modified!=orig) {
-        QString source=musicFolder+orig;
-        QString dest=musicFolder+modified;
+    QString source=s.filePath(musicFolder);
+    QString dest=musicFolder+modified;
+    if (source!=dest) {
         bool skip=false;
         if (!QFile::exists(source)) {
             if (autoSkip) {
                 skip=true;
             } else {
-                switch(MessageBox::questionYesNoCancel(this, i18n("Source file does not exist!<br/>%1", dest),
-                                                       QString(), GuiItem(i18n("Skip")), GuiItem(i18n("Auto Skip")))) {
+                switch(MessageBox::questionYesNoCancel(this, tr("Source file does not exist!")+QLatin1String("\n\n")+dest,
+                                                       QString(), GuiItem(tr("Skip")), GuiItem(tr("Auto Skip")))) {
                 case MessageBox::Yes:
                     skip=true;
                     break;
@@ -295,8 +298,8 @@ void TrackOrganiser::renameFile()
             if (autoSkip) {
                 skip=true;
             } else {
-                switch(MessageBox::questionYesNoCancel(this, i18n("Destination file already exists!<br/>%1", dest),
-                                                       QString(), GuiItem(i18n("Skip")), GuiItem(i18n("Auto Skip")))) {
+                switch(MessageBox::questionYesNoCancel(this, tr("Destination file already exists!")+QLatin1String("\n\n")+dest,
+                                                       QString(), GuiItem(tr("Skip")), GuiItem(tr("Auto Skip")))) {
                 case MessageBox::Yes:
                     skip=true;
                     break;
@@ -317,8 +320,8 @@ void TrackOrganiser::renameFile()
                 if (autoSkip) {
                     skip=true;
                 } else {
-                    switch(MessageBox::questionYesNoCancel(this, i18n("Failed to create destination folder!<br/>%1", dir.absolutePath()),
-                                                           QString(), GuiItem(i18n("Skip")), GuiItem(i18n("Auto Skip")))) {
+                    switch(MessageBox::questionYesNoCancel(this, tr("Failed to create destination folder!")+QLatin1String("\n\n")+dir.absolutePath(),
+                                                           QString(), GuiItem(tr("Skip")), GuiItem(tr("Auto Skip")))) {
                     case MessageBox::Yes:
                         skip=true;
                         break;
@@ -338,8 +341,8 @@ void TrackOrganiser::renameFile()
             if (autoSkip) {
                 skip=true;
             } else {
-                switch(MessageBox::questionYesNoCancel(this, i18n("Failed to rename %1 to %2", source, dest),
-                                                       QString(), GuiItem(i18n("Skip")), GuiItem(i18n("Auto Skip")))) {
+                switch(MessageBox::questionYesNoCancel(this, tr("Failed to rename '%1' to '%2'").arg(source, dest),
+                                                       QString(), GuiItem(tr("Skip")), GuiItem(tr("Auto Skip")))) {
                 case MessageBox::Yes:
                     skip=true;
                     break;
@@ -353,22 +356,23 @@ void TrackOrganiser::renameFile()
             }
         }
 
-        // If file was renamed, then also rename any lyrics file...
+        // If file was renamed, then also rename any other matching files...
+        QDir sDir(Utils::getDir(source));
         if (renamed) {
-            QString lyricsSource=Utils::changeExtension(source, SongView::constExtension);
-            QString lyricsDest=Utils::changeExtension(dest, SongView::constExtension);
-
-            if (QFile::exists(lyricsSource) && !QFile::exists(lyricsDest)) {
-                QFile::rename(lyricsSource, lyricsDest);
+            QFileInfoList files = sDir.entryInfoList(QStringList() << Utils::changeExtension(Utils::getFile(source), ".*"));
+            for (const auto &file : files) {
+                QString destFile = Utils::changeExtension(dest, "."+file.suffix());
+                if (!QFile::exists(destFile)) {
+                    QFile::rename(file.absoluteFilePath(), destFile);
+                }
             }
         }
 
         if (!skip) {
-            QDir sDir(Utils::getDir(source));
             QDir sArtistDir(sDir); sArtistDir.cdUp();
             QDir dDir(Utils::getDir(dest));
             #ifdef ENABLE_DEVICES_SUPPORT
-            Device *dev=deviceUdi.isEmpty() ? 0 : getDevice();
+            Device *dev=deviceUdi.isEmpty() ? nullptr : getDevice();
             if (sDir.absolutePath()!=dDir.absolutePath()) {
                 Device::moveDir(sDir.absolutePath(), dDir.absolutePath(), musicFolder, dev ? dev->coverFile()
                                                                                            : QString(Covers::albumFileName(s)+QLatin1String(".jpg")));
@@ -384,12 +388,14 @@ void TrackOrganiser::renameFile()
             if (sArtistDir.exists() && dArtistDir.exists() && sArtistDir.absolutePath()!=sDir.absolutePath() && sArtistDir.absolutePath()!=dArtistDir.absolutePath()) {
                 QStringList artistImages;
                 QFileInfoList entries=sArtistDir.entryInfoList(QDir::Files|QDir::Dirs|QDir::NoDotAndDotDot);
-                QString artistImage=Covers::artistFileName(s);
-                QSet<QString> acceptable=QSet<QString>() << artistImage+QLatin1String(".jpg") << artistImage+QLatin1String(".png")
+                QSet<QString> acceptable=QSet<QString>() << Covers::constArtistImage+QLatin1String(".jpg")
+                                                         << Covers::constArtistImage+QLatin1String(".png")
+                                                         << Covers::constComposerImage+QLatin1String(".jpg")
+                                                         << Covers::constComposerImage+QLatin1String(".png")
                                                          << ContextWidget::constBackdropFileName+QLatin1String(".jpg")
                                                          << ContextWidget::constBackdropFileName+QLatin1String(".png");
 
-                foreach (const QFileInfo &entry, entries) {
+                for (const QFileInfo &entry: entries) {
                     if (entry.isDir() || !acceptable.contains(entry.fileName())) {
                         artistImages.clear();
                         break;
@@ -399,7 +405,7 @@ void TrackOrganiser::renameFile()
                 }
                 if (!artistImages.isEmpty()) {
                     bool delDir=true;
-                    foreach (const QString &f, artistImages) {
+                    for (const QString &f: artistImages) {
                         if (!QFile::rename(sArtistDir.absolutePath()+Utils::constDirSep+f, dArtistDir.absolutePath()+Utils::constDirSep+f)) {
                             delDir=false;
                             break;
@@ -414,7 +420,7 @@ void TrackOrganiser::renameFile()
                     }
                 }
             }
-            item->setText(0, modified);
+            item->setText(0, dest);
             item->setFont(0, font());
             item->setFont(1, font());
             Song to=s;
@@ -422,6 +428,8 @@ void TrackOrganiser::renameFile()
             if (s.file.startsWith(Song::constMopidyLocal)) {
                 origPath=to.file;
                 to.file=Song::encodePath(to.file);
+            } else if (MPDConnection::self()->isForkedDaapd()) {
+                to.file=Song::constForkedDaapdLocal + dest;
             } else {
                 to.file=modified;
             }
@@ -429,10 +437,10 @@ void TrackOrganiser::renameFile()
             updated=true;
 
             if (deviceUdi.isEmpty()) {
-                MusicLibraryModel::self()->updateSongFile(s, to);
-                DirViewModel::self()->removeFileFromList(s.file);
-                DirViewModel::self()->addFileToList(origPath.isEmpty() ? to.file : origPath,
-                                                    origPath.isEmpty() ? QString() : to.file);
+//                MusicLibraryModel::self()->updateSongFile(s, to);
+//                DirViewModel::self()->removeFileFromList(s.file);
+//                DirViewModel::self()->addFileToList(origPath.isEmpty() ? to.file : origPath,
+//                                                    origPath.isEmpty() ? QString() : to.file);
             }
             #ifdef ENABLE_DEVICES_SUPPORT
             else {
@@ -463,11 +471,11 @@ void TrackOrganiser::removeItems()
         return;
     }
 
-    if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Remove the selected tracks from the list?"),
-                                                   i18n("Remove Tracks"), StdGuiItem::remove(), StdGuiItem::cancel())) {
+    if (MessageBox::Yes==MessageBox::questionYesNo(this, tr("Remove the selected tracks from the list?"),
+                                                   tr("Remove Tracks"), StdGuiItem::remove(), StdGuiItem::cancel())) {
 
         QList<QTreeWidgetItem *> selection=files->selectedItems();
-        foreach (QTreeWidgetItem *item, selection) {
+        for (QTreeWidgetItem *item: selection) {
             int idx=files->indexOfTopLevelItem(item);
             if (idx>-1 && idx<origSongs.count()) {
                 origSongs.removeAt(idx);
@@ -477,13 +485,11 @@ void TrackOrganiser::removeItems()
     }
 }
 
-void TrackOrganiser::showMopidyMessage()
+void TrackOrganiser::showRatingsMessage()
 {
-    MessageBox::information(this, i18n("Cantata has detected that you are connected to a Mopidy server.\n\n"
-                                       "Currently it is not possible for Cantata to force Mopidy to refresh its local "
-                                       "music listing. Therefore, you will need to stop Cantata, manually refresh "
-                                       "Mopidy's database, and restart Cantata for any changes to be active."),
-                            QLatin1String("Mopidy"));
+    MessageBox::information(this, tr("Song ratings are not stored in the song files, but within MPD's 'sticker' database.\n\n"
+                                       "If you rename a file (or the folder it is within), then the rating associated with the song will be lost."),
+                            QLatin1String("Ratings"));
 }
 
 void TrackOrganiser::setFilenameScheme(const QString &text)
@@ -541,21 +547,22 @@ Device * TrackOrganiser::getDevice(QWidget *p)
 {
     Device *dev=DevicesModel::self()->device(deviceUdi);
     if (!dev) {
-        MessageBox::error(p ? p : this, i18n("Device has been removed!"));
+        MessageBox::error(p ? p : this, tr("Device has been removed!"));
         reject();
-        return 0;
+        return nullptr;
     }
     if (!dev->isConnected()) {
-        MessageBox::error(p ? p : this, i18n("Device is not connected."));
+        MessageBox::error(p ? p : this, tr("Device is not connected."));
         reject();
-        return 0;
+        return nullptr;
     }
     if (!dev->isIdle()) {
-        MessageBox::error(p ? p : this, i18n("Device is busy?"));
+        MessageBox::error(p ? p : this, tr("Device is busy?"));
         reject();
-        return 0;
+        return nullptr;
     }
     return dev;
 }
 #endif
 
+#include "moc_trackorganiser.cpp"
