@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -44,7 +44,7 @@ public:
     static int instanceCount();
 
     TrackOrganiser(QWidget *parent);
-    virtual ~TrackOrganiser();
+    ~TrackOrganiser() override;
 
     void show(const QList<Song> &songs, const QString &udi, bool forceUpdate=false);
 
@@ -59,15 +59,15 @@ private Q_SLOTS:
     void renameFile();
     void controlRemoveAct();
     void removeItems();
-    void showMopidyMessage();
+    void showRatingsMessage();
     void setFilenameScheme(const QString &text);
 
 private:
     void saveOptions();
-    void slotButtonClicked(int button);
+    void slotButtonClicked(int button) override;
     void readOptions();
     #ifdef ENABLE_DEVICES_SUPPORT
-    Device * getDevice(QWidget *p=0);
+    Device * getDevice(QWidget *p=nullptr);
     #endif
     void doUpdate();
     void finish(bool ok);
