@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -51,8 +51,8 @@ void ActionItemDelegate::setup()
         constBorder=constActionIconSize>22 ? 2 : 1;
         constActionBorder=constActionIconSize>32 ? 6 : 4;
     } else {
-        constActionBorder=Utils::touchFriendly() ? 6 : 4;
-        constActionIconSize=Utils::touchFriendly() ? 22 : 16;
+        constActionBorder=4;
+        constActionIconSize=16;
         constLargeActionIconSize=22;
         constBorder=1;
     }
@@ -65,15 +65,15 @@ QRect ActionItemDelegate::calcActionRect(bool rtl, ActionPos actionPos, const QR
     QRect rect=AP_HBottom==actionPos ? QRect(o.x(), o.y()+(o.height()/2), o.width(), o.height()/2) : o;
     return rtl
                 ? AP_VTop==actionPos
-                    ? QRect(rect.x()+(constBorder*4),
-                            rect.y()+(constBorder*4),
+                    ? QRect(rect.x()+(constBorder*4)+4,
+                            rect.y()+(constBorder*4)+4,
                             iconSize, iconSize)
                     : QRect(rect.x()+constActionBorder,
                             rect.y()+((rect.height()-iconSize)/2),
                             iconSize, iconSize)
                 : AP_VTop==actionPos
-                    ? QRect(rect.x()+rect.width()-(iconSize+(constBorder*4)),
-                            rect.y()+(constBorder*4),
+                    ? QRect(rect.x()+rect.width()-(iconSize+(constBorder*4))-4,
+                            rect.y()+(constBorder*4)+4,
                             iconSize, iconSize)
                     : QRect(rect.x()+rect.width()-(iconSize+constActionBorder),
                             rect.y()+((rect.height()-iconSize)/2),
@@ -97,6 +97,21 @@ void ActionItemDelegate::adjustActionRect(bool rtl, ActionPos actionPos, QRect &
     }
 }
 
+static void drawBgnd(QPainter *painter, const QRect &rx, bool light)
+{
+    QRectF r(rx.x()-0.5, rx.y()-0.5, rx.width()+1, rx.height()+1);
+    QPainterPath p(Utils::buildPath(r, 1.0));
+    QColor c(light ? Qt::white : Qt::black);
+
+    painter->setRenderHint(QPainter::Antialiasing, true);
+    c.setAlphaF(0.75);
+    painter->fillPath(p, c);
+    c.setAlphaF(0.95);
+    painter->setPen(c);
+    painter->drawPath(p);
+    painter->setRenderHint(QPainter::Antialiasing, false);
+}
+
 ActionItemDelegate::ActionItemDelegate(QObject *p)
     : QStyledItemDelegate(p)
     , largeIcons(false)
@@ -106,21 +121,28 @@ ActionItemDelegate::ActionItemDelegate(QObject *p)
 
 void ActionItemDelegate::drawIcons(QPainter *painter, const QRect &r, bool mouseOver, bool rtl, ActionPos actionPos, const QModelIndex &index) const
 {
+    QColor textCol=QApplication::palette().color(QPalette::Normal, QPalette::WindowText);
+    bool lightBgnd=textCol.red()<=128 && textCol.green()<=128 && textCol.blue()<=128;
     int iconSize=largeIcons ? constLargeActionIconSize : constActionIconSize;
     double opacity=painter->opacity();
-    bool touch=Utils::touchFriendly();
-    bool adjustOpacity=!mouseOver && !(touch && AP_VTop==actionPos);
+    bool adjustOpacity=!mouseOver;
     if (adjustOpacity) {
         painter->setOpacity(opacity*0.25);
     }
     QRect actionRect=calcActionRect(rtl, actionPos, r);
     QList<Action *> actions=index.data(Cantata::Role_Actions).value<QList<Action *> >();
 
-    foreach (const QPointer<Action> &a, actions) {
+    for (const QPointer<Action> &a: actions) {
         QPixmap pix=a->icon().pixmap(QSize(iconSize, iconSize));
-        if (!pix.isNull() && actionRect.width()>=pix.width()/* && r.x()>=0 && r.y()>=0*/) {
-            painter->drawPixmap(actionRect.x()+(actionRect.width()-pix.width())/2,
-                                actionRect.y()+(actionRect.height()-pix.height())/2, pix);
+        QSize pixSize = pix.isNull() ? QSize(0, 0) : (pix.size() / pix.DEVICE_PIXEL_RATIO());
+
+        if (!pix.isNull() && actionRect.width()>=pixSize.width()/* && r.x()>=0 && r.y()>=0*/) {
+            drawBgnd(painter, actionRect, lightBgnd);
+            painter->drawPixmap(actionRect.x()+(actionRect.width()-pixSize.width())/2,
+                                actionRect.y()+(actionRect.height()-pixSize.height())/2, pix);
+        }
+        if (largeIcons && 2==actions.count() && AP_VTop==actionPos) {
+            adjustActionRect(rtl, actionPos, actionRect, iconSize>>4);
         }
         adjustActionRect(rtl, actionPos, actionRect, iconSize);
     }
@@ -142,20 +164,22 @@ bool ActionItemDelegate::helpEvent(QHelpEvent *e, QAbstractItemView *view, const
     return QStyledItemDelegate::helpEvent(e, view, option, index);
 }
 
-QAction * ActionItemDelegate::getAction(const QModelIndex &index) const
+QAction * ActionItemDelegate::getAction(const QModelIndex &index, int adjust) const
 {
     QList<Action *> actions=index.data(Cantata::Role_Actions).value<QList<Action *> >();
     if (actions.isEmpty()) {
-        return 0;
+        return nullptr;
     }
 
     QAbstractItemView *view=(QAbstractItemView *)parent();
     bool rtl = QApplication::isRightToLeft();
     QListView *lv=qobject_cast<QListView *>(view);
-    GroupedView *gv=lv ? 0 : qobject_cast<GroupedView *>(view);
-    ActionPos actionPos=gv ? AP_HBottom : (lv && QListView::ListMode!=lv->viewMode() && index.child(0, 0).isValid() ? AP_VTop : AP_HMiddle);
+    GroupedView *gv=lv ? nullptr : qobject_cast<GroupedView *>(view);
+    ActionPos actionPos=gv ? AP_HBottom : (lv && QListView::ListMode!=lv->viewMode() && (index.child(0, 0).isValid() || index.model()->canFetchMore(index)) ? AP_VTop : AP_HMiddle);
     QRect rect = view->visualRect(index);
     rect.moveTo(view->viewport()->mapToGlobal(QPoint(rect.x(), rect.y())));
+    // Adjust position side to take into account the fact that layout is dynamic
+    rect.adjust(0, 0, adjust, 0);
     bool showCapacity = !index.data(Cantata::Role_CapacityText).toString().isEmpty();
     if (gv || lv || showCapacity) {
         if (AP_VTop==actionPos) {
@@ -176,14 +200,19 @@ QAction * ActionItemDelegate::getAction(const QModelIndex &index) const
     ActionItemDelegate::adjustActionRect(rtl, actionPos, actionRect2, iconSize);
     QPoint cursorPos=QCursor::pos();
 
-    foreach (const QPointer<Action> &a, actions) {
+    for (const QPointer<Action> &a: actions) {
         actionRect=actionPos ? actionRect.adjusted(0, -2, 0, 2) : actionRect.adjusted(-2, 0, 2, 0);
         if (actionRect.contains(cursorPos)) {
             return a;
         }
 
+        if (largeIcons && 2==actions.count() && AP_VTop==actionPos) {
+            adjustActionRect(rtl, actionPos, actionRect, iconSize>>4);
+        }
         ActionItemDelegate::adjustActionRect(rtl, actionPos, actionRect, iconSize);
     }
 
-    return 0;
+    return nullptr;
 }
+
+#include "moc_actionitemdelegate.cpp"
