@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  * This file (c) 2012 Piotr Wicijowski <piotr.wicijowski@gmail.com>
  *
  * ----
@@ -23,7 +23,6 @@
  */
 
 #include "autohidingsplitter.h"
-#include "support/gtkstyle.h"
 #include "support/utils.h"
 #include <QSplitterHandle>
 #include <QTimer>
@@ -39,15 +38,18 @@ static int splitterSize(const QWidget *w)
     static int size=-1;
 
     if (-1==size || !w || !w->isVisible()) {
-        if (Utils::touchFriendly()) {
-            size=4;
-        } else if (qApp->style()->styleHint(QStyle::SH_ScrollView_FrameOnlyAroundContents)) {
+        #if defined Q_OS_MAC || defined Q_OS_WIN
+        size=0;
+        #else
+        size=1;
+        if (qApp->style()->styleHint(QStyle::SH_ScrollView_FrameOnlyAroundContents)) {
             int spacing=qApp->style()->pixelMetric(QStyle::PM_ScrollView_ScrollBarSpacing);
-            int splitterSize=qApp->style()->pixelMetric(QStyle::PM_SplitterWidth);
-            size=qMin(spacing+2, splitterSize);
-        } else {
-            size=GtkStyle::isActive() ? 0 : 1;
+            if (spacing>0) { // Kvantum has -ve spacing!
+                int splitterSize=qApp->style()->pixelMetric(QStyle::PM_SplitterWidth);
+                size=qMin(spacing+2, splitterSize);
+            }
         }
+        #endif
     }
     return size;
 }
@@ -63,7 +65,7 @@ class SplitterSizeAnimation:public QVariantAnimation
 public:
     SplitterSizeAnimation(QObject *parent)
         : QVariantAnimation(parent)
-        , splitter(0)
+        , splitter(nullptr)
     {
     }
     void setSplitter(QSplitter *splitter) {
@@ -71,7 +73,7 @@ public:
     }
 
 protected:
-    virtual QVariant interpolated(const QVariant &from, const QVariant &to, qreal progress) const {
+    QVariant interpolated(const QVariant &from, const QVariant &to, qreal progress) const override {
         QList<int> fromInt = from.value<QList<int> >();
         QList<int> toInt = to.value<QList<int> >();
         QList<int> returnValue;
@@ -80,7 +82,7 @@ protected:
         }
         return QVariant::fromValue(returnValue);
     }
-    virtual void updateCurrentValue(const QVariant &value) {
+    void updateCurrentValue(const QVariant &value) override {
         if (splitter) {
             splitter->setSizes(value.value<QList<int> >());
         }
@@ -115,12 +117,12 @@ AutohidingSplitter::AutohidingSplitter(QWidget *parent)
     autohideAnimation->setEasingCurve(QEasingCurve::Linear);
     //connect(this, SIGNAL(splitterMoved(int, int)), this, SLOT(updateAfterSplitterMoved(int, int)));
     setMinimumWidth(32);
-    setHandleWidth(Utils::touchFriendly() ? 4 : 1);
+    setHandleWidth(1);
 }
 
 AutohidingSplitter::~AutohidingSplitter()
 {
-    foreach(QTimer *sat, animationDelayTimer) {
+    for (QTimer *sat: animationDelayTimer) {
         sat->stop();
         delete sat;
     }
@@ -144,7 +146,7 @@ void AutohidingSplitter::addChild(QObject *pObject)
         pObject->installEventFilter(this);
 
         const QObjectList &childList = pObject->children();
-        foreach (QObject *obj, childList) {
+        for  (QObject *obj: childList) {
             addChild(obj);
         }
     }
@@ -160,7 +162,7 @@ void AutohidingSplitter::removeChild(QObject *pObject)
     if (pObject && (pObject->isWidgetType())) {
         pObject->removeEventFilter(this);
         const QObjectList& childList = pObject->children();
-        foreach (QObject *obj, childList) {
+        for (QObject *obj: childList) {
             removeChild(obj);
         }
     }
@@ -220,7 +222,7 @@ bool AutohidingSplitter::eventFilter(QObject *target, QEvent *e)
         break;
     case QEvent::MouseButtonPress:
         if(qobject_cast<AutohidingSplitterHandle *>(target)){
-            foreach(QTimer * timer, animationDelayTimer) {
+            for (QTimer *timer: animationDelayTimer) {
                 timer->stop();
             }
             haltModifications = true;
@@ -280,7 +282,7 @@ void AutohidingSplitter::resizeEvent(QResizeEvent *event)
     if (autoHideEnabled && !haltModifications && event->oldSize().width() > 0) {
 //        int oldUsableSize = event->oldSize().width()/*-(count()-1)*handleWidth()*/;
         int oldUsableSize = 0;
-        foreach(int size, expandedSizes) {
+        for (int size: expandedSizes) {
             oldUsableSize+=size;
         }
         int newUsableSize = event->size().width()/*-(count()-1)*handleWidth()*/;
@@ -500,7 +502,7 @@ void AutohidingSplitter::setWidgetForHiding()
 
 void AutohidingSplitter::updateAfterSplitterMoved(int pos, int index)
 {
-    Q_UNUSED(pos);
+    Q_UNUSED(pos)
     if (!autoHideEnabled || index<=0 || index>count()) {
         return;
     }
@@ -509,3 +511,5 @@ void AutohidingSplitter::updateAfterSplitterMoved(int pos, int index)
     expandedSizes[index-1]+=currentTemporarySizes.at(index-1)-previousTemporarySizes.at(index-1);
     expandedSizes[index]+=currentTemporarySizes.at(index)-previousTemporarySizes.at(index);
 }
+
+#include "moc_autohidingsplitter.cpp"
