@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -29,13 +29,13 @@
 class BasicItemDelegate : public QStyledItemDelegate
 {
 public:
-    static void drawLine(QPainter *p, const QRect &r, const QColor &color, bool fadeStart=true, bool fadeEnd=true);
+    static void drawLine(QPainter *p, const QRect &r, const QColor &color, bool fadeStart=true, bool fadeEnd=true, double alpha=0.1);
     BasicItemDelegate(QObject *p);
-    virtual ~BasicItemDelegate();
-    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
-private:
-    bool eventFilter(QObject *object, QEvent *event);
+    ~BasicItemDelegate() override;
+    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
 private:
+    bool eventFilter(QObject *object, QEvent *event) override;
+protected:
     bool trackMouse;
     bool underMouse;
 };
