@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,16 +23,17 @@
 
 #include "gui/covers.h"
 #include "groupedview.h"
-#include "mpd/mpdstatus.h"
-#include "mpd/song.h"
-#include "actionitemdelegate.h"
+#include "mpd-interface/mpdstatus.h"
+#include "mpd-interface/song.h"
 #include "basicitemdelegate.h"
 #include "itemview.h"
 #include "config.h"
-#include "support/localize.h"
-#include "gui/plurals.h"
 #include "widgets/icons.h"
+#include "widgets/ratingwidget.h"
 #include "support/gtkstyle.h"
+#ifdef Q_OS_MAC
+#include "support/osxstyle.h"
+#endif
 #include "support/utils.h"
 #include "models/roles.h"
 #include <QStyledItemDelegate>
@@ -52,7 +53,7 @@ static double sizeAdjust=1.25;
 void GroupedView::setup()
 {
     int height=QApplication::fontMetrics().height();
-    sizeAdjust=Utils::touchFriendly() ? 1.5 : 1.25;
+    sizeAdjust=1.25;
 
     if (height>17) {
         constCoverSize=(((int)((height*2)/4))*4);
@@ -151,7 +152,7 @@ static Type getType(const QModelIndex &index)
 {
     QModelIndex prev=index.row()>0 ? index.sibling(index.row()-1, 0) : QModelIndex();
     quint16 thisKey=index.data(Cantata::Role_Key).toUInt();
-    quint16 prevKey=prev.isValid() ? prev.data(Cantata::Role_Key).toUInt() : Song::constNullKey;
+    quint16 prevKey=prev.isValid() ? prev.data(Cantata::Role_Key).toUInt() : (quint16)Song::Null_Key;
 
     return thisKey==prevKey ? AlbumTrack : AlbumHeader;
 }
@@ -171,268 +172,300 @@ static QString streamText(const Song &song, const QString &trackTitle, bool useN
                   ? song.title
                   : song.title.isEmpty()
                     ? songName
-                    : (song.title + " - " + songName);
+                    : (song.title + QString(" – ") + songName);
     } else if (!song.title.isEmpty() && !song.artist.isEmpty()) {
         QString name=song.name();
-        return song.artist + " - " + (!useName || name.isEmpty()
+        return song.artist + QString(" – ") + (!useName || name.isEmpty()
                                         ? song.title
                                         : song.title.isEmpty()
                                             ? name
-                                            : (song.title + " - " + name));
+                                            : (song.title + QString(" – ") + name));
     } else {
         return trackTitle;
     }
 }
 
-class GroupedViewDelegate : public ActionItemDelegate
+GroupedViewDelegate::GroupedViewDelegate(GroupedView *p)
+    : ActionItemDelegate(p)
+    , view(p)
+    , ratingPainter(nullptr)
 {
-public:
-    GroupedViewDelegate(GroupedView *p)
-        : ActionItemDelegate(p)
-        , view(p)
-    {
+}
+
+GroupedViewDelegate::~GroupedViewDelegate()
+{
+    delete ratingPainter;
+}
+
+QSize GroupedViewDelegate::sizeHint(int type, bool isCollection) const
+{
+    int textHeight = QApplication::fontMetrics().height()*sizeAdjust;
+
+    if (isCollection || AlbumHeader==type) {
+        return QSize(64, qMax(constCoverSize, (qMax(constIconSize, textHeight)*2)+constBorder)+(2*constBorder));
+    }
+    return QSize(64, qMax(constIconSize, textHeight)+(2*constBorder));
+}
+
+QSize GroupedViewDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
+{
+    if (0==index.column()) {
+        return sizeHint(getType(index), index.data(Cantata::Role_IsCollection).toBool());
+    }
+    return QStyledItemDelegate::sizeHint(option, index);
+}
+
+void GroupedViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+{
+    if (!index.isValid()) {
+        return;
     }
 
-    virtual ~GroupedViewDelegate()
-    {
-    }
+    Type type=getType(index);
+    bool isCollection=index.data(Cantata::Role_IsCollection).toBool();
+    Song song=index.data(Cantata::Role_SongWithRating).value<Song>();
+    int state=index.data(Cantata::Role_Status).toInt();
+    quint32 collection=index.data(Cantata::Role_CollectionId).toUInt();
+    bool selected=option.state&QStyle::State_Selected;
+    bool mouseOver=underMouse && option.state&QStyle::State_MouseOver;
+    bool gtk=mouseOver && GtkStyle::isActive();
+    bool rtl=QApplication::isRightToLeft();
 
-    QSize sizeHint(Type type, bool isCollection) const
-    {
-        int textHeight = QApplication::fontMetrics().height()*sizeAdjust;
-
-        if (isCollection || AlbumHeader==type) {
-            return QSize(64, qMax(constCoverSize, (qMax(constIconSize, textHeight)*2)+constBorder)+(2*constBorder));
-        }
-        return QSize(64, qMax(constIconSize, textHeight)+(2*constBorder));
-    }
-
-    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
-    {
-        if (0==index.column()) {
-            return sizeHint(getType(index), index.data(Cantata::Role_IsCollection).toBool());
-        }
-        return QStyledItemDelegate::sizeHint(option, index);
-    }
-
-    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
-    {
-        if (!index.isValid()) {
-            return;
-        }
-
-        Type type=getType(index);
-        bool isCollection=index.data(Cantata::Role_IsCollection).toBool();
-        Song song=index.data(Cantata::Role_Song).value<Song>();
-        int state=index.data(Cantata::Role_Status).toInt();
-        quint32 collection=index.data(Cantata::Role_CollectionId).toUInt();
-        bool selected=option.state&QStyle::State_Selected;
-        bool mouseOver=underMouse && option.state&QStyle::State_MouseOver;
-        bool gtk=mouseOver && GtkStyle::isActive();
-        bool rtl=QApplication::isRightToLeft();
-
-        if (!isCollection && AlbumHeader==type) {
-            if (mouseOver && gtk) {
-                GtkStyle::drawSelection(option, painter, (selected ? 0.75 : 0.25)*0.75);
-            } else {
-                painter->save();
-                painter->setOpacity(0.75);
-                QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter, 0L);
-                painter->restore();
-            }
+    if (!isCollection && AlbumHeader==type) {
+        if (mouseOver && gtk) {
+            GtkStyle::drawSelection(option, painter, (selected ? 0.75 : 0.25)*0.75);
+        } else {
             painter->save();
-            painter->setClipRect(option.rect.adjusted(0, option.rect.height()/2, 0, 0), Qt::IntersectClip);
-            if (mouseOver && gtk) {
-                GtkStyle::drawSelection(option, painter, selected ? 0.75 : 0.25);
-            } else {
-                QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter, 0L);
-            }
+            painter->setOpacity(0.75);
+            QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter, view);
             painter->restore();
-            if (!state && !view->isExpanded(song.key, collection) && view->isCurrentAlbum(song.key)) {
-                QVariant cs=index.data(Cantata::Role_CurrentStatus);
-                if (cs.isValid()) {
-                    state=cs.toInt();
-                }
-            }
-        } else {
-            if (mouseOver && gtk) {
-                GtkStyle::drawSelection(option, painter, selected ? 0.75 : 0.25);
-            } else {
-                QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter, 0L);
-            }
-        }
-        QString title;
-        QString track;
-        QString duration=song.time>0 ? Utils::formatTime(song.time) : QString();
-        bool stream=!isCollection && song.isStandardStream();
-        bool isEmpty=song.title.isEmpty() && song.artist.isEmpty() && !song.file.isEmpty();
-        QString trackTitle=isEmpty
-                        ? song.file
-                        : song.diffArtist()
-                            ? song.artistSong()
-                            : song.title;
-        QFont f(QApplication::font());
-        if (stream) {
-            f.setItalic(true);
-        }
-        QFontMetrics fm(f);
-        int textHeight=fm.height()*sizeAdjust;
-
-
-        if (isCollection) {
-            title=index.data(Qt::DisplayRole).toString();
-        } else if (AlbumHeader==type) {
-            if (stream) {
-                QModelIndex next=index.sibling(index.row()+1, 0);
-                quint16 nextKey=next.isValid() ? next.data(Cantata::Role_Key).toUInt() : Song::constNullKey;
-                if (nextKey!=song.key && !song.name().isEmpty()) {
-                    title=song.name();
-                    track=streamText(song, trackTitle, false);
-                } else {
-                    title=song.isCdda() ? i18n("Audio CD") : i18n("Streams");
-                    track=streamText(song, trackTitle);
-                }
-            } else if (isEmpty) {
-                title=Song::unknown();
-                track=trackTitle;
-            } else if (song.album.isEmpty()) {
-                title=song.artistOrComposer();
-                track=song.trackAndTitleStr();
-            } else {
-                if (song.isFromOnlineService()) {
-                    title=Song::displayAlbum(song.album, Song::albumYear(song));
-                } else {
-                    title=song.artistOrComposer()+QLatin1String(" - ")+Song::displayAlbum(song.album, Song::albumYear(song));
-                }
-                if (Song::useComposer()) {
-                    while (title.contains(") (")) {
-                        title=title.replace(") (", ", ");
-                    }
-                }
-                track=song.trackAndTitleStr();
-            }
-        } else {
-            if (stream) {
-                track=streamText(song, trackTitle);
-            } else {
-                track=song.trackAndTitleStr();
-            }
-        }
-
-        if (song.priority>0) {
-            track=track+QLatin1String(" [")+QString::number(song.priority)+QChar(']');
         }
         painter->save();
-        painter->setFont(f);
-        QColor col(option.palette.color(option.state&QStyle::State_Selected ? QPalette::HighlightedText : QPalette::Text));
-        QTextOption textOpt(Qt::AlignVCenter);
-        QRect r(option.rect.adjusted(constBorder+4, constBorder, -(constBorder+4), -constBorder));
-
-        if (state && GroupedView::State_StopAfterTrack!=state) {
-            QRectF border(option.rect.x()+1.5, option.rect.y()+1.5, option.rect.width()-3, option.rect.height()-3);
-            if (!title.isEmpty()) {
-                border.adjust(0, textHeight+constBorder, 0, 0);
+        painter->setClipRect(option.rect.adjusted(0, option.rect.height()/2, 0, 0), Qt::IntersectClip);
+        if (mouseOver && gtk) {
+            GtkStyle::drawSelection(option, painter, selected ? 0.75 : 0.25);
+        } else {
+            QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter, view);
+        }
+        painter->restore();
+        if (!state && !view->isExpanded(song.key, collection) && view->isCurrentAlbum(song.key)) {
+            QVariant cs=index.data(Cantata::Role_CurrentStatus);
+            if (cs.isValid()) {
+                state=cs.toInt();
             }
-            QLinearGradient g(border.topLeft(), border.bottomLeft());
-            QColor gradCol(QApplication::palette().color(QPalette::Highlight));
-            gradCol.setAlphaF(option.state&QStyle::State_Selected ? 0.6 : 0.45);
-            g.setColorAt(0, gradCol.dark(165));
-            g.setColorAt(1, gradCol.light(165));
-            painter->setRenderHint(QPainter::Antialiasing, true);
-            painter->fillPath(Utils::buildPath(border, 3), g);
-            painter->setPen(QPen(gradCol, 1));
-            painter->drawPath(Utils::buildPath(border.adjusted(-1, -1, 1, 1), 3.5));
-            painter->setPen(QPen(QApplication::palette().color(QPalette::HighlightedText), 1));
-            painter->drawPath(Utils::buildPath(border, 3));
-            painter->setRenderHint(QPainter::Antialiasing, false);
+        }
+    } else {
+        if (mouseOver && gtk) {
+            GtkStyle::drawSelection(option, painter, selected ? 0.75 : 0.25);
+        } else {
+            QApplication::style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter, view);
+        }
+    }
+    QString title;
+    QString track;
+    QString duration=song.time>0 ? Utils::formatTime(song.time) : QString();
+    bool stream=!isCollection && song.isStandardStream();
+    bool isEmpty=song.title.isEmpty() && song.artist.isEmpty() && !song.file.isEmpty();
+    QString trackTitle=isEmpty
+            ? song.file
+            : song.diffArtist()
+              ? song.artistSong()
+              : song.title;
+    QFont f(QApplication::font());
+    if (stream) {
+        f.setItalic(true);
+    }
+    QFontMetrics fm(f);
+    int textHeight=fm.height()*sizeAdjust;
+
+
+    if (isCollection) {
+        title=index.data(Qt::DisplayRole).toString();
+    } else if (AlbumHeader==type) {
+        if (stream) {
+            QModelIndex next=index.sibling(index.row()+1, 0);
+            quint16 nextKey=next.isValid() ? next.data(Cantata::Role_Key).toUInt() : (quint16)Song::Null_Key;
+            if (nextKey!=song.key && !song.name().isEmpty()) {
+                title=song.name();
+                track=streamText(song, trackTitle, false);
+            } else {
+                title=song.isCdda() ? tr("Audio CD") : tr("Streams");
+                track=streamText(song, trackTitle);
+            }
+        } else if (isEmpty) {
+            title=Song::unknown();
+            track=trackTitle;
+        } else if (song.album.isEmpty()) {
+            title=song.artistOrComposer();
+            track=song.trackAndTitleStr();
+        } else {
+            if (song.isFromOnlineService()) {
+                title=Song::displayAlbum(song.albumName(), Song::albumYear(song));
+            } else {
+                title=song.artistOrComposer()+QString(" – ")+Song::displayAlbum(song.albumName(), Song::albumYear(song));
+            }
+            track=song.trackAndTitleStr();
+        }
+    } else {
+        if (stream) {
+            track=streamText(song, trackTitle);
+        } else {
+            track=song.trackAndTitleStr();
+        }
+    }
+
+    if (song.priority>0) {
+        track=track+QLatin1String(" [")+QString::number(song.priority)+QChar(']');
+    }
+
+    painter->save();
+    painter->setFont(f);
+    #ifdef Q_OS_WIN
+    QColor textColor(option.palette.color(QPalette::Text));
+    #else
+    QColor textColor(option.palette.color(option.state&QStyle::State_Selected ? QPalette::HighlightedText : QPalette::Text));
+    #endif
+    QTextOption textOpt(Qt::AlignVCenter);
+    QRect r(option.rect.adjusted(constBorder+4, constBorder, -(constBorder+4), -constBorder));
+
+    if (state && GroupedView::State_StopAfterTrack!=state) {
+        QRectF border(option.rect.x()+1, option.rect.y()+1, option.rect.width()-3, option.rect.height()-3);
+        if (!title.isEmpty()) {
+            border.adjust(0, (border.height()/2)+1, 0, 0);
+        }
+        #ifdef Q_OS_MAC
+        QColor gradCol(OSXStyle::self()->viewPalette().color(QPalette::Highlight));
+        QColor borderCol(OSXStyle::self()->viewPalette().color(selected ? QPalette::HighlightedText : QPalette::Highlight));
+        #else
+        QColor gradCol(QApplication::palette().color(QPalette::Highlight));
+        QColor borderCol(QApplication::palette().color(selected ? QPalette::HighlightedText : QPalette::Highlight));
+        #endif
+        if (!selected) {
+            borderCol.setAlphaF(0.5);
+        }
+        gradCol.setAlphaF(selected ? 0.4 : 0.25);
+        painter->fillRect(border, gradCol);
+        painter->setPen(QPen(borderCol, 1));
+        painter->drawRect(border);
+    }
+
+    painter->setPen(textColor);
+    bool showTrackDuration=!duration.isEmpty();
+
+    if (isCollection || AlbumHeader==type) {
+        QPixmap pix;
+        // Draw cover...
+        if (isCollection) {
+            pix=index.data(Qt::DecorationRole).value<QIcon>().pixmap(constCoverSize, constCoverSize,
+                                                                     selected && textColor==qApp->palette().color(QPalette::HighlightedText)
+                                                                     ? QIcon::Selected : QIcon::Normal);
+        } else {
+            QPixmap *cover=/*stream ? 0 : */Covers::self()->get(song, constCoverSize);
+            pix=cover ? *cover : (stream && !song.isCdda() ? Icons::self()->streamIcon : Icons::self()->albumIcon(constCoverSize)).pixmap(constCoverSize, constCoverSize);
         }
 
-        painter->setPen(col);
-        bool showTrackDuration=!duration.isEmpty();
+        int maxSize=constCoverSize*pix.DEVICE_PIXEL_RATIO();
 
-        if (isCollection || AlbumHeader==type) {
-            QPixmap pix;
-            // Draw cover...
-            if (isCollection) {
-                pix=index.data(Qt::DecorationRole).value<QIcon>().pixmap(constCoverSize, constCoverSize);
-            } else {
-                QPixmap *cover=stream ? 0 : Covers::self()->get(song, constCoverSize);
-                pix=cover ? *cover : (stream && !song.isCdda() ? Icons::self()->streamIcon : Icons::self()->albumIcon).pixmap(constCoverSize, constCoverSize);
-            }
+        if (pix.width()>maxSize) {
+            pix=pix.scaled(maxSize, maxSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+        }
 
-            if (pix.width()>constCoverSize) {
-                pix=pix.scaled(constCoverSize, constCoverSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
-            }
+        QSize pixSize = pix.isNull() ? QSize(0, 0) : (pix.size() / pix.DEVICE_PIXEL_RATIO());
 
-            if (rtl) {
-                painter->drawPixmap(r.x()+r.width()-(pix.width()-constBorder), r.y()+((r.height()-pix.height())/2), pix.width(), pix.height(), pix);
-                r.adjust(0, 0, -(constCoverSize+constBorder), 0);
-            } else {
-                painter->drawPixmap(r.x()-2, r.y()+((r.height()-pix.height())/2), pix.width(), pix.height(), pix);
-                r.adjust(constCoverSize+constBorder, 0, 0, 0);
-            }
-            int td=index.data(Cantata::Role_AlbumDuration).toUInt();
-            QString totalDuration=td>0 ? Utils::formatTime(td) : QString();
-            QRect duratioRect(r.x(), r.y(), r.width(), textHeight);
-            int totalDurationWidth=fm.width(totalDuration)+8;
-            QRect textRect(r.x(), r.y(), r.width()-(rtl ? (4*constBorder) : totalDurationWidth), textHeight);
-            QFont tf(f);
-            tf.setBold(true);
-            title = QFontMetrics(tf).elidedText(title, Qt::ElideRight, textRect.width(), QPalette::WindowText);
-            painter->setFont(tf);
-            painter->drawText(textRect, title, textOpt);
-            if (!totalDuration.isEmpty()) {
-                painter->drawText(duratioRect, totalDuration, QTextOption(Qt::AlignVCenter|Qt::AlignRight));
-            }
-            BasicItemDelegate::drawLine(painter, r.adjusted(0, 0, 0, -r.height()/2), col, rtl, !rtl, 0.45);
-            r.adjust(0, textHeight+constBorder, 0, 0);
-            r=QRect(r.x(), r.y()+r.height()-(textHeight+1), r.width(), textHeight);
-            painter->setFont(f);
-            if (rtl) {
-                r.adjust(0, 0, (constCoverSize-(constBorder*3)), 0);
-            }
-
-            if (isCollection || !view->isExpanded(song.key, collection)) {
-                showTrackDuration=false;
-                track=Plurals::tracks(index.data(Cantata::Role_SongCount).toUInt());
-            }
-        } else if (rtl) {
-            r.adjust(0, 0, -(constBorder*4), 0);
+        if (rtl) {
+            painter->drawPixmap(r.x()+r.width()-(pixSize.width()-constBorder), r.y()+((r.height()-pixSize.height())/2), pixSize.width(), pixSize.height(), pix);
+            r.adjust(0, 0, -(constCoverSize+constBorder), 0);
         } else {
+            painter->drawPixmap(r.x()-2, r.y()+((r.height()-pixSize.height())/2), pixSize.width(), pixSize.height(), pix);
             r.adjust(constCoverSize+constBorder, 0, 0, 0);
         }
 
-        GroupedView::drawPlayState(painter, option, r, state);
-        painter->setPen(col);
-
-        int durationWidth=showTrackDuration ? fm.width(duration)+8 : 0;
+        int td=index.data(Cantata::Role_AlbumDuration).toUInt();
+        QString totalDuration=td>0 && td!=song.time ? Utils::formatTime(td) : QString();
         QRect duratioRect(r.x(), r.y(), r.width(), textHeight);
-        QRect textRect(r.x(), r.y(), r.width()-durationWidth, textHeight);
-        track = fm.elidedText(track, Qt::ElideRight, textRect.width(), QPalette::WindowText);
-        painter->drawText(textRect, track, textOpt);
-        if (showTrackDuration) {
-            painter->drawText(duratioRect, duration, QTextOption(Qt::AlignVCenter|Qt::AlignRight));
+        int totalDurationWidth=fm.width(totalDuration)+8;
+        QRect textRect(r.x(), r.y(), r.width()-(rtl ? (4*constBorder) : totalDurationWidth), textHeight);
+        QFont tf(f);
+        tf.setBold(true);
+        title = QFontMetrics(tf).elidedText(title, Qt::ElideRight, textRect.width(), QPalette::WindowText);
+        painter->setFont(tf);
+        painter->drawText(textRect, title, textOpt);
+        if (!totalDuration.isEmpty()) {
+            painter->drawText(duratioRect, totalDuration, QTextOption(Qt::AlignVCenter|Qt::AlignRight));
+        }
+        BasicItemDelegate::drawLine(painter, r.adjusted(0, 0, 0, -r.height()/2), textColor, rtl, !rtl, 0.45);
+        r.adjust(0, textHeight+constBorder, 0, 0);
+        r=QRect(r.x(), r.y()+r.height()-(textHeight+1), r.width(), textHeight);
+        painter->setFont(f);
+        if (rtl) {
+            r.adjust(0, 0, (constCoverSize-(constBorder*3)), 0);
         }
 
-        if (mouseOver || Utils::touchFriendly()) {
-            drawIcons(painter, option.rect, mouseOver || (selected && Utils::touchFriendly()), rtl, AlbumHeader==type || isCollection ? AP_HBottom : AP_HMiddle, index);
+        if (isCollection || !view->isExpanded(song.key, collection)) {
+            showTrackDuration=false;
+            track=tr("%n Track(s)", "", index.data(Cantata::Role_SongCount).toUInt());
         }
-        BasicItemDelegate::drawLine(painter, option.rect, col);
-        painter->restore();
+    } else if (rtl) {
+        r.adjust(0, 0, -(constBorder*4), 0);
+    } else {
+        r.adjust(constCoverSize+constBorder, 0, 0, 0);
     }
 
-private:
-    GroupedView *view;
-};
+    GroupedView::drawPlayState(painter, option, r, state);
+    painter->setPen(textColor);
+
+    int ratingsStart=rtl ? 0 : drawRatings(painter, song, r, fm, option.palette.color(QPalette::Active, QPalette::Text)); // TODO!!!
+
+    int durationWidth=showTrackDuration ? fm.width(duration)+8 : 0;
+    QRect duratioRect(r.x(), r.y(), r.width(), textHeight);
+    QRect textRect(r.x(), r.y(), r.width()-durationWidth, textHeight);
+    if (ratingsStart>0) {
+        textRect.setWidth(ratingsStart-r.x());
+    }
+    track = fm.elidedText(track, Qt::ElideRight, textRect.width(), QPalette::WindowText);
+    painter->drawText(textRect, track, textOpt);
+    if (showTrackDuration) {
+        painter->drawText(duratioRect, duration, QTextOption(Qt::AlignVCenter|Qt::AlignRight));
+    }
+
+    if (mouseOver) {
+        drawIcons(painter, option.rect, mouseOver, rtl, AlbumHeader==type || isCollection ? AP_HBottom : AP_HMiddle, index);
+    }
+    BasicItemDelegate::drawLine(painter, option.rect, textColor);
+    painter->restore();
+}
+
+int GroupedViewDelegate::drawRatings(QPainter *painter, const Song &song, const QRect &r, const QFontMetrics &fm, const QColor &col) const
+{
+    if (song.rating>0 && song.rating<=Song::Rating_Max) {
+        if (!ratingPainter) {
+            ratingPainter=new RatingPainter(r.height()-(constBorder*8));
+            ratingPainter->setColor(col);
+        }
+
+        painter->save();
+        painter->setOpacity(painter->opacity()*0.75);
+        const QSize &ratingSize=ratingPainter->size();
+        int spacing=constBorder*2;
+        int durationWidth=fm.width("0:00:00")+spacing;
+        QRect ratingRect(r.x()+r.width()-(durationWidth+ratingSize.width()+spacing),
+                         r.y()+(r.height()-ratingSize.height())/2,
+                         ratingSize.width(), ratingSize.height());
+        ratingPainter->paint(painter, ratingRect, song.rating);
+        painter->restore();
+        return ratingRect.x()-spacing;
+    }
+    return 0;
+}
 
 GroupedView::GroupedView(QWidget *parent, bool isPlayQueue)
     : TreeView(parent, isPlayQueue)
-    , allowClose(!Utils::touchFriendly())
+    , allowClose(true)
     , startClosed(allowClose)
     , autoExpand(true)
     , filterActive(false)
     , isMultiLevel(false)
-    , currentAlbum(Song::constNullKey)
+    , currentAlbum(Song::Null_Key)
 {
     setContextMenuPolicy(Qt::CustomContextMenu);
     setAcceptDrops(true);
@@ -445,7 +478,6 @@ GroupedView::GroupedView(QWidget *parent, bool isPlayQueue)
     setSelectionBehavior(SelectRows);
     setForceSingleColumn(true);
     connect(this, SIGNAL(clicked(const QModelIndex &)), this, SLOT(itemClicked(const QModelIndex &)));
-    setStyleSheet("QTreeView::branch { border: 0px; }");
     GroupedViewDelegate *delegate=new GroupedViewDelegate(this);
     setItemDelegate(delegate);
     if (isPlayQueue) {
@@ -540,7 +572,7 @@ void GroupedView::updateRows(const QModelIndex &parent)
     }
 
     qint32 count=model()->rowCount(parent);
-    quint16 lastKey=Song::constNullKey;
+    quint16 lastKey=Song::Null_Key;
     quint32 collection=parent.data(Cantata::Role_CollectionId).toUInt();
     QSet<quint16> keys;
 
@@ -565,7 +597,7 @@ void GroupedView::updateCollectionRows()
         return;
     }
 
-    currentAlbum=Song::constNullKey;
+    currentAlbum=Song::Null_Key;
     qint32 count=model()->rowCount();
     for (int i=0; i<count; ++i) {
         updateRows(model()->index(i, 0));
@@ -619,7 +651,7 @@ QModelIndexList GroupedView::selectedIndexes(bool sorted) const
     QSet<QModelIndex> indexSet;
     QModelIndexList sel;
 
-    foreach (const QModelIndex &idx, indexes) {
+    for (const QModelIndex &idx: indexes) {
         if (!indexSet.contains(idx)) {
             indexSet.insert(idx);
             sel.append(idx);
@@ -699,11 +731,11 @@ void GroupedView::dropEvent(QDropEvent *event)
 
 void GroupedView::coverLoaded(const Song &song, int size)
 {
-    if (filterActive || !isVisible() || size!=constCoverSize || song.isArtistImageRequest()) {
+    if (filterActive || !isVisible() || size!=constCoverSize || song.isArtistImageRequest() || song.isComposerImageRequest()) {
         return;
     }
     quint32 count=model()->rowCount();
-    quint16 lastKey=Song::constNullKey;
+    quint16 lastKey=Song::Null_Key;
     QString albumArtist=song.albumArtist();
     QString album=song.album;
 
@@ -713,7 +745,7 @@ void GroupedView::coverLoaded(const Song &song, int size)
             continue;
         }
         if (isMultiLevel && model()->hasChildren(index)) {
-            lastKey=Song::constNullKey;
+            lastKey=Song::Null_Key;
             quint32 childCount=model()->rowCount(index);
             for (quint32 c=0; c<childCount; ++c) {
                 QModelIndex child=model()->index(c, 0, index);
@@ -783,17 +815,17 @@ void GroupedView::itemClicked(const QModelIndex &idx)
             if (list.count()) {
                 #if 0 // Commendted out as (as noted below) unselection is not working, and we always add to 'unsel' above (because of playlists)
                 if (unsel.isEmpty()) { // TODO: This is not working!!!   CHECK selModel if re-add!!!
-                    foreach(const QModelIndex &i, list) {
+                    for (const QModelIndex &i: list) {
                         selModel->select(i, QItemSelectionModel::Deselect|QItemSelectionModel::Rows);
                     }
                 } else {
-                    foreach(const QModelIndex &i, unsel) {
+                    for (const QModelIndex &i: unsel) {
                         selModel->select(i, QItemSelectionModel::Select|QItemSelectionModel::Rows);
                     }
                 }
                 #else
                 if (!unsel.isEmpty() && selModel) {
-                    foreach(const QModelIndex &i, unsel) {
+                    for (const QModelIndex &i: unsel) {
                         selModel->select(i, QItemSelectionModel::Select|QItemSelectionModel::Rows);
                     }
                 }
@@ -838,3 +870,10 @@ void GroupedView::collapse(const QModelIndex &idx, bool singleOnly)
         }
     }
 }
+
+void GroupedView::drawBranches(QPainter *, const QRect &, const QModelIndex &) const
+{
+    // Don't want any branch lines drawn!
+}
+
+#include "moc_groupedview.cpp"
