@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -31,25 +31,26 @@
 #include <QList>
 #include <QPair>
 
-class ProxyModel;
 class Spinner;
 class QTimer;
 class GroupedView;
 class ActionItemDelegate;
 class MessageOverlay;
-class Icon;
 class TableView;
+class QMenu;
+class Configuration;
+class CategorizedView;
 
 class KeyEventHandler : public QObject
 {
     Q_OBJECT
 public:
-    KeyEventHandler(QAbstractItemView *v, QAction *a=0);
+    KeyEventHandler(QAbstractItemView *v, QAction *a=nullptr);
     void setDeleteAction(QAction *a) { deleteAct=a; }
 Q_SIGNALS:
     void backspacePressed();
 protected:
-    bool eventFilter(QObject *obj, QEvent *event);
+    bool eventFilter(QObject *obj, QEvent *event) override;
 protected:
     QAbstractItemView *view;
     QAction *deleteAct;
@@ -61,7 +62,7 @@ class ViewEventHandler : public KeyEventHandler
 public:
     ViewEventHandler(ActionItemDelegate *d, QAbstractItemView *v);
 protected:
-    bool eventFilter(QObject *obj, QEvent *event);
+    bool eventFilter(QObject *obj, QEvent *event) override;
 private:
     ActionItemDelegate *delegate;
 };
@@ -80,6 +81,7 @@ public:
         Mode_GroupedTree,
 
         Mode_Table,
+        Mode_Categorized,
 
         Mode_List,
         Mode_IconTop,
@@ -91,19 +93,25 @@ public:
     static QString modeStr(Mode m);
     static void setup();
     static const QLatin1String constSearchActiveKey;
+    static const QLatin1String constViewModeKey;
+    static const QLatin1String constStartClosedKey;
+    static const QLatin1String constSearchCategoryKey;
 
-    ItemView(QWidget *p=0);
-    virtual ~ItemView();
+    ItemView(QWidget *p=nullptr);
+    ~ItemView() override;
 
-    void load(const QString &group);
-    void save(const QString &group);
+    void alwaysShowHeader();
+    void load(Configuration &config);
+    void save(Configuration &config);
     void allowGroupedView();
     void allowTableView(TableView *v);
+    void allowCategorized();
     void addAction(QAction *act);
+    void addSeparator();
     void setMode(Mode m);
     Mode viewMode() const { return mode; }
     QAbstractItemView * view() const;
-    void setModel(ProxyModel *m);
+    void setModel(QAbstractItemModel *m);
     void clearSelection() { view()->selectionModel()->clearSelection(); }
     void setCurrentIndex(const QModelIndex &idx) { view()->setCurrentIndex(idx); }
     void select(const QModelIndex &idx);
@@ -116,15 +124,13 @@ public:
     void setAcceptDrops(bool v);
     void setDragDropOverwriteMode(bool v);
     void setDragDropMode(QAbstractItemView::DragDropMode v);
-    void setGridSize(const QSize &sz);
-    QSize gridSize() const { return listView->gridSize(); }
     void update();
     void setDeleteAction(QAction *act);
     void setRootIsDecorated(bool v) { treeView->setRootIsDecorated(v); }
     void showIndex(const QModelIndex &idx, bool scrollTo);
-    void setSearchLabelText(const QString &text);
     void setSearchVisible(bool v);
     bool isSearchActive() const;
+    void setSearchToolTip(const QString &str);
     void setStartClosed(bool sc);
     bool isStartClosed();
     void expandAll(const QModelIndex &index=QModelIndex());
@@ -134,12 +140,15 @@ public:
     bool isAnimated() const;
     void setAnimated(bool a);
     void setPermanentSearch();
-    void setSearchCategories(const QList<QPair<QString, QString> > &categories);
+    void hideSearch();
+    void setSearchCategories(const QList<SearchWidget::Category> &categories);
     void setSearchCategory(const QString &id);
     void setSearchResetLevel(int l) { searchResetLevel=l; }
-    void setSearchIndex(const QModelIndex &idx) { searchIndex=idx; }
-    void showEvent(QShowEvent *ev);
     void goToTop();
+    void setOpenAfterSearch(bool o) { openFirstLevelAfterSearch=o; }
+    void setEnabled(bool en);
+    void setMinSearchDebounce(unsigned int val) { minSearchDebounce = val; }
+    void setInfoText(const QString &info);
 
 private:
     void setLevel(int level, bool haveChildren=true);
@@ -151,6 +160,8 @@ public Q_SLOTS:
     void focusView();
     void showSpinner(bool v=true);
     void hideSpinner();
+    void updating();
+    void updated();
     void collectionRemoved(quint32 key);
     void updateRows();
     void updateRows(const QModelIndex &idx);
@@ -164,6 +175,8 @@ Q_SIGNALS:
     void itemsSelected(bool);
     void doubleClicked(const QModelIndex &);
     void rootIndexSet(const QModelIndex &);
+    void headerClicked(int level);
+    void updateToPlayQueue(const QModelIndex &idx, bool replace);
 
 private Q_SLOTS:
     void itemClicked(const QModelIndex &index);
@@ -172,29 +185,38 @@ private Q_SLOTS:
     void doSearch();
     void searchActive(bool a);
     void activateItem(const QModelIndex &index, bool emitRootSet=true);
+    void modelReset();
+    void dataChanged(const QModelIndex &tl, const QModelIndex &br);
+    void addTitleButtonClicked();
+    void replaceTitleButtonClicked();
+    void coverLoaded(const Song &song, int size);
+    void zoomIn();
+    void zoomOut();
 
 private:
-    void collapseToLevel();
     QAction * getAction(const QModelIndex &index);
+    void setTitle();
+    void controlViewFrame();
 
 private:
     QTimer *searchTimer;
-    ProxyModel *itemModel;
+    QAbstractItemModel *itemModel;
     int currentLevel;
     Mode mode;
-    QString currentText;
-    QMap<int, QString> prevText;
-    QModelIndex prevTopIndex;
+    QModelIndexList prevTopIndex;
     QSize iconGridSize;
     QSize listGridSize;
     GroupedView *groupedView;
     TableView *tableView;
+    CategorizedView *categorizedView;
     Spinner *spinner;
     MessageOverlay *msgOverlay;
     QIcon bgndIcon;
     bool performedSearch;
     int searchResetLevel;
-    QModelIndex searchIndex;
+    bool openFirstLevelAfterSearch;
+    bool initialised;
+    unsigned int minSearchDebounce;
 };
 
 #endif
