@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -37,8 +37,9 @@
 
 ListView::ListView(QWidget *parent)
     : QListView(parent)
-    , eventFilter(0)
-    , menu(0)
+    , eventFilter(nullptr)
+    , menu(nullptr)
+    , zoomLevel(1.0)
 {
     setDragEnabled(true);
     setContextMenuPolicy(Qt::NoContextMenu);
@@ -74,7 +75,7 @@ bool ListView::haveSelectedItems() const
 bool ListView::haveUnSelectedItems() const
 {
     // Dont need the sorted type of 'selectedIndexes' here...
-    return selectionModel() && selectionModel()->selectedIndexes().count()!=model()->rowCount();
+    return selectionModel() && model() && selectionModel()->selectedIndexes().count()!=model()->rowCount();
 }
 
 void ListView::mouseReleaseEvent(QMouseEvent *event)
@@ -118,10 +119,12 @@ void ListView::addDefaultAction(QAction *act)
 void ListView::setBackgroundImage(const QIcon &icon)
 {
     QPalette pal=parentWidget()->palette();
-    if (!icon.isNull()) {
-        pal.setColor(QPalette::Base, Qt::transparent);
-    }
+//    if (!icon.isNull()) {
+//        pal.setColor(QPalette::Base, Qt::transparent);
+//    }
+    #ifndef Q_OS_MAC
     setPalette(pal);
+    #endif
     viewport()->setPalette(pal);
     bgnd=TreeView::createBgndPixmap(icon);
 }
@@ -134,6 +137,16 @@ void ListView::paintEvent(QPaintEvent *e)
         p.fillRect(0, 0, sz.width(), sz.height(), QApplication::palette().color(QPalette::Base));
         p.drawPixmap((sz.width()-bgnd.width())/2, (sz.height()-bgnd.height())/2, bgnd);
     }
+    if (!info.isEmpty() && model() && 0==model()->rowCount()) {
+        QPainter p(viewport());
+        QColor col(palette().text().color());
+        col.setAlphaF(0.5);
+        QFont f(font());
+        f.setItalic(true);
+        p.setPen(col);
+        p.setFont(f);
+        p.drawText(rect().adjusted(8, 8, -16, -16), Qt::AlignCenter|Qt::TextWordWrap, info);
+    }
     QListView::paintEvent(e);
 }
 
@@ -163,3 +176,5 @@ void ListView::checkDoubleClick(const QModelIndex &idx)
     }
     emit itemDoubleClicked(idx);
 }
+
+#include "moc_listview.cpp"
