@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,8 +23,7 @@
 
 #include "messageoverlay.h"
 #include "toolbutton.h"
-#include "icons.h"
-#include "support/localize.h"
+#include "support/monoicon.h"
 #include "support/utils.h"
 #include <QPainter>
 #include <QPaintEvent>
@@ -32,8 +31,13 @@
 #include <QApplication>
 
 MessageOverlay::MessageOverlay(QObject *p)
-    : QWidget(0)
-    , timer(0)
+    : QWidget(nullptr)
+    , timer(nullptr)
+    #ifdef Q_OS_MAC
+    , closeOnLeft(true)
+     #else
+    , closeOnLeft(Utils::Unity==Utils::currentDe())
+    #endif
 {
     Q_UNUSED(p)
     spacing=fontMetrics().height();
@@ -42,12 +46,8 @@ MessageOverlay::MessageOverlay(QObject *p)
     setMaximumHeight(spacing*2);
     cancelButton=new ToolButton(this);
     Icon::init(cancelButton);
-    cancelButton->setToolTip(i18n("Cancel"));
-    Icon icon=Icon("dialog-close");
-    if (icon.isNull()) {
-        icon=Icon("window-close");
-    }
-    cancelButton->setIcon(icon);
+    cancelButton->setToolTip(tr("Cancel"));
+    cancelButton->setIcon(MonoIcon::icon(FontAwesome::close, MonoIcon::constRed, MonoIcon::constRed));
     cancelButton->adjustSize();
     connect(cancelButton, SIGNAL(clicked()), SIGNAL(cancel()));
 }
@@ -96,8 +96,15 @@ void MessageOverlay::paintEvent(QPaintEvent *)
     p.drawPath(path);
 
     int pad=r.height()/4;
-    rf.adjust(pad, pad, cancelButton->isVisible() ? -((pad*2)+cancelButton->width()) : -pad, -pad);
-
+    if (cancelButton->isVisible()) {
+        if (Qt::LeftToRight==layoutDirection() && !closeOnLeft) {
+            rf.adjust(pad, pad, -((pad*2)+cancelButton->width()), -pad);
+        } else {
+            rf.adjust(((pad*2)+cancelButton->width()), pad, -pad, -pad);
+        }
+    } else {
+        rf.adjust(pad, pad, -pad, -pad);
+    }
     QFont fnt(QApplication::font());
     fnt.setBold(true);
     QFontMetrics fm(fnt);
@@ -133,7 +140,7 @@ void MessageOverlay::setSizeAndPosition()
     if (currentWidth!=desiredWidth) {
         int pad=height()/4;
         resize(desiredWidth, height());
-        cancelButton->move(QPoint(Qt::LeftToRight==layoutDirection()
+        cancelButton->move(QPoint(Qt::LeftToRight==layoutDirection() && !closeOnLeft
                                     ? desiredWidth-(cancelButton->width()+pad)
                                     : pad,
                                   (height()-cancelButton->height())/2));
@@ -143,3 +150,5 @@ void MessageOverlay::setSizeAndPosition()
         move(desiredPos);
     }
 }
+
+#include "moc_messageoverlay.cpp"
