@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -28,13 +28,12 @@
 #include "groupedview.h"
 #include "treeview.h"
 #include "gui/settings.h"
-#include "mpd/mpdstatus.h"
-#include "support/localize.h"
+#include "mpd-interface/mpdstatus.h"
 #include "support/spinner.h"
 #include "messageoverlay.h"
-#include "basicitemdelegate.h"
 #include "icons.h"
 #include "support/gtkstyle.h"
+#include "support/proxystyle.h"
 #include "support/actioncollection.h"
 #include "support/action.h"
 #include "models/roles.h"
@@ -46,27 +45,23 @@
 // Exported by QtGui
 void qt_blurImage(QPainter *p, QImage &blurImage, qreal radius, bool quality, bool alphaOnly, int transposed = 0);
 
-class PlayQueueTreeViewItemDelegate : public BasicItemDelegate
+class PlayQueueTreeStyle : public ProxyStyle
 {
 public:
-    PlayQueueTreeViewItemDelegate(QObject *p) : BasicItemDelegate(p) { }
-    virtual ~PlayQueueTreeViewItemDelegate() { }
-    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+    void drawPrimitive(PrimitiveElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget) const override
     {
-        if (!index.isValid()) {
-            return;
-        }
+        if (QStyle::PE_IndicatorItemViewItemDrop == element && widget) {
+            painter->setPen(QPen(QPalette::Highlight)); // make the drop indicator more visible
+            painter->setRenderHint(QPainter::Antialiasing, false);
 
-        QStyleOptionViewItemV4 v4((QStyleOptionViewItemV4 &)option);
-        if (QStyleOptionViewItemV4::Beginning==v4.viewItemPosition) {
-            v4.icon=index.data(Cantata::Role_Decoration).value<QIcon>();
-            if (!v4.icon.isNull()) {
-                v4.features |= QStyleOptionViewItemV2::HasDecoration;
-                v4.decorationSize=v4.icon.actualSize(option.decorationSize, QIcon::Normal, QIcon::Off);
-            }
-        }
+            QStyleOption opt(*option);
+            opt.rect.setLeft(0);                // let the drop indicator
+            opt.rect.setRight(widget->width()); // span a whole tree widget row
 
-        BasicItemDelegate::paint(painter, v4, index);
+            ProxyStyle::drawPrimitive(element, &opt, painter, widget);
+        } else {
+            ProxyStyle::drawPrimitive(element, option, painter, widget);
+        }
     }
 };
 
@@ -78,7 +73,6 @@ PlayQueueTreeView::PlayQueueTreeView(PlayQueueView *parent)
     setItemsExpandable(false);
     setExpandsOnDoubleClick(false);
     setRootIsDecorated(false);
-    setItemDelegate(new PlayQueueTreeViewItemDelegate(this));
 }
 
 void PlayQueueTreeView::paintEvent(QPaintEvent *e)
@@ -106,16 +100,16 @@ void PlayQueueGroupedView::paintEvent(QPaintEvent *e)
 PlayQueueView::PlayQueueView(QWidget *parent)
     : QStackedWidget(parent)
     , mode(ItemView::Mode_Count)
-    , groupedView(0)
-    , treeView(0)
-    , spinner(0)
-    , msgOverlay(0)
+    , groupedView(nullptr)
+    , treeView(nullptr)
+    , spinner(nullptr)
+    , msgOverlay(nullptr)
     , backgroundImageType(BI_None)
     , fadeValue(1.0)
     , backgroundOpacity(15)
     , backgroundBlur(0)
 {
-    removeFromAction = ActionCollection::get()->createAction("removefromplaylist", i18n("Remove From Play Queue"), "list-remove");
+    removeFromAction = new Action(Icons::self()->removeIcon, tr("Remove"), this);
     setMode(ItemView::Mode_GroupedTree);
     animator.setPropertyName("fade");
     animator.setTargetObject(this);
@@ -190,20 +184,25 @@ void PlayQueueView::setMode(ItemView::Mode m)
     case ItemView::Mode_GroupedTree:
         if (!groupedView) {
             groupedView=new PlayQueueGroupedView(this);
+            groupedView->setContextMenuPolicy(Qt::ActionsContextMenu);
             groupedView->setIndentation(0);
             groupedView->setItemsExpandable(false);
             groupedView->setExpandsOnDoubleClick(false);
-            groupedView->setContextMenuPolicy(Qt::ActionsContextMenu);
             groupedView->installFilter(new KeyEventHandler(groupedView, removeFromAction));
             addWidget(groupedView);
             connect(groupedView, SIGNAL(itemsSelected(bool)), SIGNAL(itemsSelected(bool)));
             connect(groupedView, SIGNAL(doubleClicked(const QModelIndex &)), SIGNAL(doubleClicked(const QModelIndex &)));
             updatePalette();
+            #ifdef Q_OS_MAC
+            groupedView->setAttribute(Qt::WA_MacShowFocusRect, 0);
+            #endif
+            groupedView->setProperty(ProxyStyle::constModifyFrameProp, ProxyStyle::VF_Top);
         }
         break;
     case ItemView::Mode_Table:
         if (!treeView) {
             treeView=new PlayQueueTreeView(this);
+            treeView->setStyle(new PlayQueueTreeStyle());
             treeView->setContextMenuPolicy(Qt::ActionsContextMenu);
             treeView->installFilter(new KeyEventHandler(treeView, removeFromAction));
             treeView->initHeader();
@@ -211,17 +210,21 @@ void PlayQueueView::setMode(ItemView::Mode m)
             connect(treeView, SIGNAL(itemsSelected(bool)), SIGNAL(itemsSelected(bool)));
             connect(treeView, SIGNAL(doubleClicked(const QModelIndex &)), SIGNAL(doubleClicked(const QModelIndex &)));
             updatePalette();
+            #ifdef Q_OS_MAC
+            treeView->setAttribute(Qt::WA_MacShowFocusRect, 0);
+            #endif
+            treeView->setProperty(ProxyStyle::constModifyFrameProp, ProxyStyle::VF_Top);
         }
     default:
         break;
     }
 
-    QAbstractItemModel *model=0;
+    QAbstractItemModel *model=nullptr;
     QList<QAction *> actions;
     if (ItemView::Mode_Count!=mode) {
         QAbstractItemView *v=view();
         model=v->model();
-        v->setModel(0);
+        v->setModel(nullptr);
         actions=v->actions();
     }
 
@@ -363,9 +366,9 @@ QList<Song> PlayQueueView::selectedSongs() const
     const QModelIndexList selected = selectedIndexes();
     QList<Song> songs;
 
-    foreach (const QModelIndex &idx, selected) {
+    for (const QModelIndex &idx: selected) {
         Song song=idx.data(Cantata::Role_Song).value<Song>();
-        if (!song.file.isEmpty() && !song.file.contains(":/") && !song.file.startsWith('/')) {
+        if (!song.file.isEmpty() && !song.hasProtocolOrIsAbsolute()) {
             songs.append(song);
         }
     }
@@ -404,15 +407,19 @@ void PlayQueueView::updatePalette()
 {
     QPalette pal=palette();
 
-    if (BI_None!=backgroundImageType) {
-        pal.setColor(QPalette::Base, Qt::transparent);
-    }
+//    if (BI_None!=backgroundImageType) {
+//        pal.setColor(QPalette::Base, Qt::transparent);
+//    }
     if (groupedView) {
+        #ifndef Q_OS_MAC
         groupedView->setPalette(pal);
+        #endif
         groupedView->viewport()->setPalette(pal);
     }
     if (treeView) {
+        #ifndef Q_OS_MAC
         treeView->setPalette(pal);
+        #endif
         treeView->viewport()->setPalette(pal);
     }
 }
@@ -466,6 +473,11 @@ void PlayQueueView::streamFetchStatus(const QString &msg)
     msgOverlay->setText(msg);
 }
 
+void PlayQueueView::searchActive(bool a)
+{
+    view()->setProperty(ProxyStyle::constModifyFrameProp, a ? 0 : ProxyStyle::VF_Top);
+}
+
 void PlayQueueView::drawBackdrop(QWidget *widget, const QSize &size)
 {
     if (BI_None==backgroundImageType) {
@@ -474,7 +486,7 @@ void PlayQueueView::drawBackdrop(QWidget *widget, const QSize &size)
 
     QPainter p(widget);
 
-    p.fillRect(0, 0, size.width(), size.height(), QApplication::palette().color(QPalette::Base));
+    p.fillRect(0, 0, size.width(), size.height(), QApplication::palette().color(topLevelWidget()->isActiveWindow() ? QPalette::Active : QPalette::Inactive, QPalette::Base));
     if (!curentCover.isNull() || !previousBackground.isNull()) {
         if (!curentCover.isNull() && (size!=lastBgndSize || curentBackground.isNull())) {
             curentBackground = QPixmap::fromImage(curentCover.scaled(size, Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation));
@@ -493,3 +505,5 @@ void PlayQueueView::drawBackdrop(QWidget *widget, const QSize &size)
         }
     }
 }
+
+#include "moc_playqueueview.cpp"
