@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -30,10 +30,11 @@
 #include <QPixmap>
 #include <QImage>
 #include <QPropertyAnimation>
+#include <QPainter>
 #include "tableview.h"
 #include "listview.h"
 #include "groupedview.h"
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 #include "itemview.h"
 
 class QAbstractItemModel;
@@ -41,7 +42,6 @@ class QAction;
 class Action;
 class QItemSelectionModel;
 class QModelIndex;
-class QMenu;
 class Spinner;
 class PlayQueueView;
 class MessageOverlay;
@@ -50,8 +50,8 @@ class PlayQueueTreeView : public TableView
 {
 public:
     PlayQueueTreeView(PlayQueueView *p);
-    virtual ~PlayQueueTreeView() { }
-    void paintEvent(QPaintEvent *e);
+    ~PlayQueueTreeView() override { }
+    void paintEvent(QPaintEvent *e) override;
 private:
     PlayQueueView *view;
 };
@@ -60,8 +60,8 @@ class PlayQueueGroupedView : public GroupedView
 {
 public:
     PlayQueueGroupedView(PlayQueueView *p);
-    virtual ~PlayQueueGroupedView();
-    void paintEvent(QPaintEvent *e);
+    ~PlayQueueGroupedView() override;
+    void paintEvent(QPaintEvent *e) override;
 private:
     PlayQueueView *view;
 };
@@ -78,8 +78,8 @@ public:
         BI_Custom
     };
 
-    PlayQueueView(QWidget *parent=0);
-    virtual ~PlayQueueView();
+    PlayQueueView(QWidget *parent=nullptr);
+    ~PlayQueueView() override;
 
     void readConfig();
     void saveConfig();
@@ -117,6 +117,7 @@ public Q_SLOTS:
     void hideSpinner();
     void setImage(const QImage &img);
     void streamFetchStatus(const QString &msg);
+    void searchActive(bool a);
 
 Q_SIGNALS:
     void itemsSelected(bool);
