@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,19 +22,20 @@
  */
 
 #include "searchwidget.h"
-#include "support/icon.h"
+#include "support/monoicon.h"
+#include "support/utils.h"
 #include "toolbutton.h"
-#include "support/localize.h"
-#include <QGridLayout>
+#include <QHBoxLayout>
 #include <QKeyEvent>
 #include <QKeySequence>
+#include <QSpacerItem>
 
 class EscKeyEventHandler : public QObject
 {
 public:
     EscKeyEventHandler(SearchWidget *v) : QObject(v), view(v) { }
 protected:
-    bool eventFilter(QObject *obj, QEvent *event)
+    bool eventFilter(QObject *obj, QEvent *event) override
     {
         if (view->hasFocus() && QEvent::KeyRelease==event->type()) {
             QKeyEvent *keyEvent=static_cast<QKeyEvent *>(event);
@@ -50,63 +51,65 @@ private:
     SearchWidget *view;
 };
 
-SearchWidget::SearchWidget(QWidget *p)
+SearchWidget::SearchWidget(QWidget *p, int extraSpace)
      : QWidget(p)
-     , cat(0)
+     , cat(nullptr)
      , widgetIsActive(false)
 {
-    QGridLayout *l=new QGridLayout(this);
-    l->setMargin(0);
+    QHBoxLayout *l=new QHBoxLayout(this);
+    #ifdef Q_OS_MAC
+    l->setSpacing(2);
+    l->setContentsMargins(0, 1, 1, 1+extraSpace);
+    bool closeOnLeft=true;
+    #else
     l->setSpacing(0);
-    label=new SqueezedTextLabel(this);
+    l->setContentsMargins(0, 1, 0, 1+extraSpace);
+    bool closeOnLeft=Utils::Unity==Utils::currentDe();
+    #endif
     edit=new LineEdit(this);
-    edit->setPlaceholderText(i18n("Search..."));
-    l->addWidget(label, 0, 0, 1, 3);
-    l->addWidget(edit, 1, 1);
+    edit->setPlaceholderText(tr("Search..."));
     closeButton=new ToolButton(this);
-    closeButton->setToolTip(i18n("Close Search Bar")+QLatin1String(" (")+QKeySequence(Qt::Key_Escape).toString()+QLatin1Char(')'));
-    l->addWidget(closeButton, 1, 2);
-    Icon icon=Icon("dialog-close");
-    if (icon.isNull()) {
-        icon=Icon("window-close");
+    closeButton->setToolTip(tr("Close Search Bar")+QLatin1String(" (")+QKeySequence(Qt::Key_Escape).toString()+QLatin1Char(')'));
+
+    if (closeOnLeft) {
+        l->addWidget(closeButton);
+        l->addWidget(edit);
+    } else {
+        l->addWidget(edit);
+        l->addWidget(closeButton);
     }
-    closeButton->setIcon(icon);
+
+    closeButton->setIcon(MonoIcon::icon(FontAwesome::close, MonoIcon::constRed, MonoIcon::constRed));
     Icon::init(closeButton);
     connect(closeButton, SIGNAL(clicked()), this, SLOT(close()));
     connect(edit, SIGNAL(textChanged(QString)), SIGNAL(textChanged(QString)));
     connect(edit, SIGNAL(returnPressed()), SIGNAL(returnPressed()));
-    installEventFilter(new EscKeyEventHandler(this));
-    label->setVisible(false);
-    label->setAlignment(Qt::AlignTop);
-    QFont f(font());
-    f.setBold(true);
-    label->setFont(f);
+    installEventFilter(new EscKeyEventHandler(this));;
     setTabOrder(edit, closeButton);
 }
 
-void SearchWidget::setLabel(const QString &s)
-{
-    label->setText(s);
-    label->setVisible(!s.isEmpty());
-}
-
 void SearchWidget::setPermanent()
 {
     show();
     setFocus();
     closeButton->setVisible(false);
     closeButton->deleteLater();
-    closeButton=0;
+    closeButton=nullptr;
+    layout()->setSpacing(0);
 }
 
-void SearchWidget::setCategories(const QList<QPair<QString, QString> > &categories)
+void SearchWidget::setCategories(const QList<Category> &categories)
 {
     QString currentCat;
     if (!cat) {
-        cat=new ComboBox(this);
-        ((QGridLayout *)layout())->addWidget(cat, 1, 0);
-        connect(cat, SIGNAL(currentIndexChanged(int)), SIGNAL(returnPressed()));
+        cat=new SelectorLabel(this);
+        QHBoxLayout *l=static_cast<QHBoxLayout *>(layout());
+        l->insertWidget(0, cat);
+        l->setSpacing(qMin(4, Utils::layoutSpacing(this)));
+        connect(cat, SIGNAL(activated(int)), SIGNAL(returnPressed()));
+        connect(cat, SIGNAL(activated(int)), this, SLOT(categoryActivated(int)));
         setTabOrder(cat, edit);
+        cat->setFixedHeight(edit->height());
     } else {
         currentCat=category();
         if (!currentCat.isEmpty()) {
@@ -116,23 +119,22 @@ void SearchWidget::setCategories(const QList<QPair<QString, QString> > &categori
 
     cat->clear();
 
-    QList<QPair<QString, QString> >::ConstIterator it=categories.constBegin();
-    QList<QPair<QString, QString> >::ConstIterator end=categories.constEnd();
-    for (; it!=end; ++it) {
-        cat->addItem((*it).first, (*it).second);
+    for (const Category &c: categories) {
+        cat->addItem(c.text, c.field, c.toolTip);
     }
 
     if (!currentCat.isEmpty()) {
         for (int i=0; i<cat->count(); ++i) {
-            if (cat->itemData(i).toString()==currentCat) {
+            if (cat->itemData(i)==currentCat) {
                 cat->setCurrentIndex(i);
                 cat->blockSignals(false);
+                setToolTip(cat->action(i)->toolTip());
                 return;
             }
         }
-        cat->blockSignals(false);
-        cat->setCurrentIndex(0);
     }
+    cat->blockSignals(false);
+    cat->setCurrentIndex(0);
 }
 
 void SearchWidget::setCategory(const QString &id)
@@ -141,8 +143,9 @@ void SearchWidget::setCategory(const QString &id)
         return;
     }
     for (int i=0; i<cat->count(); ++i) {
-        if (cat->itemData(i).toString()==id) {
+        if (cat->itemData(i)==id) {
             cat->setCurrentIndex(i);
+            setToolTip(cat->action(i)->toolTip());
             return;
         }
     }
@@ -175,6 +178,9 @@ void SearchWidget::activate(const QString &text)
 
 void SearchWidget::close()
 {
+    if (!closeButton) {
+        return;
+    }
     bool wasActive=widgetIsActive;
     widgetIsActive=false;
     setVisible(false);
@@ -183,3 +189,13 @@ void SearchWidget::close()
         emit active(widgetIsActive);
     }
 }
+
+void SearchWidget::categoryActivated(int c)
+{
+    QAction *a=cat->action(c);
+    if (a) {
+        setToolTip(a->toolTip());
+    }
+}
+
+#include "moc_searchwidget.cpp"
