@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,17 +24,12 @@
 #ifndef SIZEGRIP_H
 #define SIZEGRIP_H
 
-#include <QSizeGrip>
+#include <QWidget>
 
-class SizeGrip : public QSizeGrip
+class SizeGrip : public QWidget
 {
 public:
-    explicit SizeGrip(QWidget *parent = 0);
-    QSize sizeHint() const;
-    QSize minimumSizeHint() const { return sizeHint(); }
-
-private:
-    mutable QSize sh;
+    explicit SizeGrip(QWidget *parent = nullptr);
 };
 
 #endif
