@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,13 +24,13 @@
 #ifndef STATELABEL_H
 #define STATELABEL_H
 
-#include "buddylabel.h"
+#include <QLabel>
 
-class StateLabel : public BuddyLabel
+class StateLabel : public QLabel
 {
 public:
-    StateLabel(QWidget *parent=0)
-        : BuddyLabel(parent)
+    StateLabel(QWidget *parent=nullptr)
+        : QLabel(parent)
         , on(false) {
     }
 
