@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,7 +24,6 @@
 #include "toolbutton.h"
 #include "support/icon.h"
 #include "support/gtkstyle.h"
-#include "support/touchproxystyle.h"
 #include "config.h"
 #include "support/utils.h"
 #include <QMenu>
@@ -35,15 +34,11 @@
 
 ToolButton::ToolButton(QWidget *parent)
     : QToolButton(parent)
-    #ifdef USE_SYSTEM_MENU_ICON
-    , hideMenuIndicator(GtkStyle::isActive())
-    #else
-    , hideMenuIndicator(true)
-    #endif
 {
     Icon::init(this);
     #ifdef Q_OS_MAC
     setStyleSheet("QToolButton {border: 0}");
+    allowMouseOver=parent && parent->objectName()!=QLatin1String("toolbar");
     #endif
     setFocusPolicy(Qt::NoFocus);
 }
@@ -51,66 +46,56 @@ ToolButton::ToolButton(QWidget *parent)
 void ToolButton::paintEvent(QPaintEvent *e)
 {
     #ifdef Q_OS_MAC
-    if (isDown() || isChecked()) {
+    bool down=isDown() || isChecked();
+    bool mo=false;
+
+    if (allowMouseOver && !down && isEnabled()) {
+        QStyleOptionToolButton opt;
+        initStyleOption(&opt);
+        mo=opt.state&QStyle::State_MouseOver && this==QApplication::widgetAt(QCursor::pos());
+    }
+    if (down || mo) {
         QPainter p(this);
         QColor col(palette().color(QPalette::WindowText));
         QRect r(rect());
-        QPainterPath path=Utils::buildPath(QRectF(r.x()+1.5, r.y()+0.5, r.width()-2, r.height()-1), 2.5);
+        QPainterPath path=Utils::buildPath(QRectF(r.x()+1.5, r.y()+1.5, r.width()-3, r.height()-3), 2.5);
         p.setRenderHint(QPainter::Antialiasing, true);
+        col.setAlphaF(0.4);
         p.setPen(col);
         p.drawPath(path);
-        col.setAlphaF(0.1);
-        p.fillPath(path, col);
+        if (down) {
+            col.setAlphaF(0.1);
+            p.fillPath(path, col);
+        }
     }
     #endif
-    #if QT_VERSION > 0x050000
+    Q_UNUSED(e)
     // Hack to work-around Qt5 sometimes leaving toolbutton in 'raised' state.
     QStylePainter p(this);
     QStyleOptionToolButton opt;
     initStyleOption(&opt);
-    if (hideMenuIndicator) {
-        opt.features=QStyleOptionToolButton::None;
-    }
+    opt.features=QStyleOptionToolButton::None;
     if (opt.state&QStyle::State_MouseOver && this!=QApplication::widgetAt(QCursor::pos())) {
         opt.state&=~QStyle::State_MouseOver;
     }
     p.drawComplexControl(QStyle::CC_ToolButton, opt);
-    #else
-    if (menu() && hideMenuIndicator) {
-        QStylePainter p(this);
-        QStyleOptionToolButton opt;
-        initStyleOption(&opt);
-        opt.features=QStyleOptionToolButton::None;
-        p.drawComplexControl(QStyle::CC_ToolButton, opt);
-    } else {
-        QToolButton::paintEvent(e);
-    }
-    #endif
 }
 
 QSize ToolButton::sizeHint() const
 {
-    if (!hideMenuIndicator) {
-        QSize sz=QToolButton::sizeHint();
-        return Utils::touchFriendly() ? QSize(sz.width()*TouchProxyStyle::constScaleFactor, sz.height()) : sz;
-    }
+    if (!sh.isValid()) {
+        ensurePolished();
+        sh = QToolButton::sizeHint();
 
-    if (sh.isValid()) {
-        return sh;
-    }
+        if (sh.width()>sh.height()) {
+            sh.setWidth(sh.height());
+        }
 
-    ensurePolished();
-
-    if (menu()) {
-        QStyleOptionToolButton opt;
-        initStyleOption(&opt);
-        opt.features=QStyleOptionToolButton::None;
-        sh = style()->sizeFromContents(QStyle::CT_ToolButton, &opt, opt.iconSize, this).expandedTo(QApplication::globalStrut());
-    } else {
-        sh=QToolButton::sizeHint();
+        sh=QSize(qMax(sh.width(), sh.height()), qMax(sh.width(), sh.height()));
+        #ifdef Q_OS_MAC
+        sh=QSize(qMax(sh.width(), 22), qMax(sh.height(), 20));
+        #endif
     }
-    bool touchFriendly=Utils::touchFriendly();
-    sh=QSize(qMax(sh.width(), sh.height())*(touchFriendly ? TouchProxyStyle::constScaleFactor : 1.0), touchFriendly ? sh.height() : qMax(sh.width(), sh.height()));
     return sh;
 }
 
