@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,20 +25,22 @@
 #define TOOLBUTTON_H
 
 #include <QToolButton>
+#include "support/icon.h"
 
 class QMenu;
 class ToolButton : public QToolButton
 {
 public:
-    explicit ToolButton(QWidget *parent = 0);
-    QSize sizeHint() const;
+    explicit ToolButton(QWidget *parent = nullptr);
+    QSize sizeHint() const override;
     void setMenu(QMenu *m);
-    void paintEvent(QPaintEvent *e);
-    void setHideMenuIndicator(bool h) { hideMenuIndicator=h; }
+    void paintEvent(QPaintEvent *e) override;
 
 private:
-    bool hideMenuIndicator;
     mutable QSize sh;
+    #ifdef Q_OS_MAC
+    bool allowMouseOver;
+    #endif
 };
 
 #endif // MENUBUTTON_H
