@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -27,6 +27,7 @@
 #include "config.h"
 #include "basicitemdelegate.h"
 #include "support/utils.h"
+#include "mpd-interface/song.h"
 #include <QMouseEvent>
 #include <QPaintEvent>
 #include <QPainter>
@@ -38,12 +39,7 @@
 #include <QApplication>
 #include <QHeaderView>
 
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KGlobalSettings>
-#define SINGLE_CLICK KGlobalSettings::singleClick()
-#else
 #define SINGLE_CLICK style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, 0, this)
-#endif
 
 QImage TreeView::setOpacity(const QImage &orig, double opacity)
 {
@@ -88,7 +84,7 @@ bool TreeView::getForceSingleClick()
 
 TreeView::TreeView(QWidget *parent, bool menuAlwaysAllowed)
     : QTreeView(parent)
-    , eventFilter(0)
+    , eventFilter(nullptr)
     , forceSingleColumn(false)
     , alwaysAllowMenu(menuAlwaysAllowed)
 {
@@ -150,7 +146,7 @@ bool TreeView::haveSelectedItems() const
 bool TreeView::haveUnSelectedItems() const
 {
     // Dont need the sorted type of 'selectedIndexes' here...
-    return selectionModel() && selectionModel()->selectedIndexes().count()!=model()->rowCount();
+    return selectionModel() && model() && selectionModel()->selectedIndexes().count()!=model()->rowCount();
 }
 
 void TreeView::drag(Qt::DropActions supportedActions, QAbstractItemView *view, const QModelIndexList &items)
@@ -162,23 +158,8 @@ void TreeView::drag(Qt::DropActions supportedActions, QAbstractItemView *view, c
         }
         QDrag *drag = new QDrag(view);
         drag->setMimeData(data);
-        QPixmap pix;
-
-        if (1==items.count()) {
-            QVariant var=view->model()->data(items.first(), Cantata::Role_Image);
-            QImage img=var.value<QImage>();
-            if (img.isNull()) {
-                pix=var.value<QPixmap>();
-            } else {
-                pix=QPixmap::fromImage(img);
-            }
-        }
         int pixSize=Icon::stdSize(Utils::scaleForDpi(32));
-        if (pix.isNull()) {
-            drag->setPixmap(Icons::self()->audioFileIcon.pixmap(pixSize, pixSize));
-        } else {
-            drag->setPixmap(pix.width()<pixSize ? pix : pix.scaled(QSize(pixSize, pixSize), Qt::KeepAspectRatio, Qt::SmoothTransformation));
-        }
+        drag->setPixmap(Icons::self()->audioListIcon.pixmap(pixSize, pixSize));
         drag->start(supportedActions);
     }
 }
@@ -201,7 +182,7 @@ QModelIndexList TreeView::selectedIndexes(bool sorted) const
     } else if (model() && model()->columnCount()>1) {
         QModelIndexList list=selectionModel()->selectedIndexes();
         QModelIndexList sel;
-        foreach (const QModelIndex &idx, list) {
+        for (const QModelIndex &idx: list) {
             if (0==idx.column()) {
                 sel.append(idx);
             }
@@ -253,7 +234,7 @@ QModelIndexList TreeView::sortIndexes(const QModelIndexList &list)
     // have different parents. Therefore, we use the sort above - so that the hierarchy is preserved.
     // First, create the list of 'Index' items to be sorted...
     QList<Index> toSort;
-    foreach (const QModelIndex &i, list) {
+    for (const QModelIndex &i: list) {
         if (0==i.column()) {
             toSort.append(Index(i));
         }
@@ -263,7 +244,7 @@ QModelIndexList TreeView::sortIndexes(const QModelIndexList &list)
 
     // Now convert the QList<Index> into a QModelIndexList
     QModelIndexList sorted;
-    foreach (const Index &i, toSort) {
+    for (const Index &i: toSort) {
         sorted.append(i);
     }
     return sorted;
@@ -334,10 +315,12 @@ void TreeView::setUseSimpleDelegate()
 void TreeView::setBackgroundImage(const QIcon &icon)
 {
     QPalette pal=parentWidget()->palette();
-    if (!icon.isNull()) {
-        pal.setColor(QPalette::Base, Qt::transparent);
-    }
+//    if (!icon.isNull()) {
+//        pal.setColor(QPalette::Base, Qt::transparent);
+//    }
+    #ifndef Q_OS_MAC
     setPalette(pal);
+    #endif
     viewport()->setPalette(pal);
     bgnd=createBgndPixmap(icon);
 }
@@ -350,6 +333,16 @@ void TreeView::paintEvent(QPaintEvent *e)
         p.fillRect(0, 0, sz.width(), sz.height(), QApplication::palette().color(QPalette::Base));
         p.drawPixmap((sz.width()-bgnd.width())/2, (sz.height()-bgnd.height())/2, bgnd);
     }
+    if (!info.isEmpty() && model() && 0==model()->rowCount()) {
+        QPainter p(viewport());
+        QColor col(palette().text().color());
+        col.setAlphaF(0.5);
+        QFont f(font());
+        f.setItalic(true);
+        p.setPen(col);
+        p.setFont(f);
+        p.drawText(rect().adjusted(8, 8, -16, -16), Qt::AlignCenter|Qt::TextWordWrap, info);
+    }
     QTreeView::paintEvent(e);
 }
 
@@ -402,3 +395,5 @@ void TreeView::itemWasClicked(const QModelIndex &index)
         setExpanded(index, !isExpanded(index));
     }
 }
+
+#include "moc_treeview.cpp"
