@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -21,10 +21,13 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#include "config.h"
 #include "volumeslider.h"
-#include "support/localize.h"
-#include "mpd/mpdconnection.h"
-#include "mpd/mpdstatus.h"
+#include "mpd-interface/mpdconnection.h"
+#include "mpd-interface/mpdstatus.h"
+#ifdef ENABLE_HTTP_STREAM_PLAYBACK
+#include "mpd-interface/httpstream.h"
+#endif
 #include "support/action.h"
 #include "support/actioncollection.h"
 #include "gui/stdactions.h"
@@ -39,20 +42,17 @@
 #include <QMouseEvent>
 #include <QWheelEvent>
 #include <QMenu>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KShortcut>
-#endif
 
-class ProxyStyle : public QProxyStyle
+class VolumeSliderProxyStyle : public QProxyStyle
 {
 public:
-    ProxyStyle()
+    VolumeSliderProxyStyle()
         : QProxyStyle()
     {
         setBaseStyle(qApp->style());
     }
 
-    int styleHint(StyleHint stylehint, const QStyleOption *opt, const QWidget *widget, QStyleHintReturn *returnData) const
+    int styleHint(StyleHint stylehint, const QStyleOption *opt, const QWidget *widget, QStyleHintReturn *returnData) const override
     {
         if (SH_Slider_AbsoluteSetButtons==stylehint) {
             return Qt::LeftButton|QProxyStyle::styleHint(stylehint, opt, widget, returnData);
@@ -62,33 +62,19 @@ public:
     }
 };
 
-
 static int widthStep=4;
 static int constHeightStep=2;
 
-static QColor clampColor(const QColor &col)
-{
-    static const int constMin=64;
-    static const int constMax=240;
-
-    if (col.value()<constMin) {
-        return QColor(constMin, constMin, constMin);
-    } else if (col.value()>constMax) {
-        return QColor(constMax, constMax, constMax);
-    }
-    return col;
-}
-
-VolumeSlider::VolumeSlider(QWidget *p)
+VolumeSlider::VolumeSlider(bool isMpd, QWidget *p)
     : QSlider(p)
+    , isMpdVol(isMpd)
+    , isActive(isMpd)
     , lineWidth(0)
-    , shown(false)
     , down(false)
-    , fadingStop(false)
-    , muteAction(0)
-    , menu(0)
+    , muteAction(nullptr)
+    , menu(nullptr)
 {
-    widthStep=Utils::touchFriendly() ? 5 : 4;
+    widthStep=4;
     setRange(0, 100);
     setPageStep(Settings::self()->volumeStep());
     lineWidth=Utils::scaleForDpi(1);
@@ -100,11 +86,11 @@ VolumeSlider::VolumeSlider(QWidget *p)
     setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
     setOrientation(Qt::Horizontal);
     setFocusPolicy(Qt::NoFocus);
-    setStyle(new ProxyStyle());
+    setStyle(new VolumeSliderProxyStyle());
     setStyleSheet(QString("QSlider::groove:horizontal {border: 0px;} "
                           "QSlider::sub-page:horizontal {border: 0px;} "
                           "QSlider::handle:horizontal {width: 0px; height:0px; margin:0;}"));
-    textCol=clampColor(palette().color(QPalette::Active, QPalette::Text));
+    textCol=Utils::clampColor(palette().color(QPalette::Active, QPalette::Text));
     generatePixmaps();
 }
 
@@ -113,43 +99,46 @@ void VolumeSlider::initActions()
     if (muteAction) {
         return;
     }
-    muteAction = ActionCollection::get()->createAction("mute", i18n("Mute"));
+    muteAction = ActionCollection::get()->createAction("mute", tr("Mute"));
     addAction(muteAction);
-    #ifdef ENABLE_KDE_SUPPORT
-    muteAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeMute));
-    StdActions::self()->increaseVolumeAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeUp));
-    StdActions::self()->decreaseVolumeAction->setGlobalShortcut(KShortcut(Qt::Key_VolumeDown));
+    connect(muteAction, SIGNAL(triggered()), this, SLOT(muteToggled()));
+    connect(StdActions::self()->increaseVolumeAction, SIGNAL(triggered()), this, SLOT(increaseVolume()));
+    connect(StdActions::self()->decreaseVolumeAction, SIGNAL(triggered()), this, SLOT(decreaseVolume()));
+    if (isMpdVol) {
+        connect(MPDStatus::self(), SIGNAL(updated()), this, SLOT(updateStatus()));
+        connect(this, SIGNAL(valueChanged(int)), MPDConnection::self(), SLOT(setVolume(int)));
+        connect(this, SIGNAL(toggleMute()), MPDConnection::self(), SLOT(toggleMute()));
+    }
+    #ifdef ENABLE_HTTP_STREAM_PLAYBACK
+    else {
+        connect(this, SIGNAL(valueChanged(int)), HttpStream::self(), SLOT(setVolume(int)));
+        connect(this, SIGNAL(toggleMute()), HttpStream::self(), SLOT(toggleMute()));
+        connect(HttpStream::self(), SIGNAL(update()), this, SLOT(updateStatus()));
+        connect(HttpStream::self(), SIGNAL(isEnabled(bool)), this, SLOT(setEnabled(bool)));
+        connect(HttpStream::self(), SIGNAL(isEnabled(bool)), this, SIGNAL(stateChanged()));
+    }
     #endif
-    connect(muteAction, SIGNAL(triggered(bool)), MPDConnection::self(), SLOT(toggleMute()));
-    connect(MPDStatus::self(), SIGNAL(updated()), this, SLOT(updateMpdStatus()));
-    connect(StdActions::self()->increaseVolumeAction, SIGNAL(triggered(bool)), this, SLOT(increaseVolume()));
-    connect(StdActions::self()->decreaseVolumeAction, SIGNAL(triggered(bool)), this, SLOT(decreaseVolume()));
-    connect(this, SIGNAL(valueChanged(int)), MPDConnection::self(), SLOT(setVolume(int)));
     addAction(StdActions::self()->increaseVolumeAction);
     addAction(StdActions::self()->decreaseVolumeAction);
 }
 
-void VolumeSlider::showEvent(QShowEvent *ev)
+void VolumeSlider::setColor(const QColor &col)
 {
-    if (!shown) {
-        shown=true;
-        QLabel lbl(parentWidget());
-        lbl.ensurePolished();
-        QColor col=clampColor(lbl.palette().text().color());
-
-        if (col!=textCol) {
-            textCol=col;
-            generatePixmaps();
-        }
+    if (col!=textCol) {
+        textCol=col;
+        generatePixmaps();
     }
-    QSlider::showEvent(ev);
 }
 
 void VolumeSlider::paintEvent(QPaintEvent *)
 {
     bool reverse=isRightToLeft();
     QPainter p(this);
-    bool muted=MPDConnection::self()->isMuted();
+    #ifdef ENABLE_HTTP_STREAM_PLAYBACK
+    bool muted = isMpdVol ? MPDConnection::self()->isMuted() : HttpStream::self()->isMuted();
+    #else
+    bool muted = MPDConnection::self()->isMuted();
+    #endif
     if (muted || !isEnabled()) {
         p.setOpacity(0.25);
     }
@@ -228,14 +217,20 @@ void VolumeSlider::contextMenuEvent(QContextMenuEvent *ev)
     static const char *constValProp="val";
     if (!menu) {
         menu=new QMenu(this);
-        muteMenuAction=menu->addAction(i18n("Mute"));
+        muteMenuAction=menu->addAction(tr("Mute"));
         muteMenuAction->setProperty(constValProp, -1);
         for (int i=0; i<11; ++i) {
             menu->addAction(QString("%1%").arg(i*10))->setProperty(constValProp, i*10);
         }
     }
 
-    muteMenuAction->setText(MPDConnection::self()->isMuted() ? i18n("Unmute") : i18n("Mute"));
+    #ifdef ENABLE_HTTP_STREAM_PLAYBACK
+    bool muted = isMpdVol ? MPDConnection::self()->isMuted() : HttpStream::self()->isMuted();
+    #else
+    bool muted = MPDConnection::self()->isMuted();
+    #endif
+
+    muteMenuAction->setText(muted ? tr("Unmute") : tr("Mute"));
     QAction *ret = menu->exec(mapToGlobal(ev->pos()));
     if (ret) {
         int val=ret->property(constValProp).toInt();
@@ -262,13 +257,20 @@ void VolumeSlider::wheelEvent(QWheelEvent *ev)
     }
 }
 
-void VolumeSlider::updateMpdStatus()
+void VolumeSlider::muteToggled()
 {
-    if (fadingStop) {
-        return;
+    if (isActive) {
+        emit toggleMute();
     }
+}
 
+void VolumeSlider::updateStatus()
+{
+    #ifdef ENABLE_HTTP_STREAM_PLAYBACK
+    int volume=isMpdVol ? MPDStatus::self()->volume() : HttpStream::self()->volume();
+    #else
     int volume=MPDStatus::self()->volume();
+    #endif
 
     blockSignals(true);
     if (volume<0) {
@@ -276,32 +278,42 @@ void VolumeSlider::updateMpdStatus()
     } else {
         int unmuteVolume=-1;
         if (0==volume) {
+            #ifdef ENABLE_HTTP_STREAM_PLAYBACK
+            unmuteVolume=isMpdVol ? MPDConnection::self()->unmuteVolume() : HttpStream::self()->unmuteVolume();
+            #else
             unmuteVolume=MPDConnection::self()->unmuteVolume();
+            #endif
             if (unmuteVolume>0) {
                 volume=unmuteVolume;
             }
         }
-        setEnabled(true);
-        setToolTip(unmuteVolume>0 ? i18n("Volume %1% (Muted)", volume) : i18n("Volume %1%", volume));
+        setToolTip(unmuteVolume>0 ? tr("Volume %1% (Muted)").arg(volume) : tr("Volume %1%").arg(volume));
         setValue(volume);
     }
+    bool wasEnabled=isEnabled();
     setEnabled(volume>=0);
-    setVisible(volume>=0);
     update();
     muteAction->setEnabled(isEnabled());
     StdActions::self()->increaseVolumeAction->setEnabled(isEnabled());
     StdActions::self()->decreaseVolumeAction->setEnabled(isEnabled());
     blockSignals(false);
+    if (isEnabled()!=wasEnabled) {
+        emit stateChanged();
+    }
 }
 
 void VolumeSlider::increaseVolume()
 {
-    triggerAction(QAbstractSlider::SliderPageStepAdd);
+    if (isActive) {
+        triggerAction(QAbstractSlider::SliderPageStepAdd);
+    }
 }
 
 void VolumeSlider::decreaseVolume()
 {
-    triggerAction(QAbstractSlider::SliderPageStepSub);
+    if (isActive) {
+        triggerAction(QAbstractSlider::SliderPageStepSub);
+    }
 }
 
 void VolumeSlider::generatePixmaps()
@@ -333,3 +345,5 @@ QPixmap VolumeSlider::generatePixmap(bool filled)
     }
     return pix;
 }
+
+#include "moc_volumeslider.cpp"
