@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -27,7 +27,7 @@
 #include <QPainter>
 #include <QStyle>
 
-void WizardPage::setBackground(const Icon &i)
+void WizardPage::setBackground(const QIcon &i)
 {
     int size=fontMetrics().height()*10;
     size=((int)(size/4))*4;
