@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -27,16 +27,16 @@
 #include <QWizardPage>
 #include <QPixmap>
 
-class Icon;
+class QIcon;
 
 class WizardPage : public QWizardPage
 {
 public:
-    WizardPage(QWidget *parent = 0) : QWizardPage(parent) { }
-    virtual ~WizardPage() { }
+    WizardPage(QWidget *parent = nullptr) : QWizardPage(parent) { }
+    ~WizardPage() override { }
 
-    void setBackground(const Icon &i);
-    void paintEvent(QPaintEvent *e);
+    void setBackground(const QIcon &i);
+    void paintEvent(QPaintEvent *e) override;
 
 private:
     QPixmap pix;
